/*-------------------------------------------------------------------------
 *
 * equalfuncs.switch.c
 *    Generated node infrastructure code
 *
 * Portions Copyright (c) 1996-2024, PostgreSQL Global Development Group
 * Portions Copyright (c) 1994, Regents of the University of California
 *
 * NOTES
 *  ******************************
 *  *** DO NOT EDIT THIS FILE! ***
 *  ******************************
 *
 *  It has been GENERATED by src/backend/nodes/gen_node_support.pl
 *
 *-------------------------------------------------------------------------
 */
		case T_Alias:
			retval = _equalAlias(a, b);
			break;
		case T_RangeVar:
			retval = _equalRangeVar(a, b);
			break;
		case T_TableFunc:
			retval = _equalTableFunc(a, b);
			break;
		case T_IntoClause:
			retval = _equalIntoClause(a, b);
			break;
		case T_Var:
			retval = _equalVar(a, b);
			break;
		case T_Const:
			retval = _equalConst(a, b);
			break;
		case T_Param:
			retval = _equalParam(a, b);
			break;
		case T_Aggref:
			retval = _equalAggref(a, b);
			break;
		case T_GroupingFunc:
			retval = _equalGroupingFunc(a, b);
			break;
		case T_WindowFunc:
			retval = _equalWindowFunc(a, b);
			break;
		case T_WindowFuncRunCondition:
			retval = _equalWindowFuncRunCondition(a, b);
			break;
		case T_MergeSupportFunc:
			retval = _equalMergeSupportFunc(a, b);
			break;
		case T_SubscriptingRef:
			retval = _equalSubscriptingRef(a, b);
			break;
		case T_FuncExpr:
			retval = _equalFuncExpr(a, b);
			break;
		case T_NamedArgExpr:
			retval = _equalNamedArgExpr(a, b);
			break;
		case T_OpExpr:
			retval = _equalOpExpr(a, b);
			break;
		case T_DistinctExpr:
			retval = _equalDistinctExpr(a, b);
			break;
		case T_NullIfExpr:
			retval = _equalNullIfExpr(a, b);
			break;
		case T_ScalarArrayOpExpr:
			retval = _equalScalarArrayOpExpr(a, b);
			break;
		case T_BoolExpr:
			retval = _equalBoolExpr(a, b);
			break;
		case T_SubLink:
			retval = _equalSubLink(a, b);
			break;
		case T_SubPlan:
			retval = _equalSubPlan(a, b);
			break;
		case T_AlternativeSubPlan:
			retval = _equalAlternativeSubPlan(a, b);
			break;
		case T_FieldSelect:
			retval = _equalFieldSelect(a, b);
			break;
		case T_FieldStore:
			retval = _equalFieldStore(a, b);
			break;
		case T_RelabelType:
			retval = _equalRelabelType(a, b);
			break;
		case T_CoerceViaIO:
			retval = _equalCoerceViaIO(a, b);
			break;
		case T_ArrayCoerceExpr:
			retval = _equalArrayCoerceExpr(a, b);
			break;
		case T_ConvertRowtypeExpr:
			retval = _equalConvertRowtypeExpr(a, b);
			break;
		case T_CollateExpr:
			retval = _equalCollateExpr(a, b);
			break;
		case T_CaseExpr:
			retval = _equalCaseExpr(a, b);
			break;
		case T_CaseWhen:
			retval = _equalCaseWhen(a, b);
			break;
		case T_CaseTestExpr:
			retval = _equalCaseTestExpr(a, b);
			break;
		case T_ArrayExpr:
			retval = _equalArrayExpr(a, b);
			break;
		case T_RowExpr:
			retval = _equalRowExpr(a, b);
			break;
		case T_RowCompareExpr:
			retval = _equalRowCompareExpr(a, b);
			break;
		case T_CoalesceExpr:
			retval = _equalCoalesceExpr(a, b);
			break;
		case T_MinMaxExpr:
			retval = _equalMinMaxExpr(a, b);
			break;
		case T_SQLValueFunction:
			retval = _equalSQLValueFunction(a, b);
			break;
		case T_XmlExpr:
			retval = _equalXmlExpr(a, b);
			break;
		case T_JsonFormat:
			retval = _equalJsonFormat(a, b);
			break;
		case T_JsonReturning:
			retval = _equalJsonReturning(a, b);
			break;
		case T_JsonValueExpr:
			retval = _equalJsonValueExpr(a, b);
			break;
		case T_JsonConstructorExpr:
			retval = _equalJsonConstructorExpr(a, b);
			break;
		case T_JsonIsPredicate:
			retval = _equalJsonIsPredicate(a, b);
			break;
		case T_JsonBehavior:
			retval = _equalJsonBehavior(a, b);
			break;
		case T_JsonExpr:
			retval = _equalJsonExpr(a, b);
			break;
		case T_JsonTablePath:
			retval = _equalJsonTablePath(a, b);
			break;
		case T_JsonTablePathScan:
			retval = _equalJsonTablePathScan(a, b);
			break;
		case T_JsonTableSiblingJoin:
			retval = _equalJsonTableSiblingJoin(a, b);
			break;
		case T_NullTest:
			retval = _equalNullTest(a, b);
			break;
		case T_BooleanTest:
			retval = _equalBooleanTest(a, b);
			break;
		case T_MergeAction:
			retval = _equalMergeAction(a, b);
			break;
		case T_CoerceToDomain:
			retval = _equalCoerceToDomain(a, b);
			break;
		case T_CoerceToDomainValue:
			retval = _equalCoerceToDomainValue(a, b);
			break;
		case T_SetToDefault:
			retval = _equalSetToDefault(a, b);
			break;
		case T_CurrentOfExpr:
			retval = _equalCurrentOfExpr(a, b);
			break;
		case T_NextValueExpr:
			retval = _equalNextValueExpr(a, b);
			break;
		case T_InferenceElem:
			retval = _equalInferenceElem(a, b);
			break;
		case T_TargetEntry:
			retval = _equalTargetEntry(a, b);
			break;
		case T_RangeTblRef:
			retval = _equalRangeTblRef(a, b);
			break;
		case T_JoinExpr:
			retval = _equalJoinExpr(a, b);
			break;
		case T_FromExpr:
			retval = _equalFromExpr(a, b);
			break;
		case T_OnConflictExpr:
			retval = _equalOnConflictExpr(a, b);
			break;
		case T_Query:
			retval = _equalQuery(a, b);
			break;
		case T_TypeName:
			retval = _equalTypeName(a, b);
			break;
		case T_ColumnRef:
			retval = _equalColumnRef(a, b);
			break;
		case T_ParamRef:
			retval = _equalParamRef(a, b);
			break;
		case T_A_Expr:
			retval = _equalA_Expr(a, b);
			break;
		case T_A_Const:
			retval = _equalA_Const(a, b);
			break;
		case T_TypeCast:
			retval = _equalTypeCast(a, b);
			break;
		case T_CollateClause:
			retval = _equalCollateClause(a, b);
			break;
		case T_RoleSpec:
			retval = _equalRoleSpec(a, b);
			break;
		case T_FuncCall:
			retval = _equalFuncCall(a, b);
			break;
		case T_A_Star:
			retval = _equalA_Star(a, b);
			break;
		case T_A_Indices:
			retval = _equalA_Indices(a, b);
			break;
		case T_A_Indirection:
			retval = _equalA_Indirection(a, b);
			break;
		case T_A_ArrayExpr:
			retval = _equalA_ArrayExpr(a, b);
			break;
		case T_ResTarget:
			retval = _equalResTarget(a, b);
			break;
		case T_MultiAssignRef:
			retval = _equalMultiAssignRef(a, b);
			break;
		case T_SortBy:
			retval = _equalSortBy(a, b);
			break;
		case T_WindowDef:
			retval = _equalWindowDef(a, b);
			break;
		case T_RangeSubselect:
			retval = _equalRangeSubselect(a, b);
			break;
		case T_RangeFunction:
			retval = _equalRangeFunction(a, b);
			break;
		case T_RangeTableFunc:
			retval = _equalRangeTableFunc(a, b);
			break;
		case T_RangeTableFuncCol:
			retval = _equalRangeTableFuncCol(a, b);
			break;
		case T_RangeTableSample:
			retval = _equalRangeTableSample(a, b);
			break;
		case T_ColumnDef:
			retval = _equalColumnDef(a, b);
			break;
		case T_TableLikeClause:
			retval = _equalTableLikeClause(a, b);
			break;
		case T_IndexElem:
			retval = _equalIndexElem(a, b);
			break;
		case T_DefElem:
			retval = _equalDefElem(a, b);
			break;
		case T_LockingClause:
			retval = _equalLockingClause(a, b);
			break;
		case T_XmlSerialize:
			retval = _equalXmlSerialize(a, b);
			break;
		case T_PartitionElem:
			retval = _equalPartitionElem(a, b);
			break;
		case T_PartitionSpec:
			retval = _equalPartitionSpec(a, b);
			break;
		case T_PartitionBoundSpec:
			retval = _equalPartitionBoundSpec(a, b);
			break;
		case T_PartitionRangeDatum:
			retval = _equalPartitionRangeDatum(a, b);
			break;
		case T_SinglePartitionSpec:
			retval = _equalSinglePartitionSpec(a, b);
			break;
		case T_PartitionCmd:
			retval = _equalPartitionCmd(a, b);
			break;
		case T_RangeTblEntry:
			retval = _equalRangeTblEntry(a, b);
			break;
		case T_RTEPermissionInfo:
			retval = _equalRTEPermissionInfo(a, b);
			break;
		case T_RangeTblFunction:
			retval = _equalRangeTblFunction(a, b);
			break;
		case T_TableSampleClause:
			retval = _equalTableSampleClause(a, b);
			break;
		case T_WithCheckOption:
			retval = _equalWithCheckOption(a, b);
			break;
		case T_SortGroupClause:
			retval = _equalSortGroupClause(a, b);
			break;
		case T_GroupingSet:
			retval = _equalGroupingSet(a, b);
			break;
		case T_WindowClause:
			retval = _equalWindowClause(a, b);
			break;
		case T_RowMarkClause:
			retval = _equalRowMarkClause(a, b);
			break;
		case T_WithClause:
			retval = _equalWithClause(a, b);
			break;
		case T_InferClause:
			retval = _equalInferClause(a, b);
			break;
		case T_OnConflictClause:
			retval = _equalOnConflictClause(a, b);
			break;
		case T_CTESearchClause:
			retval = _equalCTESearchClause(a, b);
			break;
		case T_CTECycleClause:
			retval = _equalCTECycleClause(a, b);
			break;
		case T_CommonTableExpr:
			retval = _equalCommonTableExpr(a, b);
			break;
		case T_MergeWhenClause:
			retval = _equalMergeWhenClause(a, b);
			break;
		case T_TriggerTransition:
			retval = _equalTriggerTransition(a, b);
			break;
		case T_JsonOutput:
			retval = _equalJsonOutput(a, b);
			break;
		case T_JsonArgument:
			retval = _equalJsonArgument(a, b);
			break;
		case T_JsonFuncExpr:
			retval = _equalJsonFuncExpr(a, b);
			break;
		case T_JsonTablePathSpec:
			retval = _equalJsonTablePathSpec(a, b);
			break;
		case T_JsonTable:
			retval = _equalJsonTable(a, b);
			break;
		case T_JsonTableColumn:
			retval = _equalJsonTableColumn(a, b);
			break;
		case T_JsonKeyValue:
			retval = _equalJsonKeyValue(a, b);
			break;
		case T_JsonParseExpr:
			retval = _equalJsonParseExpr(a, b);
			break;
		case T_JsonScalarExpr:
			retval = _equalJsonScalarExpr(a, b);
			break;
		case T_JsonSerializeExpr:
			retval = _equalJsonSerializeExpr(a, b);
			break;
		case T_JsonObjectConstructor:
			retval = _equalJsonObjectConstructor(a, b);
			break;
		case T_JsonArrayConstructor:
			retval = _equalJsonArrayConstructor(a, b);
			break;
		case T_JsonArrayQueryConstructor:
			retval = _equalJsonArrayQueryConstructor(a, b);
			break;
		case T_JsonAggConstructor:
			retval = _equalJsonAggConstructor(a, b);
			break;
		case T_JsonObjectAgg:
			retval = _equalJsonObjectAgg(a, b);
			break;
		case T_JsonArrayAgg:
			retval = _equalJsonArrayAgg(a, b);
			break;
		case T_RawStmt:
			retval = _equalRawStmt(a, b);
			break;
		case T_InsertStmt:
			retval = _equalInsertStmt(a, b);
			break;
		case T_DeleteStmt:
			retval = _equalDeleteStmt(a, b);
			break;
		case T_UpdateStmt:
			retval = _equalUpdateStmt(a, b);
			break;
		case T_MergeStmt:
			retval = _equalMergeStmt(a, b);
			break;
		case T_SelectStmt:
			retval = _equalSelectStmt(a, b);
			break;
		case T_SetOperationStmt:
			retval = _equalSetOperationStmt(a, b);
			break;
		case T_ReturnStmt:
			retval = _equalReturnStmt(a, b);
			break;
		case T_PLAssignStmt:
			retval = _equalPLAssignStmt(a, b);
			break;
		case T_CreateSchemaStmt:
			retval = _equalCreateSchemaStmt(a, b);
			break;
		case T_AlterTableStmt:
			retval = _equalAlterTableStmt(a, b);
			break;
		case T_ReplicaIdentityStmt:
			retval = _equalReplicaIdentityStmt(a, b);
			break;
		case T_AlterTableCmd:
			retval = _equalAlterTableCmd(a, b);
			break;
		case T_AlterCollationStmt:
			retval = _equalAlterCollationStmt(a, b);
			break;
		case T_AlterDomainStmt:
			retval = _equalAlterDomainStmt(a, b);
			break;
		case T_GrantStmt:
			retval = _equalGrantStmt(a, b);
			break;
		case T_ObjectWithArgs:
			retval = _equalObjectWithArgs(a, b);
			break;
		case T_AccessPriv:
			retval = _equalAccessPriv(a, b);
			break;
		case T_GrantRoleStmt:
			retval = _equalGrantRoleStmt(a, b);
			break;
		case T_AlterDefaultPrivilegesStmt:
			retval = _equalAlterDefaultPrivilegesStmt(a, b);
			break;
		case T_CopyStmt:
			retval = _equalCopyStmt(a, b);
			break;
		case T_VariableSetStmt:
			retval = _equalVariableSetStmt(a, b);
			break;
		case T_VariableShowStmt:
			retval = _equalVariableShowStmt(a, b);
			break;
		case T_CreateStmt:
			retval = _equalCreateStmt(a, b);
			break;
		case T_Constraint:
			retval = _equalConstraint(a, b);
			break;
		case T_CreateTableSpaceStmt:
			retval = _equalCreateTableSpaceStmt(a, b);
			break;
		case T_DropTableSpaceStmt:
			retval = _equalDropTableSpaceStmt(a, b);
			break;
		case T_AlterTableSpaceOptionsStmt:
			retval = _equalAlterTableSpaceOptionsStmt(a, b);
			break;
		case T_AlterTableMoveAllStmt:
			retval = _equalAlterTableMoveAllStmt(a, b);
			break;
		case T_CreateExtensionStmt:
			retval = _equalCreateExtensionStmt(a, b);
			break;
		case T_AlterExtensionStmt:
			retval = _equalAlterExtensionStmt(a, b);
			break;
		case T_AlterExtensionContentsStmt:
			retval = _equalAlterExtensionContentsStmt(a, b);
			break;
		case T_CreateFdwStmt:
			retval = _equalCreateFdwStmt(a, b);
			break;
		case T_AlterFdwStmt:
			retval = _equalAlterFdwStmt(a, b);
			break;
		case T_CreateForeignServerStmt:
			retval = _equalCreateForeignServerStmt(a, b);
			break;
		case T_AlterForeignServerStmt:
			retval = _equalAlterForeignServerStmt(a, b);
			break;
		case T_CreateForeignTableStmt:
			retval = _equalCreateForeignTableStmt(a, b);
			break;
		case T_CreateUserMappingStmt:
			retval = _equalCreateUserMappingStmt(a, b);
			break;
		case T_AlterUserMappingStmt:
			retval = _equalAlterUserMappingStmt(a, b);
			break;
		case T_DropUserMappingStmt:
			retval = _equalDropUserMappingStmt(a, b);
			break;
		case T_ImportForeignSchemaStmt:
			retval = _equalImportForeignSchemaStmt(a, b);
			break;
		case T_CreatePolicyStmt:
			retval = _equalCreatePolicyStmt(a, b);
			break;
		case T_AlterPolicyStmt:
			retval = _equalAlterPolicyStmt(a, b);
			break;
		case T_CreateAmStmt:
			retval = _equalCreateAmStmt(a, b);
			break;
		case T_CreateTrigStmt:
			retval = _equalCreateTrigStmt(a, b);
			break;
		case T_CreateEventTrigStmt:
			retval = _equalCreateEventTrigStmt(a, b);
			break;
		case T_AlterEventTrigStmt:
			retval = _equalAlterEventTrigStmt(a, b);
			break;
		case T_CreatePLangStmt:
			retval = _equalCreatePLangStmt(a, b);
			break;
		case T_CreateRoleStmt:
			retval = _equalCreateRoleStmt(a, b);
			break;
		case T_AlterRoleStmt:
			retval = _equalAlterRoleStmt(a, b);
			break;
		case T_AlterRoleSetStmt:
			retval = _equalAlterRoleSetStmt(a, b);
			break;
		case T_DropRoleStmt:
			retval = _equalDropRoleStmt(a, b);
			break;
		case T_CreateSeqStmt:
			retval = _equalCreateSeqStmt(a, b);
			break;
		case T_AlterSeqStmt:
			retval = _equalAlterSeqStmt(a, b);
			break;
		case T_DefineStmt:
			retval = _equalDefineStmt(a, b);
			break;
		case T_CreateDomainStmt:
			retval = _equalCreateDomainStmt(a, b);
			break;
		case T_CreateOpClassStmt:
			retval = _equalCreateOpClassStmt(a, b);
			break;
		case T_CreateOpClassItem:
			retval = _equalCreateOpClassItem(a, b);
			break;
		case T_CreateOpFamilyStmt:
			retval = _equalCreateOpFamilyStmt(a, b);
			break;
		case T_AlterOpFamilyStmt:
			retval = _equalAlterOpFamilyStmt(a, b);
			break;
		case T_DropStmt:
			retval = _equalDropStmt(a, b);
			break;
		case T_TruncateStmt:
			retval = _equalTruncateStmt(a, b);
			break;
		case T_CommentStmt:
			retval = _equalCommentStmt(a, b);
			break;
		case T_SecLabelStmt:
			retval = _equalSecLabelStmt(a, b);
			break;
		case T_DeclareCursorStmt:
			retval = _equalDeclareCursorStmt(a, b);
			break;
		case T_ClosePortalStmt:
			retval = _equalClosePortalStmt(a, b);
			break;
		case T_FetchStmt:
			retval = _equalFetchStmt(a, b);
			break;
		case T_IndexStmt:
			retval = _equalIndexStmt(a, b);
			break;
		case T_CreateStatsStmt:
			retval = _equalCreateStatsStmt(a, b);
			break;
		case T_StatsElem:
			retval = _equalStatsElem(a, b);
			break;
		case T_AlterStatsStmt:
			retval = _equalAlterStatsStmt(a, b);
			break;
		case T_CreateFunctionStmt:
			retval = _equalCreateFunctionStmt(a, b);
			break;
		case T_FunctionParameter:
			retval = _equalFunctionParameter(a, b);
			break;
		case T_AlterFunctionStmt:
			retval = _equalAlterFunctionStmt(a, b);
			break;
		case T_DoStmt:
			retval = _equalDoStmt(a, b);
			break;
		case T_CallStmt:
			retval = _equalCallStmt(a, b);
			break;
		case T_RenameStmt:
			retval = _equalRenameStmt(a, b);
			break;
		case T_AlterObjectDependsStmt:
			retval = _equalAlterObjectDependsStmt(a, b);
			break;
		case T_AlterObjectSchemaStmt:
			retval = _equalAlterObjectSchemaStmt(a, b);
			break;
		case T_AlterOwnerStmt:
			retval = _equalAlterOwnerStmt(a, b);
			break;
		case T_AlterOperatorStmt:
			retval = _equalAlterOperatorStmt(a, b);
			break;
		case T_AlterTypeStmt:
			retval = _equalAlterTypeStmt(a, b);
			break;
		case T_RuleStmt:
			retval = _equalRuleStmt(a, b);
			break;
		case T_NotifyStmt:
			retval = _equalNotifyStmt(a, b);
			break;
		case T_ListenStmt:
			retval = _equalListenStmt(a, b);
			break;
		case T_UnlistenStmt:
			retval = _equalUnlistenStmt(a, b);
			break;
		case T_TransactionStmt:
			retval = _equalTransactionStmt(a, b);
			break;
		case T_CompositeTypeStmt:
			retval = _equalCompositeTypeStmt(a, b);
			break;
		case T_CreateEnumStmt:
			retval = _equalCreateEnumStmt(a, b);
			break;
		case T_CreateRangeStmt:
			retval = _equalCreateRangeStmt(a, b);
			break;
		case T_AlterEnumStmt:
			retval = _equalAlterEnumStmt(a, b);
			break;
		case T_ViewStmt:
			retval = _equalViewStmt(a, b);
			break;
		case T_LoadStmt:
			retval = _equalLoadStmt(a, b);
			break;
		case T_CreatedbStmt:
			retval = _equalCreatedbStmt(a, b);
			break;
		case T_AlterDatabaseStmt:
			retval = _equalAlterDatabaseStmt(a, b);
			break;
		case T_AlterDatabaseRefreshCollStmt:
			retval = _equalAlterDatabaseRefreshCollStmt(a, b);
			break;
		case T_AlterDatabaseSetStmt:
			retval = _equalAlterDatabaseSetStmt(a, b);
			break;
		case T_DropdbStmt:
			retval = _equalDropdbStmt(a, b);
			break;
		case T_AlterSystemStmt:
			retval = _equalAlterSystemStmt(a, b);
			break;
		case T_ClusterStmt:
			retval = _equalClusterStmt(a, b);
			break;
		case T_VacuumStmt:
			retval = _equalVacuumStmt(a, b);
			break;
		case T_VacuumRelation:
			retval = _equalVacuumRelation(a, b);
			break;
		case T_ExplainStmt:
			retval = _equalExplainStmt(a, b);
			break;
		case T_CreateTableAsStmt:
			retval = _equalCreateTableAsStmt(a, b);
			break;
		case T_RefreshMatViewStmt:
			retval = _equalRefreshMatViewStmt(a, b);
			break;
		case T_CheckPointStmt:
			retval = _equalCheckPointStmt(a, b);
			break;
		case T_DiscardStmt:
			retval = _equalDiscardStmt(a, b);
			break;
		case T_LockStmt:
			retval = _equalLockStmt(a, b);
			break;
		case T_ConstraintsSetStmt:
			retval = _equalConstraintsSetStmt(a, b);
			break;
		case T_ReindexStmt:
			retval = _equalReindexStmt(a, b);
			break;
		case T_CreateConversionStmt:
			retval = _equalCreateConversionStmt(a, b);
			break;
		case T_CreateCastStmt:
			retval = _equalCreateCastStmt(a, b);
			break;
		case T_CreateTransformStmt:
			retval = _equalCreateTransformStmt(a, b);
			break;
		case T_PrepareStmt:
			retval = _equalPrepareStmt(a, b);
			break;
		case T_ExecuteStmt:
			retval = _equalExecuteStmt(a, b);
			break;
		case T_DeallocateStmt:
			retval = _equalDeallocateStmt(a, b);
			break;
		case T_DropOwnedStmt:
			retval = _equalDropOwnedStmt(a, b);
			break;
		case T_ReassignOwnedStmt:
			retval = _equalReassignOwnedStmt(a, b);
			break;
		case T_AlterTSDictionaryStmt:
			retval = _equalAlterTSDictionaryStmt(a, b);
			break;
		case T_AlterTSConfigurationStmt:
			retval = _equalAlterTSConfigurationStmt(a, b);
			break;
		case T_PublicationTable:
			retval = _equalPublicationTable(a, b);
			break;
		case T_PublicationObjSpec:
			retval = _equalPublicationObjSpec(a, b);
			break;
		case T_CreatePublicationStmt:
			retval = _equalCreatePublicationStmt(a, b);
			break;
		case T_AlterPublicationStmt:
			retval = _equalAlterPublicationStmt(a, b);
			break;
		case T_CreateSubscriptionStmt:
			retval = _equalCreateSubscriptionStmt(a, b);
			break;
		case T_AlterSubscriptionStmt:
			retval = _equalAlterSubscriptionStmt(a, b);
			break;
		case T_DropSubscriptionStmt:
			retval = _equalDropSubscriptionStmt(a, b);
			break;
		case T_PathKey:
			retval = _equalPathKey(a, b);
			break;
		case T_GroupByOrdering:
			retval = _equalGroupByOrdering(a, b);
			break;
		case T_RestrictInfo:
			retval = _equalRestrictInfo(a, b);
			break;
		case T_PlaceHolderVar:
			retval = _equalPlaceHolderVar(a, b);
			break;
		case T_SpecialJoinInfo:
			retval = _equalSpecialJoinInfo(a, b);
			break;
		case T_AppendRelInfo:
			retval = _equalAppendRelInfo(a, b);
			break;
		case T_PlaceHolderInfo:
			retval = _equalPlaceHolderInfo(a, b);
			break;
		case T_Bitmapset:
			retval = _equalBitmapset(a, b);
			break;
		case T_ExtensibleNode:
			retval = _equalExtensibleNode(a, b);
			break;
		case T_Integer:
			retval = _equalInteger(a, b);
			break;
		case T_Float:
			retval = _equalFloat(a, b);
			break;
		case T_Boolean:
			retval = _equalBoolean(a, b);
			break;
		case T_String:
			retval = _equalString(a, b);
			break;
		case T_BitString:
			retval = _equalBitString(a, b);
			break;
