/* Generated by the protocol buffer compiler.  DO NOT EDIT! */
/* Generated from: protobuf/pg_query.proto */

/* Do not generate deprecated warnings for self */
#ifndef PROTOBUF_C__NO_DEPRECATED
#define PROTOBUF_C__NO_DEPRECATED
#endif

#include "protobuf/pg_query.pb-c.h"
void   pg_query__parse_result__init
                     (PgQuery__ParseResult         *message)
{
  static const PgQuery__ParseResult init_value = PG_QUERY__PARSE_RESULT__INIT;
  *message = init_value;
}
size_t pg_query__parse_result__get_packed_size
                     (const PgQuery__ParseResult *message)
{
  assert(message->base.descriptor == &pg_query__parse_result__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__parse_result__pack
                     (const PgQuery__ParseResult *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__parse_result__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__parse_result__pack_to_buffer
                     (const PgQuery__ParseResult *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__parse_result__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__ParseResult *
       pg_query__parse_result__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__ParseResult *)
     protobuf_c_message_unpack (&pg_query__parse_result__descriptor,
                                allocator, len, data);
}
void   pg_query__parse_result__free_unpacked
                     (PgQuery__ParseResult *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__parse_result__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__scan_result__init
                     (PgQuery__ScanResult         *message)
{
  static const PgQuery__ScanResult init_value = PG_QUERY__SCAN_RESULT__INIT;
  *message = init_value;
}
size_t pg_query__scan_result__get_packed_size
                     (const PgQuery__ScanResult *message)
{
  assert(message->base.descriptor == &pg_query__scan_result__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__scan_result__pack
                     (const PgQuery__ScanResult *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__scan_result__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__scan_result__pack_to_buffer
                     (const PgQuery__ScanResult *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__scan_result__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__ScanResult *
       pg_query__scan_result__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__ScanResult *)
     protobuf_c_message_unpack (&pg_query__scan_result__descriptor,
                                allocator, len, data);
}
void   pg_query__scan_result__free_unpacked
                     (PgQuery__ScanResult *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__scan_result__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__node__init
                     (PgQuery__Node         *message)
{
  static const PgQuery__Node init_value = PG_QUERY__NODE__INIT;
  *message = init_value;
}
size_t pg_query__node__get_packed_size
                     (const PgQuery__Node *message)
{
  assert(message->base.descriptor == &pg_query__node__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__node__pack
                     (const PgQuery__Node *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__node__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__node__pack_to_buffer
                     (const PgQuery__Node *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__node__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__Node *
       pg_query__node__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__Node *)
     protobuf_c_message_unpack (&pg_query__node__descriptor,
                                allocator, len, data);
}
void   pg_query__node__free_unpacked
                     (PgQuery__Node *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__node__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__integer__init
                     (PgQuery__Integer         *message)
{
  static const PgQuery__Integer init_value = PG_QUERY__INTEGER__INIT;
  *message = init_value;
}
size_t pg_query__integer__get_packed_size
                     (const PgQuery__Integer *message)
{
  assert(message->base.descriptor == &pg_query__integer__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__integer__pack
                     (const PgQuery__Integer *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__integer__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__integer__pack_to_buffer
                     (const PgQuery__Integer *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__integer__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__Integer *
       pg_query__integer__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__Integer *)
     protobuf_c_message_unpack (&pg_query__integer__descriptor,
                                allocator, len, data);
}
void   pg_query__integer__free_unpacked
                     (PgQuery__Integer *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__integer__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__float__init
                     (PgQuery__Float         *message)
{
  static const PgQuery__Float init_value = PG_QUERY__FLOAT__INIT;
  *message = init_value;
}
size_t pg_query__float__get_packed_size
                     (const PgQuery__Float *message)
{
  assert(message->base.descriptor == &pg_query__float__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__float__pack
                     (const PgQuery__Float *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__float__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__float__pack_to_buffer
                     (const PgQuery__Float *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__float__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__Float *
       pg_query__float__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__Float *)
     protobuf_c_message_unpack (&pg_query__float__descriptor,
                                allocator, len, data);
}
void   pg_query__float__free_unpacked
                     (PgQuery__Float *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__float__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__boolean__init
                     (PgQuery__Boolean         *message)
{
  static const PgQuery__Boolean init_value = PG_QUERY__BOOLEAN__INIT;
  *message = init_value;
}
size_t pg_query__boolean__get_packed_size
                     (const PgQuery__Boolean *message)
{
  assert(message->base.descriptor == &pg_query__boolean__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__boolean__pack
                     (const PgQuery__Boolean *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__boolean__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__boolean__pack_to_buffer
                     (const PgQuery__Boolean *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__boolean__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__Boolean *
       pg_query__boolean__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__Boolean *)
     protobuf_c_message_unpack (&pg_query__boolean__descriptor,
                                allocator, len, data);
}
void   pg_query__boolean__free_unpacked
                     (PgQuery__Boolean *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__boolean__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__string__init
                     (PgQuery__String         *message)
{
  static const PgQuery__String init_value = PG_QUERY__STRING__INIT;
  *message = init_value;
}
size_t pg_query__string__get_packed_size
                     (const PgQuery__String *message)
{
  assert(message->base.descriptor == &pg_query__string__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__string__pack
                     (const PgQuery__String *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__string__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__string__pack_to_buffer
                     (const PgQuery__String *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__string__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__String *
       pg_query__string__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__String *)
     protobuf_c_message_unpack (&pg_query__string__descriptor,
                                allocator, len, data);
}
void   pg_query__string__free_unpacked
                     (PgQuery__String *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__string__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__bit_string__init
                     (PgQuery__BitString         *message)
{
  static const PgQuery__BitString init_value = PG_QUERY__BIT_STRING__INIT;
  *message = init_value;
}
size_t pg_query__bit_string__get_packed_size
                     (const PgQuery__BitString *message)
{
  assert(message->base.descriptor == &pg_query__bit_string__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__bit_string__pack
                     (const PgQuery__BitString *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__bit_string__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__bit_string__pack_to_buffer
                     (const PgQuery__BitString *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__bit_string__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__BitString *
       pg_query__bit_string__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__BitString *)
     protobuf_c_message_unpack (&pg_query__bit_string__descriptor,
                                allocator, len, data);
}
void   pg_query__bit_string__free_unpacked
                     (PgQuery__BitString *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__bit_string__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__list__init
                     (PgQuery__List         *message)
{
  static const PgQuery__List init_value = PG_QUERY__LIST__INIT;
  *message = init_value;
}
size_t pg_query__list__get_packed_size
                     (const PgQuery__List *message)
{
  assert(message->base.descriptor == &pg_query__list__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__list__pack
                     (const PgQuery__List *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__list__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__list__pack_to_buffer
                     (const PgQuery__List *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__list__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__List *
       pg_query__list__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__List *)
     protobuf_c_message_unpack (&pg_query__list__descriptor,
                                allocator, len, data);
}
void   pg_query__list__free_unpacked
                     (PgQuery__List *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__list__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__oid_list__init
                     (PgQuery__OidList         *message)
{
  static const PgQuery__OidList init_value = PG_QUERY__OID_LIST__INIT;
  *message = init_value;
}
size_t pg_query__oid_list__get_packed_size
                     (const PgQuery__OidList *message)
{
  assert(message->base.descriptor == &pg_query__oid_list__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__oid_list__pack
                     (const PgQuery__OidList *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__oid_list__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__oid_list__pack_to_buffer
                     (const PgQuery__OidList *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__oid_list__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__OidList *
       pg_query__oid_list__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__OidList *)
     protobuf_c_message_unpack (&pg_query__oid_list__descriptor,
                                allocator, len, data);
}
void   pg_query__oid_list__free_unpacked
                     (PgQuery__OidList *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__oid_list__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__int_list__init
                     (PgQuery__IntList         *message)
{
  static const PgQuery__IntList init_value = PG_QUERY__INT_LIST__INIT;
  *message = init_value;
}
size_t pg_query__int_list__get_packed_size
                     (const PgQuery__IntList *message)
{
  assert(message->base.descriptor == &pg_query__int_list__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__int_list__pack
                     (const PgQuery__IntList *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__int_list__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__int_list__pack_to_buffer
                     (const PgQuery__IntList *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__int_list__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__IntList *
       pg_query__int_list__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__IntList *)
     protobuf_c_message_unpack (&pg_query__int_list__descriptor,
                                allocator, len, data);
}
void   pg_query__int_list__free_unpacked
                     (PgQuery__IntList *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__int_list__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__a__const__init
                     (PgQuery__AConst         *message)
{
  static const PgQuery__AConst init_value = PG_QUERY__A__CONST__INIT;
  *message = init_value;
}
size_t pg_query__a__const__get_packed_size
                     (const PgQuery__AConst *message)
{
  assert(message->base.descriptor == &pg_query__a__const__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__a__const__pack
                     (const PgQuery__AConst *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__a__const__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__a__const__pack_to_buffer
                     (const PgQuery__AConst *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__a__const__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AConst *
       pg_query__a__const__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AConst *)
     protobuf_c_message_unpack (&pg_query__a__const__descriptor,
                                allocator, len, data);
}
void   pg_query__a__const__free_unpacked
                     (PgQuery__AConst *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__a__const__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__alias__init
                     (PgQuery__Alias         *message)
{
  static const PgQuery__Alias init_value = PG_QUERY__ALIAS__INIT;
  *message = init_value;
}
size_t pg_query__alias__get_packed_size
                     (const PgQuery__Alias *message)
{
  assert(message->base.descriptor == &pg_query__alias__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__alias__pack
                     (const PgQuery__Alias *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__alias__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__alias__pack_to_buffer
                     (const PgQuery__Alias *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__alias__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__Alias *
       pg_query__alias__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__Alias *)
     protobuf_c_message_unpack (&pg_query__alias__descriptor,
                                allocator, len, data);
}
void   pg_query__alias__free_unpacked
                     (PgQuery__Alias *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__alias__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__range_var__init
                     (PgQuery__RangeVar         *message)
{
  static const PgQuery__RangeVar init_value = PG_QUERY__RANGE_VAR__INIT;
  *message = init_value;
}
size_t pg_query__range_var__get_packed_size
                     (const PgQuery__RangeVar *message)
{
  assert(message->base.descriptor == &pg_query__range_var__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__range_var__pack
                     (const PgQuery__RangeVar *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__range_var__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__range_var__pack_to_buffer
                     (const PgQuery__RangeVar *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__range_var__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__RangeVar *
       pg_query__range_var__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__RangeVar *)
     protobuf_c_message_unpack (&pg_query__range_var__descriptor,
                                allocator, len, data);
}
void   pg_query__range_var__free_unpacked
                     (PgQuery__RangeVar *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__range_var__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__table_func__init
                     (PgQuery__TableFunc         *message)
{
  static const PgQuery__TableFunc init_value = PG_QUERY__TABLE_FUNC__INIT;
  *message = init_value;
}
size_t pg_query__table_func__get_packed_size
                     (const PgQuery__TableFunc *message)
{
  assert(message->base.descriptor == &pg_query__table_func__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__table_func__pack
                     (const PgQuery__TableFunc *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__table_func__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__table_func__pack_to_buffer
                     (const PgQuery__TableFunc *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__table_func__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__TableFunc *
       pg_query__table_func__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__TableFunc *)
     protobuf_c_message_unpack (&pg_query__table_func__descriptor,
                                allocator, len, data);
}
void   pg_query__table_func__free_unpacked
                     (PgQuery__TableFunc *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__table_func__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__into_clause__init
                     (PgQuery__IntoClause         *message)
{
  static const PgQuery__IntoClause init_value = PG_QUERY__INTO_CLAUSE__INIT;
  *message = init_value;
}
size_t pg_query__into_clause__get_packed_size
                     (const PgQuery__IntoClause *message)
{
  assert(message->base.descriptor == &pg_query__into_clause__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__into_clause__pack
                     (const PgQuery__IntoClause *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__into_clause__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__into_clause__pack_to_buffer
                     (const PgQuery__IntoClause *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__into_clause__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__IntoClause *
       pg_query__into_clause__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__IntoClause *)
     protobuf_c_message_unpack (&pg_query__into_clause__descriptor,
                                allocator, len, data);
}
void   pg_query__into_clause__free_unpacked
                     (PgQuery__IntoClause *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__into_clause__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__var__init
                     (PgQuery__Var         *message)
{
  static const PgQuery__Var init_value = PG_QUERY__VAR__INIT;
  *message = init_value;
}
size_t pg_query__var__get_packed_size
                     (const PgQuery__Var *message)
{
  assert(message->base.descriptor == &pg_query__var__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__var__pack
                     (const PgQuery__Var *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__var__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__var__pack_to_buffer
                     (const PgQuery__Var *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__var__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__Var *
       pg_query__var__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__Var *)
     protobuf_c_message_unpack (&pg_query__var__descriptor,
                                allocator, len, data);
}
void   pg_query__var__free_unpacked
                     (PgQuery__Var *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__var__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__param__init
                     (PgQuery__Param         *message)
{
  static const PgQuery__Param init_value = PG_QUERY__PARAM__INIT;
  *message = init_value;
}
size_t pg_query__param__get_packed_size
                     (const PgQuery__Param *message)
{
  assert(message->base.descriptor == &pg_query__param__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__param__pack
                     (const PgQuery__Param *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__param__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__param__pack_to_buffer
                     (const PgQuery__Param *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__param__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__Param *
       pg_query__param__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__Param *)
     protobuf_c_message_unpack (&pg_query__param__descriptor,
                                allocator, len, data);
}
void   pg_query__param__free_unpacked
                     (PgQuery__Param *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__param__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__aggref__init
                     (PgQuery__Aggref         *message)
{
  static const PgQuery__Aggref init_value = PG_QUERY__AGGREF__INIT;
  *message = init_value;
}
size_t pg_query__aggref__get_packed_size
                     (const PgQuery__Aggref *message)
{
  assert(message->base.descriptor == &pg_query__aggref__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__aggref__pack
                     (const PgQuery__Aggref *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__aggref__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__aggref__pack_to_buffer
                     (const PgQuery__Aggref *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__aggref__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__Aggref *
       pg_query__aggref__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__Aggref *)
     protobuf_c_message_unpack (&pg_query__aggref__descriptor,
                                allocator, len, data);
}
void   pg_query__aggref__free_unpacked
                     (PgQuery__Aggref *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__aggref__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__grouping_func__init
                     (PgQuery__GroupingFunc         *message)
{
  static const PgQuery__GroupingFunc init_value = PG_QUERY__GROUPING_FUNC__INIT;
  *message = init_value;
}
size_t pg_query__grouping_func__get_packed_size
                     (const PgQuery__GroupingFunc *message)
{
  assert(message->base.descriptor == &pg_query__grouping_func__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__grouping_func__pack
                     (const PgQuery__GroupingFunc *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__grouping_func__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__grouping_func__pack_to_buffer
                     (const PgQuery__GroupingFunc *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__grouping_func__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__GroupingFunc *
       pg_query__grouping_func__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__GroupingFunc *)
     protobuf_c_message_unpack (&pg_query__grouping_func__descriptor,
                                allocator, len, data);
}
void   pg_query__grouping_func__free_unpacked
                     (PgQuery__GroupingFunc *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__grouping_func__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__window_func__init
                     (PgQuery__WindowFunc         *message)
{
  static const PgQuery__WindowFunc init_value = PG_QUERY__WINDOW_FUNC__INIT;
  *message = init_value;
}
size_t pg_query__window_func__get_packed_size
                     (const PgQuery__WindowFunc *message)
{
  assert(message->base.descriptor == &pg_query__window_func__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__window_func__pack
                     (const PgQuery__WindowFunc *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__window_func__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__window_func__pack_to_buffer
                     (const PgQuery__WindowFunc *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__window_func__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__WindowFunc *
       pg_query__window_func__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__WindowFunc *)
     protobuf_c_message_unpack (&pg_query__window_func__descriptor,
                                allocator, len, data);
}
void   pg_query__window_func__free_unpacked
                     (PgQuery__WindowFunc *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__window_func__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__window_func_run_condition__init
                     (PgQuery__WindowFuncRunCondition         *message)
{
  static const PgQuery__WindowFuncRunCondition init_value = PG_QUERY__WINDOW_FUNC_RUN_CONDITION__INIT;
  *message = init_value;
}
size_t pg_query__window_func_run_condition__get_packed_size
                     (const PgQuery__WindowFuncRunCondition *message)
{
  assert(message->base.descriptor == &pg_query__window_func_run_condition__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__window_func_run_condition__pack
                     (const PgQuery__WindowFuncRunCondition *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__window_func_run_condition__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__window_func_run_condition__pack_to_buffer
                     (const PgQuery__WindowFuncRunCondition *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__window_func_run_condition__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__WindowFuncRunCondition *
       pg_query__window_func_run_condition__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__WindowFuncRunCondition *)
     protobuf_c_message_unpack (&pg_query__window_func_run_condition__descriptor,
                                allocator, len, data);
}
void   pg_query__window_func_run_condition__free_unpacked
                     (PgQuery__WindowFuncRunCondition *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__window_func_run_condition__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__merge_support_func__init
                     (PgQuery__MergeSupportFunc         *message)
{
  static const PgQuery__MergeSupportFunc init_value = PG_QUERY__MERGE_SUPPORT_FUNC__INIT;
  *message = init_value;
}
size_t pg_query__merge_support_func__get_packed_size
                     (const PgQuery__MergeSupportFunc *message)
{
  assert(message->base.descriptor == &pg_query__merge_support_func__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__merge_support_func__pack
                     (const PgQuery__MergeSupportFunc *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__merge_support_func__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__merge_support_func__pack_to_buffer
                     (const PgQuery__MergeSupportFunc *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__merge_support_func__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__MergeSupportFunc *
       pg_query__merge_support_func__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__MergeSupportFunc *)
     protobuf_c_message_unpack (&pg_query__merge_support_func__descriptor,
                                allocator, len, data);
}
void   pg_query__merge_support_func__free_unpacked
                     (PgQuery__MergeSupportFunc *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__merge_support_func__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__subscripting_ref__init
                     (PgQuery__SubscriptingRef         *message)
{
  static const PgQuery__SubscriptingRef init_value = PG_QUERY__SUBSCRIPTING_REF__INIT;
  *message = init_value;
}
size_t pg_query__subscripting_ref__get_packed_size
                     (const PgQuery__SubscriptingRef *message)
{
  assert(message->base.descriptor == &pg_query__subscripting_ref__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__subscripting_ref__pack
                     (const PgQuery__SubscriptingRef *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__subscripting_ref__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__subscripting_ref__pack_to_buffer
                     (const PgQuery__SubscriptingRef *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__subscripting_ref__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__SubscriptingRef *
       pg_query__subscripting_ref__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__SubscriptingRef *)
     protobuf_c_message_unpack (&pg_query__subscripting_ref__descriptor,
                                allocator, len, data);
}
void   pg_query__subscripting_ref__free_unpacked
                     (PgQuery__SubscriptingRef *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__subscripting_ref__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__func_expr__init
                     (PgQuery__FuncExpr         *message)
{
  static const PgQuery__FuncExpr init_value = PG_QUERY__FUNC_EXPR__INIT;
  *message = init_value;
}
size_t pg_query__func_expr__get_packed_size
                     (const PgQuery__FuncExpr *message)
{
  assert(message->base.descriptor == &pg_query__func_expr__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__func_expr__pack
                     (const PgQuery__FuncExpr *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__func_expr__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__func_expr__pack_to_buffer
                     (const PgQuery__FuncExpr *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__func_expr__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__FuncExpr *
       pg_query__func_expr__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__FuncExpr *)
     protobuf_c_message_unpack (&pg_query__func_expr__descriptor,
                                allocator, len, data);
}
void   pg_query__func_expr__free_unpacked
                     (PgQuery__FuncExpr *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__func_expr__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__named_arg_expr__init
                     (PgQuery__NamedArgExpr         *message)
{
  static const PgQuery__NamedArgExpr init_value = PG_QUERY__NAMED_ARG_EXPR__INIT;
  *message = init_value;
}
size_t pg_query__named_arg_expr__get_packed_size
                     (const PgQuery__NamedArgExpr *message)
{
  assert(message->base.descriptor == &pg_query__named_arg_expr__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__named_arg_expr__pack
                     (const PgQuery__NamedArgExpr *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__named_arg_expr__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__named_arg_expr__pack_to_buffer
                     (const PgQuery__NamedArgExpr *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__named_arg_expr__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__NamedArgExpr *
       pg_query__named_arg_expr__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__NamedArgExpr *)
     protobuf_c_message_unpack (&pg_query__named_arg_expr__descriptor,
                                allocator, len, data);
}
void   pg_query__named_arg_expr__free_unpacked
                     (PgQuery__NamedArgExpr *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__named_arg_expr__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__op_expr__init
                     (PgQuery__OpExpr         *message)
{
  static const PgQuery__OpExpr init_value = PG_QUERY__OP_EXPR__INIT;
  *message = init_value;
}
size_t pg_query__op_expr__get_packed_size
                     (const PgQuery__OpExpr *message)
{
  assert(message->base.descriptor == &pg_query__op_expr__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__op_expr__pack
                     (const PgQuery__OpExpr *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__op_expr__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__op_expr__pack_to_buffer
                     (const PgQuery__OpExpr *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__op_expr__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__OpExpr *
       pg_query__op_expr__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__OpExpr *)
     protobuf_c_message_unpack (&pg_query__op_expr__descriptor,
                                allocator, len, data);
}
void   pg_query__op_expr__free_unpacked
                     (PgQuery__OpExpr *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__op_expr__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__distinct_expr__init
                     (PgQuery__DistinctExpr         *message)
{
  static const PgQuery__DistinctExpr init_value = PG_QUERY__DISTINCT_EXPR__INIT;
  *message = init_value;
}
size_t pg_query__distinct_expr__get_packed_size
                     (const PgQuery__DistinctExpr *message)
{
  assert(message->base.descriptor == &pg_query__distinct_expr__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__distinct_expr__pack
                     (const PgQuery__DistinctExpr *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__distinct_expr__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__distinct_expr__pack_to_buffer
                     (const PgQuery__DistinctExpr *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__distinct_expr__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__DistinctExpr *
       pg_query__distinct_expr__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__DistinctExpr *)
     protobuf_c_message_unpack (&pg_query__distinct_expr__descriptor,
                                allocator, len, data);
}
void   pg_query__distinct_expr__free_unpacked
                     (PgQuery__DistinctExpr *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__distinct_expr__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__null_if_expr__init
                     (PgQuery__NullIfExpr         *message)
{
  static const PgQuery__NullIfExpr init_value = PG_QUERY__NULL_IF_EXPR__INIT;
  *message = init_value;
}
size_t pg_query__null_if_expr__get_packed_size
                     (const PgQuery__NullIfExpr *message)
{
  assert(message->base.descriptor == &pg_query__null_if_expr__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__null_if_expr__pack
                     (const PgQuery__NullIfExpr *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__null_if_expr__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__null_if_expr__pack_to_buffer
                     (const PgQuery__NullIfExpr *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__null_if_expr__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__NullIfExpr *
       pg_query__null_if_expr__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__NullIfExpr *)
     protobuf_c_message_unpack (&pg_query__null_if_expr__descriptor,
                                allocator, len, data);
}
void   pg_query__null_if_expr__free_unpacked
                     (PgQuery__NullIfExpr *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__null_if_expr__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__scalar_array_op_expr__init
                     (PgQuery__ScalarArrayOpExpr         *message)
{
  static const PgQuery__ScalarArrayOpExpr init_value = PG_QUERY__SCALAR_ARRAY_OP_EXPR__INIT;
  *message = init_value;
}
size_t pg_query__scalar_array_op_expr__get_packed_size
                     (const PgQuery__ScalarArrayOpExpr *message)
{
  assert(message->base.descriptor == &pg_query__scalar_array_op_expr__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__scalar_array_op_expr__pack
                     (const PgQuery__ScalarArrayOpExpr *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__scalar_array_op_expr__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__scalar_array_op_expr__pack_to_buffer
                     (const PgQuery__ScalarArrayOpExpr *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__scalar_array_op_expr__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__ScalarArrayOpExpr *
       pg_query__scalar_array_op_expr__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__ScalarArrayOpExpr *)
     protobuf_c_message_unpack (&pg_query__scalar_array_op_expr__descriptor,
                                allocator, len, data);
}
void   pg_query__scalar_array_op_expr__free_unpacked
                     (PgQuery__ScalarArrayOpExpr *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__scalar_array_op_expr__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__bool_expr__init
                     (PgQuery__BoolExpr         *message)
{
  static const PgQuery__BoolExpr init_value = PG_QUERY__BOOL_EXPR__INIT;
  *message = init_value;
}
size_t pg_query__bool_expr__get_packed_size
                     (const PgQuery__BoolExpr *message)
{
  assert(message->base.descriptor == &pg_query__bool_expr__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__bool_expr__pack
                     (const PgQuery__BoolExpr *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__bool_expr__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__bool_expr__pack_to_buffer
                     (const PgQuery__BoolExpr *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__bool_expr__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__BoolExpr *
       pg_query__bool_expr__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__BoolExpr *)
     protobuf_c_message_unpack (&pg_query__bool_expr__descriptor,
                                allocator, len, data);
}
void   pg_query__bool_expr__free_unpacked
                     (PgQuery__BoolExpr *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__bool_expr__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__sub_link__init
                     (PgQuery__SubLink         *message)
{
  static const PgQuery__SubLink init_value = PG_QUERY__SUB_LINK__INIT;
  *message = init_value;
}
size_t pg_query__sub_link__get_packed_size
                     (const PgQuery__SubLink *message)
{
  assert(message->base.descriptor == &pg_query__sub_link__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__sub_link__pack
                     (const PgQuery__SubLink *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__sub_link__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__sub_link__pack_to_buffer
                     (const PgQuery__SubLink *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__sub_link__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__SubLink *
       pg_query__sub_link__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__SubLink *)
     protobuf_c_message_unpack (&pg_query__sub_link__descriptor,
                                allocator, len, data);
}
void   pg_query__sub_link__free_unpacked
                     (PgQuery__SubLink *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__sub_link__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__sub_plan__init
                     (PgQuery__SubPlan         *message)
{
  static const PgQuery__SubPlan init_value = PG_QUERY__SUB_PLAN__INIT;
  *message = init_value;
}
size_t pg_query__sub_plan__get_packed_size
                     (const PgQuery__SubPlan *message)
{
  assert(message->base.descriptor == &pg_query__sub_plan__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__sub_plan__pack
                     (const PgQuery__SubPlan *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__sub_plan__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__sub_plan__pack_to_buffer
                     (const PgQuery__SubPlan *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__sub_plan__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__SubPlan *
       pg_query__sub_plan__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__SubPlan *)
     protobuf_c_message_unpack (&pg_query__sub_plan__descriptor,
                                allocator, len, data);
}
void   pg_query__sub_plan__free_unpacked
                     (PgQuery__SubPlan *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__sub_plan__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__alternative_sub_plan__init
                     (PgQuery__AlternativeSubPlan         *message)
{
  static const PgQuery__AlternativeSubPlan init_value = PG_QUERY__ALTERNATIVE_SUB_PLAN__INIT;
  *message = init_value;
}
size_t pg_query__alternative_sub_plan__get_packed_size
                     (const PgQuery__AlternativeSubPlan *message)
{
  assert(message->base.descriptor == &pg_query__alternative_sub_plan__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__alternative_sub_plan__pack
                     (const PgQuery__AlternativeSubPlan *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__alternative_sub_plan__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__alternative_sub_plan__pack_to_buffer
                     (const PgQuery__AlternativeSubPlan *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__alternative_sub_plan__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AlternativeSubPlan *
       pg_query__alternative_sub_plan__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AlternativeSubPlan *)
     protobuf_c_message_unpack (&pg_query__alternative_sub_plan__descriptor,
                                allocator, len, data);
}
void   pg_query__alternative_sub_plan__free_unpacked
                     (PgQuery__AlternativeSubPlan *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__alternative_sub_plan__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__field_select__init
                     (PgQuery__FieldSelect         *message)
{
  static const PgQuery__FieldSelect init_value = PG_QUERY__FIELD_SELECT__INIT;
  *message = init_value;
}
size_t pg_query__field_select__get_packed_size
                     (const PgQuery__FieldSelect *message)
{
  assert(message->base.descriptor == &pg_query__field_select__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__field_select__pack
                     (const PgQuery__FieldSelect *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__field_select__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__field_select__pack_to_buffer
                     (const PgQuery__FieldSelect *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__field_select__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__FieldSelect *
       pg_query__field_select__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__FieldSelect *)
     protobuf_c_message_unpack (&pg_query__field_select__descriptor,
                                allocator, len, data);
}
void   pg_query__field_select__free_unpacked
                     (PgQuery__FieldSelect *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__field_select__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__field_store__init
                     (PgQuery__FieldStore         *message)
{
  static const PgQuery__FieldStore init_value = PG_QUERY__FIELD_STORE__INIT;
  *message = init_value;
}
size_t pg_query__field_store__get_packed_size
                     (const PgQuery__FieldStore *message)
{
  assert(message->base.descriptor == &pg_query__field_store__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__field_store__pack
                     (const PgQuery__FieldStore *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__field_store__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__field_store__pack_to_buffer
                     (const PgQuery__FieldStore *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__field_store__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__FieldStore *
       pg_query__field_store__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__FieldStore *)
     protobuf_c_message_unpack (&pg_query__field_store__descriptor,
                                allocator, len, data);
}
void   pg_query__field_store__free_unpacked
                     (PgQuery__FieldStore *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__field_store__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__relabel_type__init
                     (PgQuery__RelabelType         *message)
{
  static const PgQuery__RelabelType init_value = PG_QUERY__RELABEL_TYPE__INIT;
  *message = init_value;
}
size_t pg_query__relabel_type__get_packed_size
                     (const PgQuery__RelabelType *message)
{
  assert(message->base.descriptor == &pg_query__relabel_type__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__relabel_type__pack
                     (const PgQuery__RelabelType *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__relabel_type__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__relabel_type__pack_to_buffer
                     (const PgQuery__RelabelType *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__relabel_type__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__RelabelType *
       pg_query__relabel_type__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__RelabelType *)
     protobuf_c_message_unpack (&pg_query__relabel_type__descriptor,
                                allocator, len, data);
}
void   pg_query__relabel_type__free_unpacked
                     (PgQuery__RelabelType *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__relabel_type__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__coerce_via_io__init
                     (PgQuery__CoerceViaIO         *message)
{
  static const PgQuery__CoerceViaIO init_value = PG_QUERY__COERCE_VIA_IO__INIT;
  *message = init_value;
}
size_t pg_query__coerce_via_io__get_packed_size
                     (const PgQuery__CoerceViaIO *message)
{
  assert(message->base.descriptor == &pg_query__coerce_via_io__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__coerce_via_io__pack
                     (const PgQuery__CoerceViaIO *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__coerce_via_io__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__coerce_via_io__pack_to_buffer
                     (const PgQuery__CoerceViaIO *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__coerce_via_io__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CoerceViaIO *
       pg_query__coerce_via_io__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CoerceViaIO *)
     protobuf_c_message_unpack (&pg_query__coerce_via_io__descriptor,
                                allocator, len, data);
}
void   pg_query__coerce_via_io__free_unpacked
                     (PgQuery__CoerceViaIO *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__coerce_via_io__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__array_coerce_expr__init
                     (PgQuery__ArrayCoerceExpr         *message)
{
  static const PgQuery__ArrayCoerceExpr init_value = PG_QUERY__ARRAY_COERCE_EXPR__INIT;
  *message = init_value;
}
size_t pg_query__array_coerce_expr__get_packed_size
                     (const PgQuery__ArrayCoerceExpr *message)
{
  assert(message->base.descriptor == &pg_query__array_coerce_expr__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__array_coerce_expr__pack
                     (const PgQuery__ArrayCoerceExpr *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__array_coerce_expr__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__array_coerce_expr__pack_to_buffer
                     (const PgQuery__ArrayCoerceExpr *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__array_coerce_expr__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__ArrayCoerceExpr *
       pg_query__array_coerce_expr__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__ArrayCoerceExpr *)
     protobuf_c_message_unpack (&pg_query__array_coerce_expr__descriptor,
                                allocator, len, data);
}
void   pg_query__array_coerce_expr__free_unpacked
                     (PgQuery__ArrayCoerceExpr *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__array_coerce_expr__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__convert_rowtype_expr__init
                     (PgQuery__ConvertRowtypeExpr         *message)
{
  static const PgQuery__ConvertRowtypeExpr init_value = PG_QUERY__CONVERT_ROWTYPE_EXPR__INIT;
  *message = init_value;
}
size_t pg_query__convert_rowtype_expr__get_packed_size
                     (const PgQuery__ConvertRowtypeExpr *message)
{
  assert(message->base.descriptor == &pg_query__convert_rowtype_expr__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__convert_rowtype_expr__pack
                     (const PgQuery__ConvertRowtypeExpr *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__convert_rowtype_expr__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__convert_rowtype_expr__pack_to_buffer
                     (const PgQuery__ConvertRowtypeExpr *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__convert_rowtype_expr__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__ConvertRowtypeExpr *
       pg_query__convert_rowtype_expr__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__ConvertRowtypeExpr *)
     protobuf_c_message_unpack (&pg_query__convert_rowtype_expr__descriptor,
                                allocator, len, data);
}
void   pg_query__convert_rowtype_expr__free_unpacked
                     (PgQuery__ConvertRowtypeExpr *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__convert_rowtype_expr__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__collate_expr__init
                     (PgQuery__CollateExpr         *message)
{
  static const PgQuery__CollateExpr init_value = PG_QUERY__COLLATE_EXPR__INIT;
  *message = init_value;
}
size_t pg_query__collate_expr__get_packed_size
                     (const PgQuery__CollateExpr *message)
{
  assert(message->base.descriptor == &pg_query__collate_expr__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__collate_expr__pack
                     (const PgQuery__CollateExpr *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__collate_expr__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__collate_expr__pack_to_buffer
                     (const PgQuery__CollateExpr *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__collate_expr__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CollateExpr *
       pg_query__collate_expr__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CollateExpr *)
     protobuf_c_message_unpack (&pg_query__collate_expr__descriptor,
                                allocator, len, data);
}
void   pg_query__collate_expr__free_unpacked
                     (PgQuery__CollateExpr *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__collate_expr__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__case_expr__init
                     (PgQuery__CaseExpr         *message)
{
  static const PgQuery__CaseExpr init_value = PG_QUERY__CASE_EXPR__INIT;
  *message = init_value;
}
size_t pg_query__case_expr__get_packed_size
                     (const PgQuery__CaseExpr *message)
{
  assert(message->base.descriptor == &pg_query__case_expr__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__case_expr__pack
                     (const PgQuery__CaseExpr *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__case_expr__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__case_expr__pack_to_buffer
                     (const PgQuery__CaseExpr *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__case_expr__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CaseExpr *
       pg_query__case_expr__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CaseExpr *)
     protobuf_c_message_unpack (&pg_query__case_expr__descriptor,
                                allocator, len, data);
}
void   pg_query__case_expr__free_unpacked
                     (PgQuery__CaseExpr *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__case_expr__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__case_when__init
                     (PgQuery__CaseWhen         *message)
{
  static const PgQuery__CaseWhen init_value = PG_QUERY__CASE_WHEN__INIT;
  *message = init_value;
}
size_t pg_query__case_when__get_packed_size
                     (const PgQuery__CaseWhen *message)
{
  assert(message->base.descriptor == &pg_query__case_when__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__case_when__pack
                     (const PgQuery__CaseWhen *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__case_when__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__case_when__pack_to_buffer
                     (const PgQuery__CaseWhen *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__case_when__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CaseWhen *
       pg_query__case_when__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CaseWhen *)
     protobuf_c_message_unpack (&pg_query__case_when__descriptor,
                                allocator, len, data);
}
void   pg_query__case_when__free_unpacked
                     (PgQuery__CaseWhen *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__case_when__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__case_test_expr__init
                     (PgQuery__CaseTestExpr         *message)
{
  static const PgQuery__CaseTestExpr init_value = PG_QUERY__CASE_TEST_EXPR__INIT;
  *message = init_value;
}
size_t pg_query__case_test_expr__get_packed_size
                     (const PgQuery__CaseTestExpr *message)
{
  assert(message->base.descriptor == &pg_query__case_test_expr__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__case_test_expr__pack
                     (const PgQuery__CaseTestExpr *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__case_test_expr__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__case_test_expr__pack_to_buffer
                     (const PgQuery__CaseTestExpr *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__case_test_expr__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CaseTestExpr *
       pg_query__case_test_expr__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CaseTestExpr *)
     protobuf_c_message_unpack (&pg_query__case_test_expr__descriptor,
                                allocator, len, data);
}
void   pg_query__case_test_expr__free_unpacked
                     (PgQuery__CaseTestExpr *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__case_test_expr__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__array_expr__init
                     (PgQuery__ArrayExpr         *message)
{
  static const PgQuery__ArrayExpr init_value = PG_QUERY__ARRAY_EXPR__INIT;
  *message = init_value;
}
size_t pg_query__array_expr__get_packed_size
                     (const PgQuery__ArrayExpr *message)
{
  assert(message->base.descriptor == &pg_query__array_expr__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__array_expr__pack
                     (const PgQuery__ArrayExpr *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__array_expr__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__array_expr__pack_to_buffer
                     (const PgQuery__ArrayExpr *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__array_expr__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__ArrayExpr *
       pg_query__array_expr__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__ArrayExpr *)
     protobuf_c_message_unpack (&pg_query__array_expr__descriptor,
                                allocator, len, data);
}
void   pg_query__array_expr__free_unpacked
                     (PgQuery__ArrayExpr *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__array_expr__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__row_expr__init
                     (PgQuery__RowExpr         *message)
{
  static const PgQuery__RowExpr init_value = PG_QUERY__ROW_EXPR__INIT;
  *message = init_value;
}
size_t pg_query__row_expr__get_packed_size
                     (const PgQuery__RowExpr *message)
{
  assert(message->base.descriptor == &pg_query__row_expr__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__row_expr__pack
                     (const PgQuery__RowExpr *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__row_expr__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__row_expr__pack_to_buffer
                     (const PgQuery__RowExpr *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__row_expr__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__RowExpr *
       pg_query__row_expr__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__RowExpr *)
     protobuf_c_message_unpack (&pg_query__row_expr__descriptor,
                                allocator, len, data);
}
void   pg_query__row_expr__free_unpacked
                     (PgQuery__RowExpr *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__row_expr__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__row_compare_expr__init
                     (PgQuery__RowCompareExpr         *message)
{
  static const PgQuery__RowCompareExpr init_value = PG_QUERY__ROW_COMPARE_EXPR__INIT;
  *message = init_value;
}
size_t pg_query__row_compare_expr__get_packed_size
                     (const PgQuery__RowCompareExpr *message)
{
  assert(message->base.descriptor == &pg_query__row_compare_expr__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__row_compare_expr__pack
                     (const PgQuery__RowCompareExpr *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__row_compare_expr__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__row_compare_expr__pack_to_buffer
                     (const PgQuery__RowCompareExpr *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__row_compare_expr__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__RowCompareExpr *
       pg_query__row_compare_expr__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__RowCompareExpr *)
     protobuf_c_message_unpack (&pg_query__row_compare_expr__descriptor,
                                allocator, len, data);
}
void   pg_query__row_compare_expr__free_unpacked
                     (PgQuery__RowCompareExpr *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__row_compare_expr__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__coalesce_expr__init
                     (PgQuery__CoalesceExpr         *message)
{
  static const PgQuery__CoalesceExpr init_value = PG_QUERY__COALESCE_EXPR__INIT;
  *message = init_value;
}
size_t pg_query__coalesce_expr__get_packed_size
                     (const PgQuery__CoalesceExpr *message)
{
  assert(message->base.descriptor == &pg_query__coalesce_expr__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__coalesce_expr__pack
                     (const PgQuery__CoalesceExpr *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__coalesce_expr__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__coalesce_expr__pack_to_buffer
                     (const PgQuery__CoalesceExpr *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__coalesce_expr__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CoalesceExpr *
       pg_query__coalesce_expr__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CoalesceExpr *)
     protobuf_c_message_unpack (&pg_query__coalesce_expr__descriptor,
                                allocator, len, data);
}
void   pg_query__coalesce_expr__free_unpacked
                     (PgQuery__CoalesceExpr *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__coalesce_expr__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__min_max_expr__init
                     (PgQuery__MinMaxExpr         *message)
{
  static const PgQuery__MinMaxExpr init_value = PG_QUERY__MIN_MAX_EXPR__INIT;
  *message = init_value;
}
size_t pg_query__min_max_expr__get_packed_size
                     (const PgQuery__MinMaxExpr *message)
{
  assert(message->base.descriptor == &pg_query__min_max_expr__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__min_max_expr__pack
                     (const PgQuery__MinMaxExpr *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__min_max_expr__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__min_max_expr__pack_to_buffer
                     (const PgQuery__MinMaxExpr *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__min_max_expr__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__MinMaxExpr *
       pg_query__min_max_expr__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__MinMaxExpr *)
     protobuf_c_message_unpack (&pg_query__min_max_expr__descriptor,
                                allocator, len, data);
}
void   pg_query__min_max_expr__free_unpacked
                     (PgQuery__MinMaxExpr *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__min_max_expr__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__sqlvalue_function__init
                     (PgQuery__SQLValueFunction         *message)
{
  static const PgQuery__SQLValueFunction init_value = PG_QUERY__SQLVALUE_FUNCTION__INIT;
  *message = init_value;
}
size_t pg_query__sqlvalue_function__get_packed_size
                     (const PgQuery__SQLValueFunction *message)
{
  assert(message->base.descriptor == &pg_query__sqlvalue_function__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__sqlvalue_function__pack
                     (const PgQuery__SQLValueFunction *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__sqlvalue_function__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__sqlvalue_function__pack_to_buffer
                     (const PgQuery__SQLValueFunction *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__sqlvalue_function__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__SQLValueFunction *
       pg_query__sqlvalue_function__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__SQLValueFunction *)
     protobuf_c_message_unpack (&pg_query__sqlvalue_function__descriptor,
                                allocator, len, data);
}
void   pg_query__sqlvalue_function__free_unpacked
                     (PgQuery__SQLValueFunction *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__sqlvalue_function__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__xml_expr__init
                     (PgQuery__XmlExpr         *message)
{
  static const PgQuery__XmlExpr init_value = PG_QUERY__XML_EXPR__INIT;
  *message = init_value;
}
size_t pg_query__xml_expr__get_packed_size
                     (const PgQuery__XmlExpr *message)
{
  assert(message->base.descriptor == &pg_query__xml_expr__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__xml_expr__pack
                     (const PgQuery__XmlExpr *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__xml_expr__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__xml_expr__pack_to_buffer
                     (const PgQuery__XmlExpr *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__xml_expr__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__XmlExpr *
       pg_query__xml_expr__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__XmlExpr *)
     protobuf_c_message_unpack (&pg_query__xml_expr__descriptor,
                                allocator, len, data);
}
void   pg_query__xml_expr__free_unpacked
                     (PgQuery__XmlExpr *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__xml_expr__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__json_format__init
                     (PgQuery__JsonFormat         *message)
{
  static const PgQuery__JsonFormat init_value = PG_QUERY__JSON_FORMAT__INIT;
  *message = init_value;
}
size_t pg_query__json_format__get_packed_size
                     (const PgQuery__JsonFormat *message)
{
  assert(message->base.descriptor == &pg_query__json_format__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__json_format__pack
                     (const PgQuery__JsonFormat *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__json_format__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__json_format__pack_to_buffer
                     (const PgQuery__JsonFormat *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__json_format__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__JsonFormat *
       pg_query__json_format__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__JsonFormat *)
     protobuf_c_message_unpack (&pg_query__json_format__descriptor,
                                allocator, len, data);
}
void   pg_query__json_format__free_unpacked
                     (PgQuery__JsonFormat *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__json_format__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__json_returning__init
                     (PgQuery__JsonReturning         *message)
{
  static const PgQuery__JsonReturning init_value = PG_QUERY__JSON_RETURNING__INIT;
  *message = init_value;
}
size_t pg_query__json_returning__get_packed_size
                     (const PgQuery__JsonReturning *message)
{
  assert(message->base.descriptor == &pg_query__json_returning__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__json_returning__pack
                     (const PgQuery__JsonReturning *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__json_returning__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__json_returning__pack_to_buffer
                     (const PgQuery__JsonReturning *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__json_returning__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__JsonReturning *
       pg_query__json_returning__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__JsonReturning *)
     protobuf_c_message_unpack (&pg_query__json_returning__descriptor,
                                allocator, len, data);
}
void   pg_query__json_returning__free_unpacked
                     (PgQuery__JsonReturning *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__json_returning__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__json_value_expr__init
                     (PgQuery__JsonValueExpr         *message)
{
  static const PgQuery__JsonValueExpr init_value = PG_QUERY__JSON_VALUE_EXPR__INIT;
  *message = init_value;
}
size_t pg_query__json_value_expr__get_packed_size
                     (const PgQuery__JsonValueExpr *message)
{
  assert(message->base.descriptor == &pg_query__json_value_expr__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__json_value_expr__pack
                     (const PgQuery__JsonValueExpr *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__json_value_expr__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__json_value_expr__pack_to_buffer
                     (const PgQuery__JsonValueExpr *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__json_value_expr__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__JsonValueExpr *
       pg_query__json_value_expr__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__JsonValueExpr *)
     protobuf_c_message_unpack (&pg_query__json_value_expr__descriptor,
                                allocator, len, data);
}
void   pg_query__json_value_expr__free_unpacked
                     (PgQuery__JsonValueExpr *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__json_value_expr__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__json_constructor_expr__init
                     (PgQuery__JsonConstructorExpr         *message)
{
  static const PgQuery__JsonConstructorExpr init_value = PG_QUERY__JSON_CONSTRUCTOR_EXPR__INIT;
  *message = init_value;
}
size_t pg_query__json_constructor_expr__get_packed_size
                     (const PgQuery__JsonConstructorExpr *message)
{
  assert(message->base.descriptor == &pg_query__json_constructor_expr__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__json_constructor_expr__pack
                     (const PgQuery__JsonConstructorExpr *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__json_constructor_expr__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__json_constructor_expr__pack_to_buffer
                     (const PgQuery__JsonConstructorExpr *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__json_constructor_expr__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__JsonConstructorExpr *
       pg_query__json_constructor_expr__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__JsonConstructorExpr *)
     protobuf_c_message_unpack (&pg_query__json_constructor_expr__descriptor,
                                allocator, len, data);
}
void   pg_query__json_constructor_expr__free_unpacked
                     (PgQuery__JsonConstructorExpr *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__json_constructor_expr__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__json_is_predicate__init
                     (PgQuery__JsonIsPredicate         *message)
{
  static const PgQuery__JsonIsPredicate init_value = PG_QUERY__JSON_IS_PREDICATE__INIT;
  *message = init_value;
}
size_t pg_query__json_is_predicate__get_packed_size
                     (const PgQuery__JsonIsPredicate *message)
{
  assert(message->base.descriptor == &pg_query__json_is_predicate__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__json_is_predicate__pack
                     (const PgQuery__JsonIsPredicate *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__json_is_predicate__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__json_is_predicate__pack_to_buffer
                     (const PgQuery__JsonIsPredicate *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__json_is_predicate__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__JsonIsPredicate *
       pg_query__json_is_predicate__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__JsonIsPredicate *)
     protobuf_c_message_unpack (&pg_query__json_is_predicate__descriptor,
                                allocator, len, data);
}
void   pg_query__json_is_predicate__free_unpacked
                     (PgQuery__JsonIsPredicate *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__json_is_predicate__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__json_behavior__init
                     (PgQuery__JsonBehavior         *message)
{
  static const PgQuery__JsonBehavior init_value = PG_QUERY__JSON_BEHAVIOR__INIT;
  *message = init_value;
}
size_t pg_query__json_behavior__get_packed_size
                     (const PgQuery__JsonBehavior *message)
{
  assert(message->base.descriptor == &pg_query__json_behavior__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__json_behavior__pack
                     (const PgQuery__JsonBehavior *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__json_behavior__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__json_behavior__pack_to_buffer
                     (const PgQuery__JsonBehavior *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__json_behavior__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__JsonBehavior *
       pg_query__json_behavior__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__JsonBehavior *)
     protobuf_c_message_unpack (&pg_query__json_behavior__descriptor,
                                allocator, len, data);
}
void   pg_query__json_behavior__free_unpacked
                     (PgQuery__JsonBehavior *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__json_behavior__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__json_expr__init
                     (PgQuery__JsonExpr         *message)
{
  static const PgQuery__JsonExpr init_value = PG_QUERY__JSON_EXPR__INIT;
  *message = init_value;
}
size_t pg_query__json_expr__get_packed_size
                     (const PgQuery__JsonExpr *message)
{
  assert(message->base.descriptor == &pg_query__json_expr__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__json_expr__pack
                     (const PgQuery__JsonExpr *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__json_expr__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__json_expr__pack_to_buffer
                     (const PgQuery__JsonExpr *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__json_expr__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__JsonExpr *
       pg_query__json_expr__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__JsonExpr *)
     protobuf_c_message_unpack (&pg_query__json_expr__descriptor,
                                allocator, len, data);
}
void   pg_query__json_expr__free_unpacked
                     (PgQuery__JsonExpr *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__json_expr__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__json_table_path__init
                     (PgQuery__JsonTablePath         *message)
{
  static const PgQuery__JsonTablePath init_value = PG_QUERY__JSON_TABLE_PATH__INIT;
  *message = init_value;
}
size_t pg_query__json_table_path__get_packed_size
                     (const PgQuery__JsonTablePath *message)
{
  assert(message->base.descriptor == &pg_query__json_table_path__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__json_table_path__pack
                     (const PgQuery__JsonTablePath *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__json_table_path__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__json_table_path__pack_to_buffer
                     (const PgQuery__JsonTablePath *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__json_table_path__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__JsonTablePath *
       pg_query__json_table_path__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__JsonTablePath *)
     protobuf_c_message_unpack (&pg_query__json_table_path__descriptor,
                                allocator, len, data);
}
void   pg_query__json_table_path__free_unpacked
                     (PgQuery__JsonTablePath *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__json_table_path__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__json_table_path_scan__init
                     (PgQuery__JsonTablePathScan         *message)
{
  static const PgQuery__JsonTablePathScan init_value = PG_QUERY__JSON_TABLE_PATH_SCAN__INIT;
  *message = init_value;
}
size_t pg_query__json_table_path_scan__get_packed_size
                     (const PgQuery__JsonTablePathScan *message)
{
  assert(message->base.descriptor == &pg_query__json_table_path_scan__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__json_table_path_scan__pack
                     (const PgQuery__JsonTablePathScan *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__json_table_path_scan__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__json_table_path_scan__pack_to_buffer
                     (const PgQuery__JsonTablePathScan *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__json_table_path_scan__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__JsonTablePathScan *
       pg_query__json_table_path_scan__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__JsonTablePathScan *)
     protobuf_c_message_unpack (&pg_query__json_table_path_scan__descriptor,
                                allocator, len, data);
}
void   pg_query__json_table_path_scan__free_unpacked
                     (PgQuery__JsonTablePathScan *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__json_table_path_scan__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__json_table_sibling_join__init
                     (PgQuery__JsonTableSiblingJoin         *message)
{
  static const PgQuery__JsonTableSiblingJoin init_value = PG_QUERY__JSON_TABLE_SIBLING_JOIN__INIT;
  *message = init_value;
}
size_t pg_query__json_table_sibling_join__get_packed_size
                     (const PgQuery__JsonTableSiblingJoin *message)
{
  assert(message->base.descriptor == &pg_query__json_table_sibling_join__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__json_table_sibling_join__pack
                     (const PgQuery__JsonTableSiblingJoin *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__json_table_sibling_join__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__json_table_sibling_join__pack_to_buffer
                     (const PgQuery__JsonTableSiblingJoin *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__json_table_sibling_join__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__JsonTableSiblingJoin *
       pg_query__json_table_sibling_join__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__JsonTableSiblingJoin *)
     protobuf_c_message_unpack (&pg_query__json_table_sibling_join__descriptor,
                                allocator, len, data);
}
void   pg_query__json_table_sibling_join__free_unpacked
                     (PgQuery__JsonTableSiblingJoin *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__json_table_sibling_join__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__null_test__init
                     (PgQuery__NullTest         *message)
{
  static const PgQuery__NullTest init_value = PG_QUERY__NULL_TEST__INIT;
  *message = init_value;
}
size_t pg_query__null_test__get_packed_size
                     (const PgQuery__NullTest *message)
{
  assert(message->base.descriptor == &pg_query__null_test__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__null_test__pack
                     (const PgQuery__NullTest *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__null_test__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__null_test__pack_to_buffer
                     (const PgQuery__NullTest *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__null_test__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__NullTest *
       pg_query__null_test__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__NullTest *)
     protobuf_c_message_unpack (&pg_query__null_test__descriptor,
                                allocator, len, data);
}
void   pg_query__null_test__free_unpacked
                     (PgQuery__NullTest *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__null_test__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__boolean_test__init
                     (PgQuery__BooleanTest         *message)
{
  static const PgQuery__BooleanTest init_value = PG_QUERY__BOOLEAN_TEST__INIT;
  *message = init_value;
}
size_t pg_query__boolean_test__get_packed_size
                     (const PgQuery__BooleanTest *message)
{
  assert(message->base.descriptor == &pg_query__boolean_test__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__boolean_test__pack
                     (const PgQuery__BooleanTest *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__boolean_test__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__boolean_test__pack_to_buffer
                     (const PgQuery__BooleanTest *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__boolean_test__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__BooleanTest *
       pg_query__boolean_test__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__BooleanTest *)
     protobuf_c_message_unpack (&pg_query__boolean_test__descriptor,
                                allocator, len, data);
}
void   pg_query__boolean_test__free_unpacked
                     (PgQuery__BooleanTest *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__boolean_test__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__merge_action__init
                     (PgQuery__MergeAction         *message)
{
  static const PgQuery__MergeAction init_value = PG_QUERY__MERGE_ACTION__INIT;
  *message = init_value;
}
size_t pg_query__merge_action__get_packed_size
                     (const PgQuery__MergeAction *message)
{
  assert(message->base.descriptor == &pg_query__merge_action__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__merge_action__pack
                     (const PgQuery__MergeAction *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__merge_action__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__merge_action__pack_to_buffer
                     (const PgQuery__MergeAction *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__merge_action__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__MergeAction *
       pg_query__merge_action__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__MergeAction *)
     protobuf_c_message_unpack (&pg_query__merge_action__descriptor,
                                allocator, len, data);
}
void   pg_query__merge_action__free_unpacked
                     (PgQuery__MergeAction *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__merge_action__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__coerce_to_domain__init
                     (PgQuery__CoerceToDomain         *message)
{
  static const PgQuery__CoerceToDomain init_value = PG_QUERY__COERCE_TO_DOMAIN__INIT;
  *message = init_value;
}
size_t pg_query__coerce_to_domain__get_packed_size
                     (const PgQuery__CoerceToDomain *message)
{
  assert(message->base.descriptor == &pg_query__coerce_to_domain__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__coerce_to_domain__pack
                     (const PgQuery__CoerceToDomain *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__coerce_to_domain__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__coerce_to_domain__pack_to_buffer
                     (const PgQuery__CoerceToDomain *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__coerce_to_domain__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CoerceToDomain *
       pg_query__coerce_to_domain__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CoerceToDomain *)
     protobuf_c_message_unpack (&pg_query__coerce_to_domain__descriptor,
                                allocator, len, data);
}
void   pg_query__coerce_to_domain__free_unpacked
                     (PgQuery__CoerceToDomain *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__coerce_to_domain__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__coerce_to_domain_value__init
                     (PgQuery__CoerceToDomainValue         *message)
{
  static const PgQuery__CoerceToDomainValue init_value = PG_QUERY__COERCE_TO_DOMAIN_VALUE__INIT;
  *message = init_value;
}
size_t pg_query__coerce_to_domain_value__get_packed_size
                     (const PgQuery__CoerceToDomainValue *message)
{
  assert(message->base.descriptor == &pg_query__coerce_to_domain_value__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__coerce_to_domain_value__pack
                     (const PgQuery__CoerceToDomainValue *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__coerce_to_domain_value__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__coerce_to_domain_value__pack_to_buffer
                     (const PgQuery__CoerceToDomainValue *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__coerce_to_domain_value__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CoerceToDomainValue *
       pg_query__coerce_to_domain_value__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CoerceToDomainValue *)
     protobuf_c_message_unpack (&pg_query__coerce_to_domain_value__descriptor,
                                allocator, len, data);
}
void   pg_query__coerce_to_domain_value__free_unpacked
                     (PgQuery__CoerceToDomainValue *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__coerce_to_domain_value__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__set_to_default__init
                     (PgQuery__SetToDefault         *message)
{
  static const PgQuery__SetToDefault init_value = PG_QUERY__SET_TO_DEFAULT__INIT;
  *message = init_value;
}
size_t pg_query__set_to_default__get_packed_size
                     (const PgQuery__SetToDefault *message)
{
  assert(message->base.descriptor == &pg_query__set_to_default__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__set_to_default__pack
                     (const PgQuery__SetToDefault *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__set_to_default__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__set_to_default__pack_to_buffer
                     (const PgQuery__SetToDefault *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__set_to_default__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__SetToDefault *
       pg_query__set_to_default__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__SetToDefault *)
     protobuf_c_message_unpack (&pg_query__set_to_default__descriptor,
                                allocator, len, data);
}
void   pg_query__set_to_default__free_unpacked
                     (PgQuery__SetToDefault *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__set_to_default__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__current_of_expr__init
                     (PgQuery__CurrentOfExpr         *message)
{
  static const PgQuery__CurrentOfExpr init_value = PG_QUERY__CURRENT_OF_EXPR__INIT;
  *message = init_value;
}
size_t pg_query__current_of_expr__get_packed_size
                     (const PgQuery__CurrentOfExpr *message)
{
  assert(message->base.descriptor == &pg_query__current_of_expr__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__current_of_expr__pack
                     (const PgQuery__CurrentOfExpr *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__current_of_expr__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__current_of_expr__pack_to_buffer
                     (const PgQuery__CurrentOfExpr *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__current_of_expr__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CurrentOfExpr *
       pg_query__current_of_expr__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CurrentOfExpr *)
     protobuf_c_message_unpack (&pg_query__current_of_expr__descriptor,
                                allocator, len, data);
}
void   pg_query__current_of_expr__free_unpacked
                     (PgQuery__CurrentOfExpr *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__current_of_expr__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__next_value_expr__init
                     (PgQuery__NextValueExpr         *message)
{
  static const PgQuery__NextValueExpr init_value = PG_QUERY__NEXT_VALUE_EXPR__INIT;
  *message = init_value;
}
size_t pg_query__next_value_expr__get_packed_size
                     (const PgQuery__NextValueExpr *message)
{
  assert(message->base.descriptor == &pg_query__next_value_expr__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__next_value_expr__pack
                     (const PgQuery__NextValueExpr *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__next_value_expr__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__next_value_expr__pack_to_buffer
                     (const PgQuery__NextValueExpr *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__next_value_expr__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__NextValueExpr *
       pg_query__next_value_expr__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__NextValueExpr *)
     protobuf_c_message_unpack (&pg_query__next_value_expr__descriptor,
                                allocator, len, data);
}
void   pg_query__next_value_expr__free_unpacked
                     (PgQuery__NextValueExpr *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__next_value_expr__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__inference_elem__init
                     (PgQuery__InferenceElem         *message)
{
  static const PgQuery__InferenceElem init_value = PG_QUERY__INFERENCE_ELEM__INIT;
  *message = init_value;
}
size_t pg_query__inference_elem__get_packed_size
                     (const PgQuery__InferenceElem *message)
{
  assert(message->base.descriptor == &pg_query__inference_elem__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__inference_elem__pack
                     (const PgQuery__InferenceElem *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__inference_elem__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__inference_elem__pack_to_buffer
                     (const PgQuery__InferenceElem *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__inference_elem__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__InferenceElem *
       pg_query__inference_elem__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__InferenceElem *)
     protobuf_c_message_unpack (&pg_query__inference_elem__descriptor,
                                allocator, len, data);
}
void   pg_query__inference_elem__free_unpacked
                     (PgQuery__InferenceElem *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__inference_elem__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__target_entry__init
                     (PgQuery__TargetEntry         *message)
{
  static const PgQuery__TargetEntry init_value = PG_QUERY__TARGET_ENTRY__INIT;
  *message = init_value;
}
size_t pg_query__target_entry__get_packed_size
                     (const PgQuery__TargetEntry *message)
{
  assert(message->base.descriptor == &pg_query__target_entry__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__target_entry__pack
                     (const PgQuery__TargetEntry *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__target_entry__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__target_entry__pack_to_buffer
                     (const PgQuery__TargetEntry *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__target_entry__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__TargetEntry *
       pg_query__target_entry__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__TargetEntry *)
     protobuf_c_message_unpack (&pg_query__target_entry__descriptor,
                                allocator, len, data);
}
void   pg_query__target_entry__free_unpacked
                     (PgQuery__TargetEntry *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__target_entry__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__range_tbl_ref__init
                     (PgQuery__RangeTblRef         *message)
{
  static const PgQuery__RangeTblRef init_value = PG_QUERY__RANGE_TBL_REF__INIT;
  *message = init_value;
}
size_t pg_query__range_tbl_ref__get_packed_size
                     (const PgQuery__RangeTblRef *message)
{
  assert(message->base.descriptor == &pg_query__range_tbl_ref__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__range_tbl_ref__pack
                     (const PgQuery__RangeTblRef *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__range_tbl_ref__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__range_tbl_ref__pack_to_buffer
                     (const PgQuery__RangeTblRef *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__range_tbl_ref__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__RangeTblRef *
       pg_query__range_tbl_ref__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__RangeTblRef *)
     protobuf_c_message_unpack (&pg_query__range_tbl_ref__descriptor,
                                allocator, len, data);
}
void   pg_query__range_tbl_ref__free_unpacked
                     (PgQuery__RangeTblRef *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__range_tbl_ref__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__join_expr__init
                     (PgQuery__JoinExpr         *message)
{
  static const PgQuery__JoinExpr init_value = PG_QUERY__JOIN_EXPR__INIT;
  *message = init_value;
}
size_t pg_query__join_expr__get_packed_size
                     (const PgQuery__JoinExpr *message)
{
  assert(message->base.descriptor == &pg_query__join_expr__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__join_expr__pack
                     (const PgQuery__JoinExpr *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__join_expr__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__join_expr__pack_to_buffer
                     (const PgQuery__JoinExpr *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__join_expr__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__JoinExpr *
       pg_query__join_expr__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__JoinExpr *)
     protobuf_c_message_unpack (&pg_query__join_expr__descriptor,
                                allocator, len, data);
}
void   pg_query__join_expr__free_unpacked
                     (PgQuery__JoinExpr *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__join_expr__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__from_expr__init
                     (PgQuery__FromExpr         *message)
{
  static const PgQuery__FromExpr init_value = PG_QUERY__FROM_EXPR__INIT;
  *message = init_value;
}
size_t pg_query__from_expr__get_packed_size
                     (const PgQuery__FromExpr *message)
{
  assert(message->base.descriptor == &pg_query__from_expr__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__from_expr__pack
                     (const PgQuery__FromExpr *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__from_expr__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__from_expr__pack_to_buffer
                     (const PgQuery__FromExpr *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__from_expr__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__FromExpr *
       pg_query__from_expr__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__FromExpr *)
     protobuf_c_message_unpack (&pg_query__from_expr__descriptor,
                                allocator, len, data);
}
void   pg_query__from_expr__free_unpacked
                     (PgQuery__FromExpr *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__from_expr__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__on_conflict_expr__init
                     (PgQuery__OnConflictExpr         *message)
{
  static const PgQuery__OnConflictExpr init_value = PG_QUERY__ON_CONFLICT_EXPR__INIT;
  *message = init_value;
}
size_t pg_query__on_conflict_expr__get_packed_size
                     (const PgQuery__OnConflictExpr *message)
{
  assert(message->base.descriptor == &pg_query__on_conflict_expr__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__on_conflict_expr__pack
                     (const PgQuery__OnConflictExpr *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__on_conflict_expr__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__on_conflict_expr__pack_to_buffer
                     (const PgQuery__OnConflictExpr *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__on_conflict_expr__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__OnConflictExpr *
       pg_query__on_conflict_expr__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__OnConflictExpr *)
     protobuf_c_message_unpack (&pg_query__on_conflict_expr__descriptor,
                                allocator, len, data);
}
void   pg_query__on_conflict_expr__free_unpacked
                     (PgQuery__OnConflictExpr *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__on_conflict_expr__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__query__init
                     (PgQuery__Query         *message)
{
  static const PgQuery__Query init_value = PG_QUERY__QUERY__INIT;
  *message = init_value;
}
size_t pg_query__query__get_packed_size
                     (const PgQuery__Query *message)
{
  assert(message->base.descriptor == &pg_query__query__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__query__pack
                     (const PgQuery__Query *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__query__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__query__pack_to_buffer
                     (const PgQuery__Query *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__query__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__Query *
       pg_query__query__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__Query *)
     protobuf_c_message_unpack (&pg_query__query__descriptor,
                                allocator, len, data);
}
void   pg_query__query__free_unpacked
                     (PgQuery__Query *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__query__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__type_name__init
                     (PgQuery__TypeName         *message)
{
  static const PgQuery__TypeName init_value = PG_QUERY__TYPE_NAME__INIT;
  *message = init_value;
}
size_t pg_query__type_name__get_packed_size
                     (const PgQuery__TypeName *message)
{
  assert(message->base.descriptor == &pg_query__type_name__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__type_name__pack
                     (const PgQuery__TypeName *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__type_name__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__type_name__pack_to_buffer
                     (const PgQuery__TypeName *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__type_name__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__TypeName *
       pg_query__type_name__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__TypeName *)
     protobuf_c_message_unpack (&pg_query__type_name__descriptor,
                                allocator, len, data);
}
void   pg_query__type_name__free_unpacked
                     (PgQuery__TypeName *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__type_name__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__column_ref__init
                     (PgQuery__ColumnRef         *message)
{
  static const PgQuery__ColumnRef init_value = PG_QUERY__COLUMN_REF__INIT;
  *message = init_value;
}
size_t pg_query__column_ref__get_packed_size
                     (const PgQuery__ColumnRef *message)
{
  assert(message->base.descriptor == &pg_query__column_ref__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__column_ref__pack
                     (const PgQuery__ColumnRef *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__column_ref__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__column_ref__pack_to_buffer
                     (const PgQuery__ColumnRef *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__column_ref__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__ColumnRef *
       pg_query__column_ref__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__ColumnRef *)
     protobuf_c_message_unpack (&pg_query__column_ref__descriptor,
                                allocator, len, data);
}
void   pg_query__column_ref__free_unpacked
                     (PgQuery__ColumnRef *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__column_ref__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__param_ref__init
                     (PgQuery__ParamRef         *message)
{
  static const PgQuery__ParamRef init_value = PG_QUERY__PARAM_REF__INIT;
  *message = init_value;
}
size_t pg_query__param_ref__get_packed_size
                     (const PgQuery__ParamRef *message)
{
  assert(message->base.descriptor == &pg_query__param_ref__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__param_ref__pack
                     (const PgQuery__ParamRef *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__param_ref__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__param_ref__pack_to_buffer
                     (const PgQuery__ParamRef *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__param_ref__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__ParamRef *
       pg_query__param_ref__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__ParamRef *)
     protobuf_c_message_unpack (&pg_query__param_ref__descriptor,
                                allocator, len, data);
}
void   pg_query__param_ref__free_unpacked
                     (PgQuery__ParamRef *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__param_ref__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__a__expr__init
                     (PgQuery__AExpr         *message)
{
  static const PgQuery__AExpr init_value = PG_QUERY__A__EXPR__INIT;
  *message = init_value;
}
size_t pg_query__a__expr__get_packed_size
                     (const PgQuery__AExpr *message)
{
  assert(message->base.descriptor == &pg_query__a__expr__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__a__expr__pack
                     (const PgQuery__AExpr *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__a__expr__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__a__expr__pack_to_buffer
                     (const PgQuery__AExpr *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__a__expr__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AExpr *
       pg_query__a__expr__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AExpr *)
     protobuf_c_message_unpack (&pg_query__a__expr__descriptor,
                                allocator, len, data);
}
void   pg_query__a__expr__free_unpacked
                     (PgQuery__AExpr *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__a__expr__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__type_cast__init
                     (PgQuery__TypeCast         *message)
{
  static const PgQuery__TypeCast init_value = PG_QUERY__TYPE_CAST__INIT;
  *message = init_value;
}
size_t pg_query__type_cast__get_packed_size
                     (const PgQuery__TypeCast *message)
{
  assert(message->base.descriptor == &pg_query__type_cast__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__type_cast__pack
                     (const PgQuery__TypeCast *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__type_cast__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__type_cast__pack_to_buffer
                     (const PgQuery__TypeCast *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__type_cast__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__TypeCast *
       pg_query__type_cast__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__TypeCast *)
     protobuf_c_message_unpack (&pg_query__type_cast__descriptor,
                                allocator, len, data);
}
void   pg_query__type_cast__free_unpacked
                     (PgQuery__TypeCast *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__type_cast__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__collate_clause__init
                     (PgQuery__CollateClause         *message)
{
  static const PgQuery__CollateClause init_value = PG_QUERY__COLLATE_CLAUSE__INIT;
  *message = init_value;
}
size_t pg_query__collate_clause__get_packed_size
                     (const PgQuery__CollateClause *message)
{
  assert(message->base.descriptor == &pg_query__collate_clause__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__collate_clause__pack
                     (const PgQuery__CollateClause *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__collate_clause__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__collate_clause__pack_to_buffer
                     (const PgQuery__CollateClause *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__collate_clause__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CollateClause *
       pg_query__collate_clause__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CollateClause *)
     protobuf_c_message_unpack (&pg_query__collate_clause__descriptor,
                                allocator, len, data);
}
void   pg_query__collate_clause__free_unpacked
                     (PgQuery__CollateClause *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__collate_clause__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__role_spec__init
                     (PgQuery__RoleSpec         *message)
{
  static const PgQuery__RoleSpec init_value = PG_QUERY__ROLE_SPEC__INIT;
  *message = init_value;
}
size_t pg_query__role_spec__get_packed_size
                     (const PgQuery__RoleSpec *message)
{
  assert(message->base.descriptor == &pg_query__role_spec__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__role_spec__pack
                     (const PgQuery__RoleSpec *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__role_spec__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__role_spec__pack_to_buffer
                     (const PgQuery__RoleSpec *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__role_spec__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__RoleSpec *
       pg_query__role_spec__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__RoleSpec *)
     protobuf_c_message_unpack (&pg_query__role_spec__descriptor,
                                allocator, len, data);
}
void   pg_query__role_spec__free_unpacked
                     (PgQuery__RoleSpec *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__role_spec__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__func_call__init
                     (PgQuery__FuncCall         *message)
{
  static const PgQuery__FuncCall init_value = PG_QUERY__FUNC_CALL__INIT;
  *message = init_value;
}
size_t pg_query__func_call__get_packed_size
                     (const PgQuery__FuncCall *message)
{
  assert(message->base.descriptor == &pg_query__func_call__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__func_call__pack
                     (const PgQuery__FuncCall *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__func_call__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__func_call__pack_to_buffer
                     (const PgQuery__FuncCall *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__func_call__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__FuncCall *
       pg_query__func_call__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__FuncCall *)
     protobuf_c_message_unpack (&pg_query__func_call__descriptor,
                                allocator, len, data);
}
void   pg_query__func_call__free_unpacked
                     (PgQuery__FuncCall *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__func_call__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__a__star__init
                     (PgQuery__AStar         *message)
{
  static const PgQuery__AStar init_value = PG_QUERY__A__STAR__INIT;
  *message = init_value;
}
size_t pg_query__a__star__get_packed_size
                     (const PgQuery__AStar *message)
{
  assert(message->base.descriptor == &pg_query__a__star__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__a__star__pack
                     (const PgQuery__AStar *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__a__star__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__a__star__pack_to_buffer
                     (const PgQuery__AStar *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__a__star__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AStar *
       pg_query__a__star__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AStar *)
     protobuf_c_message_unpack (&pg_query__a__star__descriptor,
                                allocator, len, data);
}
void   pg_query__a__star__free_unpacked
                     (PgQuery__AStar *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__a__star__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__a__indices__init
                     (PgQuery__AIndices         *message)
{
  static const PgQuery__AIndices init_value = PG_QUERY__A__INDICES__INIT;
  *message = init_value;
}
size_t pg_query__a__indices__get_packed_size
                     (const PgQuery__AIndices *message)
{
  assert(message->base.descriptor == &pg_query__a__indices__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__a__indices__pack
                     (const PgQuery__AIndices *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__a__indices__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__a__indices__pack_to_buffer
                     (const PgQuery__AIndices *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__a__indices__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AIndices *
       pg_query__a__indices__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AIndices *)
     protobuf_c_message_unpack (&pg_query__a__indices__descriptor,
                                allocator, len, data);
}
void   pg_query__a__indices__free_unpacked
                     (PgQuery__AIndices *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__a__indices__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__a__indirection__init
                     (PgQuery__AIndirection         *message)
{
  static const PgQuery__AIndirection init_value = PG_QUERY__A__INDIRECTION__INIT;
  *message = init_value;
}
size_t pg_query__a__indirection__get_packed_size
                     (const PgQuery__AIndirection *message)
{
  assert(message->base.descriptor == &pg_query__a__indirection__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__a__indirection__pack
                     (const PgQuery__AIndirection *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__a__indirection__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__a__indirection__pack_to_buffer
                     (const PgQuery__AIndirection *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__a__indirection__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AIndirection *
       pg_query__a__indirection__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AIndirection *)
     protobuf_c_message_unpack (&pg_query__a__indirection__descriptor,
                                allocator, len, data);
}
void   pg_query__a__indirection__free_unpacked
                     (PgQuery__AIndirection *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__a__indirection__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__a__array_expr__init
                     (PgQuery__AArrayExpr         *message)
{
  static const PgQuery__AArrayExpr init_value = PG_QUERY__A__ARRAY_EXPR__INIT;
  *message = init_value;
}
size_t pg_query__a__array_expr__get_packed_size
                     (const PgQuery__AArrayExpr *message)
{
  assert(message->base.descriptor == &pg_query__a__array_expr__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__a__array_expr__pack
                     (const PgQuery__AArrayExpr *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__a__array_expr__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__a__array_expr__pack_to_buffer
                     (const PgQuery__AArrayExpr *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__a__array_expr__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AArrayExpr *
       pg_query__a__array_expr__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AArrayExpr *)
     protobuf_c_message_unpack (&pg_query__a__array_expr__descriptor,
                                allocator, len, data);
}
void   pg_query__a__array_expr__free_unpacked
                     (PgQuery__AArrayExpr *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__a__array_expr__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__res_target__init
                     (PgQuery__ResTarget         *message)
{
  static const PgQuery__ResTarget init_value = PG_QUERY__RES_TARGET__INIT;
  *message = init_value;
}
size_t pg_query__res_target__get_packed_size
                     (const PgQuery__ResTarget *message)
{
  assert(message->base.descriptor == &pg_query__res_target__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__res_target__pack
                     (const PgQuery__ResTarget *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__res_target__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__res_target__pack_to_buffer
                     (const PgQuery__ResTarget *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__res_target__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__ResTarget *
       pg_query__res_target__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__ResTarget *)
     protobuf_c_message_unpack (&pg_query__res_target__descriptor,
                                allocator, len, data);
}
void   pg_query__res_target__free_unpacked
                     (PgQuery__ResTarget *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__res_target__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__multi_assign_ref__init
                     (PgQuery__MultiAssignRef         *message)
{
  static const PgQuery__MultiAssignRef init_value = PG_QUERY__MULTI_ASSIGN_REF__INIT;
  *message = init_value;
}
size_t pg_query__multi_assign_ref__get_packed_size
                     (const PgQuery__MultiAssignRef *message)
{
  assert(message->base.descriptor == &pg_query__multi_assign_ref__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__multi_assign_ref__pack
                     (const PgQuery__MultiAssignRef *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__multi_assign_ref__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__multi_assign_ref__pack_to_buffer
                     (const PgQuery__MultiAssignRef *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__multi_assign_ref__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__MultiAssignRef *
       pg_query__multi_assign_ref__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__MultiAssignRef *)
     protobuf_c_message_unpack (&pg_query__multi_assign_ref__descriptor,
                                allocator, len, data);
}
void   pg_query__multi_assign_ref__free_unpacked
                     (PgQuery__MultiAssignRef *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__multi_assign_ref__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__sort_by__init
                     (PgQuery__SortBy         *message)
{
  static const PgQuery__SortBy init_value = PG_QUERY__SORT_BY__INIT;
  *message = init_value;
}
size_t pg_query__sort_by__get_packed_size
                     (const PgQuery__SortBy *message)
{
  assert(message->base.descriptor == &pg_query__sort_by__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__sort_by__pack
                     (const PgQuery__SortBy *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__sort_by__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__sort_by__pack_to_buffer
                     (const PgQuery__SortBy *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__sort_by__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__SortBy *
       pg_query__sort_by__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__SortBy *)
     protobuf_c_message_unpack (&pg_query__sort_by__descriptor,
                                allocator, len, data);
}
void   pg_query__sort_by__free_unpacked
                     (PgQuery__SortBy *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__sort_by__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__window_def__init
                     (PgQuery__WindowDef         *message)
{
  static const PgQuery__WindowDef init_value = PG_QUERY__WINDOW_DEF__INIT;
  *message = init_value;
}
size_t pg_query__window_def__get_packed_size
                     (const PgQuery__WindowDef *message)
{
  assert(message->base.descriptor == &pg_query__window_def__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__window_def__pack
                     (const PgQuery__WindowDef *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__window_def__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__window_def__pack_to_buffer
                     (const PgQuery__WindowDef *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__window_def__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__WindowDef *
       pg_query__window_def__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__WindowDef *)
     protobuf_c_message_unpack (&pg_query__window_def__descriptor,
                                allocator, len, data);
}
void   pg_query__window_def__free_unpacked
                     (PgQuery__WindowDef *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__window_def__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__range_subselect__init
                     (PgQuery__RangeSubselect         *message)
{
  static const PgQuery__RangeSubselect init_value = PG_QUERY__RANGE_SUBSELECT__INIT;
  *message = init_value;
}
size_t pg_query__range_subselect__get_packed_size
                     (const PgQuery__RangeSubselect *message)
{
  assert(message->base.descriptor == &pg_query__range_subselect__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__range_subselect__pack
                     (const PgQuery__RangeSubselect *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__range_subselect__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__range_subselect__pack_to_buffer
                     (const PgQuery__RangeSubselect *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__range_subselect__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__RangeSubselect *
       pg_query__range_subselect__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__RangeSubselect *)
     protobuf_c_message_unpack (&pg_query__range_subselect__descriptor,
                                allocator, len, data);
}
void   pg_query__range_subselect__free_unpacked
                     (PgQuery__RangeSubselect *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__range_subselect__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__range_function__init
                     (PgQuery__RangeFunction         *message)
{
  static const PgQuery__RangeFunction init_value = PG_QUERY__RANGE_FUNCTION__INIT;
  *message = init_value;
}
size_t pg_query__range_function__get_packed_size
                     (const PgQuery__RangeFunction *message)
{
  assert(message->base.descriptor == &pg_query__range_function__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__range_function__pack
                     (const PgQuery__RangeFunction *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__range_function__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__range_function__pack_to_buffer
                     (const PgQuery__RangeFunction *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__range_function__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__RangeFunction *
       pg_query__range_function__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__RangeFunction *)
     protobuf_c_message_unpack (&pg_query__range_function__descriptor,
                                allocator, len, data);
}
void   pg_query__range_function__free_unpacked
                     (PgQuery__RangeFunction *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__range_function__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__range_table_func__init
                     (PgQuery__RangeTableFunc         *message)
{
  static const PgQuery__RangeTableFunc init_value = PG_QUERY__RANGE_TABLE_FUNC__INIT;
  *message = init_value;
}
size_t pg_query__range_table_func__get_packed_size
                     (const PgQuery__RangeTableFunc *message)
{
  assert(message->base.descriptor == &pg_query__range_table_func__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__range_table_func__pack
                     (const PgQuery__RangeTableFunc *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__range_table_func__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__range_table_func__pack_to_buffer
                     (const PgQuery__RangeTableFunc *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__range_table_func__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__RangeTableFunc *
       pg_query__range_table_func__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__RangeTableFunc *)
     protobuf_c_message_unpack (&pg_query__range_table_func__descriptor,
                                allocator, len, data);
}
void   pg_query__range_table_func__free_unpacked
                     (PgQuery__RangeTableFunc *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__range_table_func__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__range_table_func_col__init
                     (PgQuery__RangeTableFuncCol         *message)
{
  static const PgQuery__RangeTableFuncCol init_value = PG_QUERY__RANGE_TABLE_FUNC_COL__INIT;
  *message = init_value;
}
size_t pg_query__range_table_func_col__get_packed_size
                     (const PgQuery__RangeTableFuncCol *message)
{
  assert(message->base.descriptor == &pg_query__range_table_func_col__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__range_table_func_col__pack
                     (const PgQuery__RangeTableFuncCol *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__range_table_func_col__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__range_table_func_col__pack_to_buffer
                     (const PgQuery__RangeTableFuncCol *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__range_table_func_col__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__RangeTableFuncCol *
       pg_query__range_table_func_col__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__RangeTableFuncCol *)
     protobuf_c_message_unpack (&pg_query__range_table_func_col__descriptor,
                                allocator, len, data);
}
void   pg_query__range_table_func_col__free_unpacked
                     (PgQuery__RangeTableFuncCol *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__range_table_func_col__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__range_table_sample__init
                     (PgQuery__RangeTableSample         *message)
{
  static const PgQuery__RangeTableSample init_value = PG_QUERY__RANGE_TABLE_SAMPLE__INIT;
  *message = init_value;
}
size_t pg_query__range_table_sample__get_packed_size
                     (const PgQuery__RangeTableSample *message)
{
  assert(message->base.descriptor == &pg_query__range_table_sample__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__range_table_sample__pack
                     (const PgQuery__RangeTableSample *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__range_table_sample__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__range_table_sample__pack_to_buffer
                     (const PgQuery__RangeTableSample *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__range_table_sample__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__RangeTableSample *
       pg_query__range_table_sample__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__RangeTableSample *)
     protobuf_c_message_unpack (&pg_query__range_table_sample__descriptor,
                                allocator, len, data);
}
void   pg_query__range_table_sample__free_unpacked
                     (PgQuery__RangeTableSample *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__range_table_sample__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__column_def__init
                     (PgQuery__ColumnDef         *message)
{
  static const PgQuery__ColumnDef init_value = PG_QUERY__COLUMN_DEF__INIT;
  *message = init_value;
}
size_t pg_query__column_def__get_packed_size
                     (const PgQuery__ColumnDef *message)
{
  assert(message->base.descriptor == &pg_query__column_def__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__column_def__pack
                     (const PgQuery__ColumnDef *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__column_def__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__column_def__pack_to_buffer
                     (const PgQuery__ColumnDef *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__column_def__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__ColumnDef *
       pg_query__column_def__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__ColumnDef *)
     protobuf_c_message_unpack (&pg_query__column_def__descriptor,
                                allocator, len, data);
}
void   pg_query__column_def__free_unpacked
                     (PgQuery__ColumnDef *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__column_def__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__table_like_clause__init
                     (PgQuery__TableLikeClause         *message)
{
  static const PgQuery__TableLikeClause init_value = PG_QUERY__TABLE_LIKE_CLAUSE__INIT;
  *message = init_value;
}
size_t pg_query__table_like_clause__get_packed_size
                     (const PgQuery__TableLikeClause *message)
{
  assert(message->base.descriptor == &pg_query__table_like_clause__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__table_like_clause__pack
                     (const PgQuery__TableLikeClause *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__table_like_clause__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__table_like_clause__pack_to_buffer
                     (const PgQuery__TableLikeClause *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__table_like_clause__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__TableLikeClause *
       pg_query__table_like_clause__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__TableLikeClause *)
     protobuf_c_message_unpack (&pg_query__table_like_clause__descriptor,
                                allocator, len, data);
}
void   pg_query__table_like_clause__free_unpacked
                     (PgQuery__TableLikeClause *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__table_like_clause__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__index_elem__init
                     (PgQuery__IndexElem         *message)
{
  static const PgQuery__IndexElem init_value = PG_QUERY__INDEX_ELEM__INIT;
  *message = init_value;
}
size_t pg_query__index_elem__get_packed_size
                     (const PgQuery__IndexElem *message)
{
  assert(message->base.descriptor == &pg_query__index_elem__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__index_elem__pack
                     (const PgQuery__IndexElem *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__index_elem__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__index_elem__pack_to_buffer
                     (const PgQuery__IndexElem *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__index_elem__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__IndexElem *
       pg_query__index_elem__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__IndexElem *)
     protobuf_c_message_unpack (&pg_query__index_elem__descriptor,
                                allocator, len, data);
}
void   pg_query__index_elem__free_unpacked
                     (PgQuery__IndexElem *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__index_elem__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__def_elem__init
                     (PgQuery__DefElem         *message)
{
  static const PgQuery__DefElem init_value = PG_QUERY__DEF_ELEM__INIT;
  *message = init_value;
}
size_t pg_query__def_elem__get_packed_size
                     (const PgQuery__DefElem *message)
{
  assert(message->base.descriptor == &pg_query__def_elem__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__def_elem__pack
                     (const PgQuery__DefElem *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__def_elem__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__def_elem__pack_to_buffer
                     (const PgQuery__DefElem *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__def_elem__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__DefElem *
       pg_query__def_elem__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__DefElem *)
     protobuf_c_message_unpack (&pg_query__def_elem__descriptor,
                                allocator, len, data);
}
void   pg_query__def_elem__free_unpacked
                     (PgQuery__DefElem *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__def_elem__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__locking_clause__init
                     (PgQuery__LockingClause         *message)
{
  static const PgQuery__LockingClause init_value = PG_QUERY__LOCKING_CLAUSE__INIT;
  *message = init_value;
}
size_t pg_query__locking_clause__get_packed_size
                     (const PgQuery__LockingClause *message)
{
  assert(message->base.descriptor == &pg_query__locking_clause__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__locking_clause__pack
                     (const PgQuery__LockingClause *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__locking_clause__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__locking_clause__pack_to_buffer
                     (const PgQuery__LockingClause *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__locking_clause__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__LockingClause *
       pg_query__locking_clause__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__LockingClause *)
     protobuf_c_message_unpack (&pg_query__locking_clause__descriptor,
                                allocator, len, data);
}
void   pg_query__locking_clause__free_unpacked
                     (PgQuery__LockingClause *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__locking_clause__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__xml_serialize__init
                     (PgQuery__XmlSerialize         *message)
{
  static const PgQuery__XmlSerialize init_value = PG_QUERY__XML_SERIALIZE__INIT;
  *message = init_value;
}
size_t pg_query__xml_serialize__get_packed_size
                     (const PgQuery__XmlSerialize *message)
{
  assert(message->base.descriptor == &pg_query__xml_serialize__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__xml_serialize__pack
                     (const PgQuery__XmlSerialize *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__xml_serialize__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__xml_serialize__pack_to_buffer
                     (const PgQuery__XmlSerialize *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__xml_serialize__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__XmlSerialize *
       pg_query__xml_serialize__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__XmlSerialize *)
     protobuf_c_message_unpack (&pg_query__xml_serialize__descriptor,
                                allocator, len, data);
}
void   pg_query__xml_serialize__free_unpacked
                     (PgQuery__XmlSerialize *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__xml_serialize__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__partition_elem__init
                     (PgQuery__PartitionElem         *message)
{
  static const PgQuery__PartitionElem init_value = PG_QUERY__PARTITION_ELEM__INIT;
  *message = init_value;
}
size_t pg_query__partition_elem__get_packed_size
                     (const PgQuery__PartitionElem *message)
{
  assert(message->base.descriptor == &pg_query__partition_elem__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__partition_elem__pack
                     (const PgQuery__PartitionElem *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__partition_elem__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__partition_elem__pack_to_buffer
                     (const PgQuery__PartitionElem *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__partition_elem__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__PartitionElem *
       pg_query__partition_elem__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__PartitionElem *)
     protobuf_c_message_unpack (&pg_query__partition_elem__descriptor,
                                allocator, len, data);
}
void   pg_query__partition_elem__free_unpacked
                     (PgQuery__PartitionElem *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__partition_elem__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__partition_spec__init
                     (PgQuery__PartitionSpec         *message)
{
  static const PgQuery__PartitionSpec init_value = PG_QUERY__PARTITION_SPEC__INIT;
  *message = init_value;
}
size_t pg_query__partition_spec__get_packed_size
                     (const PgQuery__PartitionSpec *message)
{
  assert(message->base.descriptor == &pg_query__partition_spec__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__partition_spec__pack
                     (const PgQuery__PartitionSpec *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__partition_spec__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__partition_spec__pack_to_buffer
                     (const PgQuery__PartitionSpec *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__partition_spec__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__PartitionSpec *
       pg_query__partition_spec__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__PartitionSpec *)
     protobuf_c_message_unpack (&pg_query__partition_spec__descriptor,
                                allocator, len, data);
}
void   pg_query__partition_spec__free_unpacked
                     (PgQuery__PartitionSpec *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__partition_spec__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__partition_bound_spec__init
                     (PgQuery__PartitionBoundSpec         *message)
{
  static const PgQuery__PartitionBoundSpec init_value = PG_QUERY__PARTITION_BOUND_SPEC__INIT;
  *message = init_value;
}
size_t pg_query__partition_bound_spec__get_packed_size
                     (const PgQuery__PartitionBoundSpec *message)
{
  assert(message->base.descriptor == &pg_query__partition_bound_spec__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__partition_bound_spec__pack
                     (const PgQuery__PartitionBoundSpec *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__partition_bound_spec__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__partition_bound_spec__pack_to_buffer
                     (const PgQuery__PartitionBoundSpec *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__partition_bound_spec__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__PartitionBoundSpec *
       pg_query__partition_bound_spec__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__PartitionBoundSpec *)
     protobuf_c_message_unpack (&pg_query__partition_bound_spec__descriptor,
                                allocator, len, data);
}
void   pg_query__partition_bound_spec__free_unpacked
                     (PgQuery__PartitionBoundSpec *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__partition_bound_spec__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__partition_range_datum__init
                     (PgQuery__PartitionRangeDatum         *message)
{
  static const PgQuery__PartitionRangeDatum init_value = PG_QUERY__PARTITION_RANGE_DATUM__INIT;
  *message = init_value;
}
size_t pg_query__partition_range_datum__get_packed_size
                     (const PgQuery__PartitionRangeDatum *message)
{
  assert(message->base.descriptor == &pg_query__partition_range_datum__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__partition_range_datum__pack
                     (const PgQuery__PartitionRangeDatum *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__partition_range_datum__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__partition_range_datum__pack_to_buffer
                     (const PgQuery__PartitionRangeDatum *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__partition_range_datum__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__PartitionRangeDatum *
       pg_query__partition_range_datum__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__PartitionRangeDatum *)
     protobuf_c_message_unpack (&pg_query__partition_range_datum__descriptor,
                                allocator, len, data);
}
void   pg_query__partition_range_datum__free_unpacked
                     (PgQuery__PartitionRangeDatum *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__partition_range_datum__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__single_partition_spec__init
                     (PgQuery__SinglePartitionSpec         *message)
{
  static const PgQuery__SinglePartitionSpec init_value = PG_QUERY__SINGLE_PARTITION_SPEC__INIT;
  *message = init_value;
}
size_t pg_query__single_partition_spec__get_packed_size
                     (const PgQuery__SinglePartitionSpec *message)
{
  assert(message->base.descriptor == &pg_query__single_partition_spec__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__single_partition_spec__pack
                     (const PgQuery__SinglePartitionSpec *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__single_partition_spec__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__single_partition_spec__pack_to_buffer
                     (const PgQuery__SinglePartitionSpec *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__single_partition_spec__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__SinglePartitionSpec *
       pg_query__single_partition_spec__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__SinglePartitionSpec *)
     protobuf_c_message_unpack (&pg_query__single_partition_spec__descriptor,
                                allocator, len, data);
}
void   pg_query__single_partition_spec__free_unpacked
                     (PgQuery__SinglePartitionSpec *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__single_partition_spec__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__partition_cmd__init
                     (PgQuery__PartitionCmd         *message)
{
  static const PgQuery__PartitionCmd init_value = PG_QUERY__PARTITION_CMD__INIT;
  *message = init_value;
}
size_t pg_query__partition_cmd__get_packed_size
                     (const PgQuery__PartitionCmd *message)
{
  assert(message->base.descriptor == &pg_query__partition_cmd__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__partition_cmd__pack
                     (const PgQuery__PartitionCmd *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__partition_cmd__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__partition_cmd__pack_to_buffer
                     (const PgQuery__PartitionCmd *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__partition_cmd__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__PartitionCmd *
       pg_query__partition_cmd__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__PartitionCmd *)
     protobuf_c_message_unpack (&pg_query__partition_cmd__descriptor,
                                allocator, len, data);
}
void   pg_query__partition_cmd__free_unpacked
                     (PgQuery__PartitionCmd *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__partition_cmd__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__range_tbl_entry__init
                     (PgQuery__RangeTblEntry         *message)
{
  static const PgQuery__RangeTblEntry init_value = PG_QUERY__RANGE_TBL_ENTRY__INIT;
  *message = init_value;
}
size_t pg_query__range_tbl_entry__get_packed_size
                     (const PgQuery__RangeTblEntry *message)
{
  assert(message->base.descriptor == &pg_query__range_tbl_entry__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__range_tbl_entry__pack
                     (const PgQuery__RangeTblEntry *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__range_tbl_entry__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__range_tbl_entry__pack_to_buffer
                     (const PgQuery__RangeTblEntry *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__range_tbl_entry__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__RangeTblEntry *
       pg_query__range_tbl_entry__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__RangeTblEntry *)
     protobuf_c_message_unpack (&pg_query__range_tbl_entry__descriptor,
                                allocator, len, data);
}
void   pg_query__range_tbl_entry__free_unpacked
                     (PgQuery__RangeTblEntry *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__range_tbl_entry__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__rtepermission_info__init
                     (PgQuery__RTEPermissionInfo         *message)
{
  static const PgQuery__RTEPermissionInfo init_value = PG_QUERY__RTEPERMISSION_INFO__INIT;
  *message = init_value;
}
size_t pg_query__rtepermission_info__get_packed_size
                     (const PgQuery__RTEPermissionInfo *message)
{
  assert(message->base.descriptor == &pg_query__rtepermission_info__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__rtepermission_info__pack
                     (const PgQuery__RTEPermissionInfo *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__rtepermission_info__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__rtepermission_info__pack_to_buffer
                     (const PgQuery__RTEPermissionInfo *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__rtepermission_info__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__RTEPermissionInfo *
       pg_query__rtepermission_info__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__RTEPermissionInfo *)
     protobuf_c_message_unpack (&pg_query__rtepermission_info__descriptor,
                                allocator, len, data);
}
void   pg_query__rtepermission_info__free_unpacked
                     (PgQuery__RTEPermissionInfo *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__rtepermission_info__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__range_tbl_function__init
                     (PgQuery__RangeTblFunction         *message)
{
  static const PgQuery__RangeTblFunction init_value = PG_QUERY__RANGE_TBL_FUNCTION__INIT;
  *message = init_value;
}
size_t pg_query__range_tbl_function__get_packed_size
                     (const PgQuery__RangeTblFunction *message)
{
  assert(message->base.descriptor == &pg_query__range_tbl_function__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__range_tbl_function__pack
                     (const PgQuery__RangeTblFunction *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__range_tbl_function__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__range_tbl_function__pack_to_buffer
                     (const PgQuery__RangeTblFunction *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__range_tbl_function__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__RangeTblFunction *
       pg_query__range_tbl_function__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__RangeTblFunction *)
     protobuf_c_message_unpack (&pg_query__range_tbl_function__descriptor,
                                allocator, len, data);
}
void   pg_query__range_tbl_function__free_unpacked
                     (PgQuery__RangeTblFunction *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__range_tbl_function__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__table_sample_clause__init
                     (PgQuery__TableSampleClause         *message)
{
  static const PgQuery__TableSampleClause init_value = PG_QUERY__TABLE_SAMPLE_CLAUSE__INIT;
  *message = init_value;
}
size_t pg_query__table_sample_clause__get_packed_size
                     (const PgQuery__TableSampleClause *message)
{
  assert(message->base.descriptor == &pg_query__table_sample_clause__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__table_sample_clause__pack
                     (const PgQuery__TableSampleClause *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__table_sample_clause__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__table_sample_clause__pack_to_buffer
                     (const PgQuery__TableSampleClause *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__table_sample_clause__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__TableSampleClause *
       pg_query__table_sample_clause__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__TableSampleClause *)
     protobuf_c_message_unpack (&pg_query__table_sample_clause__descriptor,
                                allocator, len, data);
}
void   pg_query__table_sample_clause__free_unpacked
                     (PgQuery__TableSampleClause *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__table_sample_clause__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__with_check_option__init
                     (PgQuery__WithCheckOption         *message)
{
  static const PgQuery__WithCheckOption init_value = PG_QUERY__WITH_CHECK_OPTION__INIT;
  *message = init_value;
}
size_t pg_query__with_check_option__get_packed_size
                     (const PgQuery__WithCheckOption *message)
{
  assert(message->base.descriptor == &pg_query__with_check_option__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__with_check_option__pack
                     (const PgQuery__WithCheckOption *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__with_check_option__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__with_check_option__pack_to_buffer
                     (const PgQuery__WithCheckOption *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__with_check_option__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__WithCheckOption *
       pg_query__with_check_option__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__WithCheckOption *)
     protobuf_c_message_unpack (&pg_query__with_check_option__descriptor,
                                allocator, len, data);
}
void   pg_query__with_check_option__free_unpacked
                     (PgQuery__WithCheckOption *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__with_check_option__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__sort_group_clause__init
                     (PgQuery__SortGroupClause         *message)
{
  static const PgQuery__SortGroupClause init_value = PG_QUERY__SORT_GROUP_CLAUSE__INIT;
  *message = init_value;
}
size_t pg_query__sort_group_clause__get_packed_size
                     (const PgQuery__SortGroupClause *message)
{
  assert(message->base.descriptor == &pg_query__sort_group_clause__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__sort_group_clause__pack
                     (const PgQuery__SortGroupClause *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__sort_group_clause__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__sort_group_clause__pack_to_buffer
                     (const PgQuery__SortGroupClause *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__sort_group_clause__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__SortGroupClause *
       pg_query__sort_group_clause__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__SortGroupClause *)
     protobuf_c_message_unpack (&pg_query__sort_group_clause__descriptor,
                                allocator, len, data);
}
void   pg_query__sort_group_clause__free_unpacked
                     (PgQuery__SortGroupClause *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__sort_group_clause__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__grouping_set__init
                     (PgQuery__GroupingSet         *message)
{
  static const PgQuery__GroupingSet init_value = PG_QUERY__GROUPING_SET__INIT;
  *message = init_value;
}
size_t pg_query__grouping_set__get_packed_size
                     (const PgQuery__GroupingSet *message)
{
  assert(message->base.descriptor == &pg_query__grouping_set__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__grouping_set__pack
                     (const PgQuery__GroupingSet *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__grouping_set__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__grouping_set__pack_to_buffer
                     (const PgQuery__GroupingSet *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__grouping_set__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__GroupingSet *
       pg_query__grouping_set__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__GroupingSet *)
     protobuf_c_message_unpack (&pg_query__grouping_set__descriptor,
                                allocator, len, data);
}
void   pg_query__grouping_set__free_unpacked
                     (PgQuery__GroupingSet *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__grouping_set__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__window_clause__init
                     (PgQuery__WindowClause         *message)
{
  static const PgQuery__WindowClause init_value = PG_QUERY__WINDOW_CLAUSE__INIT;
  *message = init_value;
}
size_t pg_query__window_clause__get_packed_size
                     (const PgQuery__WindowClause *message)
{
  assert(message->base.descriptor == &pg_query__window_clause__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__window_clause__pack
                     (const PgQuery__WindowClause *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__window_clause__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__window_clause__pack_to_buffer
                     (const PgQuery__WindowClause *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__window_clause__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__WindowClause *
       pg_query__window_clause__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__WindowClause *)
     protobuf_c_message_unpack (&pg_query__window_clause__descriptor,
                                allocator, len, data);
}
void   pg_query__window_clause__free_unpacked
                     (PgQuery__WindowClause *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__window_clause__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__row_mark_clause__init
                     (PgQuery__RowMarkClause         *message)
{
  static const PgQuery__RowMarkClause init_value = PG_QUERY__ROW_MARK_CLAUSE__INIT;
  *message = init_value;
}
size_t pg_query__row_mark_clause__get_packed_size
                     (const PgQuery__RowMarkClause *message)
{
  assert(message->base.descriptor == &pg_query__row_mark_clause__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__row_mark_clause__pack
                     (const PgQuery__RowMarkClause *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__row_mark_clause__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__row_mark_clause__pack_to_buffer
                     (const PgQuery__RowMarkClause *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__row_mark_clause__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__RowMarkClause *
       pg_query__row_mark_clause__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__RowMarkClause *)
     protobuf_c_message_unpack (&pg_query__row_mark_clause__descriptor,
                                allocator, len, data);
}
void   pg_query__row_mark_clause__free_unpacked
                     (PgQuery__RowMarkClause *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__row_mark_clause__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__with_clause__init
                     (PgQuery__WithClause         *message)
{
  static const PgQuery__WithClause init_value = PG_QUERY__WITH_CLAUSE__INIT;
  *message = init_value;
}
size_t pg_query__with_clause__get_packed_size
                     (const PgQuery__WithClause *message)
{
  assert(message->base.descriptor == &pg_query__with_clause__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__with_clause__pack
                     (const PgQuery__WithClause *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__with_clause__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__with_clause__pack_to_buffer
                     (const PgQuery__WithClause *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__with_clause__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__WithClause *
       pg_query__with_clause__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__WithClause *)
     protobuf_c_message_unpack (&pg_query__with_clause__descriptor,
                                allocator, len, data);
}
void   pg_query__with_clause__free_unpacked
                     (PgQuery__WithClause *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__with_clause__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__infer_clause__init
                     (PgQuery__InferClause         *message)
{
  static const PgQuery__InferClause init_value = PG_QUERY__INFER_CLAUSE__INIT;
  *message = init_value;
}
size_t pg_query__infer_clause__get_packed_size
                     (const PgQuery__InferClause *message)
{
  assert(message->base.descriptor == &pg_query__infer_clause__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__infer_clause__pack
                     (const PgQuery__InferClause *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__infer_clause__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__infer_clause__pack_to_buffer
                     (const PgQuery__InferClause *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__infer_clause__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__InferClause *
       pg_query__infer_clause__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__InferClause *)
     protobuf_c_message_unpack (&pg_query__infer_clause__descriptor,
                                allocator, len, data);
}
void   pg_query__infer_clause__free_unpacked
                     (PgQuery__InferClause *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__infer_clause__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__on_conflict_clause__init
                     (PgQuery__OnConflictClause         *message)
{
  static const PgQuery__OnConflictClause init_value = PG_QUERY__ON_CONFLICT_CLAUSE__INIT;
  *message = init_value;
}
size_t pg_query__on_conflict_clause__get_packed_size
                     (const PgQuery__OnConflictClause *message)
{
  assert(message->base.descriptor == &pg_query__on_conflict_clause__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__on_conflict_clause__pack
                     (const PgQuery__OnConflictClause *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__on_conflict_clause__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__on_conflict_clause__pack_to_buffer
                     (const PgQuery__OnConflictClause *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__on_conflict_clause__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__OnConflictClause *
       pg_query__on_conflict_clause__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__OnConflictClause *)
     protobuf_c_message_unpack (&pg_query__on_conflict_clause__descriptor,
                                allocator, len, data);
}
void   pg_query__on_conflict_clause__free_unpacked
                     (PgQuery__OnConflictClause *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__on_conflict_clause__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__ctesearch_clause__init
                     (PgQuery__CTESearchClause         *message)
{
  static const PgQuery__CTESearchClause init_value = PG_QUERY__CTESEARCH_CLAUSE__INIT;
  *message = init_value;
}
size_t pg_query__ctesearch_clause__get_packed_size
                     (const PgQuery__CTESearchClause *message)
{
  assert(message->base.descriptor == &pg_query__ctesearch_clause__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__ctesearch_clause__pack
                     (const PgQuery__CTESearchClause *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__ctesearch_clause__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__ctesearch_clause__pack_to_buffer
                     (const PgQuery__CTESearchClause *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__ctesearch_clause__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CTESearchClause *
       pg_query__ctesearch_clause__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CTESearchClause *)
     protobuf_c_message_unpack (&pg_query__ctesearch_clause__descriptor,
                                allocator, len, data);
}
void   pg_query__ctesearch_clause__free_unpacked
                     (PgQuery__CTESearchClause *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__ctesearch_clause__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__ctecycle_clause__init
                     (PgQuery__CTECycleClause         *message)
{
  static const PgQuery__CTECycleClause init_value = PG_QUERY__CTECYCLE_CLAUSE__INIT;
  *message = init_value;
}
size_t pg_query__ctecycle_clause__get_packed_size
                     (const PgQuery__CTECycleClause *message)
{
  assert(message->base.descriptor == &pg_query__ctecycle_clause__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__ctecycle_clause__pack
                     (const PgQuery__CTECycleClause *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__ctecycle_clause__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__ctecycle_clause__pack_to_buffer
                     (const PgQuery__CTECycleClause *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__ctecycle_clause__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CTECycleClause *
       pg_query__ctecycle_clause__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CTECycleClause *)
     protobuf_c_message_unpack (&pg_query__ctecycle_clause__descriptor,
                                allocator, len, data);
}
void   pg_query__ctecycle_clause__free_unpacked
                     (PgQuery__CTECycleClause *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__ctecycle_clause__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__common_table_expr__init
                     (PgQuery__CommonTableExpr         *message)
{
  static const PgQuery__CommonTableExpr init_value = PG_QUERY__COMMON_TABLE_EXPR__INIT;
  *message = init_value;
}
size_t pg_query__common_table_expr__get_packed_size
                     (const PgQuery__CommonTableExpr *message)
{
  assert(message->base.descriptor == &pg_query__common_table_expr__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__common_table_expr__pack
                     (const PgQuery__CommonTableExpr *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__common_table_expr__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__common_table_expr__pack_to_buffer
                     (const PgQuery__CommonTableExpr *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__common_table_expr__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CommonTableExpr *
       pg_query__common_table_expr__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CommonTableExpr *)
     protobuf_c_message_unpack (&pg_query__common_table_expr__descriptor,
                                allocator, len, data);
}
void   pg_query__common_table_expr__free_unpacked
                     (PgQuery__CommonTableExpr *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__common_table_expr__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__merge_when_clause__init
                     (PgQuery__MergeWhenClause         *message)
{
  static const PgQuery__MergeWhenClause init_value = PG_QUERY__MERGE_WHEN_CLAUSE__INIT;
  *message = init_value;
}
size_t pg_query__merge_when_clause__get_packed_size
                     (const PgQuery__MergeWhenClause *message)
{
  assert(message->base.descriptor == &pg_query__merge_when_clause__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__merge_when_clause__pack
                     (const PgQuery__MergeWhenClause *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__merge_when_clause__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__merge_when_clause__pack_to_buffer
                     (const PgQuery__MergeWhenClause *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__merge_when_clause__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__MergeWhenClause *
       pg_query__merge_when_clause__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__MergeWhenClause *)
     protobuf_c_message_unpack (&pg_query__merge_when_clause__descriptor,
                                allocator, len, data);
}
void   pg_query__merge_when_clause__free_unpacked
                     (PgQuery__MergeWhenClause *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__merge_when_clause__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__trigger_transition__init
                     (PgQuery__TriggerTransition         *message)
{
  static const PgQuery__TriggerTransition init_value = PG_QUERY__TRIGGER_TRANSITION__INIT;
  *message = init_value;
}
size_t pg_query__trigger_transition__get_packed_size
                     (const PgQuery__TriggerTransition *message)
{
  assert(message->base.descriptor == &pg_query__trigger_transition__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__trigger_transition__pack
                     (const PgQuery__TriggerTransition *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__trigger_transition__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__trigger_transition__pack_to_buffer
                     (const PgQuery__TriggerTransition *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__trigger_transition__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__TriggerTransition *
       pg_query__trigger_transition__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__TriggerTransition *)
     protobuf_c_message_unpack (&pg_query__trigger_transition__descriptor,
                                allocator, len, data);
}
void   pg_query__trigger_transition__free_unpacked
                     (PgQuery__TriggerTransition *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__trigger_transition__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__json_output__init
                     (PgQuery__JsonOutput         *message)
{
  static const PgQuery__JsonOutput init_value = PG_QUERY__JSON_OUTPUT__INIT;
  *message = init_value;
}
size_t pg_query__json_output__get_packed_size
                     (const PgQuery__JsonOutput *message)
{
  assert(message->base.descriptor == &pg_query__json_output__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__json_output__pack
                     (const PgQuery__JsonOutput *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__json_output__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__json_output__pack_to_buffer
                     (const PgQuery__JsonOutput *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__json_output__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__JsonOutput *
       pg_query__json_output__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__JsonOutput *)
     protobuf_c_message_unpack (&pg_query__json_output__descriptor,
                                allocator, len, data);
}
void   pg_query__json_output__free_unpacked
                     (PgQuery__JsonOutput *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__json_output__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__json_argument__init
                     (PgQuery__JsonArgument         *message)
{
  static const PgQuery__JsonArgument init_value = PG_QUERY__JSON_ARGUMENT__INIT;
  *message = init_value;
}
size_t pg_query__json_argument__get_packed_size
                     (const PgQuery__JsonArgument *message)
{
  assert(message->base.descriptor == &pg_query__json_argument__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__json_argument__pack
                     (const PgQuery__JsonArgument *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__json_argument__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__json_argument__pack_to_buffer
                     (const PgQuery__JsonArgument *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__json_argument__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__JsonArgument *
       pg_query__json_argument__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__JsonArgument *)
     protobuf_c_message_unpack (&pg_query__json_argument__descriptor,
                                allocator, len, data);
}
void   pg_query__json_argument__free_unpacked
                     (PgQuery__JsonArgument *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__json_argument__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__json_func_expr__init
                     (PgQuery__JsonFuncExpr         *message)
{
  static const PgQuery__JsonFuncExpr init_value = PG_QUERY__JSON_FUNC_EXPR__INIT;
  *message = init_value;
}
size_t pg_query__json_func_expr__get_packed_size
                     (const PgQuery__JsonFuncExpr *message)
{
  assert(message->base.descriptor == &pg_query__json_func_expr__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__json_func_expr__pack
                     (const PgQuery__JsonFuncExpr *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__json_func_expr__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__json_func_expr__pack_to_buffer
                     (const PgQuery__JsonFuncExpr *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__json_func_expr__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__JsonFuncExpr *
       pg_query__json_func_expr__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__JsonFuncExpr *)
     protobuf_c_message_unpack (&pg_query__json_func_expr__descriptor,
                                allocator, len, data);
}
void   pg_query__json_func_expr__free_unpacked
                     (PgQuery__JsonFuncExpr *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__json_func_expr__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__json_table_path_spec__init
                     (PgQuery__JsonTablePathSpec         *message)
{
  static const PgQuery__JsonTablePathSpec init_value = PG_QUERY__JSON_TABLE_PATH_SPEC__INIT;
  *message = init_value;
}
size_t pg_query__json_table_path_spec__get_packed_size
                     (const PgQuery__JsonTablePathSpec *message)
{
  assert(message->base.descriptor == &pg_query__json_table_path_spec__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__json_table_path_spec__pack
                     (const PgQuery__JsonTablePathSpec *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__json_table_path_spec__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__json_table_path_spec__pack_to_buffer
                     (const PgQuery__JsonTablePathSpec *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__json_table_path_spec__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__JsonTablePathSpec *
       pg_query__json_table_path_spec__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__JsonTablePathSpec *)
     protobuf_c_message_unpack (&pg_query__json_table_path_spec__descriptor,
                                allocator, len, data);
}
void   pg_query__json_table_path_spec__free_unpacked
                     (PgQuery__JsonTablePathSpec *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__json_table_path_spec__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__json_table__init
                     (PgQuery__JsonTable         *message)
{
  static const PgQuery__JsonTable init_value = PG_QUERY__JSON_TABLE__INIT;
  *message = init_value;
}
size_t pg_query__json_table__get_packed_size
                     (const PgQuery__JsonTable *message)
{
  assert(message->base.descriptor == &pg_query__json_table__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__json_table__pack
                     (const PgQuery__JsonTable *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__json_table__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__json_table__pack_to_buffer
                     (const PgQuery__JsonTable *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__json_table__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__JsonTable *
       pg_query__json_table__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__JsonTable *)
     protobuf_c_message_unpack (&pg_query__json_table__descriptor,
                                allocator, len, data);
}
void   pg_query__json_table__free_unpacked
                     (PgQuery__JsonTable *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__json_table__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__json_table_column__init
                     (PgQuery__JsonTableColumn         *message)
{
  static const PgQuery__JsonTableColumn init_value = PG_QUERY__JSON_TABLE_COLUMN__INIT;
  *message = init_value;
}
size_t pg_query__json_table_column__get_packed_size
                     (const PgQuery__JsonTableColumn *message)
{
  assert(message->base.descriptor == &pg_query__json_table_column__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__json_table_column__pack
                     (const PgQuery__JsonTableColumn *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__json_table_column__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__json_table_column__pack_to_buffer
                     (const PgQuery__JsonTableColumn *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__json_table_column__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__JsonTableColumn *
       pg_query__json_table_column__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__JsonTableColumn *)
     protobuf_c_message_unpack (&pg_query__json_table_column__descriptor,
                                allocator, len, data);
}
void   pg_query__json_table_column__free_unpacked
                     (PgQuery__JsonTableColumn *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__json_table_column__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__json_key_value__init
                     (PgQuery__JsonKeyValue         *message)
{
  static const PgQuery__JsonKeyValue init_value = PG_QUERY__JSON_KEY_VALUE__INIT;
  *message = init_value;
}
size_t pg_query__json_key_value__get_packed_size
                     (const PgQuery__JsonKeyValue *message)
{
  assert(message->base.descriptor == &pg_query__json_key_value__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__json_key_value__pack
                     (const PgQuery__JsonKeyValue *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__json_key_value__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__json_key_value__pack_to_buffer
                     (const PgQuery__JsonKeyValue *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__json_key_value__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__JsonKeyValue *
       pg_query__json_key_value__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__JsonKeyValue *)
     protobuf_c_message_unpack (&pg_query__json_key_value__descriptor,
                                allocator, len, data);
}
void   pg_query__json_key_value__free_unpacked
                     (PgQuery__JsonKeyValue *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__json_key_value__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__json_parse_expr__init
                     (PgQuery__JsonParseExpr         *message)
{
  static const PgQuery__JsonParseExpr init_value = PG_QUERY__JSON_PARSE_EXPR__INIT;
  *message = init_value;
}
size_t pg_query__json_parse_expr__get_packed_size
                     (const PgQuery__JsonParseExpr *message)
{
  assert(message->base.descriptor == &pg_query__json_parse_expr__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__json_parse_expr__pack
                     (const PgQuery__JsonParseExpr *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__json_parse_expr__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__json_parse_expr__pack_to_buffer
                     (const PgQuery__JsonParseExpr *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__json_parse_expr__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__JsonParseExpr *
       pg_query__json_parse_expr__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__JsonParseExpr *)
     protobuf_c_message_unpack (&pg_query__json_parse_expr__descriptor,
                                allocator, len, data);
}
void   pg_query__json_parse_expr__free_unpacked
                     (PgQuery__JsonParseExpr *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__json_parse_expr__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__json_scalar_expr__init
                     (PgQuery__JsonScalarExpr         *message)
{
  static const PgQuery__JsonScalarExpr init_value = PG_QUERY__JSON_SCALAR_EXPR__INIT;
  *message = init_value;
}
size_t pg_query__json_scalar_expr__get_packed_size
                     (const PgQuery__JsonScalarExpr *message)
{
  assert(message->base.descriptor == &pg_query__json_scalar_expr__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__json_scalar_expr__pack
                     (const PgQuery__JsonScalarExpr *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__json_scalar_expr__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__json_scalar_expr__pack_to_buffer
                     (const PgQuery__JsonScalarExpr *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__json_scalar_expr__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__JsonScalarExpr *
       pg_query__json_scalar_expr__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__JsonScalarExpr *)
     protobuf_c_message_unpack (&pg_query__json_scalar_expr__descriptor,
                                allocator, len, data);
}
void   pg_query__json_scalar_expr__free_unpacked
                     (PgQuery__JsonScalarExpr *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__json_scalar_expr__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__json_serialize_expr__init
                     (PgQuery__JsonSerializeExpr         *message)
{
  static const PgQuery__JsonSerializeExpr init_value = PG_QUERY__JSON_SERIALIZE_EXPR__INIT;
  *message = init_value;
}
size_t pg_query__json_serialize_expr__get_packed_size
                     (const PgQuery__JsonSerializeExpr *message)
{
  assert(message->base.descriptor == &pg_query__json_serialize_expr__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__json_serialize_expr__pack
                     (const PgQuery__JsonSerializeExpr *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__json_serialize_expr__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__json_serialize_expr__pack_to_buffer
                     (const PgQuery__JsonSerializeExpr *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__json_serialize_expr__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__JsonSerializeExpr *
       pg_query__json_serialize_expr__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__JsonSerializeExpr *)
     protobuf_c_message_unpack (&pg_query__json_serialize_expr__descriptor,
                                allocator, len, data);
}
void   pg_query__json_serialize_expr__free_unpacked
                     (PgQuery__JsonSerializeExpr *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__json_serialize_expr__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__json_object_constructor__init
                     (PgQuery__JsonObjectConstructor         *message)
{
  static const PgQuery__JsonObjectConstructor init_value = PG_QUERY__JSON_OBJECT_CONSTRUCTOR__INIT;
  *message = init_value;
}
size_t pg_query__json_object_constructor__get_packed_size
                     (const PgQuery__JsonObjectConstructor *message)
{
  assert(message->base.descriptor == &pg_query__json_object_constructor__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__json_object_constructor__pack
                     (const PgQuery__JsonObjectConstructor *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__json_object_constructor__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__json_object_constructor__pack_to_buffer
                     (const PgQuery__JsonObjectConstructor *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__json_object_constructor__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__JsonObjectConstructor *
       pg_query__json_object_constructor__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__JsonObjectConstructor *)
     protobuf_c_message_unpack (&pg_query__json_object_constructor__descriptor,
                                allocator, len, data);
}
void   pg_query__json_object_constructor__free_unpacked
                     (PgQuery__JsonObjectConstructor *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__json_object_constructor__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__json_array_constructor__init
                     (PgQuery__JsonArrayConstructor         *message)
{
  static const PgQuery__JsonArrayConstructor init_value = PG_QUERY__JSON_ARRAY_CONSTRUCTOR__INIT;
  *message = init_value;
}
size_t pg_query__json_array_constructor__get_packed_size
                     (const PgQuery__JsonArrayConstructor *message)
{
  assert(message->base.descriptor == &pg_query__json_array_constructor__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__json_array_constructor__pack
                     (const PgQuery__JsonArrayConstructor *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__json_array_constructor__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__json_array_constructor__pack_to_buffer
                     (const PgQuery__JsonArrayConstructor *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__json_array_constructor__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__JsonArrayConstructor *
       pg_query__json_array_constructor__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__JsonArrayConstructor *)
     protobuf_c_message_unpack (&pg_query__json_array_constructor__descriptor,
                                allocator, len, data);
}
void   pg_query__json_array_constructor__free_unpacked
                     (PgQuery__JsonArrayConstructor *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__json_array_constructor__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__json_array_query_constructor__init
                     (PgQuery__JsonArrayQueryConstructor         *message)
{
  static const PgQuery__JsonArrayQueryConstructor init_value = PG_QUERY__JSON_ARRAY_QUERY_CONSTRUCTOR__INIT;
  *message = init_value;
}
size_t pg_query__json_array_query_constructor__get_packed_size
                     (const PgQuery__JsonArrayQueryConstructor *message)
{
  assert(message->base.descriptor == &pg_query__json_array_query_constructor__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__json_array_query_constructor__pack
                     (const PgQuery__JsonArrayQueryConstructor *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__json_array_query_constructor__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__json_array_query_constructor__pack_to_buffer
                     (const PgQuery__JsonArrayQueryConstructor *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__json_array_query_constructor__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__JsonArrayQueryConstructor *
       pg_query__json_array_query_constructor__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__JsonArrayQueryConstructor *)
     protobuf_c_message_unpack (&pg_query__json_array_query_constructor__descriptor,
                                allocator, len, data);
}
void   pg_query__json_array_query_constructor__free_unpacked
                     (PgQuery__JsonArrayQueryConstructor *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__json_array_query_constructor__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__json_agg_constructor__init
                     (PgQuery__JsonAggConstructor         *message)
{
  static const PgQuery__JsonAggConstructor init_value = PG_QUERY__JSON_AGG_CONSTRUCTOR__INIT;
  *message = init_value;
}
size_t pg_query__json_agg_constructor__get_packed_size
                     (const PgQuery__JsonAggConstructor *message)
{
  assert(message->base.descriptor == &pg_query__json_agg_constructor__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__json_agg_constructor__pack
                     (const PgQuery__JsonAggConstructor *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__json_agg_constructor__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__json_agg_constructor__pack_to_buffer
                     (const PgQuery__JsonAggConstructor *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__json_agg_constructor__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__JsonAggConstructor *
       pg_query__json_agg_constructor__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__JsonAggConstructor *)
     protobuf_c_message_unpack (&pg_query__json_agg_constructor__descriptor,
                                allocator, len, data);
}
void   pg_query__json_agg_constructor__free_unpacked
                     (PgQuery__JsonAggConstructor *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__json_agg_constructor__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__json_object_agg__init
                     (PgQuery__JsonObjectAgg         *message)
{
  static const PgQuery__JsonObjectAgg init_value = PG_QUERY__JSON_OBJECT_AGG__INIT;
  *message = init_value;
}
size_t pg_query__json_object_agg__get_packed_size
                     (const PgQuery__JsonObjectAgg *message)
{
  assert(message->base.descriptor == &pg_query__json_object_agg__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__json_object_agg__pack
                     (const PgQuery__JsonObjectAgg *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__json_object_agg__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__json_object_agg__pack_to_buffer
                     (const PgQuery__JsonObjectAgg *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__json_object_agg__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__JsonObjectAgg *
       pg_query__json_object_agg__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__JsonObjectAgg *)
     protobuf_c_message_unpack (&pg_query__json_object_agg__descriptor,
                                allocator, len, data);
}
void   pg_query__json_object_agg__free_unpacked
                     (PgQuery__JsonObjectAgg *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__json_object_agg__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__json_array_agg__init
                     (PgQuery__JsonArrayAgg         *message)
{
  static const PgQuery__JsonArrayAgg init_value = PG_QUERY__JSON_ARRAY_AGG__INIT;
  *message = init_value;
}
size_t pg_query__json_array_agg__get_packed_size
                     (const PgQuery__JsonArrayAgg *message)
{
  assert(message->base.descriptor == &pg_query__json_array_agg__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__json_array_agg__pack
                     (const PgQuery__JsonArrayAgg *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__json_array_agg__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__json_array_agg__pack_to_buffer
                     (const PgQuery__JsonArrayAgg *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__json_array_agg__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__JsonArrayAgg *
       pg_query__json_array_agg__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__JsonArrayAgg *)
     protobuf_c_message_unpack (&pg_query__json_array_agg__descriptor,
                                allocator, len, data);
}
void   pg_query__json_array_agg__free_unpacked
                     (PgQuery__JsonArrayAgg *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__json_array_agg__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__raw_stmt__init
                     (PgQuery__RawStmt         *message)
{
  static const PgQuery__RawStmt init_value = PG_QUERY__RAW_STMT__INIT;
  *message = init_value;
}
size_t pg_query__raw_stmt__get_packed_size
                     (const PgQuery__RawStmt *message)
{
  assert(message->base.descriptor == &pg_query__raw_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__raw_stmt__pack
                     (const PgQuery__RawStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__raw_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__raw_stmt__pack_to_buffer
                     (const PgQuery__RawStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__raw_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__RawStmt *
       pg_query__raw_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__RawStmt *)
     protobuf_c_message_unpack (&pg_query__raw_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__raw_stmt__free_unpacked
                     (PgQuery__RawStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__raw_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__insert_stmt__init
                     (PgQuery__InsertStmt         *message)
{
  static const PgQuery__InsertStmt init_value = PG_QUERY__INSERT_STMT__INIT;
  *message = init_value;
}
size_t pg_query__insert_stmt__get_packed_size
                     (const PgQuery__InsertStmt *message)
{
  assert(message->base.descriptor == &pg_query__insert_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__insert_stmt__pack
                     (const PgQuery__InsertStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__insert_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__insert_stmt__pack_to_buffer
                     (const PgQuery__InsertStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__insert_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__InsertStmt *
       pg_query__insert_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__InsertStmt *)
     protobuf_c_message_unpack (&pg_query__insert_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__insert_stmt__free_unpacked
                     (PgQuery__InsertStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__insert_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__delete_stmt__init
                     (PgQuery__DeleteStmt         *message)
{
  static const PgQuery__DeleteStmt init_value = PG_QUERY__DELETE_STMT__INIT;
  *message = init_value;
}
size_t pg_query__delete_stmt__get_packed_size
                     (const PgQuery__DeleteStmt *message)
{
  assert(message->base.descriptor == &pg_query__delete_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__delete_stmt__pack
                     (const PgQuery__DeleteStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__delete_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__delete_stmt__pack_to_buffer
                     (const PgQuery__DeleteStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__delete_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__DeleteStmt *
       pg_query__delete_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__DeleteStmt *)
     protobuf_c_message_unpack (&pg_query__delete_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__delete_stmt__free_unpacked
                     (PgQuery__DeleteStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__delete_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__update_stmt__init
                     (PgQuery__UpdateStmt         *message)
{
  static const PgQuery__UpdateStmt init_value = PG_QUERY__UPDATE_STMT__INIT;
  *message = init_value;
}
size_t pg_query__update_stmt__get_packed_size
                     (const PgQuery__UpdateStmt *message)
{
  assert(message->base.descriptor == &pg_query__update_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__update_stmt__pack
                     (const PgQuery__UpdateStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__update_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__update_stmt__pack_to_buffer
                     (const PgQuery__UpdateStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__update_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__UpdateStmt *
       pg_query__update_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__UpdateStmt *)
     protobuf_c_message_unpack (&pg_query__update_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__update_stmt__free_unpacked
                     (PgQuery__UpdateStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__update_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__merge_stmt__init
                     (PgQuery__MergeStmt         *message)
{
  static const PgQuery__MergeStmt init_value = PG_QUERY__MERGE_STMT__INIT;
  *message = init_value;
}
size_t pg_query__merge_stmt__get_packed_size
                     (const PgQuery__MergeStmt *message)
{
  assert(message->base.descriptor == &pg_query__merge_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__merge_stmt__pack
                     (const PgQuery__MergeStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__merge_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__merge_stmt__pack_to_buffer
                     (const PgQuery__MergeStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__merge_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__MergeStmt *
       pg_query__merge_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__MergeStmt *)
     protobuf_c_message_unpack (&pg_query__merge_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__merge_stmt__free_unpacked
                     (PgQuery__MergeStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__merge_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__select_stmt__init
                     (PgQuery__SelectStmt         *message)
{
  static const PgQuery__SelectStmt init_value = PG_QUERY__SELECT_STMT__INIT;
  *message = init_value;
}
size_t pg_query__select_stmt__get_packed_size
                     (const PgQuery__SelectStmt *message)
{
  assert(message->base.descriptor == &pg_query__select_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__select_stmt__pack
                     (const PgQuery__SelectStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__select_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__select_stmt__pack_to_buffer
                     (const PgQuery__SelectStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__select_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__SelectStmt *
       pg_query__select_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__SelectStmt *)
     protobuf_c_message_unpack (&pg_query__select_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__select_stmt__free_unpacked
                     (PgQuery__SelectStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__select_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__set_operation_stmt__init
                     (PgQuery__SetOperationStmt         *message)
{
  static const PgQuery__SetOperationStmt init_value = PG_QUERY__SET_OPERATION_STMT__INIT;
  *message = init_value;
}
size_t pg_query__set_operation_stmt__get_packed_size
                     (const PgQuery__SetOperationStmt *message)
{
  assert(message->base.descriptor == &pg_query__set_operation_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__set_operation_stmt__pack
                     (const PgQuery__SetOperationStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__set_operation_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__set_operation_stmt__pack_to_buffer
                     (const PgQuery__SetOperationStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__set_operation_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__SetOperationStmt *
       pg_query__set_operation_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__SetOperationStmt *)
     protobuf_c_message_unpack (&pg_query__set_operation_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__set_operation_stmt__free_unpacked
                     (PgQuery__SetOperationStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__set_operation_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__return_stmt__init
                     (PgQuery__ReturnStmt         *message)
{
  static const PgQuery__ReturnStmt init_value = PG_QUERY__RETURN_STMT__INIT;
  *message = init_value;
}
size_t pg_query__return_stmt__get_packed_size
                     (const PgQuery__ReturnStmt *message)
{
  assert(message->base.descriptor == &pg_query__return_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__return_stmt__pack
                     (const PgQuery__ReturnStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__return_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__return_stmt__pack_to_buffer
                     (const PgQuery__ReturnStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__return_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__ReturnStmt *
       pg_query__return_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__ReturnStmt *)
     protobuf_c_message_unpack (&pg_query__return_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__return_stmt__free_unpacked
                     (PgQuery__ReturnStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__return_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__plassign_stmt__init
                     (PgQuery__PLAssignStmt         *message)
{
  static const PgQuery__PLAssignStmt init_value = PG_QUERY__PLASSIGN_STMT__INIT;
  *message = init_value;
}
size_t pg_query__plassign_stmt__get_packed_size
                     (const PgQuery__PLAssignStmt *message)
{
  assert(message->base.descriptor == &pg_query__plassign_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__plassign_stmt__pack
                     (const PgQuery__PLAssignStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__plassign_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__plassign_stmt__pack_to_buffer
                     (const PgQuery__PLAssignStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__plassign_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__PLAssignStmt *
       pg_query__plassign_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__PLAssignStmt *)
     protobuf_c_message_unpack (&pg_query__plassign_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__plassign_stmt__free_unpacked
                     (PgQuery__PLAssignStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__plassign_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__create_schema_stmt__init
                     (PgQuery__CreateSchemaStmt         *message)
{
  static const PgQuery__CreateSchemaStmt init_value = PG_QUERY__CREATE_SCHEMA_STMT__INIT;
  *message = init_value;
}
size_t pg_query__create_schema_stmt__get_packed_size
                     (const PgQuery__CreateSchemaStmt *message)
{
  assert(message->base.descriptor == &pg_query__create_schema_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__create_schema_stmt__pack
                     (const PgQuery__CreateSchemaStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__create_schema_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__create_schema_stmt__pack_to_buffer
                     (const PgQuery__CreateSchemaStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__create_schema_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CreateSchemaStmt *
       pg_query__create_schema_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CreateSchemaStmt *)
     protobuf_c_message_unpack (&pg_query__create_schema_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__create_schema_stmt__free_unpacked
                     (PgQuery__CreateSchemaStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__create_schema_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__alter_table_stmt__init
                     (PgQuery__AlterTableStmt         *message)
{
  static const PgQuery__AlterTableStmt init_value = PG_QUERY__ALTER_TABLE_STMT__INIT;
  *message = init_value;
}
size_t pg_query__alter_table_stmt__get_packed_size
                     (const PgQuery__AlterTableStmt *message)
{
  assert(message->base.descriptor == &pg_query__alter_table_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__alter_table_stmt__pack
                     (const PgQuery__AlterTableStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__alter_table_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__alter_table_stmt__pack_to_buffer
                     (const PgQuery__AlterTableStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__alter_table_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AlterTableStmt *
       pg_query__alter_table_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AlterTableStmt *)
     protobuf_c_message_unpack (&pg_query__alter_table_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__alter_table_stmt__free_unpacked
                     (PgQuery__AlterTableStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__alter_table_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__replica_identity_stmt__init
                     (PgQuery__ReplicaIdentityStmt         *message)
{
  static const PgQuery__ReplicaIdentityStmt init_value = PG_QUERY__REPLICA_IDENTITY_STMT__INIT;
  *message = init_value;
}
size_t pg_query__replica_identity_stmt__get_packed_size
                     (const PgQuery__ReplicaIdentityStmt *message)
{
  assert(message->base.descriptor == &pg_query__replica_identity_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__replica_identity_stmt__pack
                     (const PgQuery__ReplicaIdentityStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__replica_identity_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__replica_identity_stmt__pack_to_buffer
                     (const PgQuery__ReplicaIdentityStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__replica_identity_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__ReplicaIdentityStmt *
       pg_query__replica_identity_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__ReplicaIdentityStmt *)
     protobuf_c_message_unpack (&pg_query__replica_identity_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__replica_identity_stmt__free_unpacked
                     (PgQuery__ReplicaIdentityStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__replica_identity_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__alter_table_cmd__init
                     (PgQuery__AlterTableCmd         *message)
{
  static const PgQuery__AlterTableCmd init_value = PG_QUERY__ALTER_TABLE_CMD__INIT;
  *message = init_value;
}
size_t pg_query__alter_table_cmd__get_packed_size
                     (const PgQuery__AlterTableCmd *message)
{
  assert(message->base.descriptor == &pg_query__alter_table_cmd__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__alter_table_cmd__pack
                     (const PgQuery__AlterTableCmd *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__alter_table_cmd__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__alter_table_cmd__pack_to_buffer
                     (const PgQuery__AlterTableCmd *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__alter_table_cmd__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AlterTableCmd *
       pg_query__alter_table_cmd__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AlterTableCmd *)
     protobuf_c_message_unpack (&pg_query__alter_table_cmd__descriptor,
                                allocator, len, data);
}
void   pg_query__alter_table_cmd__free_unpacked
                     (PgQuery__AlterTableCmd *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__alter_table_cmd__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__alter_collation_stmt__init
                     (PgQuery__AlterCollationStmt         *message)
{
  static const PgQuery__AlterCollationStmt init_value = PG_QUERY__ALTER_COLLATION_STMT__INIT;
  *message = init_value;
}
size_t pg_query__alter_collation_stmt__get_packed_size
                     (const PgQuery__AlterCollationStmt *message)
{
  assert(message->base.descriptor == &pg_query__alter_collation_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__alter_collation_stmt__pack
                     (const PgQuery__AlterCollationStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__alter_collation_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__alter_collation_stmt__pack_to_buffer
                     (const PgQuery__AlterCollationStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__alter_collation_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AlterCollationStmt *
       pg_query__alter_collation_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AlterCollationStmt *)
     protobuf_c_message_unpack (&pg_query__alter_collation_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__alter_collation_stmt__free_unpacked
                     (PgQuery__AlterCollationStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__alter_collation_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__alter_domain_stmt__init
                     (PgQuery__AlterDomainStmt         *message)
{
  static const PgQuery__AlterDomainStmt init_value = PG_QUERY__ALTER_DOMAIN_STMT__INIT;
  *message = init_value;
}
size_t pg_query__alter_domain_stmt__get_packed_size
                     (const PgQuery__AlterDomainStmt *message)
{
  assert(message->base.descriptor == &pg_query__alter_domain_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__alter_domain_stmt__pack
                     (const PgQuery__AlterDomainStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__alter_domain_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__alter_domain_stmt__pack_to_buffer
                     (const PgQuery__AlterDomainStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__alter_domain_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AlterDomainStmt *
       pg_query__alter_domain_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AlterDomainStmt *)
     protobuf_c_message_unpack (&pg_query__alter_domain_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__alter_domain_stmt__free_unpacked
                     (PgQuery__AlterDomainStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__alter_domain_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__grant_stmt__init
                     (PgQuery__GrantStmt         *message)
{
  static const PgQuery__GrantStmt init_value = PG_QUERY__GRANT_STMT__INIT;
  *message = init_value;
}
size_t pg_query__grant_stmt__get_packed_size
                     (const PgQuery__GrantStmt *message)
{
  assert(message->base.descriptor == &pg_query__grant_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__grant_stmt__pack
                     (const PgQuery__GrantStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__grant_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__grant_stmt__pack_to_buffer
                     (const PgQuery__GrantStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__grant_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__GrantStmt *
       pg_query__grant_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__GrantStmt *)
     protobuf_c_message_unpack (&pg_query__grant_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__grant_stmt__free_unpacked
                     (PgQuery__GrantStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__grant_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__object_with_args__init
                     (PgQuery__ObjectWithArgs         *message)
{
  static const PgQuery__ObjectWithArgs init_value = PG_QUERY__OBJECT_WITH_ARGS__INIT;
  *message = init_value;
}
size_t pg_query__object_with_args__get_packed_size
                     (const PgQuery__ObjectWithArgs *message)
{
  assert(message->base.descriptor == &pg_query__object_with_args__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__object_with_args__pack
                     (const PgQuery__ObjectWithArgs *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__object_with_args__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__object_with_args__pack_to_buffer
                     (const PgQuery__ObjectWithArgs *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__object_with_args__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__ObjectWithArgs *
       pg_query__object_with_args__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__ObjectWithArgs *)
     protobuf_c_message_unpack (&pg_query__object_with_args__descriptor,
                                allocator, len, data);
}
void   pg_query__object_with_args__free_unpacked
                     (PgQuery__ObjectWithArgs *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__object_with_args__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__access_priv__init
                     (PgQuery__AccessPriv         *message)
{
  static const PgQuery__AccessPriv init_value = PG_QUERY__ACCESS_PRIV__INIT;
  *message = init_value;
}
size_t pg_query__access_priv__get_packed_size
                     (const PgQuery__AccessPriv *message)
{
  assert(message->base.descriptor == &pg_query__access_priv__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__access_priv__pack
                     (const PgQuery__AccessPriv *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__access_priv__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__access_priv__pack_to_buffer
                     (const PgQuery__AccessPriv *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__access_priv__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AccessPriv *
       pg_query__access_priv__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AccessPriv *)
     protobuf_c_message_unpack (&pg_query__access_priv__descriptor,
                                allocator, len, data);
}
void   pg_query__access_priv__free_unpacked
                     (PgQuery__AccessPriv *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__access_priv__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__grant_role_stmt__init
                     (PgQuery__GrantRoleStmt         *message)
{
  static const PgQuery__GrantRoleStmt init_value = PG_QUERY__GRANT_ROLE_STMT__INIT;
  *message = init_value;
}
size_t pg_query__grant_role_stmt__get_packed_size
                     (const PgQuery__GrantRoleStmt *message)
{
  assert(message->base.descriptor == &pg_query__grant_role_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__grant_role_stmt__pack
                     (const PgQuery__GrantRoleStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__grant_role_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__grant_role_stmt__pack_to_buffer
                     (const PgQuery__GrantRoleStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__grant_role_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__GrantRoleStmt *
       pg_query__grant_role_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__GrantRoleStmt *)
     protobuf_c_message_unpack (&pg_query__grant_role_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__grant_role_stmt__free_unpacked
                     (PgQuery__GrantRoleStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__grant_role_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__alter_default_privileges_stmt__init
                     (PgQuery__AlterDefaultPrivilegesStmt         *message)
{
  static const PgQuery__AlterDefaultPrivilegesStmt init_value = PG_QUERY__ALTER_DEFAULT_PRIVILEGES_STMT__INIT;
  *message = init_value;
}
size_t pg_query__alter_default_privileges_stmt__get_packed_size
                     (const PgQuery__AlterDefaultPrivilegesStmt *message)
{
  assert(message->base.descriptor == &pg_query__alter_default_privileges_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__alter_default_privileges_stmt__pack
                     (const PgQuery__AlterDefaultPrivilegesStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__alter_default_privileges_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__alter_default_privileges_stmt__pack_to_buffer
                     (const PgQuery__AlterDefaultPrivilegesStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__alter_default_privileges_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AlterDefaultPrivilegesStmt *
       pg_query__alter_default_privileges_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AlterDefaultPrivilegesStmt *)
     protobuf_c_message_unpack (&pg_query__alter_default_privileges_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__alter_default_privileges_stmt__free_unpacked
                     (PgQuery__AlterDefaultPrivilegesStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__alter_default_privileges_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__copy_stmt__init
                     (PgQuery__CopyStmt         *message)
{
  static const PgQuery__CopyStmt init_value = PG_QUERY__COPY_STMT__INIT;
  *message = init_value;
}
size_t pg_query__copy_stmt__get_packed_size
                     (const PgQuery__CopyStmt *message)
{
  assert(message->base.descriptor == &pg_query__copy_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__copy_stmt__pack
                     (const PgQuery__CopyStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__copy_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__copy_stmt__pack_to_buffer
                     (const PgQuery__CopyStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__copy_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CopyStmt *
       pg_query__copy_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CopyStmt *)
     protobuf_c_message_unpack (&pg_query__copy_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__copy_stmt__free_unpacked
                     (PgQuery__CopyStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__copy_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__variable_set_stmt__init
                     (PgQuery__VariableSetStmt         *message)
{
  static const PgQuery__VariableSetStmt init_value = PG_QUERY__VARIABLE_SET_STMT__INIT;
  *message = init_value;
}
size_t pg_query__variable_set_stmt__get_packed_size
                     (const PgQuery__VariableSetStmt *message)
{
  assert(message->base.descriptor == &pg_query__variable_set_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__variable_set_stmt__pack
                     (const PgQuery__VariableSetStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__variable_set_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__variable_set_stmt__pack_to_buffer
                     (const PgQuery__VariableSetStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__variable_set_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__VariableSetStmt *
       pg_query__variable_set_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__VariableSetStmt *)
     protobuf_c_message_unpack (&pg_query__variable_set_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__variable_set_stmt__free_unpacked
                     (PgQuery__VariableSetStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__variable_set_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__variable_show_stmt__init
                     (PgQuery__VariableShowStmt         *message)
{
  static const PgQuery__VariableShowStmt init_value = PG_QUERY__VARIABLE_SHOW_STMT__INIT;
  *message = init_value;
}
size_t pg_query__variable_show_stmt__get_packed_size
                     (const PgQuery__VariableShowStmt *message)
{
  assert(message->base.descriptor == &pg_query__variable_show_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__variable_show_stmt__pack
                     (const PgQuery__VariableShowStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__variable_show_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__variable_show_stmt__pack_to_buffer
                     (const PgQuery__VariableShowStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__variable_show_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__VariableShowStmt *
       pg_query__variable_show_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__VariableShowStmt *)
     protobuf_c_message_unpack (&pg_query__variable_show_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__variable_show_stmt__free_unpacked
                     (PgQuery__VariableShowStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__variable_show_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__create_stmt__init
                     (PgQuery__CreateStmt         *message)
{
  static const PgQuery__CreateStmt init_value = PG_QUERY__CREATE_STMT__INIT;
  *message = init_value;
}
size_t pg_query__create_stmt__get_packed_size
                     (const PgQuery__CreateStmt *message)
{
  assert(message->base.descriptor == &pg_query__create_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__create_stmt__pack
                     (const PgQuery__CreateStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__create_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__create_stmt__pack_to_buffer
                     (const PgQuery__CreateStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__create_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CreateStmt *
       pg_query__create_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CreateStmt *)
     protobuf_c_message_unpack (&pg_query__create_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__create_stmt__free_unpacked
                     (PgQuery__CreateStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__create_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__constraint__init
                     (PgQuery__Constraint         *message)
{
  static const PgQuery__Constraint init_value = PG_QUERY__CONSTRAINT__INIT;
  *message = init_value;
}
size_t pg_query__constraint__get_packed_size
                     (const PgQuery__Constraint *message)
{
  assert(message->base.descriptor == &pg_query__constraint__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__constraint__pack
                     (const PgQuery__Constraint *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__constraint__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__constraint__pack_to_buffer
                     (const PgQuery__Constraint *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__constraint__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__Constraint *
       pg_query__constraint__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__Constraint *)
     protobuf_c_message_unpack (&pg_query__constraint__descriptor,
                                allocator, len, data);
}
void   pg_query__constraint__free_unpacked
                     (PgQuery__Constraint *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__constraint__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__create_table_space_stmt__init
                     (PgQuery__CreateTableSpaceStmt         *message)
{
  static const PgQuery__CreateTableSpaceStmt init_value = PG_QUERY__CREATE_TABLE_SPACE_STMT__INIT;
  *message = init_value;
}
size_t pg_query__create_table_space_stmt__get_packed_size
                     (const PgQuery__CreateTableSpaceStmt *message)
{
  assert(message->base.descriptor == &pg_query__create_table_space_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__create_table_space_stmt__pack
                     (const PgQuery__CreateTableSpaceStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__create_table_space_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__create_table_space_stmt__pack_to_buffer
                     (const PgQuery__CreateTableSpaceStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__create_table_space_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CreateTableSpaceStmt *
       pg_query__create_table_space_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CreateTableSpaceStmt *)
     protobuf_c_message_unpack (&pg_query__create_table_space_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__create_table_space_stmt__free_unpacked
                     (PgQuery__CreateTableSpaceStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__create_table_space_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__drop_table_space_stmt__init
                     (PgQuery__DropTableSpaceStmt         *message)
{
  static const PgQuery__DropTableSpaceStmt init_value = PG_QUERY__DROP_TABLE_SPACE_STMT__INIT;
  *message = init_value;
}
size_t pg_query__drop_table_space_stmt__get_packed_size
                     (const PgQuery__DropTableSpaceStmt *message)
{
  assert(message->base.descriptor == &pg_query__drop_table_space_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__drop_table_space_stmt__pack
                     (const PgQuery__DropTableSpaceStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__drop_table_space_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__drop_table_space_stmt__pack_to_buffer
                     (const PgQuery__DropTableSpaceStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__drop_table_space_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__DropTableSpaceStmt *
       pg_query__drop_table_space_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__DropTableSpaceStmt *)
     protobuf_c_message_unpack (&pg_query__drop_table_space_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__drop_table_space_stmt__free_unpacked
                     (PgQuery__DropTableSpaceStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__drop_table_space_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__alter_table_space_options_stmt__init
                     (PgQuery__AlterTableSpaceOptionsStmt         *message)
{
  static const PgQuery__AlterTableSpaceOptionsStmt init_value = PG_QUERY__ALTER_TABLE_SPACE_OPTIONS_STMT__INIT;
  *message = init_value;
}
size_t pg_query__alter_table_space_options_stmt__get_packed_size
                     (const PgQuery__AlterTableSpaceOptionsStmt *message)
{
  assert(message->base.descriptor == &pg_query__alter_table_space_options_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__alter_table_space_options_stmt__pack
                     (const PgQuery__AlterTableSpaceOptionsStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__alter_table_space_options_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__alter_table_space_options_stmt__pack_to_buffer
                     (const PgQuery__AlterTableSpaceOptionsStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__alter_table_space_options_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AlterTableSpaceOptionsStmt *
       pg_query__alter_table_space_options_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AlterTableSpaceOptionsStmt *)
     protobuf_c_message_unpack (&pg_query__alter_table_space_options_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__alter_table_space_options_stmt__free_unpacked
                     (PgQuery__AlterTableSpaceOptionsStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__alter_table_space_options_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__alter_table_move_all_stmt__init
                     (PgQuery__AlterTableMoveAllStmt         *message)
{
  static const PgQuery__AlterTableMoveAllStmt init_value = PG_QUERY__ALTER_TABLE_MOVE_ALL_STMT__INIT;
  *message = init_value;
}
size_t pg_query__alter_table_move_all_stmt__get_packed_size
                     (const PgQuery__AlterTableMoveAllStmt *message)
{
  assert(message->base.descriptor == &pg_query__alter_table_move_all_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__alter_table_move_all_stmt__pack
                     (const PgQuery__AlterTableMoveAllStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__alter_table_move_all_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__alter_table_move_all_stmt__pack_to_buffer
                     (const PgQuery__AlterTableMoveAllStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__alter_table_move_all_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AlterTableMoveAllStmt *
       pg_query__alter_table_move_all_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AlterTableMoveAllStmt *)
     protobuf_c_message_unpack (&pg_query__alter_table_move_all_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__alter_table_move_all_stmt__free_unpacked
                     (PgQuery__AlterTableMoveAllStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__alter_table_move_all_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__create_extension_stmt__init
                     (PgQuery__CreateExtensionStmt         *message)
{
  static const PgQuery__CreateExtensionStmt init_value = PG_QUERY__CREATE_EXTENSION_STMT__INIT;
  *message = init_value;
}
size_t pg_query__create_extension_stmt__get_packed_size
                     (const PgQuery__CreateExtensionStmt *message)
{
  assert(message->base.descriptor == &pg_query__create_extension_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__create_extension_stmt__pack
                     (const PgQuery__CreateExtensionStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__create_extension_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__create_extension_stmt__pack_to_buffer
                     (const PgQuery__CreateExtensionStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__create_extension_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CreateExtensionStmt *
       pg_query__create_extension_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CreateExtensionStmt *)
     protobuf_c_message_unpack (&pg_query__create_extension_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__create_extension_stmt__free_unpacked
                     (PgQuery__CreateExtensionStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__create_extension_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__alter_extension_stmt__init
                     (PgQuery__AlterExtensionStmt         *message)
{
  static const PgQuery__AlterExtensionStmt init_value = PG_QUERY__ALTER_EXTENSION_STMT__INIT;
  *message = init_value;
}
size_t pg_query__alter_extension_stmt__get_packed_size
                     (const PgQuery__AlterExtensionStmt *message)
{
  assert(message->base.descriptor == &pg_query__alter_extension_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__alter_extension_stmt__pack
                     (const PgQuery__AlterExtensionStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__alter_extension_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__alter_extension_stmt__pack_to_buffer
                     (const PgQuery__AlterExtensionStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__alter_extension_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AlterExtensionStmt *
       pg_query__alter_extension_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AlterExtensionStmt *)
     protobuf_c_message_unpack (&pg_query__alter_extension_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__alter_extension_stmt__free_unpacked
                     (PgQuery__AlterExtensionStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__alter_extension_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__alter_extension_contents_stmt__init
                     (PgQuery__AlterExtensionContentsStmt         *message)
{
  static const PgQuery__AlterExtensionContentsStmt init_value = PG_QUERY__ALTER_EXTENSION_CONTENTS_STMT__INIT;
  *message = init_value;
}
size_t pg_query__alter_extension_contents_stmt__get_packed_size
                     (const PgQuery__AlterExtensionContentsStmt *message)
{
  assert(message->base.descriptor == &pg_query__alter_extension_contents_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__alter_extension_contents_stmt__pack
                     (const PgQuery__AlterExtensionContentsStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__alter_extension_contents_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__alter_extension_contents_stmt__pack_to_buffer
                     (const PgQuery__AlterExtensionContentsStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__alter_extension_contents_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AlterExtensionContentsStmt *
       pg_query__alter_extension_contents_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AlterExtensionContentsStmt *)
     protobuf_c_message_unpack (&pg_query__alter_extension_contents_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__alter_extension_contents_stmt__free_unpacked
                     (PgQuery__AlterExtensionContentsStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__alter_extension_contents_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__create_fdw_stmt__init
                     (PgQuery__CreateFdwStmt         *message)
{
  static const PgQuery__CreateFdwStmt init_value = PG_QUERY__CREATE_FDW_STMT__INIT;
  *message = init_value;
}
size_t pg_query__create_fdw_stmt__get_packed_size
                     (const PgQuery__CreateFdwStmt *message)
{
  assert(message->base.descriptor == &pg_query__create_fdw_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__create_fdw_stmt__pack
                     (const PgQuery__CreateFdwStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__create_fdw_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__create_fdw_stmt__pack_to_buffer
                     (const PgQuery__CreateFdwStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__create_fdw_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CreateFdwStmt *
       pg_query__create_fdw_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CreateFdwStmt *)
     protobuf_c_message_unpack (&pg_query__create_fdw_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__create_fdw_stmt__free_unpacked
                     (PgQuery__CreateFdwStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__create_fdw_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__alter_fdw_stmt__init
                     (PgQuery__AlterFdwStmt         *message)
{
  static const PgQuery__AlterFdwStmt init_value = PG_QUERY__ALTER_FDW_STMT__INIT;
  *message = init_value;
}
size_t pg_query__alter_fdw_stmt__get_packed_size
                     (const PgQuery__AlterFdwStmt *message)
{
  assert(message->base.descriptor == &pg_query__alter_fdw_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__alter_fdw_stmt__pack
                     (const PgQuery__AlterFdwStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__alter_fdw_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__alter_fdw_stmt__pack_to_buffer
                     (const PgQuery__AlterFdwStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__alter_fdw_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AlterFdwStmt *
       pg_query__alter_fdw_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AlterFdwStmt *)
     protobuf_c_message_unpack (&pg_query__alter_fdw_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__alter_fdw_stmt__free_unpacked
                     (PgQuery__AlterFdwStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__alter_fdw_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__create_foreign_server_stmt__init
                     (PgQuery__CreateForeignServerStmt         *message)
{
  static const PgQuery__CreateForeignServerStmt init_value = PG_QUERY__CREATE_FOREIGN_SERVER_STMT__INIT;
  *message = init_value;
}
size_t pg_query__create_foreign_server_stmt__get_packed_size
                     (const PgQuery__CreateForeignServerStmt *message)
{
  assert(message->base.descriptor == &pg_query__create_foreign_server_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__create_foreign_server_stmt__pack
                     (const PgQuery__CreateForeignServerStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__create_foreign_server_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__create_foreign_server_stmt__pack_to_buffer
                     (const PgQuery__CreateForeignServerStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__create_foreign_server_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CreateForeignServerStmt *
       pg_query__create_foreign_server_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CreateForeignServerStmt *)
     protobuf_c_message_unpack (&pg_query__create_foreign_server_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__create_foreign_server_stmt__free_unpacked
                     (PgQuery__CreateForeignServerStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__create_foreign_server_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__alter_foreign_server_stmt__init
                     (PgQuery__AlterForeignServerStmt         *message)
{
  static const PgQuery__AlterForeignServerStmt init_value = PG_QUERY__ALTER_FOREIGN_SERVER_STMT__INIT;
  *message = init_value;
}
size_t pg_query__alter_foreign_server_stmt__get_packed_size
                     (const PgQuery__AlterForeignServerStmt *message)
{
  assert(message->base.descriptor == &pg_query__alter_foreign_server_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__alter_foreign_server_stmt__pack
                     (const PgQuery__AlterForeignServerStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__alter_foreign_server_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__alter_foreign_server_stmt__pack_to_buffer
                     (const PgQuery__AlterForeignServerStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__alter_foreign_server_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AlterForeignServerStmt *
       pg_query__alter_foreign_server_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AlterForeignServerStmt *)
     protobuf_c_message_unpack (&pg_query__alter_foreign_server_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__alter_foreign_server_stmt__free_unpacked
                     (PgQuery__AlterForeignServerStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__alter_foreign_server_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__create_foreign_table_stmt__init
                     (PgQuery__CreateForeignTableStmt         *message)
{
  static const PgQuery__CreateForeignTableStmt init_value = PG_QUERY__CREATE_FOREIGN_TABLE_STMT__INIT;
  *message = init_value;
}
size_t pg_query__create_foreign_table_stmt__get_packed_size
                     (const PgQuery__CreateForeignTableStmt *message)
{
  assert(message->base.descriptor == &pg_query__create_foreign_table_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__create_foreign_table_stmt__pack
                     (const PgQuery__CreateForeignTableStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__create_foreign_table_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__create_foreign_table_stmt__pack_to_buffer
                     (const PgQuery__CreateForeignTableStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__create_foreign_table_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CreateForeignTableStmt *
       pg_query__create_foreign_table_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CreateForeignTableStmt *)
     protobuf_c_message_unpack (&pg_query__create_foreign_table_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__create_foreign_table_stmt__free_unpacked
                     (PgQuery__CreateForeignTableStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__create_foreign_table_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__create_user_mapping_stmt__init
                     (PgQuery__CreateUserMappingStmt         *message)
{
  static const PgQuery__CreateUserMappingStmt init_value = PG_QUERY__CREATE_USER_MAPPING_STMT__INIT;
  *message = init_value;
}
size_t pg_query__create_user_mapping_stmt__get_packed_size
                     (const PgQuery__CreateUserMappingStmt *message)
{
  assert(message->base.descriptor == &pg_query__create_user_mapping_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__create_user_mapping_stmt__pack
                     (const PgQuery__CreateUserMappingStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__create_user_mapping_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__create_user_mapping_stmt__pack_to_buffer
                     (const PgQuery__CreateUserMappingStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__create_user_mapping_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CreateUserMappingStmt *
       pg_query__create_user_mapping_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CreateUserMappingStmt *)
     protobuf_c_message_unpack (&pg_query__create_user_mapping_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__create_user_mapping_stmt__free_unpacked
                     (PgQuery__CreateUserMappingStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__create_user_mapping_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__alter_user_mapping_stmt__init
                     (PgQuery__AlterUserMappingStmt         *message)
{
  static const PgQuery__AlterUserMappingStmt init_value = PG_QUERY__ALTER_USER_MAPPING_STMT__INIT;
  *message = init_value;
}
size_t pg_query__alter_user_mapping_stmt__get_packed_size
                     (const PgQuery__AlterUserMappingStmt *message)
{
  assert(message->base.descriptor == &pg_query__alter_user_mapping_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__alter_user_mapping_stmt__pack
                     (const PgQuery__AlterUserMappingStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__alter_user_mapping_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__alter_user_mapping_stmt__pack_to_buffer
                     (const PgQuery__AlterUserMappingStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__alter_user_mapping_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AlterUserMappingStmt *
       pg_query__alter_user_mapping_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AlterUserMappingStmt *)
     protobuf_c_message_unpack (&pg_query__alter_user_mapping_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__alter_user_mapping_stmt__free_unpacked
                     (PgQuery__AlterUserMappingStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__alter_user_mapping_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__drop_user_mapping_stmt__init
                     (PgQuery__DropUserMappingStmt         *message)
{
  static const PgQuery__DropUserMappingStmt init_value = PG_QUERY__DROP_USER_MAPPING_STMT__INIT;
  *message = init_value;
}
size_t pg_query__drop_user_mapping_stmt__get_packed_size
                     (const PgQuery__DropUserMappingStmt *message)
{
  assert(message->base.descriptor == &pg_query__drop_user_mapping_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__drop_user_mapping_stmt__pack
                     (const PgQuery__DropUserMappingStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__drop_user_mapping_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__drop_user_mapping_stmt__pack_to_buffer
                     (const PgQuery__DropUserMappingStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__drop_user_mapping_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__DropUserMappingStmt *
       pg_query__drop_user_mapping_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__DropUserMappingStmt *)
     protobuf_c_message_unpack (&pg_query__drop_user_mapping_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__drop_user_mapping_stmt__free_unpacked
                     (PgQuery__DropUserMappingStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__drop_user_mapping_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__import_foreign_schema_stmt__init
                     (PgQuery__ImportForeignSchemaStmt         *message)
{
  static const PgQuery__ImportForeignSchemaStmt init_value = PG_QUERY__IMPORT_FOREIGN_SCHEMA_STMT__INIT;
  *message = init_value;
}
size_t pg_query__import_foreign_schema_stmt__get_packed_size
                     (const PgQuery__ImportForeignSchemaStmt *message)
{
  assert(message->base.descriptor == &pg_query__import_foreign_schema_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__import_foreign_schema_stmt__pack
                     (const PgQuery__ImportForeignSchemaStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__import_foreign_schema_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__import_foreign_schema_stmt__pack_to_buffer
                     (const PgQuery__ImportForeignSchemaStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__import_foreign_schema_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__ImportForeignSchemaStmt *
       pg_query__import_foreign_schema_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__ImportForeignSchemaStmt *)
     protobuf_c_message_unpack (&pg_query__import_foreign_schema_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__import_foreign_schema_stmt__free_unpacked
                     (PgQuery__ImportForeignSchemaStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__import_foreign_schema_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__create_policy_stmt__init
                     (PgQuery__CreatePolicyStmt         *message)
{
  static const PgQuery__CreatePolicyStmt init_value = PG_QUERY__CREATE_POLICY_STMT__INIT;
  *message = init_value;
}
size_t pg_query__create_policy_stmt__get_packed_size
                     (const PgQuery__CreatePolicyStmt *message)
{
  assert(message->base.descriptor == &pg_query__create_policy_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__create_policy_stmt__pack
                     (const PgQuery__CreatePolicyStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__create_policy_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__create_policy_stmt__pack_to_buffer
                     (const PgQuery__CreatePolicyStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__create_policy_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CreatePolicyStmt *
       pg_query__create_policy_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CreatePolicyStmt *)
     protobuf_c_message_unpack (&pg_query__create_policy_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__create_policy_stmt__free_unpacked
                     (PgQuery__CreatePolicyStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__create_policy_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__alter_policy_stmt__init
                     (PgQuery__AlterPolicyStmt         *message)
{
  static const PgQuery__AlterPolicyStmt init_value = PG_QUERY__ALTER_POLICY_STMT__INIT;
  *message = init_value;
}
size_t pg_query__alter_policy_stmt__get_packed_size
                     (const PgQuery__AlterPolicyStmt *message)
{
  assert(message->base.descriptor == &pg_query__alter_policy_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__alter_policy_stmt__pack
                     (const PgQuery__AlterPolicyStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__alter_policy_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__alter_policy_stmt__pack_to_buffer
                     (const PgQuery__AlterPolicyStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__alter_policy_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AlterPolicyStmt *
       pg_query__alter_policy_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AlterPolicyStmt *)
     protobuf_c_message_unpack (&pg_query__alter_policy_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__alter_policy_stmt__free_unpacked
                     (PgQuery__AlterPolicyStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__alter_policy_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__create_am_stmt__init
                     (PgQuery__CreateAmStmt         *message)
{
  static const PgQuery__CreateAmStmt init_value = PG_QUERY__CREATE_AM_STMT__INIT;
  *message = init_value;
}
size_t pg_query__create_am_stmt__get_packed_size
                     (const PgQuery__CreateAmStmt *message)
{
  assert(message->base.descriptor == &pg_query__create_am_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__create_am_stmt__pack
                     (const PgQuery__CreateAmStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__create_am_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__create_am_stmt__pack_to_buffer
                     (const PgQuery__CreateAmStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__create_am_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CreateAmStmt *
       pg_query__create_am_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CreateAmStmt *)
     protobuf_c_message_unpack (&pg_query__create_am_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__create_am_stmt__free_unpacked
                     (PgQuery__CreateAmStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__create_am_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__create_trig_stmt__init
                     (PgQuery__CreateTrigStmt         *message)
{
  static const PgQuery__CreateTrigStmt init_value = PG_QUERY__CREATE_TRIG_STMT__INIT;
  *message = init_value;
}
size_t pg_query__create_trig_stmt__get_packed_size
                     (const PgQuery__CreateTrigStmt *message)
{
  assert(message->base.descriptor == &pg_query__create_trig_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__create_trig_stmt__pack
                     (const PgQuery__CreateTrigStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__create_trig_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__create_trig_stmt__pack_to_buffer
                     (const PgQuery__CreateTrigStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__create_trig_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CreateTrigStmt *
       pg_query__create_trig_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CreateTrigStmt *)
     protobuf_c_message_unpack (&pg_query__create_trig_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__create_trig_stmt__free_unpacked
                     (PgQuery__CreateTrigStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__create_trig_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__create_event_trig_stmt__init
                     (PgQuery__CreateEventTrigStmt         *message)
{
  static const PgQuery__CreateEventTrigStmt init_value = PG_QUERY__CREATE_EVENT_TRIG_STMT__INIT;
  *message = init_value;
}
size_t pg_query__create_event_trig_stmt__get_packed_size
                     (const PgQuery__CreateEventTrigStmt *message)
{
  assert(message->base.descriptor == &pg_query__create_event_trig_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__create_event_trig_stmt__pack
                     (const PgQuery__CreateEventTrigStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__create_event_trig_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__create_event_trig_stmt__pack_to_buffer
                     (const PgQuery__CreateEventTrigStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__create_event_trig_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CreateEventTrigStmt *
       pg_query__create_event_trig_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CreateEventTrigStmt *)
     protobuf_c_message_unpack (&pg_query__create_event_trig_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__create_event_trig_stmt__free_unpacked
                     (PgQuery__CreateEventTrigStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__create_event_trig_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__alter_event_trig_stmt__init
                     (PgQuery__AlterEventTrigStmt         *message)
{
  static const PgQuery__AlterEventTrigStmt init_value = PG_QUERY__ALTER_EVENT_TRIG_STMT__INIT;
  *message = init_value;
}
size_t pg_query__alter_event_trig_stmt__get_packed_size
                     (const PgQuery__AlterEventTrigStmt *message)
{
  assert(message->base.descriptor == &pg_query__alter_event_trig_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__alter_event_trig_stmt__pack
                     (const PgQuery__AlterEventTrigStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__alter_event_trig_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__alter_event_trig_stmt__pack_to_buffer
                     (const PgQuery__AlterEventTrigStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__alter_event_trig_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AlterEventTrigStmt *
       pg_query__alter_event_trig_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AlterEventTrigStmt *)
     protobuf_c_message_unpack (&pg_query__alter_event_trig_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__alter_event_trig_stmt__free_unpacked
                     (PgQuery__AlterEventTrigStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__alter_event_trig_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__create_plang_stmt__init
                     (PgQuery__CreatePLangStmt         *message)
{
  static const PgQuery__CreatePLangStmt init_value = PG_QUERY__CREATE_PLANG_STMT__INIT;
  *message = init_value;
}
size_t pg_query__create_plang_stmt__get_packed_size
                     (const PgQuery__CreatePLangStmt *message)
{
  assert(message->base.descriptor == &pg_query__create_plang_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__create_plang_stmt__pack
                     (const PgQuery__CreatePLangStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__create_plang_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__create_plang_stmt__pack_to_buffer
                     (const PgQuery__CreatePLangStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__create_plang_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CreatePLangStmt *
       pg_query__create_plang_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CreatePLangStmt *)
     protobuf_c_message_unpack (&pg_query__create_plang_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__create_plang_stmt__free_unpacked
                     (PgQuery__CreatePLangStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__create_plang_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__create_role_stmt__init
                     (PgQuery__CreateRoleStmt         *message)
{
  static const PgQuery__CreateRoleStmt init_value = PG_QUERY__CREATE_ROLE_STMT__INIT;
  *message = init_value;
}
size_t pg_query__create_role_stmt__get_packed_size
                     (const PgQuery__CreateRoleStmt *message)
{
  assert(message->base.descriptor == &pg_query__create_role_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__create_role_stmt__pack
                     (const PgQuery__CreateRoleStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__create_role_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__create_role_stmt__pack_to_buffer
                     (const PgQuery__CreateRoleStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__create_role_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CreateRoleStmt *
       pg_query__create_role_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CreateRoleStmt *)
     protobuf_c_message_unpack (&pg_query__create_role_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__create_role_stmt__free_unpacked
                     (PgQuery__CreateRoleStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__create_role_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__alter_role_stmt__init
                     (PgQuery__AlterRoleStmt         *message)
{
  static const PgQuery__AlterRoleStmt init_value = PG_QUERY__ALTER_ROLE_STMT__INIT;
  *message = init_value;
}
size_t pg_query__alter_role_stmt__get_packed_size
                     (const PgQuery__AlterRoleStmt *message)
{
  assert(message->base.descriptor == &pg_query__alter_role_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__alter_role_stmt__pack
                     (const PgQuery__AlterRoleStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__alter_role_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__alter_role_stmt__pack_to_buffer
                     (const PgQuery__AlterRoleStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__alter_role_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AlterRoleStmt *
       pg_query__alter_role_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AlterRoleStmt *)
     protobuf_c_message_unpack (&pg_query__alter_role_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__alter_role_stmt__free_unpacked
                     (PgQuery__AlterRoleStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__alter_role_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__alter_role_set_stmt__init
                     (PgQuery__AlterRoleSetStmt         *message)
{
  static const PgQuery__AlterRoleSetStmt init_value = PG_QUERY__ALTER_ROLE_SET_STMT__INIT;
  *message = init_value;
}
size_t pg_query__alter_role_set_stmt__get_packed_size
                     (const PgQuery__AlterRoleSetStmt *message)
{
  assert(message->base.descriptor == &pg_query__alter_role_set_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__alter_role_set_stmt__pack
                     (const PgQuery__AlterRoleSetStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__alter_role_set_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__alter_role_set_stmt__pack_to_buffer
                     (const PgQuery__AlterRoleSetStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__alter_role_set_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AlterRoleSetStmt *
       pg_query__alter_role_set_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AlterRoleSetStmt *)
     protobuf_c_message_unpack (&pg_query__alter_role_set_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__alter_role_set_stmt__free_unpacked
                     (PgQuery__AlterRoleSetStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__alter_role_set_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__drop_role_stmt__init
                     (PgQuery__DropRoleStmt         *message)
{
  static const PgQuery__DropRoleStmt init_value = PG_QUERY__DROP_ROLE_STMT__INIT;
  *message = init_value;
}
size_t pg_query__drop_role_stmt__get_packed_size
                     (const PgQuery__DropRoleStmt *message)
{
  assert(message->base.descriptor == &pg_query__drop_role_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__drop_role_stmt__pack
                     (const PgQuery__DropRoleStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__drop_role_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__drop_role_stmt__pack_to_buffer
                     (const PgQuery__DropRoleStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__drop_role_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__DropRoleStmt *
       pg_query__drop_role_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__DropRoleStmt *)
     protobuf_c_message_unpack (&pg_query__drop_role_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__drop_role_stmt__free_unpacked
                     (PgQuery__DropRoleStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__drop_role_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__create_seq_stmt__init
                     (PgQuery__CreateSeqStmt         *message)
{
  static const PgQuery__CreateSeqStmt init_value = PG_QUERY__CREATE_SEQ_STMT__INIT;
  *message = init_value;
}
size_t pg_query__create_seq_stmt__get_packed_size
                     (const PgQuery__CreateSeqStmt *message)
{
  assert(message->base.descriptor == &pg_query__create_seq_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__create_seq_stmt__pack
                     (const PgQuery__CreateSeqStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__create_seq_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__create_seq_stmt__pack_to_buffer
                     (const PgQuery__CreateSeqStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__create_seq_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CreateSeqStmt *
       pg_query__create_seq_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CreateSeqStmt *)
     protobuf_c_message_unpack (&pg_query__create_seq_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__create_seq_stmt__free_unpacked
                     (PgQuery__CreateSeqStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__create_seq_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__alter_seq_stmt__init
                     (PgQuery__AlterSeqStmt         *message)
{
  static const PgQuery__AlterSeqStmt init_value = PG_QUERY__ALTER_SEQ_STMT__INIT;
  *message = init_value;
}
size_t pg_query__alter_seq_stmt__get_packed_size
                     (const PgQuery__AlterSeqStmt *message)
{
  assert(message->base.descriptor == &pg_query__alter_seq_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__alter_seq_stmt__pack
                     (const PgQuery__AlterSeqStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__alter_seq_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__alter_seq_stmt__pack_to_buffer
                     (const PgQuery__AlterSeqStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__alter_seq_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AlterSeqStmt *
       pg_query__alter_seq_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AlterSeqStmt *)
     protobuf_c_message_unpack (&pg_query__alter_seq_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__alter_seq_stmt__free_unpacked
                     (PgQuery__AlterSeqStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__alter_seq_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__define_stmt__init
                     (PgQuery__DefineStmt         *message)
{
  static const PgQuery__DefineStmt init_value = PG_QUERY__DEFINE_STMT__INIT;
  *message = init_value;
}
size_t pg_query__define_stmt__get_packed_size
                     (const PgQuery__DefineStmt *message)
{
  assert(message->base.descriptor == &pg_query__define_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__define_stmt__pack
                     (const PgQuery__DefineStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__define_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__define_stmt__pack_to_buffer
                     (const PgQuery__DefineStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__define_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__DefineStmt *
       pg_query__define_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__DefineStmt *)
     protobuf_c_message_unpack (&pg_query__define_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__define_stmt__free_unpacked
                     (PgQuery__DefineStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__define_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__create_domain_stmt__init
                     (PgQuery__CreateDomainStmt         *message)
{
  static const PgQuery__CreateDomainStmt init_value = PG_QUERY__CREATE_DOMAIN_STMT__INIT;
  *message = init_value;
}
size_t pg_query__create_domain_stmt__get_packed_size
                     (const PgQuery__CreateDomainStmt *message)
{
  assert(message->base.descriptor == &pg_query__create_domain_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__create_domain_stmt__pack
                     (const PgQuery__CreateDomainStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__create_domain_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__create_domain_stmt__pack_to_buffer
                     (const PgQuery__CreateDomainStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__create_domain_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CreateDomainStmt *
       pg_query__create_domain_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CreateDomainStmt *)
     protobuf_c_message_unpack (&pg_query__create_domain_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__create_domain_stmt__free_unpacked
                     (PgQuery__CreateDomainStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__create_domain_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__create_op_class_stmt__init
                     (PgQuery__CreateOpClassStmt         *message)
{
  static const PgQuery__CreateOpClassStmt init_value = PG_QUERY__CREATE_OP_CLASS_STMT__INIT;
  *message = init_value;
}
size_t pg_query__create_op_class_stmt__get_packed_size
                     (const PgQuery__CreateOpClassStmt *message)
{
  assert(message->base.descriptor == &pg_query__create_op_class_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__create_op_class_stmt__pack
                     (const PgQuery__CreateOpClassStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__create_op_class_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__create_op_class_stmt__pack_to_buffer
                     (const PgQuery__CreateOpClassStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__create_op_class_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CreateOpClassStmt *
       pg_query__create_op_class_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CreateOpClassStmt *)
     protobuf_c_message_unpack (&pg_query__create_op_class_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__create_op_class_stmt__free_unpacked
                     (PgQuery__CreateOpClassStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__create_op_class_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__create_op_class_item__init
                     (PgQuery__CreateOpClassItem         *message)
{
  static const PgQuery__CreateOpClassItem init_value = PG_QUERY__CREATE_OP_CLASS_ITEM__INIT;
  *message = init_value;
}
size_t pg_query__create_op_class_item__get_packed_size
                     (const PgQuery__CreateOpClassItem *message)
{
  assert(message->base.descriptor == &pg_query__create_op_class_item__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__create_op_class_item__pack
                     (const PgQuery__CreateOpClassItem *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__create_op_class_item__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__create_op_class_item__pack_to_buffer
                     (const PgQuery__CreateOpClassItem *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__create_op_class_item__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CreateOpClassItem *
       pg_query__create_op_class_item__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CreateOpClassItem *)
     protobuf_c_message_unpack (&pg_query__create_op_class_item__descriptor,
                                allocator, len, data);
}
void   pg_query__create_op_class_item__free_unpacked
                     (PgQuery__CreateOpClassItem *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__create_op_class_item__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__create_op_family_stmt__init
                     (PgQuery__CreateOpFamilyStmt         *message)
{
  static const PgQuery__CreateOpFamilyStmt init_value = PG_QUERY__CREATE_OP_FAMILY_STMT__INIT;
  *message = init_value;
}
size_t pg_query__create_op_family_stmt__get_packed_size
                     (const PgQuery__CreateOpFamilyStmt *message)
{
  assert(message->base.descriptor == &pg_query__create_op_family_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__create_op_family_stmt__pack
                     (const PgQuery__CreateOpFamilyStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__create_op_family_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__create_op_family_stmt__pack_to_buffer
                     (const PgQuery__CreateOpFamilyStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__create_op_family_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CreateOpFamilyStmt *
       pg_query__create_op_family_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CreateOpFamilyStmt *)
     protobuf_c_message_unpack (&pg_query__create_op_family_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__create_op_family_stmt__free_unpacked
                     (PgQuery__CreateOpFamilyStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__create_op_family_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__alter_op_family_stmt__init
                     (PgQuery__AlterOpFamilyStmt         *message)
{
  static const PgQuery__AlterOpFamilyStmt init_value = PG_QUERY__ALTER_OP_FAMILY_STMT__INIT;
  *message = init_value;
}
size_t pg_query__alter_op_family_stmt__get_packed_size
                     (const PgQuery__AlterOpFamilyStmt *message)
{
  assert(message->base.descriptor == &pg_query__alter_op_family_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__alter_op_family_stmt__pack
                     (const PgQuery__AlterOpFamilyStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__alter_op_family_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__alter_op_family_stmt__pack_to_buffer
                     (const PgQuery__AlterOpFamilyStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__alter_op_family_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AlterOpFamilyStmt *
       pg_query__alter_op_family_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AlterOpFamilyStmt *)
     protobuf_c_message_unpack (&pg_query__alter_op_family_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__alter_op_family_stmt__free_unpacked
                     (PgQuery__AlterOpFamilyStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__alter_op_family_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__drop_stmt__init
                     (PgQuery__DropStmt         *message)
{
  static const PgQuery__DropStmt init_value = PG_QUERY__DROP_STMT__INIT;
  *message = init_value;
}
size_t pg_query__drop_stmt__get_packed_size
                     (const PgQuery__DropStmt *message)
{
  assert(message->base.descriptor == &pg_query__drop_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__drop_stmt__pack
                     (const PgQuery__DropStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__drop_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__drop_stmt__pack_to_buffer
                     (const PgQuery__DropStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__drop_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__DropStmt *
       pg_query__drop_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__DropStmt *)
     protobuf_c_message_unpack (&pg_query__drop_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__drop_stmt__free_unpacked
                     (PgQuery__DropStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__drop_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__truncate_stmt__init
                     (PgQuery__TruncateStmt         *message)
{
  static const PgQuery__TruncateStmt init_value = PG_QUERY__TRUNCATE_STMT__INIT;
  *message = init_value;
}
size_t pg_query__truncate_stmt__get_packed_size
                     (const PgQuery__TruncateStmt *message)
{
  assert(message->base.descriptor == &pg_query__truncate_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__truncate_stmt__pack
                     (const PgQuery__TruncateStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__truncate_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__truncate_stmt__pack_to_buffer
                     (const PgQuery__TruncateStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__truncate_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__TruncateStmt *
       pg_query__truncate_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__TruncateStmt *)
     protobuf_c_message_unpack (&pg_query__truncate_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__truncate_stmt__free_unpacked
                     (PgQuery__TruncateStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__truncate_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__comment_stmt__init
                     (PgQuery__CommentStmt         *message)
{
  static const PgQuery__CommentStmt init_value = PG_QUERY__COMMENT_STMT__INIT;
  *message = init_value;
}
size_t pg_query__comment_stmt__get_packed_size
                     (const PgQuery__CommentStmt *message)
{
  assert(message->base.descriptor == &pg_query__comment_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__comment_stmt__pack
                     (const PgQuery__CommentStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__comment_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__comment_stmt__pack_to_buffer
                     (const PgQuery__CommentStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__comment_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CommentStmt *
       pg_query__comment_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CommentStmt *)
     protobuf_c_message_unpack (&pg_query__comment_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__comment_stmt__free_unpacked
                     (PgQuery__CommentStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__comment_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__sec_label_stmt__init
                     (PgQuery__SecLabelStmt         *message)
{
  static const PgQuery__SecLabelStmt init_value = PG_QUERY__SEC_LABEL_STMT__INIT;
  *message = init_value;
}
size_t pg_query__sec_label_stmt__get_packed_size
                     (const PgQuery__SecLabelStmt *message)
{
  assert(message->base.descriptor == &pg_query__sec_label_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__sec_label_stmt__pack
                     (const PgQuery__SecLabelStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__sec_label_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__sec_label_stmt__pack_to_buffer
                     (const PgQuery__SecLabelStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__sec_label_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__SecLabelStmt *
       pg_query__sec_label_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__SecLabelStmt *)
     protobuf_c_message_unpack (&pg_query__sec_label_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__sec_label_stmt__free_unpacked
                     (PgQuery__SecLabelStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__sec_label_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__declare_cursor_stmt__init
                     (PgQuery__DeclareCursorStmt         *message)
{
  static const PgQuery__DeclareCursorStmt init_value = PG_QUERY__DECLARE_CURSOR_STMT__INIT;
  *message = init_value;
}
size_t pg_query__declare_cursor_stmt__get_packed_size
                     (const PgQuery__DeclareCursorStmt *message)
{
  assert(message->base.descriptor == &pg_query__declare_cursor_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__declare_cursor_stmt__pack
                     (const PgQuery__DeclareCursorStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__declare_cursor_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__declare_cursor_stmt__pack_to_buffer
                     (const PgQuery__DeclareCursorStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__declare_cursor_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__DeclareCursorStmt *
       pg_query__declare_cursor_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__DeclareCursorStmt *)
     protobuf_c_message_unpack (&pg_query__declare_cursor_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__declare_cursor_stmt__free_unpacked
                     (PgQuery__DeclareCursorStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__declare_cursor_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__close_portal_stmt__init
                     (PgQuery__ClosePortalStmt         *message)
{
  static const PgQuery__ClosePortalStmt init_value = PG_QUERY__CLOSE_PORTAL_STMT__INIT;
  *message = init_value;
}
size_t pg_query__close_portal_stmt__get_packed_size
                     (const PgQuery__ClosePortalStmt *message)
{
  assert(message->base.descriptor == &pg_query__close_portal_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__close_portal_stmt__pack
                     (const PgQuery__ClosePortalStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__close_portal_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__close_portal_stmt__pack_to_buffer
                     (const PgQuery__ClosePortalStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__close_portal_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__ClosePortalStmt *
       pg_query__close_portal_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__ClosePortalStmt *)
     protobuf_c_message_unpack (&pg_query__close_portal_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__close_portal_stmt__free_unpacked
                     (PgQuery__ClosePortalStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__close_portal_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__fetch_stmt__init
                     (PgQuery__FetchStmt         *message)
{
  static const PgQuery__FetchStmt init_value = PG_QUERY__FETCH_STMT__INIT;
  *message = init_value;
}
size_t pg_query__fetch_stmt__get_packed_size
                     (const PgQuery__FetchStmt *message)
{
  assert(message->base.descriptor == &pg_query__fetch_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__fetch_stmt__pack
                     (const PgQuery__FetchStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__fetch_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__fetch_stmt__pack_to_buffer
                     (const PgQuery__FetchStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__fetch_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__FetchStmt *
       pg_query__fetch_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__FetchStmt *)
     protobuf_c_message_unpack (&pg_query__fetch_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__fetch_stmt__free_unpacked
                     (PgQuery__FetchStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__fetch_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__index_stmt__init
                     (PgQuery__IndexStmt         *message)
{
  static const PgQuery__IndexStmt init_value = PG_QUERY__INDEX_STMT__INIT;
  *message = init_value;
}
size_t pg_query__index_stmt__get_packed_size
                     (const PgQuery__IndexStmt *message)
{
  assert(message->base.descriptor == &pg_query__index_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__index_stmt__pack
                     (const PgQuery__IndexStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__index_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__index_stmt__pack_to_buffer
                     (const PgQuery__IndexStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__index_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__IndexStmt *
       pg_query__index_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__IndexStmt *)
     protobuf_c_message_unpack (&pg_query__index_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__index_stmt__free_unpacked
                     (PgQuery__IndexStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__index_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__create_stats_stmt__init
                     (PgQuery__CreateStatsStmt         *message)
{
  static const PgQuery__CreateStatsStmt init_value = PG_QUERY__CREATE_STATS_STMT__INIT;
  *message = init_value;
}
size_t pg_query__create_stats_stmt__get_packed_size
                     (const PgQuery__CreateStatsStmt *message)
{
  assert(message->base.descriptor == &pg_query__create_stats_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__create_stats_stmt__pack
                     (const PgQuery__CreateStatsStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__create_stats_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__create_stats_stmt__pack_to_buffer
                     (const PgQuery__CreateStatsStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__create_stats_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CreateStatsStmt *
       pg_query__create_stats_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CreateStatsStmt *)
     protobuf_c_message_unpack (&pg_query__create_stats_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__create_stats_stmt__free_unpacked
                     (PgQuery__CreateStatsStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__create_stats_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__stats_elem__init
                     (PgQuery__StatsElem         *message)
{
  static const PgQuery__StatsElem init_value = PG_QUERY__STATS_ELEM__INIT;
  *message = init_value;
}
size_t pg_query__stats_elem__get_packed_size
                     (const PgQuery__StatsElem *message)
{
  assert(message->base.descriptor == &pg_query__stats_elem__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__stats_elem__pack
                     (const PgQuery__StatsElem *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__stats_elem__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__stats_elem__pack_to_buffer
                     (const PgQuery__StatsElem *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__stats_elem__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__StatsElem *
       pg_query__stats_elem__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__StatsElem *)
     protobuf_c_message_unpack (&pg_query__stats_elem__descriptor,
                                allocator, len, data);
}
void   pg_query__stats_elem__free_unpacked
                     (PgQuery__StatsElem *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__stats_elem__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__alter_stats_stmt__init
                     (PgQuery__AlterStatsStmt         *message)
{
  static const PgQuery__AlterStatsStmt init_value = PG_QUERY__ALTER_STATS_STMT__INIT;
  *message = init_value;
}
size_t pg_query__alter_stats_stmt__get_packed_size
                     (const PgQuery__AlterStatsStmt *message)
{
  assert(message->base.descriptor == &pg_query__alter_stats_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__alter_stats_stmt__pack
                     (const PgQuery__AlterStatsStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__alter_stats_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__alter_stats_stmt__pack_to_buffer
                     (const PgQuery__AlterStatsStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__alter_stats_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AlterStatsStmt *
       pg_query__alter_stats_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AlterStatsStmt *)
     protobuf_c_message_unpack (&pg_query__alter_stats_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__alter_stats_stmt__free_unpacked
                     (PgQuery__AlterStatsStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__alter_stats_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__create_function_stmt__init
                     (PgQuery__CreateFunctionStmt         *message)
{
  static const PgQuery__CreateFunctionStmt init_value = PG_QUERY__CREATE_FUNCTION_STMT__INIT;
  *message = init_value;
}
size_t pg_query__create_function_stmt__get_packed_size
                     (const PgQuery__CreateFunctionStmt *message)
{
  assert(message->base.descriptor == &pg_query__create_function_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__create_function_stmt__pack
                     (const PgQuery__CreateFunctionStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__create_function_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__create_function_stmt__pack_to_buffer
                     (const PgQuery__CreateFunctionStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__create_function_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CreateFunctionStmt *
       pg_query__create_function_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CreateFunctionStmt *)
     protobuf_c_message_unpack (&pg_query__create_function_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__create_function_stmt__free_unpacked
                     (PgQuery__CreateFunctionStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__create_function_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__function_parameter__init
                     (PgQuery__FunctionParameter         *message)
{
  static const PgQuery__FunctionParameter init_value = PG_QUERY__FUNCTION_PARAMETER__INIT;
  *message = init_value;
}
size_t pg_query__function_parameter__get_packed_size
                     (const PgQuery__FunctionParameter *message)
{
  assert(message->base.descriptor == &pg_query__function_parameter__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__function_parameter__pack
                     (const PgQuery__FunctionParameter *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__function_parameter__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__function_parameter__pack_to_buffer
                     (const PgQuery__FunctionParameter *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__function_parameter__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__FunctionParameter *
       pg_query__function_parameter__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__FunctionParameter *)
     protobuf_c_message_unpack (&pg_query__function_parameter__descriptor,
                                allocator, len, data);
}
void   pg_query__function_parameter__free_unpacked
                     (PgQuery__FunctionParameter *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__function_parameter__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__alter_function_stmt__init
                     (PgQuery__AlterFunctionStmt         *message)
{
  static const PgQuery__AlterFunctionStmt init_value = PG_QUERY__ALTER_FUNCTION_STMT__INIT;
  *message = init_value;
}
size_t pg_query__alter_function_stmt__get_packed_size
                     (const PgQuery__AlterFunctionStmt *message)
{
  assert(message->base.descriptor == &pg_query__alter_function_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__alter_function_stmt__pack
                     (const PgQuery__AlterFunctionStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__alter_function_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__alter_function_stmt__pack_to_buffer
                     (const PgQuery__AlterFunctionStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__alter_function_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AlterFunctionStmt *
       pg_query__alter_function_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AlterFunctionStmt *)
     protobuf_c_message_unpack (&pg_query__alter_function_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__alter_function_stmt__free_unpacked
                     (PgQuery__AlterFunctionStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__alter_function_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__do_stmt__init
                     (PgQuery__DoStmt         *message)
{
  static const PgQuery__DoStmt init_value = PG_QUERY__DO_STMT__INIT;
  *message = init_value;
}
size_t pg_query__do_stmt__get_packed_size
                     (const PgQuery__DoStmt *message)
{
  assert(message->base.descriptor == &pg_query__do_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__do_stmt__pack
                     (const PgQuery__DoStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__do_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__do_stmt__pack_to_buffer
                     (const PgQuery__DoStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__do_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__DoStmt *
       pg_query__do_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__DoStmt *)
     protobuf_c_message_unpack (&pg_query__do_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__do_stmt__free_unpacked
                     (PgQuery__DoStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__do_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__inline_code_block__init
                     (PgQuery__InlineCodeBlock         *message)
{
  static const PgQuery__InlineCodeBlock init_value = PG_QUERY__INLINE_CODE_BLOCK__INIT;
  *message = init_value;
}
size_t pg_query__inline_code_block__get_packed_size
                     (const PgQuery__InlineCodeBlock *message)
{
  assert(message->base.descriptor == &pg_query__inline_code_block__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__inline_code_block__pack
                     (const PgQuery__InlineCodeBlock *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__inline_code_block__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__inline_code_block__pack_to_buffer
                     (const PgQuery__InlineCodeBlock *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__inline_code_block__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__InlineCodeBlock *
       pg_query__inline_code_block__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__InlineCodeBlock *)
     protobuf_c_message_unpack (&pg_query__inline_code_block__descriptor,
                                allocator, len, data);
}
void   pg_query__inline_code_block__free_unpacked
                     (PgQuery__InlineCodeBlock *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__inline_code_block__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__call_stmt__init
                     (PgQuery__CallStmt         *message)
{
  static const PgQuery__CallStmt init_value = PG_QUERY__CALL_STMT__INIT;
  *message = init_value;
}
size_t pg_query__call_stmt__get_packed_size
                     (const PgQuery__CallStmt *message)
{
  assert(message->base.descriptor == &pg_query__call_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__call_stmt__pack
                     (const PgQuery__CallStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__call_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__call_stmt__pack_to_buffer
                     (const PgQuery__CallStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__call_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CallStmt *
       pg_query__call_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CallStmt *)
     protobuf_c_message_unpack (&pg_query__call_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__call_stmt__free_unpacked
                     (PgQuery__CallStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__call_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__call_context__init
                     (PgQuery__CallContext         *message)
{
  static const PgQuery__CallContext init_value = PG_QUERY__CALL_CONTEXT__INIT;
  *message = init_value;
}
size_t pg_query__call_context__get_packed_size
                     (const PgQuery__CallContext *message)
{
  assert(message->base.descriptor == &pg_query__call_context__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__call_context__pack
                     (const PgQuery__CallContext *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__call_context__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__call_context__pack_to_buffer
                     (const PgQuery__CallContext *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__call_context__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CallContext *
       pg_query__call_context__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CallContext *)
     protobuf_c_message_unpack (&pg_query__call_context__descriptor,
                                allocator, len, data);
}
void   pg_query__call_context__free_unpacked
                     (PgQuery__CallContext *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__call_context__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__rename_stmt__init
                     (PgQuery__RenameStmt         *message)
{
  static const PgQuery__RenameStmt init_value = PG_QUERY__RENAME_STMT__INIT;
  *message = init_value;
}
size_t pg_query__rename_stmt__get_packed_size
                     (const PgQuery__RenameStmt *message)
{
  assert(message->base.descriptor == &pg_query__rename_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__rename_stmt__pack
                     (const PgQuery__RenameStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__rename_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__rename_stmt__pack_to_buffer
                     (const PgQuery__RenameStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__rename_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__RenameStmt *
       pg_query__rename_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__RenameStmt *)
     protobuf_c_message_unpack (&pg_query__rename_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__rename_stmt__free_unpacked
                     (PgQuery__RenameStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__rename_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__alter_object_depends_stmt__init
                     (PgQuery__AlterObjectDependsStmt         *message)
{
  static const PgQuery__AlterObjectDependsStmt init_value = PG_QUERY__ALTER_OBJECT_DEPENDS_STMT__INIT;
  *message = init_value;
}
size_t pg_query__alter_object_depends_stmt__get_packed_size
                     (const PgQuery__AlterObjectDependsStmt *message)
{
  assert(message->base.descriptor == &pg_query__alter_object_depends_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__alter_object_depends_stmt__pack
                     (const PgQuery__AlterObjectDependsStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__alter_object_depends_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__alter_object_depends_stmt__pack_to_buffer
                     (const PgQuery__AlterObjectDependsStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__alter_object_depends_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AlterObjectDependsStmt *
       pg_query__alter_object_depends_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AlterObjectDependsStmt *)
     protobuf_c_message_unpack (&pg_query__alter_object_depends_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__alter_object_depends_stmt__free_unpacked
                     (PgQuery__AlterObjectDependsStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__alter_object_depends_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__alter_object_schema_stmt__init
                     (PgQuery__AlterObjectSchemaStmt         *message)
{
  static const PgQuery__AlterObjectSchemaStmt init_value = PG_QUERY__ALTER_OBJECT_SCHEMA_STMT__INIT;
  *message = init_value;
}
size_t pg_query__alter_object_schema_stmt__get_packed_size
                     (const PgQuery__AlterObjectSchemaStmt *message)
{
  assert(message->base.descriptor == &pg_query__alter_object_schema_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__alter_object_schema_stmt__pack
                     (const PgQuery__AlterObjectSchemaStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__alter_object_schema_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__alter_object_schema_stmt__pack_to_buffer
                     (const PgQuery__AlterObjectSchemaStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__alter_object_schema_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AlterObjectSchemaStmt *
       pg_query__alter_object_schema_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AlterObjectSchemaStmt *)
     protobuf_c_message_unpack (&pg_query__alter_object_schema_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__alter_object_schema_stmt__free_unpacked
                     (PgQuery__AlterObjectSchemaStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__alter_object_schema_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__alter_owner_stmt__init
                     (PgQuery__AlterOwnerStmt         *message)
{
  static const PgQuery__AlterOwnerStmt init_value = PG_QUERY__ALTER_OWNER_STMT__INIT;
  *message = init_value;
}
size_t pg_query__alter_owner_stmt__get_packed_size
                     (const PgQuery__AlterOwnerStmt *message)
{
  assert(message->base.descriptor == &pg_query__alter_owner_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__alter_owner_stmt__pack
                     (const PgQuery__AlterOwnerStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__alter_owner_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__alter_owner_stmt__pack_to_buffer
                     (const PgQuery__AlterOwnerStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__alter_owner_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AlterOwnerStmt *
       pg_query__alter_owner_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AlterOwnerStmt *)
     protobuf_c_message_unpack (&pg_query__alter_owner_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__alter_owner_stmt__free_unpacked
                     (PgQuery__AlterOwnerStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__alter_owner_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__alter_operator_stmt__init
                     (PgQuery__AlterOperatorStmt         *message)
{
  static const PgQuery__AlterOperatorStmt init_value = PG_QUERY__ALTER_OPERATOR_STMT__INIT;
  *message = init_value;
}
size_t pg_query__alter_operator_stmt__get_packed_size
                     (const PgQuery__AlterOperatorStmt *message)
{
  assert(message->base.descriptor == &pg_query__alter_operator_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__alter_operator_stmt__pack
                     (const PgQuery__AlterOperatorStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__alter_operator_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__alter_operator_stmt__pack_to_buffer
                     (const PgQuery__AlterOperatorStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__alter_operator_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AlterOperatorStmt *
       pg_query__alter_operator_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AlterOperatorStmt *)
     protobuf_c_message_unpack (&pg_query__alter_operator_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__alter_operator_stmt__free_unpacked
                     (PgQuery__AlterOperatorStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__alter_operator_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__alter_type_stmt__init
                     (PgQuery__AlterTypeStmt         *message)
{
  static const PgQuery__AlterTypeStmt init_value = PG_QUERY__ALTER_TYPE_STMT__INIT;
  *message = init_value;
}
size_t pg_query__alter_type_stmt__get_packed_size
                     (const PgQuery__AlterTypeStmt *message)
{
  assert(message->base.descriptor == &pg_query__alter_type_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__alter_type_stmt__pack
                     (const PgQuery__AlterTypeStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__alter_type_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__alter_type_stmt__pack_to_buffer
                     (const PgQuery__AlterTypeStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__alter_type_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AlterTypeStmt *
       pg_query__alter_type_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AlterTypeStmt *)
     protobuf_c_message_unpack (&pg_query__alter_type_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__alter_type_stmt__free_unpacked
                     (PgQuery__AlterTypeStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__alter_type_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__rule_stmt__init
                     (PgQuery__RuleStmt         *message)
{
  static const PgQuery__RuleStmt init_value = PG_QUERY__RULE_STMT__INIT;
  *message = init_value;
}
size_t pg_query__rule_stmt__get_packed_size
                     (const PgQuery__RuleStmt *message)
{
  assert(message->base.descriptor == &pg_query__rule_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__rule_stmt__pack
                     (const PgQuery__RuleStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__rule_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__rule_stmt__pack_to_buffer
                     (const PgQuery__RuleStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__rule_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__RuleStmt *
       pg_query__rule_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__RuleStmt *)
     protobuf_c_message_unpack (&pg_query__rule_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__rule_stmt__free_unpacked
                     (PgQuery__RuleStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__rule_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__notify_stmt__init
                     (PgQuery__NotifyStmt         *message)
{
  static const PgQuery__NotifyStmt init_value = PG_QUERY__NOTIFY_STMT__INIT;
  *message = init_value;
}
size_t pg_query__notify_stmt__get_packed_size
                     (const PgQuery__NotifyStmt *message)
{
  assert(message->base.descriptor == &pg_query__notify_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__notify_stmt__pack
                     (const PgQuery__NotifyStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__notify_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__notify_stmt__pack_to_buffer
                     (const PgQuery__NotifyStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__notify_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__NotifyStmt *
       pg_query__notify_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__NotifyStmt *)
     protobuf_c_message_unpack (&pg_query__notify_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__notify_stmt__free_unpacked
                     (PgQuery__NotifyStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__notify_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__listen_stmt__init
                     (PgQuery__ListenStmt         *message)
{
  static const PgQuery__ListenStmt init_value = PG_QUERY__LISTEN_STMT__INIT;
  *message = init_value;
}
size_t pg_query__listen_stmt__get_packed_size
                     (const PgQuery__ListenStmt *message)
{
  assert(message->base.descriptor == &pg_query__listen_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__listen_stmt__pack
                     (const PgQuery__ListenStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__listen_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__listen_stmt__pack_to_buffer
                     (const PgQuery__ListenStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__listen_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__ListenStmt *
       pg_query__listen_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__ListenStmt *)
     protobuf_c_message_unpack (&pg_query__listen_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__listen_stmt__free_unpacked
                     (PgQuery__ListenStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__listen_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__unlisten_stmt__init
                     (PgQuery__UnlistenStmt         *message)
{
  static const PgQuery__UnlistenStmt init_value = PG_QUERY__UNLISTEN_STMT__INIT;
  *message = init_value;
}
size_t pg_query__unlisten_stmt__get_packed_size
                     (const PgQuery__UnlistenStmt *message)
{
  assert(message->base.descriptor == &pg_query__unlisten_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__unlisten_stmt__pack
                     (const PgQuery__UnlistenStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__unlisten_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__unlisten_stmt__pack_to_buffer
                     (const PgQuery__UnlistenStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__unlisten_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__UnlistenStmt *
       pg_query__unlisten_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__UnlistenStmt *)
     protobuf_c_message_unpack (&pg_query__unlisten_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__unlisten_stmt__free_unpacked
                     (PgQuery__UnlistenStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__unlisten_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__transaction_stmt__init
                     (PgQuery__TransactionStmt         *message)
{
  static const PgQuery__TransactionStmt init_value = PG_QUERY__TRANSACTION_STMT__INIT;
  *message = init_value;
}
size_t pg_query__transaction_stmt__get_packed_size
                     (const PgQuery__TransactionStmt *message)
{
  assert(message->base.descriptor == &pg_query__transaction_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__transaction_stmt__pack
                     (const PgQuery__TransactionStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__transaction_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__transaction_stmt__pack_to_buffer
                     (const PgQuery__TransactionStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__transaction_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__TransactionStmt *
       pg_query__transaction_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__TransactionStmt *)
     protobuf_c_message_unpack (&pg_query__transaction_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__transaction_stmt__free_unpacked
                     (PgQuery__TransactionStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__transaction_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__composite_type_stmt__init
                     (PgQuery__CompositeTypeStmt         *message)
{
  static const PgQuery__CompositeTypeStmt init_value = PG_QUERY__COMPOSITE_TYPE_STMT__INIT;
  *message = init_value;
}
size_t pg_query__composite_type_stmt__get_packed_size
                     (const PgQuery__CompositeTypeStmt *message)
{
  assert(message->base.descriptor == &pg_query__composite_type_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__composite_type_stmt__pack
                     (const PgQuery__CompositeTypeStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__composite_type_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__composite_type_stmt__pack_to_buffer
                     (const PgQuery__CompositeTypeStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__composite_type_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CompositeTypeStmt *
       pg_query__composite_type_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CompositeTypeStmt *)
     protobuf_c_message_unpack (&pg_query__composite_type_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__composite_type_stmt__free_unpacked
                     (PgQuery__CompositeTypeStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__composite_type_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__create_enum_stmt__init
                     (PgQuery__CreateEnumStmt         *message)
{
  static const PgQuery__CreateEnumStmt init_value = PG_QUERY__CREATE_ENUM_STMT__INIT;
  *message = init_value;
}
size_t pg_query__create_enum_stmt__get_packed_size
                     (const PgQuery__CreateEnumStmt *message)
{
  assert(message->base.descriptor == &pg_query__create_enum_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__create_enum_stmt__pack
                     (const PgQuery__CreateEnumStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__create_enum_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__create_enum_stmt__pack_to_buffer
                     (const PgQuery__CreateEnumStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__create_enum_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CreateEnumStmt *
       pg_query__create_enum_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CreateEnumStmt *)
     protobuf_c_message_unpack (&pg_query__create_enum_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__create_enum_stmt__free_unpacked
                     (PgQuery__CreateEnumStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__create_enum_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__create_range_stmt__init
                     (PgQuery__CreateRangeStmt         *message)
{
  static const PgQuery__CreateRangeStmt init_value = PG_QUERY__CREATE_RANGE_STMT__INIT;
  *message = init_value;
}
size_t pg_query__create_range_stmt__get_packed_size
                     (const PgQuery__CreateRangeStmt *message)
{
  assert(message->base.descriptor == &pg_query__create_range_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__create_range_stmt__pack
                     (const PgQuery__CreateRangeStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__create_range_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__create_range_stmt__pack_to_buffer
                     (const PgQuery__CreateRangeStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__create_range_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CreateRangeStmt *
       pg_query__create_range_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CreateRangeStmt *)
     protobuf_c_message_unpack (&pg_query__create_range_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__create_range_stmt__free_unpacked
                     (PgQuery__CreateRangeStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__create_range_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__alter_enum_stmt__init
                     (PgQuery__AlterEnumStmt         *message)
{
  static const PgQuery__AlterEnumStmt init_value = PG_QUERY__ALTER_ENUM_STMT__INIT;
  *message = init_value;
}
size_t pg_query__alter_enum_stmt__get_packed_size
                     (const PgQuery__AlterEnumStmt *message)
{
  assert(message->base.descriptor == &pg_query__alter_enum_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__alter_enum_stmt__pack
                     (const PgQuery__AlterEnumStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__alter_enum_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__alter_enum_stmt__pack_to_buffer
                     (const PgQuery__AlterEnumStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__alter_enum_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AlterEnumStmt *
       pg_query__alter_enum_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AlterEnumStmt *)
     protobuf_c_message_unpack (&pg_query__alter_enum_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__alter_enum_stmt__free_unpacked
                     (PgQuery__AlterEnumStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__alter_enum_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__view_stmt__init
                     (PgQuery__ViewStmt         *message)
{
  static const PgQuery__ViewStmt init_value = PG_QUERY__VIEW_STMT__INIT;
  *message = init_value;
}
size_t pg_query__view_stmt__get_packed_size
                     (const PgQuery__ViewStmt *message)
{
  assert(message->base.descriptor == &pg_query__view_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__view_stmt__pack
                     (const PgQuery__ViewStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__view_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__view_stmt__pack_to_buffer
                     (const PgQuery__ViewStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__view_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__ViewStmt *
       pg_query__view_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__ViewStmt *)
     protobuf_c_message_unpack (&pg_query__view_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__view_stmt__free_unpacked
                     (PgQuery__ViewStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__view_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__load_stmt__init
                     (PgQuery__LoadStmt         *message)
{
  static const PgQuery__LoadStmt init_value = PG_QUERY__LOAD_STMT__INIT;
  *message = init_value;
}
size_t pg_query__load_stmt__get_packed_size
                     (const PgQuery__LoadStmt *message)
{
  assert(message->base.descriptor == &pg_query__load_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__load_stmt__pack
                     (const PgQuery__LoadStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__load_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__load_stmt__pack_to_buffer
                     (const PgQuery__LoadStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__load_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__LoadStmt *
       pg_query__load_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__LoadStmt *)
     protobuf_c_message_unpack (&pg_query__load_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__load_stmt__free_unpacked
                     (PgQuery__LoadStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__load_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__createdb_stmt__init
                     (PgQuery__CreatedbStmt         *message)
{
  static const PgQuery__CreatedbStmt init_value = PG_QUERY__CREATEDB_STMT__INIT;
  *message = init_value;
}
size_t pg_query__createdb_stmt__get_packed_size
                     (const PgQuery__CreatedbStmt *message)
{
  assert(message->base.descriptor == &pg_query__createdb_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__createdb_stmt__pack
                     (const PgQuery__CreatedbStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__createdb_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__createdb_stmt__pack_to_buffer
                     (const PgQuery__CreatedbStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__createdb_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CreatedbStmt *
       pg_query__createdb_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CreatedbStmt *)
     protobuf_c_message_unpack (&pg_query__createdb_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__createdb_stmt__free_unpacked
                     (PgQuery__CreatedbStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__createdb_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__alter_database_stmt__init
                     (PgQuery__AlterDatabaseStmt         *message)
{
  static const PgQuery__AlterDatabaseStmt init_value = PG_QUERY__ALTER_DATABASE_STMT__INIT;
  *message = init_value;
}
size_t pg_query__alter_database_stmt__get_packed_size
                     (const PgQuery__AlterDatabaseStmt *message)
{
  assert(message->base.descriptor == &pg_query__alter_database_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__alter_database_stmt__pack
                     (const PgQuery__AlterDatabaseStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__alter_database_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__alter_database_stmt__pack_to_buffer
                     (const PgQuery__AlterDatabaseStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__alter_database_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AlterDatabaseStmt *
       pg_query__alter_database_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AlterDatabaseStmt *)
     protobuf_c_message_unpack (&pg_query__alter_database_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__alter_database_stmt__free_unpacked
                     (PgQuery__AlterDatabaseStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__alter_database_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__alter_database_refresh_coll_stmt__init
                     (PgQuery__AlterDatabaseRefreshCollStmt         *message)
{
  static const PgQuery__AlterDatabaseRefreshCollStmt init_value = PG_QUERY__ALTER_DATABASE_REFRESH_COLL_STMT__INIT;
  *message = init_value;
}
size_t pg_query__alter_database_refresh_coll_stmt__get_packed_size
                     (const PgQuery__AlterDatabaseRefreshCollStmt *message)
{
  assert(message->base.descriptor == &pg_query__alter_database_refresh_coll_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__alter_database_refresh_coll_stmt__pack
                     (const PgQuery__AlterDatabaseRefreshCollStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__alter_database_refresh_coll_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__alter_database_refresh_coll_stmt__pack_to_buffer
                     (const PgQuery__AlterDatabaseRefreshCollStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__alter_database_refresh_coll_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AlterDatabaseRefreshCollStmt *
       pg_query__alter_database_refresh_coll_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AlterDatabaseRefreshCollStmt *)
     protobuf_c_message_unpack (&pg_query__alter_database_refresh_coll_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__alter_database_refresh_coll_stmt__free_unpacked
                     (PgQuery__AlterDatabaseRefreshCollStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__alter_database_refresh_coll_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__alter_database_set_stmt__init
                     (PgQuery__AlterDatabaseSetStmt         *message)
{
  static const PgQuery__AlterDatabaseSetStmt init_value = PG_QUERY__ALTER_DATABASE_SET_STMT__INIT;
  *message = init_value;
}
size_t pg_query__alter_database_set_stmt__get_packed_size
                     (const PgQuery__AlterDatabaseSetStmt *message)
{
  assert(message->base.descriptor == &pg_query__alter_database_set_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__alter_database_set_stmt__pack
                     (const PgQuery__AlterDatabaseSetStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__alter_database_set_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__alter_database_set_stmt__pack_to_buffer
                     (const PgQuery__AlterDatabaseSetStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__alter_database_set_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AlterDatabaseSetStmt *
       pg_query__alter_database_set_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AlterDatabaseSetStmt *)
     protobuf_c_message_unpack (&pg_query__alter_database_set_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__alter_database_set_stmt__free_unpacked
                     (PgQuery__AlterDatabaseSetStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__alter_database_set_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__dropdb_stmt__init
                     (PgQuery__DropdbStmt         *message)
{
  static const PgQuery__DropdbStmt init_value = PG_QUERY__DROPDB_STMT__INIT;
  *message = init_value;
}
size_t pg_query__dropdb_stmt__get_packed_size
                     (const PgQuery__DropdbStmt *message)
{
  assert(message->base.descriptor == &pg_query__dropdb_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__dropdb_stmt__pack
                     (const PgQuery__DropdbStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__dropdb_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__dropdb_stmt__pack_to_buffer
                     (const PgQuery__DropdbStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__dropdb_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__DropdbStmt *
       pg_query__dropdb_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__DropdbStmt *)
     protobuf_c_message_unpack (&pg_query__dropdb_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__dropdb_stmt__free_unpacked
                     (PgQuery__DropdbStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__dropdb_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__alter_system_stmt__init
                     (PgQuery__AlterSystemStmt         *message)
{
  static const PgQuery__AlterSystemStmt init_value = PG_QUERY__ALTER_SYSTEM_STMT__INIT;
  *message = init_value;
}
size_t pg_query__alter_system_stmt__get_packed_size
                     (const PgQuery__AlterSystemStmt *message)
{
  assert(message->base.descriptor == &pg_query__alter_system_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__alter_system_stmt__pack
                     (const PgQuery__AlterSystemStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__alter_system_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__alter_system_stmt__pack_to_buffer
                     (const PgQuery__AlterSystemStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__alter_system_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AlterSystemStmt *
       pg_query__alter_system_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AlterSystemStmt *)
     protobuf_c_message_unpack (&pg_query__alter_system_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__alter_system_stmt__free_unpacked
                     (PgQuery__AlterSystemStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__alter_system_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__cluster_stmt__init
                     (PgQuery__ClusterStmt         *message)
{
  static const PgQuery__ClusterStmt init_value = PG_QUERY__CLUSTER_STMT__INIT;
  *message = init_value;
}
size_t pg_query__cluster_stmt__get_packed_size
                     (const PgQuery__ClusterStmt *message)
{
  assert(message->base.descriptor == &pg_query__cluster_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__cluster_stmt__pack
                     (const PgQuery__ClusterStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__cluster_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__cluster_stmt__pack_to_buffer
                     (const PgQuery__ClusterStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__cluster_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__ClusterStmt *
       pg_query__cluster_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__ClusterStmt *)
     protobuf_c_message_unpack (&pg_query__cluster_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__cluster_stmt__free_unpacked
                     (PgQuery__ClusterStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__cluster_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__vacuum_stmt__init
                     (PgQuery__VacuumStmt         *message)
{
  static const PgQuery__VacuumStmt init_value = PG_QUERY__VACUUM_STMT__INIT;
  *message = init_value;
}
size_t pg_query__vacuum_stmt__get_packed_size
                     (const PgQuery__VacuumStmt *message)
{
  assert(message->base.descriptor == &pg_query__vacuum_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__vacuum_stmt__pack
                     (const PgQuery__VacuumStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__vacuum_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__vacuum_stmt__pack_to_buffer
                     (const PgQuery__VacuumStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__vacuum_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__VacuumStmt *
       pg_query__vacuum_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__VacuumStmt *)
     protobuf_c_message_unpack (&pg_query__vacuum_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__vacuum_stmt__free_unpacked
                     (PgQuery__VacuumStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__vacuum_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__vacuum_relation__init
                     (PgQuery__VacuumRelation         *message)
{
  static const PgQuery__VacuumRelation init_value = PG_QUERY__VACUUM_RELATION__INIT;
  *message = init_value;
}
size_t pg_query__vacuum_relation__get_packed_size
                     (const PgQuery__VacuumRelation *message)
{
  assert(message->base.descriptor == &pg_query__vacuum_relation__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__vacuum_relation__pack
                     (const PgQuery__VacuumRelation *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__vacuum_relation__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__vacuum_relation__pack_to_buffer
                     (const PgQuery__VacuumRelation *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__vacuum_relation__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__VacuumRelation *
       pg_query__vacuum_relation__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__VacuumRelation *)
     protobuf_c_message_unpack (&pg_query__vacuum_relation__descriptor,
                                allocator, len, data);
}
void   pg_query__vacuum_relation__free_unpacked
                     (PgQuery__VacuumRelation *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__vacuum_relation__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__explain_stmt__init
                     (PgQuery__ExplainStmt         *message)
{
  static const PgQuery__ExplainStmt init_value = PG_QUERY__EXPLAIN_STMT__INIT;
  *message = init_value;
}
size_t pg_query__explain_stmt__get_packed_size
                     (const PgQuery__ExplainStmt *message)
{
  assert(message->base.descriptor == &pg_query__explain_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__explain_stmt__pack
                     (const PgQuery__ExplainStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__explain_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__explain_stmt__pack_to_buffer
                     (const PgQuery__ExplainStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__explain_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__ExplainStmt *
       pg_query__explain_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__ExplainStmt *)
     protobuf_c_message_unpack (&pg_query__explain_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__explain_stmt__free_unpacked
                     (PgQuery__ExplainStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__explain_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__create_table_as_stmt__init
                     (PgQuery__CreateTableAsStmt         *message)
{
  static const PgQuery__CreateTableAsStmt init_value = PG_QUERY__CREATE_TABLE_AS_STMT__INIT;
  *message = init_value;
}
size_t pg_query__create_table_as_stmt__get_packed_size
                     (const PgQuery__CreateTableAsStmt *message)
{
  assert(message->base.descriptor == &pg_query__create_table_as_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__create_table_as_stmt__pack
                     (const PgQuery__CreateTableAsStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__create_table_as_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__create_table_as_stmt__pack_to_buffer
                     (const PgQuery__CreateTableAsStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__create_table_as_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CreateTableAsStmt *
       pg_query__create_table_as_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CreateTableAsStmt *)
     protobuf_c_message_unpack (&pg_query__create_table_as_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__create_table_as_stmt__free_unpacked
                     (PgQuery__CreateTableAsStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__create_table_as_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__refresh_mat_view_stmt__init
                     (PgQuery__RefreshMatViewStmt         *message)
{
  static const PgQuery__RefreshMatViewStmt init_value = PG_QUERY__REFRESH_MAT_VIEW_STMT__INIT;
  *message = init_value;
}
size_t pg_query__refresh_mat_view_stmt__get_packed_size
                     (const PgQuery__RefreshMatViewStmt *message)
{
  assert(message->base.descriptor == &pg_query__refresh_mat_view_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__refresh_mat_view_stmt__pack
                     (const PgQuery__RefreshMatViewStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__refresh_mat_view_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__refresh_mat_view_stmt__pack_to_buffer
                     (const PgQuery__RefreshMatViewStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__refresh_mat_view_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__RefreshMatViewStmt *
       pg_query__refresh_mat_view_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__RefreshMatViewStmt *)
     protobuf_c_message_unpack (&pg_query__refresh_mat_view_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__refresh_mat_view_stmt__free_unpacked
                     (PgQuery__RefreshMatViewStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__refresh_mat_view_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__check_point_stmt__init
                     (PgQuery__CheckPointStmt         *message)
{
  static const PgQuery__CheckPointStmt init_value = PG_QUERY__CHECK_POINT_STMT__INIT;
  *message = init_value;
}
size_t pg_query__check_point_stmt__get_packed_size
                     (const PgQuery__CheckPointStmt *message)
{
  assert(message->base.descriptor == &pg_query__check_point_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__check_point_stmt__pack
                     (const PgQuery__CheckPointStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__check_point_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__check_point_stmt__pack_to_buffer
                     (const PgQuery__CheckPointStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__check_point_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CheckPointStmt *
       pg_query__check_point_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CheckPointStmt *)
     protobuf_c_message_unpack (&pg_query__check_point_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__check_point_stmt__free_unpacked
                     (PgQuery__CheckPointStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__check_point_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__discard_stmt__init
                     (PgQuery__DiscardStmt         *message)
{
  static const PgQuery__DiscardStmt init_value = PG_QUERY__DISCARD_STMT__INIT;
  *message = init_value;
}
size_t pg_query__discard_stmt__get_packed_size
                     (const PgQuery__DiscardStmt *message)
{
  assert(message->base.descriptor == &pg_query__discard_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__discard_stmt__pack
                     (const PgQuery__DiscardStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__discard_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__discard_stmt__pack_to_buffer
                     (const PgQuery__DiscardStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__discard_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__DiscardStmt *
       pg_query__discard_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__DiscardStmt *)
     protobuf_c_message_unpack (&pg_query__discard_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__discard_stmt__free_unpacked
                     (PgQuery__DiscardStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__discard_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__lock_stmt__init
                     (PgQuery__LockStmt         *message)
{
  static const PgQuery__LockStmt init_value = PG_QUERY__LOCK_STMT__INIT;
  *message = init_value;
}
size_t pg_query__lock_stmt__get_packed_size
                     (const PgQuery__LockStmt *message)
{
  assert(message->base.descriptor == &pg_query__lock_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__lock_stmt__pack
                     (const PgQuery__LockStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__lock_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__lock_stmt__pack_to_buffer
                     (const PgQuery__LockStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__lock_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__LockStmt *
       pg_query__lock_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__LockStmt *)
     protobuf_c_message_unpack (&pg_query__lock_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__lock_stmt__free_unpacked
                     (PgQuery__LockStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__lock_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__constraints_set_stmt__init
                     (PgQuery__ConstraintsSetStmt         *message)
{
  static const PgQuery__ConstraintsSetStmt init_value = PG_QUERY__CONSTRAINTS_SET_STMT__INIT;
  *message = init_value;
}
size_t pg_query__constraints_set_stmt__get_packed_size
                     (const PgQuery__ConstraintsSetStmt *message)
{
  assert(message->base.descriptor == &pg_query__constraints_set_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__constraints_set_stmt__pack
                     (const PgQuery__ConstraintsSetStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__constraints_set_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__constraints_set_stmt__pack_to_buffer
                     (const PgQuery__ConstraintsSetStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__constraints_set_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__ConstraintsSetStmt *
       pg_query__constraints_set_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__ConstraintsSetStmt *)
     protobuf_c_message_unpack (&pg_query__constraints_set_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__constraints_set_stmt__free_unpacked
                     (PgQuery__ConstraintsSetStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__constraints_set_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__reindex_stmt__init
                     (PgQuery__ReindexStmt         *message)
{
  static const PgQuery__ReindexStmt init_value = PG_QUERY__REINDEX_STMT__INIT;
  *message = init_value;
}
size_t pg_query__reindex_stmt__get_packed_size
                     (const PgQuery__ReindexStmt *message)
{
  assert(message->base.descriptor == &pg_query__reindex_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__reindex_stmt__pack
                     (const PgQuery__ReindexStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__reindex_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__reindex_stmt__pack_to_buffer
                     (const PgQuery__ReindexStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__reindex_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__ReindexStmt *
       pg_query__reindex_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__ReindexStmt *)
     protobuf_c_message_unpack (&pg_query__reindex_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__reindex_stmt__free_unpacked
                     (PgQuery__ReindexStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__reindex_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__create_conversion_stmt__init
                     (PgQuery__CreateConversionStmt         *message)
{
  static const PgQuery__CreateConversionStmt init_value = PG_QUERY__CREATE_CONVERSION_STMT__INIT;
  *message = init_value;
}
size_t pg_query__create_conversion_stmt__get_packed_size
                     (const PgQuery__CreateConversionStmt *message)
{
  assert(message->base.descriptor == &pg_query__create_conversion_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__create_conversion_stmt__pack
                     (const PgQuery__CreateConversionStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__create_conversion_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__create_conversion_stmt__pack_to_buffer
                     (const PgQuery__CreateConversionStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__create_conversion_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CreateConversionStmt *
       pg_query__create_conversion_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CreateConversionStmt *)
     protobuf_c_message_unpack (&pg_query__create_conversion_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__create_conversion_stmt__free_unpacked
                     (PgQuery__CreateConversionStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__create_conversion_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__create_cast_stmt__init
                     (PgQuery__CreateCastStmt         *message)
{
  static const PgQuery__CreateCastStmt init_value = PG_QUERY__CREATE_CAST_STMT__INIT;
  *message = init_value;
}
size_t pg_query__create_cast_stmt__get_packed_size
                     (const PgQuery__CreateCastStmt *message)
{
  assert(message->base.descriptor == &pg_query__create_cast_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__create_cast_stmt__pack
                     (const PgQuery__CreateCastStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__create_cast_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__create_cast_stmt__pack_to_buffer
                     (const PgQuery__CreateCastStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__create_cast_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CreateCastStmt *
       pg_query__create_cast_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CreateCastStmt *)
     protobuf_c_message_unpack (&pg_query__create_cast_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__create_cast_stmt__free_unpacked
                     (PgQuery__CreateCastStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__create_cast_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__create_transform_stmt__init
                     (PgQuery__CreateTransformStmt         *message)
{
  static const PgQuery__CreateTransformStmt init_value = PG_QUERY__CREATE_TRANSFORM_STMT__INIT;
  *message = init_value;
}
size_t pg_query__create_transform_stmt__get_packed_size
                     (const PgQuery__CreateTransformStmt *message)
{
  assert(message->base.descriptor == &pg_query__create_transform_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__create_transform_stmt__pack
                     (const PgQuery__CreateTransformStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__create_transform_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__create_transform_stmt__pack_to_buffer
                     (const PgQuery__CreateTransformStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__create_transform_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CreateTransformStmt *
       pg_query__create_transform_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CreateTransformStmt *)
     protobuf_c_message_unpack (&pg_query__create_transform_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__create_transform_stmt__free_unpacked
                     (PgQuery__CreateTransformStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__create_transform_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__prepare_stmt__init
                     (PgQuery__PrepareStmt         *message)
{
  static const PgQuery__PrepareStmt init_value = PG_QUERY__PREPARE_STMT__INIT;
  *message = init_value;
}
size_t pg_query__prepare_stmt__get_packed_size
                     (const PgQuery__PrepareStmt *message)
{
  assert(message->base.descriptor == &pg_query__prepare_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__prepare_stmt__pack
                     (const PgQuery__PrepareStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__prepare_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__prepare_stmt__pack_to_buffer
                     (const PgQuery__PrepareStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__prepare_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__PrepareStmt *
       pg_query__prepare_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__PrepareStmt *)
     protobuf_c_message_unpack (&pg_query__prepare_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__prepare_stmt__free_unpacked
                     (PgQuery__PrepareStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__prepare_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__execute_stmt__init
                     (PgQuery__ExecuteStmt         *message)
{
  static const PgQuery__ExecuteStmt init_value = PG_QUERY__EXECUTE_STMT__INIT;
  *message = init_value;
}
size_t pg_query__execute_stmt__get_packed_size
                     (const PgQuery__ExecuteStmt *message)
{
  assert(message->base.descriptor == &pg_query__execute_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__execute_stmt__pack
                     (const PgQuery__ExecuteStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__execute_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__execute_stmt__pack_to_buffer
                     (const PgQuery__ExecuteStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__execute_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__ExecuteStmt *
       pg_query__execute_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__ExecuteStmt *)
     protobuf_c_message_unpack (&pg_query__execute_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__execute_stmt__free_unpacked
                     (PgQuery__ExecuteStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__execute_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__deallocate_stmt__init
                     (PgQuery__DeallocateStmt         *message)
{
  static const PgQuery__DeallocateStmt init_value = PG_QUERY__DEALLOCATE_STMT__INIT;
  *message = init_value;
}
size_t pg_query__deallocate_stmt__get_packed_size
                     (const PgQuery__DeallocateStmt *message)
{
  assert(message->base.descriptor == &pg_query__deallocate_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__deallocate_stmt__pack
                     (const PgQuery__DeallocateStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__deallocate_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__deallocate_stmt__pack_to_buffer
                     (const PgQuery__DeallocateStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__deallocate_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__DeallocateStmt *
       pg_query__deallocate_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__DeallocateStmt *)
     protobuf_c_message_unpack (&pg_query__deallocate_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__deallocate_stmt__free_unpacked
                     (PgQuery__DeallocateStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__deallocate_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__drop_owned_stmt__init
                     (PgQuery__DropOwnedStmt         *message)
{
  static const PgQuery__DropOwnedStmt init_value = PG_QUERY__DROP_OWNED_STMT__INIT;
  *message = init_value;
}
size_t pg_query__drop_owned_stmt__get_packed_size
                     (const PgQuery__DropOwnedStmt *message)
{
  assert(message->base.descriptor == &pg_query__drop_owned_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__drop_owned_stmt__pack
                     (const PgQuery__DropOwnedStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__drop_owned_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__drop_owned_stmt__pack_to_buffer
                     (const PgQuery__DropOwnedStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__drop_owned_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__DropOwnedStmt *
       pg_query__drop_owned_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__DropOwnedStmt *)
     protobuf_c_message_unpack (&pg_query__drop_owned_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__drop_owned_stmt__free_unpacked
                     (PgQuery__DropOwnedStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__drop_owned_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__reassign_owned_stmt__init
                     (PgQuery__ReassignOwnedStmt         *message)
{
  static const PgQuery__ReassignOwnedStmt init_value = PG_QUERY__REASSIGN_OWNED_STMT__INIT;
  *message = init_value;
}
size_t pg_query__reassign_owned_stmt__get_packed_size
                     (const PgQuery__ReassignOwnedStmt *message)
{
  assert(message->base.descriptor == &pg_query__reassign_owned_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__reassign_owned_stmt__pack
                     (const PgQuery__ReassignOwnedStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__reassign_owned_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__reassign_owned_stmt__pack_to_buffer
                     (const PgQuery__ReassignOwnedStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__reassign_owned_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__ReassignOwnedStmt *
       pg_query__reassign_owned_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__ReassignOwnedStmt *)
     protobuf_c_message_unpack (&pg_query__reassign_owned_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__reassign_owned_stmt__free_unpacked
                     (PgQuery__ReassignOwnedStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__reassign_owned_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__alter_tsdictionary_stmt__init
                     (PgQuery__AlterTSDictionaryStmt         *message)
{
  static const PgQuery__AlterTSDictionaryStmt init_value = PG_QUERY__ALTER_TSDICTIONARY_STMT__INIT;
  *message = init_value;
}
size_t pg_query__alter_tsdictionary_stmt__get_packed_size
                     (const PgQuery__AlterTSDictionaryStmt *message)
{
  assert(message->base.descriptor == &pg_query__alter_tsdictionary_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__alter_tsdictionary_stmt__pack
                     (const PgQuery__AlterTSDictionaryStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__alter_tsdictionary_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__alter_tsdictionary_stmt__pack_to_buffer
                     (const PgQuery__AlterTSDictionaryStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__alter_tsdictionary_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AlterTSDictionaryStmt *
       pg_query__alter_tsdictionary_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AlterTSDictionaryStmt *)
     protobuf_c_message_unpack (&pg_query__alter_tsdictionary_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__alter_tsdictionary_stmt__free_unpacked
                     (PgQuery__AlterTSDictionaryStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__alter_tsdictionary_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__alter_tsconfiguration_stmt__init
                     (PgQuery__AlterTSConfigurationStmt         *message)
{
  static const PgQuery__AlterTSConfigurationStmt init_value = PG_QUERY__ALTER_TSCONFIGURATION_STMT__INIT;
  *message = init_value;
}
size_t pg_query__alter_tsconfiguration_stmt__get_packed_size
                     (const PgQuery__AlterTSConfigurationStmt *message)
{
  assert(message->base.descriptor == &pg_query__alter_tsconfiguration_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__alter_tsconfiguration_stmt__pack
                     (const PgQuery__AlterTSConfigurationStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__alter_tsconfiguration_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__alter_tsconfiguration_stmt__pack_to_buffer
                     (const PgQuery__AlterTSConfigurationStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__alter_tsconfiguration_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AlterTSConfigurationStmt *
       pg_query__alter_tsconfiguration_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AlterTSConfigurationStmt *)
     protobuf_c_message_unpack (&pg_query__alter_tsconfiguration_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__alter_tsconfiguration_stmt__free_unpacked
                     (PgQuery__AlterTSConfigurationStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__alter_tsconfiguration_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__publication_table__init
                     (PgQuery__PublicationTable         *message)
{
  static const PgQuery__PublicationTable init_value = PG_QUERY__PUBLICATION_TABLE__INIT;
  *message = init_value;
}
size_t pg_query__publication_table__get_packed_size
                     (const PgQuery__PublicationTable *message)
{
  assert(message->base.descriptor == &pg_query__publication_table__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__publication_table__pack
                     (const PgQuery__PublicationTable *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__publication_table__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__publication_table__pack_to_buffer
                     (const PgQuery__PublicationTable *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__publication_table__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__PublicationTable *
       pg_query__publication_table__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__PublicationTable *)
     protobuf_c_message_unpack (&pg_query__publication_table__descriptor,
                                allocator, len, data);
}
void   pg_query__publication_table__free_unpacked
                     (PgQuery__PublicationTable *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__publication_table__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__publication_obj_spec__init
                     (PgQuery__PublicationObjSpec         *message)
{
  static const PgQuery__PublicationObjSpec init_value = PG_QUERY__PUBLICATION_OBJ_SPEC__INIT;
  *message = init_value;
}
size_t pg_query__publication_obj_spec__get_packed_size
                     (const PgQuery__PublicationObjSpec *message)
{
  assert(message->base.descriptor == &pg_query__publication_obj_spec__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__publication_obj_spec__pack
                     (const PgQuery__PublicationObjSpec *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__publication_obj_spec__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__publication_obj_spec__pack_to_buffer
                     (const PgQuery__PublicationObjSpec *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__publication_obj_spec__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__PublicationObjSpec *
       pg_query__publication_obj_spec__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__PublicationObjSpec *)
     protobuf_c_message_unpack (&pg_query__publication_obj_spec__descriptor,
                                allocator, len, data);
}
void   pg_query__publication_obj_spec__free_unpacked
                     (PgQuery__PublicationObjSpec *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__publication_obj_spec__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__create_publication_stmt__init
                     (PgQuery__CreatePublicationStmt         *message)
{
  static const PgQuery__CreatePublicationStmt init_value = PG_QUERY__CREATE_PUBLICATION_STMT__INIT;
  *message = init_value;
}
size_t pg_query__create_publication_stmt__get_packed_size
                     (const PgQuery__CreatePublicationStmt *message)
{
  assert(message->base.descriptor == &pg_query__create_publication_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__create_publication_stmt__pack
                     (const PgQuery__CreatePublicationStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__create_publication_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__create_publication_stmt__pack_to_buffer
                     (const PgQuery__CreatePublicationStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__create_publication_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CreatePublicationStmt *
       pg_query__create_publication_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CreatePublicationStmt *)
     protobuf_c_message_unpack (&pg_query__create_publication_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__create_publication_stmt__free_unpacked
                     (PgQuery__CreatePublicationStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__create_publication_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__alter_publication_stmt__init
                     (PgQuery__AlterPublicationStmt         *message)
{
  static const PgQuery__AlterPublicationStmt init_value = PG_QUERY__ALTER_PUBLICATION_STMT__INIT;
  *message = init_value;
}
size_t pg_query__alter_publication_stmt__get_packed_size
                     (const PgQuery__AlterPublicationStmt *message)
{
  assert(message->base.descriptor == &pg_query__alter_publication_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__alter_publication_stmt__pack
                     (const PgQuery__AlterPublicationStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__alter_publication_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__alter_publication_stmt__pack_to_buffer
                     (const PgQuery__AlterPublicationStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__alter_publication_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AlterPublicationStmt *
       pg_query__alter_publication_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AlterPublicationStmt *)
     protobuf_c_message_unpack (&pg_query__alter_publication_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__alter_publication_stmt__free_unpacked
                     (PgQuery__AlterPublicationStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__alter_publication_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__create_subscription_stmt__init
                     (PgQuery__CreateSubscriptionStmt         *message)
{
  static const PgQuery__CreateSubscriptionStmt init_value = PG_QUERY__CREATE_SUBSCRIPTION_STMT__INIT;
  *message = init_value;
}
size_t pg_query__create_subscription_stmt__get_packed_size
                     (const PgQuery__CreateSubscriptionStmt *message)
{
  assert(message->base.descriptor == &pg_query__create_subscription_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__create_subscription_stmt__pack
                     (const PgQuery__CreateSubscriptionStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__create_subscription_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__create_subscription_stmt__pack_to_buffer
                     (const PgQuery__CreateSubscriptionStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__create_subscription_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__CreateSubscriptionStmt *
       pg_query__create_subscription_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__CreateSubscriptionStmt *)
     protobuf_c_message_unpack (&pg_query__create_subscription_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__create_subscription_stmt__free_unpacked
                     (PgQuery__CreateSubscriptionStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__create_subscription_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__alter_subscription_stmt__init
                     (PgQuery__AlterSubscriptionStmt         *message)
{
  static const PgQuery__AlterSubscriptionStmt init_value = PG_QUERY__ALTER_SUBSCRIPTION_STMT__INIT;
  *message = init_value;
}
size_t pg_query__alter_subscription_stmt__get_packed_size
                     (const PgQuery__AlterSubscriptionStmt *message)
{
  assert(message->base.descriptor == &pg_query__alter_subscription_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__alter_subscription_stmt__pack
                     (const PgQuery__AlterSubscriptionStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__alter_subscription_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__alter_subscription_stmt__pack_to_buffer
                     (const PgQuery__AlterSubscriptionStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__alter_subscription_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__AlterSubscriptionStmt *
       pg_query__alter_subscription_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__AlterSubscriptionStmt *)
     protobuf_c_message_unpack (&pg_query__alter_subscription_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__alter_subscription_stmt__free_unpacked
                     (PgQuery__AlterSubscriptionStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__alter_subscription_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__drop_subscription_stmt__init
                     (PgQuery__DropSubscriptionStmt         *message)
{
  static const PgQuery__DropSubscriptionStmt init_value = PG_QUERY__DROP_SUBSCRIPTION_STMT__INIT;
  *message = init_value;
}
size_t pg_query__drop_subscription_stmt__get_packed_size
                     (const PgQuery__DropSubscriptionStmt *message)
{
  assert(message->base.descriptor == &pg_query__drop_subscription_stmt__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__drop_subscription_stmt__pack
                     (const PgQuery__DropSubscriptionStmt *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__drop_subscription_stmt__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__drop_subscription_stmt__pack_to_buffer
                     (const PgQuery__DropSubscriptionStmt *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__drop_subscription_stmt__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__DropSubscriptionStmt *
       pg_query__drop_subscription_stmt__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__DropSubscriptionStmt *)
     protobuf_c_message_unpack (&pg_query__drop_subscription_stmt__descriptor,
                                allocator, len, data);
}
void   pg_query__drop_subscription_stmt__free_unpacked
                     (PgQuery__DropSubscriptionStmt *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__drop_subscription_stmt__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   pg_query__scan_token__init
                     (PgQuery__ScanToken         *message)
{
  static const PgQuery__ScanToken init_value = PG_QUERY__SCAN_TOKEN__INIT;
  *message = init_value;
}
size_t pg_query__scan_token__get_packed_size
                     (const PgQuery__ScanToken *message)
{
  assert(message->base.descriptor == &pg_query__scan_token__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t pg_query__scan_token__pack
                     (const PgQuery__ScanToken *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &pg_query__scan_token__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t pg_query__scan_token__pack_to_buffer
                     (const PgQuery__ScanToken *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &pg_query__scan_token__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
PgQuery__ScanToken *
       pg_query__scan_token__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (PgQuery__ScanToken *)
     protobuf_c_message_unpack (&pg_query__scan_token__descriptor,
                                allocator, len, data);
}
void   pg_query__scan_token__free_unpacked
                     (PgQuery__ScanToken *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &pg_query__scan_token__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
static const ProtobufCFieldDescriptor pg_query__parse_result__field_descriptors[2] =
{
  {
    "version",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ParseResult, version),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "stmts",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__ParseResult, n_stmts),
    offsetof(PgQuery__ParseResult, stmts),
    &pg_query__raw_stmt__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__parse_result__field_indices_by_name[] = {
  1,   /* field[1] = stmts */
  0,   /* field[0] = version */
};
static const ProtobufCIntRange pg_query__parse_result__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor pg_query__parse_result__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.ParseResult",
  "ParseResult",
  "PgQuery__ParseResult",
  "pg_query",
  sizeof(PgQuery__ParseResult),
  2,
  pg_query__parse_result__field_descriptors,
  pg_query__parse_result__field_indices_by_name,
  1,  pg_query__parse_result__number_ranges,
  (ProtobufCMessageInit) pg_query__parse_result__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__scan_result__field_descriptors[2] =
{
  {
    "version",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ScanResult, version),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "tokens",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__ScanResult, n_tokens),
    offsetof(PgQuery__ScanResult, tokens),
    &pg_query__scan_token__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__scan_result__field_indices_by_name[] = {
  1,   /* field[1] = tokens */
  0,   /* field[0] = version */
};
static const ProtobufCIntRange pg_query__scan_result__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor pg_query__scan_result__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.ScanResult",
  "ScanResult",
  "PgQuery__ScanResult",
  "pg_query",
  sizeof(PgQuery__ScanResult),
  2,
  pg_query__scan_result__field_descriptors,
  pg_query__scan_result__field_indices_by_name,
  1,  pg_query__scan_result__number_ranges,
  (ProtobufCMessageInit) pg_query__scan_result__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__node__field_descriptors[268] =
{
  {
    "alias",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, alias),
    &pg_query__alias__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "range_var",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, range_var),
    &pg_query__range_var__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "table_func",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, table_func),
    &pg_query__table_func__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "into_clause",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, into_clause),
    &pg_query__into_clause__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "var",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, var),
    &pg_query__var__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "param",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, param),
    &pg_query__param__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "aggref",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, aggref),
    &pg_query__aggref__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "grouping_func",
    8,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, grouping_func),
    &pg_query__grouping_func__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "window_func",
    9,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, window_func),
    &pg_query__window_func__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "window_func_run_condition",
    10,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, window_func_run_condition),
    &pg_query__window_func_run_condition__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "merge_support_func",
    11,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, merge_support_func),
    &pg_query__merge_support_func__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "subscripting_ref",
    12,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, subscripting_ref),
    &pg_query__subscripting_ref__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "func_expr",
    13,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, func_expr),
    &pg_query__func_expr__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "named_arg_expr",
    14,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, named_arg_expr),
    &pg_query__named_arg_expr__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "op_expr",
    15,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, op_expr),
    &pg_query__op_expr__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "distinct_expr",
    16,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, distinct_expr),
    &pg_query__distinct_expr__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "null_if_expr",
    17,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, null_if_expr),
    &pg_query__null_if_expr__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "scalar_array_op_expr",
    18,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, scalar_array_op_expr),
    &pg_query__scalar_array_op_expr__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "bool_expr",
    19,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, bool_expr),
    &pg_query__bool_expr__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "sub_link",
    20,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, sub_link),
    &pg_query__sub_link__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "sub_plan",
    21,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, sub_plan),
    &pg_query__sub_plan__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alternative_sub_plan",
    22,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, alternative_sub_plan),
    &pg_query__alternative_sub_plan__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "field_select",
    23,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, field_select),
    &pg_query__field_select__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "field_store",
    24,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, field_store),
    &pg_query__field_store__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "relabel_type",
    25,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, relabel_type),
    &pg_query__relabel_type__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "coerce_via_io",
    26,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, coerce_via_io),
    &pg_query__coerce_via_io__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "array_coerce_expr",
    27,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, array_coerce_expr),
    &pg_query__array_coerce_expr__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "convert_rowtype_expr",
    28,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, convert_rowtype_expr),
    &pg_query__convert_rowtype_expr__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "collate_expr",
    29,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, collate_expr),
    &pg_query__collate_expr__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "case_expr",
    30,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, case_expr),
    &pg_query__case_expr__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "case_when",
    31,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, case_when),
    &pg_query__case_when__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "case_test_expr",
    32,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, case_test_expr),
    &pg_query__case_test_expr__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "array_expr",
    33,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, array_expr),
    &pg_query__array_expr__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "row_expr",
    34,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, row_expr),
    &pg_query__row_expr__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "row_compare_expr",
    35,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, row_compare_expr),
    &pg_query__row_compare_expr__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "coalesce_expr",
    36,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, coalesce_expr),
    &pg_query__coalesce_expr__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "min_max_expr",
    37,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, min_max_expr),
    &pg_query__min_max_expr__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "sqlvalue_function",
    38,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, sqlvalue_function),
    &pg_query__sqlvalue_function__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "xml_expr",
    39,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, xml_expr),
    &pg_query__xml_expr__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "json_format",
    40,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, json_format),
    &pg_query__json_format__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "json_returning",
    41,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, json_returning),
    &pg_query__json_returning__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "json_value_expr",
    42,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, json_value_expr),
    &pg_query__json_value_expr__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "json_constructor_expr",
    43,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, json_constructor_expr),
    &pg_query__json_constructor_expr__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "json_is_predicate",
    44,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, json_is_predicate),
    &pg_query__json_is_predicate__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "json_behavior",
    45,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, json_behavior),
    &pg_query__json_behavior__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "json_expr",
    46,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, json_expr),
    &pg_query__json_expr__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "json_table_path",
    47,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, json_table_path),
    &pg_query__json_table_path__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "json_table_path_scan",
    48,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, json_table_path_scan),
    &pg_query__json_table_path_scan__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "json_table_sibling_join",
    49,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, json_table_sibling_join),
    &pg_query__json_table_sibling_join__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "null_test",
    50,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, null_test),
    &pg_query__null_test__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "boolean_test",
    51,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, boolean_test),
    &pg_query__boolean_test__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "merge_action",
    52,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, merge_action),
    &pg_query__merge_action__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "coerce_to_domain",
    53,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, coerce_to_domain),
    &pg_query__coerce_to_domain__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "coerce_to_domain_value",
    54,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, coerce_to_domain_value),
    &pg_query__coerce_to_domain_value__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "set_to_default",
    55,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, set_to_default),
    &pg_query__set_to_default__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "current_of_expr",
    56,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, current_of_expr),
    &pg_query__current_of_expr__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "next_value_expr",
    57,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, next_value_expr),
    &pg_query__next_value_expr__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "inference_elem",
    58,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, inference_elem),
    &pg_query__inference_elem__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "target_entry",
    59,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, target_entry),
    &pg_query__target_entry__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "range_tbl_ref",
    60,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, range_tbl_ref),
    &pg_query__range_tbl_ref__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "join_expr",
    61,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, join_expr),
    &pg_query__join_expr__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "from_expr",
    62,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, from_expr),
    &pg_query__from_expr__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "on_conflict_expr",
    63,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, on_conflict_expr),
    &pg_query__on_conflict_expr__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "query",
    64,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, query),
    &pg_query__query__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "type_name",
    65,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, type_name),
    &pg_query__type_name__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "column_ref",
    66,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, column_ref),
    &pg_query__column_ref__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "param_ref",
    67,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, param_ref),
    &pg_query__param_ref__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "a_expr",
    68,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, a_expr),
    &pg_query__a__expr__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "type_cast",
    69,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, type_cast),
    &pg_query__type_cast__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "collate_clause",
    70,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, collate_clause),
    &pg_query__collate_clause__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "role_spec",
    71,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, role_spec),
    &pg_query__role_spec__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "func_call",
    72,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, func_call),
    &pg_query__func_call__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "a_star",
    73,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, a_star),
    &pg_query__a__star__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "a_indices",
    74,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, a_indices),
    &pg_query__a__indices__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "a_indirection",
    75,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, a_indirection),
    &pg_query__a__indirection__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "a_array_expr",
    76,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, a_array_expr),
    &pg_query__a__array_expr__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "res_target",
    77,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, res_target),
    &pg_query__res_target__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "multi_assign_ref",
    78,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, multi_assign_ref),
    &pg_query__multi_assign_ref__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "sort_by",
    79,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, sort_by),
    &pg_query__sort_by__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "window_def",
    80,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, window_def),
    &pg_query__window_def__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "range_subselect",
    81,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, range_subselect),
    &pg_query__range_subselect__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "range_function",
    82,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, range_function),
    &pg_query__range_function__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "range_table_func",
    83,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, range_table_func),
    &pg_query__range_table_func__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "range_table_func_col",
    84,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, range_table_func_col),
    &pg_query__range_table_func_col__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "range_table_sample",
    85,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, range_table_sample),
    &pg_query__range_table_sample__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "column_def",
    86,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, column_def),
    &pg_query__column_def__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "table_like_clause",
    87,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, table_like_clause),
    &pg_query__table_like_clause__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "index_elem",
    88,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, index_elem),
    &pg_query__index_elem__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "def_elem",
    89,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, def_elem),
    &pg_query__def_elem__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "locking_clause",
    90,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, locking_clause),
    &pg_query__locking_clause__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "xml_serialize",
    91,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, xml_serialize),
    &pg_query__xml_serialize__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "partition_elem",
    92,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, partition_elem),
    &pg_query__partition_elem__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "partition_spec",
    93,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, partition_spec),
    &pg_query__partition_spec__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "partition_bound_spec",
    94,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, partition_bound_spec),
    &pg_query__partition_bound_spec__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "partition_range_datum",
    95,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, partition_range_datum),
    &pg_query__partition_range_datum__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "single_partition_spec",
    96,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, single_partition_spec),
    &pg_query__single_partition_spec__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "partition_cmd",
    97,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, partition_cmd),
    &pg_query__partition_cmd__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "range_tbl_entry",
    98,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, range_tbl_entry),
    &pg_query__range_tbl_entry__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "rtepermission_info",
    99,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, rtepermission_info),
    &pg_query__rtepermission_info__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "range_tbl_function",
    100,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, range_tbl_function),
    &pg_query__range_tbl_function__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "table_sample_clause",
    101,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, table_sample_clause),
    &pg_query__table_sample_clause__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "with_check_option",
    102,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, with_check_option),
    &pg_query__with_check_option__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "sort_group_clause",
    103,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, sort_group_clause),
    &pg_query__sort_group_clause__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "grouping_set",
    104,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, grouping_set),
    &pg_query__grouping_set__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "window_clause",
    105,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, window_clause),
    &pg_query__window_clause__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "row_mark_clause",
    106,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, row_mark_clause),
    &pg_query__row_mark_clause__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "with_clause",
    107,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, with_clause),
    &pg_query__with_clause__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "infer_clause",
    108,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, infer_clause),
    &pg_query__infer_clause__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "on_conflict_clause",
    109,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, on_conflict_clause),
    &pg_query__on_conflict_clause__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "ctesearch_clause",
    110,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, ctesearch_clause),
    &pg_query__ctesearch_clause__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "ctecycle_clause",
    111,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, ctecycle_clause),
    &pg_query__ctecycle_clause__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "common_table_expr",
    112,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, common_table_expr),
    &pg_query__common_table_expr__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "merge_when_clause",
    113,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, merge_when_clause),
    &pg_query__merge_when_clause__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "trigger_transition",
    114,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, trigger_transition),
    &pg_query__trigger_transition__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "json_output",
    115,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, json_output),
    &pg_query__json_output__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "json_argument",
    116,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, json_argument),
    &pg_query__json_argument__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "json_func_expr",
    117,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, json_func_expr),
    &pg_query__json_func_expr__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "json_table_path_spec",
    118,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, json_table_path_spec),
    &pg_query__json_table_path_spec__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "json_table",
    119,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, json_table),
    &pg_query__json_table__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "json_table_column",
    120,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, json_table_column),
    &pg_query__json_table_column__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "json_key_value",
    121,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, json_key_value),
    &pg_query__json_key_value__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "json_parse_expr",
    122,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, json_parse_expr),
    &pg_query__json_parse_expr__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "json_scalar_expr",
    123,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, json_scalar_expr),
    &pg_query__json_scalar_expr__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "json_serialize_expr",
    124,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, json_serialize_expr),
    &pg_query__json_serialize_expr__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "json_object_constructor",
    125,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, json_object_constructor),
    &pg_query__json_object_constructor__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "json_array_constructor",
    126,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, json_array_constructor),
    &pg_query__json_array_constructor__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "json_array_query_constructor",
    127,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, json_array_query_constructor),
    &pg_query__json_array_query_constructor__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "json_agg_constructor",
    128,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, json_agg_constructor),
    &pg_query__json_agg_constructor__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "json_object_agg",
    129,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, json_object_agg),
    &pg_query__json_object_agg__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "json_array_agg",
    130,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, json_array_agg),
    &pg_query__json_array_agg__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "raw_stmt",
    131,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, raw_stmt),
    &pg_query__raw_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "insert_stmt",
    132,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, insert_stmt),
    &pg_query__insert_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "delete_stmt",
    133,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, delete_stmt),
    &pg_query__delete_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "update_stmt",
    134,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, update_stmt),
    &pg_query__update_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "merge_stmt",
    135,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, merge_stmt),
    &pg_query__merge_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "select_stmt",
    136,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, select_stmt),
    &pg_query__select_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "set_operation_stmt",
    137,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, set_operation_stmt),
    &pg_query__set_operation_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "return_stmt",
    138,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, return_stmt),
    &pg_query__return_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "plassign_stmt",
    139,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, plassign_stmt),
    &pg_query__plassign_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "create_schema_stmt",
    140,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, create_schema_stmt),
    &pg_query__create_schema_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alter_table_stmt",
    141,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, alter_table_stmt),
    &pg_query__alter_table_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "replica_identity_stmt",
    142,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, replica_identity_stmt),
    &pg_query__replica_identity_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alter_table_cmd",
    143,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, alter_table_cmd),
    &pg_query__alter_table_cmd__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alter_collation_stmt",
    144,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, alter_collation_stmt),
    &pg_query__alter_collation_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alter_domain_stmt",
    145,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, alter_domain_stmt),
    &pg_query__alter_domain_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "grant_stmt",
    146,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, grant_stmt),
    &pg_query__grant_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "object_with_args",
    147,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, object_with_args),
    &pg_query__object_with_args__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "access_priv",
    148,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, access_priv),
    &pg_query__access_priv__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "grant_role_stmt",
    149,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, grant_role_stmt),
    &pg_query__grant_role_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alter_default_privileges_stmt",
    150,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, alter_default_privileges_stmt),
    &pg_query__alter_default_privileges_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "copy_stmt",
    151,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, copy_stmt),
    &pg_query__copy_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "variable_set_stmt",
    152,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, variable_set_stmt),
    &pg_query__variable_set_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "variable_show_stmt",
    153,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, variable_show_stmt),
    &pg_query__variable_show_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "create_stmt",
    154,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, create_stmt),
    &pg_query__create_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "constraint",
    155,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, constraint),
    &pg_query__constraint__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "create_table_space_stmt",
    156,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, create_table_space_stmt),
    &pg_query__create_table_space_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "drop_table_space_stmt",
    157,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, drop_table_space_stmt),
    &pg_query__drop_table_space_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alter_table_space_options_stmt",
    158,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, alter_table_space_options_stmt),
    &pg_query__alter_table_space_options_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alter_table_move_all_stmt",
    159,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, alter_table_move_all_stmt),
    &pg_query__alter_table_move_all_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "create_extension_stmt",
    160,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, create_extension_stmt),
    &pg_query__create_extension_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alter_extension_stmt",
    161,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, alter_extension_stmt),
    &pg_query__alter_extension_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alter_extension_contents_stmt",
    162,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, alter_extension_contents_stmt),
    &pg_query__alter_extension_contents_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "create_fdw_stmt",
    163,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, create_fdw_stmt),
    &pg_query__create_fdw_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alter_fdw_stmt",
    164,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, alter_fdw_stmt),
    &pg_query__alter_fdw_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "create_foreign_server_stmt",
    165,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, create_foreign_server_stmt),
    &pg_query__create_foreign_server_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alter_foreign_server_stmt",
    166,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, alter_foreign_server_stmt),
    &pg_query__alter_foreign_server_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "create_foreign_table_stmt",
    167,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, create_foreign_table_stmt),
    &pg_query__create_foreign_table_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "create_user_mapping_stmt",
    168,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, create_user_mapping_stmt),
    &pg_query__create_user_mapping_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alter_user_mapping_stmt",
    169,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, alter_user_mapping_stmt),
    &pg_query__alter_user_mapping_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "drop_user_mapping_stmt",
    170,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, drop_user_mapping_stmt),
    &pg_query__drop_user_mapping_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "import_foreign_schema_stmt",
    171,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, import_foreign_schema_stmt),
    &pg_query__import_foreign_schema_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "create_policy_stmt",
    172,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, create_policy_stmt),
    &pg_query__create_policy_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alter_policy_stmt",
    173,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, alter_policy_stmt),
    &pg_query__alter_policy_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "create_am_stmt",
    174,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, create_am_stmt),
    &pg_query__create_am_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "create_trig_stmt",
    175,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, create_trig_stmt),
    &pg_query__create_trig_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "create_event_trig_stmt",
    176,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, create_event_trig_stmt),
    &pg_query__create_event_trig_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alter_event_trig_stmt",
    177,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, alter_event_trig_stmt),
    &pg_query__alter_event_trig_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "create_plang_stmt",
    178,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, create_plang_stmt),
    &pg_query__create_plang_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "create_role_stmt",
    179,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, create_role_stmt),
    &pg_query__create_role_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alter_role_stmt",
    180,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, alter_role_stmt),
    &pg_query__alter_role_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alter_role_set_stmt",
    181,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, alter_role_set_stmt),
    &pg_query__alter_role_set_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "drop_role_stmt",
    182,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, drop_role_stmt),
    &pg_query__drop_role_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "create_seq_stmt",
    183,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, create_seq_stmt),
    &pg_query__create_seq_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alter_seq_stmt",
    184,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, alter_seq_stmt),
    &pg_query__alter_seq_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "define_stmt",
    185,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, define_stmt),
    &pg_query__define_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "create_domain_stmt",
    186,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, create_domain_stmt),
    &pg_query__create_domain_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "create_op_class_stmt",
    187,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, create_op_class_stmt),
    &pg_query__create_op_class_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "create_op_class_item",
    188,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, create_op_class_item),
    &pg_query__create_op_class_item__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "create_op_family_stmt",
    189,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, create_op_family_stmt),
    &pg_query__create_op_family_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alter_op_family_stmt",
    190,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, alter_op_family_stmt),
    &pg_query__alter_op_family_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "drop_stmt",
    191,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, drop_stmt),
    &pg_query__drop_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "truncate_stmt",
    192,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, truncate_stmt),
    &pg_query__truncate_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "comment_stmt",
    193,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, comment_stmt),
    &pg_query__comment_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "sec_label_stmt",
    194,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, sec_label_stmt),
    &pg_query__sec_label_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "declare_cursor_stmt",
    195,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, declare_cursor_stmt),
    &pg_query__declare_cursor_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "close_portal_stmt",
    196,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, close_portal_stmt),
    &pg_query__close_portal_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "fetch_stmt",
    197,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, fetch_stmt),
    &pg_query__fetch_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "index_stmt",
    198,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, index_stmt),
    &pg_query__index_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "create_stats_stmt",
    199,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, create_stats_stmt),
    &pg_query__create_stats_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "stats_elem",
    200,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, stats_elem),
    &pg_query__stats_elem__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alter_stats_stmt",
    201,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, alter_stats_stmt),
    &pg_query__alter_stats_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "create_function_stmt",
    202,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, create_function_stmt),
    &pg_query__create_function_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "function_parameter",
    203,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, function_parameter),
    &pg_query__function_parameter__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alter_function_stmt",
    204,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, alter_function_stmt),
    &pg_query__alter_function_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "do_stmt",
    205,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, do_stmt),
    &pg_query__do_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "inline_code_block",
    206,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, inline_code_block),
    &pg_query__inline_code_block__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "call_stmt",
    207,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, call_stmt),
    &pg_query__call_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "call_context",
    208,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, call_context),
    &pg_query__call_context__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "rename_stmt",
    209,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, rename_stmt),
    &pg_query__rename_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alter_object_depends_stmt",
    210,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, alter_object_depends_stmt),
    &pg_query__alter_object_depends_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alter_object_schema_stmt",
    211,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, alter_object_schema_stmt),
    &pg_query__alter_object_schema_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alter_owner_stmt",
    212,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, alter_owner_stmt),
    &pg_query__alter_owner_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alter_operator_stmt",
    213,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, alter_operator_stmt),
    &pg_query__alter_operator_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alter_type_stmt",
    214,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, alter_type_stmt),
    &pg_query__alter_type_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "rule_stmt",
    215,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, rule_stmt),
    &pg_query__rule_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "notify_stmt",
    216,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, notify_stmt),
    &pg_query__notify_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "listen_stmt",
    217,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, listen_stmt),
    &pg_query__listen_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "unlisten_stmt",
    218,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, unlisten_stmt),
    &pg_query__unlisten_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "transaction_stmt",
    219,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, transaction_stmt),
    &pg_query__transaction_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "composite_type_stmt",
    220,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, composite_type_stmt),
    &pg_query__composite_type_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "create_enum_stmt",
    221,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, create_enum_stmt),
    &pg_query__create_enum_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "create_range_stmt",
    222,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, create_range_stmt),
    &pg_query__create_range_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alter_enum_stmt",
    223,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, alter_enum_stmt),
    &pg_query__alter_enum_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "view_stmt",
    224,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, view_stmt),
    &pg_query__view_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "load_stmt",
    225,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, load_stmt),
    &pg_query__load_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "createdb_stmt",
    226,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, createdb_stmt),
    &pg_query__createdb_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alter_database_stmt",
    227,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, alter_database_stmt),
    &pg_query__alter_database_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alter_database_refresh_coll_stmt",
    228,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, alter_database_refresh_coll_stmt),
    &pg_query__alter_database_refresh_coll_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alter_database_set_stmt",
    229,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, alter_database_set_stmt),
    &pg_query__alter_database_set_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "dropdb_stmt",
    230,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, dropdb_stmt),
    &pg_query__dropdb_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alter_system_stmt",
    231,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, alter_system_stmt),
    &pg_query__alter_system_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "cluster_stmt",
    232,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, cluster_stmt),
    &pg_query__cluster_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "vacuum_stmt",
    233,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, vacuum_stmt),
    &pg_query__vacuum_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "vacuum_relation",
    234,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, vacuum_relation),
    &pg_query__vacuum_relation__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "explain_stmt",
    235,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, explain_stmt),
    &pg_query__explain_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "create_table_as_stmt",
    236,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, create_table_as_stmt),
    &pg_query__create_table_as_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "refresh_mat_view_stmt",
    237,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, refresh_mat_view_stmt),
    &pg_query__refresh_mat_view_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "check_point_stmt",
    238,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, check_point_stmt),
    &pg_query__check_point_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "discard_stmt",
    239,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, discard_stmt),
    &pg_query__discard_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "lock_stmt",
    240,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, lock_stmt),
    &pg_query__lock_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "constraints_set_stmt",
    241,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, constraints_set_stmt),
    &pg_query__constraints_set_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "reindex_stmt",
    242,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, reindex_stmt),
    &pg_query__reindex_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "create_conversion_stmt",
    243,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, create_conversion_stmt),
    &pg_query__create_conversion_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "create_cast_stmt",
    244,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, create_cast_stmt),
    &pg_query__create_cast_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "create_transform_stmt",
    245,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, create_transform_stmt),
    &pg_query__create_transform_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "prepare_stmt",
    246,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, prepare_stmt),
    &pg_query__prepare_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "execute_stmt",
    247,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, execute_stmt),
    &pg_query__execute_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "deallocate_stmt",
    248,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, deallocate_stmt),
    &pg_query__deallocate_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "drop_owned_stmt",
    249,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, drop_owned_stmt),
    &pg_query__drop_owned_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "reassign_owned_stmt",
    250,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, reassign_owned_stmt),
    &pg_query__reassign_owned_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alter_tsdictionary_stmt",
    251,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, alter_tsdictionary_stmt),
    &pg_query__alter_tsdictionary_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alter_tsconfiguration_stmt",
    252,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, alter_tsconfiguration_stmt),
    &pg_query__alter_tsconfiguration_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "publication_table",
    253,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, publication_table),
    &pg_query__publication_table__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "publication_obj_spec",
    254,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, publication_obj_spec),
    &pg_query__publication_obj_spec__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "create_publication_stmt",
    255,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, create_publication_stmt),
    &pg_query__create_publication_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alter_publication_stmt",
    256,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, alter_publication_stmt),
    &pg_query__alter_publication_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "create_subscription_stmt",
    257,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, create_subscription_stmt),
    &pg_query__create_subscription_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alter_subscription_stmt",
    258,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, alter_subscription_stmt),
    &pg_query__alter_subscription_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "drop_subscription_stmt",
    259,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, drop_subscription_stmt),
    &pg_query__drop_subscription_stmt__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "integer",
    260,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, integer),
    &pg_query__integer__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "float",
    261,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, float_),
    &pg_query__float__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "boolean",
    262,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, boolean),
    &pg_query__boolean__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "string",
    263,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, string),
    &pg_query__string__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "bit_string",
    264,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, bit_string),
    &pg_query__bit_string__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "list",
    265,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, list),
    &pg_query__list__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "int_list",
    266,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, int_list),
    &pg_query__int_list__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "oid_list",
    267,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, oid_list),
    &pg_query__oid_list__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "a_const",
    268,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Node, node_case),
    offsetof(PgQuery__Node, a_const),
    &pg_query__a__const__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__node__field_indices_by_name[] = {
  75,   /* field[75] = a_array_expr */
  267,   /* field[267] = a_const */
  67,   /* field[67] = a_expr */
  73,   /* field[73] = a_indices */
  74,   /* field[74] = a_indirection */
  72,   /* field[72] = a_star */
  147,   /* field[147] = access_priv */
  6,   /* field[6] = aggref */
  0,   /* field[0] = alias */
  143,   /* field[143] = alter_collation_stmt */
  227,   /* field[227] = alter_database_refresh_coll_stmt */
  228,   /* field[228] = alter_database_set_stmt */
  226,   /* field[226] = alter_database_stmt */
  149,   /* field[149] = alter_default_privileges_stmt */
  144,   /* field[144] = alter_domain_stmt */
  222,   /* field[222] = alter_enum_stmt */
  176,   /* field[176] = alter_event_trig_stmt */
  161,   /* field[161] = alter_extension_contents_stmt */
  160,   /* field[160] = alter_extension_stmt */
  163,   /* field[163] = alter_fdw_stmt */
  165,   /* field[165] = alter_foreign_server_stmt */
  203,   /* field[203] = alter_function_stmt */
  209,   /* field[209] = alter_object_depends_stmt */
  210,   /* field[210] = alter_object_schema_stmt */
  189,   /* field[189] = alter_op_family_stmt */
  212,   /* field[212] = alter_operator_stmt */
  211,   /* field[211] = alter_owner_stmt */
  172,   /* field[172] = alter_policy_stmt */
  255,   /* field[255] = alter_publication_stmt */
  180,   /* field[180] = alter_role_set_stmt */
  179,   /* field[179] = alter_role_stmt */
  183,   /* field[183] = alter_seq_stmt */
  200,   /* field[200] = alter_stats_stmt */
  257,   /* field[257] = alter_subscription_stmt */
  230,   /* field[230] = alter_system_stmt */
  142,   /* field[142] = alter_table_cmd */
  158,   /* field[158] = alter_table_move_all_stmt */
  157,   /* field[157] = alter_table_space_options_stmt */
  140,   /* field[140] = alter_table_stmt */
  251,   /* field[251] = alter_tsconfiguration_stmt */
  250,   /* field[250] = alter_tsdictionary_stmt */
  213,   /* field[213] = alter_type_stmt */
  168,   /* field[168] = alter_user_mapping_stmt */
  21,   /* field[21] = alternative_sub_plan */
  26,   /* field[26] = array_coerce_expr */
  32,   /* field[32] = array_expr */
  263,   /* field[263] = bit_string */
  18,   /* field[18] = bool_expr */
  261,   /* field[261] = boolean */
  50,   /* field[50] = boolean_test */
  207,   /* field[207] = call_context */
  206,   /* field[206] = call_stmt */
  29,   /* field[29] = case_expr */
  31,   /* field[31] = case_test_expr */
  30,   /* field[30] = case_when */
  237,   /* field[237] = check_point_stmt */
  195,   /* field[195] = close_portal_stmt */
  231,   /* field[231] = cluster_stmt */
  35,   /* field[35] = coalesce_expr */
  52,   /* field[52] = coerce_to_domain */
  53,   /* field[53] = coerce_to_domain_value */
  25,   /* field[25] = coerce_via_io */
  69,   /* field[69] = collate_clause */
  28,   /* field[28] = collate_expr */
  85,   /* field[85] = column_def */
  65,   /* field[65] = column_ref */
  192,   /* field[192] = comment_stmt */
  111,   /* field[111] = common_table_expr */
  219,   /* field[219] = composite_type_stmt */
  154,   /* field[154] = constraint */
  240,   /* field[240] = constraints_set_stmt */
  27,   /* field[27] = convert_rowtype_expr */
  150,   /* field[150] = copy_stmt */
  173,   /* field[173] = create_am_stmt */
  243,   /* field[243] = create_cast_stmt */
  242,   /* field[242] = create_conversion_stmt */
  185,   /* field[185] = create_domain_stmt */
  220,   /* field[220] = create_enum_stmt */
  175,   /* field[175] = create_event_trig_stmt */
  159,   /* field[159] = create_extension_stmt */
  162,   /* field[162] = create_fdw_stmt */
  164,   /* field[164] = create_foreign_server_stmt */
  166,   /* field[166] = create_foreign_table_stmt */
  201,   /* field[201] = create_function_stmt */
  187,   /* field[187] = create_op_class_item */
  186,   /* field[186] = create_op_class_stmt */
  188,   /* field[188] = create_op_family_stmt */
  177,   /* field[177] = create_plang_stmt */
  171,   /* field[171] = create_policy_stmt */
  254,   /* field[254] = create_publication_stmt */
  221,   /* field[221] = create_range_stmt */
  178,   /* field[178] = create_role_stmt */
  139,   /* field[139] = create_schema_stmt */
  182,   /* field[182] = create_seq_stmt */
  198,   /* field[198] = create_stats_stmt */
  153,   /* field[153] = create_stmt */
  256,   /* field[256] = create_subscription_stmt */
  235,   /* field[235] = create_table_as_stmt */
  155,   /* field[155] = create_table_space_stmt */
  244,   /* field[244] = create_transform_stmt */
  174,   /* field[174] = create_trig_stmt */
  167,   /* field[167] = create_user_mapping_stmt */
  225,   /* field[225] = createdb_stmt */
  110,   /* field[110] = ctecycle_clause */
  109,   /* field[109] = ctesearch_clause */
  55,   /* field[55] = current_of_expr */
  247,   /* field[247] = deallocate_stmt */
  194,   /* field[194] = declare_cursor_stmt */
  88,   /* field[88] = def_elem */
  184,   /* field[184] = define_stmt */
  132,   /* field[132] = delete_stmt */
  238,   /* field[238] = discard_stmt */
  15,   /* field[15] = distinct_expr */
  204,   /* field[204] = do_stmt */
  248,   /* field[248] = drop_owned_stmt */
  181,   /* field[181] = drop_role_stmt */
  190,   /* field[190] = drop_stmt */
  258,   /* field[258] = drop_subscription_stmt */
  156,   /* field[156] = drop_table_space_stmt */
  169,   /* field[169] = drop_user_mapping_stmt */
  229,   /* field[229] = dropdb_stmt */
  246,   /* field[246] = execute_stmt */
  234,   /* field[234] = explain_stmt */
  196,   /* field[196] = fetch_stmt */
  22,   /* field[22] = field_select */
  23,   /* field[23] = field_store */
  260,   /* field[260] = float */
  61,   /* field[61] = from_expr */
  71,   /* field[71] = func_call */
  12,   /* field[12] = func_expr */
  202,   /* field[202] = function_parameter */
  148,   /* field[148] = grant_role_stmt */
  145,   /* field[145] = grant_stmt */
  7,   /* field[7] = grouping_func */
  103,   /* field[103] = grouping_set */
  170,   /* field[170] = import_foreign_schema_stmt */
  87,   /* field[87] = index_elem */
  197,   /* field[197] = index_stmt */
  107,   /* field[107] = infer_clause */
  57,   /* field[57] = inference_elem */
  205,   /* field[205] = inline_code_block */
  131,   /* field[131] = insert_stmt */
  265,   /* field[265] = int_list */
  259,   /* field[259] = integer */
  3,   /* field[3] = into_clause */
  60,   /* field[60] = join_expr */
  127,   /* field[127] = json_agg_constructor */
  115,   /* field[115] = json_argument */
  129,   /* field[129] = json_array_agg */
  125,   /* field[125] = json_array_constructor */
  126,   /* field[126] = json_array_query_constructor */
  44,   /* field[44] = json_behavior */
  42,   /* field[42] = json_constructor_expr */
  45,   /* field[45] = json_expr */
  39,   /* field[39] = json_format */
  116,   /* field[116] = json_func_expr */
  43,   /* field[43] = json_is_predicate */
  120,   /* field[120] = json_key_value */
  128,   /* field[128] = json_object_agg */
  124,   /* field[124] = json_object_constructor */
  114,   /* field[114] = json_output */
  121,   /* field[121] = json_parse_expr */
  40,   /* field[40] = json_returning */
  122,   /* field[122] = json_scalar_expr */
  123,   /* field[123] = json_serialize_expr */
  118,   /* field[118] = json_table */
  119,   /* field[119] = json_table_column */
  46,   /* field[46] = json_table_path */
  47,   /* field[47] = json_table_path_scan */
  117,   /* field[117] = json_table_path_spec */
  48,   /* field[48] = json_table_sibling_join */
  41,   /* field[41] = json_value_expr */
  264,   /* field[264] = list */
  216,   /* field[216] = listen_stmt */
  224,   /* field[224] = load_stmt */
  239,   /* field[239] = lock_stmt */
  89,   /* field[89] = locking_clause */
  51,   /* field[51] = merge_action */
  134,   /* field[134] = merge_stmt */
  10,   /* field[10] = merge_support_func */
  112,   /* field[112] = merge_when_clause */
  36,   /* field[36] = min_max_expr */
  77,   /* field[77] = multi_assign_ref */
  13,   /* field[13] = named_arg_expr */
  56,   /* field[56] = next_value_expr */
  215,   /* field[215] = notify_stmt */
  16,   /* field[16] = null_if_expr */
  49,   /* field[49] = null_test */
  146,   /* field[146] = object_with_args */
  266,   /* field[266] = oid_list */
  108,   /* field[108] = on_conflict_clause */
  62,   /* field[62] = on_conflict_expr */
  14,   /* field[14] = op_expr */
  5,   /* field[5] = param */
  66,   /* field[66] = param_ref */
  93,   /* field[93] = partition_bound_spec */
  96,   /* field[96] = partition_cmd */
  91,   /* field[91] = partition_elem */
  94,   /* field[94] = partition_range_datum */
  92,   /* field[92] = partition_spec */
  138,   /* field[138] = plassign_stmt */
  245,   /* field[245] = prepare_stmt */
  253,   /* field[253] = publication_obj_spec */
  252,   /* field[252] = publication_table */
  63,   /* field[63] = query */
  81,   /* field[81] = range_function */
  80,   /* field[80] = range_subselect */
  82,   /* field[82] = range_table_func */
  83,   /* field[83] = range_table_func_col */
  84,   /* field[84] = range_table_sample */
  97,   /* field[97] = range_tbl_entry */
  99,   /* field[99] = range_tbl_function */
  59,   /* field[59] = range_tbl_ref */
  1,   /* field[1] = range_var */
  130,   /* field[130] = raw_stmt */
  249,   /* field[249] = reassign_owned_stmt */
  236,   /* field[236] = refresh_mat_view_stmt */
  241,   /* field[241] = reindex_stmt */
  24,   /* field[24] = relabel_type */
  208,   /* field[208] = rename_stmt */
  141,   /* field[141] = replica_identity_stmt */
  76,   /* field[76] = res_target */
  137,   /* field[137] = return_stmt */
  70,   /* field[70] = role_spec */
  34,   /* field[34] = row_compare_expr */
  33,   /* field[33] = row_expr */
  105,   /* field[105] = row_mark_clause */
  98,   /* field[98] = rtepermission_info */
  214,   /* field[214] = rule_stmt */
  17,   /* field[17] = scalar_array_op_expr */
  193,   /* field[193] = sec_label_stmt */
  135,   /* field[135] = select_stmt */
  136,   /* field[136] = set_operation_stmt */
  54,   /* field[54] = set_to_default */
  95,   /* field[95] = single_partition_spec */
  78,   /* field[78] = sort_by */
  102,   /* field[102] = sort_group_clause */
  37,   /* field[37] = sqlvalue_function */
  199,   /* field[199] = stats_elem */
  262,   /* field[262] = string */
  19,   /* field[19] = sub_link */
  20,   /* field[20] = sub_plan */
  11,   /* field[11] = subscripting_ref */
  2,   /* field[2] = table_func */
  86,   /* field[86] = table_like_clause */
  100,   /* field[100] = table_sample_clause */
  58,   /* field[58] = target_entry */
  218,   /* field[218] = transaction_stmt */
  113,   /* field[113] = trigger_transition */
  191,   /* field[191] = truncate_stmt */
  68,   /* field[68] = type_cast */
  64,   /* field[64] = type_name */
  217,   /* field[217] = unlisten_stmt */
  133,   /* field[133] = update_stmt */
  233,   /* field[233] = vacuum_relation */
  232,   /* field[232] = vacuum_stmt */
  4,   /* field[4] = var */
  151,   /* field[151] = variable_set_stmt */
  152,   /* field[152] = variable_show_stmt */
  223,   /* field[223] = view_stmt */
  104,   /* field[104] = window_clause */
  79,   /* field[79] = window_def */
  8,   /* field[8] = window_func */
  9,   /* field[9] = window_func_run_condition */
  101,   /* field[101] = with_check_option */
  106,   /* field[106] = with_clause */
  38,   /* field[38] = xml_expr */
  90,   /* field[90] = xml_serialize */
};
static const ProtobufCIntRange pg_query__node__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 268 }
};
const ProtobufCMessageDescriptor pg_query__node__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.Node",
  "Node",
  "PgQuery__Node",
  "pg_query",
  sizeof(PgQuery__Node),
  268,
  pg_query__node__field_descriptors,
  pg_query__node__field_indices_by_name,
  1,  pg_query__node__number_ranges,
  (ProtobufCMessageInit) pg_query__node__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__integer__field_descriptors[1] =
{
  {
    "ival",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Integer, ival),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__integer__field_indices_by_name[] = {
  0,   /* field[0] = ival */
};
static const ProtobufCIntRange pg_query__integer__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 1 }
};
const ProtobufCMessageDescriptor pg_query__integer__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.Integer",
  "Integer",
  "PgQuery__Integer",
  "pg_query",
  sizeof(PgQuery__Integer),
  1,
  pg_query__integer__field_descriptors,
  pg_query__integer__field_indices_by_name,
  1,  pg_query__integer__number_ranges,
  (ProtobufCMessageInit) pg_query__integer__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__float__field_descriptors[1] =
{
  {
    "fval",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Float, fval),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__float__field_indices_by_name[] = {
  0,   /* field[0] = fval */
};
static const ProtobufCIntRange pg_query__float__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 1 }
};
const ProtobufCMessageDescriptor pg_query__float__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.Float",
  "Float",
  "PgQuery__Float",
  "pg_query",
  sizeof(PgQuery__Float),
  1,
  pg_query__float__field_descriptors,
  pg_query__float__field_indices_by_name,
  1,  pg_query__float__number_ranges,
  (ProtobufCMessageInit) pg_query__float__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__boolean__field_descriptors[1] =
{
  {
    "boolval",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Boolean, boolval),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__boolean__field_indices_by_name[] = {
  0,   /* field[0] = boolval */
};
static const ProtobufCIntRange pg_query__boolean__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 1 }
};
const ProtobufCMessageDescriptor pg_query__boolean__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.Boolean",
  "Boolean",
  "PgQuery__Boolean",
  "pg_query",
  sizeof(PgQuery__Boolean),
  1,
  pg_query__boolean__field_descriptors,
  pg_query__boolean__field_indices_by_name,
  1,  pg_query__boolean__number_ranges,
  (ProtobufCMessageInit) pg_query__boolean__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__string__field_descriptors[1] =
{
  {
    "sval",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__String, sval),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__string__field_indices_by_name[] = {
  0,   /* field[0] = sval */
};
static const ProtobufCIntRange pg_query__string__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 1 }
};
const ProtobufCMessageDescriptor pg_query__string__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.String",
  "String",
  "PgQuery__String",
  "pg_query",
  sizeof(PgQuery__String),
  1,
  pg_query__string__field_descriptors,
  pg_query__string__field_indices_by_name,
  1,  pg_query__string__number_ranges,
  (ProtobufCMessageInit) pg_query__string__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__bit_string__field_descriptors[1] =
{
  {
    "bsval",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__BitString, bsval),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__bit_string__field_indices_by_name[] = {
  0,   /* field[0] = bsval */
};
static const ProtobufCIntRange pg_query__bit_string__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 1 }
};
const ProtobufCMessageDescriptor pg_query__bit_string__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.BitString",
  "BitString",
  "PgQuery__BitString",
  "pg_query",
  sizeof(PgQuery__BitString),
  1,
  pg_query__bit_string__field_descriptors,
  pg_query__bit_string__field_indices_by_name,
  1,  pg_query__bit_string__number_ranges,
  (ProtobufCMessageInit) pg_query__bit_string__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__list__field_descriptors[1] =
{
  {
    "items",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__List, n_items),
    offsetof(PgQuery__List, items),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__list__field_indices_by_name[] = {
  0,   /* field[0] = items */
};
static const ProtobufCIntRange pg_query__list__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 1 }
};
const ProtobufCMessageDescriptor pg_query__list__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.List",
  "List",
  "PgQuery__List",
  "pg_query",
  sizeof(PgQuery__List),
  1,
  pg_query__list__field_descriptors,
  pg_query__list__field_indices_by_name,
  1,  pg_query__list__number_ranges,
  (ProtobufCMessageInit) pg_query__list__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__oid_list__field_descriptors[1] =
{
  {
    "items",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__OidList, n_items),
    offsetof(PgQuery__OidList, items),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__oid_list__field_indices_by_name[] = {
  0,   /* field[0] = items */
};
static const ProtobufCIntRange pg_query__oid_list__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 1 }
};
const ProtobufCMessageDescriptor pg_query__oid_list__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.OidList",
  "OidList",
  "PgQuery__OidList",
  "pg_query",
  sizeof(PgQuery__OidList),
  1,
  pg_query__oid_list__field_descriptors,
  pg_query__oid_list__field_indices_by_name,
  1,  pg_query__oid_list__number_ranges,
  (ProtobufCMessageInit) pg_query__oid_list__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__int_list__field_descriptors[1] =
{
  {
    "items",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__IntList, n_items),
    offsetof(PgQuery__IntList, items),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__int_list__field_indices_by_name[] = {
  0,   /* field[0] = items */
};
static const ProtobufCIntRange pg_query__int_list__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 1 }
};
const ProtobufCMessageDescriptor pg_query__int_list__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.IntList",
  "IntList",
  "PgQuery__IntList",
  "pg_query",
  sizeof(PgQuery__IntList),
  1,
  pg_query__int_list__field_descriptors,
  pg_query__int_list__field_indices_by_name,
  1,  pg_query__int_list__number_ranges,
  (ProtobufCMessageInit) pg_query__int_list__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__a__const__field_descriptors[7] =
{
  {
    "ival",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AConst, val_case),
    offsetof(PgQuery__AConst, ival),
    &pg_query__integer__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "fval",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AConst, val_case),
    offsetof(PgQuery__AConst, fval),
    &pg_query__float__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "boolval",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AConst, val_case),
    offsetof(PgQuery__AConst, boolval),
    &pg_query__boolean__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "sval",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AConst, val_case),
    offsetof(PgQuery__AConst, sval),
    &pg_query__string__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "bsval",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AConst, val_case),
    offsetof(PgQuery__AConst, bsval),
    &pg_query__bit_string__descriptor,
    NULL,
    PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "isnull",
    10,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AConst, isnull),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    11,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AConst, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__a__const__field_indices_by_name[] = {
  2,   /* field[2] = boolval */
  4,   /* field[4] = bsval */
  1,   /* field[1] = fval */
  5,   /* field[5] = isnull */
  0,   /* field[0] = ival */
  6,   /* field[6] = location */
  3,   /* field[3] = sval */
};
static const ProtobufCIntRange pg_query__a__const__number_ranges[2 + 1] =
{
  { 1, 0 },
  { 10, 5 },
  { 0, 7 }
};
const ProtobufCMessageDescriptor pg_query__a__const__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.A_Const",
  "AConst",
  "PgQuery__AConst",
  "pg_query",
  sizeof(PgQuery__AConst),
  7,
  pg_query__a__const__field_descriptors,
  pg_query__a__const__field_indices_by_name,
  2,  pg_query__a__const__number_ranges,
  (ProtobufCMessageInit) pg_query__a__const__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__alias__field_descriptors[2] =
{
  {
    "aliasname",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Alias, aliasname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "colnames",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Alias, n_colnames),
    offsetof(PgQuery__Alias, colnames),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__alias__field_indices_by_name[] = {
  0,   /* field[0] = aliasname */
  1,   /* field[1] = colnames */
};
static const ProtobufCIntRange pg_query__alias__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor pg_query__alias__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.Alias",
  "Alias",
  "PgQuery__Alias",
  "pg_query",
  sizeof(PgQuery__Alias),
  2,
  pg_query__alias__field_descriptors,
  pg_query__alias__field_indices_by_name,
  1,  pg_query__alias__number_ranges,
  (ProtobufCMessageInit) pg_query__alias__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__range_var__field_descriptors[7] =
{
  {
    "catalogname",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeVar, catalogname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "schemaname",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeVar, schemaname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "relname",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeVar, relname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "inh",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeVar, inh),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "relpersistence",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeVar, relpersistence),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alias",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeVar, alias),
    &pg_query__alias__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeVar, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__range_var__field_indices_by_name[] = {
  5,   /* field[5] = alias */
  0,   /* field[0] = catalogname */
  3,   /* field[3] = inh */
  6,   /* field[6] = location */
  2,   /* field[2] = relname */
  4,   /* field[4] = relpersistence */
  1,   /* field[1] = schemaname */
};
static const ProtobufCIntRange pg_query__range_var__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 7 }
};
const ProtobufCMessageDescriptor pg_query__range_var__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.RangeVar",
  "RangeVar",
  "PgQuery__RangeVar",
  "pg_query",
  sizeof(PgQuery__RangeVar),
  7,
  pg_query__range_var__field_descriptors,
  pg_query__range_var__field_indices_by_name,
  1,  pg_query__range_var__number_ranges,
  (ProtobufCMessageInit) pg_query__range_var__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__table_func__field_descriptors[17] =
{
  {
    "functype",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__TableFunc, functype),
    &pg_query__table_func_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "ns_uris",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__TableFunc, n_ns_uris),
    offsetof(PgQuery__TableFunc, ns_uris),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "ns_names",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__TableFunc, n_ns_names),
    offsetof(PgQuery__TableFunc, ns_names),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "docexpr",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__TableFunc, docexpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "rowexpr",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__TableFunc, rowexpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "colnames",
    6,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__TableFunc, n_colnames),
    offsetof(PgQuery__TableFunc, colnames),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "coltypes",
    7,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__TableFunc, n_coltypes),
    offsetof(PgQuery__TableFunc, coltypes),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "coltypmods",
    8,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__TableFunc, n_coltypmods),
    offsetof(PgQuery__TableFunc, coltypmods),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "colcollations",
    9,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__TableFunc, n_colcollations),
    offsetof(PgQuery__TableFunc, colcollations),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "colexprs",
    10,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__TableFunc, n_colexprs),
    offsetof(PgQuery__TableFunc, colexprs),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "coldefexprs",
    11,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__TableFunc, n_coldefexprs),
    offsetof(PgQuery__TableFunc, coldefexprs),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "colvalexprs",
    12,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__TableFunc, n_colvalexprs),
    offsetof(PgQuery__TableFunc, colvalexprs),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "passingvalexprs",
    13,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__TableFunc, n_passingvalexprs),
    offsetof(PgQuery__TableFunc, passingvalexprs),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "notnulls",
    14,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_UINT64,
    offsetof(PgQuery__TableFunc, n_notnulls),
    offsetof(PgQuery__TableFunc, notnulls),
    NULL,
    NULL,
    PROTOBUF_C_FIELD_FLAG_PACKED,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "plan",
    15,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__TableFunc, plan),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "ordinalitycol",
    16,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__TableFunc, ordinalitycol),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    17,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__TableFunc, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__table_func__field_indices_by_name[] = {
  8,   /* field[8] = colcollations */
  10,   /* field[10] = coldefexprs */
  9,   /* field[9] = colexprs */
  5,   /* field[5] = colnames */
  6,   /* field[6] = coltypes */
  7,   /* field[7] = coltypmods */
  11,   /* field[11] = colvalexprs */
  3,   /* field[3] = docexpr */
  0,   /* field[0] = functype */
  16,   /* field[16] = location */
  13,   /* field[13] = notnulls */
  2,   /* field[2] = ns_names */
  1,   /* field[1] = ns_uris */
  15,   /* field[15] = ordinalitycol */
  12,   /* field[12] = passingvalexprs */
  14,   /* field[14] = plan */
  4,   /* field[4] = rowexpr */
};
static const ProtobufCIntRange pg_query__table_func__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 17 }
};
const ProtobufCMessageDescriptor pg_query__table_func__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.TableFunc",
  "TableFunc",
  "PgQuery__TableFunc",
  "pg_query",
  sizeof(PgQuery__TableFunc),
  17,
  pg_query__table_func__field_descriptors,
  pg_query__table_func__field_indices_by_name,
  1,  pg_query__table_func__number_ranges,
  (ProtobufCMessageInit) pg_query__table_func__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__into_clause__field_descriptors[8] =
{
  {
    "rel",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__IntoClause, rel),
    &pg_query__range_var__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "col_names",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__IntoClause, n_col_names),
    offsetof(PgQuery__IntoClause, col_names),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "access_method",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__IntoClause, access_method),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "options",
    4,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__IntoClause, n_options),
    offsetof(PgQuery__IntoClause, options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "on_commit",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__IntoClause, on_commit),
    &pg_query__on_commit_action__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "table_space_name",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__IntoClause, table_space_name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "view_query",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__IntoClause, view_query),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "skip_data",
    8,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__IntoClause, skip_data),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__into_clause__field_indices_by_name[] = {
  2,   /* field[2] = access_method */
  1,   /* field[1] = col_names */
  4,   /* field[4] = on_commit */
  3,   /* field[3] = options */
  0,   /* field[0] = rel */
  7,   /* field[7] = skip_data */
  5,   /* field[5] = table_space_name */
  6,   /* field[6] = view_query */
};
static const ProtobufCIntRange pg_query__into_clause__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 8 }
};
const ProtobufCMessageDescriptor pg_query__into_clause__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.IntoClause",
  "IntoClause",
  "PgQuery__IntoClause",
  "pg_query",
  sizeof(PgQuery__IntoClause),
  8,
  pg_query__into_clause__field_descriptors,
  pg_query__into_clause__field_indices_by_name,
  1,  pg_query__into_clause__number_ranges,
  (ProtobufCMessageInit) pg_query__into_clause__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__var__field_descriptors[9] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Var, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "varno",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Var, varno),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "varattno",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Var, varattno),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "vartype",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Var, vartype),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "vartypmod",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Var, vartypmod),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "varcollid",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Var, varcollid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "varnullingrels",
    7,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_UINT64,
    offsetof(PgQuery__Var, n_varnullingrels),
    offsetof(PgQuery__Var, varnullingrels),
    NULL,
    NULL,
    PROTOBUF_C_FIELD_FLAG_PACKED,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "varlevelsup",
    8,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Var, varlevelsup),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    9,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Var, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__var__field_indices_by_name[] = {
  8,   /* field[8] = location */
  2,   /* field[2] = varattno */
  5,   /* field[5] = varcollid */
  7,   /* field[7] = varlevelsup */
  1,   /* field[1] = varno */
  6,   /* field[6] = varnullingrels */
  3,   /* field[3] = vartype */
  4,   /* field[4] = vartypmod */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__var__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 9 }
};
const ProtobufCMessageDescriptor pg_query__var__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.Var",
  "Var",
  "PgQuery__Var",
  "pg_query",
  sizeof(PgQuery__Var),
  9,
  pg_query__var__field_descriptors,
  pg_query__var__field_indices_by_name,
  1,  pg_query__var__number_ranges,
  (ProtobufCMessageInit) pg_query__var__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__param__field_descriptors[7] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Param, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "paramkind",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Param, paramkind),
    &pg_query__param_kind__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "paramid",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Param, paramid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "paramtype",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Param, paramtype),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "paramtypmod",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Param, paramtypmod),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "paramcollid",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Param, paramcollid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Param, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__param__field_indices_by_name[] = {
  6,   /* field[6] = location */
  5,   /* field[5] = paramcollid */
  2,   /* field[2] = paramid */
  1,   /* field[1] = paramkind */
  3,   /* field[3] = paramtype */
  4,   /* field[4] = paramtypmod */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__param__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 7 }
};
const ProtobufCMessageDescriptor pg_query__param__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.Param",
  "Param",
  "PgQuery__Param",
  "pg_query",
  sizeof(PgQuery__Param),
  7,
  pg_query__param__field_descriptors,
  pg_query__param__field_indices_by_name,
  1,  pg_query__param__number_ranges,
  (ProtobufCMessageInit) pg_query__param__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__aggref__field_descriptors[19] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Aggref, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "aggfnoid",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Aggref, aggfnoid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "aggtype",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Aggref, aggtype),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "aggcollid",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Aggref, aggcollid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "inputcollid",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Aggref, inputcollid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "aggargtypes",
    6,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Aggref, n_aggargtypes),
    offsetof(PgQuery__Aggref, aggargtypes),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "aggdirectargs",
    7,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Aggref, n_aggdirectargs),
    offsetof(PgQuery__Aggref, aggdirectargs),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "args",
    8,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Aggref, n_args),
    offsetof(PgQuery__Aggref, args),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "aggorder",
    9,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Aggref, n_aggorder),
    offsetof(PgQuery__Aggref, aggorder),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "aggdistinct",
    10,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Aggref, n_aggdistinct),
    offsetof(PgQuery__Aggref, aggdistinct),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "aggfilter",
    11,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Aggref, aggfilter),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "aggstar",
    12,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Aggref, aggstar),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "aggvariadic",
    13,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Aggref, aggvariadic),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "aggkind",
    14,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Aggref, aggkind),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "agglevelsup",
    15,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Aggref, agglevelsup),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "aggsplit",
    16,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Aggref, aggsplit),
    &pg_query__agg_split__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "aggno",
    17,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Aggref, aggno),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "aggtransno",
    18,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Aggref, aggtransno),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    19,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Aggref, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__aggref__field_indices_by_name[] = {
  5,   /* field[5] = aggargtypes */
  3,   /* field[3] = aggcollid */
  6,   /* field[6] = aggdirectargs */
  9,   /* field[9] = aggdistinct */
  10,   /* field[10] = aggfilter */
  1,   /* field[1] = aggfnoid */
  13,   /* field[13] = aggkind */
  14,   /* field[14] = agglevelsup */
  16,   /* field[16] = aggno */
  8,   /* field[8] = aggorder */
  15,   /* field[15] = aggsplit */
  11,   /* field[11] = aggstar */
  17,   /* field[17] = aggtransno */
  2,   /* field[2] = aggtype */
  12,   /* field[12] = aggvariadic */
  7,   /* field[7] = args */
  4,   /* field[4] = inputcollid */
  18,   /* field[18] = location */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__aggref__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 19 }
};
const ProtobufCMessageDescriptor pg_query__aggref__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.Aggref",
  "Aggref",
  "PgQuery__Aggref",
  "pg_query",
  sizeof(PgQuery__Aggref),
  19,
  pg_query__aggref__field_descriptors,
  pg_query__aggref__field_indices_by_name,
  1,  pg_query__aggref__number_ranges,
  (ProtobufCMessageInit) pg_query__aggref__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__grouping_func__field_descriptors[5] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__GroupingFunc, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "args",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__GroupingFunc, n_args),
    offsetof(PgQuery__GroupingFunc, args),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "refs",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__GroupingFunc, n_refs),
    offsetof(PgQuery__GroupingFunc, refs),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "agglevelsup",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__GroupingFunc, agglevelsup),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__GroupingFunc, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__grouping_func__field_indices_by_name[] = {
  3,   /* field[3] = agglevelsup */
  1,   /* field[1] = args */
  4,   /* field[4] = location */
  2,   /* field[2] = refs */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__grouping_func__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor pg_query__grouping_func__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.GroupingFunc",
  "GroupingFunc",
  "PgQuery__GroupingFunc",
  "pg_query",
  sizeof(PgQuery__GroupingFunc),
  5,
  pg_query__grouping_func__field_descriptors,
  pg_query__grouping_func__field_indices_by_name,
  1,  pg_query__grouping_func__number_ranges,
  (ProtobufCMessageInit) pg_query__grouping_func__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__window_func__field_descriptors[12] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WindowFunc, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "winfnoid",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WindowFunc, winfnoid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "wintype",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WindowFunc, wintype),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "wincollid",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WindowFunc, wincollid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "inputcollid",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WindowFunc, inputcollid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "args",
    6,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__WindowFunc, n_args),
    offsetof(PgQuery__WindowFunc, args),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "aggfilter",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WindowFunc, aggfilter),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "run_condition",
    8,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__WindowFunc, n_run_condition),
    offsetof(PgQuery__WindowFunc, run_condition),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "winref",
    9,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WindowFunc, winref),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "winstar",
    10,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WindowFunc, winstar),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "winagg",
    11,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WindowFunc, winagg),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    12,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WindowFunc, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__window_func__field_indices_by_name[] = {
  6,   /* field[6] = aggfilter */
  5,   /* field[5] = args */
  4,   /* field[4] = inputcollid */
  11,   /* field[11] = location */
  7,   /* field[7] = run_condition */
  10,   /* field[10] = winagg */
  3,   /* field[3] = wincollid */
  1,   /* field[1] = winfnoid */
  8,   /* field[8] = winref */
  9,   /* field[9] = winstar */
  2,   /* field[2] = wintype */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__window_func__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 12 }
};
const ProtobufCMessageDescriptor pg_query__window_func__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.WindowFunc",
  "WindowFunc",
  "PgQuery__WindowFunc",
  "pg_query",
  sizeof(PgQuery__WindowFunc),
  12,
  pg_query__window_func__field_descriptors,
  pg_query__window_func__field_indices_by_name,
  1,  pg_query__window_func__number_ranges,
  (ProtobufCMessageInit) pg_query__window_func__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__window_func_run_condition__field_descriptors[5] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WindowFuncRunCondition, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "opno",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WindowFuncRunCondition, opno),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "inputcollid",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WindowFuncRunCondition, inputcollid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "wfunc_left",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WindowFuncRunCondition, wfunc_left),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "arg",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WindowFuncRunCondition, arg),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__window_func_run_condition__field_indices_by_name[] = {
  4,   /* field[4] = arg */
  2,   /* field[2] = inputcollid */
  1,   /* field[1] = opno */
  3,   /* field[3] = wfunc_left */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__window_func_run_condition__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor pg_query__window_func_run_condition__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.WindowFuncRunCondition",
  "WindowFuncRunCondition",
  "PgQuery__WindowFuncRunCondition",
  "pg_query",
  sizeof(PgQuery__WindowFuncRunCondition),
  5,
  pg_query__window_func_run_condition__field_descriptors,
  pg_query__window_func_run_condition__field_indices_by_name,
  1,  pg_query__window_func_run_condition__number_ranges,
  (ProtobufCMessageInit) pg_query__window_func_run_condition__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__merge_support_func__field_descriptors[4] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__MergeSupportFunc, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "msftype",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__MergeSupportFunc, msftype),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "msfcollid",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__MergeSupportFunc, msfcollid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__MergeSupportFunc, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__merge_support_func__field_indices_by_name[] = {
  3,   /* field[3] = location */
  2,   /* field[2] = msfcollid */
  1,   /* field[1] = msftype */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__merge_support_func__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor pg_query__merge_support_func__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.MergeSupportFunc",
  "MergeSupportFunc",
  "PgQuery__MergeSupportFunc",
  "pg_query",
  sizeof(PgQuery__MergeSupportFunc),
  4,
  pg_query__merge_support_func__field_descriptors,
  pg_query__merge_support_func__field_indices_by_name,
  1,  pg_query__merge_support_func__number_ranges,
  (ProtobufCMessageInit) pg_query__merge_support_func__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__subscripting_ref__field_descriptors[10] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SubscriptingRef, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "refcontainertype",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SubscriptingRef, refcontainertype),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "refelemtype",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SubscriptingRef, refelemtype),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "refrestype",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SubscriptingRef, refrestype),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "reftypmod",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SubscriptingRef, reftypmod),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "refcollid",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SubscriptingRef, refcollid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "refupperindexpr",
    7,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__SubscriptingRef, n_refupperindexpr),
    offsetof(PgQuery__SubscriptingRef, refupperindexpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "reflowerindexpr",
    8,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__SubscriptingRef, n_reflowerindexpr),
    offsetof(PgQuery__SubscriptingRef, reflowerindexpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "refexpr",
    9,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SubscriptingRef, refexpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "refassgnexpr",
    10,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SubscriptingRef, refassgnexpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__subscripting_ref__field_indices_by_name[] = {
  9,   /* field[9] = refassgnexpr */
  5,   /* field[5] = refcollid */
  1,   /* field[1] = refcontainertype */
  2,   /* field[2] = refelemtype */
  8,   /* field[8] = refexpr */
  7,   /* field[7] = reflowerindexpr */
  3,   /* field[3] = refrestype */
  4,   /* field[4] = reftypmod */
  6,   /* field[6] = refupperindexpr */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__subscripting_ref__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 10 }
};
const ProtobufCMessageDescriptor pg_query__subscripting_ref__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.SubscriptingRef",
  "SubscriptingRef",
  "PgQuery__SubscriptingRef",
  "pg_query",
  sizeof(PgQuery__SubscriptingRef),
  10,
  pg_query__subscripting_ref__field_descriptors,
  pg_query__subscripting_ref__field_indices_by_name,
  1,  pg_query__subscripting_ref__number_ranges,
  (ProtobufCMessageInit) pg_query__subscripting_ref__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__func_expr__field_descriptors[10] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__FuncExpr, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "funcid",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__FuncExpr, funcid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "funcresulttype",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__FuncExpr, funcresulttype),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "funcretset",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__FuncExpr, funcretset),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "funcvariadic",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__FuncExpr, funcvariadic),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "funcformat",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__FuncExpr, funcformat),
    &pg_query__coercion_form__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "funccollid",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__FuncExpr, funccollid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "inputcollid",
    8,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__FuncExpr, inputcollid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "args",
    9,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__FuncExpr, n_args),
    offsetof(PgQuery__FuncExpr, args),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    10,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__FuncExpr, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__func_expr__field_indices_by_name[] = {
  8,   /* field[8] = args */
  6,   /* field[6] = funccollid */
  5,   /* field[5] = funcformat */
  1,   /* field[1] = funcid */
  2,   /* field[2] = funcresulttype */
  3,   /* field[3] = funcretset */
  4,   /* field[4] = funcvariadic */
  7,   /* field[7] = inputcollid */
  9,   /* field[9] = location */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__func_expr__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 10 }
};
const ProtobufCMessageDescriptor pg_query__func_expr__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.FuncExpr",
  "FuncExpr",
  "PgQuery__FuncExpr",
  "pg_query",
  sizeof(PgQuery__FuncExpr),
  10,
  pg_query__func_expr__field_descriptors,
  pg_query__func_expr__field_indices_by_name,
  1,  pg_query__func_expr__number_ranges,
  (ProtobufCMessageInit) pg_query__func_expr__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__named_arg_expr__field_descriptors[5] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__NamedArgExpr, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "arg",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__NamedArgExpr, arg),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "name",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__NamedArgExpr, name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "argnumber",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__NamedArgExpr, argnumber),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__NamedArgExpr, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__named_arg_expr__field_indices_by_name[] = {
  1,   /* field[1] = arg */
  3,   /* field[3] = argnumber */
  4,   /* field[4] = location */
  2,   /* field[2] = name */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__named_arg_expr__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor pg_query__named_arg_expr__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.NamedArgExpr",
  "NamedArgExpr",
  "PgQuery__NamedArgExpr",
  "pg_query",
  sizeof(PgQuery__NamedArgExpr),
  5,
  pg_query__named_arg_expr__field_descriptors,
  pg_query__named_arg_expr__field_indices_by_name,
  1,  pg_query__named_arg_expr__number_ranges,
  (ProtobufCMessageInit) pg_query__named_arg_expr__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__op_expr__field_descriptors[8] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__OpExpr, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "opno",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__OpExpr, opno),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "opresulttype",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__OpExpr, opresulttype),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "opretset",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__OpExpr, opretset),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "opcollid",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__OpExpr, opcollid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "inputcollid",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__OpExpr, inputcollid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "args",
    7,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__OpExpr, n_args),
    offsetof(PgQuery__OpExpr, args),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    8,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__OpExpr, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__op_expr__field_indices_by_name[] = {
  6,   /* field[6] = args */
  5,   /* field[5] = inputcollid */
  7,   /* field[7] = location */
  4,   /* field[4] = opcollid */
  1,   /* field[1] = opno */
  2,   /* field[2] = opresulttype */
  3,   /* field[3] = opretset */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__op_expr__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 8 }
};
const ProtobufCMessageDescriptor pg_query__op_expr__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.OpExpr",
  "OpExpr",
  "PgQuery__OpExpr",
  "pg_query",
  sizeof(PgQuery__OpExpr),
  8,
  pg_query__op_expr__field_descriptors,
  pg_query__op_expr__field_indices_by_name,
  1,  pg_query__op_expr__number_ranges,
  (ProtobufCMessageInit) pg_query__op_expr__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__distinct_expr__field_descriptors[8] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DistinctExpr, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "opno",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DistinctExpr, opno),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "opresulttype",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DistinctExpr, opresulttype),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "opretset",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DistinctExpr, opretset),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "opcollid",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DistinctExpr, opcollid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "inputcollid",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DistinctExpr, inputcollid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "args",
    7,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__DistinctExpr, n_args),
    offsetof(PgQuery__DistinctExpr, args),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    8,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DistinctExpr, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__distinct_expr__field_indices_by_name[] = {
  6,   /* field[6] = args */
  5,   /* field[5] = inputcollid */
  7,   /* field[7] = location */
  4,   /* field[4] = opcollid */
  1,   /* field[1] = opno */
  2,   /* field[2] = opresulttype */
  3,   /* field[3] = opretset */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__distinct_expr__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 8 }
};
const ProtobufCMessageDescriptor pg_query__distinct_expr__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.DistinctExpr",
  "DistinctExpr",
  "PgQuery__DistinctExpr",
  "pg_query",
  sizeof(PgQuery__DistinctExpr),
  8,
  pg_query__distinct_expr__field_descriptors,
  pg_query__distinct_expr__field_indices_by_name,
  1,  pg_query__distinct_expr__number_ranges,
  (ProtobufCMessageInit) pg_query__distinct_expr__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__null_if_expr__field_descriptors[8] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__NullIfExpr, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "opno",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__NullIfExpr, opno),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "opresulttype",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__NullIfExpr, opresulttype),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "opretset",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__NullIfExpr, opretset),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "opcollid",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__NullIfExpr, opcollid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "inputcollid",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__NullIfExpr, inputcollid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "args",
    7,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__NullIfExpr, n_args),
    offsetof(PgQuery__NullIfExpr, args),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    8,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__NullIfExpr, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__null_if_expr__field_indices_by_name[] = {
  6,   /* field[6] = args */
  5,   /* field[5] = inputcollid */
  7,   /* field[7] = location */
  4,   /* field[4] = opcollid */
  1,   /* field[1] = opno */
  2,   /* field[2] = opresulttype */
  3,   /* field[3] = opretset */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__null_if_expr__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 8 }
};
const ProtobufCMessageDescriptor pg_query__null_if_expr__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.NullIfExpr",
  "NullIfExpr",
  "PgQuery__NullIfExpr",
  "pg_query",
  sizeof(PgQuery__NullIfExpr),
  8,
  pg_query__null_if_expr__field_descriptors,
  pg_query__null_if_expr__field_indices_by_name,
  1,  pg_query__null_if_expr__number_ranges,
  (ProtobufCMessageInit) pg_query__null_if_expr__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__scalar_array_op_expr__field_descriptors[6] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ScalarArrayOpExpr, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "opno",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ScalarArrayOpExpr, opno),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "use_or",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ScalarArrayOpExpr, use_or),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "inputcollid",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ScalarArrayOpExpr, inputcollid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "args",
    5,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__ScalarArrayOpExpr, n_args),
    offsetof(PgQuery__ScalarArrayOpExpr, args),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ScalarArrayOpExpr, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__scalar_array_op_expr__field_indices_by_name[] = {
  4,   /* field[4] = args */
  3,   /* field[3] = inputcollid */
  5,   /* field[5] = location */
  1,   /* field[1] = opno */
  2,   /* field[2] = use_or */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__scalar_array_op_expr__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 6 }
};
const ProtobufCMessageDescriptor pg_query__scalar_array_op_expr__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.ScalarArrayOpExpr",
  "ScalarArrayOpExpr",
  "PgQuery__ScalarArrayOpExpr",
  "pg_query",
  sizeof(PgQuery__ScalarArrayOpExpr),
  6,
  pg_query__scalar_array_op_expr__field_descriptors,
  pg_query__scalar_array_op_expr__field_indices_by_name,
  1,  pg_query__scalar_array_op_expr__number_ranges,
  (ProtobufCMessageInit) pg_query__scalar_array_op_expr__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__bool_expr__field_descriptors[4] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__BoolExpr, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "boolop",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__BoolExpr, boolop),
    &pg_query__bool_expr_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "args",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__BoolExpr, n_args),
    offsetof(PgQuery__BoolExpr, args),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__BoolExpr, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__bool_expr__field_indices_by_name[] = {
  2,   /* field[2] = args */
  1,   /* field[1] = boolop */
  3,   /* field[3] = location */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__bool_expr__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor pg_query__bool_expr__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.BoolExpr",
  "BoolExpr",
  "PgQuery__BoolExpr",
  "pg_query",
  sizeof(PgQuery__BoolExpr),
  4,
  pg_query__bool_expr__field_descriptors,
  pg_query__bool_expr__field_indices_by_name,
  1,  pg_query__bool_expr__number_ranges,
  (ProtobufCMessageInit) pg_query__bool_expr__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__sub_link__field_descriptors[7] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SubLink, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "sub_link_type",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SubLink, sub_link_type),
    &pg_query__sub_link_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "sub_link_id",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SubLink, sub_link_id),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "testexpr",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SubLink, testexpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "oper_name",
    5,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__SubLink, n_oper_name),
    offsetof(PgQuery__SubLink, oper_name),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "subselect",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SubLink, subselect),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SubLink, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__sub_link__field_indices_by_name[] = {
  6,   /* field[6] = location */
  4,   /* field[4] = oper_name */
  2,   /* field[2] = sub_link_id */
  1,   /* field[1] = sub_link_type */
  5,   /* field[5] = subselect */
  3,   /* field[3] = testexpr */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__sub_link__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 7 }
};
const ProtobufCMessageDescriptor pg_query__sub_link__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.SubLink",
  "SubLink",
  "PgQuery__SubLink",
  "pg_query",
  sizeof(PgQuery__SubLink),
  7,
  pg_query__sub_link__field_descriptors,
  pg_query__sub_link__field_indices_by_name,
  1,  pg_query__sub_link__number_ranges,
  (ProtobufCMessageInit) pg_query__sub_link__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__sub_plan__field_descriptors[17] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SubPlan, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "sub_link_type",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SubPlan, sub_link_type),
    &pg_query__sub_link_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "testexpr",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SubPlan, testexpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "param_ids",
    4,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__SubPlan, n_param_ids),
    offsetof(PgQuery__SubPlan, param_ids),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "plan_id",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SubPlan, plan_id),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "plan_name",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SubPlan, plan_name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "first_col_type",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SubPlan, first_col_type),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "first_col_typmod",
    8,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SubPlan, first_col_typmod),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "first_col_collation",
    9,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SubPlan, first_col_collation),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "use_hash_table",
    10,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SubPlan, use_hash_table),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "unknown_eq_false",
    11,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SubPlan, unknown_eq_false),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "parallel_safe",
    12,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SubPlan, parallel_safe),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "set_param",
    13,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__SubPlan, n_set_param),
    offsetof(PgQuery__SubPlan, set_param),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "par_param",
    14,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__SubPlan, n_par_param),
    offsetof(PgQuery__SubPlan, par_param),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "args",
    15,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__SubPlan, n_args),
    offsetof(PgQuery__SubPlan, args),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "startup_cost",
    16,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_DOUBLE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SubPlan, startup_cost),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "per_call_cost",
    17,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_DOUBLE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SubPlan, per_call_cost),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__sub_plan__field_indices_by_name[] = {
  14,   /* field[14] = args */
  8,   /* field[8] = first_col_collation */
  6,   /* field[6] = first_col_type */
  7,   /* field[7] = first_col_typmod */
  13,   /* field[13] = par_param */
  11,   /* field[11] = parallel_safe */
  3,   /* field[3] = param_ids */
  16,   /* field[16] = per_call_cost */
  4,   /* field[4] = plan_id */
  5,   /* field[5] = plan_name */
  12,   /* field[12] = set_param */
  15,   /* field[15] = startup_cost */
  1,   /* field[1] = sub_link_type */
  2,   /* field[2] = testexpr */
  10,   /* field[10] = unknown_eq_false */
  9,   /* field[9] = use_hash_table */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__sub_plan__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 17 }
};
const ProtobufCMessageDescriptor pg_query__sub_plan__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.SubPlan",
  "SubPlan",
  "PgQuery__SubPlan",
  "pg_query",
  sizeof(PgQuery__SubPlan),
  17,
  pg_query__sub_plan__field_descriptors,
  pg_query__sub_plan__field_indices_by_name,
  1,  pg_query__sub_plan__number_ranges,
  (ProtobufCMessageInit) pg_query__sub_plan__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__alternative_sub_plan__field_descriptors[2] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlternativeSubPlan, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "subplans",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AlternativeSubPlan, n_subplans),
    offsetof(PgQuery__AlternativeSubPlan, subplans),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__alternative_sub_plan__field_indices_by_name[] = {
  1,   /* field[1] = subplans */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__alternative_sub_plan__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor pg_query__alternative_sub_plan__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.AlternativeSubPlan",
  "AlternativeSubPlan",
  "PgQuery__AlternativeSubPlan",
  "pg_query",
  sizeof(PgQuery__AlternativeSubPlan),
  2,
  pg_query__alternative_sub_plan__field_descriptors,
  pg_query__alternative_sub_plan__field_indices_by_name,
  1,  pg_query__alternative_sub_plan__number_ranges,
  (ProtobufCMessageInit) pg_query__alternative_sub_plan__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__field_select__field_descriptors[6] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__FieldSelect, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "arg",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__FieldSelect, arg),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "fieldnum",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__FieldSelect, fieldnum),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "resulttype",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__FieldSelect, resulttype),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "resulttypmod",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__FieldSelect, resulttypmod),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "resultcollid",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__FieldSelect, resultcollid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__field_select__field_indices_by_name[] = {
  1,   /* field[1] = arg */
  2,   /* field[2] = fieldnum */
  5,   /* field[5] = resultcollid */
  3,   /* field[3] = resulttype */
  4,   /* field[4] = resulttypmod */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__field_select__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 6 }
};
const ProtobufCMessageDescriptor pg_query__field_select__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.FieldSelect",
  "FieldSelect",
  "PgQuery__FieldSelect",
  "pg_query",
  sizeof(PgQuery__FieldSelect),
  6,
  pg_query__field_select__field_descriptors,
  pg_query__field_select__field_indices_by_name,
  1,  pg_query__field_select__number_ranges,
  (ProtobufCMessageInit) pg_query__field_select__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__field_store__field_descriptors[5] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__FieldStore, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "arg",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__FieldStore, arg),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "newvals",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__FieldStore, n_newvals),
    offsetof(PgQuery__FieldStore, newvals),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "fieldnums",
    4,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__FieldStore, n_fieldnums),
    offsetof(PgQuery__FieldStore, fieldnums),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "resulttype",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__FieldStore, resulttype),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__field_store__field_indices_by_name[] = {
  1,   /* field[1] = arg */
  3,   /* field[3] = fieldnums */
  2,   /* field[2] = newvals */
  4,   /* field[4] = resulttype */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__field_store__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor pg_query__field_store__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.FieldStore",
  "FieldStore",
  "PgQuery__FieldStore",
  "pg_query",
  sizeof(PgQuery__FieldStore),
  5,
  pg_query__field_store__field_descriptors,
  pg_query__field_store__field_indices_by_name,
  1,  pg_query__field_store__number_ranges,
  (ProtobufCMessageInit) pg_query__field_store__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__relabel_type__field_descriptors[7] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RelabelType, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "arg",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RelabelType, arg),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "resulttype",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RelabelType, resulttype),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "resulttypmod",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RelabelType, resulttypmod),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "resultcollid",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RelabelType, resultcollid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "relabelformat",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RelabelType, relabelformat),
    &pg_query__coercion_form__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RelabelType, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__relabel_type__field_indices_by_name[] = {
  1,   /* field[1] = arg */
  6,   /* field[6] = location */
  5,   /* field[5] = relabelformat */
  4,   /* field[4] = resultcollid */
  2,   /* field[2] = resulttype */
  3,   /* field[3] = resulttypmod */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__relabel_type__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 7 }
};
const ProtobufCMessageDescriptor pg_query__relabel_type__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.RelabelType",
  "RelabelType",
  "PgQuery__RelabelType",
  "pg_query",
  sizeof(PgQuery__RelabelType),
  7,
  pg_query__relabel_type__field_descriptors,
  pg_query__relabel_type__field_indices_by_name,
  1,  pg_query__relabel_type__number_ranges,
  (ProtobufCMessageInit) pg_query__relabel_type__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__coerce_via_io__field_descriptors[6] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CoerceViaIO, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "arg",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CoerceViaIO, arg),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "resulttype",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CoerceViaIO, resulttype),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "resultcollid",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CoerceViaIO, resultcollid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "coerceformat",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CoerceViaIO, coerceformat),
    &pg_query__coercion_form__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CoerceViaIO, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__coerce_via_io__field_indices_by_name[] = {
  1,   /* field[1] = arg */
  4,   /* field[4] = coerceformat */
  5,   /* field[5] = location */
  3,   /* field[3] = resultcollid */
  2,   /* field[2] = resulttype */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__coerce_via_io__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 6 }
};
const ProtobufCMessageDescriptor pg_query__coerce_via_io__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CoerceViaIO",
  "CoerceViaIO",
  "PgQuery__CoerceViaIO",
  "pg_query",
  sizeof(PgQuery__CoerceViaIO),
  6,
  pg_query__coerce_via_io__field_descriptors,
  pg_query__coerce_via_io__field_indices_by_name,
  1,  pg_query__coerce_via_io__number_ranges,
  (ProtobufCMessageInit) pg_query__coerce_via_io__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__array_coerce_expr__field_descriptors[8] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ArrayCoerceExpr, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "arg",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ArrayCoerceExpr, arg),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "elemexpr",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ArrayCoerceExpr, elemexpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "resulttype",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ArrayCoerceExpr, resulttype),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "resulttypmod",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ArrayCoerceExpr, resulttypmod),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "resultcollid",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ArrayCoerceExpr, resultcollid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "coerceformat",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ArrayCoerceExpr, coerceformat),
    &pg_query__coercion_form__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    8,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ArrayCoerceExpr, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__array_coerce_expr__field_indices_by_name[] = {
  1,   /* field[1] = arg */
  6,   /* field[6] = coerceformat */
  2,   /* field[2] = elemexpr */
  7,   /* field[7] = location */
  5,   /* field[5] = resultcollid */
  3,   /* field[3] = resulttype */
  4,   /* field[4] = resulttypmod */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__array_coerce_expr__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 8 }
};
const ProtobufCMessageDescriptor pg_query__array_coerce_expr__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.ArrayCoerceExpr",
  "ArrayCoerceExpr",
  "PgQuery__ArrayCoerceExpr",
  "pg_query",
  sizeof(PgQuery__ArrayCoerceExpr),
  8,
  pg_query__array_coerce_expr__field_descriptors,
  pg_query__array_coerce_expr__field_indices_by_name,
  1,  pg_query__array_coerce_expr__number_ranges,
  (ProtobufCMessageInit) pg_query__array_coerce_expr__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__convert_rowtype_expr__field_descriptors[5] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ConvertRowtypeExpr, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "arg",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ConvertRowtypeExpr, arg),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "resulttype",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ConvertRowtypeExpr, resulttype),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "convertformat",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ConvertRowtypeExpr, convertformat),
    &pg_query__coercion_form__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ConvertRowtypeExpr, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__convert_rowtype_expr__field_indices_by_name[] = {
  1,   /* field[1] = arg */
  3,   /* field[3] = convertformat */
  4,   /* field[4] = location */
  2,   /* field[2] = resulttype */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__convert_rowtype_expr__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor pg_query__convert_rowtype_expr__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.ConvertRowtypeExpr",
  "ConvertRowtypeExpr",
  "PgQuery__ConvertRowtypeExpr",
  "pg_query",
  sizeof(PgQuery__ConvertRowtypeExpr),
  5,
  pg_query__convert_rowtype_expr__field_descriptors,
  pg_query__convert_rowtype_expr__field_indices_by_name,
  1,  pg_query__convert_rowtype_expr__number_ranges,
  (ProtobufCMessageInit) pg_query__convert_rowtype_expr__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__collate_expr__field_descriptors[4] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CollateExpr, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "arg",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CollateExpr, arg),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "coll_oid",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CollateExpr, coll_oid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CollateExpr, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__collate_expr__field_indices_by_name[] = {
  1,   /* field[1] = arg */
  2,   /* field[2] = coll_oid */
  3,   /* field[3] = location */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__collate_expr__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor pg_query__collate_expr__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CollateExpr",
  "CollateExpr",
  "PgQuery__CollateExpr",
  "pg_query",
  sizeof(PgQuery__CollateExpr),
  4,
  pg_query__collate_expr__field_descriptors,
  pg_query__collate_expr__field_indices_by_name,
  1,  pg_query__collate_expr__number_ranges,
  (ProtobufCMessageInit) pg_query__collate_expr__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__case_expr__field_descriptors[7] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CaseExpr, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "casetype",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CaseExpr, casetype),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "casecollid",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CaseExpr, casecollid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "arg",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CaseExpr, arg),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "args",
    5,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CaseExpr, n_args),
    offsetof(PgQuery__CaseExpr, args),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "defresult",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CaseExpr, defresult),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CaseExpr, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__case_expr__field_indices_by_name[] = {
  3,   /* field[3] = arg */
  4,   /* field[4] = args */
  2,   /* field[2] = casecollid */
  1,   /* field[1] = casetype */
  5,   /* field[5] = defresult */
  6,   /* field[6] = location */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__case_expr__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 7 }
};
const ProtobufCMessageDescriptor pg_query__case_expr__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CaseExpr",
  "CaseExpr",
  "PgQuery__CaseExpr",
  "pg_query",
  sizeof(PgQuery__CaseExpr),
  7,
  pg_query__case_expr__field_descriptors,
  pg_query__case_expr__field_indices_by_name,
  1,  pg_query__case_expr__number_ranges,
  (ProtobufCMessageInit) pg_query__case_expr__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__case_when__field_descriptors[4] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CaseWhen, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "expr",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CaseWhen, expr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "result",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CaseWhen, result),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CaseWhen, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__case_when__field_indices_by_name[] = {
  1,   /* field[1] = expr */
  3,   /* field[3] = location */
  2,   /* field[2] = result */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__case_when__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor pg_query__case_when__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CaseWhen",
  "CaseWhen",
  "PgQuery__CaseWhen",
  "pg_query",
  sizeof(PgQuery__CaseWhen),
  4,
  pg_query__case_when__field_descriptors,
  pg_query__case_when__field_indices_by_name,
  1,  pg_query__case_when__number_ranges,
  (ProtobufCMessageInit) pg_query__case_when__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__case_test_expr__field_descriptors[4] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CaseTestExpr, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "type_id",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CaseTestExpr, type_id),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "type_mod",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CaseTestExpr, type_mod),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "collation",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CaseTestExpr, collation),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__case_test_expr__field_indices_by_name[] = {
  3,   /* field[3] = collation */
  1,   /* field[1] = type_id */
  2,   /* field[2] = type_mod */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__case_test_expr__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor pg_query__case_test_expr__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CaseTestExpr",
  "CaseTestExpr",
  "PgQuery__CaseTestExpr",
  "pg_query",
  sizeof(PgQuery__CaseTestExpr),
  4,
  pg_query__case_test_expr__field_descriptors,
  pg_query__case_test_expr__field_indices_by_name,
  1,  pg_query__case_test_expr__number_ranges,
  (ProtobufCMessageInit) pg_query__case_test_expr__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__array_expr__field_descriptors[7] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ArrayExpr, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "array_typeid",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ArrayExpr, array_typeid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "array_collid",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ArrayExpr, array_collid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "element_typeid",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ArrayExpr, element_typeid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "elements",
    5,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__ArrayExpr, n_elements),
    offsetof(PgQuery__ArrayExpr, elements),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "multidims",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ArrayExpr, multidims),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ArrayExpr, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__array_expr__field_indices_by_name[] = {
  2,   /* field[2] = array_collid */
  1,   /* field[1] = array_typeid */
  3,   /* field[3] = element_typeid */
  4,   /* field[4] = elements */
  6,   /* field[6] = location */
  5,   /* field[5] = multidims */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__array_expr__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 7 }
};
const ProtobufCMessageDescriptor pg_query__array_expr__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.ArrayExpr",
  "ArrayExpr",
  "PgQuery__ArrayExpr",
  "pg_query",
  sizeof(PgQuery__ArrayExpr),
  7,
  pg_query__array_expr__field_descriptors,
  pg_query__array_expr__field_indices_by_name,
  1,  pg_query__array_expr__number_ranges,
  (ProtobufCMessageInit) pg_query__array_expr__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__row_expr__field_descriptors[6] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RowExpr, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "args",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__RowExpr, n_args),
    offsetof(PgQuery__RowExpr, args),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "row_typeid",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RowExpr, row_typeid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "row_format",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RowExpr, row_format),
    &pg_query__coercion_form__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "colnames",
    5,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__RowExpr, n_colnames),
    offsetof(PgQuery__RowExpr, colnames),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RowExpr, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__row_expr__field_indices_by_name[] = {
  1,   /* field[1] = args */
  4,   /* field[4] = colnames */
  5,   /* field[5] = location */
  3,   /* field[3] = row_format */
  2,   /* field[2] = row_typeid */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__row_expr__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 6 }
};
const ProtobufCMessageDescriptor pg_query__row_expr__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.RowExpr",
  "RowExpr",
  "PgQuery__RowExpr",
  "pg_query",
  sizeof(PgQuery__RowExpr),
  6,
  pg_query__row_expr__field_descriptors,
  pg_query__row_expr__field_indices_by_name,
  1,  pg_query__row_expr__number_ranges,
  (ProtobufCMessageInit) pg_query__row_expr__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__row_compare_expr__field_descriptors[7] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RowCompareExpr, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "rctype",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RowCompareExpr, rctype),
    &pg_query__row_compare_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "opnos",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__RowCompareExpr, n_opnos),
    offsetof(PgQuery__RowCompareExpr, opnos),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "opfamilies",
    4,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__RowCompareExpr, n_opfamilies),
    offsetof(PgQuery__RowCompareExpr, opfamilies),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "inputcollids",
    5,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__RowCompareExpr, n_inputcollids),
    offsetof(PgQuery__RowCompareExpr, inputcollids),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "largs",
    6,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__RowCompareExpr, n_largs),
    offsetof(PgQuery__RowCompareExpr, largs),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "rargs",
    7,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__RowCompareExpr, n_rargs),
    offsetof(PgQuery__RowCompareExpr, rargs),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__row_compare_expr__field_indices_by_name[] = {
  4,   /* field[4] = inputcollids */
  5,   /* field[5] = largs */
  3,   /* field[3] = opfamilies */
  2,   /* field[2] = opnos */
  6,   /* field[6] = rargs */
  1,   /* field[1] = rctype */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__row_compare_expr__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 7 }
};
const ProtobufCMessageDescriptor pg_query__row_compare_expr__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.RowCompareExpr",
  "RowCompareExpr",
  "PgQuery__RowCompareExpr",
  "pg_query",
  sizeof(PgQuery__RowCompareExpr),
  7,
  pg_query__row_compare_expr__field_descriptors,
  pg_query__row_compare_expr__field_indices_by_name,
  1,  pg_query__row_compare_expr__number_ranges,
  (ProtobufCMessageInit) pg_query__row_compare_expr__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__coalesce_expr__field_descriptors[5] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CoalesceExpr, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "coalescetype",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CoalesceExpr, coalescetype),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "coalescecollid",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CoalesceExpr, coalescecollid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "args",
    4,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CoalesceExpr, n_args),
    offsetof(PgQuery__CoalesceExpr, args),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CoalesceExpr, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__coalesce_expr__field_indices_by_name[] = {
  3,   /* field[3] = args */
  2,   /* field[2] = coalescecollid */
  1,   /* field[1] = coalescetype */
  4,   /* field[4] = location */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__coalesce_expr__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor pg_query__coalesce_expr__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CoalesceExpr",
  "CoalesceExpr",
  "PgQuery__CoalesceExpr",
  "pg_query",
  sizeof(PgQuery__CoalesceExpr),
  5,
  pg_query__coalesce_expr__field_descriptors,
  pg_query__coalesce_expr__field_indices_by_name,
  1,  pg_query__coalesce_expr__number_ranges,
  (ProtobufCMessageInit) pg_query__coalesce_expr__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__min_max_expr__field_descriptors[7] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__MinMaxExpr, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "minmaxtype",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__MinMaxExpr, minmaxtype),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "minmaxcollid",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__MinMaxExpr, minmaxcollid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "inputcollid",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__MinMaxExpr, inputcollid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "op",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__MinMaxExpr, op),
    &pg_query__min_max_op__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "args",
    6,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__MinMaxExpr, n_args),
    offsetof(PgQuery__MinMaxExpr, args),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__MinMaxExpr, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__min_max_expr__field_indices_by_name[] = {
  5,   /* field[5] = args */
  3,   /* field[3] = inputcollid */
  6,   /* field[6] = location */
  2,   /* field[2] = minmaxcollid */
  1,   /* field[1] = minmaxtype */
  4,   /* field[4] = op */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__min_max_expr__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 7 }
};
const ProtobufCMessageDescriptor pg_query__min_max_expr__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.MinMaxExpr",
  "MinMaxExpr",
  "PgQuery__MinMaxExpr",
  "pg_query",
  sizeof(PgQuery__MinMaxExpr),
  7,
  pg_query__min_max_expr__field_descriptors,
  pg_query__min_max_expr__field_indices_by_name,
  1,  pg_query__min_max_expr__number_ranges,
  (ProtobufCMessageInit) pg_query__min_max_expr__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__sqlvalue_function__field_descriptors[5] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SQLValueFunction, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "op",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SQLValueFunction, op),
    &pg_query__sqlvalue_function_op__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "type",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SQLValueFunction, type),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "typmod",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SQLValueFunction, typmod),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SQLValueFunction, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__sqlvalue_function__field_indices_by_name[] = {
  4,   /* field[4] = location */
  1,   /* field[1] = op */
  2,   /* field[2] = type */
  3,   /* field[3] = typmod */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__sqlvalue_function__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor pg_query__sqlvalue_function__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.SQLValueFunction",
  "SQLValueFunction",
  "PgQuery__SQLValueFunction",
  "pg_query",
  sizeof(PgQuery__SQLValueFunction),
  5,
  pg_query__sqlvalue_function__field_descriptors,
  pg_query__sqlvalue_function__field_indices_by_name,
  1,  pg_query__sqlvalue_function__number_ranges,
  (ProtobufCMessageInit) pg_query__sqlvalue_function__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__xml_expr__field_descriptors[11] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__XmlExpr, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "op",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__XmlExpr, op),
    &pg_query__xml_expr_op__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "name",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__XmlExpr, name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "named_args",
    4,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__XmlExpr, n_named_args),
    offsetof(PgQuery__XmlExpr, named_args),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "arg_names",
    5,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__XmlExpr, n_arg_names),
    offsetof(PgQuery__XmlExpr, arg_names),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "args",
    6,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__XmlExpr, n_args),
    offsetof(PgQuery__XmlExpr, args),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "xmloption",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__XmlExpr, xmloption),
    &pg_query__xml_option_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "indent",
    8,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__XmlExpr, indent),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "type",
    9,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__XmlExpr, type),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "typmod",
    10,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__XmlExpr, typmod),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    11,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__XmlExpr, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__xml_expr__field_indices_by_name[] = {
  4,   /* field[4] = arg_names */
  5,   /* field[5] = args */
  7,   /* field[7] = indent */
  10,   /* field[10] = location */
  2,   /* field[2] = name */
  3,   /* field[3] = named_args */
  1,   /* field[1] = op */
  8,   /* field[8] = type */
  9,   /* field[9] = typmod */
  6,   /* field[6] = xmloption */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__xml_expr__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 11 }
};
const ProtobufCMessageDescriptor pg_query__xml_expr__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.XmlExpr",
  "XmlExpr",
  "PgQuery__XmlExpr",
  "pg_query",
  sizeof(PgQuery__XmlExpr),
  11,
  pg_query__xml_expr__field_descriptors,
  pg_query__xml_expr__field_indices_by_name,
  1,  pg_query__xml_expr__number_ranges,
  (ProtobufCMessageInit) pg_query__xml_expr__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__json_format__field_descriptors[3] =
{
  {
    "format_type",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonFormat, format_type),
    &pg_query__json_format_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "encoding",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonFormat, encoding),
    &pg_query__json_encoding__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonFormat, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__json_format__field_indices_by_name[] = {
  1,   /* field[1] = encoding */
  0,   /* field[0] = format_type */
  2,   /* field[2] = location */
};
static const ProtobufCIntRange pg_query__json_format__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__json_format__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.JsonFormat",
  "JsonFormat",
  "PgQuery__JsonFormat",
  "pg_query",
  sizeof(PgQuery__JsonFormat),
  3,
  pg_query__json_format__field_descriptors,
  pg_query__json_format__field_indices_by_name,
  1,  pg_query__json_format__number_ranges,
  (ProtobufCMessageInit) pg_query__json_format__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__json_returning__field_descriptors[3] =
{
  {
    "format",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonReturning, format),
    &pg_query__json_format__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "typid",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonReturning, typid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "typmod",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonReturning, typmod),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__json_returning__field_indices_by_name[] = {
  0,   /* field[0] = format */
  1,   /* field[1] = typid */
  2,   /* field[2] = typmod */
};
static const ProtobufCIntRange pg_query__json_returning__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__json_returning__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.JsonReturning",
  "JsonReturning",
  "PgQuery__JsonReturning",
  "pg_query",
  sizeof(PgQuery__JsonReturning),
  3,
  pg_query__json_returning__field_descriptors,
  pg_query__json_returning__field_indices_by_name,
  1,  pg_query__json_returning__number_ranges,
  (ProtobufCMessageInit) pg_query__json_returning__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__json_value_expr__field_descriptors[3] =
{
  {
    "raw_expr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonValueExpr, raw_expr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "formatted_expr",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonValueExpr, formatted_expr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "format",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonValueExpr, format),
    &pg_query__json_format__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__json_value_expr__field_indices_by_name[] = {
  2,   /* field[2] = format */
  1,   /* field[1] = formatted_expr */
  0,   /* field[0] = raw_expr */
};
static const ProtobufCIntRange pg_query__json_value_expr__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__json_value_expr__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.JsonValueExpr",
  "JsonValueExpr",
  "PgQuery__JsonValueExpr",
  "pg_query",
  sizeof(PgQuery__JsonValueExpr),
  3,
  pg_query__json_value_expr__field_descriptors,
  pg_query__json_value_expr__field_indices_by_name,
  1,  pg_query__json_value_expr__number_ranges,
  (ProtobufCMessageInit) pg_query__json_value_expr__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__json_constructor_expr__field_descriptors[9] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonConstructorExpr, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "type",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonConstructorExpr, type),
    &pg_query__json_constructor_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "args",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__JsonConstructorExpr, n_args),
    offsetof(PgQuery__JsonConstructorExpr, args),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "func",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonConstructorExpr, func),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "coercion",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonConstructorExpr, coercion),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "returning",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonConstructorExpr, returning),
    &pg_query__json_returning__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "absent_on_null",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonConstructorExpr, absent_on_null),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "unique",
    8,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonConstructorExpr, unique),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    9,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonConstructorExpr, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__json_constructor_expr__field_indices_by_name[] = {
  6,   /* field[6] = absent_on_null */
  2,   /* field[2] = args */
  4,   /* field[4] = coercion */
  3,   /* field[3] = func */
  8,   /* field[8] = location */
  5,   /* field[5] = returning */
  1,   /* field[1] = type */
  7,   /* field[7] = unique */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__json_constructor_expr__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 9 }
};
const ProtobufCMessageDescriptor pg_query__json_constructor_expr__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.JsonConstructorExpr",
  "JsonConstructorExpr",
  "PgQuery__JsonConstructorExpr",
  "pg_query",
  sizeof(PgQuery__JsonConstructorExpr),
  9,
  pg_query__json_constructor_expr__field_descriptors,
  pg_query__json_constructor_expr__field_indices_by_name,
  1,  pg_query__json_constructor_expr__number_ranges,
  (ProtobufCMessageInit) pg_query__json_constructor_expr__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__json_is_predicate__field_descriptors[5] =
{
  {
    "expr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonIsPredicate, expr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "format",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonIsPredicate, format),
    &pg_query__json_format__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "item_type",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonIsPredicate, item_type),
    &pg_query__json_value_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "unique_keys",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonIsPredicate, unique_keys),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonIsPredicate, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__json_is_predicate__field_indices_by_name[] = {
  0,   /* field[0] = expr */
  1,   /* field[1] = format */
  2,   /* field[2] = item_type */
  4,   /* field[4] = location */
  3,   /* field[3] = unique_keys */
};
static const ProtobufCIntRange pg_query__json_is_predicate__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor pg_query__json_is_predicate__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.JsonIsPredicate",
  "JsonIsPredicate",
  "PgQuery__JsonIsPredicate",
  "pg_query",
  sizeof(PgQuery__JsonIsPredicate),
  5,
  pg_query__json_is_predicate__field_descriptors,
  pg_query__json_is_predicate__field_indices_by_name,
  1,  pg_query__json_is_predicate__number_ranges,
  (ProtobufCMessageInit) pg_query__json_is_predicate__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__json_behavior__field_descriptors[4] =
{
  {
    "btype",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonBehavior, btype),
    &pg_query__json_behavior_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "expr",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonBehavior, expr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "coerce",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonBehavior, coerce),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonBehavior, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__json_behavior__field_indices_by_name[] = {
  0,   /* field[0] = btype */
  2,   /* field[2] = coerce */
  1,   /* field[1] = expr */
  3,   /* field[3] = location */
};
static const ProtobufCIntRange pg_query__json_behavior__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor pg_query__json_behavior__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.JsonBehavior",
  "JsonBehavior",
  "PgQuery__JsonBehavior",
  "pg_query",
  sizeof(PgQuery__JsonBehavior),
  4,
  pg_query__json_behavior__field_descriptors,
  pg_query__json_behavior__field_indices_by_name,
  1,  pg_query__json_behavior__number_ranges,
  (ProtobufCMessageInit) pg_query__json_behavior__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__json_expr__field_descriptors[17] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonExpr, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "op",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonExpr, op),
    &pg_query__json_expr_op__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "column_name",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonExpr, column_name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "formatted_expr",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonExpr, formatted_expr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "format",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonExpr, format),
    &pg_query__json_format__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "path_spec",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonExpr, path_spec),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "returning",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonExpr, returning),
    &pg_query__json_returning__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "passing_names",
    8,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__JsonExpr, n_passing_names),
    offsetof(PgQuery__JsonExpr, passing_names),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "passing_values",
    9,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__JsonExpr, n_passing_values),
    offsetof(PgQuery__JsonExpr, passing_values),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "on_empty",
    10,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonExpr, on_empty),
    &pg_query__json_behavior__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "on_error",
    11,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonExpr, on_error),
    &pg_query__json_behavior__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "use_io_coercion",
    12,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonExpr, use_io_coercion),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "use_json_coercion",
    13,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonExpr, use_json_coercion),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "wrapper",
    14,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonExpr, wrapper),
    &pg_query__json_wrapper__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "omit_quotes",
    15,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonExpr, omit_quotes),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "collation",
    16,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonExpr, collation),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    17,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonExpr, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__json_expr__field_indices_by_name[] = {
  15,   /* field[15] = collation */
  2,   /* field[2] = column_name */
  4,   /* field[4] = format */
  3,   /* field[3] = formatted_expr */
  16,   /* field[16] = location */
  14,   /* field[14] = omit_quotes */
  9,   /* field[9] = on_empty */
  10,   /* field[10] = on_error */
  1,   /* field[1] = op */
  7,   /* field[7] = passing_names */
  8,   /* field[8] = passing_values */
  5,   /* field[5] = path_spec */
  6,   /* field[6] = returning */
  11,   /* field[11] = use_io_coercion */
  12,   /* field[12] = use_json_coercion */
  13,   /* field[13] = wrapper */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__json_expr__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 17 }
};
const ProtobufCMessageDescriptor pg_query__json_expr__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.JsonExpr",
  "JsonExpr",
  "PgQuery__JsonExpr",
  "pg_query",
  sizeof(PgQuery__JsonExpr),
  17,
  pg_query__json_expr__field_descriptors,
  pg_query__json_expr__field_indices_by_name,
  1,  pg_query__json_expr__number_ranges,
  (ProtobufCMessageInit) pg_query__json_expr__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__json_table_path__field_descriptors[1] =
{
  {
    "name",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonTablePath, name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__json_table_path__field_indices_by_name[] = {
  0,   /* field[0] = name */
};
static const ProtobufCIntRange pg_query__json_table_path__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 1 }
};
const ProtobufCMessageDescriptor pg_query__json_table_path__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.JsonTablePath",
  "JsonTablePath",
  "PgQuery__JsonTablePath",
  "pg_query",
  sizeof(PgQuery__JsonTablePath),
  1,
  pg_query__json_table_path__field_descriptors,
  pg_query__json_table_path__field_indices_by_name,
  1,  pg_query__json_table_path__number_ranges,
  (ProtobufCMessageInit) pg_query__json_table_path__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__json_table_path_scan__field_descriptors[6] =
{
  {
    "plan",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonTablePathScan, plan),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "path",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonTablePathScan, path),
    &pg_query__json_table_path__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "error_on_error",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonTablePathScan, error_on_error),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "child",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonTablePathScan, child),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "col_min",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonTablePathScan, col_min),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "col_max",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonTablePathScan, col_max),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__json_table_path_scan__field_indices_by_name[] = {
  3,   /* field[3] = child */
  5,   /* field[5] = col_max */
  4,   /* field[4] = col_min */
  2,   /* field[2] = error_on_error */
  1,   /* field[1] = path */
  0,   /* field[0] = plan */
};
static const ProtobufCIntRange pg_query__json_table_path_scan__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 6 }
};
const ProtobufCMessageDescriptor pg_query__json_table_path_scan__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.JsonTablePathScan",
  "JsonTablePathScan",
  "PgQuery__JsonTablePathScan",
  "pg_query",
  sizeof(PgQuery__JsonTablePathScan),
  6,
  pg_query__json_table_path_scan__field_descriptors,
  pg_query__json_table_path_scan__field_indices_by_name,
  1,  pg_query__json_table_path_scan__number_ranges,
  (ProtobufCMessageInit) pg_query__json_table_path_scan__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__json_table_sibling_join__field_descriptors[3] =
{
  {
    "plan",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonTableSiblingJoin, plan),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "lplan",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonTableSiblingJoin, lplan),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "rplan",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonTableSiblingJoin, rplan),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__json_table_sibling_join__field_indices_by_name[] = {
  1,   /* field[1] = lplan */
  0,   /* field[0] = plan */
  2,   /* field[2] = rplan */
};
static const ProtobufCIntRange pg_query__json_table_sibling_join__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__json_table_sibling_join__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.JsonTableSiblingJoin",
  "JsonTableSiblingJoin",
  "PgQuery__JsonTableSiblingJoin",
  "pg_query",
  sizeof(PgQuery__JsonTableSiblingJoin),
  3,
  pg_query__json_table_sibling_join__field_descriptors,
  pg_query__json_table_sibling_join__field_indices_by_name,
  1,  pg_query__json_table_sibling_join__number_ranges,
  (ProtobufCMessageInit) pg_query__json_table_sibling_join__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__null_test__field_descriptors[5] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__NullTest, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "arg",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__NullTest, arg),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "nulltesttype",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__NullTest, nulltesttype),
    &pg_query__null_test_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "argisrow",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__NullTest, argisrow),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__NullTest, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__null_test__field_indices_by_name[] = {
  1,   /* field[1] = arg */
  3,   /* field[3] = argisrow */
  4,   /* field[4] = location */
  2,   /* field[2] = nulltesttype */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__null_test__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor pg_query__null_test__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.NullTest",
  "NullTest",
  "PgQuery__NullTest",
  "pg_query",
  sizeof(PgQuery__NullTest),
  5,
  pg_query__null_test__field_descriptors,
  pg_query__null_test__field_indices_by_name,
  1,  pg_query__null_test__number_ranges,
  (ProtobufCMessageInit) pg_query__null_test__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__boolean_test__field_descriptors[4] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__BooleanTest, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "arg",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__BooleanTest, arg),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "booltesttype",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__BooleanTest, booltesttype),
    &pg_query__bool_test_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__BooleanTest, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__boolean_test__field_indices_by_name[] = {
  1,   /* field[1] = arg */
  2,   /* field[2] = booltesttype */
  3,   /* field[3] = location */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__boolean_test__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor pg_query__boolean_test__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.BooleanTest",
  "BooleanTest",
  "PgQuery__BooleanTest",
  "pg_query",
  sizeof(PgQuery__BooleanTest),
  4,
  pg_query__boolean_test__field_descriptors,
  pg_query__boolean_test__field_indices_by_name,
  1,  pg_query__boolean_test__number_ranges,
  (ProtobufCMessageInit) pg_query__boolean_test__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__merge_action__field_descriptors[6] =
{
  {
    "match_kind",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__MergeAction, match_kind),
    &pg_query__merge_match_kind__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "command_type",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__MergeAction, command_type),
    &pg_query__cmd_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "override",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__MergeAction, override),
    &pg_query__overriding_kind__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "qual",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__MergeAction, qual),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "target_list",
    5,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__MergeAction, n_target_list),
    offsetof(PgQuery__MergeAction, target_list),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "update_colnos",
    6,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__MergeAction, n_update_colnos),
    offsetof(PgQuery__MergeAction, update_colnos),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__merge_action__field_indices_by_name[] = {
  1,   /* field[1] = command_type */
  0,   /* field[0] = match_kind */
  2,   /* field[2] = override */
  3,   /* field[3] = qual */
  4,   /* field[4] = target_list */
  5,   /* field[5] = update_colnos */
};
static const ProtobufCIntRange pg_query__merge_action__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 6 }
};
const ProtobufCMessageDescriptor pg_query__merge_action__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.MergeAction",
  "MergeAction",
  "PgQuery__MergeAction",
  "pg_query",
  sizeof(PgQuery__MergeAction),
  6,
  pg_query__merge_action__field_descriptors,
  pg_query__merge_action__field_indices_by_name,
  1,  pg_query__merge_action__number_ranges,
  (ProtobufCMessageInit) pg_query__merge_action__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__coerce_to_domain__field_descriptors[7] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CoerceToDomain, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "arg",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CoerceToDomain, arg),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "resulttype",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CoerceToDomain, resulttype),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "resulttypmod",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CoerceToDomain, resulttypmod),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "resultcollid",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CoerceToDomain, resultcollid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "coercionformat",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CoerceToDomain, coercionformat),
    &pg_query__coercion_form__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CoerceToDomain, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__coerce_to_domain__field_indices_by_name[] = {
  1,   /* field[1] = arg */
  5,   /* field[5] = coercionformat */
  6,   /* field[6] = location */
  4,   /* field[4] = resultcollid */
  2,   /* field[2] = resulttype */
  3,   /* field[3] = resulttypmod */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__coerce_to_domain__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 7 }
};
const ProtobufCMessageDescriptor pg_query__coerce_to_domain__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CoerceToDomain",
  "CoerceToDomain",
  "PgQuery__CoerceToDomain",
  "pg_query",
  sizeof(PgQuery__CoerceToDomain),
  7,
  pg_query__coerce_to_domain__field_descriptors,
  pg_query__coerce_to_domain__field_indices_by_name,
  1,  pg_query__coerce_to_domain__number_ranges,
  (ProtobufCMessageInit) pg_query__coerce_to_domain__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__coerce_to_domain_value__field_descriptors[5] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CoerceToDomainValue, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "type_id",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CoerceToDomainValue, type_id),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "type_mod",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CoerceToDomainValue, type_mod),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "collation",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CoerceToDomainValue, collation),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CoerceToDomainValue, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__coerce_to_domain_value__field_indices_by_name[] = {
  3,   /* field[3] = collation */
  4,   /* field[4] = location */
  1,   /* field[1] = type_id */
  2,   /* field[2] = type_mod */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__coerce_to_domain_value__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor pg_query__coerce_to_domain_value__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CoerceToDomainValue",
  "CoerceToDomainValue",
  "PgQuery__CoerceToDomainValue",
  "pg_query",
  sizeof(PgQuery__CoerceToDomainValue),
  5,
  pg_query__coerce_to_domain_value__field_descriptors,
  pg_query__coerce_to_domain_value__field_indices_by_name,
  1,  pg_query__coerce_to_domain_value__number_ranges,
  (ProtobufCMessageInit) pg_query__coerce_to_domain_value__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__set_to_default__field_descriptors[5] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SetToDefault, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "type_id",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SetToDefault, type_id),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "type_mod",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SetToDefault, type_mod),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "collation",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SetToDefault, collation),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SetToDefault, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__set_to_default__field_indices_by_name[] = {
  3,   /* field[3] = collation */
  4,   /* field[4] = location */
  1,   /* field[1] = type_id */
  2,   /* field[2] = type_mod */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__set_to_default__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor pg_query__set_to_default__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.SetToDefault",
  "SetToDefault",
  "PgQuery__SetToDefault",
  "pg_query",
  sizeof(PgQuery__SetToDefault),
  5,
  pg_query__set_to_default__field_descriptors,
  pg_query__set_to_default__field_indices_by_name,
  1,  pg_query__set_to_default__number_ranges,
  (ProtobufCMessageInit) pg_query__set_to_default__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__current_of_expr__field_descriptors[4] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CurrentOfExpr, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "cvarno",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CurrentOfExpr, cvarno),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "cursor_name",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CurrentOfExpr, cursor_name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "cursor_param",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CurrentOfExpr, cursor_param),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__current_of_expr__field_indices_by_name[] = {
  2,   /* field[2] = cursor_name */
  3,   /* field[3] = cursor_param */
  1,   /* field[1] = cvarno */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__current_of_expr__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor pg_query__current_of_expr__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CurrentOfExpr",
  "CurrentOfExpr",
  "PgQuery__CurrentOfExpr",
  "pg_query",
  sizeof(PgQuery__CurrentOfExpr),
  4,
  pg_query__current_of_expr__field_descriptors,
  pg_query__current_of_expr__field_indices_by_name,
  1,  pg_query__current_of_expr__number_ranges,
  (ProtobufCMessageInit) pg_query__current_of_expr__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__next_value_expr__field_descriptors[3] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__NextValueExpr, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "seqid",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__NextValueExpr, seqid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "type_id",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__NextValueExpr, type_id),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__next_value_expr__field_indices_by_name[] = {
  1,   /* field[1] = seqid */
  2,   /* field[2] = type_id */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__next_value_expr__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__next_value_expr__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.NextValueExpr",
  "NextValueExpr",
  "PgQuery__NextValueExpr",
  "pg_query",
  sizeof(PgQuery__NextValueExpr),
  3,
  pg_query__next_value_expr__field_descriptors,
  pg_query__next_value_expr__field_indices_by_name,
  1,  pg_query__next_value_expr__number_ranges,
  (ProtobufCMessageInit) pg_query__next_value_expr__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__inference_elem__field_descriptors[4] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__InferenceElem, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "expr",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__InferenceElem, expr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "infercollid",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__InferenceElem, infercollid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "inferopclass",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__InferenceElem, inferopclass),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__inference_elem__field_indices_by_name[] = {
  1,   /* field[1] = expr */
  2,   /* field[2] = infercollid */
  3,   /* field[3] = inferopclass */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__inference_elem__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor pg_query__inference_elem__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.InferenceElem",
  "InferenceElem",
  "PgQuery__InferenceElem",
  "pg_query",
  sizeof(PgQuery__InferenceElem),
  4,
  pg_query__inference_elem__field_descriptors,
  pg_query__inference_elem__field_indices_by_name,
  1,  pg_query__inference_elem__number_ranges,
  (ProtobufCMessageInit) pg_query__inference_elem__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__target_entry__field_descriptors[8] =
{
  {
    "xpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__TargetEntry, xpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "expr",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__TargetEntry, expr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "resno",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__TargetEntry, resno),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "resname",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__TargetEntry, resname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "ressortgroupref",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__TargetEntry, ressortgroupref),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "resorigtbl",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__TargetEntry, resorigtbl),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "resorigcol",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__TargetEntry, resorigcol),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "resjunk",
    8,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__TargetEntry, resjunk),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__target_entry__field_indices_by_name[] = {
  1,   /* field[1] = expr */
  7,   /* field[7] = resjunk */
  3,   /* field[3] = resname */
  2,   /* field[2] = resno */
  6,   /* field[6] = resorigcol */
  5,   /* field[5] = resorigtbl */
  4,   /* field[4] = ressortgroupref */
  0,   /* field[0] = xpr */
};
static const ProtobufCIntRange pg_query__target_entry__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 8 }
};
const ProtobufCMessageDescriptor pg_query__target_entry__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.TargetEntry",
  "TargetEntry",
  "PgQuery__TargetEntry",
  "pg_query",
  sizeof(PgQuery__TargetEntry),
  8,
  pg_query__target_entry__field_descriptors,
  pg_query__target_entry__field_indices_by_name,
  1,  pg_query__target_entry__number_ranges,
  (ProtobufCMessageInit) pg_query__target_entry__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__range_tbl_ref__field_descriptors[1] =
{
  {
    "rtindex",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTblRef, rtindex),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__range_tbl_ref__field_indices_by_name[] = {
  0,   /* field[0] = rtindex */
};
static const ProtobufCIntRange pg_query__range_tbl_ref__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 1 }
};
const ProtobufCMessageDescriptor pg_query__range_tbl_ref__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.RangeTblRef",
  "RangeTblRef",
  "PgQuery__RangeTblRef",
  "pg_query",
  sizeof(PgQuery__RangeTblRef),
  1,
  pg_query__range_tbl_ref__field_descriptors,
  pg_query__range_tbl_ref__field_indices_by_name,
  1,  pg_query__range_tbl_ref__number_ranges,
  (ProtobufCMessageInit) pg_query__range_tbl_ref__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__join_expr__field_descriptors[9] =
{
  {
    "jointype",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JoinExpr, jointype),
    &pg_query__join_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "is_natural",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JoinExpr, is_natural),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "larg",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JoinExpr, larg),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "rarg",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JoinExpr, rarg),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "using_clause",
    5,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__JoinExpr, n_using_clause),
    offsetof(PgQuery__JoinExpr, using_clause),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "join_using_alias",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JoinExpr, join_using_alias),
    &pg_query__alias__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "quals",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JoinExpr, quals),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alias",
    8,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JoinExpr, alias),
    &pg_query__alias__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "rtindex",
    9,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JoinExpr, rtindex),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__join_expr__field_indices_by_name[] = {
  7,   /* field[7] = alias */
  1,   /* field[1] = is_natural */
  5,   /* field[5] = join_using_alias */
  0,   /* field[0] = jointype */
  2,   /* field[2] = larg */
  6,   /* field[6] = quals */
  3,   /* field[3] = rarg */
  8,   /* field[8] = rtindex */
  4,   /* field[4] = using_clause */
};
static const ProtobufCIntRange pg_query__join_expr__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 9 }
};
const ProtobufCMessageDescriptor pg_query__join_expr__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.JoinExpr",
  "JoinExpr",
  "PgQuery__JoinExpr",
  "pg_query",
  sizeof(PgQuery__JoinExpr),
  9,
  pg_query__join_expr__field_descriptors,
  pg_query__join_expr__field_indices_by_name,
  1,  pg_query__join_expr__number_ranges,
  (ProtobufCMessageInit) pg_query__join_expr__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__from_expr__field_descriptors[2] =
{
  {
    "fromlist",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__FromExpr, n_fromlist),
    offsetof(PgQuery__FromExpr, fromlist),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "quals",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__FromExpr, quals),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__from_expr__field_indices_by_name[] = {
  0,   /* field[0] = fromlist */
  1,   /* field[1] = quals */
};
static const ProtobufCIntRange pg_query__from_expr__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor pg_query__from_expr__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.FromExpr",
  "FromExpr",
  "PgQuery__FromExpr",
  "pg_query",
  sizeof(PgQuery__FromExpr),
  2,
  pg_query__from_expr__field_descriptors,
  pg_query__from_expr__field_indices_by_name,
  1,  pg_query__from_expr__number_ranges,
  (ProtobufCMessageInit) pg_query__from_expr__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__on_conflict_expr__field_descriptors[8] =
{
  {
    "action",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__OnConflictExpr, action),
    &pg_query__on_conflict_action__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "arbiter_elems",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__OnConflictExpr, n_arbiter_elems),
    offsetof(PgQuery__OnConflictExpr, arbiter_elems),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "arbiter_where",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__OnConflictExpr, arbiter_where),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "constraint",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__OnConflictExpr, constraint),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "on_conflict_set",
    5,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__OnConflictExpr, n_on_conflict_set),
    offsetof(PgQuery__OnConflictExpr, on_conflict_set),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "on_conflict_where",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__OnConflictExpr, on_conflict_where),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "excl_rel_index",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__OnConflictExpr, excl_rel_index),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "excl_rel_tlist",
    8,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__OnConflictExpr, n_excl_rel_tlist),
    offsetof(PgQuery__OnConflictExpr, excl_rel_tlist),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__on_conflict_expr__field_indices_by_name[] = {
  0,   /* field[0] = action */
  1,   /* field[1] = arbiter_elems */
  2,   /* field[2] = arbiter_where */
  3,   /* field[3] = constraint */
  6,   /* field[6] = excl_rel_index */
  7,   /* field[7] = excl_rel_tlist */
  4,   /* field[4] = on_conflict_set */
  5,   /* field[5] = on_conflict_where */
};
static const ProtobufCIntRange pg_query__on_conflict_expr__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 8 }
};
const ProtobufCMessageDescriptor pg_query__on_conflict_expr__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.OnConflictExpr",
  "OnConflictExpr",
  "PgQuery__OnConflictExpr",
  "pg_query",
  sizeof(PgQuery__OnConflictExpr),
  8,
  pg_query__on_conflict_expr__field_descriptors,
  pg_query__on_conflict_expr__field_indices_by_name,
  1,  pg_query__on_conflict_expr__number_ranges,
  (ProtobufCMessageInit) pg_query__on_conflict_expr__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__query__field_descriptors[42] =
{
  {
    "command_type",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Query, command_type),
    &pg_query__cmd_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "query_source",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Query, query_source),
    &pg_query__query_source__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "can_set_tag",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Query, can_set_tag),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "utility_stmt",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Query, utility_stmt),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "result_relation",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Query, result_relation),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "has_aggs",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Query, has_aggs),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "has_window_funcs",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Query, has_window_funcs),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "has_target_srfs",
    8,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Query, has_target_srfs),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "has_sub_links",
    9,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Query, has_sub_links),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "has_distinct_on",
    10,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Query, has_distinct_on),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "has_recursive",
    11,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Query, has_recursive),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "has_modifying_cte",
    12,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Query, has_modifying_cte),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "has_for_update",
    13,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Query, has_for_update),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "has_row_security",
    14,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Query, has_row_security),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "is_return",
    15,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Query, is_return),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "cte_list",
    16,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Query, n_cte_list),
    offsetof(PgQuery__Query, cte_list),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "rtable",
    17,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Query, n_rtable),
    offsetof(PgQuery__Query, rtable),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "rteperminfos",
    18,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Query, n_rteperminfos),
    offsetof(PgQuery__Query, rteperminfos),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "jointree",
    19,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Query, jointree),
    &pg_query__from_expr__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "merge_action_list",
    20,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Query, n_merge_action_list),
    offsetof(PgQuery__Query, merge_action_list),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "merge_target_relation",
    21,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Query, merge_target_relation),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "merge_join_condition",
    22,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Query, merge_join_condition),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "target_list",
    23,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Query, n_target_list),
    offsetof(PgQuery__Query, target_list),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "override",
    24,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Query, override),
    &pg_query__overriding_kind__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "on_conflict",
    25,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Query, on_conflict),
    &pg_query__on_conflict_expr__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "returning_list",
    26,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Query, n_returning_list),
    offsetof(PgQuery__Query, returning_list),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "group_clause",
    27,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Query, n_group_clause),
    offsetof(PgQuery__Query, group_clause),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "group_distinct",
    28,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Query, group_distinct),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "grouping_sets",
    29,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Query, n_grouping_sets),
    offsetof(PgQuery__Query, grouping_sets),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "having_qual",
    30,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Query, having_qual),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "window_clause",
    31,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Query, n_window_clause),
    offsetof(PgQuery__Query, window_clause),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "distinct_clause",
    32,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Query, n_distinct_clause),
    offsetof(PgQuery__Query, distinct_clause),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "sort_clause",
    33,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Query, n_sort_clause),
    offsetof(PgQuery__Query, sort_clause),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "limit_offset",
    34,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Query, limit_offset),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "limit_count",
    35,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Query, limit_count),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "limit_option",
    36,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Query, limit_option),
    &pg_query__limit_option__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "row_marks",
    37,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Query, n_row_marks),
    offsetof(PgQuery__Query, row_marks),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "set_operations",
    38,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Query, set_operations),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "constraint_deps",
    39,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Query, n_constraint_deps),
    offsetof(PgQuery__Query, constraint_deps),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "with_check_options",
    40,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Query, n_with_check_options),
    offsetof(PgQuery__Query, with_check_options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "stmt_location",
    41,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Query, stmt_location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "stmt_len",
    42,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Query, stmt_len),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__query__field_indices_by_name[] = {
  2,   /* field[2] = can_set_tag */
  0,   /* field[0] = command_type */
  38,   /* field[38] = constraint_deps */
  15,   /* field[15] = cte_list */
  31,   /* field[31] = distinct_clause */
  26,   /* field[26] = group_clause */
  27,   /* field[27] = group_distinct */
  28,   /* field[28] = grouping_sets */
  5,   /* field[5] = has_aggs */
  9,   /* field[9] = has_distinct_on */
  12,   /* field[12] = has_for_update */
  11,   /* field[11] = has_modifying_cte */
  10,   /* field[10] = has_recursive */
  13,   /* field[13] = has_row_security */
  8,   /* field[8] = has_sub_links */
  7,   /* field[7] = has_target_srfs */
  6,   /* field[6] = has_window_funcs */
  29,   /* field[29] = having_qual */
  14,   /* field[14] = is_return */
  18,   /* field[18] = jointree */
  34,   /* field[34] = limit_count */
  33,   /* field[33] = limit_offset */
  35,   /* field[35] = limit_option */
  19,   /* field[19] = merge_action_list */
  21,   /* field[21] = merge_join_condition */
  20,   /* field[20] = merge_target_relation */
  24,   /* field[24] = on_conflict */
  23,   /* field[23] = override */
  1,   /* field[1] = query_source */
  4,   /* field[4] = result_relation */
  25,   /* field[25] = returning_list */
  36,   /* field[36] = row_marks */
  16,   /* field[16] = rtable */
  17,   /* field[17] = rteperminfos */
  37,   /* field[37] = set_operations */
  32,   /* field[32] = sort_clause */
  41,   /* field[41] = stmt_len */
  40,   /* field[40] = stmt_location */
  22,   /* field[22] = target_list */
  3,   /* field[3] = utility_stmt */
  30,   /* field[30] = window_clause */
  39,   /* field[39] = with_check_options */
};
static const ProtobufCIntRange pg_query__query__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 42 }
};
const ProtobufCMessageDescriptor pg_query__query__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.Query",
  "Query",
  "PgQuery__Query",
  "pg_query",
  sizeof(PgQuery__Query),
  42,
  pg_query__query__field_descriptors,
  pg_query__query__field_indices_by_name,
  1,  pg_query__query__number_ranges,
  (ProtobufCMessageInit) pg_query__query__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__type_name__field_descriptors[8] =
{
  {
    "names",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__TypeName, n_names),
    offsetof(PgQuery__TypeName, names),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "type_oid",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__TypeName, type_oid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "setof",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__TypeName, setof),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "pct_type",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__TypeName, pct_type),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "typmods",
    5,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__TypeName, n_typmods),
    offsetof(PgQuery__TypeName, typmods),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "typemod",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__TypeName, typemod),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "array_bounds",
    7,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__TypeName, n_array_bounds),
    offsetof(PgQuery__TypeName, array_bounds),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    8,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__TypeName, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__type_name__field_indices_by_name[] = {
  6,   /* field[6] = array_bounds */
  7,   /* field[7] = location */
  0,   /* field[0] = names */
  3,   /* field[3] = pct_type */
  2,   /* field[2] = setof */
  1,   /* field[1] = type_oid */
  5,   /* field[5] = typemod */
  4,   /* field[4] = typmods */
};
static const ProtobufCIntRange pg_query__type_name__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 8 }
};
const ProtobufCMessageDescriptor pg_query__type_name__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.TypeName",
  "TypeName",
  "PgQuery__TypeName",
  "pg_query",
  sizeof(PgQuery__TypeName),
  8,
  pg_query__type_name__field_descriptors,
  pg_query__type_name__field_indices_by_name,
  1,  pg_query__type_name__number_ranges,
  (ProtobufCMessageInit) pg_query__type_name__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__column_ref__field_descriptors[2] =
{
  {
    "fields",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__ColumnRef, n_fields),
    offsetof(PgQuery__ColumnRef, fields),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ColumnRef, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__column_ref__field_indices_by_name[] = {
  0,   /* field[0] = fields */
  1,   /* field[1] = location */
};
static const ProtobufCIntRange pg_query__column_ref__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor pg_query__column_ref__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.ColumnRef",
  "ColumnRef",
  "PgQuery__ColumnRef",
  "pg_query",
  sizeof(PgQuery__ColumnRef),
  2,
  pg_query__column_ref__field_descriptors,
  pg_query__column_ref__field_indices_by_name,
  1,  pg_query__column_ref__number_ranges,
  (ProtobufCMessageInit) pg_query__column_ref__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__param_ref__field_descriptors[2] =
{
  {
    "number",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ParamRef, number),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ParamRef, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__param_ref__field_indices_by_name[] = {
  1,   /* field[1] = location */
  0,   /* field[0] = number */
};
static const ProtobufCIntRange pg_query__param_ref__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor pg_query__param_ref__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.ParamRef",
  "ParamRef",
  "PgQuery__ParamRef",
  "pg_query",
  sizeof(PgQuery__ParamRef),
  2,
  pg_query__param_ref__field_descriptors,
  pg_query__param_ref__field_indices_by_name,
  1,  pg_query__param_ref__number_ranges,
  (ProtobufCMessageInit) pg_query__param_ref__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__a__expr__field_descriptors[5] =
{
  {
    "kind",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AExpr, kind),
    &pg_query__a__expr__kind__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "name",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AExpr, n_name),
    offsetof(PgQuery__AExpr, name),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "lexpr",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AExpr, lexpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "rexpr",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AExpr, rexpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AExpr, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__a__expr__field_indices_by_name[] = {
  0,   /* field[0] = kind */
  2,   /* field[2] = lexpr */
  4,   /* field[4] = location */
  1,   /* field[1] = name */
  3,   /* field[3] = rexpr */
};
static const ProtobufCIntRange pg_query__a__expr__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor pg_query__a__expr__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.A_Expr",
  "AExpr",
  "PgQuery__AExpr",
  "pg_query",
  sizeof(PgQuery__AExpr),
  5,
  pg_query__a__expr__field_descriptors,
  pg_query__a__expr__field_indices_by_name,
  1,  pg_query__a__expr__number_ranges,
  (ProtobufCMessageInit) pg_query__a__expr__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__type_cast__field_descriptors[3] =
{
  {
    "arg",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__TypeCast, arg),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "type_name",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__TypeCast, type_name),
    &pg_query__type_name__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__TypeCast, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__type_cast__field_indices_by_name[] = {
  0,   /* field[0] = arg */
  2,   /* field[2] = location */
  1,   /* field[1] = type_name */
};
static const ProtobufCIntRange pg_query__type_cast__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__type_cast__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.TypeCast",
  "TypeCast",
  "PgQuery__TypeCast",
  "pg_query",
  sizeof(PgQuery__TypeCast),
  3,
  pg_query__type_cast__field_descriptors,
  pg_query__type_cast__field_indices_by_name,
  1,  pg_query__type_cast__number_ranges,
  (ProtobufCMessageInit) pg_query__type_cast__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__collate_clause__field_descriptors[3] =
{
  {
    "arg",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CollateClause, arg),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "collname",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CollateClause, n_collname),
    offsetof(PgQuery__CollateClause, collname),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CollateClause, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__collate_clause__field_indices_by_name[] = {
  0,   /* field[0] = arg */
  1,   /* field[1] = collname */
  2,   /* field[2] = location */
};
static const ProtobufCIntRange pg_query__collate_clause__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__collate_clause__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CollateClause",
  "CollateClause",
  "PgQuery__CollateClause",
  "pg_query",
  sizeof(PgQuery__CollateClause),
  3,
  pg_query__collate_clause__field_descriptors,
  pg_query__collate_clause__field_indices_by_name,
  1,  pg_query__collate_clause__number_ranges,
  (ProtobufCMessageInit) pg_query__collate_clause__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__role_spec__field_descriptors[3] =
{
  {
    "roletype",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RoleSpec, roletype),
    &pg_query__role_spec_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "rolename",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RoleSpec, rolename),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RoleSpec, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__role_spec__field_indices_by_name[] = {
  2,   /* field[2] = location */
  1,   /* field[1] = rolename */
  0,   /* field[0] = roletype */
};
static const ProtobufCIntRange pg_query__role_spec__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__role_spec__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.RoleSpec",
  "RoleSpec",
  "PgQuery__RoleSpec",
  "pg_query",
  sizeof(PgQuery__RoleSpec),
  3,
  pg_query__role_spec__field_descriptors,
  pg_query__role_spec__field_indices_by_name,
  1,  pg_query__role_spec__number_ranges,
  (ProtobufCMessageInit) pg_query__role_spec__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__func_call__field_descriptors[11] =
{
  {
    "funcname",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__FuncCall, n_funcname),
    offsetof(PgQuery__FuncCall, funcname),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "args",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__FuncCall, n_args),
    offsetof(PgQuery__FuncCall, args),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "agg_order",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__FuncCall, n_agg_order),
    offsetof(PgQuery__FuncCall, agg_order),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "agg_filter",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__FuncCall, agg_filter),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "over",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__FuncCall, over),
    &pg_query__window_def__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "agg_within_group",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__FuncCall, agg_within_group),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "agg_star",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__FuncCall, agg_star),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "agg_distinct",
    8,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__FuncCall, agg_distinct),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "func_variadic",
    9,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__FuncCall, func_variadic),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "funcformat",
    10,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__FuncCall, funcformat),
    &pg_query__coercion_form__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    11,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__FuncCall, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__func_call__field_indices_by_name[] = {
  7,   /* field[7] = agg_distinct */
  3,   /* field[3] = agg_filter */
  2,   /* field[2] = agg_order */
  6,   /* field[6] = agg_star */
  5,   /* field[5] = agg_within_group */
  1,   /* field[1] = args */
  8,   /* field[8] = func_variadic */
  9,   /* field[9] = funcformat */
  0,   /* field[0] = funcname */
  10,   /* field[10] = location */
  4,   /* field[4] = over */
};
static const ProtobufCIntRange pg_query__func_call__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 11 }
};
const ProtobufCMessageDescriptor pg_query__func_call__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.FuncCall",
  "FuncCall",
  "PgQuery__FuncCall",
  "pg_query",
  sizeof(PgQuery__FuncCall),
  11,
  pg_query__func_call__field_descriptors,
  pg_query__func_call__field_indices_by_name,
  1,  pg_query__func_call__number_ranges,
  (ProtobufCMessageInit) pg_query__func_call__init,
  NULL,NULL,NULL    /* reserved[123] */
};
#define pg_query__a__star__field_descriptors NULL
#define pg_query__a__star__field_indices_by_name NULL
#define pg_query__a__star__number_ranges NULL
const ProtobufCMessageDescriptor pg_query__a__star__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.A_Star",
  "AStar",
  "PgQuery__AStar",
  "pg_query",
  sizeof(PgQuery__AStar),
  0,
  pg_query__a__star__field_descriptors,
  pg_query__a__star__field_indices_by_name,
  0,  pg_query__a__star__number_ranges,
  (ProtobufCMessageInit) pg_query__a__star__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__a__indices__field_descriptors[3] =
{
  {
    "is_slice",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AIndices, is_slice),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "lidx",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AIndices, lidx),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "uidx",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AIndices, uidx),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__a__indices__field_indices_by_name[] = {
  0,   /* field[0] = is_slice */
  1,   /* field[1] = lidx */
  2,   /* field[2] = uidx */
};
static const ProtobufCIntRange pg_query__a__indices__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__a__indices__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.A_Indices",
  "AIndices",
  "PgQuery__AIndices",
  "pg_query",
  sizeof(PgQuery__AIndices),
  3,
  pg_query__a__indices__field_descriptors,
  pg_query__a__indices__field_indices_by_name,
  1,  pg_query__a__indices__number_ranges,
  (ProtobufCMessageInit) pg_query__a__indices__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__a__indirection__field_descriptors[2] =
{
  {
    "arg",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AIndirection, arg),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "indirection",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AIndirection, n_indirection),
    offsetof(PgQuery__AIndirection, indirection),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__a__indirection__field_indices_by_name[] = {
  0,   /* field[0] = arg */
  1,   /* field[1] = indirection */
};
static const ProtobufCIntRange pg_query__a__indirection__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor pg_query__a__indirection__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.A_Indirection",
  "AIndirection",
  "PgQuery__AIndirection",
  "pg_query",
  sizeof(PgQuery__AIndirection),
  2,
  pg_query__a__indirection__field_descriptors,
  pg_query__a__indirection__field_indices_by_name,
  1,  pg_query__a__indirection__number_ranges,
  (ProtobufCMessageInit) pg_query__a__indirection__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__a__array_expr__field_descriptors[2] =
{
  {
    "elements",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AArrayExpr, n_elements),
    offsetof(PgQuery__AArrayExpr, elements),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AArrayExpr, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__a__array_expr__field_indices_by_name[] = {
  0,   /* field[0] = elements */
  1,   /* field[1] = location */
};
static const ProtobufCIntRange pg_query__a__array_expr__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor pg_query__a__array_expr__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.A_ArrayExpr",
  "AArrayExpr",
  "PgQuery__AArrayExpr",
  "pg_query",
  sizeof(PgQuery__AArrayExpr),
  2,
  pg_query__a__array_expr__field_descriptors,
  pg_query__a__array_expr__field_indices_by_name,
  1,  pg_query__a__array_expr__number_ranges,
  (ProtobufCMessageInit) pg_query__a__array_expr__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__res_target__field_descriptors[4] =
{
  {
    "name",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ResTarget, name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "indirection",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__ResTarget, n_indirection),
    offsetof(PgQuery__ResTarget, indirection),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "val",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ResTarget, val),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ResTarget, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__res_target__field_indices_by_name[] = {
  1,   /* field[1] = indirection */
  3,   /* field[3] = location */
  0,   /* field[0] = name */
  2,   /* field[2] = val */
};
static const ProtobufCIntRange pg_query__res_target__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor pg_query__res_target__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.ResTarget",
  "ResTarget",
  "PgQuery__ResTarget",
  "pg_query",
  sizeof(PgQuery__ResTarget),
  4,
  pg_query__res_target__field_descriptors,
  pg_query__res_target__field_indices_by_name,
  1,  pg_query__res_target__number_ranges,
  (ProtobufCMessageInit) pg_query__res_target__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__multi_assign_ref__field_descriptors[3] =
{
  {
    "source",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__MultiAssignRef, source),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "colno",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__MultiAssignRef, colno),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "ncolumns",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__MultiAssignRef, ncolumns),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__multi_assign_ref__field_indices_by_name[] = {
  1,   /* field[1] = colno */
  2,   /* field[2] = ncolumns */
  0,   /* field[0] = source */
};
static const ProtobufCIntRange pg_query__multi_assign_ref__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__multi_assign_ref__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.MultiAssignRef",
  "MultiAssignRef",
  "PgQuery__MultiAssignRef",
  "pg_query",
  sizeof(PgQuery__MultiAssignRef),
  3,
  pg_query__multi_assign_ref__field_descriptors,
  pg_query__multi_assign_ref__field_indices_by_name,
  1,  pg_query__multi_assign_ref__number_ranges,
  (ProtobufCMessageInit) pg_query__multi_assign_ref__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__sort_by__field_descriptors[5] =
{
  {
    "node",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SortBy, node),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "sortby_dir",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SortBy, sortby_dir),
    &pg_query__sort_by_dir__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "sortby_nulls",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SortBy, sortby_nulls),
    &pg_query__sort_by_nulls__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "use_op",
    4,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__SortBy, n_use_op),
    offsetof(PgQuery__SortBy, use_op),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SortBy, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__sort_by__field_indices_by_name[] = {
  4,   /* field[4] = location */
  0,   /* field[0] = node */
  1,   /* field[1] = sortby_dir */
  2,   /* field[2] = sortby_nulls */
  3,   /* field[3] = use_op */
};
static const ProtobufCIntRange pg_query__sort_by__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor pg_query__sort_by__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.SortBy",
  "SortBy",
  "PgQuery__SortBy",
  "pg_query",
  sizeof(PgQuery__SortBy),
  5,
  pg_query__sort_by__field_descriptors,
  pg_query__sort_by__field_indices_by_name,
  1,  pg_query__sort_by__number_ranges,
  (ProtobufCMessageInit) pg_query__sort_by__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__window_def__field_descriptors[8] =
{
  {
    "name",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WindowDef, name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "refname",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WindowDef, refname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "partition_clause",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__WindowDef, n_partition_clause),
    offsetof(PgQuery__WindowDef, partition_clause),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "order_clause",
    4,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__WindowDef, n_order_clause),
    offsetof(PgQuery__WindowDef, order_clause),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "frame_options",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WindowDef, frame_options),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "start_offset",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WindowDef, start_offset),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "end_offset",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WindowDef, end_offset),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    8,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WindowDef, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__window_def__field_indices_by_name[] = {
  6,   /* field[6] = end_offset */
  4,   /* field[4] = frame_options */
  7,   /* field[7] = location */
  0,   /* field[0] = name */
  3,   /* field[3] = order_clause */
  2,   /* field[2] = partition_clause */
  1,   /* field[1] = refname */
  5,   /* field[5] = start_offset */
};
static const ProtobufCIntRange pg_query__window_def__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 8 }
};
const ProtobufCMessageDescriptor pg_query__window_def__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.WindowDef",
  "WindowDef",
  "PgQuery__WindowDef",
  "pg_query",
  sizeof(PgQuery__WindowDef),
  8,
  pg_query__window_def__field_descriptors,
  pg_query__window_def__field_indices_by_name,
  1,  pg_query__window_def__number_ranges,
  (ProtobufCMessageInit) pg_query__window_def__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__range_subselect__field_descriptors[3] =
{
  {
    "lateral",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeSubselect, lateral),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "subquery",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeSubselect, subquery),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alias",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeSubselect, alias),
    &pg_query__alias__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__range_subselect__field_indices_by_name[] = {
  2,   /* field[2] = alias */
  0,   /* field[0] = lateral */
  1,   /* field[1] = subquery */
};
static const ProtobufCIntRange pg_query__range_subselect__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__range_subselect__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.RangeSubselect",
  "RangeSubselect",
  "PgQuery__RangeSubselect",
  "pg_query",
  sizeof(PgQuery__RangeSubselect),
  3,
  pg_query__range_subselect__field_descriptors,
  pg_query__range_subselect__field_indices_by_name,
  1,  pg_query__range_subselect__number_ranges,
  (ProtobufCMessageInit) pg_query__range_subselect__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__range_function__field_descriptors[6] =
{
  {
    "lateral",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeFunction, lateral),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "ordinality",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeFunction, ordinality),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "is_rowsfrom",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeFunction, is_rowsfrom),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "functions",
    4,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__RangeFunction, n_functions),
    offsetof(PgQuery__RangeFunction, functions),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alias",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeFunction, alias),
    &pg_query__alias__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "coldeflist",
    6,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__RangeFunction, n_coldeflist),
    offsetof(PgQuery__RangeFunction, coldeflist),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__range_function__field_indices_by_name[] = {
  4,   /* field[4] = alias */
  5,   /* field[5] = coldeflist */
  3,   /* field[3] = functions */
  2,   /* field[2] = is_rowsfrom */
  0,   /* field[0] = lateral */
  1,   /* field[1] = ordinality */
};
static const ProtobufCIntRange pg_query__range_function__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 6 }
};
const ProtobufCMessageDescriptor pg_query__range_function__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.RangeFunction",
  "RangeFunction",
  "PgQuery__RangeFunction",
  "pg_query",
  sizeof(PgQuery__RangeFunction),
  6,
  pg_query__range_function__field_descriptors,
  pg_query__range_function__field_indices_by_name,
  1,  pg_query__range_function__number_ranges,
  (ProtobufCMessageInit) pg_query__range_function__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__range_table_func__field_descriptors[7] =
{
  {
    "lateral",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTableFunc, lateral),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "docexpr",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTableFunc, docexpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "rowexpr",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTableFunc, rowexpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "namespaces",
    4,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__RangeTableFunc, n_namespaces),
    offsetof(PgQuery__RangeTableFunc, namespaces),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "columns",
    5,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__RangeTableFunc, n_columns),
    offsetof(PgQuery__RangeTableFunc, columns),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alias",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTableFunc, alias),
    &pg_query__alias__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTableFunc, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__range_table_func__field_indices_by_name[] = {
  5,   /* field[5] = alias */
  4,   /* field[4] = columns */
  1,   /* field[1] = docexpr */
  0,   /* field[0] = lateral */
  6,   /* field[6] = location */
  3,   /* field[3] = namespaces */
  2,   /* field[2] = rowexpr */
};
static const ProtobufCIntRange pg_query__range_table_func__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 7 }
};
const ProtobufCMessageDescriptor pg_query__range_table_func__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.RangeTableFunc",
  "RangeTableFunc",
  "PgQuery__RangeTableFunc",
  "pg_query",
  sizeof(PgQuery__RangeTableFunc),
  7,
  pg_query__range_table_func__field_descriptors,
  pg_query__range_table_func__field_indices_by_name,
  1,  pg_query__range_table_func__number_ranges,
  (ProtobufCMessageInit) pg_query__range_table_func__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__range_table_func_col__field_descriptors[7] =
{
  {
    "colname",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTableFuncCol, colname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "type_name",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTableFuncCol, type_name),
    &pg_query__type_name__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "for_ordinality",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTableFuncCol, for_ordinality),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "is_not_null",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTableFuncCol, is_not_null),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "colexpr",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTableFuncCol, colexpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "coldefexpr",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTableFuncCol, coldefexpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTableFuncCol, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__range_table_func_col__field_indices_by_name[] = {
  5,   /* field[5] = coldefexpr */
  4,   /* field[4] = colexpr */
  0,   /* field[0] = colname */
  2,   /* field[2] = for_ordinality */
  3,   /* field[3] = is_not_null */
  6,   /* field[6] = location */
  1,   /* field[1] = type_name */
};
static const ProtobufCIntRange pg_query__range_table_func_col__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 7 }
};
const ProtobufCMessageDescriptor pg_query__range_table_func_col__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.RangeTableFuncCol",
  "RangeTableFuncCol",
  "PgQuery__RangeTableFuncCol",
  "pg_query",
  sizeof(PgQuery__RangeTableFuncCol),
  7,
  pg_query__range_table_func_col__field_descriptors,
  pg_query__range_table_func_col__field_indices_by_name,
  1,  pg_query__range_table_func_col__number_ranges,
  (ProtobufCMessageInit) pg_query__range_table_func_col__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__range_table_sample__field_descriptors[5] =
{
  {
    "relation",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTableSample, relation),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "method",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__RangeTableSample, n_method),
    offsetof(PgQuery__RangeTableSample, method),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "args",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__RangeTableSample, n_args),
    offsetof(PgQuery__RangeTableSample, args),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "repeatable",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTableSample, repeatable),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTableSample, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__range_table_sample__field_indices_by_name[] = {
  2,   /* field[2] = args */
  4,   /* field[4] = location */
  1,   /* field[1] = method */
  0,   /* field[0] = relation */
  3,   /* field[3] = repeatable */
};
static const ProtobufCIntRange pg_query__range_table_sample__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor pg_query__range_table_sample__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.RangeTableSample",
  "RangeTableSample",
  "PgQuery__RangeTableSample",
  "pg_query",
  sizeof(PgQuery__RangeTableSample),
  5,
  pg_query__range_table_sample__field_descriptors,
  pg_query__range_table_sample__field_indices_by_name,
  1,  pg_query__range_table_sample__number_ranges,
  (ProtobufCMessageInit) pg_query__range_table_sample__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__column_def__field_descriptors[19] =
{
  {
    "colname",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ColumnDef, colname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "type_name",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ColumnDef, type_name),
    &pg_query__type_name__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "compression",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ColumnDef, compression),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "inhcount",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ColumnDef, inhcount),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "is_local",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ColumnDef, is_local),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "is_not_null",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ColumnDef, is_not_null),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "is_from_type",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ColumnDef, is_from_type),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "storage",
    8,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ColumnDef, storage),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "storage_name",
    9,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ColumnDef, storage_name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "raw_default",
    10,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ColumnDef, raw_default),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "cooked_default",
    11,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ColumnDef, cooked_default),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "identity",
    12,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ColumnDef, identity),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "identity_sequence",
    13,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ColumnDef, identity_sequence),
    &pg_query__range_var__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "generated",
    14,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ColumnDef, generated),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "coll_clause",
    15,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ColumnDef, coll_clause),
    &pg_query__collate_clause__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "coll_oid",
    16,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ColumnDef, coll_oid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "constraints",
    17,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__ColumnDef, n_constraints),
    offsetof(PgQuery__ColumnDef, constraints),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "fdwoptions",
    18,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__ColumnDef, n_fdwoptions),
    offsetof(PgQuery__ColumnDef, fdwoptions),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    19,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ColumnDef, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__column_def__field_indices_by_name[] = {
  14,   /* field[14] = coll_clause */
  15,   /* field[15] = coll_oid */
  0,   /* field[0] = colname */
  2,   /* field[2] = compression */
  16,   /* field[16] = constraints */
  10,   /* field[10] = cooked_default */
  17,   /* field[17] = fdwoptions */
  13,   /* field[13] = generated */
  11,   /* field[11] = identity */
  12,   /* field[12] = identity_sequence */
  3,   /* field[3] = inhcount */
  6,   /* field[6] = is_from_type */
  4,   /* field[4] = is_local */
  5,   /* field[5] = is_not_null */
  18,   /* field[18] = location */
  9,   /* field[9] = raw_default */
  7,   /* field[7] = storage */
  8,   /* field[8] = storage_name */
  1,   /* field[1] = type_name */
};
static const ProtobufCIntRange pg_query__column_def__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 19 }
};
const ProtobufCMessageDescriptor pg_query__column_def__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.ColumnDef",
  "ColumnDef",
  "PgQuery__ColumnDef",
  "pg_query",
  sizeof(PgQuery__ColumnDef),
  19,
  pg_query__column_def__field_descriptors,
  pg_query__column_def__field_indices_by_name,
  1,  pg_query__column_def__number_ranges,
  (ProtobufCMessageInit) pg_query__column_def__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__table_like_clause__field_descriptors[3] =
{
  {
    "relation",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__TableLikeClause, relation),
    &pg_query__range_var__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "options",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__TableLikeClause, options),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "relation_oid",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__TableLikeClause, relation_oid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__table_like_clause__field_indices_by_name[] = {
  1,   /* field[1] = options */
  0,   /* field[0] = relation */
  2,   /* field[2] = relation_oid */
};
static const ProtobufCIntRange pg_query__table_like_clause__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__table_like_clause__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.TableLikeClause",
  "TableLikeClause",
  "PgQuery__TableLikeClause",
  "pg_query",
  sizeof(PgQuery__TableLikeClause),
  3,
  pg_query__table_like_clause__field_descriptors,
  pg_query__table_like_clause__field_indices_by_name,
  1,  pg_query__table_like_clause__number_ranges,
  (ProtobufCMessageInit) pg_query__table_like_clause__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__index_elem__field_descriptors[8] =
{
  {
    "name",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__IndexElem, name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "expr",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__IndexElem, expr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "indexcolname",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__IndexElem, indexcolname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "collation",
    4,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__IndexElem, n_collation),
    offsetof(PgQuery__IndexElem, collation),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "opclass",
    5,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__IndexElem, n_opclass),
    offsetof(PgQuery__IndexElem, opclass),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "opclassopts",
    6,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__IndexElem, n_opclassopts),
    offsetof(PgQuery__IndexElem, opclassopts),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "ordering",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__IndexElem, ordering),
    &pg_query__sort_by_dir__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "nulls_ordering",
    8,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__IndexElem, nulls_ordering),
    &pg_query__sort_by_nulls__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__index_elem__field_indices_by_name[] = {
  3,   /* field[3] = collation */
  1,   /* field[1] = expr */
  2,   /* field[2] = indexcolname */
  0,   /* field[0] = name */
  7,   /* field[7] = nulls_ordering */
  4,   /* field[4] = opclass */
  5,   /* field[5] = opclassopts */
  6,   /* field[6] = ordering */
};
static const ProtobufCIntRange pg_query__index_elem__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 8 }
};
const ProtobufCMessageDescriptor pg_query__index_elem__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.IndexElem",
  "IndexElem",
  "PgQuery__IndexElem",
  "pg_query",
  sizeof(PgQuery__IndexElem),
  8,
  pg_query__index_elem__field_descriptors,
  pg_query__index_elem__field_indices_by_name,
  1,  pg_query__index_elem__number_ranges,
  (ProtobufCMessageInit) pg_query__index_elem__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__def_elem__field_descriptors[5] =
{
  {
    "defnamespace",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DefElem, defnamespace),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "defname",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DefElem, defname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "arg",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DefElem, arg),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "defaction",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DefElem, defaction),
    &pg_query__def_elem_action__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DefElem, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__def_elem__field_indices_by_name[] = {
  2,   /* field[2] = arg */
  3,   /* field[3] = defaction */
  1,   /* field[1] = defname */
  0,   /* field[0] = defnamespace */
  4,   /* field[4] = location */
};
static const ProtobufCIntRange pg_query__def_elem__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor pg_query__def_elem__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.DefElem",
  "DefElem",
  "PgQuery__DefElem",
  "pg_query",
  sizeof(PgQuery__DefElem),
  5,
  pg_query__def_elem__field_descriptors,
  pg_query__def_elem__field_indices_by_name,
  1,  pg_query__def_elem__number_ranges,
  (ProtobufCMessageInit) pg_query__def_elem__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__locking_clause__field_descriptors[3] =
{
  {
    "locked_rels",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__LockingClause, n_locked_rels),
    offsetof(PgQuery__LockingClause, locked_rels),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "strength",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__LockingClause, strength),
    &pg_query__lock_clause_strength__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "wait_policy",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__LockingClause, wait_policy),
    &pg_query__lock_wait_policy__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__locking_clause__field_indices_by_name[] = {
  0,   /* field[0] = locked_rels */
  1,   /* field[1] = strength */
  2,   /* field[2] = wait_policy */
};
static const ProtobufCIntRange pg_query__locking_clause__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__locking_clause__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.LockingClause",
  "LockingClause",
  "PgQuery__LockingClause",
  "pg_query",
  sizeof(PgQuery__LockingClause),
  3,
  pg_query__locking_clause__field_descriptors,
  pg_query__locking_clause__field_indices_by_name,
  1,  pg_query__locking_clause__number_ranges,
  (ProtobufCMessageInit) pg_query__locking_clause__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__xml_serialize__field_descriptors[5] =
{
  {
    "xmloption",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__XmlSerialize, xmloption),
    &pg_query__xml_option_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "expr",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__XmlSerialize, expr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "type_name",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__XmlSerialize, type_name),
    &pg_query__type_name__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "indent",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__XmlSerialize, indent),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__XmlSerialize, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__xml_serialize__field_indices_by_name[] = {
  1,   /* field[1] = expr */
  3,   /* field[3] = indent */
  4,   /* field[4] = location */
  2,   /* field[2] = type_name */
  0,   /* field[0] = xmloption */
};
static const ProtobufCIntRange pg_query__xml_serialize__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor pg_query__xml_serialize__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.XmlSerialize",
  "XmlSerialize",
  "PgQuery__XmlSerialize",
  "pg_query",
  sizeof(PgQuery__XmlSerialize),
  5,
  pg_query__xml_serialize__field_descriptors,
  pg_query__xml_serialize__field_indices_by_name,
  1,  pg_query__xml_serialize__number_ranges,
  (ProtobufCMessageInit) pg_query__xml_serialize__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__partition_elem__field_descriptors[5] =
{
  {
    "name",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__PartitionElem, name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "expr",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__PartitionElem, expr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "collation",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__PartitionElem, n_collation),
    offsetof(PgQuery__PartitionElem, collation),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "opclass",
    4,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__PartitionElem, n_opclass),
    offsetof(PgQuery__PartitionElem, opclass),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__PartitionElem, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__partition_elem__field_indices_by_name[] = {
  2,   /* field[2] = collation */
  1,   /* field[1] = expr */
  4,   /* field[4] = location */
  0,   /* field[0] = name */
  3,   /* field[3] = opclass */
};
static const ProtobufCIntRange pg_query__partition_elem__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor pg_query__partition_elem__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.PartitionElem",
  "PartitionElem",
  "PgQuery__PartitionElem",
  "pg_query",
  sizeof(PgQuery__PartitionElem),
  5,
  pg_query__partition_elem__field_descriptors,
  pg_query__partition_elem__field_indices_by_name,
  1,  pg_query__partition_elem__number_ranges,
  (ProtobufCMessageInit) pg_query__partition_elem__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__partition_spec__field_descriptors[3] =
{
  {
    "strategy",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__PartitionSpec, strategy),
    &pg_query__partition_strategy__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "part_params",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__PartitionSpec, n_part_params),
    offsetof(PgQuery__PartitionSpec, part_params),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__PartitionSpec, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__partition_spec__field_indices_by_name[] = {
  2,   /* field[2] = location */
  1,   /* field[1] = part_params */
  0,   /* field[0] = strategy */
};
static const ProtobufCIntRange pg_query__partition_spec__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__partition_spec__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.PartitionSpec",
  "PartitionSpec",
  "PgQuery__PartitionSpec",
  "pg_query",
  sizeof(PgQuery__PartitionSpec),
  3,
  pg_query__partition_spec__field_descriptors,
  pg_query__partition_spec__field_indices_by_name,
  1,  pg_query__partition_spec__number_ranges,
  (ProtobufCMessageInit) pg_query__partition_spec__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__partition_bound_spec__field_descriptors[8] =
{
  {
    "strategy",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__PartitionBoundSpec, strategy),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "is_default",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__PartitionBoundSpec, is_default),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "modulus",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__PartitionBoundSpec, modulus),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "remainder",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__PartitionBoundSpec, remainder),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "listdatums",
    5,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__PartitionBoundSpec, n_listdatums),
    offsetof(PgQuery__PartitionBoundSpec, listdatums),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "lowerdatums",
    6,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__PartitionBoundSpec, n_lowerdatums),
    offsetof(PgQuery__PartitionBoundSpec, lowerdatums),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "upperdatums",
    7,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__PartitionBoundSpec, n_upperdatums),
    offsetof(PgQuery__PartitionBoundSpec, upperdatums),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    8,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__PartitionBoundSpec, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__partition_bound_spec__field_indices_by_name[] = {
  1,   /* field[1] = is_default */
  4,   /* field[4] = listdatums */
  7,   /* field[7] = location */
  5,   /* field[5] = lowerdatums */
  2,   /* field[2] = modulus */
  3,   /* field[3] = remainder */
  0,   /* field[0] = strategy */
  6,   /* field[6] = upperdatums */
};
static const ProtobufCIntRange pg_query__partition_bound_spec__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 8 }
};
const ProtobufCMessageDescriptor pg_query__partition_bound_spec__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.PartitionBoundSpec",
  "PartitionBoundSpec",
  "PgQuery__PartitionBoundSpec",
  "pg_query",
  sizeof(PgQuery__PartitionBoundSpec),
  8,
  pg_query__partition_bound_spec__field_descriptors,
  pg_query__partition_bound_spec__field_indices_by_name,
  1,  pg_query__partition_bound_spec__number_ranges,
  (ProtobufCMessageInit) pg_query__partition_bound_spec__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__partition_range_datum__field_descriptors[3] =
{
  {
    "kind",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__PartitionRangeDatum, kind),
    &pg_query__partition_range_datum_kind__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "value",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__PartitionRangeDatum, value),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__PartitionRangeDatum, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__partition_range_datum__field_indices_by_name[] = {
  0,   /* field[0] = kind */
  2,   /* field[2] = location */
  1,   /* field[1] = value */
};
static const ProtobufCIntRange pg_query__partition_range_datum__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__partition_range_datum__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.PartitionRangeDatum",
  "PartitionRangeDatum",
  "PgQuery__PartitionRangeDatum",
  "pg_query",
  sizeof(PgQuery__PartitionRangeDatum),
  3,
  pg_query__partition_range_datum__field_descriptors,
  pg_query__partition_range_datum__field_indices_by_name,
  1,  pg_query__partition_range_datum__number_ranges,
  (ProtobufCMessageInit) pg_query__partition_range_datum__init,
  NULL,NULL,NULL    /* reserved[123] */
};
#define pg_query__single_partition_spec__field_descriptors NULL
#define pg_query__single_partition_spec__field_indices_by_name NULL
#define pg_query__single_partition_spec__number_ranges NULL
const ProtobufCMessageDescriptor pg_query__single_partition_spec__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.SinglePartitionSpec",
  "SinglePartitionSpec",
  "PgQuery__SinglePartitionSpec",
  "pg_query",
  sizeof(PgQuery__SinglePartitionSpec),
  0,
  pg_query__single_partition_spec__field_descriptors,
  pg_query__single_partition_spec__field_indices_by_name,
  0,  pg_query__single_partition_spec__number_ranges,
  (ProtobufCMessageInit) pg_query__single_partition_spec__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__partition_cmd__field_descriptors[3] =
{
  {
    "name",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__PartitionCmd, name),
    &pg_query__range_var__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "bound",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__PartitionCmd, bound),
    &pg_query__partition_bound_spec__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "concurrent",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__PartitionCmd, concurrent),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__partition_cmd__field_indices_by_name[] = {
  1,   /* field[1] = bound */
  2,   /* field[2] = concurrent */
  0,   /* field[0] = name */
};
static const ProtobufCIntRange pg_query__partition_cmd__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__partition_cmd__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.PartitionCmd",
  "PartitionCmd",
  "PgQuery__PartitionCmd",
  "pg_query",
  sizeof(PgQuery__PartitionCmd),
  3,
  pg_query__partition_cmd__field_descriptors,
  pg_query__partition_cmd__field_indices_by_name,
  1,  pg_query__partition_cmd__number_ranges,
  (ProtobufCMessageInit) pg_query__partition_cmd__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__range_tbl_entry__field_descriptors[32] =
{
  {
    "alias",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTblEntry, alias),
    &pg_query__alias__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "eref",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTblEntry, eref),
    &pg_query__alias__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "rtekind",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTblEntry, rtekind),
    &pg_query__rtekind__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "relid",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTblEntry, relid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "inh",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTblEntry, inh),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "relkind",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTblEntry, relkind),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "rellockmode",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTblEntry, rellockmode),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "perminfoindex",
    8,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTblEntry, perminfoindex),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "tablesample",
    9,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTblEntry, tablesample),
    &pg_query__table_sample_clause__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "subquery",
    10,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTblEntry, subquery),
    &pg_query__query__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "security_barrier",
    11,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTblEntry, security_barrier),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "jointype",
    12,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTblEntry, jointype),
    &pg_query__join_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "joinmergedcols",
    13,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTblEntry, joinmergedcols),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "joinaliasvars",
    14,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__RangeTblEntry, n_joinaliasvars),
    offsetof(PgQuery__RangeTblEntry, joinaliasvars),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "joinleftcols",
    15,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__RangeTblEntry, n_joinleftcols),
    offsetof(PgQuery__RangeTblEntry, joinleftcols),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "joinrightcols",
    16,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__RangeTblEntry, n_joinrightcols),
    offsetof(PgQuery__RangeTblEntry, joinrightcols),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "join_using_alias",
    17,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTblEntry, join_using_alias),
    &pg_query__alias__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "functions",
    18,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__RangeTblEntry, n_functions),
    offsetof(PgQuery__RangeTblEntry, functions),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "funcordinality",
    19,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTblEntry, funcordinality),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "tablefunc",
    20,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTblEntry, tablefunc),
    &pg_query__table_func__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "values_lists",
    21,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__RangeTblEntry, n_values_lists),
    offsetof(PgQuery__RangeTblEntry, values_lists),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "ctename",
    22,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTblEntry, ctename),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "ctelevelsup",
    23,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTblEntry, ctelevelsup),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "self_reference",
    24,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTblEntry, self_reference),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "coltypes",
    25,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__RangeTblEntry, n_coltypes),
    offsetof(PgQuery__RangeTblEntry, coltypes),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "coltypmods",
    26,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__RangeTblEntry, n_coltypmods),
    offsetof(PgQuery__RangeTblEntry, coltypmods),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "colcollations",
    27,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__RangeTblEntry, n_colcollations),
    offsetof(PgQuery__RangeTblEntry, colcollations),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "enrname",
    28,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTblEntry, enrname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "enrtuples",
    29,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_DOUBLE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTblEntry, enrtuples),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "lateral",
    30,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTblEntry, lateral),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "in_from_cl",
    31,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTblEntry, in_from_cl),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "security_quals",
    32,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__RangeTblEntry, n_security_quals),
    offsetof(PgQuery__RangeTblEntry, security_quals),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__range_tbl_entry__field_indices_by_name[] = {
  0,   /* field[0] = alias */
  26,   /* field[26] = colcollations */
  24,   /* field[24] = coltypes */
  25,   /* field[25] = coltypmods */
  22,   /* field[22] = ctelevelsup */
  21,   /* field[21] = ctename */
  27,   /* field[27] = enrname */
  28,   /* field[28] = enrtuples */
  1,   /* field[1] = eref */
  18,   /* field[18] = funcordinality */
  17,   /* field[17] = functions */
  30,   /* field[30] = in_from_cl */
  4,   /* field[4] = inh */
  16,   /* field[16] = join_using_alias */
  13,   /* field[13] = joinaliasvars */
  14,   /* field[14] = joinleftcols */
  12,   /* field[12] = joinmergedcols */
  15,   /* field[15] = joinrightcols */
  11,   /* field[11] = jointype */
  29,   /* field[29] = lateral */
  7,   /* field[7] = perminfoindex */
  3,   /* field[3] = relid */
  5,   /* field[5] = relkind */
  6,   /* field[6] = rellockmode */
  2,   /* field[2] = rtekind */
  10,   /* field[10] = security_barrier */
  31,   /* field[31] = security_quals */
  23,   /* field[23] = self_reference */
  9,   /* field[9] = subquery */
  19,   /* field[19] = tablefunc */
  8,   /* field[8] = tablesample */
  20,   /* field[20] = values_lists */
};
static const ProtobufCIntRange pg_query__range_tbl_entry__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 32 }
};
const ProtobufCMessageDescriptor pg_query__range_tbl_entry__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.RangeTblEntry",
  "RangeTblEntry",
  "PgQuery__RangeTblEntry",
  "pg_query",
  sizeof(PgQuery__RangeTblEntry),
  32,
  pg_query__range_tbl_entry__field_descriptors,
  pg_query__range_tbl_entry__field_indices_by_name,
  1,  pg_query__range_tbl_entry__number_ranges,
  (ProtobufCMessageInit) pg_query__range_tbl_entry__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__rtepermission_info__field_descriptors[7] =
{
  {
    "relid",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RTEPermissionInfo, relid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "inh",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RTEPermissionInfo, inh),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "required_perms",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT64,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RTEPermissionInfo, required_perms),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "check_as_user",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RTEPermissionInfo, check_as_user),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "selected_cols",
    5,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_UINT64,
    offsetof(PgQuery__RTEPermissionInfo, n_selected_cols),
    offsetof(PgQuery__RTEPermissionInfo, selected_cols),
    NULL,
    NULL,
    PROTOBUF_C_FIELD_FLAG_PACKED,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "inserted_cols",
    6,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_UINT64,
    offsetof(PgQuery__RTEPermissionInfo, n_inserted_cols),
    offsetof(PgQuery__RTEPermissionInfo, inserted_cols),
    NULL,
    NULL,
    PROTOBUF_C_FIELD_FLAG_PACKED,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "updated_cols",
    7,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_UINT64,
    offsetof(PgQuery__RTEPermissionInfo, n_updated_cols),
    offsetof(PgQuery__RTEPermissionInfo, updated_cols),
    NULL,
    NULL,
    PROTOBUF_C_FIELD_FLAG_PACKED,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__rtepermission_info__field_indices_by_name[] = {
  3,   /* field[3] = check_as_user */
  1,   /* field[1] = inh */
  5,   /* field[5] = inserted_cols */
  0,   /* field[0] = relid */
  2,   /* field[2] = required_perms */
  4,   /* field[4] = selected_cols */
  6,   /* field[6] = updated_cols */
};
static const ProtobufCIntRange pg_query__rtepermission_info__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 7 }
};
const ProtobufCMessageDescriptor pg_query__rtepermission_info__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.RTEPermissionInfo",
  "RTEPermissionInfo",
  "PgQuery__RTEPermissionInfo",
  "pg_query",
  sizeof(PgQuery__RTEPermissionInfo),
  7,
  pg_query__rtepermission_info__field_descriptors,
  pg_query__rtepermission_info__field_indices_by_name,
  1,  pg_query__rtepermission_info__number_ranges,
  (ProtobufCMessageInit) pg_query__rtepermission_info__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__range_tbl_function__field_descriptors[7] =
{
  {
    "funcexpr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTblFunction, funcexpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "funccolcount",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RangeTblFunction, funccolcount),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "funccolnames",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__RangeTblFunction, n_funccolnames),
    offsetof(PgQuery__RangeTblFunction, funccolnames),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "funccoltypes",
    4,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__RangeTblFunction, n_funccoltypes),
    offsetof(PgQuery__RangeTblFunction, funccoltypes),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "funccoltypmods",
    5,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__RangeTblFunction, n_funccoltypmods),
    offsetof(PgQuery__RangeTblFunction, funccoltypmods),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "funccolcollations",
    6,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__RangeTblFunction, n_funccolcollations),
    offsetof(PgQuery__RangeTblFunction, funccolcollations),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "funcparams",
    7,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_UINT64,
    offsetof(PgQuery__RangeTblFunction, n_funcparams),
    offsetof(PgQuery__RangeTblFunction, funcparams),
    NULL,
    NULL,
    PROTOBUF_C_FIELD_FLAG_PACKED,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__range_tbl_function__field_indices_by_name[] = {
  5,   /* field[5] = funccolcollations */
  1,   /* field[1] = funccolcount */
  2,   /* field[2] = funccolnames */
  3,   /* field[3] = funccoltypes */
  4,   /* field[4] = funccoltypmods */
  0,   /* field[0] = funcexpr */
  6,   /* field[6] = funcparams */
};
static const ProtobufCIntRange pg_query__range_tbl_function__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 7 }
};
const ProtobufCMessageDescriptor pg_query__range_tbl_function__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.RangeTblFunction",
  "RangeTblFunction",
  "PgQuery__RangeTblFunction",
  "pg_query",
  sizeof(PgQuery__RangeTblFunction),
  7,
  pg_query__range_tbl_function__field_descriptors,
  pg_query__range_tbl_function__field_indices_by_name,
  1,  pg_query__range_tbl_function__number_ranges,
  (ProtobufCMessageInit) pg_query__range_tbl_function__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__table_sample_clause__field_descriptors[3] =
{
  {
    "tsmhandler",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__TableSampleClause, tsmhandler),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "args",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__TableSampleClause, n_args),
    offsetof(PgQuery__TableSampleClause, args),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "repeatable",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__TableSampleClause, repeatable),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__table_sample_clause__field_indices_by_name[] = {
  1,   /* field[1] = args */
  2,   /* field[2] = repeatable */
  0,   /* field[0] = tsmhandler */
};
static const ProtobufCIntRange pg_query__table_sample_clause__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__table_sample_clause__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.TableSampleClause",
  "TableSampleClause",
  "PgQuery__TableSampleClause",
  "pg_query",
  sizeof(PgQuery__TableSampleClause),
  3,
  pg_query__table_sample_clause__field_descriptors,
  pg_query__table_sample_clause__field_indices_by_name,
  1,  pg_query__table_sample_clause__number_ranges,
  (ProtobufCMessageInit) pg_query__table_sample_clause__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__with_check_option__field_descriptors[5] =
{
  {
    "kind",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WithCheckOption, kind),
    &pg_query__wcokind__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "relname",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WithCheckOption, relname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "polname",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WithCheckOption, polname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "qual",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WithCheckOption, qual),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "cascaded",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WithCheckOption, cascaded),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__with_check_option__field_indices_by_name[] = {
  4,   /* field[4] = cascaded */
  0,   /* field[0] = kind */
  2,   /* field[2] = polname */
  3,   /* field[3] = qual */
  1,   /* field[1] = relname */
};
static const ProtobufCIntRange pg_query__with_check_option__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor pg_query__with_check_option__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.WithCheckOption",
  "WithCheckOption",
  "PgQuery__WithCheckOption",
  "pg_query",
  sizeof(PgQuery__WithCheckOption),
  5,
  pg_query__with_check_option__field_descriptors,
  pg_query__with_check_option__field_indices_by_name,
  1,  pg_query__with_check_option__number_ranges,
  (ProtobufCMessageInit) pg_query__with_check_option__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__sort_group_clause__field_descriptors[5] =
{
  {
    "tle_sort_group_ref",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SortGroupClause, tle_sort_group_ref),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "eqop",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SortGroupClause, eqop),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "sortop",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SortGroupClause, sortop),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "nulls_first",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SortGroupClause, nulls_first),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "hashable",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SortGroupClause, hashable),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__sort_group_clause__field_indices_by_name[] = {
  1,   /* field[1] = eqop */
  4,   /* field[4] = hashable */
  3,   /* field[3] = nulls_first */
  2,   /* field[2] = sortop */
  0,   /* field[0] = tle_sort_group_ref */
};
static const ProtobufCIntRange pg_query__sort_group_clause__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor pg_query__sort_group_clause__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.SortGroupClause",
  "SortGroupClause",
  "PgQuery__SortGroupClause",
  "pg_query",
  sizeof(PgQuery__SortGroupClause),
  5,
  pg_query__sort_group_clause__field_descriptors,
  pg_query__sort_group_clause__field_indices_by_name,
  1,  pg_query__sort_group_clause__number_ranges,
  (ProtobufCMessageInit) pg_query__sort_group_clause__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__grouping_set__field_descriptors[3] =
{
  {
    "kind",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__GroupingSet, kind),
    &pg_query__grouping_set_kind__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "content",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__GroupingSet, n_content),
    offsetof(PgQuery__GroupingSet, content),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__GroupingSet, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__grouping_set__field_indices_by_name[] = {
  1,   /* field[1] = content */
  0,   /* field[0] = kind */
  2,   /* field[2] = location */
};
static const ProtobufCIntRange pg_query__grouping_set__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__grouping_set__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.GroupingSet",
  "GroupingSet",
  "PgQuery__GroupingSet",
  "pg_query",
  sizeof(PgQuery__GroupingSet),
  3,
  pg_query__grouping_set__field_descriptors,
  pg_query__grouping_set__field_indices_by_name,
  1,  pg_query__grouping_set__number_ranges,
  (ProtobufCMessageInit) pg_query__grouping_set__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__window_clause__field_descriptors[14] =
{
  {
    "name",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WindowClause, name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "refname",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WindowClause, refname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "partition_clause",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__WindowClause, n_partition_clause),
    offsetof(PgQuery__WindowClause, partition_clause),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "order_clause",
    4,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__WindowClause, n_order_clause),
    offsetof(PgQuery__WindowClause, order_clause),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "frame_options",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WindowClause, frame_options),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "start_offset",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WindowClause, start_offset),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "end_offset",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WindowClause, end_offset),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "start_in_range_func",
    8,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WindowClause, start_in_range_func),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "end_in_range_func",
    9,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WindowClause, end_in_range_func),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "in_range_coll",
    10,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WindowClause, in_range_coll),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "in_range_asc",
    11,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WindowClause, in_range_asc),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "in_range_nulls_first",
    12,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WindowClause, in_range_nulls_first),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "winref",
    13,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WindowClause, winref),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "copied_order",
    14,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WindowClause, copied_order),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__window_clause__field_indices_by_name[] = {
  13,   /* field[13] = copied_order */
  8,   /* field[8] = end_in_range_func */
  6,   /* field[6] = end_offset */
  4,   /* field[4] = frame_options */
  10,   /* field[10] = in_range_asc */
  9,   /* field[9] = in_range_coll */
  11,   /* field[11] = in_range_nulls_first */
  0,   /* field[0] = name */
  3,   /* field[3] = order_clause */
  2,   /* field[2] = partition_clause */
  1,   /* field[1] = refname */
  7,   /* field[7] = start_in_range_func */
  5,   /* field[5] = start_offset */
  12,   /* field[12] = winref */
};
static const ProtobufCIntRange pg_query__window_clause__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 14 }
};
const ProtobufCMessageDescriptor pg_query__window_clause__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.WindowClause",
  "WindowClause",
  "PgQuery__WindowClause",
  "pg_query",
  sizeof(PgQuery__WindowClause),
  14,
  pg_query__window_clause__field_descriptors,
  pg_query__window_clause__field_indices_by_name,
  1,  pg_query__window_clause__number_ranges,
  (ProtobufCMessageInit) pg_query__window_clause__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__row_mark_clause__field_descriptors[4] =
{
  {
    "rti",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RowMarkClause, rti),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "strength",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RowMarkClause, strength),
    &pg_query__lock_clause_strength__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "wait_policy",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RowMarkClause, wait_policy),
    &pg_query__lock_wait_policy__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "pushed_down",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RowMarkClause, pushed_down),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__row_mark_clause__field_indices_by_name[] = {
  3,   /* field[3] = pushed_down */
  0,   /* field[0] = rti */
  1,   /* field[1] = strength */
  2,   /* field[2] = wait_policy */
};
static const ProtobufCIntRange pg_query__row_mark_clause__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor pg_query__row_mark_clause__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.RowMarkClause",
  "RowMarkClause",
  "PgQuery__RowMarkClause",
  "pg_query",
  sizeof(PgQuery__RowMarkClause),
  4,
  pg_query__row_mark_clause__field_descriptors,
  pg_query__row_mark_clause__field_indices_by_name,
  1,  pg_query__row_mark_clause__number_ranges,
  (ProtobufCMessageInit) pg_query__row_mark_clause__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__with_clause__field_descriptors[3] =
{
  {
    "ctes",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__WithClause, n_ctes),
    offsetof(PgQuery__WithClause, ctes),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "recursive",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WithClause, recursive),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__WithClause, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__with_clause__field_indices_by_name[] = {
  0,   /* field[0] = ctes */
  2,   /* field[2] = location */
  1,   /* field[1] = recursive */
};
static const ProtobufCIntRange pg_query__with_clause__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__with_clause__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.WithClause",
  "WithClause",
  "PgQuery__WithClause",
  "pg_query",
  sizeof(PgQuery__WithClause),
  3,
  pg_query__with_clause__field_descriptors,
  pg_query__with_clause__field_indices_by_name,
  1,  pg_query__with_clause__number_ranges,
  (ProtobufCMessageInit) pg_query__with_clause__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__infer_clause__field_descriptors[4] =
{
  {
    "index_elems",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__InferClause, n_index_elems),
    offsetof(PgQuery__InferClause, index_elems),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "where_clause",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__InferClause, where_clause),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "conname",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__InferClause, conname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__InferClause, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__infer_clause__field_indices_by_name[] = {
  2,   /* field[2] = conname */
  0,   /* field[0] = index_elems */
  3,   /* field[3] = location */
  1,   /* field[1] = where_clause */
};
static const ProtobufCIntRange pg_query__infer_clause__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor pg_query__infer_clause__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.InferClause",
  "InferClause",
  "PgQuery__InferClause",
  "pg_query",
  sizeof(PgQuery__InferClause),
  4,
  pg_query__infer_clause__field_descriptors,
  pg_query__infer_clause__field_indices_by_name,
  1,  pg_query__infer_clause__number_ranges,
  (ProtobufCMessageInit) pg_query__infer_clause__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__on_conflict_clause__field_descriptors[5] =
{
  {
    "action",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__OnConflictClause, action),
    &pg_query__on_conflict_action__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "infer",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__OnConflictClause, infer),
    &pg_query__infer_clause__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "target_list",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__OnConflictClause, n_target_list),
    offsetof(PgQuery__OnConflictClause, target_list),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "where_clause",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__OnConflictClause, where_clause),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__OnConflictClause, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__on_conflict_clause__field_indices_by_name[] = {
  0,   /* field[0] = action */
  1,   /* field[1] = infer */
  4,   /* field[4] = location */
  2,   /* field[2] = target_list */
  3,   /* field[3] = where_clause */
};
static const ProtobufCIntRange pg_query__on_conflict_clause__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor pg_query__on_conflict_clause__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.OnConflictClause",
  "OnConflictClause",
  "PgQuery__OnConflictClause",
  "pg_query",
  sizeof(PgQuery__OnConflictClause),
  5,
  pg_query__on_conflict_clause__field_descriptors,
  pg_query__on_conflict_clause__field_indices_by_name,
  1,  pg_query__on_conflict_clause__number_ranges,
  (ProtobufCMessageInit) pg_query__on_conflict_clause__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__ctesearch_clause__field_descriptors[4] =
{
  {
    "search_col_list",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CTESearchClause, n_search_col_list),
    offsetof(PgQuery__CTESearchClause, search_col_list),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "search_breadth_first",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CTESearchClause, search_breadth_first),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "search_seq_column",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CTESearchClause, search_seq_column),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CTESearchClause, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__ctesearch_clause__field_indices_by_name[] = {
  3,   /* field[3] = location */
  1,   /* field[1] = search_breadth_first */
  0,   /* field[0] = search_col_list */
  2,   /* field[2] = search_seq_column */
};
static const ProtobufCIntRange pg_query__ctesearch_clause__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor pg_query__ctesearch_clause__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CTESearchClause",
  "CTESearchClause",
  "PgQuery__CTESearchClause",
  "pg_query",
  sizeof(PgQuery__CTESearchClause),
  4,
  pg_query__ctesearch_clause__field_descriptors,
  pg_query__ctesearch_clause__field_indices_by_name,
  1,  pg_query__ctesearch_clause__number_ranges,
  (ProtobufCMessageInit) pg_query__ctesearch_clause__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__ctecycle_clause__field_descriptors[10] =
{
  {
    "cycle_col_list",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CTECycleClause, n_cycle_col_list),
    offsetof(PgQuery__CTECycleClause, cycle_col_list),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "cycle_mark_column",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CTECycleClause, cycle_mark_column),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "cycle_mark_value",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CTECycleClause, cycle_mark_value),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "cycle_mark_default",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CTECycleClause, cycle_mark_default),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "cycle_path_column",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CTECycleClause, cycle_path_column),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CTECycleClause, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "cycle_mark_type",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CTECycleClause, cycle_mark_type),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "cycle_mark_typmod",
    8,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CTECycleClause, cycle_mark_typmod),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "cycle_mark_collation",
    9,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CTECycleClause, cycle_mark_collation),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "cycle_mark_neop",
    10,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CTECycleClause, cycle_mark_neop),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__ctecycle_clause__field_indices_by_name[] = {
  0,   /* field[0] = cycle_col_list */
  8,   /* field[8] = cycle_mark_collation */
  1,   /* field[1] = cycle_mark_column */
  3,   /* field[3] = cycle_mark_default */
  9,   /* field[9] = cycle_mark_neop */
  6,   /* field[6] = cycle_mark_type */
  7,   /* field[7] = cycle_mark_typmod */
  2,   /* field[2] = cycle_mark_value */
  4,   /* field[4] = cycle_path_column */
  5,   /* field[5] = location */
};
static const ProtobufCIntRange pg_query__ctecycle_clause__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 10 }
};
const ProtobufCMessageDescriptor pg_query__ctecycle_clause__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CTECycleClause",
  "CTECycleClause",
  "PgQuery__CTECycleClause",
  "pg_query",
  sizeof(PgQuery__CTECycleClause),
  10,
  pg_query__ctecycle_clause__field_descriptors,
  pg_query__ctecycle_clause__field_indices_by_name,
  1,  pg_query__ctecycle_clause__number_ranges,
  (ProtobufCMessageInit) pg_query__ctecycle_clause__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__common_table_expr__field_descriptors[13] =
{
  {
    "ctename",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CommonTableExpr, ctename),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "aliascolnames",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CommonTableExpr, n_aliascolnames),
    offsetof(PgQuery__CommonTableExpr, aliascolnames),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "ctematerialized",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CommonTableExpr, ctematerialized),
    &pg_query__ctematerialize__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "ctequery",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CommonTableExpr, ctequery),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "search_clause",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CommonTableExpr, search_clause),
    &pg_query__ctesearch_clause__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "cycle_clause",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CommonTableExpr, cycle_clause),
    &pg_query__ctecycle_clause__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CommonTableExpr, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "cterecursive",
    8,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CommonTableExpr, cterecursive),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "cterefcount",
    9,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CommonTableExpr, cterefcount),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "ctecolnames",
    10,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CommonTableExpr, n_ctecolnames),
    offsetof(PgQuery__CommonTableExpr, ctecolnames),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "ctecoltypes",
    11,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CommonTableExpr, n_ctecoltypes),
    offsetof(PgQuery__CommonTableExpr, ctecoltypes),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "ctecoltypmods",
    12,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CommonTableExpr, n_ctecoltypmods),
    offsetof(PgQuery__CommonTableExpr, ctecoltypmods),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "ctecolcollations",
    13,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CommonTableExpr, n_ctecolcollations),
    offsetof(PgQuery__CommonTableExpr, ctecolcollations),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__common_table_expr__field_indices_by_name[] = {
  1,   /* field[1] = aliascolnames */
  12,   /* field[12] = ctecolcollations */
  9,   /* field[9] = ctecolnames */
  10,   /* field[10] = ctecoltypes */
  11,   /* field[11] = ctecoltypmods */
  2,   /* field[2] = ctematerialized */
  0,   /* field[0] = ctename */
  3,   /* field[3] = ctequery */
  7,   /* field[7] = cterecursive */
  8,   /* field[8] = cterefcount */
  5,   /* field[5] = cycle_clause */
  6,   /* field[6] = location */
  4,   /* field[4] = search_clause */
};
static const ProtobufCIntRange pg_query__common_table_expr__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 13 }
};
const ProtobufCMessageDescriptor pg_query__common_table_expr__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CommonTableExpr",
  "CommonTableExpr",
  "PgQuery__CommonTableExpr",
  "pg_query",
  sizeof(PgQuery__CommonTableExpr),
  13,
  pg_query__common_table_expr__field_descriptors,
  pg_query__common_table_expr__field_indices_by_name,
  1,  pg_query__common_table_expr__number_ranges,
  (ProtobufCMessageInit) pg_query__common_table_expr__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__merge_when_clause__field_descriptors[6] =
{
  {
    "match_kind",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__MergeWhenClause, match_kind),
    &pg_query__merge_match_kind__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "command_type",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__MergeWhenClause, command_type),
    &pg_query__cmd_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "override",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__MergeWhenClause, override),
    &pg_query__overriding_kind__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "condition",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__MergeWhenClause, condition),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "target_list",
    5,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__MergeWhenClause, n_target_list),
    offsetof(PgQuery__MergeWhenClause, target_list),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "values",
    6,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__MergeWhenClause, n_values),
    offsetof(PgQuery__MergeWhenClause, values),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__merge_when_clause__field_indices_by_name[] = {
  1,   /* field[1] = command_type */
  3,   /* field[3] = condition */
  0,   /* field[0] = match_kind */
  2,   /* field[2] = override */
  4,   /* field[4] = target_list */
  5,   /* field[5] = values */
};
static const ProtobufCIntRange pg_query__merge_when_clause__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 6 }
};
const ProtobufCMessageDescriptor pg_query__merge_when_clause__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.MergeWhenClause",
  "MergeWhenClause",
  "PgQuery__MergeWhenClause",
  "pg_query",
  sizeof(PgQuery__MergeWhenClause),
  6,
  pg_query__merge_when_clause__field_descriptors,
  pg_query__merge_when_clause__field_indices_by_name,
  1,  pg_query__merge_when_clause__number_ranges,
  (ProtobufCMessageInit) pg_query__merge_when_clause__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__trigger_transition__field_descriptors[3] =
{
  {
    "name",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__TriggerTransition, name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "is_new",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__TriggerTransition, is_new),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "is_table",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__TriggerTransition, is_table),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__trigger_transition__field_indices_by_name[] = {
  1,   /* field[1] = is_new */
  2,   /* field[2] = is_table */
  0,   /* field[0] = name */
};
static const ProtobufCIntRange pg_query__trigger_transition__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__trigger_transition__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.TriggerTransition",
  "TriggerTransition",
  "PgQuery__TriggerTransition",
  "pg_query",
  sizeof(PgQuery__TriggerTransition),
  3,
  pg_query__trigger_transition__field_descriptors,
  pg_query__trigger_transition__field_indices_by_name,
  1,  pg_query__trigger_transition__number_ranges,
  (ProtobufCMessageInit) pg_query__trigger_transition__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__json_output__field_descriptors[2] =
{
  {
    "type_name",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonOutput, type_name),
    &pg_query__type_name__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "returning",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonOutput, returning),
    &pg_query__json_returning__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__json_output__field_indices_by_name[] = {
  1,   /* field[1] = returning */
  0,   /* field[0] = type_name */
};
static const ProtobufCIntRange pg_query__json_output__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor pg_query__json_output__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.JsonOutput",
  "JsonOutput",
  "PgQuery__JsonOutput",
  "pg_query",
  sizeof(PgQuery__JsonOutput),
  2,
  pg_query__json_output__field_descriptors,
  pg_query__json_output__field_indices_by_name,
  1,  pg_query__json_output__number_ranges,
  (ProtobufCMessageInit) pg_query__json_output__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__json_argument__field_descriptors[2] =
{
  {
    "val",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonArgument, val),
    &pg_query__json_value_expr__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "name",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonArgument, name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__json_argument__field_indices_by_name[] = {
  1,   /* field[1] = name */
  0,   /* field[0] = val */
};
static const ProtobufCIntRange pg_query__json_argument__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor pg_query__json_argument__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.JsonArgument",
  "JsonArgument",
  "PgQuery__JsonArgument",
  "pg_query",
  sizeof(PgQuery__JsonArgument),
  2,
  pg_query__json_argument__field_descriptors,
  pg_query__json_argument__field_indices_by_name,
  1,  pg_query__json_argument__number_ranges,
  (ProtobufCMessageInit) pg_query__json_argument__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__json_func_expr__field_descriptors[11] =
{
  {
    "op",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonFuncExpr, op),
    &pg_query__json_expr_op__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "column_name",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonFuncExpr, column_name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "context_item",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonFuncExpr, context_item),
    &pg_query__json_value_expr__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "pathspec",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonFuncExpr, pathspec),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "passing",
    5,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__JsonFuncExpr, n_passing),
    offsetof(PgQuery__JsonFuncExpr, passing),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "output",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonFuncExpr, output),
    &pg_query__json_output__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "on_empty",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonFuncExpr, on_empty),
    &pg_query__json_behavior__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "on_error",
    8,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonFuncExpr, on_error),
    &pg_query__json_behavior__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "wrapper",
    9,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonFuncExpr, wrapper),
    &pg_query__json_wrapper__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "quotes",
    10,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonFuncExpr, quotes),
    &pg_query__json_quotes__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    11,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonFuncExpr, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__json_func_expr__field_indices_by_name[] = {
  1,   /* field[1] = column_name */
  2,   /* field[2] = context_item */
  10,   /* field[10] = location */
  6,   /* field[6] = on_empty */
  7,   /* field[7] = on_error */
  0,   /* field[0] = op */
  5,   /* field[5] = output */
  4,   /* field[4] = passing */
  3,   /* field[3] = pathspec */
  9,   /* field[9] = quotes */
  8,   /* field[8] = wrapper */
};
static const ProtobufCIntRange pg_query__json_func_expr__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 11 }
};
const ProtobufCMessageDescriptor pg_query__json_func_expr__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.JsonFuncExpr",
  "JsonFuncExpr",
  "PgQuery__JsonFuncExpr",
  "pg_query",
  sizeof(PgQuery__JsonFuncExpr),
  11,
  pg_query__json_func_expr__field_descriptors,
  pg_query__json_func_expr__field_indices_by_name,
  1,  pg_query__json_func_expr__number_ranges,
  (ProtobufCMessageInit) pg_query__json_func_expr__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__json_table_path_spec__field_descriptors[4] =
{
  {
    "string",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonTablePathSpec, string),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "name",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonTablePathSpec, name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "name_location",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonTablePathSpec, name_location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonTablePathSpec, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__json_table_path_spec__field_indices_by_name[] = {
  3,   /* field[3] = location */
  1,   /* field[1] = name */
  2,   /* field[2] = name_location */
  0,   /* field[0] = string */
};
static const ProtobufCIntRange pg_query__json_table_path_spec__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor pg_query__json_table_path_spec__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.JsonTablePathSpec",
  "JsonTablePathSpec",
  "PgQuery__JsonTablePathSpec",
  "pg_query",
  sizeof(PgQuery__JsonTablePathSpec),
  4,
  pg_query__json_table_path_spec__field_descriptors,
  pg_query__json_table_path_spec__field_indices_by_name,
  1,  pg_query__json_table_path_spec__number_ranges,
  (ProtobufCMessageInit) pg_query__json_table_path_spec__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__json_table__field_descriptors[8] =
{
  {
    "context_item",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonTable, context_item),
    &pg_query__json_value_expr__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "pathspec",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonTable, pathspec),
    &pg_query__json_table_path_spec__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "passing",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__JsonTable, n_passing),
    offsetof(PgQuery__JsonTable, passing),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "columns",
    4,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__JsonTable, n_columns),
    offsetof(PgQuery__JsonTable, columns),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "on_error",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonTable, on_error),
    &pg_query__json_behavior__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "alias",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonTable, alias),
    &pg_query__alias__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "lateral",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonTable, lateral),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    8,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonTable, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__json_table__field_indices_by_name[] = {
  5,   /* field[5] = alias */
  3,   /* field[3] = columns */
  0,   /* field[0] = context_item */
  6,   /* field[6] = lateral */
  7,   /* field[7] = location */
  4,   /* field[4] = on_error */
  2,   /* field[2] = passing */
  1,   /* field[1] = pathspec */
};
static const ProtobufCIntRange pg_query__json_table__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 8 }
};
const ProtobufCMessageDescriptor pg_query__json_table__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.JsonTable",
  "JsonTable",
  "PgQuery__JsonTable",
  "pg_query",
  sizeof(PgQuery__JsonTable),
  8,
  pg_query__json_table__field_descriptors,
  pg_query__json_table__field_indices_by_name,
  1,  pg_query__json_table__number_ranges,
  (ProtobufCMessageInit) pg_query__json_table__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__json_table_column__field_descriptors[11] =
{
  {
    "coltype",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonTableColumn, coltype),
    &pg_query__json_table_column_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "name",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonTableColumn, name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "type_name",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonTableColumn, type_name),
    &pg_query__type_name__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "pathspec",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonTableColumn, pathspec),
    &pg_query__json_table_path_spec__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "format",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonTableColumn, format),
    &pg_query__json_format__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "wrapper",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonTableColumn, wrapper),
    &pg_query__json_wrapper__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "quotes",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonTableColumn, quotes),
    &pg_query__json_quotes__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "columns",
    8,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__JsonTableColumn, n_columns),
    offsetof(PgQuery__JsonTableColumn, columns),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "on_empty",
    9,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonTableColumn, on_empty),
    &pg_query__json_behavior__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "on_error",
    10,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonTableColumn, on_error),
    &pg_query__json_behavior__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    11,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonTableColumn, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__json_table_column__field_indices_by_name[] = {
  0,   /* field[0] = coltype */
  7,   /* field[7] = columns */
  4,   /* field[4] = format */
  10,   /* field[10] = location */
  1,   /* field[1] = name */
  8,   /* field[8] = on_empty */
  9,   /* field[9] = on_error */
  3,   /* field[3] = pathspec */
  6,   /* field[6] = quotes */
  2,   /* field[2] = type_name */
  5,   /* field[5] = wrapper */
};
static const ProtobufCIntRange pg_query__json_table_column__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 11 }
};
const ProtobufCMessageDescriptor pg_query__json_table_column__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.JsonTableColumn",
  "JsonTableColumn",
  "PgQuery__JsonTableColumn",
  "pg_query",
  sizeof(PgQuery__JsonTableColumn),
  11,
  pg_query__json_table_column__field_descriptors,
  pg_query__json_table_column__field_indices_by_name,
  1,  pg_query__json_table_column__number_ranges,
  (ProtobufCMessageInit) pg_query__json_table_column__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__json_key_value__field_descriptors[2] =
{
  {
    "key",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonKeyValue, key),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "value",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonKeyValue, value),
    &pg_query__json_value_expr__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__json_key_value__field_indices_by_name[] = {
  0,   /* field[0] = key */
  1,   /* field[1] = value */
};
static const ProtobufCIntRange pg_query__json_key_value__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor pg_query__json_key_value__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.JsonKeyValue",
  "JsonKeyValue",
  "PgQuery__JsonKeyValue",
  "pg_query",
  sizeof(PgQuery__JsonKeyValue),
  2,
  pg_query__json_key_value__field_descriptors,
  pg_query__json_key_value__field_indices_by_name,
  1,  pg_query__json_key_value__number_ranges,
  (ProtobufCMessageInit) pg_query__json_key_value__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__json_parse_expr__field_descriptors[4] =
{
  {
    "expr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonParseExpr, expr),
    &pg_query__json_value_expr__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "output",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonParseExpr, output),
    &pg_query__json_output__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "unique_keys",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonParseExpr, unique_keys),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonParseExpr, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__json_parse_expr__field_indices_by_name[] = {
  0,   /* field[0] = expr */
  3,   /* field[3] = location */
  1,   /* field[1] = output */
  2,   /* field[2] = unique_keys */
};
static const ProtobufCIntRange pg_query__json_parse_expr__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor pg_query__json_parse_expr__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.JsonParseExpr",
  "JsonParseExpr",
  "PgQuery__JsonParseExpr",
  "pg_query",
  sizeof(PgQuery__JsonParseExpr),
  4,
  pg_query__json_parse_expr__field_descriptors,
  pg_query__json_parse_expr__field_indices_by_name,
  1,  pg_query__json_parse_expr__number_ranges,
  (ProtobufCMessageInit) pg_query__json_parse_expr__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__json_scalar_expr__field_descriptors[3] =
{
  {
    "expr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonScalarExpr, expr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "output",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonScalarExpr, output),
    &pg_query__json_output__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonScalarExpr, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__json_scalar_expr__field_indices_by_name[] = {
  0,   /* field[0] = expr */
  2,   /* field[2] = location */
  1,   /* field[1] = output */
};
static const ProtobufCIntRange pg_query__json_scalar_expr__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__json_scalar_expr__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.JsonScalarExpr",
  "JsonScalarExpr",
  "PgQuery__JsonScalarExpr",
  "pg_query",
  sizeof(PgQuery__JsonScalarExpr),
  3,
  pg_query__json_scalar_expr__field_descriptors,
  pg_query__json_scalar_expr__field_indices_by_name,
  1,  pg_query__json_scalar_expr__number_ranges,
  (ProtobufCMessageInit) pg_query__json_scalar_expr__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__json_serialize_expr__field_descriptors[3] =
{
  {
    "expr",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonSerializeExpr, expr),
    &pg_query__json_value_expr__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "output",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonSerializeExpr, output),
    &pg_query__json_output__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonSerializeExpr, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__json_serialize_expr__field_indices_by_name[] = {
  0,   /* field[0] = expr */
  2,   /* field[2] = location */
  1,   /* field[1] = output */
};
static const ProtobufCIntRange pg_query__json_serialize_expr__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__json_serialize_expr__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.JsonSerializeExpr",
  "JsonSerializeExpr",
  "PgQuery__JsonSerializeExpr",
  "pg_query",
  sizeof(PgQuery__JsonSerializeExpr),
  3,
  pg_query__json_serialize_expr__field_descriptors,
  pg_query__json_serialize_expr__field_indices_by_name,
  1,  pg_query__json_serialize_expr__number_ranges,
  (ProtobufCMessageInit) pg_query__json_serialize_expr__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__json_object_constructor__field_descriptors[5] =
{
  {
    "exprs",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__JsonObjectConstructor, n_exprs),
    offsetof(PgQuery__JsonObjectConstructor, exprs),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "output",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonObjectConstructor, output),
    &pg_query__json_output__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "absent_on_null",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonObjectConstructor, absent_on_null),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "unique",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonObjectConstructor, unique),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonObjectConstructor, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__json_object_constructor__field_indices_by_name[] = {
  2,   /* field[2] = absent_on_null */
  0,   /* field[0] = exprs */
  4,   /* field[4] = location */
  1,   /* field[1] = output */
  3,   /* field[3] = unique */
};
static const ProtobufCIntRange pg_query__json_object_constructor__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor pg_query__json_object_constructor__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.JsonObjectConstructor",
  "JsonObjectConstructor",
  "PgQuery__JsonObjectConstructor",
  "pg_query",
  sizeof(PgQuery__JsonObjectConstructor),
  5,
  pg_query__json_object_constructor__field_descriptors,
  pg_query__json_object_constructor__field_indices_by_name,
  1,  pg_query__json_object_constructor__number_ranges,
  (ProtobufCMessageInit) pg_query__json_object_constructor__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__json_array_constructor__field_descriptors[4] =
{
  {
    "exprs",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__JsonArrayConstructor, n_exprs),
    offsetof(PgQuery__JsonArrayConstructor, exprs),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "output",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonArrayConstructor, output),
    &pg_query__json_output__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "absent_on_null",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonArrayConstructor, absent_on_null),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonArrayConstructor, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__json_array_constructor__field_indices_by_name[] = {
  2,   /* field[2] = absent_on_null */
  0,   /* field[0] = exprs */
  3,   /* field[3] = location */
  1,   /* field[1] = output */
};
static const ProtobufCIntRange pg_query__json_array_constructor__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor pg_query__json_array_constructor__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.JsonArrayConstructor",
  "JsonArrayConstructor",
  "PgQuery__JsonArrayConstructor",
  "pg_query",
  sizeof(PgQuery__JsonArrayConstructor),
  4,
  pg_query__json_array_constructor__field_descriptors,
  pg_query__json_array_constructor__field_indices_by_name,
  1,  pg_query__json_array_constructor__number_ranges,
  (ProtobufCMessageInit) pg_query__json_array_constructor__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__json_array_query_constructor__field_descriptors[5] =
{
  {
    "query",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonArrayQueryConstructor, query),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "output",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonArrayQueryConstructor, output),
    &pg_query__json_output__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "format",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonArrayQueryConstructor, format),
    &pg_query__json_format__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "absent_on_null",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonArrayQueryConstructor, absent_on_null),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonArrayQueryConstructor, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__json_array_query_constructor__field_indices_by_name[] = {
  3,   /* field[3] = absent_on_null */
  2,   /* field[2] = format */
  4,   /* field[4] = location */
  1,   /* field[1] = output */
  0,   /* field[0] = query */
};
static const ProtobufCIntRange pg_query__json_array_query_constructor__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor pg_query__json_array_query_constructor__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.JsonArrayQueryConstructor",
  "JsonArrayQueryConstructor",
  "PgQuery__JsonArrayQueryConstructor",
  "pg_query",
  sizeof(PgQuery__JsonArrayQueryConstructor),
  5,
  pg_query__json_array_query_constructor__field_descriptors,
  pg_query__json_array_query_constructor__field_indices_by_name,
  1,  pg_query__json_array_query_constructor__number_ranges,
  (ProtobufCMessageInit) pg_query__json_array_query_constructor__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__json_agg_constructor__field_descriptors[5] =
{
  {
    "output",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonAggConstructor, output),
    &pg_query__json_output__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "agg_filter",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonAggConstructor, agg_filter),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "agg_order",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__JsonAggConstructor, n_agg_order),
    offsetof(PgQuery__JsonAggConstructor, agg_order),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "over",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonAggConstructor, over),
    &pg_query__window_def__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonAggConstructor, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__json_agg_constructor__field_indices_by_name[] = {
  1,   /* field[1] = agg_filter */
  2,   /* field[2] = agg_order */
  4,   /* field[4] = location */
  0,   /* field[0] = output */
  3,   /* field[3] = over */
};
static const ProtobufCIntRange pg_query__json_agg_constructor__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor pg_query__json_agg_constructor__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.JsonAggConstructor",
  "JsonAggConstructor",
  "PgQuery__JsonAggConstructor",
  "pg_query",
  sizeof(PgQuery__JsonAggConstructor),
  5,
  pg_query__json_agg_constructor__field_descriptors,
  pg_query__json_agg_constructor__field_indices_by_name,
  1,  pg_query__json_agg_constructor__number_ranges,
  (ProtobufCMessageInit) pg_query__json_agg_constructor__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__json_object_agg__field_descriptors[4] =
{
  {
    "constructor",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonObjectAgg, constructor),
    &pg_query__json_agg_constructor__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "arg",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonObjectAgg, arg),
    &pg_query__json_key_value__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "absent_on_null",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonObjectAgg, absent_on_null),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "unique",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonObjectAgg, unique),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__json_object_agg__field_indices_by_name[] = {
  2,   /* field[2] = absent_on_null */
  1,   /* field[1] = arg */
  0,   /* field[0] = constructor */
  3,   /* field[3] = unique */
};
static const ProtobufCIntRange pg_query__json_object_agg__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor pg_query__json_object_agg__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.JsonObjectAgg",
  "JsonObjectAgg",
  "PgQuery__JsonObjectAgg",
  "pg_query",
  sizeof(PgQuery__JsonObjectAgg),
  4,
  pg_query__json_object_agg__field_descriptors,
  pg_query__json_object_agg__field_indices_by_name,
  1,  pg_query__json_object_agg__number_ranges,
  (ProtobufCMessageInit) pg_query__json_object_agg__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__json_array_agg__field_descriptors[3] =
{
  {
    "constructor",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonArrayAgg, constructor),
    &pg_query__json_agg_constructor__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "arg",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonArrayAgg, arg),
    &pg_query__json_value_expr__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "absent_on_null",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__JsonArrayAgg, absent_on_null),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__json_array_agg__field_indices_by_name[] = {
  2,   /* field[2] = absent_on_null */
  1,   /* field[1] = arg */
  0,   /* field[0] = constructor */
};
static const ProtobufCIntRange pg_query__json_array_agg__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__json_array_agg__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.JsonArrayAgg",
  "JsonArrayAgg",
  "PgQuery__JsonArrayAgg",
  "pg_query",
  sizeof(PgQuery__JsonArrayAgg),
  3,
  pg_query__json_array_agg__field_descriptors,
  pg_query__json_array_agg__field_indices_by_name,
  1,  pg_query__json_array_agg__number_ranges,
  (ProtobufCMessageInit) pg_query__json_array_agg__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__raw_stmt__field_descriptors[3] =
{
  {
    "stmt",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RawStmt, stmt),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "stmt_location",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RawStmt, stmt_location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "stmt_len",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RawStmt, stmt_len),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__raw_stmt__field_indices_by_name[] = {
  0,   /* field[0] = stmt */
  2,   /* field[2] = stmt_len */
  1,   /* field[1] = stmt_location */
};
static const ProtobufCIntRange pg_query__raw_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__raw_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.RawStmt",
  "RawStmt",
  "PgQuery__RawStmt",
  "pg_query",
  sizeof(PgQuery__RawStmt),
  3,
  pg_query__raw_stmt__field_descriptors,
  pg_query__raw_stmt__field_indices_by_name,
  1,  pg_query__raw_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__raw_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__insert_stmt__field_descriptors[7] =
{
  {
    "relation",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__InsertStmt, relation),
    &pg_query__range_var__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "cols",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__InsertStmt, n_cols),
    offsetof(PgQuery__InsertStmt, cols),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "select_stmt",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__InsertStmt, select_stmt),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "on_conflict_clause",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__InsertStmt, on_conflict_clause),
    &pg_query__on_conflict_clause__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "returning_list",
    5,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__InsertStmt, n_returning_list),
    offsetof(PgQuery__InsertStmt, returning_list),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "with_clause",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__InsertStmt, with_clause),
    &pg_query__with_clause__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "override",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__InsertStmt, override),
    &pg_query__overriding_kind__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__insert_stmt__field_indices_by_name[] = {
  1,   /* field[1] = cols */
  3,   /* field[3] = on_conflict_clause */
  6,   /* field[6] = override */
  0,   /* field[0] = relation */
  4,   /* field[4] = returning_list */
  2,   /* field[2] = select_stmt */
  5,   /* field[5] = with_clause */
};
static const ProtobufCIntRange pg_query__insert_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 7 }
};
const ProtobufCMessageDescriptor pg_query__insert_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.InsertStmt",
  "InsertStmt",
  "PgQuery__InsertStmt",
  "pg_query",
  sizeof(PgQuery__InsertStmt),
  7,
  pg_query__insert_stmt__field_descriptors,
  pg_query__insert_stmt__field_indices_by_name,
  1,  pg_query__insert_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__insert_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__delete_stmt__field_descriptors[5] =
{
  {
    "relation",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DeleteStmt, relation),
    &pg_query__range_var__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "using_clause",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__DeleteStmt, n_using_clause),
    offsetof(PgQuery__DeleteStmt, using_clause),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "where_clause",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DeleteStmt, where_clause),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "returning_list",
    4,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__DeleteStmt, n_returning_list),
    offsetof(PgQuery__DeleteStmt, returning_list),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "with_clause",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DeleteStmt, with_clause),
    &pg_query__with_clause__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__delete_stmt__field_indices_by_name[] = {
  0,   /* field[0] = relation */
  3,   /* field[3] = returning_list */
  1,   /* field[1] = using_clause */
  2,   /* field[2] = where_clause */
  4,   /* field[4] = with_clause */
};
static const ProtobufCIntRange pg_query__delete_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor pg_query__delete_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.DeleteStmt",
  "DeleteStmt",
  "PgQuery__DeleteStmt",
  "pg_query",
  sizeof(PgQuery__DeleteStmt),
  5,
  pg_query__delete_stmt__field_descriptors,
  pg_query__delete_stmt__field_indices_by_name,
  1,  pg_query__delete_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__delete_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__update_stmt__field_descriptors[6] =
{
  {
    "relation",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__UpdateStmt, relation),
    &pg_query__range_var__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "target_list",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__UpdateStmt, n_target_list),
    offsetof(PgQuery__UpdateStmt, target_list),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "where_clause",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__UpdateStmt, where_clause),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "from_clause",
    4,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__UpdateStmt, n_from_clause),
    offsetof(PgQuery__UpdateStmt, from_clause),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "returning_list",
    5,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__UpdateStmt, n_returning_list),
    offsetof(PgQuery__UpdateStmt, returning_list),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "with_clause",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__UpdateStmt, with_clause),
    &pg_query__with_clause__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__update_stmt__field_indices_by_name[] = {
  3,   /* field[3] = from_clause */
  0,   /* field[0] = relation */
  4,   /* field[4] = returning_list */
  1,   /* field[1] = target_list */
  2,   /* field[2] = where_clause */
  5,   /* field[5] = with_clause */
};
static const ProtobufCIntRange pg_query__update_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 6 }
};
const ProtobufCMessageDescriptor pg_query__update_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.UpdateStmt",
  "UpdateStmt",
  "PgQuery__UpdateStmt",
  "pg_query",
  sizeof(PgQuery__UpdateStmt),
  6,
  pg_query__update_stmt__field_descriptors,
  pg_query__update_stmt__field_indices_by_name,
  1,  pg_query__update_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__update_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__merge_stmt__field_descriptors[6] =
{
  {
    "relation",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__MergeStmt, relation),
    &pg_query__range_var__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "source_relation",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__MergeStmt, source_relation),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "join_condition",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__MergeStmt, join_condition),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "merge_when_clauses",
    4,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__MergeStmt, n_merge_when_clauses),
    offsetof(PgQuery__MergeStmt, merge_when_clauses),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "returning_list",
    5,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__MergeStmt, n_returning_list),
    offsetof(PgQuery__MergeStmt, returning_list),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "with_clause",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__MergeStmt, with_clause),
    &pg_query__with_clause__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__merge_stmt__field_indices_by_name[] = {
  2,   /* field[2] = join_condition */
  3,   /* field[3] = merge_when_clauses */
  0,   /* field[0] = relation */
  4,   /* field[4] = returning_list */
  1,   /* field[1] = source_relation */
  5,   /* field[5] = with_clause */
};
static const ProtobufCIntRange pg_query__merge_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 6 }
};
const ProtobufCMessageDescriptor pg_query__merge_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.MergeStmt",
  "MergeStmt",
  "PgQuery__MergeStmt",
  "pg_query",
  sizeof(PgQuery__MergeStmt),
  6,
  pg_query__merge_stmt__field_descriptors,
  pg_query__merge_stmt__field_indices_by_name,
  1,  pg_query__merge_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__merge_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__select_stmt__field_descriptors[20] =
{
  {
    "distinct_clause",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__SelectStmt, n_distinct_clause),
    offsetof(PgQuery__SelectStmt, distinct_clause),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "into_clause",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SelectStmt, into_clause),
    &pg_query__into_clause__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "target_list",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__SelectStmt, n_target_list),
    offsetof(PgQuery__SelectStmt, target_list),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "from_clause",
    4,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__SelectStmt, n_from_clause),
    offsetof(PgQuery__SelectStmt, from_clause),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "where_clause",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SelectStmt, where_clause),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "group_clause",
    6,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__SelectStmt, n_group_clause),
    offsetof(PgQuery__SelectStmt, group_clause),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "group_distinct",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SelectStmt, group_distinct),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "having_clause",
    8,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SelectStmt, having_clause),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "window_clause",
    9,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__SelectStmt, n_window_clause),
    offsetof(PgQuery__SelectStmt, window_clause),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "values_lists",
    10,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__SelectStmt, n_values_lists),
    offsetof(PgQuery__SelectStmt, values_lists),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "sort_clause",
    11,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__SelectStmt, n_sort_clause),
    offsetof(PgQuery__SelectStmt, sort_clause),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "limit_offset",
    12,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SelectStmt, limit_offset),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "limit_count",
    13,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SelectStmt, limit_count),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "limit_option",
    14,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SelectStmt, limit_option),
    &pg_query__limit_option__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "locking_clause",
    15,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__SelectStmt, n_locking_clause),
    offsetof(PgQuery__SelectStmt, locking_clause),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "with_clause",
    16,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SelectStmt, with_clause),
    &pg_query__with_clause__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "op",
    17,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SelectStmt, op),
    &pg_query__set_operation__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "all",
    18,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SelectStmt, all),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "larg",
    19,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SelectStmt, larg),
    &pg_query__select_stmt__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "rarg",
    20,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SelectStmt, rarg),
    &pg_query__select_stmt__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__select_stmt__field_indices_by_name[] = {
  17,   /* field[17] = all */
  0,   /* field[0] = distinct_clause */
  3,   /* field[3] = from_clause */
  5,   /* field[5] = group_clause */
  6,   /* field[6] = group_distinct */
  7,   /* field[7] = having_clause */
  1,   /* field[1] = into_clause */
  18,   /* field[18] = larg */
  12,   /* field[12] = limit_count */
  11,   /* field[11] = limit_offset */
  13,   /* field[13] = limit_option */
  14,   /* field[14] = locking_clause */
  16,   /* field[16] = op */
  19,   /* field[19] = rarg */
  10,   /* field[10] = sort_clause */
  2,   /* field[2] = target_list */
  9,   /* field[9] = values_lists */
  4,   /* field[4] = where_clause */
  8,   /* field[8] = window_clause */
  15,   /* field[15] = with_clause */
};
static const ProtobufCIntRange pg_query__select_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 20 }
};
const ProtobufCMessageDescriptor pg_query__select_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.SelectStmt",
  "SelectStmt",
  "PgQuery__SelectStmt",
  "pg_query",
  sizeof(PgQuery__SelectStmt),
  20,
  pg_query__select_stmt__field_descriptors,
  pg_query__select_stmt__field_indices_by_name,
  1,  pg_query__select_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__select_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__set_operation_stmt__field_descriptors[8] =
{
  {
    "op",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SetOperationStmt, op),
    &pg_query__set_operation__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "all",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SetOperationStmt, all),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "larg",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SetOperationStmt, larg),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "rarg",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SetOperationStmt, rarg),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "col_types",
    5,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__SetOperationStmt, n_col_types),
    offsetof(PgQuery__SetOperationStmt, col_types),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "col_typmods",
    6,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__SetOperationStmt, n_col_typmods),
    offsetof(PgQuery__SetOperationStmt, col_typmods),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "col_collations",
    7,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__SetOperationStmt, n_col_collations),
    offsetof(PgQuery__SetOperationStmt, col_collations),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "group_clauses",
    8,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__SetOperationStmt, n_group_clauses),
    offsetof(PgQuery__SetOperationStmt, group_clauses),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__set_operation_stmt__field_indices_by_name[] = {
  1,   /* field[1] = all */
  6,   /* field[6] = col_collations */
  4,   /* field[4] = col_types */
  5,   /* field[5] = col_typmods */
  7,   /* field[7] = group_clauses */
  2,   /* field[2] = larg */
  0,   /* field[0] = op */
  3,   /* field[3] = rarg */
};
static const ProtobufCIntRange pg_query__set_operation_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 8 }
};
const ProtobufCMessageDescriptor pg_query__set_operation_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.SetOperationStmt",
  "SetOperationStmt",
  "PgQuery__SetOperationStmt",
  "pg_query",
  sizeof(PgQuery__SetOperationStmt),
  8,
  pg_query__set_operation_stmt__field_descriptors,
  pg_query__set_operation_stmt__field_indices_by_name,
  1,  pg_query__set_operation_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__set_operation_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__return_stmt__field_descriptors[1] =
{
  {
    "returnval",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ReturnStmt, returnval),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__return_stmt__field_indices_by_name[] = {
  0,   /* field[0] = returnval */
};
static const ProtobufCIntRange pg_query__return_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 1 }
};
const ProtobufCMessageDescriptor pg_query__return_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.ReturnStmt",
  "ReturnStmt",
  "PgQuery__ReturnStmt",
  "pg_query",
  sizeof(PgQuery__ReturnStmt),
  1,
  pg_query__return_stmt__field_descriptors,
  pg_query__return_stmt__field_indices_by_name,
  1,  pg_query__return_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__return_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__plassign_stmt__field_descriptors[5] =
{
  {
    "name",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__PLAssignStmt, name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "indirection",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__PLAssignStmt, n_indirection),
    offsetof(PgQuery__PLAssignStmt, indirection),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "nnames",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__PLAssignStmt, nnames),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "val",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__PLAssignStmt, val),
    &pg_query__select_stmt__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__PLAssignStmt, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__plassign_stmt__field_indices_by_name[] = {
  1,   /* field[1] = indirection */
  4,   /* field[4] = location */
  0,   /* field[0] = name */
  2,   /* field[2] = nnames */
  3,   /* field[3] = val */
};
static const ProtobufCIntRange pg_query__plassign_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor pg_query__plassign_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.PLAssignStmt",
  "PLAssignStmt",
  "PgQuery__PLAssignStmt",
  "pg_query",
  sizeof(PgQuery__PLAssignStmt),
  5,
  pg_query__plassign_stmt__field_descriptors,
  pg_query__plassign_stmt__field_indices_by_name,
  1,  pg_query__plassign_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__plassign_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__create_schema_stmt__field_descriptors[4] =
{
  {
    "schemaname",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateSchemaStmt, schemaname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "authrole",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateSchemaStmt, authrole),
    &pg_query__role_spec__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "schema_elts",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateSchemaStmt, n_schema_elts),
    offsetof(PgQuery__CreateSchemaStmt, schema_elts),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "if_not_exists",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateSchemaStmt, if_not_exists),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__create_schema_stmt__field_indices_by_name[] = {
  1,   /* field[1] = authrole */
  3,   /* field[3] = if_not_exists */
  2,   /* field[2] = schema_elts */
  0,   /* field[0] = schemaname */
};
static const ProtobufCIntRange pg_query__create_schema_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor pg_query__create_schema_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CreateSchemaStmt",
  "CreateSchemaStmt",
  "PgQuery__CreateSchemaStmt",
  "pg_query",
  sizeof(PgQuery__CreateSchemaStmt),
  4,
  pg_query__create_schema_stmt__field_descriptors,
  pg_query__create_schema_stmt__field_indices_by_name,
  1,  pg_query__create_schema_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__create_schema_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__alter_table_stmt__field_descriptors[4] =
{
  {
    "relation",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterTableStmt, relation),
    &pg_query__range_var__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "cmds",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AlterTableStmt, n_cmds),
    offsetof(PgQuery__AlterTableStmt, cmds),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "objtype",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterTableStmt, objtype),
    &pg_query__object_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "missing_ok",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterTableStmt, missing_ok),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__alter_table_stmt__field_indices_by_name[] = {
  1,   /* field[1] = cmds */
  3,   /* field[3] = missing_ok */
  2,   /* field[2] = objtype */
  0,   /* field[0] = relation */
};
static const ProtobufCIntRange pg_query__alter_table_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor pg_query__alter_table_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.AlterTableStmt",
  "AlterTableStmt",
  "PgQuery__AlterTableStmt",
  "pg_query",
  sizeof(PgQuery__AlterTableStmt),
  4,
  pg_query__alter_table_stmt__field_descriptors,
  pg_query__alter_table_stmt__field_indices_by_name,
  1,  pg_query__alter_table_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__alter_table_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__replica_identity_stmt__field_descriptors[2] =
{
  {
    "identity_type",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ReplicaIdentityStmt, identity_type),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "name",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ReplicaIdentityStmt, name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__replica_identity_stmt__field_indices_by_name[] = {
  0,   /* field[0] = identity_type */
  1,   /* field[1] = name */
};
static const ProtobufCIntRange pg_query__replica_identity_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor pg_query__replica_identity_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.ReplicaIdentityStmt",
  "ReplicaIdentityStmt",
  "PgQuery__ReplicaIdentityStmt",
  "pg_query",
  sizeof(PgQuery__ReplicaIdentityStmt),
  2,
  pg_query__replica_identity_stmt__field_descriptors,
  pg_query__replica_identity_stmt__field_indices_by_name,
  1,  pg_query__replica_identity_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__replica_identity_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__alter_table_cmd__field_descriptors[8] =
{
  {
    "subtype",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterTableCmd, subtype),
    &pg_query__alter_table_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "name",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterTableCmd, name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "num",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterTableCmd, num),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "newowner",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterTableCmd, newowner),
    &pg_query__role_spec__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "def",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterTableCmd, def),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "behavior",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterTableCmd, behavior),
    &pg_query__drop_behavior__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "missing_ok",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterTableCmd, missing_ok),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "recurse",
    8,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterTableCmd, recurse),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__alter_table_cmd__field_indices_by_name[] = {
  5,   /* field[5] = behavior */
  4,   /* field[4] = def */
  6,   /* field[6] = missing_ok */
  1,   /* field[1] = name */
  3,   /* field[3] = newowner */
  2,   /* field[2] = num */
  7,   /* field[7] = recurse */
  0,   /* field[0] = subtype */
};
static const ProtobufCIntRange pg_query__alter_table_cmd__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 8 }
};
const ProtobufCMessageDescriptor pg_query__alter_table_cmd__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.AlterTableCmd",
  "AlterTableCmd",
  "PgQuery__AlterTableCmd",
  "pg_query",
  sizeof(PgQuery__AlterTableCmd),
  8,
  pg_query__alter_table_cmd__field_descriptors,
  pg_query__alter_table_cmd__field_indices_by_name,
  1,  pg_query__alter_table_cmd__number_ranges,
  (ProtobufCMessageInit) pg_query__alter_table_cmd__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__alter_collation_stmt__field_descriptors[1] =
{
  {
    "collname",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AlterCollationStmt, n_collname),
    offsetof(PgQuery__AlterCollationStmt, collname),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__alter_collation_stmt__field_indices_by_name[] = {
  0,   /* field[0] = collname */
};
static const ProtobufCIntRange pg_query__alter_collation_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 1 }
};
const ProtobufCMessageDescriptor pg_query__alter_collation_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.AlterCollationStmt",
  "AlterCollationStmt",
  "PgQuery__AlterCollationStmt",
  "pg_query",
  sizeof(PgQuery__AlterCollationStmt),
  1,
  pg_query__alter_collation_stmt__field_descriptors,
  pg_query__alter_collation_stmt__field_indices_by_name,
  1,  pg_query__alter_collation_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__alter_collation_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__alter_domain_stmt__field_descriptors[6] =
{
  {
    "subtype",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterDomainStmt, subtype),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "type_name",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AlterDomainStmt, n_type_name),
    offsetof(PgQuery__AlterDomainStmt, type_name),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "name",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterDomainStmt, name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "def",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterDomainStmt, def),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "behavior",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterDomainStmt, behavior),
    &pg_query__drop_behavior__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "missing_ok",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterDomainStmt, missing_ok),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__alter_domain_stmt__field_indices_by_name[] = {
  4,   /* field[4] = behavior */
  3,   /* field[3] = def */
  5,   /* field[5] = missing_ok */
  2,   /* field[2] = name */
  0,   /* field[0] = subtype */
  1,   /* field[1] = type_name */
};
static const ProtobufCIntRange pg_query__alter_domain_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 6 }
};
const ProtobufCMessageDescriptor pg_query__alter_domain_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.AlterDomainStmt",
  "AlterDomainStmt",
  "PgQuery__AlterDomainStmt",
  "pg_query",
  sizeof(PgQuery__AlterDomainStmt),
  6,
  pg_query__alter_domain_stmt__field_descriptors,
  pg_query__alter_domain_stmt__field_indices_by_name,
  1,  pg_query__alter_domain_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__alter_domain_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__grant_stmt__field_descriptors[9] =
{
  {
    "is_grant",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__GrantStmt, is_grant),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "targtype",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__GrantStmt, targtype),
    &pg_query__grant_target_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "objtype",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__GrantStmt, objtype),
    &pg_query__object_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "objects",
    4,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__GrantStmt, n_objects),
    offsetof(PgQuery__GrantStmt, objects),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "privileges",
    5,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__GrantStmt, n_privileges),
    offsetof(PgQuery__GrantStmt, privileges),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "grantees",
    6,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__GrantStmt, n_grantees),
    offsetof(PgQuery__GrantStmt, grantees),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "grant_option",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__GrantStmt, grant_option),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "grantor",
    8,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__GrantStmt, grantor),
    &pg_query__role_spec__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "behavior",
    9,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__GrantStmt, behavior),
    &pg_query__drop_behavior__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__grant_stmt__field_indices_by_name[] = {
  8,   /* field[8] = behavior */
  6,   /* field[6] = grant_option */
  5,   /* field[5] = grantees */
  7,   /* field[7] = grantor */
  0,   /* field[0] = is_grant */
  3,   /* field[3] = objects */
  2,   /* field[2] = objtype */
  4,   /* field[4] = privileges */
  1,   /* field[1] = targtype */
};
static const ProtobufCIntRange pg_query__grant_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 9 }
};
const ProtobufCMessageDescriptor pg_query__grant_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.GrantStmt",
  "GrantStmt",
  "PgQuery__GrantStmt",
  "pg_query",
  sizeof(PgQuery__GrantStmt),
  9,
  pg_query__grant_stmt__field_descriptors,
  pg_query__grant_stmt__field_indices_by_name,
  1,  pg_query__grant_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__grant_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__object_with_args__field_descriptors[4] =
{
  {
    "objname",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__ObjectWithArgs, n_objname),
    offsetof(PgQuery__ObjectWithArgs, objname),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "objargs",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__ObjectWithArgs, n_objargs),
    offsetof(PgQuery__ObjectWithArgs, objargs),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "objfuncargs",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__ObjectWithArgs, n_objfuncargs),
    offsetof(PgQuery__ObjectWithArgs, objfuncargs),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "args_unspecified",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ObjectWithArgs, args_unspecified),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__object_with_args__field_indices_by_name[] = {
  3,   /* field[3] = args_unspecified */
  1,   /* field[1] = objargs */
  2,   /* field[2] = objfuncargs */
  0,   /* field[0] = objname */
};
static const ProtobufCIntRange pg_query__object_with_args__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor pg_query__object_with_args__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.ObjectWithArgs",
  "ObjectWithArgs",
  "PgQuery__ObjectWithArgs",
  "pg_query",
  sizeof(PgQuery__ObjectWithArgs),
  4,
  pg_query__object_with_args__field_descriptors,
  pg_query__object_with_args__field_indices_by_name,
  1,  pg_query__object_with_args__number_ranges,
  (ProtobufCMessageInit) pg_query__object_with_args__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__access_priv__field_descriptors[2] =
{
  {
    "priv_name",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AccessPriv, priv_name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "cols",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AccessPriv, n_cols),
    offsetof(PgQuery__AccessPriv, cols),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__access_priv__field_indices_by_name[] = {
  1,   /* field[1] = cols */
  0,   /* field[0] = priv_name */
};
static const ProtobufCIntRange pg_query__access_priv__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor pg_query__access_priv__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.AccessPriv",
  "AccessPriv",
  "PgQuery__AccessPriv",
  "pg_query",
  sizeof(PgQuery__AccessPriv),
  2,
  pg_query__access_priv__field_descriptors,
  pg_query__access_priv__field_indices_by_name,
  1,  pg_query__access_priv__number_ranges,
  (ProtobufCMessageInit) pg_query__access_priv__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__grant_role_stmt__field_descriptors[6] =
{
  {
    "granted_roles",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__GrantRoleStmt, n_granted_roles),
    offsetof(PgQuery__GrantRoleStmt, granted_roles),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "grantee_roles",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__GrantRoleStmt, n_grantee_roles),
    offsetof(PgQuery__GrantRoleStmt, grantee_roles),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "is_grant",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__GrantRoleStmt, is_grant),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "opt",
    4,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__GrantRoleStmt, n_opt),
    offsetof(PgQuery__GrantRoleStmt, opt),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "grantor",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__GrantRoleStmt, grantor),
    &pg_query__role_spec__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "behavior",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__GrantRoleStmt, behavior),
    &pg_query__drop_behavior__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__grant_role_stmt__field_indices_by_name[] = {
  5,   /* field[5] = behavior */
  0,   /* field[0] = granted_roles */
  1,   /* field[1] = grantee_roles */
  4,   /* field[4] = grantor */
  2,   /* field[2] = is_grant */
  3,   /* field[3] = opt */
};
static const ProtobufCIntRange pg_query__grant_role_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 6 }
};
const ProtobufCMessageDescriptor pg_query__grant_role_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.GrantRoleStmt",
  "GrantRoleStmt",
  "PgQuery__GrantRoleStmt",
  "pg_query",
  sizeof(PgQuery__GrantRoleStmt),
  6,
  pg_query__grant_role_stmt__field_descriptors,
  pg_query__grant_role_stmt__field_indices_by_name,
  1,  pg_query__grant_role_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__grant_role_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__alter_default_privileges_stmt__field_descriptors[2] =
{
  {
    "options",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AlterDefaultPrivilegesStmt, n_options),
    offsetof(PgQuery__AlterDefaultPrivilegesStmt, options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "action",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterDefaultPrivilegesStmt, action),
    &pg_query__grant_stmt__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__alter_default_privileges_stmt__field_indices_by_name[] = {
  1,   /* field[1] = action */
  0,   /* field[0] = options */
};
static const ProtobufCIntRange pg_query__alter_default_privileges_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor pg_query__alter_default_privileges_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.AlterDefaultPrivilegesStmt",
  "AlterDefaultPrivilegesStmt",
  "PgQuery__AlterDefaultPrivilegesStmt",
  "pg_query",
  sizeof(PgQuery__AlterDefaultPrivilegesStmt),
  2,
  pg_query__alter_default_privileges_stmt__field_descriptors,
  pg_query__alter_default_privileges_stmt__field_indices_by_name,
  1,  pg_query__alter_default_privileges_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__alter_default_privileges_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__copy_stmt__field_descriptors[8] =
{
  {
    "relation",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CopyStmt, relation),
    &pg_query__range_var__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "query",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CopyStmt, query),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "attlist",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CopyStmt, n_attlist),
    offsetof(PgQuery__CopyStmt, attlist),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "is_from",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CopyStmt, is_from),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "is_program",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CopyStmt, is_program),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "filename",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CopyStmt, filename),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "options",
    7,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CopyStmt, n_options),
    offsetof(PgQuery__CopyStmt, options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "where_clause",
    8,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CopyStmt, where_clause),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__copy_stmt__field_indices_by_name[] = {
  2,   /* field[2] = attlist */
  5,   /* field[5] = filename */
  3,   /* field[3] = is_from */
  4,   /* field[4] = is_program */
  6,   /* field[6] = options */
  1,   /* field[1] = query */
  0,   /* field[0] = relation */
  7,   /* field[7] = where_clause */
};
static const ProtobufCIntRange pg_query__copy_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 8 }
};
const ProtobufCMessageDescriptor pg_query__copy_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CopyStmt",
  "CopyStmt",
  "PgQuery__CopyStmt",
  "pg_query",
  sizeof(PgQuery__CopyStmt),
  8,
  pg_query__copy_stmt__field_descriptors,
  pg_query__copy_stmt__field_indices_by_name,
  1,  pg_query__copy_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__copy_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__variable_set_stmt__field_descriptors[4] =
{
  {
    "kind",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__VariableSetStmt, kind),
    &pg_query__variable_set_kind__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "name",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__VariableSetStmt, name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "args",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__VariableSetStmt, n_args),
    offsetof(PgQuery__VariableSetStmt, args),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "is_local",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__VariableSetStmt, is_local),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__variable_set_stmt__field_indices_by_name[] = {
  2,   /* field[2] = args */
  3,   /* field[3] = is_local */
  0,   /* field[0] = kind */
  1,   /* field[1] = name */
};
static const ProtobufCIntRange pg_query__variable_set_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor pg_query__variable_set_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.VariableSetStmt",
  "VariableSetStmt",
  "PgQuery__VariableSetStmt",
  "pg_query",
  sizeof(PgQuery__VariableSetStmt),
  4,
  pg_query__variable_set_stmt__field_descriptors,
  pg_query__variable_set_stmt__field_indices_by_name,
  1,  pg_query__variable_set_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__variable_set_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__variable_show_stmt__field_descriptors[1] =
{
  {
    "name",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__VariableShowStmt, name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__variable_show_stmt__field_indices_by_name[] = {
  0,   /* field[0] = name */
};
static const ProtobufCIntRange pg_query__variable_show_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 1 }
};
const ProtobufCMessageDescriptor pg_query__variable_show_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.VariableShowStmt",
  "VariableShowStmt",
  "PgQuery__VariableShowStmt",
  "pg_query",
  sizeof(PgQuery__VariableShowStmt),
  1,
  pg_query__variable_show_stmt__field_descriptors,
  pg_query__variable_show_stmt__field_indices_by_name,
  1,  pg_query__variable_show_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__variable_show_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__create_stmt__field_descriptors[12] =
{
  {
    "relation",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateStmt, relation),
    &pg_query__range_var__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "table_elts",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateStmt, n_table_elts),
    offsetof(PgQuery__CreateStmt, table_elts),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "inh_relations",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateStmt, n_inh_relations),
    offsetof(PgQuery__CreateStmt, inh_relations),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "partbound",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateStmt, partbound),
    &pg_query__partition_bound_spec__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "partspec",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateStmt, partspec),
    &pg_query__partition_spec__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "of_typename",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateStmt, of_typename),
    &pg_query__type_name__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "constraints",
    7,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateStmt, n_constraints),
    offsetof(PgQuery__CreateStmt, constraints),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "options",
    8,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateStmt, n_options),
    offsetof(PgQuery__CreateStmt, options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "oncommit",
    9,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateStmt, oncommit),
    &pg_query__on_commit_action__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "tablespacename",
    10,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateStmt, tablespacename),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "access_method",
    11,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateStmt, access_method),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "if_not_exists",
    12,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateStmt, if_not_exists),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__create_stmt__field_indices_by_name[] = {
  10,   /* field[10] = access_method */
  6,   /* field[6] = constraints */
  11,   /* field[11] = if_not_exists */
  2,   /* field[2] = inh_relations */
  5,   /* field[5] = of_typename */
  8,   /* field[8] = oncommit */
  7,   /* field[7] = options */
  3,   /* field[3] = partbound */
  4,   /* field[4] = partspec */
  0,   /* field[0] = relation */
  1,   /* field[1] = table_elts */
  9,   /* field[9] = tablespacename */
};
static const ProtobufCIntRange pg_query__create_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 12 }
};
const ProtobufCMessageDescriptor pg_query__create_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CreateStmt",
  "CreateStmt",
  "PgQuery__CreateStmt",
  "pg_query",
  sizeof(PgQuery__CreateStmt),
  12,
  pg_query__create_stmt__field_descriptors,
  pg_query__create_stmt__field_indices_by_name,
  1,  pg_query__create_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__create_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__constraint__field_descriptors[31] =
{
  {
    "contype",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Constraint, contype),
    &pg_query__constr_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "conname",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Constraint, conname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "deferrable",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Constraint, deferrable),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "initdeferred",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Constraint, initdeferred),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "skip_validation",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Constraint, skip_validation),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "initially_valid",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Constraint, initially_valid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "is_no_inherit",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Constraint, is_no_inherit),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "raw_expr",
    8,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Constraint, raw_expr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "cooked_expr",
    9,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Constraint, cooked_expr),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "generated_when",
    10,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Constraint, generated_when),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "inhcount",
    11,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Constraint, inhcount),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "nulls_not_distinct",
    12,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Constraint, nulls_not_distinct),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "keys",
    13,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Constraint, n_keys),
    offsetof(PgQuery__Constraint, keys),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "including",
    14,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Constraint, n_including),
    offsetof(PgQuery__Constraint, including),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "exclusions",
    15,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Constraint, n_exclusions),
    offsetof(PgQuery__Constraint, exclusions),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "options",
    16,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Constraint, n_options),
    offsetof(PgQuery__Constraint, options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "indexname",
    17,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Constraint, indexname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "indexspace",
    18,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Constraint, indexspace),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "reset_default_tblspc",
    19,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Constraint, reset_default_tblspc),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "access_method",
    20,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Constraint, access_method),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "where_clause",
    21,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Constraint, where_clause),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "pktable",
    22,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Constraint, pktable),
    &pg_query__range_var__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "fk_attrs",
    23,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Constraint, n_fk_attrs),
    offsetof(PgQuery__Constraint, fk_attrs),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "pk_attrs",
    24,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Constraint, n_pk_attrs),
    offsetof(PgQuery__Constraint, pk_attrs),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "fk_matchtype",
    25,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Constraint, fk_matchtype),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "fk_upd_action",
    26,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Constraint, fk_upd_action),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "fk_del_action",
    27,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Constraint, fk_del_action),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "fk_del_set_cols",
    28,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Constraint, n_fk_del_set_cols),
    offsetof(PgQuery__Constraint, fk_del_set_cols),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "old_conpfeqop",
    29,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__Constraint, n_old_conpfeqop),
    offsetof(PgQuery__Constraint, old_conpfeqop),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "old_pktable_oid",
    30,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Constraint, old_pktable_oid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    31,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__Constraint, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__constraint__field_indices_by_name[] = {
  19,   /* field[19] = access_method */
  1,   /* field[1] = conname */
  0,   /* field[0] = contype */
  8,   /* field[8] = cooked_expr */
  2,   /* field[2] = deferrable */
  14,   /* field[14] = exclusions */
  22,   /* field[22] = fk_attrs */
  26,   /* field[26] = fk_del_action */
  27,   /* field[27] = fk_del_set_cols */
  24,   /* field[24] = fk_matchtype */
  25,   /* field[25] = fk_upd_action */
  9,   /* field[9] = generated_when */
  13,   /* field[13] = including */
  16,   /* field[16] = indexname */
  17,   /* field[17] = indexspace */
  10,   /* field[10] = inhcount */
  3,   /* field[3] = initdeferred */
  5,   /* field[5] = initially_valid */
  6,   /* field[6] = is_no_inherit */
  12,   /* field[12] = keys */
  30,   /* field[30] = location */
  11,   /* field[11] = nulls_not_distinct */
  28,   /* field[28] = old_conpfeqop */
  29,   /* field[29] = old_pktable_oid */
  15,   /* field[15] = options */
  23,   /* field[23] = pk_attrs */
  21,   /* field[21] = pktable */
  7,   /* field[7] = raw_expr */
  18,   /* field[18] = reset_default_tblspc */
  4,   /* field[4] = skip_validation */
  20,   /* field[20] = where_clause */
};
static const ProtobufCIntRange pg_query__constraint__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 31 }
};
const ProtobufCMessageDescriptor pg_query__constraint__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.Constraint",
  "Constraint",
  "PgQuery__Constraint",
  "pg_query",
  sizeof(PgQuery__Constraint),
  31,
  pg_query__constraint__field_descriptors,
  pg_query__constraint__field_indices_by_name,
  1,  pg_query__constraint__number_ranges,
  (ProtobufCMessageInit) pg_query__constraint__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__create_table_space_stmt__field_descriptors[4] =
{
  {
    "tablespacename",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateTableSpaceStmt, tablespacename),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "owner",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateTableSpaceStmt, owner),
    &pg_query__role_spec__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateTableSpaceStmt, location),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "options",
    4,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateTableSpaceStmt, n_options),
    offsetof(PgQuery__CreateTableSpaceStmt, options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__create_table_space_stmt__field_indices_by_name[] = {
  2,   /* field[2] = location */
  3,   /* field[3] = options */
  1,   /* field[1] = owner */
  0,   /* field[0] = tablespacename */
};
static const ProtobufCIntRange pg_query__create_table_space_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor pg_query__create_table_space_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CreateTableSpaceStmt",
  "CreateTableSpaceStmt",
  "PgQuery__CreateTableSpaceStmt",
  "pg_query",
  sizeof(PgQuery__CreateTableSpaceStmt),
  4,
  pg_query__create_table_space_stmt__field_descriptors,
  pg_query__create_table_space_stmt__field_indices_by_name,
  1,  pg_query__create_table_space_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__create_table_space_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__drop_table_space_stmt__field_descriptors[2] =
{
  {
    "tablespacename",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DropTableSpaceStmt, tablespacename),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "missing_ok",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DropTableSpaceStmt, missing_ok),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__drop_table_space_stmt__field_indices_by_name[] = {
  1,   /* field[1] = missing_ok */
  0,   /* field[0] = tablespacename */
};
static const ProtobufCIntRange pg_query__drop_table_space_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor pg_query__drop_table_space_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.DropTableSpaceStmt",
  "DropTableSpaceStmt",
  "PgQuery__DropTableSpaceStmt",
  "pg_query",
  sizeof(PgQuery__DropTableSpaceStmt),
  2,
  pg_query__drop_table_space_stmt__field_descriptors,
  pg_query__drop_table_space_stmt__field_indices_by_name,
  1,  pg_query__drop_table_space_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__drop_table_space_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__alter_table_space_options_stmt__field_descriptors[3] =
{
  {
    "tablespacename",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterTableSpaceOptionsStmt, tablespacename),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "options",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AlterTableSpaceOptionsStmt, n_options),
    offsetof(PgQuery__AlterTableSpaceOptionsStmt, options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "is_reset",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterTableSpaceOptionsStmt, is_reset),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__alter_table_space_options_stmt__field_indices_by_name[] = {
  2,   /* field[2] = is_reset */
  1,   /* field[1] = options */
  0,   /* field[0] = tablespacename */
};
static const ProtobufCIntRange pg_query__alter_table_space_options_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__alter_table_space_options_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.AlterTableSpaceOptionsStmt",
  "AlterTableSpaceOptionsStmt",
  "PgQuery__AlterTableSpaceOptionsStmt",
  "pg_query",
  sizeof(PgQuery__AlterTableSpaceOptionsStmt),
  3,
  pg_query__alter_table_space_options_stmt__field_descriptors,
  pg_query__alter_table_space_options_stmt__field_indices_by_name,
  1,  pg_query__alter_table_space_options_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__alter_table_space_options_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__alter_table_move_all_stmt__field_descriptors[5] =
{
  {
    "orig_tablespacename",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterTableMoveAllStmt, orig_tablespacename),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "objtype",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterTableMoveAllStmt, objtype),
    &pg_query__object_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "roles",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AlterTableMoveAllStmt, n_roles),
    offsetof(PgQuery__AlterTableMoveAllStmt, roles),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "new_tablespacename",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterTableMoveAllStmt, new_tablespacename),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "nowait",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterTableMoveAllStmt, nowait),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__alter_table_move_all_stmt__field_indices_by_name[] = {
  3,   /* field[3] = new_tablespacename */
  4,   /* field[4] = nowait */
  1,   /* field[1] = objtype */
  0,   /* field[0] = orig_tablespacename */
  2,   /* field[2] = roles */
};
static const ProtobufCIntRange pg_query__alter_table_move_all_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor pg_query__alter_table_move_all_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.AlterTableMoveAllStmt",
  "AlterTableMoveAllStmt",
  "PgQuery__AlterTableMoveAllStmt",
  "pg_query",
  sizeof(PgQuery__AlterTableMoveAllStmt),
  5,
  pg_query__alter_table_move_all_stmt__field_descriptors,
  pg_query__alter_table_move_all_stmt__field_indices_by_name,
  1,  pg_query__alter_table_move_all_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__alter_table_move_all_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__create_extension_stmt__field_descriptors[3] =
{
  {
    "extname",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateExtensionStmt, extname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "if_not_exists",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateExtensionStmt, if_not_exists),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "options",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateExtensionStmt, n_options),
    offsetof(PgQuery__CreateExtensionStmt, options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__create_extension_stmt__field_indices_by_name[] = {
  0,   /* field[0] = extname */
  1,   /* field[1] = if_not_exists */
  2,   /* field[2] = options */
};
static const ProtobufCIntRange pg_query__create_extension_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__create_extension_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CreateExtensionStmt",
  "CreateExtensionStmt",
  "PgQuery__CreateExtensionStmt",
  "pg_query",
  sizeof(PgQuery__CreateExtensionStmt),
  3,
  pg_query__create_extension_stmt__field_descriptors,
  pg_query__create_extension_stmt__field_indices_by_name,
  1,  pg_query__create_extension_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__create_extension_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__alter_extension_stmt__field_descriptors[2] =
{
  {
    "extname",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterExtensionStmt, extname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "options",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AlterExtensionStmt, n_options),
    offsetof(PgQuery__AlterExtensionStmt, options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__alter_extension_stmt__field_indices_by_name[] = {
  0,   /* field[0] = extname */
  1,   /* field[1] = options */
};
static const ProtobufCIntRange pg_query__alter_extension_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor pg_query__alter_extension_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.AlterExtensionStmt",
  "AlterExtensionStmt",
  "PgQuery__AlterExtensionStmt",
  "pg_query",
  sizeof(PgQuery__AlterExtensionStmt),
  2,
  pg_query__alter_extension_stmt__field_descriptors,
  pg_query__alter_extension_stmt__field_indices_by_name,
  1,  pg_query__alter_extension_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__alter_extension_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__alter_extension_contents_stmt__field_descriptors[4] =
{
  {
    "extname",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterExtensionContentsStmt, extname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "action",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterExtensionContentsStmt, action),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "objtype",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterExtensionContentsStmt, objtype),
    &pg_query__object_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "object",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterExtensionContentsStmt, object),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__alter_extension_contents_stmt__field_indices_by_name[] = {
  1,   /* field[1] = action */
  0,   /* field[0] = extname */
  3,   /* field[3] = object */
  2,   /* field[2] = objtype */
};
static const ProtobufCIntRange pg_query__alter_extension_contents_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor pg_query__alter_extension_contents_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.AlterExtensionContentsStmt",
  "AlterExtensionContentsStmt",
  "PgQuery__AlterExtensionContentsStmt",
  "pg_query",
  sizeof(PgQuery__AlterExtensionContentsStmt),
  4,
  pg_query__alter_extension_contents_stmt__field_descriptors,
  pg_query__alter_extension_contents_stmt__field_indices_by_name,
  1,  pg_query__alter_extension_contents_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__alter_extension_contents_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__create_fdw_stmt__field_descriptors[3] =
{
  {
    "fdwname",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateFdwStmt, fdwname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "func_options",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateFdwStmt, n_func_options),
    offsetof(PgQuery__CreateFdwStmt, func_options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "options",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateFdwStmt, n_options),
    offsetof(PgQuery__CreateFdwStmt, options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__create_fdw_stmt__field_indices_by_name[] = {
  0,   /* field[0] = fdwname */
  1,   /* field[1] = func_options */
  2,   /* field[2] = options */
};
static const ProtobufCIntRange pg_query__create_fdw_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__create_fdw_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CreateFdwStmt",
  "CreateFdwStmt",
  "PgQuery__CreateFdwStmt",
  "pg_query",
  sizeof(PgQuery__CreateFdwStmt),
  3,
  pg_query__create_fdw_stmt__field_descriptors,
  pg_query__create_fdw_stmt__field_indices_by_name,
  1,  pg_query__create_fdw_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__create_fdw_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__alter_fdw_stmt__field_descriptors[3] =
{
  {
    "fdwname",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterFdwStmt, fdwname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "func_options",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AlterFdwStmt, n_func_options),
    offsetof(PgQuery__AlterFdwStmt, func_options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "options",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AlterFdwStmt, n_options),
    offsetof(PgQuery__AlterFdwStmt, options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__alter_fdw_stmt__field_indices_by_name[] = {
  0,   /* field[0] = fdwname */
  1,   /* field[1] = func_options */
  2,   /* field[2] = options */
};
static const ProtobufCIntRange pg_query__alter_fdw_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__alter_fdw_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.AlterFdwStmt",
  "AlterFdwStmt",
  "PgQuery__AlterFdwStmt",
  "pg_query",
  sizeof(PgQuery__AlterFdwStmt),
  3,
  pg_query__alter_fdw_stmt__field_descriptors,
  pg_query__alter_fdw_stmt__field_indices_by_name,
  1,  pg_query__alter_fdw_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__alter_fdw_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__create_foreign_server_stmt__field_descriptors[6] =
{
  {
    "servername",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateForeignServerStmt, servername),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "servertype",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateForeignServerStmt, servertype),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "version",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateForeignServerStmt, version),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "fdwname",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateForeignServerStmt, fdwname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "if_not_exists",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateForeignServerStmt, if_not_exists),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "options",
    6,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateForeignServerStmt, n_options),
    offsetof(PgQuery__CreateForeignServerStmt, options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__create_foreign_server_stmt__field_indices_by_name[] = {
  3,   /* field[3] = fdwname */
  4,   /* field[4] = if_not_exists */
  5,   /* field[5] = options */
  0,   /* field[0] = servername */
  1,   /* field[1] = servertype */
  2,   /* field[2] = version */
};
static const ProtobufCIntRange pg_query__create_foreign_server_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 6 }
};
const ProtobufCMessageDescriptor pg_query__create_foreign_server_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CreateForeignServerStmt",
  "CreateForeignServerStmt",
  "PgQuery__CreateForeignServerStmt",
  "pg_query",
  sizeof(PgQuery__CreateForeignServerStmt),
  6,
  pg_query__create_foreign_server_stmt__field_descriptors,
  pg_query__create_foreign_server_stmt__field_indices_by_name,
  1,  pg_query__create_foreign_server_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__create_foreign_server_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__alter_foreign_server_stmt__field_descriptors[4] =
{
  {
    "servername",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterForeignServerStmt, servername),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "version",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterForeignServerStmt, version),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "options",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AlterForeignServerStmt, n_options),
    offsetof(PgQuery__AlterForeignServerStmt, options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "has_version",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterForeignServerStmt, has_version),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__alter_foreign_server_stmt__field_indices_by_name[] = {
  3,   /* field[3] = has_version */
  2,   /* field[2] = options */
  0,   /* field[0] = servername */
  1,   /* field[1] = version */
};
static const ProtobufCIntRange pg_query__alter_foreign_server_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor pg_query__alter_foreign_server_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.AlterForeignServerStmt",
  "AlterForeignServerStmt",
  "PgQuery__AlterForeignServerStmt",
  "pg_query",
  sizeof(PgQuery__AlterForeignServerStmt),
  4,
  pg_query__alter_foreign_server_stmt__field_descriptors,
  pg_query__alter_foreign_server_stmt__field_indices_by_name,
  1,  pg_query__alter_foreign_server_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__alter_foreign_server_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__create_foreign_table_stmt__field_descriptors[3] =
{
  {
    "base_stmt",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateForeignTableStmt, base_stmt),
    &pg_query__create_stmt__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "servername",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateForeignTableStmt, servername),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "options",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateForeignTableStmt, n_options),
    offsetof(PgQuery__CreateForeignTableStmt, options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__create_foreign_table_stmt__field_indices_by_name[] = {
  0,   /* field[0] = base_stmt */
  2,   /* field[2] = options */
  1,   /* field[1] = servername */
};
static const ProtobufCIntRange pg_query__create_foreign_table_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__create_foreign_table_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CreateForeignTableStmt",
  "CreateForeignTableStmt",
  "PgQuery__CreateForeignTableStmt",
  "pg_query",
  sizeof(PgQuery__CreateForeignTableStmt),
  3,
  pg_query__create_foreign_table_stmt__field_descriptors,
  pg_query__create_foreign_table_stmt__field_indices_by_name,
  1,  pg_query__create_foreign_table_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__create_foreign_table_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__create_user_mapping_stmt__field_descriptors[4] =
{
  {
    "user",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateUserMappingStmt, user),
    &pg_query__role_spec__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "servername",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateUserMappingStmt, servername),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "if_not_exists",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateUserMappingStmt, if_not_exists),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "options",
    4,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateUserMappingStmt, n_options),
    offsetof(PgQuery__CreateUserMappingStmt, options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__create_user_mapping_stmt__field_indices_by_name[] = {
  2,   /* field[2] = if_not_exists */
  3,   /* field[3] = options */
  1,   /* field[1] = servername */
  0,   /* field[0] = user */
};
static const ProtobufCIntRange pg_query__create_user_mapping_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor pg_query__create_user_mapping_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CreateUserMappingStmt",
  "CreateUserMappingStmt",
  "PgQuery__CreateUserMappingStmt",
  "pg_query",
  sizeof(PgQuery__CreateUserMappingStmt),
  4,
  pg_query__create_user_mapping_stmt__field_descriptors,
  pg_query__create_user_mapping_stmt__field_indices_by_name,
  1,  pg_query__create_user_mapping_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__create_user_mapping_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__alter_user_mapping_stmt__field_descriptors[3] =
{
  {
    "user",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterUserMappingStmt, user),
    &pg_query__role_spec__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "servername",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterUserMappingStmt, servername),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "options",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AlterUserMappingStmt, n_options),
    offsetof(PgQuery__AlterUserMappingStmt, options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__alter_user_mapping_stmt__field_indices_by_name[] = {
  2,   /* field[2] = options */
  1,   /* field[1] = servername */
  0,   /* field[0] = user */
};
static const ProtobufCIntRange pg_query__alter_user_mapping_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__alter_user_mapping_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.AlterUserMappingStmt",
  "AlterUserMappingStmt",
  "PgQuery__AlterUserMappingStmt",
  "pg_query",
  sizeof(PgQuery__AlterUserMappingStmt),
  3,
  pg_query__alter_user_mapping_stmt__field_descriptors,
  pg_query__alter_user_mapping_stmt__field_indices_by_name,
  1,  pg_query__alter_user_mapping_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__alter_user_mapping_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__drop_user_mapping_stmt__field_descriptors[3] =
{
  {
    "user",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DropUserMappingStmt, user),
    &pg_query__role_spec__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "servername",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DropUserMappingStmt, servername),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "missing_ok",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DropUserMappingStmt, missing_ok),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__drop_user_mapping_stmt__field_indices_by_name[] = {
  2,   /* field[2] = missing_ok */
  1,   /* field[1] = servername */
  0,   /* field[0] = user */
};
static const ProtobufCIntRange pg_query__drop_user_mapping_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__drop_user_mapping_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.DropUserMappingStmt",
  "DropUserMappingStmt",
  "PgQuery__DropUserMappingStmt",
  "pg_query",
  sizeof(PgQuery__DropUserMappingStmt),
  3,
  pg_query__drop_user_mapping_stmt__field_descriptors,
  pg_query__drop_user_mapping_stmt__field_indices_by_name,
  1,  pg_query__drop_user_mapping_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__drop_user_mapping_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__import_foreign_schema_stmt__field_descriptors[6] =
{
  {
    "server_name",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ImportForeignSchemaStmt, server_name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "remote_schema",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ImportForeignSchemaStmt, remote_schema),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "local_schema",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ImportForeignSchemaStmt, local_schema),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "list_type",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ImportForeignSchemaStmt, list_type),
    &pg_query__import_foreign_schema_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "table_list",
    5,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__ImportForeignSchemaStmt, n_table_list),
    offsetof(PgQuery__ImportForeignSchemaStmt, table_list),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "options",
    6,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__ImportForeignSchemaStmt, n_options),
    offsetof(PgQuery__ImportForeignSchemaStmt, options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__import_foreign_schema_stmt__field_indices_by_name[] = {
  3,   /* field[3] = list_type */
  2,   /* field[2] = local_schema */
  5,   /* field[5] = options */
  1,   /* field[1] = remote_schema */
  0,   /* field[0] = server_name */
  4,   /* field[4] = table_list */
};
static const ProtobufCIntRange pg_query__import_foreign_schema_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 6 }
};
const ProtobufCMessageDescriptor pg_query__import_foreign_schema_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.ImportForeignSchemaStmt",
  "ImportForeignSchemaStmt",
  "PgQuery__ImportForeignSchemaStmt",
  "pg_query",
  sizeof(PgQuery__ImportForeignSchemaStmt),
  6,
  pg_query__import_foreign_schema_stmt__field_descriptors,
  pg_query__import_foreign_schema_stmt__field_indices_by_name,
  1,  pg_query__import_foreign_schema_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__import_foreign_schema_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__create_policy_stmt__field_descriptors[7] =
{
  {
    "policy_name",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreatePolicyStmt, policy_name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "table",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreatePolicyStmt, table),
    &pg_query__range_var__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "cmd_name",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreatePolicyStmt, cmd_name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "permissive",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreatePolicyStmt, permissive),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "roles",
    5,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreatePolicyStmt, n_roles),
    offsetof(PgQuery__CreatePolicyStmt, roles),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "qual",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreatePolicyStmt, qual),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "with_check",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreatePolicyStmt, with_check),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__create_policy_stmt__field_indices_by_name[] = {
  2,   /* field[2] = cmd_name */
  3,   /* field[3] = permissive */
  0,   /* field[0] = policy_name */
  5,   /* field[5] = qual */
  4,   /* field[4] = roles */
  1,   /* field[1] = table */
  6,   /* field[6] = with_check */
};
static const ProtobufCIntRange pg_query__create_policy_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 7 }
};
const ProtobufCMessageDescriptor pg_query__create_policy_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CreatePolicyStmt",
  "CreatePolicyStmt",
  "PgQuery__CreatePolicyStmt",
  "pg_query",
  sizeof(PgQuery__CreatePolicyStmt),
  7,
  pg_query__create_policy_stmt__field_descriptors,
  pg_query__create_policy_stmt__field_indices_by_name,
  1,  pg_query__create_policy_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__create_policy_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__alter_policy_stmt__field_descriptors[5] =
{
  {
    "policy_name",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterPolicyStmt, policy_name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "table",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterPolicyStmt, table),
    &pg_query__range_var__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "roles",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AlterPolicyStmt, n_roles),
    offsetof(PgQuery__AlterPolicyStmt, roles),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "qual",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterPolicyStmt, qual),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "with_check",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterPolicyStmt, with_check),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__alter_policy_stmt__field_indices_by_name[] = {
  0,   /* field[0] = policy_name */
  3,   /* field[3] = qual */
  2,   /* field[2] = roles */
  1,   /* field[1] = table */
  4,   /* field[4] = with_check */
};
static const ProtobufCIntRange pg_query__alter_policy_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor pg_query__alter_policy_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.AlterPolicyStmt",
  "AlterPolicyStmt",
  "PgQuery__AlterPolicyStmt",
  "pg_query",
  sizeof(PgQuery__AlterPolicyStmt),
  5,
  pg_query__alter_policy_stmt__field_descriptors,
  pg_query__alter_policy_stmt__field_indices_by_name,
  1,  pg_query__alter_policy_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__alter_policy_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__create_am_stmt__field_descriptors[3] =
{
  {
    "amname",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateAmStmt, amname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "handler_name",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateAmStmt, n_handler_name),
    offsetof(PgQuery__CreateAmStmt, handler_name),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "amtype",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateAmStmt, amtype),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__create_am_stmt__field_indices_by_name[] = {
  0,   /* field[0] = amname */
  2,   /* field[2] = amtype */
  1,   /* field[1] = handler_name */
};
static const ProtobufCIntRange pg_query__create_am_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__create_am_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CreateAmStmt",
  "CreateAmStmt",
  "PgQuery__CreateAmStmt",
  "pg_query",
  sizeof(PgQuery__CreateAmStmt),
  3,
  pg_query__create_am_stmt__field_descriptors,
  pg_query__create_am_stmt__field_indices_by_name,
  1,  pg_query__create_am_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__create_am_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__create_trig_stmt__field_descriptors[15] =
{
  {
    "replace",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateTrigStmt, replace),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "isconstraint",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateTrigStmt, isconstraint),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "trigname",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateTrigStmt, trigname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "relation",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateTrigStmt, relation),
    &pg_query__range_var__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "funcname",
    5,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateTrigStmt, n_funcname),
    offsetof(PgQuery__CreateTrigStmt, funcname),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "args",
    6,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateTrigStmt, n_args),
    offsetof(PgQuery__CreateTrigStmt, args),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "row",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateTrigStmt, row),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "timing",
    8,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateTrigStmt, timing),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "events",
    9,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateTrigStmt, events),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "columns",
    10,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateTrigStmt, n_columns),
    offsetof(PgQuery__CreateTrigStmt, columns),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "when_clause",
    11,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateTrigStmt, when_clause),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "transition_rels",
    12,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateTrigStmt, n_transition_rels),
    offsetof(PgQuery__CreateTrigStmt, transition_rels),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "deferrable",
    13,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateTrigStmt, deferrable),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "initdeferred",
    14,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateTrigStmt, initdeferred),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "constrrel",
    15,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateTrigStmt, constrrel),
    &pg_query__range_var__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__create_trig_stmt__field_indices_by_name[] = {
  5,   /* field[5] = args */
  9,   /* field[9] = columns */
  14,   /* field[14] = constrrel */
  12,   /* field[12] = deferrable */
  8,   /* field[8] = events */
  4,   /* field[4] = funcname */
  13,   /* field[13] = initdeferred */
  1,   /* field[1] = isconstraint */
  3,   /* field[3] = relation */
  0,   /* field[0] = replace */
  6,   /* field[6] = row */
  7,   /* field[7] = timing */
  11,   /* field[11] = transition_rels */
  2,   /* field[2] = trigname */
  10,   /* field[10] = when_clause */
};
static const ProtobufCIntRange pg_query__create_trig_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 15 }
};
const ProtobufCMessageDescriptor pg_query__create_trig_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CreateTrigStmt",
  "CreateTrigStmt",
  "PgQuery__CreateTrigStmt",
  "pg_query",
  sizeof(PgQuery__CreateTrigStmt),
  15,
  pg_query__create_trig_stmt__field_descriptors,
  pg_query__create_trig_stmt__field_indices_by_name,
  1,  pg_query__create_trig_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__create_trig_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__create_event_trig_stmt__field_descriptors[4] =
{
  {
    "trigname",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateEventTrigStmt, trigname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "eventname",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateEventTrigStmt, eventname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "whenclause",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateEventTrigStmt, n_whenclause),
    offsetof(PgQuery__CreateEventTrigStmt, whenclause),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "funcname",
    4,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateEventTrigStmt, n_funcname),
    offsetof(PgQuery__CreateEventTrigStmt, funcname),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__create_event_trig_stmt__field_indices_by_name[] = {
  1,   /* field[1] = eventname */
  3,   /* field[3] = funcname */
  0,   /* field[0] = trigname */
  2,   /* field[2] = whenclause */
};
static const ProtobufCIntRange pg_query__create_event_trig_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor pg_query__create_event_trig_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CreateEventTrigStmt",
  "CreateEventTrigStmt",
  "PgQuery__CreateEventTrigStmt",
  "pg_query",
  sizeof(PgQuery__CreateEventTrigStmt),
  4,
  pg_query__create_event_trig_stmt__field_descriptors,
  pg_query__create_event_trig_stmt__field_indices_by_name,
  1,  pg_query__create_event_trig_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__create_event_trig_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__alter_event_trig_stmt__field_descriptors[2] =
{
  {
    "trigname",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterEventTrigStmt, trigname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "tgenabled",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterEventTrigStmt, tgenabled),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__alter_event_trig_stmt__field_indices_by_name[] = {
  1,   /* field[1] = tgenabled */
  0,   /* field[0] = trigname */
};
static const ProtobufCIntRange pg_query__alter_event_trig_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor pg_query__alter_event_trig_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.AlterEventTrigStmt",
  "AlterEventTrigStmt",
  "PgQuery__AlterEventTrigStmt",
  "pg_query",
  sizeof(PgQuery__AlterEventTrigStmt),
  2,
  pg_query__alter_event_trig_stmt__field_descriptors,
  pg_query__alter_event_trig_stmt__field_indices_by_name,
  1,  pg_query__alter_event_trig_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__alter_event_trig_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__create_plang_stmt__field_descriptors[6] =
{
  {
    "replace",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreatePLangStmt, replace),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "plname",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreatePLangStmt, plname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "plhandler",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreatePLangStmt, n_plhandler),
    offsetof(PgQuery__CreatePLangStmt, plhandler),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "plinline",
    4,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreatePLangStmt, n_plinline),
    offsetof(PgQuery__CreatePLangStmt, plinline),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "plvalidator",
    5,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreatePLangStmt, n_plvalidator),
    offsetof(PgQuery__CreatePLangStmt, plvalidator),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "pltrusted",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreatePLangStmt, pltrusted),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__create_plang_stmt__field_indices_by_name[] = {
  2,   /* field[2] = plhandler */
  3,   /* field[3] = plinline */
  1,   /* field[1] = plname */
  5,   /* field[5] = pltrusted */
  4,   /* field[4] = plvalidator */
  0,   /* field[0] = replace */
};
static const ProtobufCIntRange pg_query__create_plang_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 6 }
};
const ProtobufCMessageDescriptor pg_query__create_plang_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CreatePLangStmt",
  "CreatePLangStmt",
  "PgQuery__CreatePLangStmt",
  "pg_query",
  sizeof(PgQuery__CreatePLangStmt),
  6,
  pg_query__create_plang_stmt__field_descriptors,
  pg_query__create_plang_stmt__field_indices_by_name,
  1,  pg_query__create_plang_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__create_plang_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__create_role_stmt__field_descriptors[3] =
{
  {
    "stmt_type",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateRoleStmt, stmt_type),
    &pg_query__role_stmt_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "role",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateRoleStmt, role),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "options",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateRoleStmt, n_options),
    offsetof(PgQuery__CreateRoleStmt, options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__create_role_stmt__field_indices_by_name[] = {
  2,   /* field[2] = options */
  1,   /* field[1] = role */
  0,   /* field[0] = stmt_type */
};
static const ProtobufCIntRange pg_query__create_role_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__create_role_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CreateRoleStmt",
  "CreateRoleStmt",
  "PgQuery__CreateRoleStmt",
  "pg_query",
  sizeof(PgQuery__CreateRoleStmt),
  3,
  pg_query__create_role_stmt__field_descriptors,
  pg_query__create_role_stmt__field_indices_by_name,
  1,  pg_query__create_role_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__create_role_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__alter_role_stmt__field_descriptors[3] =
{
  {
    "role",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterRoleStmt, role),
    &pg_query__role_spec__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "options",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AlterRoleStmt, n_options),
    offsetof(PgQuery__AlterRoleStmt, options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "action",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterRoleStmt, action),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__alter_role_stmt__field_indices_by_name[] = {
  2,   /* field[2] = action */
  1,   /* field[1] = options */
  0,   /* field[0] = role */
};
static const ProtobufCIntRange pg_query__alter_role_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__alter_role_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.AlterRoleStmt",
  "AlterRoleStmt",
  "PgQuery__AlterRoleStmt",
  "pg_query",
  sizeof(PgQuery__AlterRoleStmt),
  3,
  pg_query__alter_role_stmt__field_descriptors,
  pg_query__alter_role_stmt__field_indices_by_name,
  1,  pg_query__alter_role_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__alter_role_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__alter_role_set_stmt__field_descriptors[3] =
{
  {
    "role",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterRoleSetStmt, role),
    &pg_query__role_spec__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "database",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterRoleSetStmt, database),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "setstmt",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterRoleSetStmt, setstmt),
    &pg_query__variable_set_stmt__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__alter_role_set_stmt__field_indices_by_name[] = {
  1,   /* field[1] = database */
  0,   /* field[0] = role */
  2,   /* field[2] = setstmt */
};
static const ProtobufCIntRange pg_query__alter_role_set_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__alter_role_set_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.AlterRoleSetStmt",
  "AlterRoleSetStmt",
  "PgQuery__AlterRoleSetStmt",
  "pg_query",
  sizeof(PgQuery__AlterRoleSetStmt),
  3,
  pg_query__alter_role_set_stmt__field_descriptors,
  pg_query__alter_role_set_stmt__field_indices_by_name,
  1,  pg_query__alter_role_set_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__alter_role_set_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__drop_role_stmt__field_descriptors[2] =
{
  {
    "roles",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__DropRoleStmt, n_roles),
    offsetof(PgQuery__DropRoleStmt, roles),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "missing_ok",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DropRoleStmt, missing_ok),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__drop_role_stmt__field_indices_by_name[] = {
  1,   /* field[1] = missing_ok */
  0,   /* field[0] = roles */
};
static const ProtobufCIntRange pg_query__drop_role_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor pg_query__drop_role_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.DropRoleStmt",
  "DropRoleStmt",
  "PgQuery__DropRoleStmt",
  "pg_query",
  sizeof(PgQuery__DropRoleStmt),
  2,
  pg_query__drop_role_stmt__field_descriptors,
  pg_query__drop_role_stmt__field_indices_by_name,
  1,  pg_query__drop_role_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__drop_role_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__create_seq_stmt__field_descriptors[5] =
{
  {
    "sequence",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateSeqStmt, sequence),
    &pg_query__range_var__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "options",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateSeqStmt, n_options),
    offsetof(PgQuery__CreateSeqStmt, options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "owner_id",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateSeqStmt, owner_id),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "for_identity",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateSeqStmt, for_identity),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "if_not_exists",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateSeqStmt, if_not_exists),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__create_seq_stmt__field_indices_by_name[] = {
  3,   /* field[3] = for_identity */
  4,   /* field[4] = if_not_exists */
  1,   /* field[1] = options */
  2,   /* field[2] = owner_id */
  0,   /* field[0] = sequence */
};
static const ProtobufCIntRange pg_query__create_seq_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor pg_query__create_seq_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CreateSeqStmt",
  "CreateSeqStmt",
  "PgQuery__CreateSeqStmt",
  "pg_query",
  sizeof(PgQuery__CreateSeqStmt),
  5,
  pg_query__create_seq_stmt__field_descriptors,
  pg_query__create_seq_stmt__field_indices_by_name,
  1,  pg_query__create_seq_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__create_seq_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__alter_seq_stmt__field_descriptors[4] =
{
  {
    "sequence",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterSeqStmt, sequence),
    &pg_query__range_var__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "options",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AlterSeqStmt, n_options),
    offsetof(PgQuery__AlterSeqStmt, options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "for_identity",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterSeqStmt, for_identity),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "missing_ok",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterSeqStmt, missing_ok),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__alter_seq_stmt__field_indices_by_name[] = {
  2,   /* field[2] = for_identity */
  3,   /* field[3] = missing_ok */
  1,   /* field[1] = options */
  0,   /* field[0] = sequence */
};
static const ProtobufCIntRange pg_query__alter_seq_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor pg_query__alter_seq_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.AlterSeqStmt",
  "AlterSeqStmt",
  "PgQuery__AlterSeqStmt",
  "pg_query",
  sizeof(PgQuery__AlterSeqStmt),
  4,
  pg_query__alter_seq_stmt__field_descriptors,
  pg_query__alter_seq_stmt__field_indices_by_name,
  1,  pg_query__alter_seq_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__alter_seq_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__define_stmt__field_descriptors[7] =
{
  {
    "kind",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DefineStmt, kind),
    &pg_query__object_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "oldstyle",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DefineStmt, oldstyle),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "defnames",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__DefineStmt, n_defnames),
    offsetof(PgQuery__DefineStmt, defnames),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "args",
    4,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__DefineStmt, n_args),
    offsetof(PgQuery__DefineStmt, args),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "definition",
    5,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__DefineStmt, n_definition),
    offsetof(PgQuery__DefineStmt, definition),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "if_not_exists",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DefineStmt, if_not_exists),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "replace",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DefineStmt, replace),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__define_stmt__field_indices_by_name[] = {
  3,   /* field[3] = args */
  4,   /* field[4] = definition */
  2,   /* field[2] = defnames */
  5,   /* field[5] = if_not_exists */
  0,   /* field[0] = kind */
  1,   /* field[1] = oldstyle */
  6,   /* field[6] = replace */
};
static const ProtobufCIntRange pg_query__define_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 7 }
};
const ProtobufCMessageDescriptor pg_query__define_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.DefineStmt",
  "DefineStmt",
  "PgQuery__DefineStmt",
  "pg_query",
  sizeof(PgQuery__DefineStmt),
  7,
  pg_query__define_stmt__field_descriptors,
  pg_query__define_stmt__field_indices_by_name,
  1,  pg_query__define_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__define_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__create_domain_stmt__field_descriptors[4] =
{
  {
    "domainname",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateDomainStmt, n_domainname),
    offsetof(PgQuery__CreateDomainStmt, domainname),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "type_name",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateDomainStmt, type_name),
    &pg_query__type_name__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "coll_clause",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateDomainStmt, coll_clause),
    &pg_query__collate_clause__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "constraints",
    4,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateDomainStmt, n_constraints),
    offsetof(PgQuery__CreateDomainStmt, constraints),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__create_domain_stmt__field_indices_by_name[] = {
  2,   /* field[2] = coll_clause */
  3,   /* field[3] = constraints */
  0,   /* field[0] = domainname */
  1,   /* field[1] = type_name */
};
static const ProtobufCIntRange pg_query__create_domain_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor pg_query__create_domain_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CreateDomainStmt",
  "CreateDomainStmt",
  "PgQuery__CreateDomainStmt",
  "pg_query",
  sizeof(PgQuery__CreateDomainStmt),
  4,
  pg_query__create_domain_stmt__field_descriptors,
  pg_query__create_domain_stmt__field_indices_by_name,
  1,  pg_query__create_domain_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__create_domain_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__create_op_class_stmt__field_descriptors[6] =
{
  {
    "opclassname",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateOpClassStmt, n_opclassname),
    offsetof(PgQuery__CreateOpClassStmt, opclassname),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "opfamilyname",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateOpClassStmt, n_opfamilyname),
    offsetof(PgQuery__CreateOpClassStmt, opfamilyname),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "amname",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateOpClassStmt, amname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "datatype",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateOpClassStmt, datatype),
    &pg_query__type_name__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "items",
    5,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateOpClassStmt, n_items),
    offsetof(PgQuery__CreateOpClassStmt, items),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "is_default",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateOpClassStmt, is_default),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__create_op_class_stmt__field_indices_by_name[] = {
  2,   /* field[2] = amname */
  3,   /* field[3] = datatype */
  5,   /* field[5] = is_default */
  4,   /* field[4] = items */
  0,   /* field[0] = opclassname */
  1,   /* field[1] = opfamilyname */
};
static const ProtobufCIntRange pg_query__create_op_class_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 6 }
};
const ProtobufCMessageDescriptor pg_query__create_op_class_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CreateOpClassStmt",
  "CreateOpClassStmt",
  "PgQuery__CreateOpClassStmt",
  "pg_query",
  sizeof(PgQuery__CreateOpClassStmt),
  6,
  pg_query__create_op_class_stmt__field_descriptors,
  pg_query__create_op_class_stmt__field_indices_by_name,
  1,  pg_query__create_op_class_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__create_op_class_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__create_op_class_item__field_descriptors[6] =
{
  {
    "itemtype",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateOpClassItem, itemtype),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "name",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateOpClassItem, name),
    &pg_query__object_with_args__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "number",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateOpClassItem, number),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "order_family",
    4,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateOpClassItem, n_order_family),
    offsetof(PgQuery__CreateOpClassItem, order_family),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "class_args",
    5,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateOpClassItem, n_class_args),
    offsetof(PgQuery__CreateOpClassItem, class_args),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "storedtype",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateOpClassItem, storedtype),
    &pg_query__type_name__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__create_op_class_item__field_indices_by_name[] = {
  4,   /* field[4] = class_args */
  0,   /* field[0] = itemtype */
  1,   /* field[1] = name */
  2,   /* field[2] = number */
  3,   /* field[3] = order_family */
  5,   /* field[5] = storedtype */
};
static const ProtobufCIntRange pg_query__create_op_class_item__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 6 }
};
const ProtobufCMessageDescriptor pg_query__create_op_class_item__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CreateOpClassItem",
  "CreateOpClassItem",
  "PgQuery__CreateOpClassItem",
  "pg_query",
  sizeof(PgQuery__CreateOpClassItem),
  6,
  pg_query__create_op_class_item__field_descriptors,
  pg_query__create_op_class_item__field_indices_by_name,
  1,  pg_query__create_op_class_item__number_ranges,
  (ProtobufCMessageInit) pg_query__create_op_class_item__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__create_op_family_stmt__field_descriptors[2] =
{
  {
    "opfamilyname",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateOpFamilyStmt, n_opfamilyname),
    offsetof(PgQuery__CreateOpFamilyStmt, opfamilyname),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "amname",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateOpFamilyStmt, amname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__create_op_family_stmt__field_indices_by_name[] = {
  1,   /* field[1] = amname */
  0,   /* field[0] = opfamilyname */
};
static const ProtobufCIntRange pg_query__create_op_family_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor pg_query__create_op_family_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CreateOpFamilyStmt",
  "CreateOpFamilyStmt",
  "PgQuery__CreateOpFamilyStmt",
  "pg_query",
  sizeof(PgQuery__CreateOpFamilyStmt),
  2,
  pg_query__create_op_family_stmt__field_descriptors,
  pg_query__create_op_family_stmt__field_indices_by_name,
  1,  pg_query__create_op_family_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__create_op_family_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__alter_op_family_stmt__field_descriptors[4] =
{
  {
    "opfamilyname",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AlterOpFamilyStmt, n_opfamilyname),
    offsetof(PgQuery__AlterOpFamilyStmt, opfamilyname),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "amname",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterOpFamilyStmt, amname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "is_drop",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterOpFamilyStmt, is_drop),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "items",
    4,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AlterOpFamilyStmt, n_items),
    offsetof(PgQuery__AlterOpFamilyStmt, items),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__alter_op_family_stmt__field_indices_by_name[] = {
  1,   /* field[1] = amname */
  2,   /* field[2] = is_drop */
  3,   /* field[3] = items */
  0,   /* field[0] = opfamilyname */
};
static const ProtobufCIntRange pg_query__alter_op_family_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor pg_query__alter_op_family_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.AlterOpFamilyStmt",
  "AlterOpFamilyStmt",
  "PgQuery__AlterOpFamilyStmt",
  "pg_query",
  sizeof(PgQuery__AlterOpFamilyStmt),
  4,
  pg_query__alter_op_family_stmt__field_descriptors,
  pg_query__alter_op_family_stmt__field_indices_by_name,
  1,  pg_query__alter_op_family_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__alter_op_family_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__drop_stmt__field_descriptors[5] =
{
  {
    "objects",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__DropStmt, n_objects),
    offsetof(PgQuery__DropStmt, objects),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "remove_type",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DropStmt, remove_type),
    &pg_query__object_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "behavior",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DropStmt, behavior),
    &pg_query__drop_behavior__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "missing_ok",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DropStmt, missing_ok),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "concurrent",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DropStmt, concurrent),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__drop_stmt__field_indices_by_name[] = {
  2,   /* field[2] = behavior */
  4,   /* field[4] = concurrent */
  3,   /* field[3] = missing_ok */
  0,   /* field[0] = objects */
  1,   /* field[1] = remove_type */
};
static const ProtobufCIntRange pg_query__drop_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor pg_query__drop_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.DropStmt",
  "DropStmt",
  "PgQuery__DropStmt",
  "pg_query",
  sizeof(PgQuery__DropStmt),
  5,
  pg_query__drop_stmt__field_descriptors,
  pg_query__drop_stmt__field_indices_by_name,
  1,  pg_query__drop_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__drop_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__truncate_stmt__field_descriptors[3] =
{
  {
    "relations",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__TruncateStmt, n_relations),
    offsetof(PgQuery__TruncateStmt, relations),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "restart_seqs",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__TruncateStmt, restart_seqs),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "behavior",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__TruncateStmt, behavior),
    &pg_query__drop_behavior__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__truncate_stmt__field_indices_by_name[] = {
  2,   /* field[2] = behavior */
  0,   /* field[0] = relations */
  1,   /* field[1] = restart_seqs */
};
static const ProtobufCIntRange pg_query__truncate_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__truncate_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.TruncateStmt",
  "TruncateStmt",
  "PgQuery__TruncateStmt",
  "pg_query",
  sizeof(PgQuery__TruncateStmt),
  3,
  pg_query__truncate_stmt__field_descriptors,
  pg_query__truncate_stmt__field_indices_by_name,
  1,  pg_query__truncate_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__truncate_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__comment_stmt__field_descriptors[3] =
{
  {
    "objtype",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CommentStmt, objtype),
    &pg_query__object_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "object",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CommentStmt, object),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "comment",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CommentStmt, comment),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__comment_stmt__field_indices_by_name[] = {
  2,   /* field[2] = comment */
  1,   /* field[1] = object */
  0,   /* field[0] = objtype */
};
static const ProtobufCIntRange pg_query__comment_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__comment_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CommentStmt",
  "CommentStmt",
  "PgQuery__CommentStmt",
  "pg_query",
  sizeof(PgQuery__CommentStmt),
  3,
  pg_query__comment_stmt__field_descriptors,
  pg_query__comment_stmt__field_indices_by_name,
  1,  pg_query__comment_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__comment_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__sec_label_stmt__field_descriptors[4] =
{
  {
    "objtype",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SecLabelStmt, objtype),
    &pg_query__object_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "object",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SecLabelStmt, object),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "provider",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SecLabelStmt, provider),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "label",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__SecLabelStmt, label),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__sec_label_stmt__field_indices_by_name[] = {
  3,   /* field[3] = label */
  1,   /* field[1] = object */
  0,   /* field[0] = objtype */
  2,   /* field[2] = provider */
};
static const ProtobufCIntRange pg_query__sec_label_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor pg_query__sec_label_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.SecLabelStmt",
  "SecLabelStmt",
  "PgQuery__SecLabelStmt",
  "pg_query",
  sizeof(PgQuery__SecLabelStmt),
  4,
  pg_query__sec_label_stmt__field_descriptors,
  pg_query__sec_label_stmt__field_indices_by_name,
  1,  pg_query__sec_label_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__sec_label_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__declare_cursor_stmt__field_descriptors[3] =
{
  {
    "portalname",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DeclareCursorStmt, portalname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "options",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DeclareCursorStmt, options),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "query",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DeclareCursorStmt, query),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__declare_cursor_stmt__field_indices_by_name[] = {
  1,   /* field[1] = options */
  0,   /* field[0] = portalname */
  2,   /* field[2] = query */
};
static const ProtobufCIntRange pg_query__declare_cursor_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__declare_cursor_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.DeclareCursorStmt",
  "DeclareCursorStmt",
  "PgQuery__DeclareCursorStmt",
  "pg_query",
  sizeof(PgQuery__DeclareCursorStmt),
  3,
  pg_query__declare_cursor_stmt__field_descriptors,
  pg_query__declare_cursor_stmt__field_indices_by_name,
  1,  pg_query__declare_cursor_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__declare_cursor_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__close_portal_stmt__field_descriptors[1] =
{
  {
    "portalname",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ClosePortalStmt, portalname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__close_portal_stmt__field_indices_by_name[] = {
  0,   /* field[0] = portalname */
};
static const ProtobufCIntRange pg_query__close_portal_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 1 }
};
const ProtobufCMessageDescriptor pg_query__close_portal_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.ClosePortalStmt",
  "ClosePortalStmt",
  "PgQuery__ClosePortalStmt",
  "pg_query",
  sizeof(PgQuery__ClosePortalStmt),
  1,
  pg_query__close_portal_stmt__field_descriptors,
  pg_query__close_portal_stmt__field_indices_by_name,
  1,  pg_query__close_portal_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__close_portal_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__fetch_stmt__field_descriptors[4] =
{
  {
    "direction",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__FetchStmt, direction),
    &pg_query__fetch_direction__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "how_many",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT64,
    0,   /* quantifier_offset */
    offsetof(PgQuery__FetchStmt, how_many),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "portalname",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__FetchStmt, portalname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "ismove",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__FetchStmt, ismove),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__fetch_stmt__field_indices_by_name[] = {
  0,   /* field[0] = direction */
  1,   /* field[1] = how_many */
  3,   /* field[3] = ismove */
  2,   /* field[2] = portalname */
};
static const ProtobufCIntRange pg_query__fetch_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor pg_query__fetch_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.FetchStmt",
  "FetchStmt",
  "PgQuery__FetchStmt",
  "pg_query",
  sizeof(PgQuery__FetchStmt),
  4,
  pg_query__fetch_stmt__field_descriptors,
  pg_query__fetch_stmt__field_indices_by_name,
  1,  pg_query__fetch_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__fetch_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__index_stmt__field_descriptors[24] =
{
  {
    "idxname",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__IndexStmt, idxname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "relation",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__IndexStmt, relation),
    &pg_query__range_var__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "access_method",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__IndexStmt, access_method),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "table_space",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__IndexStmt, table_space),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "index_params",
    5,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__IndexStmt, n_index_params),
    offsetof(PgQuery__IndexStmt, index_params),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "index_including_params",
    6,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__IndexStmt, n_index_including_params),
    offsetof(PgQuery__IndexStmt, index_including_params),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "options",
    7,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__IndexStmt, n_options),
    offsetof(PgQuery__IndexStmt, options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "where_clause",
    8,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__IndexStmt, where_clause),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "exclude_op_names",
    9,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__IndexStmt, n_exclude_op_names),
    offsetof(PgQuery__IndexStmt, exclude_op_names),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "idxcomment",
    10,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__IndexStmt, idxcomment),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "index_oid",
    11,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__IndexStmt, index_oid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "old_number",
    12,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__IndexStmt, old_number),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "old_create_subid",
    13,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__IndexStmt, old_create_subid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "old_first_relfilelocator_subid",
    14,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__IndexStmt, old_first_relfilelocator_subid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "unique",
    15,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__IndexStmt, unique),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "nulls_not_distinct",
    16,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__IndexStmt, nulls_not_distinct),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "primary",
    17,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__IndexStmt, primary),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "isconstraint",
    18,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__IndexStmt, isconstraint),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "deferrable",
    19,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__IndexStmt, deferrable),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "initdeferred",
    20,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__IndexStmt, initdeferred),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "transformed",
    21,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__IndexStmt, transformed),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "concurrent",
    22,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__IndexStmt, concurrent),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "if_not_exists",
    23,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__IndexStmt, if_not_exists),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "reset_default_tblspc",
    24,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__IndexStmt, reset_default_tblspc),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__index_stmt__field_indices_by_name[] = {
  2,   /* field[2] = access_method */
  21,   /* field[21] = concurrent */
  18,   /* field[18] = deferrable */
  8,   /* field[8] = exclude_op_names */
  9,   /* field[9] = idxcomment */
  0,   /* field[0] = idxname */
  22,   /* field[22] = if_not_exists */
  5,   /* field[5] = index_including_params */
  10,   /* field[10] = index_oid */
  4,   /* field[4] = index_params */
  19,   /* field[19] = initdeferred */
  17,   /* field[17] = isconstraint */
  15,   /* field[15] = nulls_not_distinct */
  12,   /* field[12] = old_create_subid */
  13,   /* field[13] = old_first_relfilelocator_subid */
  11,   /* field[11] = old_number */
  6,   /* field[6] = options */
  16,   /* field[16] = primary */
  1,   /* field[1] = relation */
  23,   /* field[23] = reset_default_tblspc */
  3,   /* field[3] = table_space */
  20,   /* field[20] = transformed */
  14,   /* field[14] = unique */
  7,   /* field[7] = where_clause */
};
static const ProtobufCIntRange pg_query__index_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 24 }
};
const ProtobufCMessageDescriptor pg_query__index_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.IndexStmt",
  "IndexStmt",
  "PgQuery__IndexStmt",
  "pg_query",
  sizeof(PgQuery__IndexStmt),
  24,
  pg_query__index_stmt__field_descriptors,
  pg_query__index_stmt__field_indices_by_name,
  1,  pg_query__index_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__index_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__create_stats_stmt__field_descriptors[7] =
{
  {
    "defnames",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateStatsStmt, n_defnames),
    offsetof(PgQuery__CreateStatsStmt, defnames),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "stat_types",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateStatsStmt, n_stat_types),
    offsetof(PgQuery__CreateStatsStmt, stat_types),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "exprs",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateStatsStmt, n_exprs),
    offsetof(PgQuery__CreateStatsStmt, exprs),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "relations",
    4,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateStatsStmt, n_relations),
    offsetof(PgQuery__CreateStatsStmt, relations),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "stxcomment",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateStatsStmt, stxcomment),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "transformed",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateStatsStmt, transformed),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "if_not_exists",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateStatsStmt, if_not_exists),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__create_stats_stmt__field_indices_by_name[] = {
  0,   /* field[0] = defnames */
  2,   /* field[2] = exprs */
  6,   /* field[6] = if_not_exists */
  3,   /* field[3] = relations */
  1,   /* field[1] = stat_types */
  4,   /* field[4] = stxcomment */
  5,   /* field[5] = transformed */
};
static const ProtobufCIntRange pg_query__create_stats_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 7 }
};
const ProtobufCMessageDescriptor pg_query__create_stats_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CreateStatsStmt",
  "CreateStatsStmt",
  "PgQuery__CreateStatsStmt",
  "pg_query",
  sizeof(PgQuery__CreateStatsStmt),
  7,
  pg_query__create_stats_stmt__field_descriptors,
  pg_query__create_stats_stmt__field_indices_by_name,
  1,  pg_query__create_stats_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__create_stats_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__stats_elem__field_descriptors[2] =
{
  {
    "name",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__StatsElem, name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "expr",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__StatsElem, expr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__stats_elem__field_indices_by_name[] = {
  1,   /* field[1] = expr */
  0,   /* field[0] = name */
};
static const ProtobufCIntRange pg_query__stats_elem__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor pg_query__stats_elem__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.StatsElem",
  "StatsElem",
  "PgQuery__StatsElem",
  "pg_query",
  sizeof(PgQuery__StatsElem),
  2,
  pg_query__stats_elem__field_descriptors,
  pg_query__stats_elem__field_indices_by_name,
  1,  pg_query__stats_elem__number_ranges,
  (ProtobufCMessageInit) pg_query__stats_elem__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__alter_stats_stmt__field_descriptors[3] =
{
  {
    "defnames",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AlterStatsStmt, n_defnames),
    offsetof(PgQuery__AlterStatsStmt, defnames),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "stxstattarget",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterStatsStmt, stxstattarget),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "missing_ok",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterStatsStmt, missing_ok),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__alter_stats_stmt__field_indices_by_name[] = {
  0,   /* field[0] = defnames */
  2,   /* field[2] = missing_ok */
  1,   /* field[1] = stxstattarget */
};
static const ProtobufCIntRange pg_query__alter_stats_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__alter_stats_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.AlterStatsStmt",
  "AlterStatsStmt",
  "PgQuery__AlterStatsStmt",
  "pg_query",
  sizeof(PgQuery__AlterStatsStmt),
  3,
  pg_query__alter_stats_stmt__field_descriptors,
  pg_query__alter_stats_stmt__field_indices_by_name,
  1,  pg_query__alter_stats_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__alter_stats_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__create_function_stmt__field_descriptors[7] =
{
  {
    "is_procedure",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateFunctionStmt, is_procedure),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "replace",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateFunctionStmt, replace),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "funcname",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateFunctionStmt, n_funcname),
    offsetof(PgQuery__CreateFunctionStmt, funcname),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "parameters",
    4,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateFunctionStmt, n_parameters),
    offsetof(PgQuery__CreateFunctionStmt, parameters),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "return_type",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateFunctionStmt, return_type),
    &pg_query__type_name__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "options",
    6,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateFunctionStmt, n_options),
    offsetof(PgQuery__CreateFunctionStmt, options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "sql_body",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateFunctionStmt, sql_body),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__create_function_stmt__field_indices_by_name[] = {
  2,   /* field[2] = funcname */
  0,   /* field[0] = is_procedure */
  5,   /* field[5] = options */
  3,   /* field[3] = parameters */
  1,   /* field[1] = replace */
  4,   /* field[4] = return_type */
  6,   /* field[6] = sql_body */
};
static const ProtobufCIntRange pg_query__create_function_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 7 }
};
const ProtobufCMessageDescriptor pg_query__create_function_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CreateFunctionStmt",
  "CreateFunctionStmt",
  "PgQuery__CreateFunctionStmt",
  "pg_query",
  sizeof(PgQuery__CreateFunctionStmt),
  7,
  pg_query__create_function_stmt__field_descriptors,
  pg_query__create_function_stmt__field_indices_by_name,
  1,  pg_query__create_function_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__create_function_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__function_parameter__field_descriptors[4] =
{
  {
    "name",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__FunctionParameter, name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "arg_type",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__FunctionParameter, arg_type),
    &pg_query__type_name__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "mode",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__FunctionParameter, mode),
    &pg_query__function_parameter_mode__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "defexpr",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__FunctionParameter, defexpr),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__function_parameter__field_indices_by_name[] = {
  1,   /* field[1] = arg_type */
  3,   /* field[3] = defexpr */
  2,   /* field[2] = mode */
  0,   /* field[0] = name */
};
static const ProtobufCIntRange pg_query__function_parameter__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor pg_query__function_parameter__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.FunctionParameter",
  "FunctionParameter",
  "PgQuery__FunctionParameter",
  "pg_query",
  sizeof(PgQuery__FunctionParameter),
  4,
  pg_query__function_parameter__field_descriptors,
  pg_query__function_parameter__field_indices_by_name,
  1,  pg_query__function_parameter__number_ranges,
  (ProtobufCMessageInit) pg_query__function_parameter__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__alter_function_stmt__field_descriptors[3] =
{
  {
    "objtype",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterFunctionStmt, objtype),
    &pg_query__object_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "func",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterFunctionStmt, func),
    &pg_query__object_with_args__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "actions",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AlterFunctionStmt, n_actions),
    offsetof(PgQuery__AlterFunctionStmt, actions),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__alter_function_stmt__field_indices_by_name[] = {
  2,   /* field[2] = actions */
  1,   /* field[1] = func */
  0,   /* field[0] = objtype */
};
static const ProtobufCIntRange pg_query__alter_function_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__alter_function_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.AlterFunctionStmt",
  "AlterFunctionStmt",
  "PgQuery__AlterFunctionStmt",
  "pg_query",
  sizeof(PgQuery__AlterFunctionStmt),
  3,
  pg_query__alter_function_stmt__field_descriptors,
  pg_query__alter_function_stmt__field_indices_by_name,
  1,  pg_query__alter_function_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__alter_function_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__do_stmt__field_descriptors[1] =
{
  {
    "args",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__DoStmt, n_args),
    offsetof(PgQuery__DoStmt, args),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__do_stmt__field_indices_by_name[] = {
  0,   /* field[0] = args */
};
static const ProtobufCIntRange pg_query__do_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 1 }
};
const ProtobufCMessageDescriptor pg_query__do_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.DoStmt",
  "DoStmt",
  "PgQuery__DoStmt",
  "pg_query",
  sizeof(PgQuery__DoStmt),
  1,
  pg_query__do_stmt__field_descriptors,
  pg_query__do_stmt__field_indices_by_name,
  1,  pg_query__do_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__do_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__inline_code_block__field_descriptors[4] =
{
  {
    "source_text",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__InlineCodeBlock, source_text),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "lang_oid",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__InlineCodeBlock, lang_oid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "lang_is_trusted",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__InlineCodeBlock, lang_is_trusted),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "atomic",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__InlineCodeBlock, atomic),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__inline_code_block__field_indices_by_name[] = {
  3,   /* field[3] = atomic */
  2,   /* field[2] = lang_is_trusted */
  1,   /* field[1] = lang_oid */
  0,   /* field[0] = source_text */
};
static const ProtobufCIntRange pg_query__inline_code_block__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor pg_query__inline_code_block__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.InlineCodeBlock",
  "InlineCodeBlock",
  "PgQuery__InlineCodeBlock",
  "pg_query",
  sizeof(PgQuery__InlineCodeBlock),
  4,
  pg_query__inline_code_block__field_descriptors,
  pg_query__inline_code_block__field_indices_by_name,
  1,  pg_query__inline_code_block__number_ranges,
  (ProtobufCMessageInit) pg_query__inline_code_block__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__call_stmt__field_descriptors[3] =
{
  {
    "funccall",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CallStmt, funccall),
    &pg_query__func_call__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "funcexpr",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CallStmt, funcexpr),
    &pg_query__func_expr__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "outargs",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CallStmt, n_outargs),
    offsetof(PgQuery__CallStmt, outargs),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__call_stmt__field_indices_by_name[] = {
  0,   /* field[0] = funccall */
  1,   /* field[1] = funcexpr */
  2,   /* field[2] = outargs */
};
static const ProtobufCIntRange pg_query__call_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__call_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CallStmt",
  "CallStmt",
  "PgQuery__CallStmt",
  "pg_query",
  sizeof(PgQuery__CallStmt),
  3,
  pg_query__call_stmt__field_descriptors,
  pg_query__call_stmt__field_indices_by_name,
  1,  pg_query__call_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__call_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__call_context__field_descriptors[1] =
{
  {
    "atomic",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CallContext, atomic),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__call_context__field_indices_by_name[] = {
  0,   /* field[0] = atomic */
};
static const ProtobufCIntRange pg_query__call_context__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 1 }
};
const ProtobufCMessageDescriptor pg_query__call_context__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CallContext",
  "CallContext",
  "PgQuery__CallContext",
  "pg_query",
  sizeof(PgQuery__CallContext),
  1,
  pg_query__call_context__field_descriptors,
  pg_query__call_context__field_indices_by_name,
  1,  pg_query__call_context__number_ranges,
  (ProtobufCMessageInit) pg_query__call_context__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__rename_stmt__field_descriptors[8] =
{
  {
    "rename_type",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RenameStmt, rename_type),
    &pg_query__object_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "relation_type",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RenameStmt, relation_type),
    &pg_query__object_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "relation",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RenameStmt, relation),
    &pg_query__range_var__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "object",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RenameStmt, object),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "subname",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RenameStmt, subname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "newname",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RenameStmt, newname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "behavior",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RenameStmt, behavior),
    &pg_query__drop_behavior__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "missing_ok",
    8,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RenameStmt, missing_ok),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__rename_stmt__field_indices_by_name[] = {
  6,   /* field[6] = behavior */
  7,   /* field[7] = missing_ok */
  5,   /* field[5] = newname */
  3,   /* field[3] = object */
  2,   /* field[2] = relation */
  1,   /* field[1] = relation_type */
  0,   /* field[0] = rename_type */
  4,   /* field[4] = subname */
};
static const ProtobufCIntRange pg_query__rename_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 8 }
};
const ProtobufCMessageDescriptor pg_query__rename_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.RenameStmt",
  "RenameStmt",
  "PgQuery__RenameStmt",
  "pg_query",
  sizeof(PgQuery__RenameStmt),
  8,
  pg_query__rename_stmt__field_descriptors,
  pg_query__rename_stmt__field_indices_by_name,
  1,  pg_query__rename_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__rename_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__alter_object_depends_stmt__field_descriptors[5] =
{
  {
    "object_type",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterObjectDependsStmt, object_type),
    &pg_query__object_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "relation",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterObjectDependsStmt, relation),
    &pg_query__range_var__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "object",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterObjectDependsStmt, object),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "extname",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterObjectDependsStmt, extname),
    &pg_query__string__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "remove",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterObjectDependsStmt, remove),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__alter_object_depends_stmt__field_indices_by_name[] = {
  3,   /* field[3] = extname */
  2,   /* field[2] = object */
  0,   /* field[0] = object_type */
  1,   /* field[1] = relation */
  4,   /* field[4] = remove */
};
static const ProtobufCIntRange pg_query__alter_object_depends_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor pg_query__alter_object_depends_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.AlterObjectDependsStmt",
  "AlterObjectDependsStmt",
  "PgQuery__AlterObjectDependsStmt",
  "pg_query",
  sizeof(PgQuery__AlterObjectDependsStmt),
  5,
  pg_query__alter_object_depends_stmt__field_descriptors,
  pg_query__alter_object_depends_stmt__field_indices_by_name,
  1,  pg_query__alter_object_depends_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__alter_object_depends_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__alter_object_schema_stmt__field_descriptors[5] =
{
  {
    "object_type",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterObjectSchemaStmt, object_type),
    &pg_query__object_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "relation",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterObjectSchemaStmt, relation),
    &pg_query__range_var__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "object",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterObjectSchemaStmt, object),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "newschema",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterObjectSchemaStmt, newschema),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "missing_ok",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterObjectSchemaStmt, missing_ok),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__alter_object_schema_stmt__field_indices_by_name[] = {
  4,   /* field[4] = missing_ok */
  3,   /* field[3] = newschema */
  2,   /* field[2] = object */
  0,   /* field[0] = object_type */
  1,   /* field[1] = relation */
};
static const ProtobufCIntRange pg_query__alter_object_schema_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor pg_query__alter_object_schema_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.AlterObjectSchemaStmt",
  "AlterObjectSchemaStmt",
  "PgQuery__AlterObjectSchemaStmt",
  "pg_query",
  sizeof(PgQuery__AlterObjectSchemaStmt),
  5,
  pg_query__alter_object_schema_stmt__field_descriptors,
  pg_query__alter_object_schema_stmt__field_indices_by_name,
  1,  pg_query__alter_object_schema_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__alter_object_schema_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__alter_owner_stmt__field_descriptors[4] =
{
  {
    "object_type",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterOwnerStmt, object_type),
    &pg_query__object_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "relation",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterOwnerStmt, relation),
    &pg_query__range_var__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "object",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterOwnerStmt, object),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "newowner",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterOwnerStmt, newowner),
    &pg_query__role_spec__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__alter_owner_stmt__field_indices_by_name[] = {
  3,   /* field[3] = newowner */
  2,   /* field[2] = object */
  0,   /* field[0] = object_type */
  1,   /* field[1] = relation */
};
static const ProtobufCIntRange pg_query__alter_owner_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor pg_query__alter_owner_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.AlterOwnerStmt",
  "AlterOwnerStmt",
  "PgQuery__AlterOwnerStmt",
  "pg_query",
  sizeof(PgQuery__AlterOwnerStmt),
  4,
  pg_query__alter_owner_stmt__field_descriptors,
  pg_query__alter_owner_stmt__field_indices_by_name,
  1,  pg_query__alter_owner_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__alter_owner_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__alter_operator_stmt__field_descriptors[2] =
{
  {
    "opername",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterOperatorStmt, opername),
    &pg_query__object_with_args__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "options",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AlterOperatorStmt, n_options),
    offsetof(PgQuery__AlterOperatorStmt, options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__alter_operator_stmt__field_indices_by_name[] = {
  0,   /* field[0] = opername */
  1,   /* field[1] = options */
};
static const ProtobufCIntRange pg_query__alter_operator_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor pg_query__alter_operator_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.AlterOperatorStmt",
  "AlterOperatorStmt",
  "PgQuery__AlterOperatorStmt",
  "pg_query",
  sizeof(PgQuery__AlterOperatorStmt),
  2,
  pg_query__alter_operator_stmt__field_descriptors,
  pg_query__alter_operator_stmt__field_indices_by_name,
  1,  pg_query__alter_operator_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__alter_operator_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__alter_type_stmt__field_descriptors[2] =
{
  {
    "type_name",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AlterTypeStmt, n_type_name),
    offsetof(PgQuery__AlterTypeStmt, type_name),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "options",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AlterTypeStmt, n_options),
    offsetof(PgQuery__AlterTypeStmt, options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__alter_type_stmt__field_indices_by_name[] = {
  1,   /* field[1] = options */
  0,   /* field[0] = type_name */
};
static const ProtobufCIntRange pg_query__alter_type_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor pg_query__alter_type_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.AlterTypeStmt",
  "AlterTypeStmt",
  "PgQuery__AlterTypeStmt",
  "pg_query",
  sizeof(PgQuery__AlterTypeStmt),
  2,
  pg_query__alter_type_stmt__field_descriptors,
  pg_query__alter_type_stmt__field_indices_by_name,
  1,  pg_query__alter_type_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__alter_type_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__rule_stmt__field_descriptors[7] =
{
  {
    "relation",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RuleStmt, relation),
    &pg_query__range_var__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "rulename",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RuleStmt, rulename),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "where_clause",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RuleStmt, where_clause),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "event",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RuleStmt, event),
    &pg_query__cmd_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "instead",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RuleStmt, instead),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "actions",
    6,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__RuleStmt, n_actions),
    offsetof(PgQuery__RuleStmt, actions),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "replace",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RuleStmt, replace),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__rule_stmt__field_indices_by_name[] = {
  5,   /* field[5] = actions */
  3,   /* field[3] = event */
  4,   /* field[4] = instead */
  0,   /* field[0] = relation */
  6,   /* field[6] = replace */
  1,   /* field[1] = rulename */
  2,   /* field[2] = where_clause */
};
static const ProtobufCIntRange pg_query__rule_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 7 }
};
const ProtobufCMessageDescriptor pg_query__rule_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.RuleStmt",
  "RuleStmt",
  "PgQuery__RuleStmt",
  "pg_query",
  sizeof(PgQuery__RuleStmt),
  7,
  pg_query__rule_stmt__field_descriptors,
  pg_query__rule_stmt__field_indices_by_name,
  1,  pg_query__rule_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__rule_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__notify_stmt__field_descriptors[2] =
{
  {
    "conditionname",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__NotifyStmt, conditionname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "payload",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__NotifyStmt, payload),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__notify_stmt__field_indices_by_name[] = {
  0,   /* field[0] = conditionname */
  1,   /* field[1] = payload */
};
static const ProtobufCIntRange pg_query__notify_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor pg_query__notify_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.NotifyStmt",
  "NotifyStmt",
  "PgQuery__NotifyStmt",
  "pg_query",
  sizeof(PgQuery__NotifyStmt),
  2,
  pg_query__notify_stmt__field_descriptors,
  pg_query__notify_stmt__field_indices_by_name,
  1,  pg_query__notify_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__notify_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__listen_stmt__field_descriptors[1] =
{
  {
    "conditionname",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ListenStmt, conditionname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__listen_stmt__field_indices_by_name[] = {
  0,   /* field[0] = conditionname */
};
static const ProtobufCIntRange pg_query__listen_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 1 }
};
const ProtobufCMessageDescriptor pg_query__listen_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.ListenStmt",
  "ListenStmt",
  "PgQuery__ListenStmt",
  "pg_query",
  sizeof(PgQuery__ListenStmt),
  1,
  pg_query__listen_stmt__field_descriptors,
  pg_query__listen_stmt__field_indices_by_name,
  1,  pg_query__listen_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__listen_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__unlisten_stmt__field_descriptors[1] =
{
  {
    "conditionname",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__UnlistenStmt, conditionname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__unlisten_stmt__field_indices_by_name[] = {
  0,   /* field[0] = conditionname */
};
static const ProtobufCIntRange pg_query__unlisten_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 1 }
};
const ProtobufCMessageDescriptor pg_query__unlisten_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.UnlistenStmt",
  "UnlistenStmt",
  "PgQuery__UnlistenStmt",
  "pg_query",
  sizeof(PgQuery__UnlistenStmt),
  1,
  pg_query__unlisten_stmt__field_descriptors,
  pg_query__unlisten_stmt__field_indices_by_name,
  1,  pg_query__unlisten_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__unlisten_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__transaction_stmt__field_descriptors[6] =
{
  {
    "kind",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__TransactionStmt, kind),
    &pg_query__transaction_stmt_kind__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "options",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__TransactionStmt, n_options),
    offsetof(PgQuery__TransactionStmt, options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "savepoint_name",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__TransactionStmt, savepoint_name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "gid",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__TransactionStmt, gid),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "chain",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__TransactionStmt, chain),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__TransactionStmt, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__transaction_stmt__field_indices_by_name[] = {
  4,   /* field[4] = chain */
  3,   /* field[3] = gid */
  0,   /* field[0] = kind */
  5,   /* field[5] = location */
  1,   /* field[1] = options */
  2,   /* field[2] = savepoint_name */
};
static const ProtobufCIntRange pg_query__transaction_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 6 }
};
const ProtobufCMessageDescriptor pg_query__transaction_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.TransactionStmt",
  "TransactionStmt",
  "PgQuery__TransactionStmt",
  "pg_query",
  sizeof(PgQuery__TransactionStmt),
  6,
  pg_query__transaction_stmt__field_descriptors,
  pg_query__transaction_stmt__field_indices_by_name,
  1,  pg_query__transaction_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__transaction_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__composite_type_stmt__field_descriptors[2] =
{
  {
    "typevar",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CompositeTypeStmt, typevar),
    &pg_query__range_var__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "coldeflist",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CompositeTypeStmt, n_coldeflist),
    offsetof(PgQuery__CompositeTypeStmt, coldeflist),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__composite_type_stmt__field_indices_by_name[] = {
  1,   /* field[1] = coldeflist */
  0,   /* field[0] = typevar */
};
static const ProtobufCIntRange pg_query__composite_type_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor pg_query__composite_type_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CompositeTypeStmt",
  "CompositeTypeStmt",
  "PgQuery__CompositeTypeStmt",
  "pg_query",
  sizeof(PgQuery__CompositeTypeStmt),
  2,
  pg_query__composite_type_stmt__field_descriptors,
  pg_query__composite_type_stmt__field_indices_by_name,
  1,  pg_query__composite_type_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__composite_type_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__create_enum_stmt__field_descriptors[2] =
{
  {
    "type_name",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateEnumStmt, n_type_name),
    offsetof(PgQuery__CreateEnumStmt, type_name),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "vals",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateEnumStmt, n_vals),
    offsetof(PgQuery__CreateEnumStmt, vals),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__create_enum_stmt__field_indices_by_name[] = {
  0,   /* field[0] = type_name */
  1,   /* field[1] = vals */
};
static const ProtobufCIntRange pg_query__create_enum_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor pg_query__create_enum_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CreateEnumStmt",
  "CreateEnumStmt",
  "PgQuery__CreateEnumStmt",
  "pg_query",
  sizeof(PgQuery__CreateEnumStmt),
  2,
  pg_query__create_enum_stmt__field_descriptors,
  pg_query__create_enum_stmt__field_indices_by_name,
  1,  pg_query__create_enum_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__create_enum_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__create_range_stmt__field_descriptors[2] =
{
  {
    "type_name",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateRangeStmt, n_type_name),
    offsetof(PgQuery__CreateRangeStmt, type_name),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "params",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateRangeStmt, n_params),
    offsetof(PgQuery__CreateRangeStmt, params),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__create_range_stmt__field_indices_by_name[] = {
  1,   /* field[1] = params */
  0,   /* field[0] = type_name */
};
static const ProtobufCIntRange pg_query__create_range_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor pg_query__create_range_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CreateRangeStmt",
  "CreateRangeStmt",
  "PgQuery__CreateRangeStmt",
  "pg_query",
  sizeof(PgQuery__CreateRangeStmt),
  2,
  pg_query__create_range_stmt__field_descriptors,
  pg_query__create_range_stmt__field_indices_by_name,
  1,  pg_query__create_range_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__create_range_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__alter_enum_stmt__field_descriptors[6] =
{
  {
    "type_name",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AlterEnumStmt, n_type_name),
    offsetof(PgQuery__AlterEnumStmt, type_name),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "old_val",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterEnumStmt, old_val),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "new_val",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterEnumStmt, new_val),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "new_val_neighbor",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterEnumStmt, new_val_neighbor),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "new_val_is_after",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterEnumStmt, new_val_is_after),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "skip_if_new_val_exists",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterEnumStmt, skip_if_new_val_exists),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__alter_enum_stmt__field_indices_by_name[] = {
  2,   /* field[2] = new_val */
  4,   /* field[4] = new_val_is_after */
  3,   /* field[3] = new_val_neighbor */
  1,   /* field[1] = old_val */
  5,   /* field[5] = skip_if_new_val_exists */
  0,   /* field[0] = type_name */
};
static const ProtobufCIntRange pg_query__alter_enum_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 6 }
};
const ProtobufCMessageDescriptor pg_query__alter_enum_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.AlterEnumStmt",
  "AlterEnumStmt",
  "PgQuery__AlterEnumStmt",
  "pg_query",
  sizeof(PgQuery__AlterEnumStmt),
  6,
  pg_query__alter_enum_stmt__field_descriptors,
  pg_query__alter_enum_stmt__field_indices_by_name,
  1,  pg_query__alter_enum_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__alter_enum_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__view_stmt__field_descriptors[6] =
{
  {
    "view",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ViewStmt, view),
    &pg_query__range_var__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "aliases",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__ViewStmt, n_aliases),
    offsetof(PgQuery__ViewStmt, aliases),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "query",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ViewStmt, query),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "replace",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ViewStmt, replace),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "options",
    5,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__ViewStmt, n_options),
    offsetof(PgQuery__ViewStmt, options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "with_check_option",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ViewStmt, with_check_option),
    &pg_query__view_check_option__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__view_stmt__field_indices_by_name[] = {
  1,   /* field[1] = aliases */
  4,   /* field[4] = options */
  2,   /* field[2] = query */
  3,   /* field[3] = replace */
  0,   /* field[0] = view */
  5,   /* field[5] = with_check_option */
};
static const ProtobufCIntRange pg_query__view_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 6 }
};
const ProtobufCMessageDescriptor pg_query__view_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.ViewStmt",
  "ViewStmt",
  "PgQuery__ViewStmt",
  "pg_query",
  sizeof(PgQuery__ViewStmt),
  6,
  pg_query__view_stmt__field_descriptors,
  pg_query__view_stmt__field_indices_by_name,
  1,  pg_query__view_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__view_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__load_stmt__field_descriptors[1] =
{
  {
    "filename",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__LoadStmt, filename),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__load_stmt__field_indices_by_name[] = {
  0,   /* field[0] = filename */
};
static const ProtobufCIntRange pg_query__load_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 1 }
};
const ProtobufCMessageDescriptor pg_query__load_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.LoadStmt",
  "LoadStmt",
  "PgQuery__LoadStmt",
  "pg_query",
  sizeof(PgQuery__LoadStmt),
  1,
  pg_query__load_stmt__field_descriptors,
  pg_query__load_stmt__field_indices_by_name,
  1,  pg_query__load_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__load_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__createdb_stmt__field_descriptors[2] =
{
  {
    "dbname",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreatedbStmt, dbname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "options",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreatedbStmt, n_options),
    offsetof(PgQuery__CreatedbStmt, options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__createdb_stmt__field_indices_by_name[] = {
  0,   /* field[0] = dbname */
  1,   /* field[1] = options */
};
static const ProtobufCIntRange pg_query__createdb_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor pg_query__createdb_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CreatedbStmt",
  "CreatedbStmt",
  "PgQuery__CreatedbStmt",
  "pg_query",
  sizeof(PgQuery__CreatedbStmt),
  2,
  pg_query__createdb_stmt__field_descriptors,
  pg_query__createdb_stmt__field_indices_by_name,
  1,  pg_query__createdb_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__createdb_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__alter_database_stmt__field_descriptors[2] =
{
  {
    "dbname",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterDatabaseStmt, dbname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "options",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AlterDatabaseStmt, n_options),
    offsetof(PgQuery__AlterDatabaseStmt, options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__alter_database_stmt__field_indices_by_name[] = {
  0,   /* field[0] = dbname */
  1,   /* field[1] = options */
};
static const ProtobufCIntRange pg_query__alter_database_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor pg_query__alter_database_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.AlterDatabaseStmt",
  "AlterDatabaseStmt",
  "PgQuery__AlterDatabaseStmt",
  "pg_query",
  sizeof(PgQuery__AlterDatabaseStmt),
  2,
  pg_query__alter_database_stmt__field_descriptors,
  pg_query__alter_database_stmt__field_indices_by_name,
  1,  pg_query__alter_database_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__alter_database_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__alter_database_refresh_coll_stmt__field_descriptors[1] =
{
  {
    "dbname",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterDatabaseRefreshCollStmt, dbname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__alter_database_refresh_coll_stmt__field_indices_by_name[] = {
  0,   /* field[0] = dbname */
};
static const ProtobufCIntRange pg_query__alter_database_refresh_coll_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 1 }
};
const ProtobufCMessageDescriptor pg_query__alter_database_refresh_coll_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.AlterDatabaseRefreshCollStmt",
  "AlterDatabaseRefreshCollStmt",
  "PgQuery__AlterDatabaseRefreshCollStmt",
  "pg_query",
  sizeof(PgQuery__AlterDatabaseRefreshCollStmt),
  1,
  pg_query__alter_database_refresh_coll_stmt__field_descriptors,
  pg_query__alter_database_refresh_coll_stmt__field_indices_by_name,
  1,  pg_query__alter_database_refresh_coll_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__alter_database_refresh_coll_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__alter_database_set_stmt__field_descriptors[2] =
{
  {
    "dbname",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterDatabaseSetStmt, dbname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "setstmt",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterDatabaseSetStmt, setstmt),
    &pg_query__variable_set_stmt__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__alter_database_set_stmt__field_indices_by_name[] = {
  0,   /* field[0] = dbname */
  1,   /* field[1] = setstmt */
};
static const ProtobufCIntRange pg_query__alter_database_set_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor pg_query__alter_database_set_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.AlterDatabaseSetStmt",
  "AlterDatabaseSetStmt",
  "PgQuery__AlterDatabaseSetStmt",
  "pg_query",
  sizeof(PgQuery__AlterDatabaseSetStmt),
  2,
  pg_query__alter_database_set_stmt__field_descriptors,
  pg_query__alter_database_set_stmt__field_indices_by_name,
  1,  pg_query__alter_database_set_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__alter_database_set_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__dropdb_stmt__field_descriptors[3] =
{
  {
    "dbname",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DropdbStmt, dbname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "missing_ok",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DropdbStmt, missing_ok),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "options",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__DropdbStmt, n_options),
    offsetof(PgQuery__DropdbStmt, options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__dropdb_stmt__field_indices_by_name[] = {
  0,   /* field[0] = dbname */
  1,   /* field[1] = missing_ok */
  2,   /* field[2] = options */
};
static const ProtobufCIntRange pg_query__dropdb_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__dropdb_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.DropdbStmt",
  "DropdbStmt",
  "PgQuery__DropdbStmt",
  "pg_query",
  sizeof(PgQuery__DropdbStmt),
  3,
  pg_query__dropdb_stmt__field_descriptors,
  pg_query__dropdb_stmt__field_indices_by_name,
  1,  pg_query__dropdb_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__dropdb_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__alter_system_stmt__field_descriptors[1] =
{
  {
    "setstmt",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterSystemStmt, setstmt),
    &pg_query__variable_set_stmt__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__alter_system_stmt__field_indices_by_name[] = {
  0,   /* field[0] = setstmt */
};
static const ProtobufCIntRange pg_query__alter_system_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 1 }
};
const ProtobufCMessageDescriptor pg_query__alter_system_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.AlterSystemStmt",
  "AlterSystemStmt",
  "PgQuery__AlterSystemStmt",
  "pg_query",
  sizeof(PgQuery__AlterSystemStmt),
  1,
  pg_query__alter_system_stmt__field_descriptors,
  pg_query__alter_system_stmt__field_indices_by_name,
  1,  pg_query__alter_system_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__alter_system_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__cluster_stmt__field_descriptors[3] =
{
  {
    "relation",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ClusterStmt, relation),
    &pg_query__range_var__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "indexname",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ClusterStmt, indexname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "params",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__ClusterStmt, n_params),
    offsetof(PgQuery__ClusterStmt, params),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__cluster_stmt__field_indices_by_name[] = {
  1,   /* field[1] = indexname */
  2,   /* field[2] = params */
  0,   /* field[0] = relation */
};
static const ProtobufCIntRange pg_query__cluster_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__cluster_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.ClusterStmt",
  "ClusterStmt",
  "PgQuery__ClusterStmt",
  "pg_query",
  sizeof(PgQuery__ClusterStmt),
  3,
  pg_query__cluster_stmt__field_descriptors,
  pg_query__cluster_stmt__field_indices_by_name,
  1,  pg_query__cluster_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__cluster_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__vacuum_stmt__field_descriptors[3] =
{
  {
    "options",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__VacuumStmt, n_options),
    offsetof(PgQuery__VacuumStmt, options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "rels",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__VacuumStmt, n_rels),
    offsetof(PgQuery__VacuumStmt, rels),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "is_vacuumcmd",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__VacuumStmt, is_vacuumcmd),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__vacuum_stmt__field_indices_by_name[] = {
  2,   /* field[2] = is_vacuumcmd */
  0,   /* field[0] = options */
  1,   /* field[1] = rels */
};
static const ProtobufCIntRange pg_query__vacuum_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__vacuum_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.VacuumStmt",
  "VacuumStmt",
  "PgQuery__VacuumStmt",
  "pg_query",
  sizeof(PgQuery__VacuumStmt),
  3,
  pg_query__vacuum_stmt__field_descriptors,
  pg_query__vacuum_stmt__field_indices_by_name,
  1,  pg_query__vacuum_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__vacuum_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__vacuum_relation__field_descriptors[3] =
{
  {
    "relation",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__VacuumRelation, relation),
    &pg_query__range_var__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "oid",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_UINT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__VacuumRelation, oid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "va_cols",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__VacuumRelation, n_va_cols),
    offsetof(PgQuery__VacuumRelation, va_cols),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__vacuum_relation__field_indices_by_name[] = {
  1,   /* field[1] = oid */
  0,   /* field[0] = relation */
  2,   /* field[2] = va_cols */
};
static const ProtobufCIntRange pg_query__vacuum_relation__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__vacuum_relation__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.VacuumRelation",
  "VacuumRelation",
  "PgQuery__VacuumRelation",
  "pg_query",
  sizeof(PgQuery__VacuumRelation),
  3,
  pg_query__vacuum_relation__field_descriptors,
  pg_query__vacuum_relation__field_indices_by_name,
  1,  pg_query__vacuum_relation__number_ranges,
  (ProtobufCMessageInit) pg_query__vacuum_relation__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__explain_stmt__field_descriptors[2] =
{
  {
    "query",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ExplainStmt, query),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "options",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__ExplainStmt, n_options),
    offsetof(PgQuery__ExplainStmt, options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__explain_stmt__field_indices_by_name[] = {
  1,   /* field[1] = options */
  0,   /* field[0] = query */
};
static const ProtobufCIntRange pg_query__explain_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor pg_query__explain_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.ExplainStmt",
  "ExplainStmt",
  "PgQuery__ExplainStmt",
  "pg_query",
  sizeof(PgQuery__ExplainStmt),
  2,
  pg_query__explain_stmt__field_descriptors,
  pg_query__explain_stmt__field_indices_by_name,
  1,  pg_query__explain_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__explain_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__create_table_as_stmt__field_descriptors[5] =
{
  {
    "query",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateTableAsStmt, query),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "into",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateTableAsStmt, into),
    &pg_query__into_clause__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "objtype",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateTableAsStmt, objtype),
    &pg_query__object_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "is_select_into",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateTableAsStmt, is_select_into),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "if_not_exists",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateTableAsStmt, if_not_exists),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__create_table_as_stmt__field_indices_by_name[] = {
  4,   /* field[4] = if_not_exists */
  1,   /* field[1] = into */
  3,   /* field[3] = is_select_into */
  2,   /* field[2] = objtype */
  0,   /* field[0] = query */
};
static const ProtobufCIntRange pg_query__create_table_as_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor pg_query__create_table_as_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CreateTableAsStmt",
  "CreateTableAsStmt",
  "PgQuery__CreateTableAsStmt",
  "pg_query",
  sizeof(PgQuery__CreateTableAsStmt),
  5,
  pg_query__create_table_as_stmt__field_descriptors,
  pg_query__create_table_as_stmt__field_indices_by_name,
  1,  pg_query__create_table_as_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__create_table_as_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__refresh_mat_view_stmt__field_descriptors[3] =
{
  {
    "concurrent",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RefreshMatViewStmt, concurrent),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "skip_data",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RefreshMatViewStmt, skip_data),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "relation",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__RefreshMatViewStmt, relation),
    &pg_query__range_var__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__refresh_mat_view_stmt__field_indices_by_name[] = {
  0,   /* field[0] = concurrent */
  2,   /* field[2] = relation */
  1,   /* field[1] = skip_data */
};
static const ProtobufCIntRange pg_query__refresh_mat_view_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__refresh_mat_view_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.RefreshMatViewStmt",
  "RefreshMatViewStmt",
  "PgQuery__RefreshMatViewStmt",
  "pg_query",
  sizeof(PgQuery__RefreshMatViewStmt),
  3,
  pg_query__refresh_mat_view_stmt__field_descriptors,
  pg_query__refresh_mat_view_stmt__field_indices_by_name,
  1,  pg_query__refresh_mat_view_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__refresh_mat_view_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
#define pg_query__check_point_stmt__field_descriptors NULL
#define pg_query__check_point_stmt__field_indices_by_name NULL
#define pg_query__check_point_stmt__number_ranges NULL
const ProtobufCMessageDescriptor pg_query__check_point_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CheckPointStmt",
  "CheckPointStmt",
  "PgQuery__CheckPointStmt",
  "pg_query",
  sizeof(PgQuery__CheckPointStmt),
  0,
  pg_query__check_point_stmt__field_descriptors,
  pg_query__check_point_stmt__field_indices_by_name,
  0,  pg_query__check_point_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__check_point_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__discard_stmt__field_descriptors[1] =
{
  {
    "target",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DiscardStmt, target),
    &pg_query__discard_mode__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__discard_stmt__field_indices_by_name[] = {
  0,   /* field[0] = target */
};
static const ProtobufCIntRange pg_query__discard_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 1 }
};
const ProtobufCMessageDescriptor pg_query__discard_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.DiscardStmt",
  "DiscardStmt",
  "PgQuery__DiscardStmt",
  "pg_query",
  sizeof(PgQuery__DiscardStmt),
  1,
  pg_query__discard_stmt__field_descriptors,
  pg_query__discard_stmt__field_indices_by_name,
  1,  pg_query__discard_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__discard_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__lock_stmt__field_descriptors[3] =
{
  {
    "relations",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__LockStmt, n_relations),
    offsetof(PgQuery__LockStmt, relations),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "mode",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__LockStmt, mode),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "nowait",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__LockStmt, nowait),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__lock_stmt__field_indices_by_name[] = {
  1,   /* field[1] = mode */
  2,   /* field[2] = nowait */
  0,   /* field[0] = relations */
};
static const ProtobufCIntRange pg_query__lock_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__lock_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.LockStmt",
  "LockStmt",
  "PgQuery__LockStmt",
  "pg_query",
  sizeof(PgQuery__LockStmt),
  3,
  pg_query__lock_stmt__field_descriptors,
  pg_query__lock_stmt__field_indices_by_name,
  1,  pg_query__lock_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__lock_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__constraints_set_stmt__field_descriptors[2] =
{
  {
    "constraints",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__ConstraintsSetStmt, n_constraints),
    offsetof(PgQuery__ConstraintsSetStmt, constraints),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "deferred",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ConstraintsSetStmt, deferred),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__constraints_set_stmt__field_indices_by_name[] = {
  0,   /* field[0] = constraints */
  1,   /* field[1] = deferred */
};
static const ProtobufCIntRange pg_query__constraints_set_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor pg_query__constraints_set_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.ConstraintsSetStmt",
  "ConstraintsSetStmt",
  "PgQuery__ConstraintsSetStmt",
  "pg_query",
  sizeof(PgQuery__ConstraintsSetStmt),
  2,
  pg_query__constraints_set_stmt__field_descriptors,
  pg_query__constraints_set_stmt__field_indices_by_name,
  1,  pg_query__constraints_set_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__constraints_set_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__reindex_stmt__field_descriptors[4] =
{
  {
    "kind",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ReindexStmt, kind),
    &pg_query__reindex_object_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "relation",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ReindexStmt, relation),
    &pg_query__range_var__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "name",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ReindexStmt, name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "params",
    4,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__ReindexStmt, n_params),
    offsetof(PgQuery__ReindexStmt, params),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__reindex_stmt__field_indices_by_name[] = {
  0,   /* field[0] = kind */
  2,   /* field[2] = name */
  3,   /* field[3] = params */
  1,   /* field[1] = relation */
};
static const ProtobufCIntRange pg_query__reindex_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor pg_query__reindex_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.ReindexStmt",
  "ReindexStmt",
  "PgQuery__ReindexStmt",
  "pg_query",
  sizeof(PgQuery__ReindexStmt),
  4,
  pg_query__reindex_stmt__field_descriptors,
  pg_query__reindex_stmt__field_indices_by_name,
  1,  pg_query__reindex_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__reindex_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__create_conversion_stmt__field_descriptors[5] =
{
  {
    "conversion_name",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateConversionStmt, n_conversion_name),
    offsetof(PgQuery__CreateConversionStmt, conversion_name),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "for_encoding_name",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateConversionStmt, for_encoding_name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "to_encoding_name",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateConversionStmt, to_encoding_name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "func_name",
    4,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateConversionStmt, n_func_name),
    offsetof(PgQuery__CreateConversionStmt, func_name),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "def",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateConversionStmt, def),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__create_conversion_stmt__field_indices_by_name[] = {
  0,   /* field[0] = conversion_name */
  4,   /* field[4] = def */
  1,   /* field[1] = for_encoding_name */
  3,   /* field[3] = func_name */
  2,   /* field[2] = to_encoding_name */
};
static const ProtobufCIntRange pg_query__create_conversion_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor pg_query__create_conversion_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CreateConversionStmt",
  "CreateConversionStmt",
  "PgQuery__CreateConversionStmt",
  "pg_query",
  sizeof(PgQuery__CreateConversionStmt),
  5,
  pg_query__create_conversion_stmt__field_descriptors,
  pg_query__create_conversion_stmt__field_indices_by_name,
  1,  pg_query__create_conversion_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__create_conversion_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__create_cast_stmt__field_descriptors[5] =
{
  {
    "sourcetype",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateCastStmt, sourcetype),
    &pg_query__type_name__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "targettype",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateCastStmt, targettype),
    &pg_query__type_name__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "func",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateCastStmt, func),
    &pg_query__object_with_args__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "context",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateCastStmt, context),
    &pg_query__coercion_context__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "inout",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateCastStmt, inout),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__create_cast_stmt__field_indices_by_name[] = {
  3,   /* field[3] = context */
  2,   /* field[2] = func */
  4,   /* field[4] = inout */
  0,   /* field[0] = sourcetype */
  1,   /* field[1] = targettype */
};
static const ProtobufCIntRange pg_query__create_cast_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor pg_query__create_cast_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CreateCastStmt",
  "CreateCastStmt",
  "PgQuery__CreateCastStmt",
  "pg_query",
  sizeof(PgQuery__CreateCastStmt),
  5,
  pg_query__create_cast_stmt__field_descriptors,
  pg_query__create_cast_stmt__field_indices_by_name,
  1,  pg_query__create_cast_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__create_cast_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__create_transform_stmt__field_descriptors[5] =
{
  {
    "replace",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateTransformStmt, replace),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "type_name",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateTransformStmt, type_name),
    &pg_query__type_name__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "lang",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateTransformStmt, lang),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "fromsql",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateTransformStmt, fromsql),
    &pg_query__object_with_args__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "tosql",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateTransformStmt, tosql),
    &pg_query__object_with_args__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__create_transform_stmt__field_indices_by_name[] = {
  3,   /* field[3] = fromsql */
  2,   /* field[2] = lang */
  0,   /* field[0] = replace */
  4,   /* field[4] = tosql */
  1,   /* field[1] = type_name */
};
static const ProtobufCIntRange pg_query__create_transform_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor pg_query__create_transform_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CreateTransformStmt",
  "CreateTransformStmt",
  "PgQuery__CreateTransformStmt",
  "pg_query",
  sizeof(PgQuery__CreateTransformStmt),
  5,
  pg_query__create_transform_stmt__field_descriptors,
  pg_query__create_transform_stmt__field_indices_by_name,
  1,  pg_query__create_transform_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__create_transform_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__prepare_stmt__field_descriptors[3] =
{
  {
    "name",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__PrepareStmt, name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "argtypes",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__PrepareStmt, n_argtypes),
    offsetof(PgQuery__PrepareStmt, argtypes),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "query",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__PrepareStmt, query),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__prepare_stmt__field_indices_by_name[] = {
  1,   /* field[1] = argtypes */
  0,   /* field[0] = name */
  2,   /* field[2] = query */
};
static const ProtobufCIntRange pg_query__prepare_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__prepare_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.PrepareStmt",
  "PrepareStmt",
  "PgQuery__PrepareStmt",
  "pg_query",
  sizeof(PgQuery__PrepareStmt),
  3,
  pg_query__prepare_stmt__field_descriptors,
  pg_query__prepare_stmt__field_indices_by_name,
  1,  pg_query__prepare_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__prepare_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__execute_stmt__field_descriptors[2] =
{
  {
    "name",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ExecuteStmt, name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "params",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__ExecuteStmt, n_params),
    offsetof(PgQuery__ExecuteStmt, params),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__execute_stmt__field_indices_by_name[] = {
  0,   /* field[0] = name */
  1,   /* field[1] = params */
};
static const ProtobufCIntRange pg_query__execute_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor pg_query__execute_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.ExecuteStmt",
  "ExecuteStmt",
  "PgQuery__ExecuteStmt",
  "pg_query",
  sizeof(PgQuery__ExecuteStmt),
  2,
  pg_query__execute_stmt__field_descriptors,
  pg_query__execute_stmt__field_indices_by_name,
  1,  pg_query__execute_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__execute_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__deallocate_stmt__field_descriptors[3] =
{
  {
    "name",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DeallocateStmt, name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "isall",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DeallocateStmt, isall),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DeallocateStmt, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__deallocate_stmt__field_indices_by_name[] = {
  1,   /* field[1] = isall */
  2,   /* field[2] = location */
  0,   /* field[0] = name */
};
static const ProtobufCIntRange pg_query__deallocate_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__deallocate_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.DeallocateStmt",
  "DeallocateStmt",
  "PgQuery__DeallocateStmt",
  "pg_query",
  sizeof(PgQuery__DeallocateStmt),
  3,
  pg_query__deallocate_stmt__field_descriptors,
  pg_query__deallocate_stmt__field_indices_by_name,
  1,  pg_query__deallocate_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__deallocate_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__drop_owned_stmt__field_descriptors[2] =
{
  {
    "roles",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__DropOwnedStmt, n_roles),
    offsetof(PgQuery__DropOwnedStmt, roles),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "behavior",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DropOwnedStmt, behavior),
    &pg_query__drop_behavior__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__drop_owned_stmt__field_indices_by_name[] = {
  1,   /* field[1] = behavior */
  0,   /* field[0] = roles */
};
static const ProtobufCIntRange pg_query__drop_owned_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor pg_query__drop_owned_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.DropOwnedStmt",
  "DropOwnedStmt",
  "PgQuery__DropOwnedStmt",
  "pg_query",
  sizeof(PgQuery__DropOwnedStmt),
  2,
  pg_query__drop_owned_stmt__field_descriptors,
  pg_query__drop_owned_stmt__field_indices_by_name,
  1,  pg_query__drop_owned_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__drop_owned_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__reassign_owned_stmt__field_descriptors[2] =
{
  {
    "roles",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__ReassignOwnedStmt, n_roles),
    offsetof(PgQuery__ReassignOwnedStmt, roles),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "newrole",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ReassignOwnedStmt, newrole),
    &pg_query__role_spec__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__reassign_owned_stmt__field_indices_by_name[] = {
  1,   /* field[1] = newrole */
  0,   /* field[0] = roles */
};
static const ProtobufCIntRange pg_query__reassign_owned_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor pg_query__reassign_owned_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.ReassignOwnedStmt",
  "ReassignOwnedStmt",
  "PgQuery__ReassignOwnedStmt",
  "pg_query",
  sizeof(PgQuery__ReassignOwnedStmt),
  2,
  pg_query__reassign_owned_stmt__field_descriptors,
  pg_query__reassign_owned_stmt__field_indices_by_name,
  1,  pg_query__reassign_owned_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__reassign_owned_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__alter_tsdictionary_stmt__field_descriptors[2] =
{
  {
    "dictname",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AlterTSDictionaryStmt, n_dictname),
    offsetof(PgQuery__AlterTSDictionaryStmt, dictname),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "options",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AlterTSDictionaryStmt, n_options),
    offsetof(PgQuery__AlterTSDictionaryStmt, options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__alter_tsdictionary_stmt__field_indices_by_name[] = {
  0,   /* field[0] = dictname */
  1,   /* field[1] = options */
};
static const ProtobufCIntRange pg_query__alter_tsdictionary_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor pg_query__alter_tsdictionary_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.AlterTSDictionaryStmt",
  "AlterTSDictionaryStmt",
  "PgQuery__AlterTSDictionaryStmt",
  "pg_query",
  sizeof(PgQuery__AlterTSDictionaryStmt),
  2,
  pg_query__alter_tsdictionary_stmt__field_descriptors,
  pg_query__alter_tsdictionary_stmt__field_indices_by_name,
  1,  pg_query__alter_tsdictionary_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__alter_tsdictionary_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__alter_tsconfiguration_stmt__field_descriptors[7] =
{
  {
    "kind",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterTSConfigurationStmt, kind),
    &pg_query__alter_tsconfig_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "cfgname",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AlterTSConfigurationStmt, n_cfgname),
    offsetof(PgQuery__AlterTSConfigurationStmt, cfgname),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "tokentype",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AlterTSConfigurationStmt, n_tokentype),
    offsetof(PgQuery__AlterTSConfigurationStmt, tokentype),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "dicts",
    4,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AlterTSConfigurationStmt, n_dicts),
    offsetof(PgQuery__AlterTSConfigurationStmt, dicts),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "override",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterTSConfigurationStmt, override),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "replace",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterTSConfigurationStmt, replace),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "missing_ok",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterTSConfigurationStmt, missing_ok),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__alter_tsconfiguration_stmt__field_indices_by_name[] = {
  1,   /* field[1] = cfgname */
  3,   /* field[3] = dicts */
  0,   /* field[0] = kind */
  6,   /* field[6] = missing_ok */
  4,   /* field[4] = override */
  5,   /* field[5] = replace */
  2,   /* field[2] = tokentype */
};
static const ProtobufCIntRange pg_query__alter_tsconfiguration_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 7 }
};
const ProtobufCMessageDescriptor pg_query__alter_tsconfiguration_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.AlterTSConfigurationStmt",
  "AlterTSConfigurationStmt",
  "PgQuery__AlterTSConfigurationStmt",
  "pg_query",
  sizeof(PgQuery__AlterTSConfigurationStmt),
  7,
  pg_query__alter_tsconfiguration_stmt__field_descriptors,
  pg_query__alter_tsconfiguration_stmt__field_indices_by_name,
  1,  pg_query__alter_tsconfiguration_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__alter_tsconfiguration_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__publication_table__field_descriptors[3] =
{
  {
    "relation",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__PublicationTable, relation),
    &pg_query__range_var__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "where_clause",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__PublicationTable, where_clause),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "columns",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__PublicationTable, n_columns),
    offsetof(PgQuery__PublicationTable, columns),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__publication_table__field_indices_by_name[] = {
  2,   /* field[2] = columns */
  0,   /* field[0] = relation */
  1,   /* field[1] = where_clause */
};
static const ProtobufCIntRange pg_query__publication_table__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__publication_table__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.PublicationTable",
  "PublicationTable",
  "PgQuery__PublicationTable",
  "pg_query",
  sizeof(PgQuery__PublicationTable),
  3,
  pg_query__publication_table__field_descriptors,
  pg_query__publication_table__field_indices_by_name,
  1,  pg_query__publication_table__number_ranges,
  (ProtobufCMessageInit) pg_query__publication_table__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__publication_obj_spec__field_descriptors[4] =
{
  {
    "pubobjtype",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__PublicationObjSpec, pubobjtype),
    &pg_query__publication_obj_spec_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "name",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__PublicationObjSpec, name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "pubtable",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(PgQuery__PublicationObjSpec, pubtable),
    &pg_query__publication_table__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "location",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__PublicationObjSpec, location),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__publication_obj_spec__field_indices_by_name[] = {
  3,   /* field[3] = location */
  1,   /* field[1] = name */
  0,   /* field[0] = pubobjtype */
  2,   /* field[2] = pubtable */
};
static const ProtobufCIntRange pg_query__publication_obj_spec__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor pg_query__publication_obj_spec__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.PublicationObjSpec",
  "PublicationObjSpec",
  "PgQuery__PublicationObjSpec",
  "pg_query",
  sizeof(PgQuery__PublicationObjSpec),
  4,
  pg_query__publication_obj_spec__field_descriptors,
  pg_query__publication_obj_spec__field_indices_by_name,
  1,  pg_query__publication_obj_spec__number_ranges,
  (ProtobufCMessageInit) pg_query__publication_obj_spec__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__create_publication_stmt__field_descriptors[4] =
{
  {
    "pubname",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreatePublicationStmt, pubname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "options",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreatePublicationStmt, n_options),
    offsetof(PgQuery__CreatePublicationStmt, options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "pubobjects",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreatePublicationStmt, n_pubobjects),
    offsetof(PgQuery__CreatePublicationStmt, pubobjects),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "for_all_tables",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreatePublicationStmt, for_all_tables),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__create_publication_stmt__field_indices_by_name[] = {
  3,   /* field[3] = for_all_tables */
  1,   /* field[1] = options */
  0,   /* field[0] = pubname */
  2,   /* field[2] = pubobjects */
};
static const ProtobufCIntRange pg_query__create_publication_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor pg_query__create_publication_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CreatePublicationStmt",
  "CreatePublicationStmt",
  "PgQuery__CreatePublicationStmt",
  "pg_query",
  sizeof(PgQuery__CreatePublicationStmt),
  4,
  pg_query__create_publication_stmt__field_descriptors,
  pg_query__create_publication_stmt__field_indices_by_name,
  1,  pg_query__create_publication_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__create_publication_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__alter_publication_stmt__field_descriptors[5] =
{
  {
    "pubname",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterPublicationStmt, pubname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "options",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AlterPublicationStmt, n_options),
    offsetof(PgQuery__AlterPublicationStmt, options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "pubobjects",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AlterPublicationStmt, n_pubobjects),
    offsetof(PgQuery__AlterPublicationStmt, pubobjects),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "for_all_tables",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterPublicationStmt, for_all_tables),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "action",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterPublicationStmt, action),
    &pg_query__alter_publication_action__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__alter_publication_stmt__field_indices_by_name[] = {
  4,   /* field[4] = action */
  3,   /* field[3] = for_all_tables */
  1,   /* field[1] = options */
  0,   /* field[0] = pubname */
  2,   /* field[2] = pubobjects */
};
static const ProtobufCIntRange pg_query__alter_publication_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor pg_query__alter_publication_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.AlterPublicationStmt",
  "AlterPublicationStmt",
  "PgQuery__AlterPublicationStmt",
  "pg_query",
  sizeof(PgQuery__AlterPublicationStmt),
  5,
  pg_query__alter_publication_stmt__field_descriptors,
  pg_query__alter_publication_stmt__field_indices_by_name,
  1,  pg_query__alter_publication_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__alter_publication_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__create_subscription_stmt__field_descriptors[4] =
{
  {
    "subname",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateSubscriptionStmt, subname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "conninfo",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__CreateSubscriptionStmt, conninfo),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "publication",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateSubscriptionStmt, n_publication),
    offsetof(PgQuery__CreateSubscriptionStmt, publication),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "options",
    4,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__CreateSubscriptionStmt, n_options),
    offsetof(PgQuery__CreateSubscriptionStmt, options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__create_subscription_stmt__field_indices_by_name[] = {
  1,   /* field[1] = conninfo */
  3,   /* field[3] = options */
  2,   /* field[2] = publication */
  0,   /* field[0] = subname */
};
static const ProtobufCIntRange pg_query__create_subscription_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor pg_query__create_subscription_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.CreateSubscriptionStmt",
  "CreateSubscriptionStmt",
  "PgQuery__CreateSubscriptionStmt",
  "pg_query",
  sizeof(PgQuery__CreateSubscriptionStmt),
  4,
  pg_query__create_subscription_stmt__field_descriptors,
  pg_query__create_subscription_stmt__field_indices_by_name,
  1,  pg_query__create_subscription_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__create_subscription_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__alter_subscription_stmt__field_descriptors[5] =
{
  {
    "kind",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterSubscriptionStmt, kind),
    &pg_query__alter_subscription_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "subname",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterSubscriptionStmt, subname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "conninfo",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__AlterSubscriptionStmt, conninfo),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "publication",
    4,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AlterSubscriptionStmt, n_publication),
    offsetof(PgQuery__AlterSubscriptionStmt, publication),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "options",
    5,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(PgQuery__AlterSubscriptionStmt, n_options),
    offsetof(PgQuery__AlterSubscriptionStmt, options),
    &pg_query__node__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__alter_subscription_stmt__field_indices_by_name[] = {
  2,   /* field[2] = conninfo */
  0,   /* field[0] = kind */
  4,   /* field[4] = options */
  3,   /* field[3] = publication */
  1,   /* field[1] = subname */
};
static const ProtobufCIntRange pg_query__alter_subscription_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor pg_query__alter_subscription_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.AlterSubscriptionStmt",
  "AlterSubscriptionStmt",
  "PgQuery__AlterSubscriptionStmt",
  "pg_query",
  sizeof(PgQuery__AlterSubscriptionStmt),
  5,
  pg_query__alter_subscription_stmt__field_descriptors,
  pg_query__alter_subscription_stmt__field_indices_by_name,
  1,  pg_query__alter_subscription_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__alter_subscription_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__drop_subscription_stmt__field_descriptors[3] =
{
  {
    "subname",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DropSubscriptionStmt, subname),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "missing_ok",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DropSubscriptionStmt, missing_ok),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "behavior",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__DropSubscriptionStmt, behavior),
    &pg_query__drop_behavior__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__drop_subscription_stmt__field_indices_by_name[] = {
  2,   /* field[2] = behavior */
  1,   /* field[1] = missing_ok */
  0,   /* field[0] = subname */
};
static const ProtobufCIntRange pg_query__drop_subscription_stmt__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor pg_query__drop_subscription_stmt__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.DropSubscriptionStmt",
  "DropSubscriptionStmt",
  "PgQuery__DropSubscriptionStmt",
  "pg_query",
  sizeof(PgQuery__DropSubscriptionStmt),
  3,
  pg_query__drop_subscription_stmt__field_descriptors,
  pg_query__drop_subscription_stmt__field_indices_by_name,
  1,  pg_query__drop_subscription_stmt__number_ranges,
  (ProtobufCMessageInit) pg_query__drop_subscription_stmt__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor pg_query__scan_token__field_descriptors[4] =
{
  {
    "start",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ScanToken, start),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "end",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ScanToken, end),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "token",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ScanToken, token),
    &pg_query__token__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "keyword_kind",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(PgQuery__ScanToken, keyword_kind),
    &pg_query__keyword_kind__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned pg_query__scan_token__field_indices_by_name[] = {
  1,   /* field[1] = end */
  3,   /* field[3] = keyword_kind */
  0,   /* field[0] = start */
  2,   /* field[2] = token */
};
static const ProtobufCIntRange pg_query__scan_token__number_ranges[2 + 1] =
{
  { 1, 0 },
  { 4, 2 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor pg_query__scan_token__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "pg_query.ScanToken",
  "ScanToken",
  "PgQuery__ScanToken",
  "pg_query",
  sizeof(PgQuery__ScanToken),
  4,
  pg_query__scan_token__field_descriptors,
  pg_query__scan_token__field_indices_by_name,
  2,  pg_query__scan_token__number_ranges,
  (ProtobufCMessageInit) pg_query__scan_token__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCEnumValue pg_query__query_source__enum_values_by_number[6] =
{
  { "QUERY_SOURCE_UNDEFINED", "PG_QUERY__QUERY_SOURCE__QUERY_SOURCE_UNDEFINED", 0 },
  { "QSRC_ORIGINAL", "PG_QUERY__QUERY_SOURCE__QSRC_ORIGINAL", 1 },
  { "QSRC_PARSER", "PG_QUERY__QUERY_SOURCE__QSRC_PARSER", 2 },
  { "QSRC_INSTEAD_RULE", "PG_QUERY__QUERY_SOURCE__QSRC_INSTEAD_RULE", 3 },
  { "QSRC_QUAL_INSTEAD_RULE", "PG_QUERY__QUERY_SOURCE__QSRC_QUAL_INSTEAD_RULE", 4 },
  { "QSRC_NON_INSTEAD_RULE", "PG_QUERY__QUERY_SOURCE__QSRC_NON_INSTEAD_RULE", 5 },
};
static const ProtobufCIntRange pg_query__query_source__value_ranges[] = {
{0, 0},{0, 6}
};
static const ProtobufCEnumValueIndex pg_query__query_source__enum_values_by_name[6] =
{
  { "QSRC_INSTEAD_RULE", 3 },
  { "QSRC_NON_INSTEAD_RULE", 5 },
  { "QSRC_ORIGINAL", 1 },
  { "QSRC_PARSER", 2 },
  { "QSRC_QUAL_INSTEAD_RULE", 4 },
  { "QUERY_SOURCE_UNDEFINED", 0 },
};
const ProtobufCEnumDescriptor pg_query__query_source__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.QuerySource",
  "QuerySource",
  "PgQuery__QuerySource",
  "pg_query",
  6,
  pg_query__query_source__enum_values_by_number,
  6,
  pg_query__query_source__enum_values_by_name,
  1,
  pg_query__query_source__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__sort_by_dir__enum_values_by_number[5] =
{
  { "SORT_BY_DIR_UNDEFINED", "PG_QUERY__SORT_BY_DIR__SORT_BY_DIR_UNDEFINED", 0 },
  { "SORTBY_DEFAULT", "PG_QUERY__SORT_BY_DIR__SORTBY_DEFAULT", 1 },
  { "SORTBY_ASC", "PG_QUERY__SORT_BY_DIR__SORTBY_ASC", 2 },
  { "SORTBY_DESC", "PG_QUERY__SORT_BY_DIR__SORTBY_DESC", 3 },
  { "SORTBY_USING", "PG_QUERY__SORT_BY_DIR__SORTBY_USING", 4 },
};
static const ProtobufCIntRange pg_query__sort_by_dir__value_ranges[] = {
{0, 0},{0, 5}
};
static const ProtobufCEnumValueIndex pg_query__sort_by_dir__enum_values_by_name[5] =
{
  { "SORTBY_ASC", 2 },
  { "SORTBY_DEFAULT", 1 },
  { "SORTBY_DESC", 3 },
  { "SORTBY_USING", 4 },
  { "SORT_BY_DIR_UNDEFINED", 0 },
};
const ProtobufCEnumDescriptor pg_query__sort_by_dir__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.SortByDir",
  "SortByDir",
  "PgQuery__SortByDir",
  "pg_query",
  5,
  pg_query__sort_by_dir__enum_values_by_number,
  5,
  pg_query__sort_by_dir__enum_values_by_name,
  1,
  pg_query__sort_by_dir__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__sort_by_nulls__enum_values_by_number[4] =
{
  { "SORT_BY_NULLS_UNDEFINED", "PG_QUERY__SORT_BY_NULLS__SORT_BY_NULLS_UNDEFINED", 0 },
  { "SORTBY_NULLS_DEFAULT", "PG_QUERY__SORT_BY_NULLS__SORTBY_NULLS_DEFAULT", 1 },
  { "SORTBY_NULLS_FIRST", "PG_QUERY__SORT_BY_NULLS__SORTBY_NULLS_FIRST", 2 },
  { "SORTBY_NULLS_LAST", "PG_QUERY__SORT_BY_NULLS__SORTBY_NULLS_LAST", 3 },
};
static const ProtobufCIntRange pg_query__sort_by_nulls__value_ranges[] = {
{0, 0},{0, 4}
};
static const ProtobufCEnumValueIndex pg_query__sort_by_nulls__enum_values_by_name[4] =
{
  { "SORTBY_NULLS_DEFAULT", 1 },
  { "SORTBY_NULLS_FIRST", 2 },
  { "SORTBY_NULLS_LAST", 3 },
  { "SORT_BY_NULLS_UNDEFINED", 0 },
};
const ProtobufCEnumDescriptor pg_query__sort_by_nulls__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.SortByNulls",
  "SortByNulls",
  "PgQuery__SortByNulls",
  "pg_query",
  4,
  pg_query__sort_by_nulls__enum_values_by_number,
  4,
  pg_query__sort_by_nulls__enum_values_by_name,
  1,
  pg_query__sort_by_nulls__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__set_quantifier__enum_values_by_number[4] =
{
  { "SET_QUANTIFIER_UNDEFINED", "PG_QUERY__SET_QUANTIFIER__SET_QUANTIFIER_UNDEFINED", 0 },
  { "SET_QUANTIFIER_DEFAULT", "PG_QUERY__SET_QUANTIFIER__SET_QUANTIFIER_DEFAULT", 1 },
  { "SET_QUANTIFIER_ALL", "PG_QUERY__SET_QUANTIFIER__SET_QUANTIFIER_ALL", 2 },
  { "SET_QUANTIFIER_DISTINCT", "PG_QUERY__SET_QUANTIFIER__SET_QUANTIFIER_DISTINCT", 3 },
};
static const ProtobufCIntRange pg_query__set_quantifier__value_ranges[] = {
{0, 0},{0, 4}
};
static const ProtobufCEnumValueIndex pg_query__set_quantifier__enum_values_by_name[4] =
{
  { "SET_QUANTIFIER_ALL", 2 },
  { "SET_QUANTIFIER_DEFAULT", 1 },
  { "SET_QUANTIFIER_DISTINCT", 3 },
  { "SET_QUANTIFIER_UNDEFINED", 0 },
};
const ProtobufCEnumDescriptor pg_query__set_quantifier__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.SetQuantifier",
  "SetQuantifier",
  "PgQuery__SetQuantifier",
  "pg_query",
  4,
  pg_query__set_quantifier__enum_values_by_number,
  4,
  pg_query__set_quantifier__enum_values_by_name,
  1,
  pg_query__set_quantifier__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__a__expr__kind__enum_values_by_number[15] =
{
  { "A_EXPR_KIND_UNDEFINED", "PG_QUERY__A__EXPR__KIND__A_EXPR_KIND_UNDEFINED", 0 },
  { "AEXPR_OP", "PG_QUERY__A__EXPR__KIND__AEXPR_OP", 1 },
  { "AEXPR_OP_ANY", "PG_QUERY__A__EXPR__KIND__AEXPR_OP_ANY", 2 },
  { "AEXPR_OP_ALL", "PG_QUERY__A__EXPR__KIND__AEXPR_OP_ALL", 3 },
  { "AEXPR_DISTINCT", "PG_QUERY__A__EXPR__KIND__AEXPR_DISTINCT", 4 },
  { "AEXPR_NOT_DISTINCT", "PG_QUERY__A__EXPR__KIND__AEXPR_NOT_DISTINCT", 5 },
  { "AEXPR_NULLIF", "PG_QUERY__A__EXPR__KIND__AEXPR_NULLIF", 6 },
  { "AEXPR_IN", "PG_QUERY__A__EXPR__KIND__AEXPR_IN", 7 },
  { "AEXPR_LIKE", "PG_QUERY__A__EXPR__KIND__AEXPR_LIKE", 8 },
  { "AEXPR_ILIKE", "PG_QUERY__A__EXPR__KIND__AEXPR_ILIKE", 9 },
  { "AEXPR_SIMILAR", "PG_QUERY__A__EXPR__KIND__AEXPR_SIMILAR", 10 },
  { "AEXPR_BETWEEN", "PG_QUERY__A__EXPR__KIND__AEXPR_BETWEEN", 11 },
  { "AEXPR_NOT_BETWEEN", "PG_QUERY__A__EXPR__KIND__AEXPR_NOT_BETWEEN", 12 },
  { "AEXPR_BETWEEN_SYM", "PG_QUERY__A__EXPR__KIND__AEXPR_BETWEEN_SYM", 13 },
  { "AEXPR_NOT_BETWEEN_SYM", "PG_QUERY__A__EXPR__KIND__AEXPR_NOT_BETWEEN_SYM", 14 },
};
static const ProtobufCIntRange pg_query__a__expr__kind__value_ranges[] = {
{0, 0},{0, 15}
};
static const ProtobufCEnumValueIndex pg_query__a__expr__kind__enum_values_by_name[15] =
{
  { "AEXPR_BETWEEN", 11 },
  { "AEXPR_BETWEEN_SYM", 13 },
  { "AEXPR_DISTINCT", 4 },
  { "AEXPR_ILIKE", 9 },
  { "AEXPR_IN", 7 },
  { "AEXPR_LIKE", 8 },
  { "AEXPR_NOT_BETWEEN", 12 },
  { "AEXPR_NOT_BETWEEN_SYM", 14 },
  { "AEXPR_NOT_DISTINCT", 5 },
  { "AEXPR_NULLIF", 6 },
  { "AEXPR_OP", 1 },
  { "AEXPR_OP_ALL", 3 },
  { "AEXPR_OP_ANY", 2 },
  { "AEXPR_SIMILAR", 10 },
  { "A_EXPR_KIND_UNDEFINED", 0 },
};
const ProtobufCEnumDescriptor pg_query__a__expr__kind__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.A_Expr_Kind",
  "A_Expr_Kind",
  "PgQuery__AExprKind",
  "pg_query",
  15,
  pg_query__a__expr__kind__enum_values_by_number,
  15,
  pg_query__a__expr__kind__enum_values_by_name,
  1,
  pg_query__a__expr__kind__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__role_spec_type__enum_values_by_number[6] =
{
  { "ROLE_SPEC_TYPE_UNDEFINED", "PG_QUERY__ROLE_SPEC_TYPE__ROLE_SPEC_TYPE_UNDEFINED", 0 },
  { "ROLESPEC_CSTRING", "PG_QUERY__ROLE_SPEC_TYPE__ROLESPEC_CSTRING", 1 },
  { "ROLESPEC_CURRENT_ROLE", "PG_QUERY__ROLE_SPEC_TYPE__ROLESPEC_CURRENT_ROLE", 2 },
  { "ROLESPEC_CURRENT_USER", "PG_QUERY__ROLE_SPEC_TYPE__ROLESPEC_CURRENT_USER", 3 },
  { "ROLESPEC_SESSION_USER", "PG_QUERY__ROLE_SPEC_TYPE__ROLESPEC_SESSION_USER", 4 },
  { "ROLESPEC_PUBLIC", "PG_QUERY__ROLE_SPEC_TYPE__ROLESPEC_PUBLIC", 5 },
};
static const ProtobufCIntRange pg_query__role_spec_type__value_ranges[] = {
{0, 0},{0, 6}
};
static const ProtobufCEnumValueIndex pg_query__role_spec_type__enum_values_by_name[6] =
{
  { "ROLESPEC_CSTRING", 1 },
  { "ROLESPEC_CURRENT_ROLE", 2 },
  { "ROLESPEC_CURRENT_USER", 3 },
  { "ROLESPEC_PUBLIC", 5 },
  { "ROLESPEC_SESSION_USER", 4 },
  { "ROLE_SPEC_TYPE_UNDEFINED", 0 },
};
const ProtobufCEnumDescriptor pg_query__role_spec_type__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.RoleSpecType",
  "RoleSpecType",
  "PgQuery__RoleSpecType",
  "pg_query",
  6,
  pg_query__role_spec_type__enum_values_by_number,
  6,
  pg_query__role_spec_type__enum_values_by_name,
  1,
  pg_query__role_spec_type__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__table_like_option__enum_values_by_number[11] =
{
  { "TABLE_LIKE_OPTION_UNDEFINED", "PG_QUERY__TABLE_LIKE_OPTION__TABLE_LIKE_OPTION_UNDEFINED", 0 },
  { "CREATE_TABLE_LIKE_COMMENTS", "PG_QUERY__TABLE_LIKE_OPTION__CREATE_TABLE_LIKE_COMMENTS", 1 },
  { "CREATE_TABLE_LIKE_COMPRESSION", "PG_QUERY__TABLE_LIKE_OPTION__CREATE_TABLE_LIKE_COMPRESSION", 2 },
  { "CREATE_TABLE_LIKE_CONSTRAINTS", "PG_QUERY__TABLE_LIKE_OPTION__CREATE_TABLE_LIKE_CONSTRAINTS", 3 },
  { "CREATE_TABLE_LIKE_DEFAULTS", "PG_QUERY__TABLE_LIKE_OPTION__CREATE_TABLE_LIKE_DEFAULTS", 4 },
  { "CREATE_TABLE_LIKE_GENERATED", "PG_QUERY__TABLE_LIKE_OPTION__CREATE_TABLE_LIKE_GENERATED", 5 },
  { "CREATE_TABLE_LIKE_IDENTITY", "PG_QUERY__TABLE_LIKE_OPTION__CREATE_TABLE_LIKE_IDENTITY", 6 },
  { "CREATE_TABLE_LIKE_INDEXES", "PG_QUERY__TABLE_LIKE_OPTION__CREATE_TABLE_LIKE_INDEXES", 7 },
  { "CREATE_TABLE_LIKE_STATISTICS", "PG_QUERY__TABLE_LIKE_OPTION__CREATE_TABLE_LIKE_STATISTICS", 8 },
  { "CREATE_TABLE_LIKE_STORAGE", "PG_QUERY__TABLE_LIKE_OPTION__CREATE_TABLE_LIKE_STORAGE", 9 },
  { "CREATE_TABLE_LIKE_ALL", "PG_QUERY__TABLE_LIKE_OPTION__CREATE_TABLE_LIKE_ALL", 10 },
};
static const ProtobufCIntRange pg_query__table_like_option__value_ranges[] = {
{0, 0},{0, 11}
};
static const ProtobufCEnumValueIndex pg_query__table_like_option__enum_values_by_name[11] =
{
  { "CREATE_TABLE_LIKE_ALL", 10 },
  { "CREATE_TABLE_LIKE_COMMENTS", 1 },
  { "CREATE_TABLE_LIKE_COMPRESSION", 2 },
  { "CREATE_TABLE_LIKE_CONSTRAINTS", 3 },
  { "CREATE_TABLE_LIKE_DEFAULTS", 4 },
  { "CREATE_TABLE_LIKE_GENERATED", 5 },
  { "CREATE_TABLE_LIKE_IDENTITY", 6 },
  { "CREATE_TABLE_LIKE_INDEXES", 7 },
  { "CREATE_TABLE_LIKE_STATISTICS", 8 },
  { "CREATE_TABLE_LIKE_STORAGE", 9 },
  { "TABLE_LIKE_OPTION_UNDEFINED", 0 },
};
const ProtobufCEnumDescriptor pg_query__table_like_option__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.TableLikeOption",
  "TableLikeOption",
  "PgQuery__TableLikeOption",
  "pg_query",
  11,
  pg_query__table_like_option__enum_values_by_number,
  11,
  pg_query__table_like_option__enum_values_by_name,
  1,
  pg_query__table_like_option__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__def_elem_action__enum_values_by_number[5] =
{
  { "DEF_ELEM_ACTION_UNDEFINED", "PG_QUERY__DEF_ELEM_ACTION__DEF_ELEM_ACTION_UNDEFINED", 0 },
  { "DEFELEM_UNSPEC", "PG_QUERY__DEF_ELEM_ACTION__DEFELEM_UNSPEC", 1 },
  { "DEFELEM_SET", "PG_QUERY__DEF_ELEM_ACTION__DEFELEM_SET", 2 },
  { "DEFELEM_ADD", "PG_QUERY__DEF_ELEM_ACTION__DEFELEM_ADD", 3 },
  { "DEFELEM_DROP", "PG_QUERY__DEF_ELEM_ACTION__DEFELEM_DROP", 4 },
};
static const ProtobufCIntRange pg_query__def_elem_action__value_ranges[] = {
{0, 0},{0, 5}
};
static const ProtobufCEnumValueIndex pg_query__def_elem_action__enum_values_by_name[5] =
{
  { "DEFELEM_ADD", 3 },
  { "DEFELEM_DROP", 4 },
  { "DEFELEM_SET", 2 },
  { "DEFELEM_UNSPEC", 1 },
  { "DEF_ELEM_ACTION_UNDEFINED", 0 },
};
const ProtobufCEnumDescriptor pg_query__def_elem_action__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.DefElemAction",
  "DefElemAction",
  "PgQuery__DefElemAction",
  "pg_query",
  5,
  pg_query__def_elem_action__enum_values_by_number,
  5,
  pg_query__def_elem_action__enum_values_by_name,
  1,
  pg_query__def_elem_action__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__partition_strategy__enum_values_by_number[4] =
{
  { "PARTITION_STRATEGY_UNDEFINED", "PG_QUERY__PARTITION_STRATEGY__PARTITION_STRATEGY_UNDEFINED", 0 },
  { "PARTITION_STRATEGY_LIST", "PG_QUERY__PARTITION_STRATEGY__PARTITION_STRATEGY_LIST", 1 },
  { "PARTITION_STRATEGY_RANGE", "PG_QUERY__PARTITION_STRATEGY__PARTITION_STRATEGY_RANGE", 2 },
  { "PARTITION_STRATEGY_HASH", "PG_QUERY__PARTITION_STRATEGY__PARTITION_STRATEGY_HASH", 3 },
};
static const ProtobufCIntRange pg_query__partition_strategy__value_ranges[] = {
{0, 0},{0, 4}
};
static const ProtobufCEnumValueIndex pg_query__partition_strategy__enum_values_by_name[4] =
{
  { "PARTITION_STRATEGY_HASH", 3 },
  { "PARTITION_STRATEGY_LIST", 1 },
  { "PARTITION_STRATEGY_RANGE", 2 },
  { "PARTITION_STRATEGY_UNDEFINED", 0 },
};
const ProtobufCEnumDescriptor pg_query__partition_strategy__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.PartitionStrategy",
  "PartitionStrategy",
  "PgQuery__PartitionStrategy",
  "pg_query",
  4,
  pg_query__partition_strategy__enum_values_by_number,
  4,
  pg_query__partition_strategy__enum_values_by_name,
  1,
  pg_query__partition_strategy__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__partition_range_datum_kind__enum_values_by_number[4] =
{
  { "PARTITION_RANGE_DATUM_KIND_UNDEFINED", "PG_QUERY__PARTITION_RANGE_DATUM_KIND__PARTITION_RANGE_DATUM_KIND_UNDEFINED", 0 },
  { "PARTITION_RANGE_DATUM_MINVALUE", "PG_QUERY__PARTITION_RANGE_DATUM_KIND__PARTITION_RANGE_DATUM_MINVALUE", 1 },
  { "PARTITION_RANGE_DATUM_VALUE", "PG_QUERY__PARTITION_RANGE_DATUM_KIND__PARTITION_RANGE_DATUM_VALUE", 2 },
  { "PARTITION_RANGE_DATUM_MAXVALUE", "PG_QUERY__PARTITION_RANGE_DATUM_KIND__PARTITION_RANGE_DATUM_MAXVALUE", 3 },
};
static const ProtobufCIntRange pg_query__partition_range_datum_kind__value_ranges[] = {
{0, 0},{0, 4}
};
static const ProtobufCEnumValueIndex pg_query__partition_range_datum_kind__enum_values_by_name[4] =
{
  { "PARTITION_RANGE_DATUM_KIND_UNDEFINED", 0 },
  { "PARTITION_RANGE_DATUM_MAXVALUE", 3 },
  { "PARTITION_RANGE_DATUM_MINVALUE", 1 },
  { "PARTITION_RANGE_DATUM_VALUE", 2 },
};
const ProtobufCEnumDescriptor pg_query__partition_range_datum_kind__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.PartitionRangeDatumKind",
  "PartitionRangeDatumKind",
  "PgQuery__PartitionRangeDatumKind",
  "pg_query",
  4,
  pg_query__partition_range_datum_kind__enum_values_by_number,
  4,
  pg_query__partition_range_datum_kind__enum_values_by_name,
  1,
  pg_query__partition_range_datum_kind__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__rtekind__enum_values_by_number[10] =
{
  { "RTEKIND_UNDEFINED", "PG_QUERY__RTEKIND__RTEKIND_UNDEFINED", 0 },
  { "RTE_RELATION", "PG_QUERY__RTEKIND__RTE_RELATION", 1 },
  { "RTE_SUBQUERY", "PG_QUERY__RTEKIND__RTE_SUBQUERY", 2 },
  { "RTE_JOIN", "PG_QUERY__RTEKIND__RTE_JOIN", 3 },
  { "RTE_FUNCTION", "PG_QUERY__RTEKIND__RTE_FUNCTION", 4 },
  { "RTE_TABLEFUNC", "PG_QUERY__RTEKIND__RTE_TABLEFUNC", 5 },
  { "RTE_VALUES", "PG_QUERY__RTEKIND__RTE_VALUES", 6 },
  { "RTE_CTE", "PG_QUERY__RTEKIND__RTE_CTE", 7 },
  { "RTE_NAMEDTUPLESTORE", "PG_QUERY__RTEKIND__RTE_NAMEDTUPLESTORE", 8 },
  { "RTE_RESULT", "PG_QUERY__RTEKIND__RTE_RESULT", 9 },
};
static const ProtobufCIntRange pg_query__rtekind__value_ranges[] = {
{0, 0},{0, 10}
};
static const ProtobufCEnumValueIndex pg_query__rtekind__enum_values_by_name[10] =
{
  { "RTEKIND_UNDEFINED", 0 },
  { "RTE_CTE", 7 },
  { "RTE_FUNCTION", 4 },
  { "RTE_JOIN", 3 },
  { "RTE_NAMEDTUPLESTORE", 8 },
  { "RTE_RELATION", 1 },
  { "RTE_RESULT", 9 },
  { "RTE_SUBQUERY", 2 },
  { "RTE_TABLEFUNC", 5 },
  { "RTE_VALUES", 6 },
};
const ProtobufCEnumDescriptor pg_query__rtekind__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.RTEKind",
  "RTEKind",
  "PgQuery__RTEKind",
  "pg_query",
  10,
  pg_query__rtekind__enum_values_by_number,
  10,
  pg_query__rtekind__enum_values_by_name,
  1,
  pg_query__rtekind__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__wcokind__enum_values_by_number[7] =
{
  { "WCOKIND_UNDEFINED", "PG_QUERY__WCOKIND__WCOKIND_UNDEFINED", 0 },
  { "WCO_VIEW_CHECK", "PG_QUERY__WCOKIND__WCO_VIEW_CHECK", 1 },
  { "WCO_RLS_INSERT_CHECK", "PG_QUERY__WCOKIND__WCO_RLS_INSERT_CHECK", 2 },
  { "WCO_RLS_UPDATE_CHECK", "PG_QUERY__WCOKIND__WCO_RLS_UPDATE_CHECK", 3 },
  { "WCO_RLS_CONFLICT_CHECK", "PG_QUERY__WCOKIND__WCO_RLS_CONFLICT_CHECK", 4 },
  { "WCO_RLS_MERGE_UPDATE_CHECK", "PG_QUERY__WCOKIND__WCO_RLS_MERGE_UPDATE_CHECK", 5 },
  { "WCO_RLS_MERGE_DELETE_CHECK", "PG_QUERY__WCOKIND__WCO_RLS_MERGE_DELETE_CHECK", 6 },
};
static const ProtobufCIntRange pg_query__wcokind__value_ranges[] = {
{0, 0},{0, 7}
};
static const ProtobufCEnumValueIndex pg_query__wcokind__enum_values_by_name[7] =
{
  { "WCOKIND_UNDEFINED", 0 },
  { "WCO_RLS_CONFLICT_CHECK", 4 },
  { "WCO_RLS_INSERT_CHECK", 2 },
  { "WCO_RLS_MERGE_DELETE_CHECK", 6 },
  { "WCO_RLS_MERGE_UPDATE_CHECK", 5 },
  { "WCO_RLS_UPDATE_CHECK", 3 },
  { "WCO_VIEW_CHECK", 1 },
};
const ProtobufCEnumDescriptor pg_query__wcokind__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.WCOKind",
  "WCOKind",
  "PgQuery__WCOKind",
  "pg_query",
  7,
  pg_query__wcokind__enum_values_by_number,
  7,
  pg_query__wcokind__enum_values_by_name,
  1,
  pg_query__wcokind__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__grouping_set_kind__enum_values_by_number[6] =
{
  { "GROUPING_SET_KIND_UNDEFINED", "PG_QUERY__GROUPING_SET_KIND__GROUPING_SET_KIND_UNDEFINED", 0 },
  { "GROUPING_SET_EMPTY", "PG_QUERY__GROUPING_SET_KIND__GROUPING_SET_EMPTY", 1 },
  { "GROUPING_SET_SIMPLE", "PG_QUERY__GROUPING_SET_KIND__GROUPING_SET_SIMPLE", 2 },
  { "GROUPING_SET_ROLLUP", "PG_QUERY__GROUPING_SET_KIND__GROUPING_SET_ROLLUP", 3 },
  { "GROUPING_SET_CUBE", "PG_QUERY__GROUPING_SET_KIND__GROUPING_SET_CUBE", 4 },
  { "GROUPING_SET_SETS", "PG_QUERY__GROUPING_SET_KIND__GROUPING_SET_SETS", 5 },
};
static const ProtobufCIntRange pg_query__grouping_set_kind__value_ranges[] = {
{0, 0},{0, 6}
};
static const ProtobufCEnumValueIndex pg_query__grouping_set_kind__enum_values_by_name[6] =
{
  { "GROUPING_SET_CUBE", 4 },
  { "GROUPING_SET_EMPTY", 1 },
  { "GROUPING_SET_KIND_UNDEFINED", 0 },
  { "GROUPING_SET_ROLLUP", 3 },
  { "GROUPING_SET_SETS", 5 },
  { "GROUPING_SET_SIMPLE", 2 },
};
const ProtobufCEnumDescriptor pg_query__grouping_set_kind__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.GroupingSetKind",
  "GroupingSetKind",
  "PgQuery__GroupingSetKind",
  "pg_query",
  6,
  pg_query__grouping_set_kind__enum_values_by_number,
  6,
  pg_query__grouping_set_kind__enum_values_by_name,
  1,
  pg_query__grouping_set_kind__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__ctematerialize__enum_values_by_number[4] =
{
  { "CTEMATERIALIZE_UNDEFINED", "PG_QUERY__CTEMATERIALIZE__CTEMATERIALIZE_UNDEFINED", 0 },
  { "CTEMaterializeDefault", "PG_QUERY__CTEMATERIALIZE__CTEMaterializeDefault", 1 },
  { "CTEMaterializeAlways", "PG_QUERY__CTEMATERIALIZE__CTEMaterializeAlways", 2 },
  { "CTEMaterializeNever", "PG_QUERY__CTEMATERIALIZE__CTEMaterializeNever", 3 },
};
static const ProtobufCIntRange pg_query__ctematerialize__value_ranges[] = {
{0, 0},{0, 4}
};
static const ProtobufCEnumValueIndex pg_query__ctematerialize__enum_values_by_name[4] =
{
  { "CTEMATERIALIZE_UNDEFINED", 0 },
  { "CTEMaterializeAlways", 2 },
  { "CTEMaterializeDefault", 1 },
  { "CTEMaterializeNever", 3 },
};
const ProtobufCEnumDescriptor pg_query__ctematerialize__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.CTEMaterialize",
  "CTEMaterialize",
  "PgQuery__CTEMaterialize",
  "pg_query",
  4,
  pg_query__ctematerialize__enum_values_by_number,
  4,
  pg_query__ctematerialize__enum_values_by_name,
  1,
  pg_query__ctematerialize__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__json_quotes__enum_values_by_number[4] =
{
  { "JSON_QUOTES_UNDEFINED", "PG_QUERY__JSON_QUOTES__JSON_QUOTES_UNDEFINED", 0 },
  { "JS_QUOTES_UNSPEC", "PG_QUERY__JSON_QUOTES__JS_QUOTES_UNSPEC", 1 },
  { "JS_QUOTES_KEEP", "PG_QUERY__JSON_QUOTES__JS_QUOTES_KEEP", 2 },
  { "JS_QUOTES_OMIT", "PG_QUERY__JSON_QUOTES__JS_QUOTES_OMIT", 3 },
};
static const ProtobufCIntRange pg_query__json_quotes__value_ranges[] = {
{0, 0},{0, 4}
};
static const ProtobufCEnumValueIndex pg_query__json_quotes__enum_values_by_name[4] =
{
  { "JSON_QUOTES_UNDEFINED", 0 },
  { "JS_QUOTES_KEEP", 2 },
  { "JS_QUOTES_OMIT", 3 },
  { "JS_QUOTES_UNSPEC", 1 },
};
const ProtobufCEnumDescriptor pg_query__json_quotes__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.JsonQuotes",
  "JsonQuotes",
  "PgQuery__JsonQuotes",
  "pg_query",
  4,
  pg_query__json_quotes__enum_values_by_number,
  4,
  pg_query__json_quotes__enum_values_by_name,
  1,
  pg_query__json_quotes__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__json_table_column_type__enum_values_by_number[6] =
{
  { "JSON_TABLE_COLUMN_TYPE_UNDEFINED", "PG_QUERY__JSON_TABLE_COLUMN_TYPE__JSON_TABLE_COLUMN_TYPE_UNDEFINED", 0 },
  { "JTC_FOR_ORDINALITY", "PG_QUERY__JSON_TABLE_COLUMN_TYPE__JTC_FOR_ORDINALITY", 1 },
  { "JTC_REGULAR", "PG_QUERY__JSON_TABLE_COLUMN_TYPE__JTC_REGULAR", 2 },
  { "JTC_EXISTS", "PG_QUERY__JSON_TABLE_COLUMN_TYPE__JTC_EXISTS", 3 },
  { "JTC_FORMATTED", "PG_QUERY__JSON_TABLE_COLUMN_TYPE__JTC_FORMATTED", 4 },
  { "JTC_NESTED", "PG_QUERY__JSON_TABLE_COLUMN_TYPE__JTC_NESTED", 5 },
};
static const ProtobufCIntRange pg_query__json_table_column_type__value_ranges[] = {
{0, 0},{0, 6}
};
static const ProtobufCEnumValueIndex pg_query__json_table_column_type__enum_values_by_name[6] =
{
  { "JSON_TABLE_COLUMN_TYPE_UNDEFINED", 0 },
  { "JTC_EXISTS", 3 },
  { "JTC_FORMATTED", 4 },
  { "JTC_FOR_ORDINALITY", 1 },
  { "JTC_NESTED", 5 },
  { "JTC_REGULAR", 2 },
};
const ProtobufCEnumDescriptor pg_query__json_table_column_type__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.JsonTableColumnType",
  "JsonTableColumnType",
  "PgQuery__JsonTableColumnType",
  "pg_query",
  6,
  pg_query__json_table_column_type__enum_values_by_number,
  6,
  pg_query__json_table_column_type__enum_values_by_name,
  1,
  pg_query__json_table_column_type__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__set_operation__enum_values_by_number[5] =
{
  { "SET_OPERATION_UNDEFINED", "PG_QUERY__SET_OPERATION__SET_OPERATION_UNDEFINED", 0 },
  { "SETOP_NONE", "PG_QUERY__SET_OPERATION__SETOP_NONE", 1 },
  { "SETOP_UNION", "PG_QUERY__SET_OPERATION__SETOP_UNION", 2 },
  { "SETOP_INTERSECT", "PG_QUERY__SET_OPERATION__SETOP_INTERSECT", 3 },
  { "SETOP_EXCEPT", "PG_QUERY__SET_OPERATION__SETOP_EXCEPT", 4 },
};
static const ProtobufCIntRange pg_query__set_operation__value_ranges[] = {
{0, 0},{0, 5}
};
static const ProtobufCEnumValueIndex pg_query__set_operation__enum_values_by_name[5] =
{
  { "SETOP_EXCEPT", 4 },
  { "SETOP_INTERSECT", 3 },
  { "SETOP_NONE", 1 },
  { "SETOP_UNION", 2 },
  { "SET_OPERATION_UNDEFINED", 0 },
};
const ProtobufCEnumDescriptor pg_query__set_operation__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.SetOperation",
  "SetOperation",
  "PgQuery__SetOperation",
  "pg_query",
  5,
  pg_query__set_operation__enum_values_by_number,
  5,
  pg_query__set_operation__enum_values_by_name,
  1,
  pg_query__set_operation__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__object_type__enum_values_by_number[53] =
{
  { "OBJECT_TYPE_UNDEFINED", "PG_QUERY__OBJECT_TYPE__OBJECT_TYPE_UNDEFINED", 0 },
  { "OBJECT_ACCESS_METHOD", "PG_QUERY__OBJECT_TYPE__OBJECT_ACCESS_METHOD", 1 },
  { "OBJECT_AGGREGATE", "PG_QUERY__OBJECT_TYPE__OBJECT_AGGREGATE", 2 },
  { "OBJECT_AMOP", "PG_QUERY__OBJECT_TYPE__OBJECT_AMOP", 3 },
  { "OBJECT_AMPROC", "PG_QUERY__OBJECT_TYPE__OBJECT_AMPROC", 4 },
  { "OBJECT_ATTRIBUTE", "PG_QUERY__OBJECT_TYPE__OBJECT_ATTRIBUTE", 5 },
  { "OBJECT_CAST", "PG_QUERY__OBJECT_TYPE__OBJECT_CAST", 6 },
  { "OBJECT_COLUMN", "PG_QUERY__OBJECT_TYPE__OBJECT_COLUMN", 7 },
  { "OBJECT_COLLATION", "PG_QUERY__OBJECT_TYPE__OBJECT_COLLATION", 8 },
  { "OBJECT_CONVERSION", "PG_QUERY__OBJECT_TYPE__OBJECT_CONVERSION", 9 },
  { "OBJECT_DATABASE", "PG_QUERY__OBJECT_TYPE__OBJECT_DATABASE", 10 },
  { "OBJECT_DEFAULT", "PG_QUERY__OBJECT_TYPE__OBJECT_DEFAULT", 11 },
  { "OBJECT_DEFACL", "PG_QUERY__OBJECT_TYPE__OBJECT_DEFACL", 12 },
  { "OBJECT_DOMAIN", "PG_QUERY__OBJECT_TYPE__OBJECT_DOMAIN", 13 },
  { "OBJECT_DOMCONSTRAINT", "PG_QUERY__OBJECT_TYPE__OBJECT_DOMCONSTRAINT", 14 },
  { "OBJECT_EVENT_TRIGGER", "PG_QUERY__OBJECT_TYPE__OBJECT_EVENT_TRIGGER", 15 },
  { "OBJECT_EXTENSION", "PG_QUERY__OBJECT_TYPE__OBJECT_EXTENSION", 16 },
  { "OBJECT_FDW", "PG_QUERY__OBJECT_TYPE__OBJECT_FDW", 17 },
  { "OBJECT_FOREIGN_SERVER", "PG_QUERY__OBJECT_TYPE__OBJECT_FOREIGN_SERVER", 18 },
  { "OBJECT_FOREIGN_TABLE", "PG_QUERY__OBJECT_TYPE__OBJECT_FOREIGN_TABLE", 19 },
  { "OBJECT_FUNCTION", "PG_QUERY__OBJECT_TYPE__OBJECT_FUNCTION", 20 },
  { "OBJECT_INDEX", "PG_QUERY__OBJECT_TYPE__OBJECT_INDEX", 21 },
  { "OBJECT_LANGUAGE", "PG_QUERY__OBJECT_TYPE__OBJECT_LANGUAGE", 22 },
  { "OBJECT_LARGEOBJECT", "PG_QUERY__OBJECT_TYPE__OBJECT_LARGEOBJECT", 23 },
  { "OBJECT_MATVIEW", "PG_QUERY__OBJECT_TYPE__OBJECT_MATVIEW", 24 },
  { "OBJECT_OPCLASS", "PG_QUERY__OBJECT_TYPE__OBJECT_OPCLASS", 25 },
  { "OBJECT_OPERATOR", "PG_QUERY__OBJECT_TYPE__OBJECT_OPERATOR", 26 },
  { "OBJECT_OPFAMILY", "PG_QUERY__OBJECT_TYPE__OBJECT_OPFAMILY", 27 },
  { "OBJECT_PARAMETER_ACL", "PG_QUERY__OBJECT_TYPE__OBJECT_PARAMETER_ACL", 28 },
  { "OBJECT_POLICY", "PG_QUERY__OBJECT_TYPE__OBJECT_POLICY", 29 },
  { "OBJECT_PROCEDURE", "PG_QUERY__OBJECT_TYPE__OBJECT_PROCEDURE", 30 },
  { "OBJECT_PUBLICATION", "PG_QUERY__OBJECT_TYPE__OBJECT_PUBLICATION", 31 },
  { "OBJECT_PUBLICATION_NAMESPACE", "PG_QUERY__OBJECT_TYPE__OBJECT_PUBLICATION_NAMESPACE", 32 },
  { "OBJECT_PUBLICATION_REL", "PG_QUERY__OBJECT_TYPE__OBJECT_PUBLICATION_REL", 33 },
  { "OBJECT_ROLE", "PG_QUERY__OBJECT_TYPE__OBJECT_ROLE", 34 },
  { "OBJECT_ROUTINE", "PG_QUERY__OBJECT_TYPE__OBJECT_ROUTINE", 35 },
  { "OBJECT_RULE", "PG_QUERY__OBJECT_TYPE__OBJECT_RULE", 36 },
  { "OBJECT_SCHEMA", "PG_QUERY__OBJECT_TYPE__OBJECT_SCHEMA", 37 },
  { "OBJECT_SEQUENCE", "PG_QUERY__OBJECT_TYPE__OBJECT_SEQUENCE", 38 },
  { "OBJECT_SUBSCRIPTION", "PG_QUERY__OBJECT_TYPE__OBJECT_SUBSCRIPTION", 39 },
  { "OBJECT_STATISTIC_EXT", "PG_QUERY__OBJECT_TYPE__OBJECT_STATISTIC_EXT", 40 },
  { "OBJECT_TABCONSTRAINT", "PG_QUERY__OBJECT_TYPE__OBJECT_TABCONSTRAINT", 41 },
  { "OBJECT_TABLE", "PG_QUERY__OBJECT_TYPE__OBJECT_TABLE", 42 },
  { "OBJECT_TABLESPACE", "PG_QUERY__OBJECT_TYPE__OBJECT_TABLESPACE", 43 },
  { "OBJECT_TRANSFORM", "PG_QUERY__OBJECT_TYPE__OBJECT_TRANSFORM", 44 },
  { "OBJECT_TRIGGER", "PG_QUERY__OBJECT_TYPE__OBJECT_TRIGGER", 45 },
  { "OBJECT_TSCONFIGURATION", "PG_QUERY__OBJECT_TYPE__OBJECT_TSCONFIGURATION", 46 },
  { "OBJECT_TSDICTIONARY", "PG_QUERY__OBJECT_TYPE__OBJECT_TSDICTIONARY", 47 },
  { "OBJECT_TSPARSER", "PG_QUERY__OBJECT_TYPE__OBJECT_TSPARSER", 48 },
  { "OBJECT_TSTEMPLATE", "PG_QUERY__OBJECT_TYPE__OBJECT_TSTEMPLATE", 49 },
  { "OBJECT_TYPE", "PG_QUERY__OBJECT_TYPE__OBJECT_TYPE", 50 },
  { "OBJECT_USER_MAPPING", "PG_QUERY__OBJECT_TYPE__OBJECT_USER_MAPPING", 51 },
  { "OBJECT_VIEW", "PG_QUERY__OBJECT_TYPE__OBJECT_VIEW", 52 },
};
static const ProtobufCIntRange pg_query__object_type__value_ranges[] = {
{0, 0},{0, 53}
};
static const ProtobufCEnumValueIndex pg_query__object_type__enum_values_by_name[53] =
{
  { "OBJECT_ACCESS_METHOD", 1 },
  { "OBJECT_AGGREGATE", 2 },
  { "OBJECT_AMOP", 3 },
  { "OBJECT_AMPROC", 4 },
  { "OBJECT_ATTRIBUTE", 5 },
  { "OBJECT_CAST", 6 },
  { "OBJECT_COLLATION", 8 },
  { "OBJECT_COLUMN", 7 },
  { "OBJECT_CONVERSION", 9 },
  { "OBJECT_DATABASE", 10 },
  { "OBJECT_DEFACL", 12 },
  { "OBJECT_DEFAULT", 11 },
  { "OBJECT_DOMAIN", 13 },
  { "OBJECT_DOMCONSTRAINT", 14 },
  { "OBJECT_EVENT_TRIGGER", 15 },
  { "OBJECT_EXTENSION", 16 },
  { "OBJECT_FDW", 17 },
  { "OBJECT_FOREIGN_SERVER", 18 },
  { "OBJECT_FOREIGN_TABLE", 19 },
  { "OBJECT_FUNCTION", 20 },
  { "OBJECT_INDEX", 21 },
  { "OBJECT_LANGUAGE", 22 },
  { "OBJECT_LARGEOBJECT", 23 },
  { "OBJECT_MATVIEW", 24 },
  { "OBJECT_OPCLASS", 25 },
  { "OBJECT_OPERATOR", 26 },
  { "OBJECT_OPFAMILY", 27 },
  { "OBJECT_PARAMETER_ACL", 28 },
  { "OBJECT_POLICY", 29 },
  { "OBJECT_PROCEDURE", 30 },
  { "OBJECT_PUBLICATION", 31 },
  { "OBJECT_PUBLICATION_NAMESPACE", 32 },
  { "OBJECT_PUBLICATION_REL", 33 },
  { "OBJECT_ROLE", 34 },
  { "OBJECT_ROUTINE", 35 },
  { "OBJECT_RULE", 36 },
  { "OBJECT_SCHEMA", 37 },
  { "OBJECT_SEQUENCE", 38 },
  { "OBJECT_STATISTIC_EXT", 40 },
  { "OBJECT_SUBSCRIPTION", 39 },
  { "OBJECT_TABCONSTRAINT", 41 },
  { "OBJECT_TABLE", 42 },
  { "OBJECT_TABLESPACE", 43 },
  { "OBJECT_TRANSFORM", 44 },
  { "OBJECT_TRIGGER", 45 },
  { "OBJECT_TSCONFIGURATION", 46 },
  { "OBJECT_TSDICTIONARY", 47 },
  { "OBJECT_TSPARSER", 48 },
  { "OBJECT_TSTEMPLATE", 49 },
  { "OBJECT_TYPE", 50 },
  { "OBJECT_TYPE_UNDEFINED", 0 },
  { "OBJECT_USER_MAPPING", 51 },
  { "OBJECT_VIEW", 52 },
};
const ProtobufCEnumDescriptor pg_query__object_type__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.ObjectType",
  "ObjectType",
  "PgQuery__ObjectType",
  "pg_query",
  53,
  pg_query__object_type__enum_values_by_number,
  53,
  pg_query__object_type__enum_values_by_name,
  1,
  pg_query__object_type__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__drop_behavior__enum_values_by_number[3] =
{
  { "DROP_BEHAVIOR_UNDEFINED", "PG_QUERY__DROP_BEHAVIOR__DROP_BEHAVIOR_UNDEFINED", 0 },
  { "DROP_RESTRICT", "PG_QUERY__DROP_BEHAVIOR__DROP_RESTRICT", 1 },
  { "DROP_CASCADE", "PG_QUERY__DROP_BEHAVIOR__DROP_CASCADE", 2 },
};
static const ProtobufCIntRange pg_query__drop_behavior__value_ranges[] = {
{0, 0},{0, 3}
};
static const ProtobufCEnumValueIndex pg_query__drop_behavior__enum_values_by_name[3] =
{
  { "DROP_BEHAVIOR_UNDEFINED", 0 },
  { "DROP_CASCADE", 2 },
  { "DROP_RESTRICT", 1 },
};
const ProtobufCEnumDescriptor pg_query__drop_behavior__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.DropBehavior",
  "DropBehavior",
  "PgQuery__DropBehavior",
  "pg_query",
  3,
  pg_query__drop_behavior__enum_values_by_number,
  3,
  pg_query__drop_behavior__enum_values_by_name,
  1,
  pg_query__drop_behavior__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__alter_table_type__enum_values_by_number[68] =
{
  { "ALTER_TABLE_TYPE_UNDEFINED", "PG_QUERY__ALTER_TABLE_TYPE__ALTER_TABLE_TYPE_UNDEFINED", 0 },
  { "AT_AddColumn", "PG_QUERY__ALTER_TABLE_TYPE__AT_AddColumn", 1 },
  { "AT_AddColumnToView", "PG_QUERY__ALTER_TABLE_TYPE__AT_AddColumnToView", 2 },
  { "AT_ColumnDefault", "PG_QUERY__ALTER_TABLE_TYPE__AT_ColumnDefault", 3 },
  { "AT_CookedColumnDefault", "PG_QUERY__ALTER_TABLE_TYPE__AT_CookedColumnDefault", 4 },
  { "AT_DropNotNull", "PG_QUERY__ALTER_TABLE_TYPE__AT_DropNotNull", 5 },
  { "AT_SetNotNull", "PG_QUERY__ALTER_TABLE_TYPE__AT_SetNotNull", 6 },
  { "AT_SetExpression", "PG_QUERY__ALTER_TABLE_TYPE__AT_SetExpression", 7 },
  { "AT_DropExpression", "PG_QUERY__ALTER_TABLE_TYPE__AT_DropExpression", 8 },
  { "AT_CheckNotNull", "PG_QUERY__ALTER_TABLE_TYPE__AT_CheckNotNull", 9 },
  { "AT_SetStatistics", "PG_QUERY__ALTER_TABLE_TYPE__AT_SetStatistics", 10 },
  { "AT_SetOptions", "PG_QUERY__ALTER_TABLE_TYPE__AT_SetOptions", 11 },
  { "AT_ResetOptions", "PG_QUERY__ALTER_TABLE_TYPE__AT_ResetOptions", 12 },
  { "AT_SetStorage", "PG_QUERY__ALTER_TABLE_TYPE__AT_SetStorage", 13 },
  { "AT_SetCompression", "PG_QUERY__ALTER_TABLE_TYPE__AT_SetCompression", 14 },
  { "AT_DropColumn", "PG_QUERY__ALTER_TABLE_TYPE__AT_DropColumn", 15 },
  { "AT_AddIndex", "PG_QUERY__ALTER_TABLE_TYPE__AT_AddIndex", 16 },
  { "AT_ReAddIndex", "PG_QUERY__ALTER_TABLE_TYPE__AT_ReAddIndex", 17 },
  { "AT_AddConstraint", "PG_QUERY__ALTER_TABLE_TYPE__AT_AddConstraint", 18 },
  { "AT_ReAddConstraint", "PG_QUERY__ALTER_TABLE_TYPE__AT_ReAddConstraint", 19 },
  { "AT_ReAddDomainConstraint", "PG_QUERY__ALTER_TABLE_TYPE__AT_ReAddDomainConstraint", 20 },
  { "AT_AlterConstraint", "PG_QUERY__ALTER_TABLE_TYPE__AT_AlterConstraint", 21 },
  { "AT_ValidateConstraint", "PG_QUERY__ALTER_TABLE_TYPE__AT_ValidateConstraint", 22 },
  { "AT_AddIndexConstraint", "PG_QUERY__ALTER_TABLE_TYPE__AT_AddIndexConstraint", 23 },
  { "AT_DropConstraint", "PG_QUERY__ALTER_TABLE_TYPE__AT_DropConstraint", 24 },
  { "AT_ReAddComment", "PG_QUERY__ALTER_TABLE_TYPE__AT_ReAddComment", 25 },
  { "AT_AlterColumnType", "PG_QUERY__ALTER_TABLE_TYPE__AT_AlterColumnType", 26 },
  { "AT_AlterColumnGenericOptions", "PG_QUERY__ALTER_TABLE_TYPE__AT_AlterColumnGenericOptions", 27 },
  { "AT_ChangeOwner", "PG_QUERY__ALTER_TABLE_TYPE__AT_ChangeOwner", 28 },
  { "AT_ClusterOn", "PG_QUERY__ALTER_TABLE_TYPE__AT_ClusterOn", 29 },
  { "AT_DropCluster", "PG_QUERY__ALTER_TABLE_TYPE__AT_DropCluster", 30 },
  { "AT_SetLogged", "PG_QUERY__ALTER_TABLE_TYPE__AT_SetLogged", 31 },
  { "AT_SetUnLogged", "PG_QUERY__ALTER_TABLE_TYPE__AT_SetUnLogged", 32 },
  { "AT_DropOids", "PG_QUERY__ALTER_TABLE_TYPE__AT_DropOids", 33 },
  { "AT_SetAccessMethod", "PG_QUERY__ALTER_TABLE_TYPE__AT_SetAccessMethod", 34 },
  { "AT_SetTableSpace", "PG_QUERY__ALTER_TABLE_TYPE__AT_SetTableSpace", 35 },
  { "AT_SetRelOptions", "PG_QUERY__ALTER_TABLE_TYPE__AT_SetRelOptions", 36 },
  { "AT_ResetRelOptions", "PG_QUERY__ALTER_TABLE_TYPE__AT_ResetRelOptions", 37 },
  { "AT_ReplaceRelOptions", "PG_QUERY__ALTER_TABLE_TYPE__AT_ReplaceRelOptions", 38 },
  { "AT_EnableTrig", "PG_QUERY__ALTER_TABLE_TYPE__AT_EnableTrig", 39 },
  { "AT_EnableAlwaysTrig", "PG_QUERY__ALTER_TABLE_TYPE__AT_EnableAlwaysTrig", 40 },
  { "AT_EnableReplicaTrig", "PG_QUERY__ALTER_TABLE_TYPE__AT_EnableReplicaTrig", 41 },
  { "AT_DisableTrig", "PG_QUERY__ALTER_TABLE_TYPE__AT_DisableTrig", 42 },
  { "AT_EnableTrigAll", "PG_QUERY__ALTER_TABLE_TYPE__AT_EnableTrigAll", 43 },
  { "AT_DisableTrigAll", "PG_QUERY__ALTER_TABLE_TYPE__AT_DisableTrigAll", 44 },
  { "AT_EnableTrigUser", "PG_QUERY__ALTER_TABLE_TYPE__AT_EnableTrigUser", 45 },
  { "AT_DisableTrigUser", "PG_QUERY__ALTER_TABLE_TYPE__AT_DisableTrigUser", 46 },
  { "AT_EnableRule", "PG_QUERY__ALTER_TABLE_TYPE__AT_EnableRule", 47 },
  { "AT_EnableAlwaysRule", "PG_QUERY__ALTER_TABLE_TYPE__AT_EnableAlwaysRule", 48 },
  { "AT_EnableReplicaRule", "PG_QUERY__ALTER_TABLE_TYPE__AT_EnableReplicaRule", 49 },
  { "AT_DisableRule", "PG_QUERY__ALTER_TABLE_TYPE__AT_DisableRule", 50 },
  { "AT_AddInherit", "PG_QUERY__ALTER_TABLE_TYPE__AT_AddInherit", 51 },
  { "AT_DropInherit", "PG_QUERY__ALTER_TABLE_TYPE__AT_DropInherit", 52 },
  { "AT_AddOf", "PG_QUERY__ALTER_TABLE_TYPE__AT_AddOf", 53 },
  { "AT_DropOf", "PG_QUERY__ALTER_TABLE_TYPE__AT_DropOf", 54 },
  { "AT_ReplicaIdentity", "PG_QUERY__ALTER_TABLE_TYPE__AT_ReplicaIdentity", 55 },
  { "AT_EnableRowSecurity", "PG_QUERY__ALTER_TABLE_TYPE__AT_EnableRowSecurity", 56 },
  { "AT_DisableRowSecurity", "PG_QUERY__ALTER_TABLE_TYPE__AT_DisableRowSecurity", 57 },
  { "AT_ForceRowSecurity", "PG_QUERY__ALTER_TABLE_TYPE__AT_ForceRowSecurity", 58 },
  { "AT_NoForceRowSecurity", "PG_QUERY__ALTER_TABLE_TYPE__AT_NoForceRowSecurity", 59 },
  { "AT_GenericOptions", "PG_QUERY__ALTER_TABLE_TYPE__AT_GenericOptions", 60 },
  { "AT_AttachPartition", "PG_QUERY__ALTER_TABLE_TYPE__AT_AttachPartition", 61 },
  { "AT_DetachPartition", "PG_QUERY__ALTER_TABLE_TYPE__AT_DetachPartition", 62 },
  { "AT_DetachPartitionFinalize", "PG_QUERY__ALTER_TABLE_TYPE__AT_DetachPartitionFinalize", 63 },
  { "AT_AddIdentity", "PG_QUERY__ALTER_TABLE_TYPE__AT_AddIdentity", 64 },
  { "AT_SetIdentity", "PG_QUERY__ALTER_TABLE_TYPE__AT_SetIdentity", 65 },
  { "AT_DropIdentity", "PG_QUERY__ALTER_TABLE_TYPE__AT_DropIdentity", 66 },
  { "AT_ReAddStatistics", "PG_QUERY__ALTER_TABLE_TYPE__AT_ReAddStatistics", 67 },
};
static const ProtobufCIntRange pg_query__alter_table_type__value_ranges[] = {
{0, 0},{0, 68}
};
static const ProtobufCEnumValueIndex pg_query__alter_table_type__enum_values_by_name[68] =
{
  { "ALTER_TABLE_TYPE_UNDEFINED", 0 },
  { "AT_AddColumn", 1 },
  { "AT_AddColumnToView", 2 },
  { "AT_AddConstraint", 18 },
  { "AT_AddIdentity", 64 },
  { "AT_AddIndex", 16 },
  { "AT_AddIndexConstraint", 23 },
  { "AT_AddInherit", 51 },
  { "AT_AddOf", 53 },
  { "AT_AlterColumnGenericOptions", 27 },
  { "AT_AlterColumnType", 26 },
  { "AT_AlterConstraint", 21 },
  { "AT_AttachPartition", 61 },
  { "AT_ChangeOwner", 28 },
  { "AT_CheckNotNull", 9 },
  { "AT_ClusterOn", 29 },
  { "AT_ColumnDefault", 3 },
  { "AT_CookedColumnDefault", 4 },
  { "AT_DetachPartition", 62 },
  { "AT_DetachPartitionFinalize", 63 },
  { "AT_DisableRowSecurity", 57 },
  { "AT_DisableRule", 50 },
  { "AT_DisableTrig", 42 },
  { "AT_DisableTrigAll", 44 },
  { "AT_DisableTrigUser", 46 },
  { "AT_DropCluster", 30 },
  { "AT_DropColumn", 15 },
  { "AT_DropConstraint", 24 },
  { "AT_DropExpression", 8 },
  { "AT_DropIdentity", 66 },
  { "AT_DropInherit", 52 },
  { "AT_DropNotNull", 5 },
  { "AT_DropOf", 54 },
  { "AT_DropOids", 33 },
  { "AT_EnableAlwaysRule", 48 },
  { "AT_EnableAlwaysTrig", 40 },
  { "AT_EnableReplicaRule", 49 },
  { "AT_EnableReplicaTrig", 41 },
  { "AT_EnableRowSecurity", 56 },
  { "AT_EnableRule", 47 },
  { "AT_EnableTrig", 39 },
  { "AT_EnableTrigAll", 43 },
  { "AT_EnableTrigUser", 45 },
  { "AT_ForceRowSecurity", 58 },
  { "AT_GenericOptions", 60 },
  { "AT_NoForceRowSecurity", 59 },
  { "AT_ReAddComment", 25 },
  { "AT_ReAddConstraint", 19 },
  { "AT_ReAddDomainConstraint", 20 },
  { "AT_ReAddIndex", 17 },
  { "AT_ReAddStatistics", 67 },
  { "AT_ReplaceRelOptions", 38 },
  { "AT_ReplicaIdentity", 55 },
  { "AT_ResetOptions", 12 },
  { "AT_ResetRelOptions", 37 },
  { "AT_SetAccessMethod", 34 },
  { "AT_SetCompression", 14 },
  { "AT_SetExpression", 7 },
  { "AT_SetIdentity", 65 },
  { "AT_SetLogged", 31 },
  { "AT_SetNotNull", 6 },
  { "AT_SetOptions", 11 },
  { "AT_SetRelOptions", 36 },
  { "AT_SetStatistics", 10 },
  { "AT_SetStorage", 13 },
  { "AT_SetTableSpace", 35 },
  { "AT_SetUnLogged", 32 },
  { "AT_ValidateConstraint", 22 },
};
const ProtobufCEnumDescriptor pg_query__alter_table_type__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.AlterTableType",
  "AlterTableType",
  "PgQuery__AlterTableType",
  "pg_query",
  68,
  pg_query__alter_table_type__enum_values_by_number,
  68,
  pg_query__alter_table_type__enum_values_by_name,
  1,
  pg_query__alter_table_type__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__grant_target_type__enum_values_by_number[4] =
{
  { "GRANT_TARGET_TYPE_UNDEFINED", "PG_QUERY__GRANT_TARGET_TYPE__GRANT_TARGET_TYPE_UNDEFINED", 0 },
  { "ACL_TARGET_OBJECT", "PG_QUERY__GRANT_TARGET_TYPE__ACL_TARGET_OBJECT", 1 },
  { "ACL_TARGET_ALL_IN_SCHEMA", "PG_QUERY__GRANT_TARGET_TYPE__ACL_TARGET_ALL_IN_SCHEMA", 2 },
  { "ACL_TARGET_DEFAULTS", "PG_QUERY__GRANT_TARGET_TYPE__ACL_TARGET_DEFAULTS", 3 },
};
static const ProtobufCIntRange pg_query__grant_target_type__value_ranges[] = {
{0, 0},{0, 4}
};
static const ProtobufCEnumValueIndex pg_query__grant_target_type__enum_values_by_name[4] =
{
  { "ACL_TARGET_ALL_IN_SCHEMA", 2 },
  { "ACL_TARGET_DEFAULTS", 3 },
  { "ACL_TARGET_OBJECT", 1 },
  { "GRANT_TARGET_TYPE_UNDEFINED", 0 },
};
const ProtobufCEnumDescriptor pg_query__grant_target_type__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.GrantTargetType",
  "GrantTargetType",
  "PgQuery__GrantTargetType",
  "pg_query",
  4,
  pg_query__grant_target_type__enum_values_by_number,
  4,
  pg_query__grant_target_type__enum_values_by_name,
  1,
  pg_query__grant_target_type__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__variable_set_kind__enum_values_by_number[7] =
{
  { "VARIABLE_SET_KIND_UNDEFINED", "PG_QUERY__VARIABLE_SET_KIND__VARIABLE_SET_KIND_UNDEFINED", 0 },
  { "VAR_SET_VALUE", "PG_QUERY__VARIABLE_SET_KIND__VAR_SET_VALUE", 1 },
  { "VAR_SET_DEFAULT", "PG_QUERY__VARIABLE_SET_KIND__VAR_SET_DEFAULT", 2 },
  { "VAR_SET_CURRENT", "PG_QUERY__VARIABLE_SET_KIND__VAR_SET_CURRENT", 3 },
  { "VAR_SET_MULTI", "PG_QUERY__VARIABLE_SET_KIND__VAR_SET_MULTI", 4 },
  { "VAR_RESET", "PG_QUERY__VARIABLE_SET_KIND__VAR_RESET", 5 },
  { "VAR_RESET_ALL", "PG_QUERY__VARIABLE_SET_KIND__VAR_RESET_ALL", 6 },
};
static const ProtobufCIntRange pg_query__variable_set_kind__value_ranges[] = {
{0, 0},{0, 7}
};
static const ProtobufCEnumValueIndex pg_query__variable_set_kind__enum_values_by_name[7] =
{
  { "VARIABLE_SET_KIND_UNDEFINED", 0 },
  { "VAR_RESET", 5 },
  { "VAR_RESET_ALL", 6 },
  { "VAR_SET_CURRENT", 3 },
  { "VAR_SET_DEFAULT", 2 },
  { "VAR_SET_MULTI", 4 },
  { "VAR_SET_VALUE", 1 },
};
const ProtobufCEnumDescriptor pg_query__variable_set_kind__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.VariableSetKind",
  "VariableSetKind",
  "PgQuery__VariableSetKind",
  "pg_query",
  7,
  pg_query__variable_set_kind__enum_values_by_number,
  7,
  pg_query__variable_set_kind__enum_values_by_name,
  1,
  pg_query__variable_set_kind__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__constr_type__enum_values_by_number[15] =
{
  { "CONSTR_TYPE_UNDEFINED", "PG_QUERY__CONSTR_TYPE__CONSTR_TYPE_UNDEFINED", 0 },
  { "CONSTR_NULL", "PG_QUERY__CONSTR_TYPE__CONSTR_NULL", 1 },
  { "CONSTR_NOTNULL", "PG_QUERY__CONSTR_TYPE__CONSTR_NOTNULL", 2 },
  { "CONSTR_DEFAULT", "PG_QUERY__CONSTR_TYPE__CONSTR_DEFAULT", 3 },
  { "CONSTR_IDENTITY", "PG_QUERY__CONSTR_TYPE__CONSTR_IDENTITY", 4 },
  { "CONSTR_GENERATED", "PG_QUERY__CONSTR_TYPE__CONSTR_GENERATED", 5 },
  { "CONSTR_CHECK", "PG_QUERY__CONSTR_TYPE__CONSTR_CHECK", 6 },
  { "CONSTR_PRIMARY", "PG_QUERY__CONSTR_TYPE__CONSTR_PRIMARY", 7 },
  { "CONSTR_UNIQUE", "PG_QUERY__CONSTR_TYPE__CONSTR_UNIQUE", 8 },
  { "CONSTR_EXCLUSION", "PG_QUERY__CONSTR_TYPE__CONSTR_EXCLUSION", 9 },
  { "CONSTR_FOREIGN", "PG_QUERY__CONSTR_TYPE__CONSTR_FOREIGN", 10 },
  { "CONSTR_ATTR_DEFERRABLE", "PG_QUERY__CONSTR_TYPE__CONSTR_ATTR_DEFERRABLE", 11 },
  { "CONSTR_ATTR_NOT_DEFERRABLE", "PG_QUERY__CONSTR_TYPE__CONSTR_ATTR_NOT_DEFERRABLE", 12 },
  { "CONSTR_ATTR_DEFERRED", "PG_QUERY__CONSTR_TYPE__CONSTR_ATTR_DEFERRED", 13 },
  { "CONSTR_ATTR_IMMEDIATE", "PG_QUERY__CONSTR_TYPE__CONSTR_ATTR_IMMEDIATE", 14 },
};
static const ProtobufCIntRange pg_query__constr_type__value_ranges[] = {
{0, 0},{0, 15}
};
static const ProtobufCEnumValueIndex pg_query__constr_type__enum_values_by_name[15] =
{
  { "CONSTR_ATTR_DEFERRABLE", 11 },
  { "CONSTR_ATTR_DEFERRED", 13 },
  { "CONSTR_ATTR_IMMEDIATE", 14 },
  { "CONSTR_ATTR_NOT_DEFERRABLE", 12 },
  { "CONSTR_CHECK", 6 },
  { "CONSTR_DEFAULT", 3 },
  { "CONSTR_EXCLUSION", 9 },
  { "CONSTR_FOREIGN", 10 },
  { "CONSTR_GENERATED", 5 },
  { "CONSTR_IDENTITY", 4 },
  { "CONSTR_NOTNULL", 2 },
  { "CONSTR_NULL", 1 },
  { "CONSTR_PRIMARY", 7 },
  { "CONSTR_TYPE_UNDEFINED", 0 },
  { "CONSTR_UNIQUE", 8 },
};
const ProtobufCEnumDescriptor pg_query__constr_type__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.ConstrType",
  "ConstrType",
  "PgQuery__ConstrType",
  "pg_query",
  15,
  pg_query__constr_type__enum_values_by_number,
  15,
  pg_query__constr_type__enum_values_by_name,
  1,
  pg_query__constr_type__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__import_foreign_schema_type__enum_values_by_number[4] =
{
  { "IMPORT_FOREIGN_SCHEMA_TYPE_UNDEFINED", "PG_QUERY__IMPORT_FOREIGN_SCHEMA_TYPE__IMPORT_FOREIGN_SCHEMA_TYPE_UNDEFINED", 0 },
  { "FDW_IMPORT_SCHEMA_ALL", "PG_QUERY__IMPORT_FOREIGN_SCHEMA_TYPE__FDW_IMPORT_SCHEMA_ALL", 1 },
  { "FDW_IMPORT_SCHEMA_LIMIT_TO", "PG_QUERY__IMPORT_FOREIGN_SCHEMA_TYPE__FDW_IMPORT_SCHEMA_LIMIT_TO", 2 },
  { "FDW_IMPORT_SCHEMA_EXCEPT", "PG_QUERY__IMPORT_FOREIGN_SCHEMA_TYPE__FDW_IMPORT_SCHEMA_EXCEPT", 3 },
};
static const ProtobufCIntRange pg_query__import_foreign_schema_type__value_ranges[] = {
{0, 0},{0, 4}
};
static const ProtobufCEnumValueIndex pg_query__import_foreign_schema_type__enum_values_by_name[4] =
{
  { "FDW_IMPORT_SCHEMA_ALL", 1 },
  { "FDW_IMPORT_SCHEMA_EXCEPT", 3 },
  { "FDW_IMPORT_SCHEMA_LIMIT_TO", 2 },
  { "IMPORT_FOREIGN_SCHEMA_TYPE_UNDEFINED", 0 },
};
const ProtobufCEnumDescriptor pg_query__import_foreign_schema_type__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.ImportForeignSchemaType",
  "ImportForeignSchemaType",
  "PgQuery__ImportForeignSchemaType",
  "pg_query",
  4,
  pg_query__import_foreign_schema_type__enum_values_by_number,
  4,
  pg_query__import_foreign_schema_type__enum_values_by_name,
  1,
  pg_query__import_foreign_schema_type__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__role_stmt_type__enum_values_by_number[4] =
{
  { "ROLE_STMT_TYPE_UNDEFINED", "PG_QUERY__ROLE_STMT_TYPE__ROLE_STMT_TYPE_UNDEFINED", 0 },
  { "ROLESTMT_ROLE", "PG_QUERY__ROLE_STMT_TYPE__ROLESTMT_ROLE", 1 },
  { "ROLESTMT_USER", "PG_QUERY__ROLE_STMT_TYPE__ROLESTMT_USER", 2 },
  { "ROLESTMT_GROUP", "PG_QUERY__ROLE_STMT_TYPE__ROLESTMT_GROUP", 3 },
};
static const ProtobufCIntRange pg_query__role_stmt_type__value_ranges[] = {
{0, 0},{0, 4}
};
static const ProtobufCEnumValueIndex pg_query__role_stmt_type__enum_values_by_name[4] =
{
  { "ROLESTMT_GROUP", 3 },
  { "ROLESTMT_ROLE", 1 },
  { "ROLESTMT_USER", 2 },
  { "ROLE_STMT_TYPE_UNDEFINED", 0 },
};
const ProtobufCEnumDescriptor pg_query__role_stmt_type__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.RoleStmtType",
  "RoleStmtType",
  "PgQuery__RoleStmtType",
  "pg_query",
  4,
  pg_query__role_stmt_type__enum_values_by_number,
  4,
  pg_query__role_stmt_type__enum_values_by_name,
  1,
  pg_query__role_stmt_type__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__fetch_direction__enum_values_by_number[5] =
{
  { "FETCH_DIRECTION_UNDEFINED", "PG_QUERY__FETCH_DIRECTION__FETCH_DIRECTION_UNDEFINED", 0 },
  { "FETCH_FORWARD", "PG_QUERY__FETCH_DIRECTION__FETCH_FORWARD", 1 },
  { "FETCH_BACKWARD", "PG_QUERY__FETCH_DIRECTION__FETCH_BACKWARD", 2 },
  { "FETCH_ABSOLUTE", "PG_QUERY__FETCH_DIRECTION__FETCH_ABSOLUTE", 3 },
  { "FETCH_RELATIVE", "PG_QUERY__FETCH_DIRECTION__FETCH_RELATIVE", 4 },
};
static const ProtobufCIntRange pg_query__fetch_direction__value_ranges[] = {
{0, 0},{0, 5}
};
static const ProtobufCEnumValueIndex pg_query__fetch_direction__enum_values_by_name[5] =
{
  { "FETCH_ABSOLUTE", 3 },
  { "FETCH_BACKWARD", 2 },
  { "FETCH_DIRECTION_UNDEFINED", 0 },
  { "FETCH_FORWARD", 1 },
  { "FETCH_RELATIVE", 4 },
};
const ProtobufCEnumDescriptor pg_query__fetch_direction__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.FetchDirection",
  "FetchDirection",
  "PgQuery__FetchDirection",
  "pg_query",
  5,
  pg_query__fetch_direction__enum_values_by_number,
  5,
  pg_query__fetch_direction__enum_values_by_name,
  1,
  pg_query__fetch_direction__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__function_parameter_mode__enum_values_by_number[7] =
{
  { "FUNCTION_PARAMETER_MODE_UNDEFINED", "PG_QUERY__FUNCTION_PARAMETER_MODE__FUNCTION_PARAMETER_MODE_UNDEFINED", 0 },
  { "FUNC_PARAM_IN", "PG_QUERY__FUNCTION_PARAMETER_MODE__FUNC_PARAM_IN", 1 },
  { "FUNC_PARAM_OUT", "PG_QUERY__FUNCTION_PARAMETER_MODE__FUNC_PARAM_OUT", 2 },
  { "FUNC_PARAM_INOUT", "PG_QUERY__FUNCTION_PARAMETER_MODE__FUNC_PARAM_INOUT", 3 },
  { "FUNC_PARAM_VARIADIC", "PG_QUERY__FUNCTION_PARAMETER_MODE__FUNC_PARAM_VARIADIC", 4 },
  { "FUNC_PARAM_TABLE", "PG_QUERY__FUNCTION_PARAMETER_MODE__FUNC_PARAM_TABLE", 5 },
  { "FUNC_PARAM_DEFAULT", "PG_QUERY__FUNCTION_PARAMETER_MODE__FUNC_PARAM_DEFAULT", 6 },
};
static const ProtobufCIntRange pg_query__function_parameter_mode__value_ranges[] = {
{0, 0},{0, 7}
};
static const ProtobufCEnumValueIndex pg_query__function_parameter_mode__enum_values_by_name[7] =
{
  { "FUNCTION_PARAMETER_MODE_UNDEFINED", 0 },
  { "FUNC_PARAM_DEFAULT", 6 },
  { "FUNC_PARAM_IN", 1 },
  { "FUNC_PARAM_INOUT", 3 },
  { "FUNC_PARAM_OUT", 2 },
  { "FUNC_PARAM_TABLE", 5 },
  { "FUNC_PARAM_VARIADIC", 4 },
};
const ProtobufCEnumDescriptor pg_query__function_parameter_mode__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.FunctionParameterMode",
  "FunctionParameterMode",
  "PgQuery__FunctionParameterMode",
  "pg_query",
  7,
  pg_query__function_parameter_mode__enum_values_by_number,
  7,
  pg_query__function_parameter_mode__enum_values_by_name,
  1,
  pg_query__function_parameter_mode__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__transaction_stmt_kind__enum_values_by_number[11] =
{
  { "TRANSACTION_STMT_KIND_UNDEFINED", "PG_QUERY__TRANSACTION_STMT_KIND__TRANSACTION_STMT_KIND_UNDEFINED", 0 },
  { "TRANS_STMT_BEGIN", "PG_QUERY__TRANSACTION_STMT_KIND__TRANS_STMT_BEGIN", 1 },
  { "TRANS_STMT_START", "PG_QUERY__TRANSACTION_STMT_KIND__TRANS_STMT_START", 2 },
  { "TRANS_STMT_COMMIT", "PG_QUERY__TRANSACTION_STMT_KIND__TRANS_STMT_COMMIT", 3 },
  { "TRANS_STMT_ROLLBACK", "PG_QUERY__TRANSACTION_STMT_KIND__TRANS_STMT_ROLLBACK", 4 },
  { "TRANS_STMT_SAVEPOINT", "PG_QUERY__TRANSACTION_STMT_KIND__TRANS_STMT_SAVEPOINT", 5 },
  { "TRANS_STMT_RELEASE", "PG_QUERY__TRANSACTION_STMT_KIND__TRANS_STMT_RELEASE", 6 },
  { "TRANS_STMT_ROLLBACK_TO", "PG_QUERY__TRANSACTION_STMT_KIND__TRANS_STMT_ROLLBACK_TO", 7 },
  { "TRANS_STMT_PREPARE", "PG_QUERY__TRANSACTION_STMT_KIND__TRANS_STMT_PREPARE", 8 },
  { "TRANS_STMT_COMMIT_PREPARED", "PG_QUERY__TRANSACTION_STMT_KIND__TRANS_STMT_COMMIT_PREPARED", 9 },
  { "TRANS_STMT_ROLLBACK_PREPARED", "PG_QUERY__TRANSACTION_STMT_KIND__TRANS_STMT_ROLLBACK_PREPARED", 10 },
};
static const ProtobufCIntRange pg_query__transaction_stmt_kind__value_ranges[] = {
{0, 0},{0, 11}
};
static const ProtobufCEnumValueIndex pg_query__transaction_stmt_kind__enum_values_by_name[11] =
{
  { "TRANSACTION_STMT_KIND_UNDEFINED", 0 },
  { "TRANS_STMT_BEGIN", 1 },
  { "TRANS_STMT_COMMIT", 3 },
  { "TRANS_STMT_COMMIT_PREPARED", 9 },
  { "TRANS_STMT_PREPARE", 8 },
  { "TRANS_STMT_RELEASE", 6 },
  { "TRANS_STMT_ROLLBACK", 4 },
  { "TRANS_STMT_ROLLBACK_PREPARED", 10 },
  { "TRANS_STMT_ROLLBACK_TO", 7 },
  { "TRANS_STMT_SAVEPOINT", 5 },
  { "TRANS_STMT_START", 2 },
};
const ProtobufCEnumDescriptor pg_query__transaction_stmt_kind__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.TransactionStmtKind",
  "TransactionStmtKind",
  "PgQuery__TransactionStmtKind",
  "pg_query",
  11,
  pg_query__transaction_stmt_kind__enum_values_by_number,
  11,
  pg_query__transaction_stmt_kind__enum_values_by_name,
  1,
  pg_query__transaction_stmt_kind__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__view_check_option__enum_values_by_number[4] =
{
  { "VIEW_CHECK_OPTION_UNDEFINED", "PG_QUERY__VIEW_CHECK_OPTION__VIEW_CHECK_OPTION_UNDEFINED", 0 },
  { "NO_CHECK_OPTION", "PG_QUERY__VIEW_CHECK_OPTION__NO_CHECK_OPTION", 1 },
  { "LOCAL_CHECK_OPTION", "PG_QUERY__VIEW_CHECK_OPTION__LOCAL_CHECK_OPTION", 2 },
  { "CASCADED_CHECK_OPTION", "PG_QUERY__VIEW_CHECK_OPTION__CASCADED_CHECK_OPTION", 3 },
};
static const ProtobufCIntRange pg_query__view_check_option__value_ranges[] = {
{0, 0},{0, 4}
};
static const ProtobufCEnumValueIndex pg_query__view_check_option__enum_values_by_name[4] =
{
  { "CASCADED_CHECK_OPTION", 3 },
  { "LOCAL_CHECK_OPTION", 2 },
  { "NO_CHECK_OPTION", 1 },
  { "VIEW_CHECK_OPTION_UNDEFINED", 0 },
};
const ProtobufCEnumDescriptor pg_query__view_check_option__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.ViewCheckOption",
  "ViewCheckOption",
  "PgQuery__ViewCheckOption",
  "pg_query",
  4,
  pg_query__view_check_option__enum_values_by_number,
  4,
  pg_query__view_check_option__enum_values_by_name,
  1,
  pg_query__view_check_option__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__discard_mode__enum_values_by_number[5] =
{
  { "DISCARD_MODE_UNDEFINED", "PG_QUERY__DISCARD_MODE__DISCARD_MODE_UNDEFINED", 0 },
  { "DISCARD_ALL", "PG_QUERY__DISCARD_MODE__DISCARD_ALL", 1 },
  { "DISCARD_PLANS", "PG_QUERY__DISCARD_MODE__DISCARD_PLANS", 2 },
  { "DISCARD_SEQUENCES", "PG_QUERY__DISCARD_MODE__DISCARD_SEQUENCES", 3 },
  { "DISCARD_TEMP", "PG_QUERY__DISCARD_MODE__DISCARD_TEMP", 4 },
};
static const ProtobufCIntRange pg_query__discard_mode__value_ranges[] = {
{0, 0},{0, 5}
};
static const ProtobufCEnumValueIndex pg_query__discard_mode__enum_values_by_name[5] =
{
  { "DISCARD_ALL", 1 },
  { "DISCARD_MODE_UNDEFINED", 0 },
  { "DISCARD_PLANS", 2 },
  { "DISCARD_SEQUENCES", 3 },
  { "DISCARD_TEMP", 4 },
};
const ProtobufCEnumDescriptor pg_query__discard_mode__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.DiscardMode",
  "DiscardMode",
  "PgQuery__DiscardMode",
  "pg_query",
  5,
  pg_query__discard_mode__enum_values_by_number,
  5,
  pg_query__discard_mode__enum_values_by_name,
  1,
  pg_query__discard_mode__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__reindex_object_type__enum_values_by_number[6] =
{
  { "REINDEX_OBJECT_TYPE_UNDEFINED", "PG_QUERY__REINDEX_OBJECT_TYPE__REINDEX_OBJECT_TYPE_UNDEFINED", 0 },
  { "REINDEX_OBJECT_INDEX", "PG_QUERY__REINDEX_OBJECT_TYPE__REINDEX_OBJECT_INDEX", 1 },
  { "REINDEX_OBJECT_TABLE", "PG_QUERY__REINDEX_OBJECT_TYPE__REINDEX_OBJECT_TABLE", 2 },
  { "REINDEX_OBJECT_SCHEMA", "PG_QUERY__REINDEX_OBJECT_TYPE__REINDEX_OBJECT_SCHEMA", 3 },
  { "REINDEX_OBJECT_SYSTEM", "PG_QUERY__REINDEX_OBJECT_TYPE__REINDEX_OBJECT_SYSTEM", 4 },
  { "REINDEX_OBJECT_DATABASE", "PG_QUERY__REINDEX_OBJECT_TYPE__REINDEX_OBJECT_DATABASE", 5 },
};
static const ProtobufCIntRange pg_query__reindex_object_type__value_ranges[] = {
{0, 0},{0, 6}
};
static const ProtobufCEnumValueIndex pg_query__reindex_object_type__enum_values_by_name[6] =
{
  { "REINDEX_OBJECT_DATABASE", 5 },
  { "REINDEX_OBJECT_INDEX", 1 },
  { "REINDEX_OBJECT_SCHEMA", 3 },
  { "REINDEX_OBJECT_SYSTEM", 4 },
  { "REINDEX_OBJECT_TABLE", 2 },
  { "REINDEX_OBJECT_TYPE_UNDEFINED", 0 },
};
const ProtobufCEnumDescriptor pg_query__reindex_object_type__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.ReindexObjectType",
  "ReindexObjectType",
  "PgQuery__ReindexObjectType",
  "pg_query",
  6,
  pg_query__reindex_object_type__enum_values_by_number,
  6,
  pg_query__reindex_object_type__enum_values_by_name,
  1,
  pg_query__reindex_object_type__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__alter_tsconfig_type__enum_values_by_number[6] =
{
  { "ALTER_TSCONFIG_TYPE_UNDEFINED", "PG_QUERY__ALTER_TSCONFIG_TYPE__ALTER_TSCONFIG_TYPE_UNDEFINED", 0 },
  { "ALTER_TSCONFIG_ADD_MAPPING", "PG_QUERY__ALTER_TSCONFIG_TYPE__ALTER_TSCONFIG_ADD_MAPPING", 1 },
  { "ALTER_TSCONFIG_ALTER_MAPPING_FOR_TOKEN", "PG_QUERY__ALTER_TSCONFIG_TYPE__ALTER_TSCONFIG_ALTER_MAPPING_FOR_TOKEN", 2 },
  { "ALTER_TSCONFIG_REPLACE_DICT", "PG_QUERY__ALTER_TSCONFIG_TYPE__ALTER_TSCONFIG_REPLACE_DICT", 3 },
  { "ALTER_TSCONFIG_REPLACE_DICT_FOR_TOKEN", "PG_QUERY__ALTER_TSCONFIG_TYPE__ALTER_TSCONFIG_REPLACE_DICT_FOR_TOKEN", 4 },
  { "ALTER_TSCONFIG_DROP_MAPPING", "PG_QUERY__ALTER_TSCONFIG_TYPE__ALTER_TSCONFIG_DROP_MAPPING", 5 },
};
static const ProtobufCIntRange pg_query__alter_tsconfig_type__value_ranges[] = {
{0, 0},{0, 6}
};
static const ProtobufCEnumValueIndex pg_query__alter_tsconfig_type__enum_values_by_name[6] =
{
  { "ALTER_TSCONFIG_ADD_MAPPING", 1 },
  { "ALTER_TSCONFIG_ALTER_MAPPING_FOR_TOKEN", 2 },
  { "ALTER_TSCONFIG_DROP_MAPPING", 5 },
  { "ALTER_TSCONFIG_REPLACE_DICT", 3 },
  { "ALTER_TSCONFIG_REPLACE_DICT_FOR_TOKEN", 4 },
  { "ALTER_TSCONFIG_TYPE_UNDEFINED", 0 },
};
const ProtobufCEnumDescriptor pg_query__alter_tsconfig_type__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.AlterTSConfigType",
  "AlterTSConfigType",
  "PgQuery__AlterTSConfigType",
  "pg_query",
  6,
  pg_query__alter_tsconfig_type__enum_values_by_number,
  6,
  pg_query__alter_tsconfig_type__enum_values_by_name,
  1,
  pg_query__alter_tsconfig_type__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__publication_obj_spec_type__enum_values_by_number[5] =
{
  { "PUBLICATION_OBJ_SPEC_TYPE_UNDEFINED", "PG_QUERY__PUBLICATION_OBJ_SPEC_TYPE__PUBLICATION_OBJ_SPEC_TYPE_UNDEFINED", 0 },
  { "PUBLICATIONOBJ_TABLE", "PG_QUERY__PUBLICATION_OBJ_SPEC_TYPE__PUBLICATIONOBJ_TABLE", 1 },
  { "PUBLICATIONOBJ_TABLES_IN_SCHEMA", "PG_QUERY__PUBLICATION_OBJ_SPEC_TYPE__PUBLICATIONOBJ_TABLES_IN_SCHEMA", 2 },
  { "PUBLICATIONOBJ_TABLES_IN_CUR_SCHEMA", "PG_QUERY__PUBLICATION_OBJ_SPEC_TYPE__PUBLICATIONOBJ_TABLES_IN_CUR_SCHEMA", 3 },
  { "PUBLICATIONOBJ_CONTINUATION", "PG_QUERY__PUBLICATION_OBJ_SPEC_TYPE__PUBLICATIONOBJ_CONTINUATION", 4 },
};
static const ProtobufCIntRange pg_query__publication_obj_spec_type__value_ranges[] = {
{0, 0},{0, 5}
};
static const ProtobufCEnumValueIndex pg_query__publication_obj_spec_type__enum_values_by_name[5] =
{
  { "PUBLICATIONOBJ_CONTINUATION", 4 },
  { "PUBLICATIONOBJ_TABLE", 1 },
  { "PUBLICATIONOBJ_TABLES_IN_CUR_SCHEMA", 3 },
  { "PUBLICATIONOBJ_TABLES_IN_SCHEMA", 2 },
  { "PUBLICATION_OBJ_SPEC_TYPE_UNDEFINED", 0 },
};
const ProtobufCEnumDescriptor pg_query__publication_obj_spec_type__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.PublicationObjSpecType",
  "PublicationObjSpecType",
  "PgQuery__PublicationObjSpecType",
  "pg_query",
  5,
  pg_query__publication_obj_spec_type__enum_values_by_number,
  5,
  pg_query__publication_obj_spec_type__enum_values_by_name,
  1,
  pg_query__publication_obj_spec_type__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__alter_publication_action__enum_values_by_number[4] =
{
  { "ALTER_PUBLICATION_ACTION_UNDEFINED", "PG_QUERY__ALTER_PUBLICATION_ACTION__ALTER_PUBLICATION_ACTION_UNDEFINED", 0 },
  { "AP_AddObjects", "PG_QUERY__ALTER_PUBLICATION_ACTION__AP_AddObjects", 1 },
  { "AP_DropObjects", "PG_QUERY__ALTER_PUBLICATION_ACTION__AP_DropObjects", 2 },
  { "AP_SetObjects", "PG_QUERY__ALTER_PUBLICATION_ACTION__AP_SetObjects", 3 },
};
static const ProtobufCIntRange pg_query__alter_publication_action__value_ranges[] = {
{0, 0},{0, 4}
};
static const ProtobufCEnumValueIndex pg_query__alter_publication_action__enum_values_by_name[4] =
{
  { "ALTER_PUBLICATION_ACTION_UNDEFINED", 0 },
  { "AP_AddObjects", 1 },
  { "AP_DropObjects", 2 },
  { "AP_SetObjects", 3 },
};
const ProtobufCEnumDescriptor pg_query__alter_publication_action__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.AlterPublicationAction",
  "AlterPublicationAction",
  "PgQuery__AlterPublicationAction",
  "pg_query",
  4,
  pg_query__alter_publication_action__enum_values_by_number,
  4,
  pg_query__alter_publication_action__enum_values_by_name,
  1,
  pg_query__alter_publication_action__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__alter_subscription_type__enum_values_by_number[9] =
{
  { "ALTER_SUBSCRIPTION_TYPE_UNDEFINED", "PG_QUERY__ALTER_SUBSCRIPTION_TYPE__ALTER_SUBSCRIPTION_TYPE_UNDEFINED", 0 },
  { "ALTER_SUBSCRIPTION_OPTIONS", "PG_QUERY__ALTER_SUBSCRIPTION_TYPE__ALTER_SUBSCRIPTION_OPTIONS", 1 },
  { "ALTER_SUBSCRIPTION_CONNECTION", "PG_QUERY__ALTER_SUBSCRIPTION_TYPE__ALTER_SUBSCRIPTION_CONNECTION", 2 },
  { "ALTER_SUBSCRIPTION_SET_PUBLICATION", "PG_QUERY__ALTER_SUBSCRIPTION_TYPE__ALTER_SUBSCRIPTION_SET_PUBLICATION", 3 },
  { "ALTER_SUBSCRIPTION_ADD_PUBLICATION", "PG_QUERY__ALTER_SUBSCRIPTION_TYPE__ALTER_SUBSCRIPTION_ADD_PUBLICATION", 4 },
  { "ALTER_SUBSCRIPTION_DROP_PUBLICATION", "PG_QUERY__ALTER_SUBSCRIPTION_TYPE__ALTER_SUBSCRIPTION_DROP_PUBLICATION", 5 },
  { "ALTER_SUBSCRIPTION_REFRESH", "PG_QUERY__ALTER_SUBSCRIPTION_TYPE__ALTER_SUBSCRIPTION_REFRESH", 6 },
  { "ALTER_SUBSCRIPTION_ENABLED", "PG_QUERY__ALTER_SUBSCRIPTION_TYPE__ALTER_SUBSCRIPTION_ENABLED", 7 },
  { "ALTER_SUBSCRIPTION_SKIP", "PG_QUERY__ALTER_SUBSCRIPTION_TYPE__ALTER_SUBSCRIPTION_SKIP", 8 },
};
static const ProtobufCIntRange pg_query__alter_subscription_type__value_ranges[] = {
{0, 0},{0, 9}
};
static const ProtobufCEnumValueIndex pg_query__alter_subscription_type__enum_values_by_name[9] =
{
  { "ALTER_SUBSCRIPTION_ADD_PUBLICATION", 4 },
  { "ALTER_SUBSCRIPTION_CONNECTION", 2 },
  { "ALTER_SUBSCRIPTION_DROP_PUBLICATION", 5 },
  { "ALTER_SUBSCRIPTION_ENABLED", 7 },
  { "ALTER_SUBSCRIPTION_OPTIONS", 1 },
  { "ALTER_SUBSCRIPTION_REFRESH", 6 },
  { "ALTER_SUBSCRIPTION_SET_PUBLICATION", 3 },
  { "ALTER_SUBSCRIPTION_SKIP", 8 },
  { "ALTER_SUBSCRIPTION_TYPE_UNDEFINED", 0 },
};
const ProtobufCEnumDescriptor pg_query__alter_subscription_type__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.AlterSubscriptionType",
  "AlterSubscriptionType",
  "PgQuery__AlterSubscriptionType",
  "pg_query",
  9,
  pg_query__alter_subscription_type__enum_values_by_number,
  9,
  pg_query__alter_subscription_type__enum_values_by_name,
  1,
  pg_query__alter_subscription_type__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__overriding_kind__enum_values_by_number[4] =
{
  { "OVERRIDING_KIND_UNDEFINED", "PG_QUERY__OVERRIDING_KIND__OVERRIDING_KIND_UNDEFINED", 0 },
  { "OVERRIDING_NOT_SET", "PG_QUERY__OVERRIDING_KIND__OVERRIDING_NOT_SET", 1 },
  { "OVERRIDING_USER_VALUE", "PG_QUERY__OVERRIDING_KIND__OVERRIDING_USER_VALUE", 2 },
  { "OVERRIDING_SYSTEM_VALUE", "PG_QUERY__OVERRIDING_KIND__OVERRIDING_SYSTEM_VALUE", 3 },
};
static const ProtobufCIntRange pg_query__overriding_kind__value_ranges[] = {
{0, 0},{0, 4}
};
static const ProtobufCEnumValueIndex pg_query__overriding_kind__enum_values_by_name[4] =
{
  { "OVERRIDING_KIND_UNDEFINED", 0 },
  { "OVERRIDING_NOT_SET", 1 },
  { "OVERRIDING_SYSTEM_VALUE", 3 },
  { "OVERRIDING_USER_VALUE", 2 },
};
const ProtobufCEnumDescriptor pg_query__overriding_kind__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.OverridingKind",
  "OverridingKind",
  "PgQuery__OverridingKind",
  "pg_query",
  4,
  pg_query__overriding_kind__enum_values_by_number,
  4,
  pg_query__overriding_kind__enum_values_by_name,
  1,
  pg_query__overriding_kind__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__on_commit_action__enum_values_by_number[5] =
{
  { "ON_COMMIT_ACTION_UNDEFINED", "PG_QUERY__ON_COMMIT_ACTION__ON_COMMIT_ACTION_UNDEFINED", 0 },
  { "ONCOMMIT_NOOP", "PG_QUERY__ON_COMMIT_ACTION__ONCOMMIT_NOOP", 1 },
  { "ONCOMMIT_PRESERVE_ROWS", "PG_QUERY__ON_COMMIT_ACTION__ONCOMMIT_PRESERVE_ROWS", 2 },
  { "ONCOMMIT_DELETE_ROWS", "PG_QUERY__ON_COMMIT_ACTION__ONCOMMIT_DELETE_ROWS", 3 },
  { "ONCOMMIT_DROP", "PG_QUERY__ON_COMMIT_ACTION__ONCOMMIT_DROP", 4 },
};
static const ProtobufCIntRange pg_query__on_commit_action__value_ranges[] = {
{0, 0},{0, 5}
};
static const ProtobufCEnumValueIndex pg_query__on_commit_action__enum_values_by_name[5] =
{
  { "ONCOMMIT_DELETE_ROWS", 3 },
  { "ONCOMMIT_DROP", 4 },
  { "ONCOMMIT_NOOP", 1 },
  { "ONCOMMIT_PRESERVE_ROWS", 2 },
  { "ON_COMMIT_ACTION_UNDEFINED", 0 },
};
const ProtobufCEnumDescriptor pg_query__on_commit_action__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.OnCommitAction",
  "OnCommitAction",
  "PgQuery__OnCommitAction",
  "pg_query",
  5,
  pg_query__on_commit_action__enum_values_by_number,
  5,
  pg_query__on_commit_action__enum_values_by_name,
  1,
  pg_query__on_commit_action__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__table_func_type__enum_values_by_number[3] =
{
  { "TABLE_FUNC_TYPE_UNDEFINED", "PG_QUERY__TABLE_FUNC_TYPE__TABLE_FUNC_TYPE_UNDEFINED", 0 },
  { "TFT_XMLTABLE", "PG_QUERY__TABLE_FUNC_TYPE__TFT_XMLTABLE", 1 },
  { "TFT_JSON_TABLE", "PG_QUERY__TABLE_FUNC_TYPE__TFT_JSON_TABLE", 2 },
};
static const ProtobufCIntRange pg_query__table_func_type__value_ranges[] = {
{0, 0},{0, 3}
};
static const ProtobufCEnumValueIndex pg_query__table_func_type__enum_values_by_name[3] =
{
  { "TABLE_FUNC_TYPE_UNDEFINED", 0 },
  { "TFT_JSON_TABLE", 2 },
  { "TFT_XMLTABLE", 1 },
};
const ProtobufCEnumDescriptor pg_query__table_func_type__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.TableFuncType",
  "TableFuncType",
  "PgQuery__TableFuncType",
  "pg_query",
  3,
  pg_query__table_func_type__enum_values_by_number,
  3,
  pg_query__table_func_type__enum_values_by_name,
  1,
  pg_query__table_func_type__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__param_kind__enum_values_by_number[5] =
{
  { "PARAM_KIND_UNDEFINED", "PG_QUERY__PARAM_KIND__PARAM_KIND_UNDEFINED", 0 },
  { "PARAM_EXTERN", "PG_QUERY__PARAM_KIND__PARAM_EXTERN", 1 },
  { "PARAM_EXEC", "PG_QUERY__PARAM_KIND__PARAM_EXEC", 2 },
  { "PARAM_SUBLINK", "PG_QUERY__PARAM_KIND__PARAM_SUBLINK", 3 },
  { "PARAM_MULTIEXPR", "PG_QUERY__PARAM_KIND__PARAM_MULTIEXPR", 4 },
};
static const ProtobufCIntRange pg_query__param_kind__value_ranges[] = {
{0, 0},{0, 5}
};
static const ProtobufCEnumValueIndex pg_query__param_kind__enum_values_by_name[5] =
{
  { "PARAM_EXEC", 2 },
  { "PARAM_EXTERN", 1 },
  { "PARAM_KIND_UNDEFINED", 0 },
  { "PARAM_MULTIEXPR", 4 },
  { "PARAM_SUBLINK", 3 },
};
const ProtobufCEnumDescriptor pg_query__param_kind__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.ParamKind",
  "ParamKind",
  "PgQuery__ParamKind",
  "pg_query",
  5,
  pg_query__param_kind__enum_values_by_number,
  5,
  pg_query__param_kind__enum_values_by_name,
  1,
  pg_query__param_kind__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__coercion_context__enum_values_by_number[5] =
{
  { "COERCION_CONTEXT_UNDEFINED", "PG_QUERY__COERCION_CONTEXT__COERCION_CONTEXT_UNDEFINED", 0 },
  { "COERCION_IMPLICIT", "PG_QUERY__COERCION_CONTEXT__COERCION_IMPLICIT", 1 },
  { "COERCION_ASSIGNMENT", "PG_QUERY__COERCION_CONTEXT__COERCION_ASSIGNMENT", 2 },
  { "COERCION_PLPGSQL", "PG_QUERY__COERCION_CONTEXT__COERCION_PLPGSQL", 3 },
  { "COERCION_EXPLICIT", "PG_QUERY__COERCION_CONTEXT__COERCION_EXPLICIT", 4 },
};
static const ProtobufCIntRange pg_query__coercion_context__value_ranges[] = {
{0, 0},{0, 5}
};
static const ProtobufCEnumValueIndex pg_query__coercion_context__enum_values_by_name[5] =
{
  { "COERCION_ASSIGNMENT", 2 },
  { "COERCION_CONTEXT_UNDEFINED", 0 },
  { "COERCION_EXPLICIT", 4 },
  { "COERCION_IMPLICIT", 1 },
  { "COERCION_PLPGSQL", 3 },
};
const ProtobufCEnumDescriptor pg_query__coercion_context__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.CoercionContext",
  "CoercionContext",
  "PgQuery__CoercionContext",
  "pg_query",
  5,
  pg_query__coercion_context__enum_values_by_number,
  5,
  pg_query__coercion_context__enum_values_by_name,
  1,
  pg_query__coercion_context__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__coercion_form__enum_values_by_number[5] =
{
  { "COERCION_FORM_UNDEFINED", "PG_QUERY__COERCION_FORM__COERCION_FORM_UNDEFINED", 0 },
  { "COERCE_EXPLICIT_CALL", "PG_QUERY__COERCION_FORM__COERCE_EXPLICIT_CALL", 1 },
  { "COERCE_EXPLICIT_CAST", "PG_QUERY__COERCION_FORM__COERCE_EXPLICIT_CAST", 2 },
  { "COERCE_IMPLICIT_CAST", "PG_QUERY__COERCION_FORM__COERCE_IMPLICIT_CAST", 3 },
  { "COERCE_SQL_SYNTAX", "PG_QUERY__COERCION_FORM__COERCE_SQL_SYNTAX", 4 },
};
static const ProtobufCIntRange pg_query__coercion_form__value_ranges[] = {
{0, 0},{0, 5}
};
static const ProtobufCEnumValueIndex pg_query__coercion_form__enum_values_by_name[5] =
{
  { "COERCE_EXPLICIT_CALL", 1 },
  { "COERCE_EXPLICIT_CAST", 2 },
  { "COERCE_IMPLICIT_CAST", 3 },
  { "COERCE_SQL_SYNTAX", 4 },
  { "COERCION_FORM_UNDEFINED", 0 },
};
const ProtobufCEnumDescriptor pg_query__coercion_form__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.CoercionForm",
  "CoercionForm",
  "PgQuery__CoercionForm",
  "pg_query",
  5,
  pg_query__coercion_form__enum_values_by_number,
  5,
  pg_query__coercion_form__enum_values_by_name,
  1,
  pg_query__coercion_form__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__bool_expr_type__enum_values_by_number[4] =
{
  { "BOOL_EXPR_TYPE_UNDEFINED", "PG_QUERY__BOOL_EXPR_TYPE__BOOL_EXPR_TYPE_UNDEFINED", 0 },
  { "AND_EXPR", "PG_QUERY__BOOL_EXPR_TYPE__AND_EXPR", 1 },
  { "OR_EXPR", "PG_QUERY__BOOL_EXPR_TYPE__OR_EXPR", 2 },
  { "NOT_EXPR", "PG_QUERY__BOOL_EXPR_TYPE__NOT_EXPR", 3 },
};
static const ProtobufCIntRange pg_query__bool_expr_type__value_ranges[] = {
{0, 0},{0, 4}
};
static const ProtobufCEnumValueIndex pg_query__bool_expr_type__enum_values_by_name[4] =
{
  { "AND_EXPR", 1 },
  { "BOOL_EXPR_TYPE_UNDEFINED", 0 },
  { "NOT_EXPR", 3 },
  { "OR_EXPR", 2 },
};
const ProtobufCEnumDescriptor pg_query__bool_expr_type__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.BoolExprType",
  "BoolExprType",
  "PgQuery__BoolExprType",
  "pg_query",
  4,
  pg_query__bool_expr_type__enum_values_by_number,
  4,
  pg_query__bool_expr_type__enum_values_by_name,
  1,
  pg_query__bool_expr_type__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__sub_link_type__enum_values_by_number[9] =
{
  { "SUB_LINK_TYPE_UNDEFINED", "PG_QUERY__SUB_LINK_TYPE__SUB_LINK_TYPE_UNDEFINED", 0 },
  { "EXISTS_SUBLINK", "PG_QUERY__SUB_LINK_TYPE__EXISTS_SUBLINK", 1 },
  { "ALL_SUBLINK", "PG_QUERY__SUB_LINK_TYPE__ALL_SUBLINK", 2 },
  { "ANY_SUBLINK", "PG_QUERY__SUB_LINK_TYPE__ANY_SUBLINK", 3 },
  { "ROWCOMPARE_SUBLINK", "PG_QUERY__SUB_LINK_TYPE__ROWCOMPARE_SUBLINK", 4 },
  { "EXPR_SUBLINK", "PG_QUERY__SUB_LINK_TYPE__EXPR_SUBLINK", 5 },
  { "MULTIEXPR_SUBLINK", "PG_QUERY__SUB_LINK_TYPE__MULTIEXPR_SUBLINK", 6 },
  { "ARRAY_SUBLINK", "PG_QUERY__SUB_LINK_TYPE__ARRAY_SUBLINK", 7 },
  { "CTE_SUBLINK", "PG_QUERY__SUB_LINK_TYPE__CTE_SUBLINK", 8 },
};
static const ProtobufCIntRange pg_query__sub_link_type__value_ranges[] = {
{0, 0},{0, 9}
};
static const ProtobufCEnumValueIndex pg_query__sub_link_type__enum_values_by_name[9] =
{
  { "ALL_SUBLINK", 2 },
  { "ANY_SUBLINK", 3 },
  { "ARRAY_SUBLINK", 7 },
  { "CTE_SUBLINK", 8 },
  { "EXISTS_SUBLINK", 1 },
  { "EXPR_SUBLINK", 5 },
  { "MULTIEXPR_SUBLINK", 6 },
  { "ROWCOMPARE_SUBLINK", 4 },
  { "SUB_LINK_TYPE_UNDEFINED", 0 },
};
const ProtobufCEnumDescriptor pg_query__sub_link_type__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.SubLinkType",
  "SubLinkType",
  "PgQuery__SubLinkType",
  "pg_query",
  9,
  pg_query__sub_link_type__enum_values_by_number,
  9,
  pg_query__sub_link_type__enum_values_by_name,
  1,
  pg_query__sub_link_type__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__row_compare_type__enum_values_by_number[7] =
{
  { "ROW_COMPARE_TYPE_UNDEFINED", "PG_QUERY__ROW_COMPARE_TYPE__ROW_COMPARE_TYPE_UNDEFINED", 0 },
  { "ROWCOMPARE_LT", "PG_QUERY__ROW_COMPARE_TYPE__ROWCOMPARE_LT", 1 },
  { "ROWCOMPARE_LE", "PG_QUERY__ROW_COMPARE_TYPE__ROWCOMPARE_LE", 2 },
  { "ROWCOMPARE_EQ", "PG_QUERY__ROW_COMPARE_TYPE__ROWCOMPARE_EQ", 3 },
  { "ROWCOMPARE_GE", "PG_QUERY__ROW_COMPARE_TYPE__ROWCOMPARE_GE", 4 },
  { "ROWCOMPARE_GT", "PG_QUERY__ROW_COMPARE_TYPE__ROWCOMPARE_GT", 5 },
  { "ROWCOMPARE_NE", "PG_QUERY__ROW_COMPARE_TYPE__ROWCOMPARE_NE", 6 },
};
static const ProtobufCIntRange pg_query__row_compare_type__value_ranges[] = {
{0, 0},{0, 7}
};
static const ProtobufCEnumValueIndex pg_query__row_compare_type__enum_values_by_name[7] =
{
  { "ROWCOMPARE_EQ", 3 },
  { "ROWCOMPARE_GE", 4 },
  { "ROWCOMPARE_GT", 5 },
  { "ROWCOMPARE_LE", 2 },
  { "ROWCOMPARE_LT", 1 },
  { "ROWCOMPARE_NE", 6 },
  { "ROW_COMPARE_TYPE_UNDEFINED", 0 },
};
const ProtobufCEnumDescriptor pg_query__row_compare_type__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.RowCompareType",
  "RowCompareType",
  "PgQuery__RowCompareType",
  "pg_query",
  7,
  pg_query__row_compare_type__enum_values_by_number,
  7,
  pg_query__row_compare_type__enum_values_by_name,
  1,
  pg_query__row_compare_type__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__min_max_op__enum_values_by_number[3] =
{
  { "MIN_MAX_OP_UNDEFINED", "PG_QUERY__MIN_MAX_OP__MIN_MAX_OP_UNDEFINED", 0 },
  { "IS_GREATEST", "PG_QUERY__MIN_MAX_OP__IS_GREATEST", 1 },
  { "IS_LEAST", "PG_QUERY__MIN_MAX_OP__IS_LEAST", 2 },
};
static const ProtobufCIntRange pg_query__min_max_op__value_ranges[] = {
{0, 0},{0, 3}
};
static const ProtobufCEnumValueIndex pg_query__min_max_op__enum_values_by_name[3] =
{
  { "IS_GREATEST", 1 },
  { "IS_LEAST", 2 },
  { "MIN_MAX_OP_UNDEFINED", 0 },
};
const ProtobufCEnumDescriptor pg_query__min_max_op__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.MinMaxOp",
  "MinMaxOp",
  "PgQuery__MinMaxOp",
  "pg_query",
  3,
  pg_query__min_max_op__enum_values_by_number,
  3,
  pg_query__min_max_op__enum_values_by_name,
  1,
  pg_query__min_max_op__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__sqlvalue_function_op__enum_values_by_number[16] =
{
  { "SQLVALUE_FUNCTION_OP_UNDEFINED", "PG_QUERY__SQLVALUE_FUNCTION_OP__SQLVALUE_FUNCTION_OP_UNDEFINED", 0 },
  { "SVFOP_CURRENT_DATE", "PG_QUERY__SQLVALUE_FUNCTION_OP__SVFOP_CURRENT_DATE", 1 },
  { "SVFOP_CURRENT_TIME", "PG_QUERY__SQLVALUE_FUNCTION_OP__SVFOP_CURRENT_TIME", 2 },
  { "SVFOP_CURRENT_TIME_N", "PG_QUERY__SQLVALUE_FUNCTION_OP__SVFOP_CURRENT_TIME_N", 3 },
  { "SVFOP_CURRENT_TIMESTAMP", "PG_QUERY__SQLVALUE_FUNCTION_OP__SVFOP_CURRENT_TIMESTAMP", 4 },
  { "SVFOP_CURRENT_TIMESTAMP_N", "PG_QUERY__SQLVALUE_FUNCTION_OP__SVFOP_CURRENT_TIMESTAMP_N", 5 },
  { "SVFOP_LOCALTIME", "PG_QUERY__SQLVALUE_FUNCTION_OP__SVFOP_LOCALTIME", 6 },
  { "SVFOP_LOCALTIME_N", "PG_QUERY__SQLVALUE_FUNCTION_OP__SVFOP_LOCALTIME_N", 7 },
  { "SVFOP_LOCALTIMESTAMP", "PG_QUERY__SQLVALUE_FUNCTION_OP__SVFOP_LOCALTIMESTAMP", 8 },
  { "SVFOP_LOCALTIMESTAMP_N", "PG_QUERY__SQLVALUE_FUNCTION_OP__SVFOP_LOCALTIMESTAMP_N", 9 },
  { "SVFOP_CURRENT_ROLE", "PG_QUERY__SQLVALUE_FUNCTION_OP__SVFOP_CURRENT_ROLE", 10 },
  { "SVFOP_CURRENT_USER", "PG_QUERY__SQLVALUE_FUNCTION_OP__SVFOP_CURRENT_USER", 11 },
  { "SVFOP_USER", "PG_QUERY__SQLVALUE_FUNCTION_OP__SVFOP_USER", 12 },
  { "SVFOP_SESSION_USER", "PG_QUERY__SQLVALUE_FUNCTION_OP__SVFOP_SESSION_USER", 13 },
  { "SVFOP_CURRENT_CATALOG", "PG_QUERY__SQLVALUE_FUNCTION_OP__SVFOP_CURRENT_CATALOG", 14 },
  { "SVFOP_CURRENT_SCHEMA", "PG_QUERY__SQLVALUE_FUNCTION_OP__SVFOP_CURRENT_SCHEMA", 15 },
};
static const ProtobufCIntRange pg_query__sqlvalue_function_op__value_ranges[] = {
{0, 0},{0, 16}
};
static const ProtobufCEnumValueIndex pg_query__sqlvalue_function_op__enum_values_by_name[16] =
{
  { "SQLVALUE_FUNCTION_OP_UNDEFINED", 0 },
  { "SVFOP_CURRENT_CATALOG", 14 },
  { "SVFOP_CURRENT_DATE", 1 },
  { "SVFOP_CURRENT_ROLE", 10 },
  { "SVFOP_CURRENT_SCHEMA", 15 },
  { "SVFOP_CURRENT_TIME", 2 },
  { "SVFOP_CURRENT_TIMESTAMP", 4 },
  { "SVFOP_CURRENT_TIMESTAMP_N", 5 },
  { "SVFOP_CURRENT_TIME_N", 3 },
  { "SVFOP_CURRENT_USER", 11 },
  { "SVFOP_LOCALTIME", 6 },
  { "SVFOP_LOCALTIMESTAMP", 8 },
  { "SVFOP_LOCALTIMESTAMP_N", 9 },
  { "SVFOP_LOCALTIME_N", 7 },
  { "SVFOP_SESSION_USER", 13 },
  { "SVFOP_USER", 12 },
};
const ProtobufCEnumDescriptor pg_query__sqlvalue_function_op__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.SQLValueFunctionOp",
  "SQLValueFunctionOp",
  "PgQuery__SQLValueFunctionOp",
  "pg_query",
  16,
  pg_query__sqlvalue_function_op__enum_values_by_number,
  16,
  pg_query__sqlvalue_function_op__enum_values_by_name,
  1,
  pg_query__sqlvalue_function_op__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__xml_expr_op__enum_values_by_number[9] =
{
  { "XML_EXPR_OP_UNDEFINED", "PG_QUERY__XML_EXPR_OP__XML_EXPR_OP_UNDEFINED", 0 },
  { "IS_XMLCONCAT", "PG_QUERY__XML_EXPR_OP__IS_XMLCONCAT", 1 },
  { "IS_XMLELEMENT", "PG_QUERY__XML_EXPR_OP__IS_XMLELEMENT", 2 },
  { "IS_XMLFOREST", "PG_QUERY__XML_EXPR_OP__IS_XMLFOREST", 3 },
  { "IS_XMLPARSE", "PG_QUERY__XML_EXPR_OP__IS_XMLPARSE", 4 },
  { "IS_XMLPI", "PG_QUERY__XML_EXPR_OP__IS_XMLPI", 5 },
  { "IS_XMLROOT", "PG_QUERY__XML_EXPR_OP__IS_XMLROOT", 6 },
  { "IS_XMLSERIALIZE", "PG_QUERY__XML_EXPR_OP__IS_XMLSERIALIZE", 7 },
  { "IS_DOCUMENT", "PG_QUERY__XML_EXPR_OP__IS_DOCUMENT", 8 },
};
static const ProtobufCIntRange pg_query__xml_expr_op__value_ranges[] = {
{0, 0},{0, 9}
};
static const ProtobufCEnumValueIndex pg_query__xml_expr_op__enum_values_by_name[9] =
{
  { "IS_DOCUMENT", 8 },
  { "IS_XMLCONCAT", 1 },
  { "IS_XMLELEMENT", 2 },
  { "IS_XMLFOREST", 3 },
  { "IS_XMLPARSE", 4 },
  { "IS_XMLPI", 5 },
  { "IS_XMLROOT", 6 },
  { "IS_XMLSERIALIZE", 7 },
  { "XML_EXPR_OP_UNDEFINED", 0 },
};
const ProtobufCEnumDescriptor pg_query__xml_expr_op__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.XmlExprOp",
  "XmlExprOp",
  "PgQuery__XmlExprOp",
  "pg_query",
  9,
  pg_query__xml_expr_op__enum_values_by_number,
  9,
  pg_query__xml_expr_op__enum_values_by_name,
  1,
  pg_query__xml_expr_op__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__xml_option_type__enum_values_by_number[3] =
{
  { "XML_OPTION_TYPE_UNDEFINED", "PG_QUERY__XML_OPTION_TYPE__XML_OPTION_TYPE_UNDEFINED", 0 },
  { "XMLOPTION_DOCUMENT", "PG_QUERY__XML_OPTION_TYPE__XMLOPTION_DOCUMENT", 1 },
  { "XMLOPTION_CONTENT", "PG_QUERY__XML_OPTION_TYPE__XMLOPTION_CONTENT", 2 },
};
static const ProtobufCIntRange pg_query__xml_option_type__value_ranges[] = {
{0, 0},{0, 3}
};
static const ProtobufCEnumValueIndex pg_query__xml_option_type__enum_values_by_name[3] =
{
  { "XMLOPTION_CONTENT", 2 },
  { "XMLOPTION_DOCUMENT", 1 },
  { "XML_OPTION_TYPE_UNDEFINED", 0 },
};
const ProtobufCEnumDescriptor pg_query__xml_option_type__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.XmlOptionType",
  "XmlOptionType",
  "PgQuery__XmlOptionType",
  "pg_query",
  3,
  pg_query__xml_option_type__enum_values_by_number,
  3,
  pg_query__xml_option_type__enum_values_by_name,
  1,
  pg_query__xml_option_type__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__json_encoding__enum_values_by_number[5] =
{
  { "JSON_ENCODING_UNDEFINED", "PG_QUERY__JSON_ENCODING__JSON_ENCODING_UNDEFINED", 0 },
  { "JS_ENC_DEFAULT", "PG_QUERY__JSON_ENCODING__JS_ENC_DEFAULT", 1 },
  { "JS_ENC_UTF8", "PG_QUERY__JSON_ENCODING__JS_ENC_UTF8", 2 },
  { "JS_ENC_UTF16", "PG_QUERY__JSON_ENCODING__JS_ENC_UTF16", 3 },
  { "JS_ENC_UTF32", "PG_QUERY__JSON_ENCODING__JS_ENC_UTF32", 4 },
};
static const ProtobufCIntRange pg_query__json_encoding__value_ranges[] = {
{0, 0},{0, 5}
};
static const ProtobufCEnumValueIndex pg_query__json_encoding__enum_values_by_name[5] =
{
  { "JSON_ENCODING_UNDEFINED", 0 },
  { "JS_ENC_DEFAULT", 1 },
  { "JS_ENC_UTF16", 3 },
  { "JS_ENC_UTF32", 4 },
  { "JS_ENC_UTF8", 2 },
};
const ProtobufCEnumDescriptor pg_query__json_encoding__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.JsonEncoding",
  "JsonEncoding",
  "PgQuery__JsonEncoding",
  "pg_query",
  5,
  pg_query__json_encoding__enum_values_by_number,
  5,
  pg_query__json_encoding__enum_values_by_name,
  1,
  pg_query__json_encoding__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__json_format_type__enum_values_by_number[4] =
{
  { "JSON_FORMAT_TYPE_UNDEFINED", "PG_QUERY__JSON_FORMAT_TYPE__JSON_FORMAT_TYPE_UNDEFINED", 0 },
  { "JS_FORMAT_DEFAULT", "PG_QUERY__JSON_FORMAT_TYPE__JS_FORMAT_DEFAULT", 1 },
  { "JS_FORMAT_JSON", "PG_QUERY__JSON_FORMAT_TYPE__JS_FORMAT_JSON", 2 },
  { "JS_FORMAT_JSONB", "PG_QUERY__JSON_FORMAT_TYPE__JS_FORMAT_JSONB", 3 },
};
static const ProtobufCIntRange pg_query__json_format_type__value_ranges[] = {
{0, 0},{0, 4}
};
static const ProtobufCEnumValueIndex pg_query__json_format_type__enum_values_by_name[4] =
{
  { "JSON_FORMAT_TYPE_UNDEFINED", 0 },
  { "JS_FORMAT_DEFAULT", 1 },
  { "JS_FORMAT_JSON", 2 },
  { "JS_FORMAT_JSONB", 3 },
};
const ProtobufCEnumDescriptor pg_query__json_format_type__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.JsonFormatType",
  "JsonFormatType",
  "PgQuery__JsonFormatType",
  "pg_query",
  4,
  pg_query__json_format_type__enum_values_by_number,
  4,
  pg_query__json_format_type__enum_values_by_name,
  1,
  pg_query__json_format_type__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__json_constructor_type__enum_values_by_number[8] =
{
  { "JSON_CONSTRUCTOR_TYPE_UNDEFINED", "PG_QUERY__JSON_CONSTRUCTOR_TYPE__JSON_CONSTRUCTOR_TYPE_UNDEFINED", 0 },
  { "JSCTOR_JSON_OBJECT", "PG_QUERY__JSON_CONSTRUCTOR_TYPE__JSCTOR_JSON_OBJECT", 1 },
  { "JSCTOR_JSON_ARRAY", "PG_QUERY__JSON_CONSTRUCTOR_TYPE__JSCTOR_JSON_ARRAY", 2 },
  { "JSCTOR_JSON_OBJECTAGG", "PG_QUERY__JSON_CONSTRUCTOR_TYPE__JSCTOR_JSON_OBJECTAGG", 3 },
  { "JSCTOR_JSON_ARRAYAGG", "PG_QUERY__JSON_CONSTRUCTOR_TYPE__JSCTOR_JSON_ARRAYAGG", 4 },
  { "JSCTOR_JSON_PARSE", "PG_QUERY__JSON_CONSTRUCTOR_TYPE__JSCTOR_JSON_PARSE", 5 },
  { "JSCTOR_JSON_SCALAR", "PG_QUERY__JSON_CONSTRUCTOR_TYPE__JSCTOR_JSON_SCALAR", 6 },
  { "JSCTOR_JSON_SERIALIZE", "PG_QUERY__JSON_CONSTRUCTOR_TYPE__JSCTOR_JSON_SERIALIZE", 7 },
};
static const ProtobufCIntRange pg_query__json_constructor_type__value_ranges[] = {
{0, 0},{0, 8}
};
static const ProtobufCEnumValueIndex pg_query__json_constructor_type__enum_values_by_name[8] =
{
  { "JSCTOR_JSON_ARRAY", 2 },
  { "JSCTOR_JSON_ARRAYAGG", 4 },
  { "JSCTOR_JSON_OBJECT", 1 },
  { "JSCTOR_JSON_OBJECTAGG", 3 },
  { "JSCTOR_JSON_PARSE", 5 },
  { "JSCTOR_JSON_SCALAR", 6 },
  { "JSCTOR_JSON_SERIALIZE", 7 },
  { "JSON_CONSTRUCTOR_TYPE_UNDEFINED", 0 },
};
const ProtobufCEnumDescriptor pg_query__json_constructor_type__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.JsonConstructorType",
  "JsonConstructorType",
  "PgQuery__JsonConstructorType",
  "pg_query",
  8,
  pg_query__json_constructor_type__enum_values_by_number,
  8,
  pg_query__json_constructor_type__enum_values_by_name,
  1,
  pg_query__json_constructor_type__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__json_value_type__enum_values_by_number[5] =
{
  { "JSON_VALUE_TYPE_UNDEFINED", "PG_QUERY__JSON_VALUE_TYPE__JSON_VALUE_TYPE_UNDEFINED", 0 },
  { "JS_TYPE_ANY", "PG_QUERY__JSON_VALUE_TYPE__JS_TYPE_ANY", 1 },
  { "JS_TYPE_OBJECT", "PG_QUERY__JSON_VALUE_TYPE__JS_TYPE_OBJECT", 2 },
  { "JS_TYPE_ARRAY", "PG_QUERY__JSON_VALUE_TYPE__JS_TYPE_ARRAY", 3 },
  { "JS_TYPE_SCALAR", "PG_QUERY__JSON_VALUE_TYPE__JS_TYPE_SCALAR", 4 },
};
static const ProtobufCIntRange pg_query__json_value_type__value_ranges[] = {
{0, 0},{0, 5}
};
static const ProtobufCEnumValueIndex pg_query__json_value_type__enum_values_by_name[5] =
{
  { "JSON_VALUE_TYPE_UNDEFINED", 0 },
  { "JS_TYPE_ANY", 1 },
  { "JS_TYPE_ARRAY", 3 },
  { "JS_TYPE_OBJECT", 2 },
  { "JS_TYPE_SCALAR", 4 },
};
const ProtobufCEnumDescriptor pg_query__json_value_type__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.JsonValueType",
  "JsonValueType",
  "PgQuery__JsonValueType",
  "pg_query",
  5,
  pg_query__json_value_type__enum_values_by_number,
  5,
  pg_query__json_value_type__enum_values_by_name,
  1,
  pg_query__json_value_type__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__json_wrapper__enum_values_by_number[5] =
{
  { "JSON_WRAPPER_UNDEFINED", "PG_QUERY__JSON_WRAPPER__JSON_WRAPPER_UNDEFINED", 0 },
  { "JSW_UNSPEC", "PG_QUERY__JSON_WRAPPER__JSW_UNSPEC", 1 },
  { "JSW_NONE", "PG_QUERY__JSON_WRAPPER__JSW_NONE", 2 },
  { "JSW_CONDITIONAL", "PG_QUERY__JSON_WRAPPER__JSW_CONDITIONAL", 3 },
  { "JSW_UNCONDITIONAL", "PG_QUERY__JSON_WRAPPER__JSW_UNCONDITIONAL", 4 },
};
static const ProtobufCIntRange pg_query__json_wrapper__value_ranges[] = {
{0, 0},{0, 5}
};
static const ProtobufCEnumValueIndex pg_query__json_wrapper__enum_values_by_name[5] =
{
  { "JSON_WRAPPER_UNDEFINED", 0 },
  { "JSW_CONDITIONAL", 3 },
  { "JSW_NONE", 2 },
  { "JSW_UNCONDITIONAL", 4 },
  { "JSW_UNSPEC", 1 },
};
const ProtobufCEnumDescriptor pg_query__json_wrapper__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.JsonWrapper",
  "JsonWrapper",
  "PgQuery__JsonWrapper",
  "pg_query",
  5,
  pg_query__json_wrapper__enum_values_by_number,
  5,
  pg_query__json_wrapper__enum_values_by_name,
  1,
  pg_query__json_wrapper__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__json_behavior_type__enum_values_by_number[10] =
{
  { "JSON_BEHAVIOR_TYPE_UNDEFINED", "PG_QUERY__JSON_BEHAVIOR_TYPE__JSON_BEHAVIOR_TYPE_UNDEFINED", 0 },
  { "JSON_BEHAVIOR_NULL", "PG_QUERY__JSON_BEHAVIOR_TYPE__JSON_BEHAVIOR_NULL", 1 },
  { "JSON_BEHAVIOR_ERROR", "PG_QUERY__JSON_BEHAVIOR_TYPE__JSON_BEHAVIOR_ERROR", 2 },
  { "JSON_BEHAVIOR_EMPTY", "PG_QUERY__JSON_BEHAVIOR_TYPE__JSON_BEHAVIOR_EMPTY", 3 },
  { "JSON_BEHAVIOR_TRUE", "PG_QUERY__JSON_BEHAVIOR_TYPE__JSON_BEHAVIOR_TRUE", 4 },
  { "JSON_BEHAVIOR_FALSE", "PG_QUERY__JSON_BEHAVIOR_TYPE__JSON_BEHAVIOR_FALSE", 5 },
  { "JSON_BEHAVIOR_UNKNOWN", "PG_QUERY__JSON_BEHAVIOR_TYPE__JSON_BEHAVIOR_UNKNOWN", 6 },
  { "JSON_BEHAVIOR_EMPTY_ARRAY", "PG_QUERY__JSON_BEHAVIOR_TYPE__JSON_BEHAVIOR_EMPTY_ARRAY", 7 },
  { "JSON_BEHAVIOR_EMPTY_OBJECT", "PG_QUERY__JSON_BEHAVIOR_TYPE__JSON_BEHAVIOR_EMPTY_OBJECT", 8 },
  { "JSON_BEHAVIOR_DEFAULT", "PG_QUERY__JSON_BEHAVIOR_TYPE__JSON_BEHAVIOR_DEFAULT", 9 },
};
static const ProtobufCIntRange pg_query__json_behavior_type__value_ranges[] = {
{0, 0},{0, 10}
};
static const ProtobufCEnumValueIndex pg_query__json_behavior_type__enum_values_by_name[10] =
{
  { "JSON_BEHAVIOR_DEFAULT", 9 },
  { "JSON_BEHAVIOR_EMPTY", 3 },
  { "JSON_BEHAVIOR_EMPTY_ARRAY", 7 },
  { "JSON_BEHAVIOR_EMPTY_OBJECT", 8 },
  { "JSON_BEHAVIOR_ERROR", 2 },
  { "JSON_BEHAVIOR_FALSE", 5 },
  { "JSON_BEHAVIOR_NULL", 1 },
  { "JSON_BEHAVIOR_TRUE", 4 },
  { "JSON_BEHAVIOR_TYPE_UNDEFINED", 0 },
  { "JSON_BEHAVIOR_UNKNOWN", 6 },
};
const ProtobufCEnumDescriptor pg_query__json_behavior_type__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.JsonBehaviorType",
  "JsonBehaviorType",
  "PgQuery__JsonBehaviorType",
  "pg_query",
  10,
  pg_query__json_behavior_type__enum_values_by_number,
  10,
  pg_query__json_behavior_type__enum_values_by_name,
  1,
  pg_query__json_behavior_type__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__json_expr_op__enum_values_by_number[5] =
{
  { "JSON_EXPR_OP_UNDEFINED", "PG_QUERY__JSON_EXPR_OP__JSON_EXPR_OP_UNDEFINED", 0 },
  { "JSON_EXISTS_OP", "PG_QUERY__JSON_EXPR_OP__JSON_EXISTS_OP", 1 },
  { "JSON_QUERY_OP", "PG_QUERY__JSON_EXPR_OP__JSON_QUERY_OP", 2 },
  { "JSON_VALUE_OP", "PG_QUERY__JSON_EXPR_OP__JSON_VALUE_OP", 3 },
  { "JSON_TABLE_OP", "PG_QUERY__JSON_EXPR_OP__JSON_TABLE_OP", 4 },
};
static const ProtobufCIntRange pg_query__json_expr_op__value_ranges[] = {
{0, 0},{0, 5}
};
static const ProtobufCEnumValueIndex pg_query__json_expr_op__enum_values_by_name[5] =
{
  { "JSON_EXISTS_OP", 1 },
  { "JSON_EXPR_OP_UNDEFINED", 0 },
  { "JSON_QUERY_OP", 2 },
  { "JSON_TABLE_OP", 4 },
  { "JSON_VALUE_OP", 3 },
};
const ProtobufCEnumDescriptor pg_query__json_expr_op__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.JsonExprOp",
  "JsonExprOp",
  "PgQuery__JsonExprOp",
  "pg_query",
  5,
  pg_query__json_expr_op__enum_values_by_number,
  5,
  pg_query__json_expr_op__enum_values_by_name,
  1,
  pg_query__json_expr_op__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__null_test_type__enum_values_by_number[3] =
{
  { "NULL_TEST_TYPE_UNDEFINED", "PG_QUERY__NULL_TEST_TYPE__NULL_TEST_TYPE_UNDEFINED", 0 },
  { "IS_NULL", "PG_QUERY__NULL_TEST_TYPE__IS_NULL", 1 },
  { "IS_NOT_NULL", "PG_QUERY__NULL_TEST_TYPE__IS_NOT_NULL", 2 },
};
static const ProtobufCIntRange pg_query__null_test_type__value_ranges[] = {
{0, 0},{0, 3}
};
static const ProtobufCEnumValueIndex pg_query__null_test_type__enum_values_by_name[3] =
{
  { "IS_NOT_NULL", 2 },
  { "IS_NULL", 1 },
  { "NULL_TEST_TYPE_UNDEFINED", 0 },
};
const ProtobufCEnumDescriptor pg_query__null_test_type__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.NullTestType",
  "NullTestType",
  "PgQuery__NullTestType",
  "pg_query",
  3,
  pg_query__null_test_type__enum_values_by_number,
  3,
  pg_query__null_test_type__enum_values_by_name,
  1,
  pg_query__null_test_type__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__bool_test_type__enum_values_by_number[7] =
{
  { "BOOL_TEST_TYPE_UNDEFINED", "PG_QUERY__BOOL_TEST_TYPE__BOOL_TEST_TYPE_UNDEFINED", 0 },
  { "IS_TRUE", "PG_QUERY__BOOL_TEST_TYPE__IS_TRUE", 1 },
  { "IS_NOT_TRUE", "PG_QUERY__BOOL_TEST_TYPE__IS_NOT_TRUE", 2 },
  { "IS_FALSE", "PG_QUERY__BOOL_TEST_TYPE__IS_FALSE", 3 },
  { "IS_NOT_FALSE", "PG_QUERY__BOOL_TEST_TYPE__IS_NOT_FALSE", 4 },
  { "IS_UNKNOWN", "PG_QUERY__BOOL_TEST_TYPE__IS_UNKNOWN", 5 },
  { "IS_NOT_UNKNOWN", "PG_QUERY__BOOL_TEST_TYPE__IS_NOT_UNKNOWN", 6 },
};
static const ProtobufCIntRange pg_query__bool_test_type__value_ranges[] = {
{0, 0},{0, 7}
};
static const ProtobufCEnumValueIndex pg_query__bool_test_type__enum_values_by_name[7] =
{
  { "BOOL_TEST_TYPE_UNDEFINED", 0 },
  { "IS_FALSE", 3 },
  { "IS_NOT_FALSE", 4 },
  { "IS_NOT_TRUE", 2 },
  { "IS_NOT_UNKNOWN", 6 },
  { "IS_TRUE", 1 },
  { "IS_UNKNOWN", 5 },
};
const ProtobufCEnumDescriptor pg_query__bool_test_type__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.BoolTestType",
  "BoolTestType",
  "PgQuery__BoolTestType",
  "pg_query",
  7,
  pg_query__bool_test_type__enum_values_by_number,
  7,
  pg_query__bool_test_type__enum_values_by_name,
  1,
  pg_query__bool_test_type__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__merge_match_kind__enum_values_by_number[4] =
{
  { "MERGE_MATCH_KIND_UNDEFINED", "PG_QUERY__MERGE_MATCH_KIND__MERGE_MATCH_KIND_UNDEFINED", 0 },
  { "MERGE_WHEN_MATCHED", "PG_QUERY__MERGE_MATCH_KIND__MERGE_WHEN_MATCHED", 1 },
  { "MERGE_WHEN_NOT_MATCHED_BY_SOURCE", "PG_QUERY__MERGE_MATCH_KIND__MERGE_WHEN_NOT_MATCHED_BY_SOURCE", 2 },
  { "MERGE_WHEN_NOT_MATCHED_BY_TARGET", "PG_QUERY__MERGE_MATCH_KIND__MERGE_WHEN_NOT_MATCHED_BY_TARGET", 3 },
};
static const ProtobufCIntRange pg_query__merge_match_kind__value_ranges[] = {
{0, 0},{0, 4}
};
static const ProtobufCEnumValueIndex pg_query__merge_match_kind__enum_values_by_name[4] =
{
  { "MERGE_MATCH_KIND_UNDEFINED", 0 },
  { "MERGE_WHEN_MATCHED", 1 },
  { "MERGE_WHEN_NOT_MATCHED_BY_SOURCE", 2 },
  { "MERGE_WHEN_NOT_MATCHED_BY_TARGET", 3 },
};
const ProtobufCEnumDescriptor pg_query__merge_match_kind__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.MergeMatchKind",
  "MergeMatchKind",
  "PgQuery__MergeMatchKind",
  "pg_query",
  4,
  pg_query__merge_match_kind__enum_values_by_number,
  4,
  pg_query__merge_match_kind__enum_values_by_name,
  1,
  pg_query__merge_match_kind__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__cmd_type__enum_values_by_number[9] =
{
  { "CMD_TYPE_UNDEFINED", "PG_QUERY__CMD_TYPE__CMD_TYPE_UNDEFINED", 0 },
  { "CMD_UNKNOWN", "PG_QUERY__CMD_TYPE__CMD_UNKNOWN", 1 },
  { "CMD_SELECT", "PG_QUERY__CMD_TYPE__CMD_SELECT", 2 },
  { "CMD_UPDATE", "PG_QUERY__CMD_TYPE__CMD_UPDATE", 3 },
  { "CMD_INSERT", "PG_QUERY__CMD_TYPE__CMD_INSERT", 4 },
  { "CMD_DELETE", "PG_QUERY__CMD_TYPE__CMD_DELETE", 5 },
  { "CMD_MERGE", "PG_QUERY__CMD_TYPE__CMD_MERGE", 6 },
  { "CMD_UTILITY", "PG_QUERY__CMD_TYPE__CMD_UTILITY", 7 },
  { "CMD_NOTHING", "PG_QUERY__CMD_TYPE__CMD_NOTHING", 8 },
};
static const ProtobufCIntRange pg_query__cmd_type__value_ranges[] = {
{0, 0},{0, 9}
};
static const ProtobufCEnumValueIndex pg_query__cmd_type__enum_values_by_name[9] =
{
  { "CMD_DELETE", 5 },
  { "CMD_INSERT", 4 },
  { "CMD_MERGE", 6 },
  { "CMD_NOTHING", 8 },
  { "CMD_SELECT", 2 },
  { "CMD_TYPE_UNDEFINED", 0 },
  { "CMD_UNKNOWN", 1 },
  { "CMD_UPDATE", 3 },
  { "CMD_UTILITY", 7 },
};
const ProtobufCEnumDescriptor pg_query__cmd_type__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.CmdType",
  "CmdType",
  "PgQuery__CmdType",
  "pg_query",
  9,
  pg_query__cmd_type__enum_values_by_number,
  9,
  pg_query__cmd_type__enum_values_by_name,
  1,
  pg_query__cmd_type__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__join_type__enum_values_by_number[10] =
{
  { "JOIN_TYPE_UNDEFINED", "PG_QUERY__JOIN_TYPE__JOIN_TYPE_UNDEFINED", 0 },
  { "JOIN_INNER", "PG_QUERY__JOIN_TYPE__JOIN_INNER", 1 },
  { "JOIN_LEFT", "PG_QUERY__JOIN_TYPE__JOIN_LEFT", 2 },
  { "JOIN_FULL", "PG_QUERY__JOIN_TYPE__JOIN_FULL", 3 },
  { "JOIN_RIGHT", "PG_QUERY__JOIN_TYPE__JOIN_RIGHT", 4 },
  { "JOIN_SEMI", "PG_QUERY__JOIN_TYPE__JOIN_SEMI", 5 },
  { "JOIN_ANTI", "PG_QUERY__JOIN_TYPE__JOIN_ANTI", 6 },
  { "JOIN_RIGHT_ANTI", "PG_QUERY__JOIN_TYPE__JOIN_RIGHT_ANTI", 7 },
  { "JOIN_UNIQUE_OUTER", "PG_QUERY__JOIN_TYPE__JOIN_UNIQUE_OUTER", 8 },
  { "JOIN_UNIQUE_INNER", "PG_QUERY__JOIN_TYPE__JOIN_UNIQUE_INNER", 9 },
};
static const ProtobufCIntRange pg_query__join_type__value_ranges[] = {
{0, 0},{0, 10}
};
static const ProtobufCEnumValueIndex pg_query__join_type__enum_values_by_name[10] =
{
  { "JOIN_ANTI", 6 },
  { "JOIN_FULL", 3 },
  { "JOIN_INNER", 1 },
  { "JOIN_LEFT", 2 },
  { "JOIN_RIGHT", 4 },
  { "JOIN_RIGHT_ANTI", 7 },
  { "JOIN_SEMI", 5 },
  { "JOIN_TYPE_UNDEFINED", 0 },
  { "JOIN_UNIQUE_INNER", 9 },
  { "JOIN_UNIQUE_OUTER", 8 },
};
const ProtobufCEnumDescriptor pg_query__join_type__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.JoinType",
  "JoinType",
  "PgQuery__JoinType",
  "pg_query",
  10,
  pg_query__join_type__enum_values_by_number,
  10,
  pg_query__join_type__enum_values_by_name,
  1,
  pg_query__join_type__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__agg_strategy__enum_values_by_number[5] =
{
  { "AGG_STRATEGY_UNDEFINED", "PG_QUERY__AGG_STRATEGY__AGG_STRATEGY_UNDEFINED", 0 },
  { "AGG_PLAIN", "PG_QUERY__AGG_STRATEGY__AGG_PLAIN", 1 },
  { "AGG_SORTED", "PG_QUERY__AGG_STRATEGY__AGG_SORTED", 2 },
  { "AGG_HASHED", "PG_QUERY__AGG_STRATEGY__AGG_HASHED", 3 },
  { "AGG_MIXED", "PG_QUERY__AGG_STRATEGY__AGG_MIXED", 4 },
};
static const ProtobufCIntRange pg_query__agg_strategy__value_ranges[] = {
{0, 0},{0, 5}
};
static const ProtobufCEnumValueIndex pg_query__agg_strategy__enum_values_by_name[5] =
{
  { "AGG_HASHED", 3 },
  { "AGG_MIXED", 4 },
  { "AGG_PLAIN", 1 },
  { "AGG_SORTED", 2 },
  { "AGG_STRATEGY_UNDEFINED", 0 },
};
const ProtobufCEnumDescriptor pg_query__agg_strategy__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.AggStrategy",
  "AggStrategy",
  "PgQuery__AggStrategy",
  "pg_query",
  5,
  pg_query__agg_strategy__enum_values_by_number,
  5,
  pg_query__agg_strategy__enum_values_by_name,
  1,
  pg_query__agg_strategy__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__agg_split__enum_values_by_number[4] =
{
  { "AGG_SPLIT_UNDEFINED", "PG_QUERY__AGG_SPLIT__AGG_SPLIT_UNDEFINED", 0 },
  { "AGGSPLIT_SIMPLE", "PG_QUERY__AGG_SPLIT__AGGSPLIT_SIMPLE", 1 },
  { "AGGSPLIT_INITIAL_SERIAL", "PG_QUERY__AGG_SPLIT__AGGSPLIT_INITIAL_SERIAL", 2 },
  { "AGGSPLIT_FINAL_DESERIAL", "PG_QUERY__AGG_SPLIT__AGGSPLIT_FINAL_DESERIAL", 3 },
};
static const ProtobufCIntRange pg_query__agg_split__value_ranges[] = {
{0, 0},{0, 4}
};
static const ProtobufCEnumValueIndex pg_query__agg_split__enum_values_by_name[4] =
{
  { "AGGSPLIT_FINAL_DESERIAL", 3 },
  { "AGGSPLIT_INITIAL_SERIAL", 2 },
  { "AGGSPLIT_SIMPLE", 1 },
  { "AGG_SPLIT_UNDEFINED", 0 },
};
const ProtobufCEnumDescriptor pg_query__agg_split__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.AggSplit",
  "AggSplit",
  "PgQuery__AggSplit",
  "pg_query",
  4,
  pg_query__agg_split__enum_values_by_number,
  4,
  pg_query__agg_split__enum_values_by_name,
  1,
  pg_query__agg_split__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__set_op_cmd__enum_values_by_number[5] =
{
  { "SET_OP_CMD_UNDEFINED", "PG_QUERY__SET_OP_CMD__SET_OP_CMD_UNDEFINED", 0 },
  { "SETOPCMD_INTERSECT", "PG_QUERY__SET_OP_CMD__SETOPCMD_INTERSECT", 1 },
  { "SETOPCMD_INTERSECT_ALL", "PG_QUERY__SET_OP_CMD__SETOPCMD_INTERSECT_ALL", 2 },
  { "SETOPCMD_EXCEPT", "PG_QUERY__SET_OP_CMD__SETOPCMD_EXCEPT", 3 },
  { "SETOPCMD_EXCEPT_ALL", "PG_QUERY__SET_OP_CMD__SETOPCMD_EXCEPT_ALL", 4 },
};
static const ProtobufCIntRange pg_query__set_op_cmd__value_ranges[] = {
{0, 0},{0, 5}
};
static const ProtobufCEnumValueIndex pg_query__set_op_cmd__enum_values_by_name[5] =
{
  { "SETOPCMD_EXCEPT", 3 },
  { "SETOPCMD_EXCEPT_ALL", 4 },
  { "SETOPCMD_INTERSECT", 1 },
  { "SETOPCMD_INTERSECT_ALL", 2 },
  { "SET_OP_CMD_UNDEFINED", 0 },
};
const ProtobufCEnumDescriptor pg_query__set_op_cmd__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.SetOpCmd",
  "SetOpCmd",
  "PgQuery__SetOpCmd",
  "pg_query",
  5,
  pg_query__set_op_cmd__enum_values_by_number,
  5,
  pg_query__set_op_cmd__enum_values_by_name,
  1,
  pg_query__set_op_cmd__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__set_op_strategy__enum_values_by_number[3] =
{
  { "SET_OP_STRATEGY_UNDEFINED", "PG_QUERY__SET_OP_STRATEGY__SET_OP_STRATEGY_UNDEFINED", 0 },
  { "SETOP_SORTED", "PG_QUERY__SET_OP_STRATEGY__SETOP_SORTED", 1 },
  { "SETOP_HASHED", "PG_QUERY__SET_OP_STRATEGY__SETOP_HASHED", 2 },
};
static const ProtobufCIntRange pg_query__set_op_strategy__value_ranges[] = {
{0, 0},{0, 3}
};
static const ProtobufCEnumValueIndex pg_query__set_op_strategy__enum_values_by_name[3] =
{
  { "SETOP_HASHED", 2 },
  { "SETOP_SORTED", 1 },
  { "SET_OP_STRATEGY_UNDEFINED", 0 },
};
const ProtobufCEnumDescriptor pg_query__set_op_strategy__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.SetOpStrategy",
  "SetOpStrategy",
  "PgQuery__SetOpStrategy",
  "pg_query",
  3,
  pg_query__set_op_strategy__enum_values_by_number,
  3,
  pg_query__set_op_strategy__enum_values_by_name,
  1,
  pg_query__set_op_strategy__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__on_conflict_action__enum_values_by_number[4] =
{
  { "ON_CONFLICT_ACTION_UNDEFINED", "PG_QUERY__ON_CONFLICT_ACTION__ON_CONFLICT_ACTION_UNDEFINED", 0 },
  { "ONCONFLICT_NONE", "PG_QUERY__ON_CONFLICT_ACTION__ONCONFLICT_NONE", 1 },
  { "ONCONFLICT_NOTHING", "PG_QUERY__ON_CONFLICT_ACTION__ONCONFLICT_NOTHING", 2 },
  { "ONCONFLICT_UPDATE", "PG_QUERY__ON_CONFLICT_ACTION__ONCONFLICT_UPDATE", 3 },
};
static const ProtobufCIntRange pg_query__on_conflict_action__value_ranges[] = {
{0, 0},{0, 4}
};
static const ProtobufCEnumValueIndex pg_query__on_conflict_action__enum_values_by_name[4] =
{
  { "ONCONFLICT_NONE", 1 },
  { "ONCONFLICT_NOTHING", 2 },
  { "ONCONFLICT_UPDATE", 3 },
  { "ON_CONFLICT_ACTION_UNDEFINED", 0 },
};
const ProtobufCEnumDescriptor pg_query__on_conflict_action__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.OnConflictAction",
  "OnConflictAction",
  "PgQuery__OnConflictAction",
  "pg_query",
  4,
  pg_query__on_conflict_action__enum_values_by_number,
  4,
  pg_query__on_conflict_action__enum_values_by_name,
  1,
  pg_query__on_conflict_action__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__limit_option__enum_values_by_number[4] =
{
  { "LIMIT_OPTION_UNDEFINED", "PG_QUERY__LIMIT_OPTION__LIMIT_OPTION_UNDEFINED", 0 },
  { "LIMIT_OPTION_DEFAULT", "PG_QUERY__LIMIT_OPTION__LIMIT_OPTION_DEFAULT", 1 },
  { "LIMIT_OPTION_COUNT", "PG_QUERY__LIMIT_OPTION__LIMIT_OPTION_COUNT", 2 },
  { "LIMIT_OPTION_WITH_TIES", "PG_QUERY__LIMIT_OPTION__LIMIT_OPTION_WITH_TIES", 3 },
};
static const ProtobufCIntRange pg_query__limit_option__value_ranges[] = {
{0, 0},{0, 4}
};
static const ProtobufCEnumValueIndex pg_query__limit_option__enum_values_by_name[4] =
{
  { "LIMIT_OPTION_COUNT", 2 },
  { "LIMIT_OPTION_DEFAULT", 1 },
  { "LIMIT_OPTION_UNDEFINED", 0 },
  { "LIMIT_OPTION_WITH_TIES", 3 },
};
const ProtobufCEnumDescriptor pg_query__limit_option__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.LimitOption",
  "LimitOption",
  "PgQuery__LimitOption",
  "pg_query",
  4,
  pg_query__limit_option__enum_values_by_number,
  4,
  pg_query__limit_option__enum_values_by_name,
  1,
  pg_query__limit_option__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__lock_clause_strength__enum_values_by_number[6] =
{
  { "LOCK_CLAUSE_STRENGTH_UNDEFINED", "PG_QUERY__LOCK_CLAUSE_STRENGTH__LOCK_CLAUSE_STRENGTH_UNDEFINED", 0 },
  { "LCS_NONE", "PG_QUERY__LOCK_CLAUSE_STRENGTH__LCS_NONE", 1 },
  { "LCS_FORKEYSHARE", "PG_QUERY__LOCK_CLAUSE_STRENGTH__LCS_FORKEYSHARE", 2 },
  { "LCS_FORSHARE", "PG_QUERY__LOCK_CLAUSE_STRENGTH__LCS_FORSHARE", 3 },
  { "LCS_FORNOKEYUPDATE", "PG_QUERY__LOCK_CLAUSE_STRENGTH__LCS_FORNOKEYUPDATE", 4 },
  { "LCS_FORUPDATE", "PG_QUERY__LOCK_CLAUSE_STRENGTH__LCS_FORUPDATE", 5 },
};
static const ProtobufCIntRange pg_query__lock_clause_strength__value_ranges[] = {
{0, 0},{0, 6}
};
static const ProtobufCEnumValueIndex pg_query__lock_clause_strength__enum_values_by_name[6] =
{
  { "LCS_FORKEYSHARE", 2 },
  { "LCS_FORNOKEYUPDATE", 4 },
  { "LCS_FORSHARE", 3 },
  { "LCS_FORUPDATE", 5 },
  { "LCS_NONE", 1 },
  { "LOCK_CLAUSE_STRENGTH_UNDEFINED", 0 },
};
const ProtobufCEnumDescriptor pg_query__lock_clause_strength__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.LockClauseStrength",
  "LockClauseStrength",
  "PgQuery__LockClauseStrength",
  "pg_query",
  6,
  pg_query__lock_clause_strength__enum_values_by_number,
  6,
  pg_query__lock_clause_strength__enum_values_by_name,
  1,
  pg_query__lock_clause_strength__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__lock_wait_policy__enum_values_by_number[4] =
{
  { "LOCK_WAIT_POLICY_UNDEFINED", "PG_QUERY__LOCK_WAIT_POLICY__LOCK_WAIT_POLICY_UNDEFINED", 0 },
  { "LockWaitBlock", "PG_QUERY__LOCK_WAIT_POLICY__LockWaitBlock", 1 },
  { "LockWaitSkip", "PG_QUERY__LOCK_WAIT_POLICY__LockWaitSkip", 2 },
  { "LockWaitError", "PG_QUERY__LOCK_WAIT_POLICY__LockWaitError", 3 },
};
static const ProtobufCIntRange pg_query__lock_wait_policy__value_ranges[] = {
{0, 0},{0, 4}
};
static const ProtobufCEnumValueIndex pg_query__lock_wait_policy__enum_values_by_name[4] =
{
  { "LOCK_WAIT_POLICY_UNDEFINED", 0 },
  { "LockWaitBlock", 1 },
  { "LockWaitError", 3 },
  { "LockWaitSkip", 2 },
};
const ProtobufCEnumDescriptor pg_query__lock_wait_policy__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.LockWaitPolicy",
  "LockWaitPolicy",
  "PgQuery__LockWaitPolicy",
  "pg_query",
  4,
  pg_query__lock_wait_policy__enum_values_by_number,
  4,
  pg_query__lock_wait_policy__enum_values_by_name,
  1,
  pg_query__lock_wait_policy__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__lock_tuple_mode__enum_values_by_number[5] =
{
  { "LOCK_TUPLE_MODE_UNDEFINED", "PG_QUERY__LOCK_TUPLE_MODE__LOCK_TUPLE_MODE_UNDEFINED", 0 },
  { "LockTupleKeyShare", "PG_QUERY__LOCK_TUPLE_MODE__LockTupleKeyShare", 1 },
  { "LockTupleShare", "PG_QUERY__LOCK_TUPLE_MODE__LockTupleShare", 2 },
  { "LockTupleNoKeyExclusive", "PG_QUERY__LOCK_TUPLE_MODE__LockTupleNoKeyExclusive", 3 },
  { "LockTupleExclusive", "PG_QUERY__LOCK_TUPLE_MODE__LockTupleExclusive", 4 },
};
static const ProtobufCIntRange pg_query__lock_tuple_mode__value_ranges[] = {
{0, 0},{0, 5}
};
static const ProtobufCEnumValueIndex pg_query__lock_tuple_mode__enum_values_by_name[5] =
{
  { "LOCK_TUPLE_MODE_UNDEFINED", 0 },
  { "LockTupleExclusive", 4 },
  { "LockTupleKeyShare", 1 },
  { "LockTupleNoKeyExclusive", 3 },
  { "LockTupleShare", 2 },
};
const ProtobufCEnumDescriptor pg_query__lock_tuple_mode__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.LockTupleMode",
  "LockTupleMode",
  "PgQuery__LockTupleMode",
  "pg_query",
  5,
  pg_query__lock_tuple_mode__enum_values_by_number,
  5,
  pg_query__lock_tuple_mode__enum_values_by_name,
  1,
  pg_query__lock_tuple_mode__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__keyword_kind__enum_values_by_number[5] =
{
  { "NO_KEYWORD", "PG_QUERY__KEYWORD_KIND__NO_KEYWORD", 0 },
  { "UNRESERVED_KEYWORD", "PG_QUERY__KEYWORD_KIND__UNRESERVED_KEYWORD", 1 },
  { "COL_NAME_KEYWORD", "PG_QUERY__KEYWORD_KIND__COL_NAME_KEYWORD", 2 },
  { "TYPE_FUNC_NAME_KEYWORD", "PG_QUERY__KEYWORD_KIND__TYPE_FUNC_NAME_KEYWORD", 3 },
  { "RESERVED_KEYWORD", "PG_QUERY__KEYWORD_KIND__RESERVED_KEYWORD", 4 },
};
static const ProtobufCIntRange pg_query__keyword_kind__value_ranges[] = {
{0, 0},{0, 5}
};
static const ProtobufCEnumValueIndex pg_query__keyword_kind__enum_values_by_name[5] =
{
  { "COL_NAME_KEYWORD", 2 },
  { "NO_KEYWORD", 0 },
  { "RESERVED_KEYWORD", 4 },
  { "TYPE_FUNC_NAME_KEYWORD", 3 },
  { "UNRESERVED_KEYWORD", 1 },
};
const ProtobufCEnumDescriptor pg_query__keyword_kind__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.KeywordKind",
  "KeywordKind",
  "PgQuery__KeywordKind",
  "pg_query",
  5,
  pg_query__keyword_kind__enum_values_by_number,
  5,
  pg_query__keyword_kind__enum_values_by_name,
  1,
  pg_query__keyword_kind__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCEnumValue pg_query__token__enum_values_by_number[542] =
{
  { "NUL", "PG_QUERY__TOKEN__NUL", 0 },
  { "ASCII_36", "PG_QUERY__TOKEN__ASCII_36", 36 },
  { "ASCII_37", "PG_QUERY__TOKEN__ASCII_37", 37 },
  { "ASCII_40", "PG_QUERY__TOKEN__ASCII_40", 40 },
  { "ASCII_41", "PG_QUERY__TOKEN__ASCII_41", 41 },
  { "ASCII_42", "PG_QUERY__TOKEN__ASCII_42", 42 },
  { "ASCII_43", "PG_QUERY__TOKEN__ASCII_43", 43 },
  { "ASCII_44", "PG_QUERY__TOKEN__ASCII_44", 44 },
  { "ASCII_45", "PG_QUERY__TOKEN__ASCII_45", 45 },
  { "ASCII_46", "PG_QUERY__TOKEN__ASCII_46", 46 },
  { "ASCII_47", "PG_QUERY__TOKEN__ASCII_47", 47 },
  { "ASCII_58", "PG_QUERY__TOKEN__ASCII_58", 58 },
  { "ASCII_59", "PG_QUERY__TOKEN__ASCII_59", 59 },
  { "ASCII_60", "PG_QUERY__TOKEN__ASCII_60", 60 },
  { "ASCII_61", "PG_QUERY__TOKEN__ASCII_61", 61 },
  { "ASCII_62", "PG_QUERY__TOKEN__ASCII_62", 62 },
  { "ASCII_63", "PG_QUERY__TOKEN__ASCII_63", 63 },
  { "ASCII_91", "PG_QUERY__TOKEN__ASCII_91", 91 },
  { "ASCII_92", "PG_QUERY__TOKEN__ASCII_92", 92 },
  { "ASCII_93", "PG_QUERY__TOKEN__ASCII_93", 93 },
  { "ASCII_94", "PG_QUERY__TOKEN__ASCII_94", 94 },
  { "IDENT", "PG_QUERY__TOKEN__IDENT", 258 },
  { "UIDENT", "PG_QUERY__TOKEN__UIDENT", 259 },
  { "FCONST", "PG_QUERY__TOKEN__FCONST", 260 },
  { "SCONST", "PG_QUERY__TOKEN__SCONST", 261 },
  { "USCONST", "PG_QUERY__TOKEN__USCONST", 262 },
  { "BCONST", "PG_QUERY__TOKEN__BCONST", 263 },
  { "XCONST", "PG_QUERY__TOKEN__XCONST", 264 },
  { "Op", "PG_QUERY__TOKEN__Op", 265 },
  { "ICONST", "PG_QUERY__TOKEN__ICONST", 266 },
  { "PARAM", "PG_QUERY__TOKEN__PARAM", 267 },
  { "TYPECAST", "PG_QUERY__TOKEN__TYPECAST", 268 },
  { "DOT_DOT", "PG_QUERY__TOKEN__DOT_DOT", 269 },
  { "COLON_EQUALS", "PG_QUERY__TOKEN__COLON_EQUALS", 270 },
  { "EQUALS_GREATER", "PG_QUERY__TOKEN__EQUALS_GREATER", 271 },
  { "LESS_EQUALS", "PG_QUERY__TOKEN__LESS_EQUALS", 272 },
  { "GREATER_EQUALS", "PG_QUERY__TOKEN__GREATER_EQUALS", 273 },
  { "NOT_EQUALS", "PG_QUERY__TOKEN__NOT_EQUALS", 274 },
  { "SQL_COMMENT", "PG_QUERY__TOKEN__SQL_COMMENT", 275 },
  { "C_COMMENT", "PG_QUERY__TOKEN__C_COMMENT", 276 },
  { "ABORT_P", "PG_QUERY__TOKEN__ABORT_P", 277 },
  { "ABSENT", "PG_QUERY__TOKEN__ABSENT", 278 },
  { "ABSOLUTE_P", "PG_QUERY__TOKEN__ABSOLUTE_P", 279 },
  { "ACCESS", "PG_QUERY__TOKEN__ACCESS", 280 },
  { "ACTION", "PG_QUERY__TOKEN__ACTION", 281 },
  { "ADD_P", "PG_QUERY__TOKEN__ADD_P", 282 },
  { "ADMIN", "PG_QUERY__TOKEN__ADMIN", 283 },
  { "AFTER", "PG_QUERY__TOKEN__AFTER", 284 },
  { "AGGREGATE", "PG_QUERY__TOKEN__AGGREGATE", 285 },
  { "ALL", "PG_QUERY__TOKEN__ALL", 286 },
  { "ALSO", "PG_QUERY__TOKEN__ALSO", 287 },
  { "ALTER", "PG_QUERY__TOKEN__ALTER", 288 },
  { "ALWAYS", "PG_QUERY__TOKEN__ALWAYS", 289 },
  { "ANALYSE", "PG_QUERY__TOKEN__ANALYSE", 290 },
  { "ANALYZE", "PG_QUERY__TOKEN__ANALYZE", 291 },
  { "AND", "PG_QUERY__TOKEN__AND", 292 },
  { "ANY", "PG_QUERY__TOKEN__ANY", 293 },
  { "ARRAY", "PG_QUERY__TOKEN__ARRAY", 294 },
  { "AS", "PG_QUERY__TOKEN__AS", 295 },
  { "ASC", "PG_QUERY__TOKEN__ASC", 296 },
  { "ASENSITIVE", "PG_QUERY__TOKEN__ASENSITIVE", 297 },
  { "ASSERTION", "PG_QUERY__TOKEN__ASSERTION", 298 },
  { "ASSIGNMENT", "PG_QUERY__TOKEN__ASSIGNMENT", 299 },
  { "ASYMMETRIC", "PG_QUERY__TOKEN__ASYMMETRIC", 300 },
  { "ATOMIC", "PG_QUERY__TOKEN__ATOMIC", 301 },
  { "AT", "PG_QUERY__TOKEN__AT", 302 },
  { "ATTACH", "PG_QUERY__TOKEN__ATTACH", 303 },
  { "ATTRIBUTE", "PG_QUERY__TOKEN__ATTRIBUTE", 304 },
  { "AUTHORIZATION", "PG_QUERY__TOKEN__AUTHORIZATION", 305 },
  { "BACKWARD", "PG_QUERY__TOKEN__BACKWARD", 306 },
  { "BEFORE", "PG_QUERY__TOKEN__BEFORE", 307 },
  { "BEGIN_P", "PG_QUERY__TOKEN__BEGIN_P", 308 },
  { "BETWEEN", "PG_QUERY__TOKEN__BETWEEN", 309 },
  { "BIGINT", "PG_QUERY__TOKEN__BIGINT", 310 },
  { "BINARY", "PG_QUERY__TOKEN__BINARY", 311 },
  { "BIT", "PG_QUERY__TOKEN__BIT", 312 },
  { "BOOLEAN_P", "PG_QUERY__TOKEN__BOOLEAN_P", 313 },
  { "BOTH", "PG_QUERY__TOKEN__BOTH", 314 },
  { "BREADTH", "PG_QUERY__TOKEN__BREADTH", 315 },
  { "BY", "PG_QUERY__TOKEN__BY", 316 },
  { "CACHE", "PG_QUERY__TOKEN__CACHE", 317 },
  { "CALL", "PG_QUERY__TOKEN__CALL", 318 },
  { "CALLED", "PG_QUERY__TOKEN__CALLED", 319 },
  { "CASCADE", "PG_QUERY__TOKEN__CASCADE", 320 },
  { "CASCADED", "PG_QUERY__TOKEN__CASCADED", 321 },
  { "CASE", "PG_QUERY__TOKEN__CASE", 322 },
  { "CAST", "PG_QUERY__TOKEN__CAST", 323 },
  { "CATALOG_P", "PG_QUERY__TOKEN__CATALOG_P", 324 },
  { "CHAIN", "PG_QUERY__TOKEN__CHAIN", 325 },
  { "CHAR_P", "PG_QUERY__TOKEN__CHAR_P", 326 },
  { "CHARACTER", "PG_QUERY__TOKEN__CHARACTER", 327 },
  { "CHARACTERISTICS", "PG_QUERY__TOKEN__CHARACTERISTICS", 328 },
  { "CHECK", "PG_QUERY__TOKEN__CHECK", 329 },
  { "CHECKPOINT", "PG_QUERY__TOKEN__CHECKPOINT", 330 },
  { "CLASS", "PG_QUERY__TOKEN__CLASS", 331 },
  { "CLOSE", "PG_QUERY__TOKEN__CLOSE", 332 },
  { "CLUSTER", "PG_QUERY__TOKEN__CLUSTER", 333 },
  { "COALESCE", "PG_QUERY__TOKEN__COALESCE", 334 },
  { "COLLATE", "PG_QUERY__TOKEN__COLLATE", 335 },
  { "COLLATION", "PG_QUERY__TOKEN__COLLATION", 336 },
  { "COLUMN", "PG_QUERY__TOKEN__COLUMN", 337 },
  { "COLUMNS", "PG_QUERY__TOKEN__COLUMNS", 338 },
  { "COMMENT", "PG_QUERY__TOKEN__COMMENT", 339 },
  { "COMMENTS", "PG_QUERY__TOKEN__COMMENTS", 340 },
  { "COMMIT", "PG_QUERY__TOKEN__COMMIT", 341 },
  { "COMMITTED", "PG_QUERY__TOKEN__COMMITTED", 342 },
  { "COMPRESSION", "PG_QUERY__TOKEN__COMPRESSION", 343 },
  { "CONCURRENTLY", "PG_QUERY__TOKEN__CONCURRENTLY", 344 },
  { "CONDITIONAL", "PG_QUERY__TOKEN__CONDITIONAL", 345 },
  { "CONFIGURATION", "PG_QUERY__TOKEN__CONFIGURATION", 346 },
  { "CONFLICT", "PG_QUERY__TOKEN__CONFLICT", 347 },
  { "CONNECTION", "PG_QUERY__TOKEN__CONNECTION", 348 },
  { "CONSTRAINT", "PG_QUERY__TOKEN__CONSTRAINT", 349 },
  { "CONSTRAINTS", "PG_QUERY__TOKEN__CONSTRAINTS", 350 },
  { "CONTENT_P", "PG_QUERY__TOKEN__CONTENT_P", 351 },
  { "CONTINUE_P", "PG_QUERY__TOKEN__CONTINUE_P", 352 },
  { "CONVERSION_P", "PG_QUERY__TOKEN__CONVERSION_P", 353 },
  { "COPY", "PG_QUERY__TOKEN__COPY", 354 },
  { "COST", "PG_QUERY__TOKEN__COST", 355 },
  { "CREATE", "PG_QUERY__TOKEN__CREATE", 356 },
  { "CROSS", "PG_QUERY__TOKEN__CROSS", 357 },
  { "CSV", "PG_QUERY__TOKEN__CSV", 358 },
  { "CUBE", "PG_QUERY__TOKEN__CUBE", 359 },
  { "CURRENT_P", "PG_QUERY__TOKEN__CURRENT_P", 360 },
  { "CURRENT_CATALOG", "PG_QUERY__TOKEN__CURRENT_CATALOG", 361 },
  { "CURRENT_DATE", "PG_QUERY__TOKEN__CURRENT_DATE", 362 },
  { "CURRENT_ROLE", "PG_QUERY__TOKEN__CURRENT_ROLE", 363 },
  { "CURRENT_SCHEMA", "PG_QUERY__TOKEN__CURRENT_SCHEMA", 364 },
  { "CURRENT_TIME", "PG_QUERY__TOKEN__CURRENT_TIME", 365 },
  { "CURRENT_TIMESTAMP", "PG_QUERY__TOKEN__CURRENT_TIMESTAMP", 366 },
  { "CURRENT_USER", "PG_QUERY__TOKEN__CURRENT_USER", 367 },
  { "CURSOR", "PG_QUERY__TOKEN__CURSOR", 368 },
  { "CYCLE", "PG_QUERY__TOKEN__CYCLE", 369 },
  { "DATA_P", "PG_QUERY__TOKEN__DATA_P", 370 },
  { "DATABASE", "PG_QUERY__TOKEN__DATABASE", 371 },
  { "DAY_P", "PG_QUERY__TOKEN__DAY_P", 372 },
  { "DEALLOCATE", "PG_QUERY__TOKEN__DEALLOCATE", 373 },
  { "DEC", "PG_QUERY__TOKEN__DEC", 374 },
  { "DECIMAL_P", "PG_QUERY__TOKEN__DECIMAL_P", 375 },
  { "DECLARE", "PG_QUERY__TOKEN__DECLARE", 376 },
  { "DEFAULT", "PG_QUERY__TOKEN__DEFAULT", 377 },
  { "DEFAULTS", "PG_QUERY__TOKEN__DEFAULTS", 378 },
  { "DEFERRABLE", "PG_QUERY__TOKEN__DEFERRABLE", 379 },
  { "DEFERRED", "PG_QUERY__TOKEN__DEFERRED", 380 },
  { "DEFINER", "PG_QUERY__TOKEN__DEFINER", 381 },
  { "DELETE_P", "PG_QUERY__TOKEN__DELETE_P", 382 },
  { "DELIMITER", "PG_QUERY__TOKEN__DELIMITER", 383 },
  { "DELIMITERS", "PG_QUERY__TOKEN__DELIMITERS", 384 },
  { "DEPENDS", "PG_QUERY__TOKEN__DEPENDS", 385 },
  { "DEPTH", "PG_QUERY__TOKEN__DEPTH", 386 },
  { "DESC", "PG_QUERY__TOKEN__DESC", 387 },
  { "DETACH", "PG_QUERY__TOKEN__DETACH", 388 },
  { "DICTIONARY", "PG_QUERY__TOKEN__DICTIONARY", 389 },
  { "DISABLE_P", "PG_QUERY__TOKEN__DISABLE_P", 390 },
  { "DISCARD", "PG_QUERY__TOKEN__DISCARD", 391 },
  { "DISTINCT", "PG_QUERY__TOKEN__DISTINCT", 392 },
  { "DO", "PG_QUERY__TOKEN__DO", 393 },
  { "DOCUMENT_P", "PG_QUERY__TOKEN__DOCUMENT_P", 394 },
  { "DOMAIN_P", "PG_QUERY__TOKEN__DOMAIN_P", 395 },
  { "DOUBLE_P", "PG_QUERY__TOKEN__DOUBLE_P", 396 },
  { "DROP", "PG_QUERY__TOKEN__DROP", 397 },
  { "EACH", "PG_QUERY__TOKEN__EACH", 398 },
  { "ELSE", "PG_QUERY__TOKEN__ELSE", 399 },
  { "EMPTY_P", "PG_QUERY__TOKEN__EMPTY_P", 400 },
  { "ENABLE_P", "PG_QUERY__TOKEN__ENABLE_P", 401 },
  { "ENCODING", "PG_QUERY__TOKEN__ENCODING", 402 },
  { "ENCRYPTED", "PG_QUERY__TOKEN__ENCRYPTED", 403 },
  { "END_P", "PG_QUERY__TOKEN__END_P", 404 },
  { "ENUM_P", "PG_QUERY__TOKEN__ENUM_P", 405 },
  { "ERROR_P", "PG_QUERY__TOKEN__ERROR_P", 406 },
  { "ESCAPE", "PG_QUERY__TOKEN__ESCAPE", 407 },
  { "EVENT", "PG_QUERY__TOKEN__EVENT", 408 },
  { "EXCEPT", "PG_QUERY__TOKEN__EXCEPT", 409 },
  { "EXCLUDE", "PG_QUERY__TOKEN__EXCLUDE", 410 },
  { "EXCLUDING", "PG_QUERY__TOKEN__EXCLUDING", 411 },
  { "EXCLUSIVE", "PG_QUERY__TOKEN__EXCLUSIVE", 412 },
  { "EXECUTE", "PG_QUERY__TOKEN__EXECUTE", 413 },
  { "EXISTS", "PG_QUERY__TOKEN__EXISTS", 414 },
  { "EXPLAIN", "PG_QUERY__TOKEN__EXPLAIN", 415 },
  { "EXPRESSION", "PG_QUERY__TOKEN__EXPRESSION", 416 },
  { "EXTENSION", "PG_QUERY__TOKEN__EXTENSION", 417 },
  { "EXTERNAL", "PG_QUERY__TOKEN__EXTERNAL", 418 },
  { "EXTRACT", "PG_QUERY__TOKEN__EXTRACT", 419 },
  { "FALSE_P", "PG_QUERY__TOKEN__FALSE_P", 420 },
  { "FAMILY", "PG_QUERY__TOKEN__FAMILY", 421 },
  { "FETCH", "PG_QUERY__TOKEN__FETCH", 422 },
  { "FILTER", "PG_QUERY__TOKEN__FILTER", 423 },
  { "FINALIZE", "PG_QUERY__TOKEN__FINALIZE", 424 },
  { "FIRST_P", "PG_QUERY__TOKEN__FIRST_P", 425 },
  { "FLOAT_P", "PG_QUERY__TOKEN__FLOAT_P", 426 },
  { "FOLLOWING", "PG_QUERY__TOKEN__FOLLOWING", 427 },
  { "FOR", "PG_QUERY__TOKEN__FOR", 428 },
  { "FORCE", "PG_QUERY__TOKEN__FORCE", 429 },
  { "FOREIGN", "PG_QUERY__TOKEN__FOREIGN", 430 },
  { "FORMAT", "PG_QUERY__TOKEN__FORMAT", 431 },
  { "FORWARD", "PG_QUERY__TOKEN__FORWARD", 432 },
  { "FREEZE", "PG_QUERY__TOKEN__FREEZE", 433 },
  { "FROM", "PG_QUERY__TOKEN__FROM", 434 },
  { "FULL", "PG_QUERY__TOKEN__FULL", 435 },
  { "FUNCTION", "PG_QUERY__TOKEN__FUNCTION", 436 },
  { "FUNCTIONS", "PG_QUERY__TOKEN__FUNCTIONS", 437 },
  { "GENERATED", "PG_QUERY__TOKEN__GENERATED", 438 },
  { "GLOBAL", "PG_QUERY__TOKEN__GLOBAL", 439 },
  { "GRANT", "PG_QUERY__TOKEN__GRANT", 440 },
  { "GRANTED", "PG_QUERY__TOKEN__GRANTED", 441 },
  { "GREATEST", "PG_QUERY__TOKEN__GREATEST", 442 },
  { "GROUP_P", "PG_QUERY__TOKEN__GROUP_P", 443 },
  { "GROUPING", "PG_QUERY__TOKEN__GROUPING", 444 },
  { "GROUPS", "PG_QUERY__TOKEN__GROUPS", 445 },
  { "HANDLER", "PG_QUERY__TOKEN__HANDLER", 446 },
  { "HAVING", "PG_QUERY__TOKEN__HAVING", 447 },
  { "HEADER_P", "PG_QUERY__TOKEN__HEADER_P", 448 },
  { "HOLD", "PG_QUERY__TOKEN__HOLD", 449 },
  { "HOUR_P", "PG_QUERY__TOKEN__HOUR_P", 450 },
  { "IDENTITY_P", "PG_QUERY__TOKEN__IDENTITY_P", 451 },
  { "IF_P", "PG_QUERY__TOKEN__IF_P", 452 },
  { "ILIKE", "PG_QUERY__TOKEN__ILIKE", 453 },
  { "IMMEDIATE", "PG_QUERY__TOKEN__IMMEDIATE", 454 },
  { "IMMUTABLE", "PG_QUERY__TOKEN__IMMUTABLE", 455 },
  { "IMPLICIT_P", "PG_QUERY__TOKEN__IMPLICIT_P", 456 },
  { "IMPORT_P", "PG_QUERY__TOKEN__IMPORT_P", 457 },
  { "IN_P", "PG_QUERY__TOKEN__IN_P", 458 },
  { "INCLUDE", "PG_QUERY__TOKEN__INCLUDE", 459 },
  { "INCLUDING", "PG_QUERY__TOKEN__INCLUDING", 460 },
  { "INCREMENT", "PG_QUERY__TOKEN__INCREMENT", 461 },
  { "INDENT", "PG_QUERY__TOKEN__INDENT", 462 },
  { "INDEX", "PG_QUERY__TOKEN__INDEX", 463 },
  { "INDEXES", "PG_QUERY__TOKEN__INDEXES", 464 },
  { "INHERIT", "PG_QUERY__TOKEN__INHERIT", 465 },
  { "INHERITS", "PG_QUERY__TOKEN__INHERITS", 466 },
  { "INITIALLY", "PG_QUERY__TOKEN__INITIALLY", 467 },
  { "INLINE_P", "PG_QUERY__TOKEN__INLINE_P", 468 },
  { "INNER_P", "PG_QUERY__TOKEN__INNER_P", 469 },
  { "INOUT", "PG_QUERY__TOKEN__INOUT", 470 },
  { "INPUT_P", "PG_QUERY__TOKEN__INPUT_P", 471 },
  { "INSENSITIVE", "PG_QUERY__TOKEN__INSENSITIVE", 472 },
  { "INSERT", "PG_QUERY__TOKEN__INSERT", 473 },
  { "INSTEAD", "PG_QUERY__TOKEN__INSTEAD", 474 },
  { "INT_P", "PG_QUERY__TOKEN__INT_P", 475 },
  { "INTEGER", "PG_QUERY__TOKEN__INTEGER", 476 },
  { "INTERSECT", "PG_QUERY__TOKEN__INTERSECT", 477 },
  { "INTERVAL", "PG_QUERY__TOKEN__INTERVAL", 478 },
  { "INTO", "PG_QUERY__TOKEN__INTO", 479 },
  { "INVOKER", "PG_QUERY__TOKEN__INVOKER", 480 },
  { "IS", "PG_QUERY__TOKEN__IS", 481 },
  { "ISNULL", "PG_QUERY__TOKEN__ISNULL", 482 },
  { "ISOLATION", "PG_QUERY__TOKEN__ISOLATION", 483 },
  { "JOIN", "PG_QUERY__TOKEN__JOIN", 484 },
  { "JSON", "PG_QUERY__TOKEN__JSON", 485 },
  { "JSON_ARRAY", "PG_QUERY__TOKEN__JSON_ARRAY", 486 },
  { "JSON_ARRAYAGG", "PG_QUERY__TOKEN__JSON_ARRAYAGG", 487 },
  { "JSON_EXISTS", "PG_QUERY__TOKEN__JSON_EXISTS", 488 },
  { "JSON_OBJECT", "PG_QUERY__TOKEN__JSON_OBJECT", 489 },
  { "JSON_OBJECTAGG", "PG_QUERY__TOKEN__JSON_OBJECTAGG", 490 },
  { "JSON_QUERY", "PG_QUERY__TOKEN__JSON_QUERY", 491 },
  { "JSON_SCALAR", "PG_QUERY__TOKEN__JSON_SCALAR", 492 },
  { "JSON_SERIALIZE", "PG_QUERY__TOKEN__JSON_SERIALIZE", 493 },
  { "JSON_TABLE", "PG_QUERY__TOKEN__JSON_TABLE", 494 },
  { "JSON_VALUE", "PG_QUERY__TOKEN__JSON_VALUE", 495 },
  { "KEEP", "PG_QUERY__TOKEN__KEEP", 496 },
  { "KEY", "PG_QUERY__TOKEN__KEY", 497 },
  { "KEYS", "PG_QUERY__TOKEN__KEYS", 498 },
  { "LABEL", "PG_QUERY__TOKEN__LABEL", 499 },
  { "LANGUAGE", "PG_QUERY__TOKEN__LANGUAGE", 500 },
  { "LARGE_P", "PG_QUERY__TOKEN__LARGE_P", 501 },
  { "LAST_P", "PG_QUERY__TOKEN__LAST_P", 502 },
  { "LATERAL_P", "PG_QUERY__TOKEN__LATERAL_P", 503 },
  { "LEADING", "PG_QUERY__TOKEN__LEADING", 504 },
  { "LEAKPROOF", "PG_QUERY__TOKEN__LEAKPROOF", 505 },
  { "LEAST", "PG_QUERY__TOKEN__LEAST", 506 },
  { "LEFT", "PG_QUERY__TOKEN__LEFT", 507 },
  { "LEVEL", "PG_QUERY__TOKEN__LEVEL", 508 },
  { "LIKE", "PG_QUERY__TOKEN__LIKE", 509 },
  { "LIMIT", "PG_QUERY__TOKEN__LIMIT", 510 },
  { "LISTEN", "PG_QUERY__TOKEN__LISTEN", 511 },
  { "LOAD", "PG_QUERY__TOKEN__LOAD", 512 },
  { "LOCAL", "PG_QUERY__TOKEN__LOCAL", 513 },
  { "LOCALTIME", "PG_QUERY__TOKEN__LOCALTIME", 514 },
  { "LOCALTIMESTAMP", "PG_QUERY__TOKEN__LOCALTIMESTAMP", 515 },
  { "LOCATION", "PG_QUERY__TOKEN__LOCATION", 516 },
  { "LOCK_P", "PG_QUERY__TOKEN__LOCK_P", 517 },
  { "LOCKED", "PG_QUERY__TOKEN__LOCKED", 518 },
  { "LOGGED", "PG_QUERY__TOKEN__LOGGED", 519 },
  { "MAPPING", "PG_QUERY__TOKEN__MAPPING", 520 },
  { "MATCH", "PG_QUERY__TOKEN__MATCH", 521 },
  { "MATCHED", "PG_QUERY__TOKEN__MATCHED", 522 },
  { "MATERIALIZED", "PG_QUERY__TOKEN__MATERIALIZED", 523 },
  { "MAXVALUE", "PG_QUERY__TOKEN__MAXVALUE", 524 },
  { "MERGE", "PG_QUERY__TOKEN__MERGE", 525 },
  { "MERGE_ACTION", "PG_QUERY__TOKEN__MERGE_ACTION", 526 },
  { "METHOD", "PG_QUERY__TOKEN__METHOD", 527 },
  { "MINUTE_P", "PG_QUERY__TOKEN__MINUTE_P", 528 },
  { "MINVALUE", "PG_QUERY__TOKEN__MINVALUE", 529 },
  { "MODE", "PG_QUERY__TOKEN__MODE", 530 },
  { "MONTH_P", "PG_QUERY__TOKEN__MONTH_P", 531 },
  { "MOVE", "PG_QUERY__TOKEN__MOVE", 532 },
  { "NAME_P", "PG_QUERY__TOKEN__NAME_P", 533 },
  { "NAMES", "PG_QUERY__TOKEN__NAMES", 534 },
  { "NATIONAL", "PG_QUERY__TOKEN__NATIONAL", 535 },
  { "NATURAL", "PG_QUERY__TOKEN__NATURAL", 536 },
  { "NCHAR", "PG_QUERY__TOKEN__NCHAR", 537 },
  { "NESTED", "PG_QUERY__TOKEN__NESTED", 538 },
  { "NEW", "PG_QUERY__TOKEN__NEW", 539 },
  { "NEXT", "PG_QUERY__TOKEN__NEXT", 540 },
  { "NFC", "PG_QUERY__TOKEN__NFC", 541 },
  { "NFD", "PG_QUERY__TOKEN__NFD", 542 },
  { "NFKC", "PG_QUERY__TOKEN__NFKC", 543 },
  { "NFKD", "PG_QUERY__TOKEN__NFKD", 544 },
  { "NO", "PG_QUERY__TOKEN__NO", 545 },
  { "NONE", "PG_QUERY__TOKEN__NONE", 546 },
  { "NORMALIZE", "PG_QUERY__TOKEN__NORMALIZE", 547 },
  { "NORMALIZED", "PG_QUERY__TOKEN__NORMALIZED", 548 },
  { "NOT", "PG_QUERY__TOKEN__NOT", 549 },
  { "NOTHING", "PG_QUERY__TOKEN__NOTHING", 550 },
  { "NOTIFY", "PG_QUERY__TOKEN__NOTIFY", 551 },
  { "NOTNULL", "PG_QUERY__TOKEN__NOTNULL", 552 },
  { "NOWAIT", "PG_QUERY__TOKEN__NOWAIT", 553 },
  { "NULL_P", "PG_QUERY__TOKEN__NULL_P", 554 },
  { "NULLIF", "PG_QUERY__TOKEN__NULLIF", 555 },
  { "NULLS_P", "PG_QUERY__TOKEN__NULLS_P", 556 },
  { "NUMERIC", "PG_QUERY__TOKEN__NUMERIC", 557 },
  { "OBJECT_P", "PG_QUERY__TOKEN__OBJECT_P", 558 },
  { "OF", "PG_QUERY__TOKEN__OF", 559 },
  { "OFF", "PG_QUERY__TOKEN__OFF", 560 },
  { "OFFSET", "PG_QUERY__TOKEN__OFFSET", 561 },
  { "OIDS", "PG_QUERY__TOKEN__OIDS", 562 },
  { "OLD", "PG_QUERY__TOKEN__OLD", 563 },
  { "OMIT", "PG_QUERY__TOKEN__OMIT", 564 },
  { "ON", "PG_QUERY__TOKEN__ON", 565 },
  { "ONLY", "PG_QUERY__TOKEN__ONLY", 566 },
  { "OPERATOR", "PG_QUERY__TOKEN__OPERATOR", 567 },
  { "OPTION", "PG_QUERY__TOKEN__OPTION", 568 },
  { "OPTIONS", "PG_QUERY__TOKEN__OPTIONS", 569 },
  { "OR", "PG_QUERY__TOKEN__OR", 570 },
  { "ORDER", "PG_QUERY__TOKEN__ORDER", 571 },
  { "ORDINALITY", "PG_QUERY__TOKEN__ORDINALITY", 572 },
  { "OTHERS", "PG_QUERY__TOKEN__OTHERS", 573 },
  { "OUT_P", "PG_QUERY__TOKEN__OUT_P", 574 },
  { "OUTER_P", "PG_QUERY__TOKEN__OUTER_P", 575 },
  { "OVER", "PG_QUERY__TOKEN__OVER", 576 },
  { "OVERLAPS", "PG_QUERY__TOKEN__OVERLAPS", 577 },
  { "OVERLAY", "PG_QUERY__TOKEN__OVERLAY", 578 },
  { "OVERRIDING", "PG_QUERY__TOKEN__OVERRIDING", 579 },
  { "OWNED", "PG_QUERY__TOKEN__OWNED", 580 },
  { "OWNER", "PG_QUERY__TOKEN__OWNER", 581 },
  { "PARALLEL", "PG_QUERY__TOKEN__PARALLEL", 582 },
  { "PARAMETER", "PG_QUERY__TOKEN__PARAMETER", 583 },
  { "PARSER", "PG_QUERY__TOKEN__PARSER", 584 },
  { "PARTIAL", "PG_QUERY__TOKEN__PARTIAL", 585 },
  { "PARTITION", "PG_QUERY__TOKEN__PARTITION", 586 },
  { "PASSING", "PG_QUERY__TOKEN__PASSING", 587 },
  { "PASSWORD", "PG_QUERY__TOKEN__PASSWORD", 588 },
  { "PATH", "PG_QUERY__TOKEN__PATH", 589 },
  { "PLACING", "PG_QUERY__TOKEN__PLACING", 590 },
  { "PLAN", "PG_QUERY__TOKEN__PLAN", 591 },
  { "PLANS", "PG_QUERY__TOKEN__PLANS", 592 },
  { "POLICY", "PG_QUERY__TOKEN__POLICY", 593 },
  { "POSITION", "PG_QUERY__TOKEN__POSITION", 594 },
  { "PRECEDING", "PG_QUERY__TOKEN__PRECEDING", 595 },
  { "PRECISION", "PG_QUERY__TOKEN__PRECISION", 596 },
  { "PRESERVE", "PG_QUERY__TOKEN__PRESERVE", 597 },
  { "PREPARE", "PG_QUERY__TOKEN__PREPARE", 598 },
  { "PREPARED", "PG_QUERY__TOKEN__PREPARED", 599 },
  { "PRIMARY", "PG_QUERY__TOKEN__PRIMARY", 600 },
  { "PRIOR", "PG_QUERY__TOKEN__PRIOR", 601 },
  { "PRIVILEGES", "PG_QUERY__TOKEN__PRIVILEGES", 602 },
  { "PROCEDURAL", "PG_QUERY__TOKEN__PROCEDURAL", 603 },
  { "PROCEDURE", "PG_QUERY__TOKEN__PROCEDURE", 604 },
  { "PROCEDURES", "PG_QUERY__TOKEN__PROCEDURES", 605 },
  { "PROGRAM", "PG_QUERY__TOKEN__PROGRAM", 606 },
  { "PUBLICATION", "PG_QUERY__TOKEN__PUBLICATION", 607 },
  { "QUOTE", "PG_QUERY__TOKEN__QUOTE", 608 },
  { "QUOTES", "PG_QUERY__TOKEN__QUOTES", 609 },
  { "RANGE", "PG_QUERY__TOKEN__RANGE", 610 },
  { "READ", "PG_QUERY__TOKEN__READ", 611 },
  { "REAL", "PG_QUERY__TOKEN__REAL", 612 },
  { "REASSIGN", "PG_QUERY__TOKEN__REASSIGN", 613 },
  { "RECHECK", "PG_QUERY__TOKEN__RECHECK", 614 },
  { "RECURSIVE", "PG_QUERY__TOKEN__RECURSIVE", 615 },
  { "REF_P", "PG_QUERY__TOKEN__REF_P", 616 },
  { "REFERENCES", "PG_QUERY__TOKEN__REFERENCES", 617 },
  { "REFERENCING", "PG_QUERY__TOKEN__REFERENCING", 618 },
  { "REFRESH", "PG_QUERY__TOKEN__REFRESH", 619 },
  { "REINDEX", "PG_QUERY__TOKEN__REINDEX", 620 },
  { "RELATIVE_P", "PG_QUERY__TOKEN__RELATIVE_P", 621 },
  { "RELEASE", "PG_QUERY__TOKEN__RELEASE", 622 },
  { "RENAME", "PG_QUERY__TOKEN__RENAME", 623 },
  { "REPEATABLE", "PG_QUERY__TOKEN__REPEATABLE", 624 },
  { "REPLACE", "PG_QUERY__TOKEN__REPLACE", 625 },
  { "REPLICA", "PG_QUERY__TOKEN__REPLICA", 626 },
  { "RESET", "PG_QUERY__TOKEN__RESET", 627 },
  { "RESTART", "PG_QUERY__TOKEN__RESTART", 628 },
  { "RESTRICT", "PG_QUERY__TOKEN__RESTRICT", 629 },
  { "RETURN", "PG_QUERY__TOKEN__RETURN", 630 },
  { "RETURNING", "PG_QUERY__TOKEN__RETURNING", 631 },
  { "RETURNS", "PG_QUERY__TOKEN__RETURNS", 632 },
  { "REVOKE", "PG_QUERY__TOKEN__REVOKE", 633 },
  { "RIGHT", "PG_QUERY__TOKEN__RIGHT", 634 },
  { "ROLE", "PG_QUERY__TOKEN__ROLE", 635 },
  { "ROLLBACK", "PG_QUERY__TOKEN__ROLLBACK", 636 },
  { "ROLLUP", "PG_QUERY__TOKEN__ROLLUP", 637 },
  { "ROUTINE", "PG_QUERY__TOKEN__ROUTINE", 638 },
  { "ROUTINES", "PG_QUERY__TOKEN__ROUTINES", 639 },
  { "ROW", "PG_QUERY__TOKEN__ROW", 640 },
  { "ROWS", "PG_QUERY__TOKEN__ROWS", 641 },
  { "RULE", "PG_QUERY__TOKEN__RULE", 642 },
  { "SAVEPOINT", "PG_QUERY__TOKEN__SAVEPOINT", 643 },
  { "SCALAR", "PG_QUERY__TOKEN__SCALAR", 644 },
  { "SCHEMA", "PG_QUERY__TOKEN__SCHEMA", 645 },
  { "SCHEMAS", "PG_QUERY__TOKEN__SCHEMAS", 646 },
  { "SCROLL", "PG_QUERY__TOKEN__SCROLL", 647 },
  { "SEARCH", "PG_QUERY__TOKEN__SEARCH", 648 },
  { "SECOND_P", "PG_QUERY__TOKEN__SECOND_P", 649 },
  { "SECURITY", "PG_QUERY__TOKEN__SECURITY", 650 },
  { "SELECT", "PG_QUERY__TOKEN__SELECT", 651 },
  { "SEQUENCE", "PG_QUERY__TOKEN__SEQUENCE", 652 },
  { "SEQUENCES", "PG_QUERY__TOKEN__SEQUENCES", 653 },
  { "SERIALIZABLE", "PG_QUERY__TOKEN__SERIALIZABLE", 654 },
  { "SERVER", "PG_QUERY__TOKEN__SERVER", 655 },
  { "SESSION", "PG_QUERY__TOKEN__SESSION", 656 },
  { "SESSION_USER", "PG_QUERY__TOKEN__SESSION_USER", 657 },
  { "SET", "PG_QUERY__TOKEN__SET", 658 },
  { "SETS", "PG_QUERY__TOKEN__SETS", 659 },
  { "SETOF", "PG_QUERY__TOKEN__SETOF", 660 },
  { "SHARE", "PG_QUERY__TOKEN__SHARE", 661 },
  { "SHOW", "PG_QUERY__TOKEN__SHOW", 662 },
  { "SIMILAR", "PG_QUERY__TOKEN__SIMILAR", 663 },
  { "SIMPLE", "PG_QUERY__TOKEN__SIMPLE", 664 },
  { "SKIP", "PG_QUERY__TOKEN__SKIP", 665 },
  { "SMALLINT", "PG_QUERY__TOKEN__SMALLINT", 666 },
  { "SNAPSHOT", "PG_QUERY__TOKEN__SNAPSHOT", 667 },
  { "SOME", "PG_QUERY__TOKEN__SOME", 668 },
  { "SOURCE", "PG_QUERY__TOKEN__SOURCE", 669 },
  { "SQL_P", "PG_QUERY__TOKEN__SQL_P", 670 },
  { "STABLE", "PG_QUERY__TOKEN__STABLE", 671 },
  { "STANDALONE_P", "PG_QUERY__TOKEN__STANDALONE_P", 672 },
  { "START", "PG_QUERY__TOKEN__START", 673 },
  { "STATEMENT", "PG_QUERY__TOKEN__STATEMENT", 674 },
  { "STATISTICS", "PG_QUERY__TOKEN__STATISTICS", 675 },
  { "STDIN", "PG_QUERY__TOKEN__STDIN", 676 },
  { "STDOUT", "PG_QUERY__TOKEN__STDOUT", 677 },
  { "STORAGE", "PG_QUERY__TOKEN__STORAGE", 678 },
  { "STORED", "PG_QUERY__TOKEN__STORED", 679 },
  { "STRICT_P", "PG_QUERY__TOKEN__STRICT_P", 680 },
  { "STRING_P", "PG_QUERY__TOKEN__STRING_P", 681 },
  { "STRIP_P", "PG_QUERY__TOKEN__STRIP_P", 682 },
  { "SUBSCRIPTION", "PG_QUERY__TOKEN__SUBSCRIPTION", 683 },
  { "SUBSTRING", "PG_QUERY__TOKEN__SUBSTRING", 684 },
  { "SUPPORT", "PG_QUERY__TOKEN__SUPPORT", 685 },
  { "SYMMETRIC", "PG_QUERY__TOKEN__SYMMETRIC", 686 },
  { "SYSID", "PG_QUERY__TOKEN__SYSID", 687 },
  { "SYSTEM_P", "PG_QUERY__TOKEN__SYSTEM_P", 688 },
  { "SYSTEM_USER", "PG_QUERY__TOKEN__SYSTEM_USER", 689 },
  { "TABLE", "PG_QUERY__TOKEN__TABLE", 690 },
  { "TABLES", "PG_QUERY__TOKEN__TABLES", 691 },
  { "TABLESAMPLE", "PG_QUERY__TOKEN__TABLESAMPLE", 692 },
  { "TABLESPACE", "PG_QUERY__TOKEN__TABLESPACE", 693 },
  { "TARGET", "PG_QUERY__TOKEN__TARGET", 694 },
  { "TEMP", "PG_QUERY__TOKEN__TEMP", 695 },
  { "TEMPLATE", "PG_QUERY__TOKEN__TEMPLATE", 696 },
  { "TEMPORARY", "PG_QUERY__TOKEN__TEMPORARY", 697 },
  { "TEXT_P", "PG_QUERY__TOKEN__TEXT_P", 698 },
  { "THEN", "PG_QUERY__TOKEN__THEN", 699 },
  { "TIES", "PG_QUERY__TOKEN__TIES", 700 },
  { "TIME", "PG_QUERY__TOKEN__TIME", 701 },
  { "TIMESTAMP", "PG_QUERY__TOKEN__TIMESTAMP", 702 },
  { "TO", "PG_QUERY__TOKEN__TO", 703 },
  { "TRAILING", "PG_QUERY__TOKEN__TRAILING", 704 },
  { "TRANSACTION", "PG_QUERY__TOKEN__TRANSACTION", 705 },
  { "TRANSFORM", "PG_QUERY__TOKEN__TRANSFORM", 706 },
  { "TREAT", "PG_QUERY__TOKEN__TREAT", 707 },
  { "TRIGGER", "PG_QUERY__TOKEN__TRIGGER", 708 },
  { "TRIM", "PG_QUERY__TOKEN__TRIM", 709 },
  { "TRUE_P", "PG_QUERY__TOKEN__TRUE_P", 710 },
  { "TRUNCATE", "PG_QUERY__TOKEN__TRUNCATE", 711 },
  { "TRUSTED", "PG_QUERY__TOKEN__TRUSTED", 712 },
  { "TYPE_P", "PG_QUERY__TOKEN__TYPE_P", 713 },
  { "TYPES_P", "PG_QUERY__TOKEN__TYPES_P", 714 },
  { "UESCAPE", "PG_QUERY__TOKEN__UESCAPE", 715 },
  { "UNBOUNDED", "PG_QUERY__TOKEN__UNBOUNDED", 716 },
  { "UNCONDITIONAL", "PG_QUERY__TOKEN__UNCONDITIONAL", 717 },
  { "UNCOMMITTED", "PG_QUERY__TOKEN__UNCOMMITTED", 718 },
  { "UNENCRYPTED", "PG_QUERY__TOKEN__UNENCRYPTED", 719 },
  { "UNION", "PG_QUERY__TOKEN__UNION", 720 },
  { "UNIQUE", "PG_QUERY__TOKEN__UNIQUE", 721 },
  { "UNKNOWN", "PG_QUERY__TOKEN__UNKNOWN", 722 },
  { "UNLISTEN", "PG_QUERY__TOKEN__UNLISTEN", 723 },
  { "UNLOGGED", "PG_QUERY__TOKEN__UNLOGGED", 724 },
  { "UNTIL", "PG_QUERY__TOKEN__UNTIL", 725 },
  { "UPDATE", "PG_QUERY__TOKEN__UPDATE", 726 },
  { "USER", "PG_QUERY__TOKEN__USER", 727 },
  { "USING", "PG_QUERY__TOKEN__USING", 728 },
  { "VACUUM", "PG_QUERY__TOKEN__VACUUM", 729 },
  { "VALID", "PG_QUERY__TOKEN__VALID", 730 },
  { "VALIDATE", "PG_QUERY__TOKEN__VALIDATE", 731 },
  { "VALIDATOR", "PG_QUERY__TOKEN__VALIDATOR", 732 },
  { "VALUE_P", "PG_QUERY__TOKEN__VALUE_P", 733 },
  { "VALUES", "PG_QUERY__TOKEN__VALUES", 734 },
  { "VARCHAR", "PG_QUERY__TOKEN__VARCHAR", 735 },
  { "VARIADIC", "PG_QUERY__TOKEN__VARIADIC", 736 },
  { "VARYING", "PG_QUERY__TOKEN__VARYING", 737 },
  { "VERBOSE", "PG_QUERY__TOKEN__VERBOSE", 738 },
  { "VERSION_P", "PG_QUERY__TOKEN__VERSION_P", 739 },
  { "VIEW", "PG_QUERY__TOKEN__VIEW", 740 },
  { "VIEWS", "PG_QUERY__TOKEN__VIEWS", 741 },
  { "VOLATILE", "PG_QUERY__TOKEN__VOLATILE", 742 },
  { "WHEN", "PG_QUERY__TOKEN__WHEN", 743 },
  { "WHERE", "PG_QUERY__TOKEN__WHERE", 744 },
  { "WHITESPACE_P", "PG_QUERY__TOKEN__WHITESPACE_P", 745 },
  { "WINDOW", "PG_QUERY__TOKEN__WINDOW", 746 },
  { "WITH", "PG_QUERY__TOKEN__WITH", 747 },
  { "WITHIN", "PG_QUERY__TOKEN__WITHIN", 748 },
  { "WITHOUT", "PG_QUERY__TOKEN__WITHOUT", 749 },
  { "WORK", "PG_QUERY__TOKEN__WORK", 750 },
  { "WRAPPER", "PG_QUERY__TOKEN__WRAPPER", 751 },
  { "WRITE", "PG_QUERY__TOKEN__WRITE", 752 },
  { "XML_P", "PG_QUERY__TOKEN__XML_P", 753 },
  { "XMLATTRIBUTES", "PG_QUERY__TOKEN__XMLATTRIBUTES", 754 },
  { "XMLCONCAT", "PG_QUERY__TOKEN__XMLCONCAT", 755 },
  { "XMLELEMENT", "PG_QUERY__TOKEN__XMLELEMENT", 756 },
  { "XMLEXISTS", "PG_QUERY__TOKEN__XMLEXISTS", 757 },
  { "XMLFOREST", "PG_QUERY__TOKEN__XMLFOREST", 758 },
  { "XMLNAMESPACES", "PG_QUERY__TOKEN__XMLNAMESPACES", 759 },
  { "XMLPARSE", "PG_QUERY__TOKEN__XMLPARSE", 760 },
  { "XMLPI", "PG_QUERY__TOKEN__XMLPI", 761 },
  { "XMLROOT", "PG_QUERY__TOKEN__XMLROOT", 762 },
  { "XMLSERIALIZE", "PG_QUERY__TOKEN__XMLSERIALIZE", 763 },
  { "XMLTABLE", "PG_QUERY__TOKEN__XMLTABLE", 764 },
  { "YEAR_P", "PG_QUERY__TOKEN__YEAR_P", 765 },
  { "YES_P", "PG_QUERY__TOKEN__YES_P", 766 },
  { "ZONE", "PG_QUERY__TOKEN__ZONE", 767 },
  { "FORMAT_LA", "PG_QUERY__TOKEN__FORMAT_LA", 768 },
  { "NOT_LA", "PG_QUERY__TOKEN__NOT_LA", 769 },
  { "NULLS_LA", "PG_QUERY__TOKEN__NULLS_LA", 770 },
  { "WITH_LA", "PG_QUERY__TOKEN__WITH_LA", 771 },
  { "WITHOUT_LA", "PG_QUERY__TOKEN__WITHOUT_LA", 772 },
  { "MODE_TYPE_NAME", "PG_QUERY__TOKEN__MODE_TYPE_NAME", 773 },
  { "MODE_PLPGSQL_EXPR", "PG_QUERY__TOKEN__MODE_PLPGSQL_EXPR", 774 },
  { "MODE_PLPGSQL_ASSIGN1", "PG_QUERY__TOKEN__MODE_PLPGSQL_ASSIGN1", 775 },
  { "MODE_PLPGSQL_ASSIGN2", "PG_QUERY__TOKEN__MODE_PLPGSQL_ASSIGN2", 776 },
  { "MODE_PLPGSQL_ASSIGN3", "PG_QUERY__TOKEN__MODE_PLPGSQL_ASSIGN3", 777 },
  { "UMINUS", "PG_QUERY__TOKEN__UMINUS", 778 },
};
static const ProtobufCIntRange pg_query__token__value_ranges[] = {
{0, 0},{36, 1},{40, 3},{58, 11},{91, 17},{258, 21},{0, 542}
};
static const ProtobufCEnumValueIndex pg_query__token__enum_values_by_name[542] =
{
  { "ABORT_P", 40 },
  { "ABSENT", 41 },
  { "ABSOLUTE_P", 42 },
  { "ACCESS", 43 },
  { "ACTION", 44 },
  { "ADD_P", 45 },
  { "ADMIN", 46 },
  { "AFTER", 47 },
  { "AGGREGATE", 48 },
  { "ALL", 49 },
  { "ALSO", 50 },
  { "ALTER", 51 },
  { "ALWAYS", 52 },
  { "ANALYSE", 53 },
  { "ANALYZE", 54 },
  { "AND", 55 },
  { "ANY", 56 },
  { "ARRAY", 57 },
  { "AS", 58 },
  { "ASC", 59 },
  { "ASCII_36", 1 },
  { "ASCII_37", 2 },
  { "ASCII_40", 3 },
  { "ASCII_41", 4 },
  { "ASCII_42", 5 },
  { "ASCII_43", 6 },
  { "ASCII_44", 7 },
  { "ASCII_45", 8 },
  { "ASCII_46", 9 },
  { "ASCII_47", 10 },
  { "ASCII_58", 11 },
  { "ASCII_59", 12 },
  { "ASCII_60", 13 },
  { "ASCII_61", 14 },
  { "ASCII_62", 15 },
  { "ASCII_63", 16 },
  { "ASCII_91", 17 },
  { "ASCII_92", 18 },
  { "ASCII_93", 19 },
  { "ASCII_94", 20 },
  { "ASENSITIVE", 60 },
  { "ASSERTION", 61 },
  { "ASSIGNMENT", 62 },
  { "ASYMMETRIC", 63 },
  { "AT", 65 },
  { "ATOMIC", 64 },
  { "ATTACH", 66 },
  { "ATTRIBUTE", 67 },
  { "AUTHORIZATION", 68 },
  { "BACKWARD", 69 },
  { "BCONST", 26 },
  { "BEFORE", 70 },
  { "BEGIN_P", 71 },
  { "BETWEEN", 72 },
  { "BIGINT", 73 },
  { "BINARY", 74 },
  { "BIT", 75 },
  { "BOOLEAN_P", 76 },
  { "BOTH", 77 },
  { "BREADTH", 78 },
  { "BY", 79 },
  { "CACHE", 80 },
  { "CALL", 81 },
  { "CALLED", 82 },
  { "CASCADE", 83 },
  { "CASCADED", 84 },
  { "CASE", 85 },
  { "CAST", 86 },
  { "CATALOG_P", 87 },
  { "CHAIN", 88 },
  { "CHARACTER", 90 },
  { "CHARACTERISTICS", 91 },
  { "CHAR_P", 89 },
  { "CHECK", 92 },
  { "CHECKPOINT", 93 },
  { "CLASS", 94 },
  { "CLOSE", 95 },
  { "CLUSTER", 96 },
  { "COALESCE", 97 },
  { "COLLATE", 98 },
  { "COLLATION", 99 },
  { "COLON_EQUALS", 33 },
  { "COLUMN", 100 },
  { "COLUMNS", 101 },
  { "COMMENT", 102 },
  { "COMMENTS", 103 },
  { "COMMIT", 104 },
  { "COMMITTED", 105 },
  { "COMPRESSION", 106 },
  { "CONCURRENTLY", 107 },
  { "CONDITIONAL", 108 },
  { "CONFIGURATION", 109 },
  { "CONFLICT", 110 },
  { "CONNECTION", 111 },
  { "CONSTRAINT", 112 },
  { "CONSTRAINTS", 113 },
  { "CONTENT_P", 114 },
  { "CONTINUE_P", 115 },
  { "CONVERSION_P", 116 },
  { "COPY", 117 },
  { "COST", 118 },
  { "CREATE", 119 },
  { "CROSS", 120 },
  { "CSV", 121 },
  { "CUBE", 122 },
  { "CURRENT_CATALOG", 124 },
  { "CURRENT_DATE", 125 },
  { "CURRENT_P", 123 },
  { "CURRENT_ROLE", 126 },
  { "CURRENT_SCHEMA", 127 },
  { "CURRENT_TIME", 128 },
  { "CURRENT_TIMESTAMP", 129 },
  { "CURRENT_USER", 130 },
  { "CURSOR", 131 },
  { "CYCLE", 132 },
  { "C_COMMENT", 39 },
  { "DATABASE", 134 },
  { "DATA_P", 133 },
  { "DAY_P", 135 },
  { "DEALLOCATE", 136 },
  { "DEC", 137 },
  { "DECIMAL_P", 138 },
  { "DECLARE", 139 },
  { "DEFAULT", 140 },
  { "DEFAULTS", 141 },
  { "DEFERRABLE", 142 },
  { "DEFERRED", 143 },
  { "DEFINER", 144 },
  { "DELETE_P", 145 },
  { "DELIMITER", 146 },
  { "DELIMITERS", 147 },
  { "DEPENDS", 148 },
  { "DEPTH", 149 },
  { "DESC", 150 },
  { "DETACH", 151 },
  { "DICTIONARY", 152 },
  { "DISABLE_P", 153 },
  { "DISCARD", 154 },
  { "DISTINCT", 155 },
  { "DO", 156 },
  { "DOCUMENT_P", 157 },
  { "DOMAIN_P", 158 },
  { "DOT_DOT", 32 },
  { "DOUBLE_P", 159 },
  { "DROP", 160 },
  { "EACH", 161 },
  { "ELSE", 162 },
  { "EMPTY_P", 163 },
  { "ENABLE_P", 164 },
  { "ENCODING", 165 },
  { "ENCRYPTED", 166 },
  { "END_P", 167 },
  { "ENUM_P", 168 },
  { "EQUALS_GREATER", 34 },
  { "ERROR_P", 169 },
  { "ESCAPE", 170 },
  { "EVENT", 171 },
  { "EXCEPT", 172 },
  { "EXCLUDE", 173 },
  { "EXCLUDING", 174 },
  { "EXCLUSIVE", 175 },
  { "EXECUTE", 176 },
  { "EXISTS", 177 },
  { "EXPLAIN", 178 },
  { "EXPRESSION", 179 },
  { "EXTENSION", 180 },
  { "EXTERNAL", 181 },
  { "EXTRACT", 182 },
  { "FALSE_P", 183 },
  { "FAMILY", 184 },
  { "FCONST", 23 },
  { "FETCH", 185 },
  { "FILTER", 186 },
  { "FINALIZE", 187 },
  { "FIRST_P", 188 },
  { "FLOAT_P", 189 },
  { "FOLLOWING", 190 },
  { "FOR", 191 },
  { "FORCE", 192 },
  { "FOREIGN", 193 },
  { "FORMAT", 194 },
  { "FORMAT_LA", 531 },
  { "FORWARD", 195 },
  { "FREEZE", 196 },
  { "FROM", 197 },
  { "FULL", 198 },
  { "FUNCTION", 199 },
  { "FUNCTIONS", 200 },
  { "GENERATED", 201 },
  { "GLOBAL", 202 },
  { "GRANT", 203 },
  { "GRANTED", 204 },
  { "GREATER_EQUALS", 36 },
  { "GREATEST", 205 },
  { "GROUPING", 207 },
  { "GROUPS", 208 },
  { "GROUP_P", 206 },
  { "HANDLER", 209 },
  { "HAVING", 210 },
  { "HEADER_P", 211 },
  { "HOLD", 212 },
  { "HOUR_P", 213 },
  { "ICONST", 29 },
  { "IDENT", 21 },
  { "IDENTITY_P", 214 },
  { "IF_P", 215 },
  { "ILIKE", 216 },
  { "IMMEDIATE", 217 },
  { "IMMUTABLE", 218 },
  { "IMPLICIT_P", 219 },
  { "IMPORT_P", 220 },
  { "INCLUDE", 222 },
  { "INCLUDING", 223 },
  { "INCREMENT", 224 },
  { "INDENT", 225 },
  { "INDEX", 226 },
  { "INDEXES", 227 },
  { "INHERIT", 228 },
  { "INHERITS", 229 },
  { "INITIALLY", 230 },
  { "INLINE_P", 231 },
  { "INNER_P", 232 },
  { "INOUT", 233 },
  { "INPUT_P", 234 },
  { "INSENSITIVE", 235 },
  { "INSERT", 236 },
  { "INSTEAD", 237 },
  { "INTEGER", 239 },
  { "INTERSECT", 240 },
  { "INTERVAL", 241 },
  { "INTO", 242 },
  { "INT_P", 238 },
  { "INVOKER", 243 },
  { "IN_P", 221 },
  { "IS", 244 },
  { "ISNULL", 245 },
  { "ISOLATION", 246 },
  { "JOIN", 247 },
  { "JSON", 248 },
  { "JSON_ARRAY", 249 },
  { "JSON_ARRAYAGG", 250 },
  { "JSON_EXISTS", 251 },
  { "JSON_OBJECT", 252 },
  { "JSON_OBJECTAGG", 253 },
  { "JSON_QUERY", 254 },
  { "JSON_SCALAR", 255 },
  { "JSON_SERIALIZE", 256 },
  { "JSON_TABLE", 257 },
  { "JSON_VALUE", 258 },
  { "KEEP", 259 },
  { "KEY", 260 },
  { "KEYS", 261 },
  { "LABEL", 262 },
  { "LANGUAGE", 263 },
  { "LARGE_P", 264 },
  { "LAST_P", 265 },
  { "LATERAL_P", 266 },
  { "LEADING", 267 },
  { "LEAKPROOF", 268 },
  { "LEAST", 269 },
  { "LEFT", 270 },
  { "LESS_EQUALS", 35 },
  { "LEVEL", 271 },
  { "LIKE", 272 },
  { "LIMIT", 273 },
  { "LISTEN", 274 },
  { "LOAD", 275 },
  { "LOCAL", 276 },
  { "LOCALTIME", 277 },
  { "LOCALTIMESTAMP", 278 },
  { "LOCATION", 279 },
  { "LOCKED", 281 },
  { "LOCK_P", 280 },
  { "LOGGED", 282 },
  { "MAPPING", 283 },
  { "MATCH", 284 },
  { "MATCHED", 285 },
  { "MATERIALIZED", 286 },
  { "MAXVALUE", 287 },
  { "MERGE", 288 },
  { "MERGE_ACTION", 289 },
  { "METHOD", 290 },
  { "MINUTE_P", 291 },
  { "MINVALUE", 292 },
  { "MODE", 293 },
  { "MODE_PLPGSQL_ASSIGN1", 538 },
  { "MODE_PLPGSQL_ASSIGN2", 539 },
  { "MODE_PLPGSQL_ASSIGN3", 540 },
  { "MODE_PLPGSQL_EXPR", 537 },
  { "MODE_TYPE_NAME", 536 },
  { "MONTH_P", 294 },
  { "MOVE", 295 },
  { "NAMES", 297 },
  { "NAME_P", 296 },
  { "NATIONAL", 298 },
  { "NATURAL", 299 },
  { "NCHAR", 300 },
  { "NESTED", 301 },
  { "NEW", 302 },
  { "NEXT", 303 },
  { "NFC", 304 },
  { "NFD", 305 },
  { "NFKC", 306 },
  { "NFKD", 307 },
  { "NO", 308 },
  { "NONE", 309 },
  { "NORMALIZE", 310 },
  { "NORMALIZED", 311 },
  { "NOT", 312 },
  { "NOTHING", 313 },
  { "NOTIFY", 314 },
  { "NOTNULL", 315 },
  { "NOT_EQUALS", 37 },
  { "NOT_LA", 532 },
  { "NOWAIT", 316 },
  { "NUL", 0 },
  { "NULLIF", 318 },
  { "NULLS_LA", 533 },
  { "NULLS_P", 319 },
  { "NULL_P", 317 },
  { "NUMERIC", 320 },
  { "OBJECT_P", 321 },
  { "OF", 322 },
  { "OFF", 323 },
  { "OFFSET", 324 },
  { "OIDS", 325 },
  { "OLD", 326 },
  { "OMIT", 327 },
  { "ON", 328 },
  { "ONLY", 329 },
  { "OPERATOR", 330 },
  { "OPTION", 331 },
  { "OPTIONS", 332 },
  { "OR", 333 },
  { "ORDER", 334 },
  { "ORDINALITY", 335 },
  { "OTHERS", 336 },
  { "OUTER_P", 338 },
  { "OUT_P", 337 },
  { "OVER", 339 },
  { "OVERLAPS", 340 },
  { "OVERLAY", 341 },
  { "OVERRIDING", 342 },
  { "OWNED", 343 },
  { "OWNER", 344 },
  { "Op", 28 },
  { "PARALLEL", 345 },
  { "PARAM", 30 },
  { "PARAMETER", 346 },
  { "PARSER", 347 },
  { "PARTIAL", 348 },
  { "PARTITION", 349 },
  { "PASSING", 350 },
  { "PASSWORD", 351 },
  { "PATH", 352 },
  { "PLACING", 353 },
  { "PLAN", 354 },
  { "PLANS", 355 },
  { "POLICY", 356 },
  { "POSITION", 357 },
  { "PRECEDING", 358 },
  { "PRECISION", 359 },
  { "PREPARE", 361 },
  { "PREPARED", 362 },
  { "PRESERVE", 360 },
  { "PRIMARY", 363 },
  { "PRIOR", 364 },
  { "PRIVILEGES", 365 },
  { "PROCEDURAL", 366 },
  { "PROCEDURE", 367 },
  { "PROCEDURES", 368 },
  { "PROGRAM", 369 },
  { "PUBLICATION", 370 },
  { "QUOTE", 371 },
  { "QUOTES", 372 },
  { "RANGE", 373 },
  { "READ", 374 },
  { "REAL", 375 },
  { "REASSIGN", 376 },
  { "RECHECK", 377 },
  { "RECURSIVE", 378 },
  { "REFERENCES", 380 },
  { "REFERENCING", 381 },
  { "REFRESH", 382 },
  { "REF_P", 379 },
  { "REINDEX", 383 },
  { "RELATIVE_P", 384 },
  { "RELEASE", 385 },
  { "RENAME", 386 },
  { "REPEATABLE", 387 },
  { "REPLACE", 388 },
  { "REPLICA", 389 },
  { "RESET", 390 },
  { "RESTART", 391 },
  { "RESTRICT", 392 },
  { "RETURN", 393 },
  { "RETURNING", 394 },
  { "RETURNS", 395 },
  { "REVOKE", 396 },
  { "RIGHT", 397 },
  { "ROLE", 398 },
  { "ROLLBACK", 399 },
  { "ROLLUP", 400 },
  { "ROUTINE", 401 },
  { "ROUTINES", 402 },
  { "ROW", 403 },
  { "ROWS", 404 },
  { "RULE", 405 },
  { "SAVEPOINT", 406 },
  { "SCALAR", 407 },
  { "SCHEMA", 408 },
  { "SCHEMAS", 409 },
  { "SCONST", 24 },
  { "SCROLL", 410 },
  { "SEARCH", 411 },
  { "SECOND_P", 412 },
  { "SECURITY", 413 },
  { "SELECT", 414 },
  { "SEQUENCE", 415 },
  { "SEQUENCES", 416 },
  { "SERIALIZABLE", 417 },
  { "SERVER", 418 },
  { "SESSION", 419 },
  { "SESSION_USER", 420 },
  { "SET", 421 },
  { "SETOF", 423 },
  { "SETS", 422 },
  { "SHARE", 424 },
  { "SHOW", 425 },
  { "SIMILAR", 426 },
  { "SIMPLE", 427 },
  { "SKIP", 428 },
  { "SMALLINT", 429 },
  { "SNAPSHOT", 430 },
  { "SOME", 431 },
  { "SOURCE", 432 },
  { "SQL_COMMENT", 38 },
  { "SQL_P", 433 },
  { "STABLE", 434 },
  { "STANDALONE_P", 435 },
  { "START", 436 },
  { "STATEMENT", 437 },
  { "STATISTICS", 438 },
  { "STDIN", 439 },
  { "STDOUT", 440 },
  { "STORAGE", 441 },
  { "STORED", 442 },
  { "STRICT_P", 443 },
  { "STRING_P", 444 },
  { "STRIP_P", 445 },
  { "SUBSCRIPTION", 446 },
  { "SUBSTRING", 447 },
  { "SUPPORT", 448 },
  { "SYMMETRIC", 449 },
  { "SYSID", 450 },
  { "SYSTEM_P", 451 },
  { "SYSTEM_USER", 452 },
  { "TABLE", 453 },
  { "TABLES", 454 },
  { "TABLESAMPLE", 455 },
  { "TABLESPACE", 456 },
  { "TARGET", 457 },
  { "TEMP", 458 },
  { "TEMPLATE", 459 },
  { "TEMPORARY", 460 },
  { "TEXT_P", 461 },
  { "THEN", 462 },
  { "TIES", 463 },
  { "TIME", 464 },
  { "TIMESTAMP", 465 },
  { "TO", 466 },
  { "TRAILING", 467 },
  { "TRANSACTION", 468 },
  { "TRANSFORM", 469 },
  { "TREAT", 470 },
  { "TRIGGER", 471 },
  { "TRIM", 472 },
  { "TRUE_P", 473 },
  { "TRUNCATE", 474 },
  { "TRUSTED", 475 },
  { "TYPECAST", 31 },
  { "TYPES_P", 477 },
  { "TYPE_P", 476 },
  { "UESCAPE", 478 },
  { "UIDENT", 22 },
  { "UMINUS", 541 },
  { "UNBOUNDED", 479 },
  { "UNCOMMITTED", 481 },
  { "UNCONDITIONAL", 480 },
  { "UNENCRYPTED", 482 },
  { "UNION", 483 },
  { "UNIQUE", 484 },
  { "UNKNOWN", 485 },
  { "UNLISTEN", 486 },
  { "UNLOGGED", 487 },
  { "UNTIL", 488 },
  { "UPDATE", 489 },
  { "USCONST", 25 },
  { "USER", 490 },
  { "USING", 491 },
  { "VACUUM", 492 },
  { "VALID", 493 },
  { "VALIDATE", 494 },
  { "VALIDATOR", 495 },
  { "VALUES", 497 },
  { "VALUE_P", 496 },
  { "VARCHAR", 498 },
  { "VARIADIC", 499 },
  { "VARYING", 500 },
  { "VERBOSE", 501 },
  { "VERSION_P", 502 },
  { "VIEW", 503 },
  { "VIEWS", 504 },
  { "VOLATILE", 505 },
  { "WHEN", 506 },
  { "WHERE", 507 },
  { "WHITESPACE_P", 508 },
  { "WINDOW", 509 },
  { "WITH", 510 },
  { "WITHIN", 511 },
  { "WITHOUT", 512 },
  { "WITHOUT_LA", 535 },
  { "WITH_LA", 534 },
  { "WORK", 513 },
  { "WRAPPER", 514 },
  { "WRITE", 515 },
  { "XCONST", 27 },
  { "XMLATTRIBUTES", 517 },
  { "XMLCONCAT", 518 },
  { "XMLELEMENT", 519 },
  { "XMLEXISTS", 520 },
  { "XMLFOREST", 521 },
  { "XMLNAMESPACES", 522 },
  { "XMLPARSE", 523 },
  { "XMLPI", 524 },
  { "XMLROOT", 525 },
  { "XMLSERIALIZE", 526 },
  { "XMLTABLE", 527 },
  { "XML_P", 516 },
  { "YEAR_P", 528 },
  { "YES_P", 529 },
  { "ZONE", 530 },
};
const ProtobufCEnumDescriptor pg_query__token__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "pg_query.Token",
  "Token",
  "PgQuery__Token",
  "pg_query",
  542,
  pg_query__token__enum_values_by_number,
  542,
  pg_query__token__enum_values_by_name,
  6,
  pg_query__token__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
